/* -*- Mode: C++ -*-
 * $Id: ActuatorInterface.h,v 1.2 2000/10/23 12:05:50 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: ActuatorInterface.h
 * Unitname: BasicSystem
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/10/23 12:05:50 $
 *
 *
 * HISTORY
 *
 * $Log: ActuatorInterface.h,v $
 * Revision 1.2  2000/10/23 12:05:50  frehe
 * Implemented the addToPlan(Step) method.
 *
 * Revision 1.1  2000/09/03 20:41:06  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file ActuatorInterface.h
 * Include file for the class ActuatorInterface.
 *
 * @par Unit:
 *      BasicSystem
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2000/10/23 12:05:50 $
 */


#ifndef ACTUATORINTERFACE_H
#define ACTUATORINTERFACE_H


#include "Plan.h"


RS_BEGIN_NAMESPACE


class Actuators;


/**
 * @class ActuatorInterface ActuatorInterface.h
 * This class is the interface between the decision maker
 * and the actuators.
 *
 * <!-- @invariant The class invariant is... -->
 *
 * @par Unit:
 *      BasicSystem
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2000/10/23 12:05:50 $
 */
class ActuatorInterface {
public:
  ActuatorInterface();
  virtual ~ActuatorInterface();

  /** Initialize the actuator interface. */
  virtual void init(Actuators* actuators);
  
  /** Add a command to the current step in the current plan.
      This is currently implemented as sending this command
      this step. */
  virtual bool addToPlan(Command* command);
  /** Add a step to the current step in the current plan.
      This is currently implemented as sending all the commands
      in the step right away. */
  virtual bool addToPlan(Step step);
  /** Tries to replace the current plan with a new plan. 
    This is currently *not* implemented!*/
  virtual bool replacePlan(const Plan& plan);

  /** Returns the next step in the plan. */
  Step getNextStep() const;

  /** Returns @c true iff the current plan is empty. */
  bool empty() const;

protected:
  Actuators* theActuators;
  Plan currentPlan;

  //Hide copy-constructor and assignment
  ActuatorInterface(const ActuatorInterface&);
  ActuatorInterface& operator= (const ActuatorInterface&);
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "ActuatorInterface.icc"
#endif


#endif // !ACTUATORINTERFACE_H
