/* -*- Mode: C++ -*-
 * $Id: Actuators.h,v 1.1 2000/09/03 20:41:07 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: Actuators.h
 * Unitname: BasicSystem
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/03 20:41:07 $
 *
 *
 * HISTORY
 *
 * $Log: Actuators.h,v $
 * Revision 1.1  2000/09/03 20:41:07  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file Actuators.h
 * Include file for the class Actuators.
 *
 * @par Unit:
 *      BasicSystem
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/03 20:41:07 $
 */


#ifndef ACTUATORS_H
#define ACTUATORS_H


#include <queue>

#include "ActuatorInterface.h"
#include "Controller.h"


RS_BEGIN_NAMESPACE


class Command;
class ServerCommand;
class ControlCommand;


/**
 * @class Actuators Actuators.h
 * This class is responsible for turning steps into
 * executable server or control commands and with the
 * help of the controller execute them.
 *
 * <!-- @invariant The class invariant is... -->
 *
 * @par Unit:
 *      BasicSystem
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/03 20:41:07 $
 */
class Actuators
{
public:
  Actuators(ActuatorInterface*);

  void init(Controller*);

  void executeCommand(Command* cmd);

  UInt16 removeAllPeriodicActions();
  UInt16 removeAllImmediateActions();
  UInt16 removeAllControlCommands();
  
  bool waitingPeriodicAction() const;
  bool waitingImmediateAction() const;
  bool waitingControlCommand() const;

  void executeNextPeriodicAction();
  void executeNextImmediateAction();
  void executeNextControlCommand();

protected:
  Controller* theController;
  ActuatorInterface* theActuatorInterface;
  std::queue<ServerCommand*> periodicActionQueue;
  std::queue<ServerCommand*> immediateActionQueue;
  std::queue<ControlCommand*> controlCommandQueue;
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "Actuators.icc"
#endif


#endif // !ACTUATORS_H
