/* -*- Mode: C++ -*-
 * $Id: Agent.h,v 1.1 2000/09/03 20:41:07 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: Agent.h
 * Unitname: BasicSystem
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/03 20:41:07 $
 *
 *
 * HISTORY
 *
 * $Log: Agent.h,v $
 * Revision 1.1  2000/09/03 20:41:07  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file Agent.h
 * Include file for the class Agent.
 *
 * @par Unit:
 *      BasicSystem
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/03 20:41:07 $
 */


#ifndef AGENT_H
#define AGENT_H


#include <string>

#include "machine_dependent_compiler_directives.h"


RS_BEGIN_NAMESPACE


class ServerInterface;
class SensorInterface;
class Sensors;
class ActuatorInterface;
class Actuators;
class Decision;
class ControlInterface;
class Controller;


/**
 * @class Agent Agent.h
 * This is a class that can be used to create the basic
 * system for an agent so the user do not have to create and
 * connect all the units herself.
 * This is an abstract base class so it can not be used as it is.
 *
 * <!-- @invariant The class invariant is... -->
 *
 * @par Unit:
 *      BasicSystem
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/03 20:41:07 $
 */
class Agent {
public:
  Agent();

  virtual ~Agent();

  virtual bool start() = 0;

  
protected:
  ServerInterface* serverInterface;
  SensorInterface* sensorInterface;
  Sensors* sensors;
  ActuatorInterface* actuatorInterface;
  Actuators* actuators;
  Decision* decision;
  ControlInterface* controlInterface;
  Controller* controller;
};


RS_END_NAMESPACE


#endif // !AGENT_H
