/* -*- Mode: C++ -*-
 * $Id: BasicAgent.h,v 1.1 2000/09/03 20:41:08 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: BasicAgent.h
 * Unitname: BasicSystem
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/03 20:41:08 $
 *
 *
 * HISTORY
 *
 * $Log: BasicAgent.h,v $
 * Revision 1.1  2000/09/03 20:41:08  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file BasicAgent.h
 * Include file for the class BasicAgent.
 *
 * @par Unit:
 *      BasicSystem
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/03 20:41:08 $
 */


#ifndef BASICAGENT_H
#define BASICAGENT_H


#include <string>

#include "library_types.h"

#ifdef RS_USE_SERVER_CONSTANTS
#  include "server_constants.h"
#else
   RS_BEGIN_NAMESPACE

   const std::string SP_host = "localhost";
   const UInt16 SP_port = 6000;
   const UInt32 SP_buffer_size = 2048;
   const UInt16 SP_simulator_step = 100;
   const UInt16 SP_recv_step = 10;
   const UInt16 SP_sense_body_step = 100;
   const UInt16 SP_send_step = 150;

   RS_END_NAMESPACE
#endif

#ifdef RS_USE_CLIENT_CONSTANTS
#  include "client_constants.h"
#else
   RS_BEGIN_NAMESPACE

   const std::string CP_team_name = "RoboSoc";
   const UInt16 CP_cycles_since_io_before_shutdown = 10;
   const UInt16 CP_min_ticks_before_sending_command = 2;
   const UInt16 CP_ticks_before_command_warning = 5;
   const UInt16 CP_max_ticks_before_sending_command = 9;

   RS_END_NAMESPACE
#endif


RS_BEGIN_NAMESPACE


class ServerInterface;
class SensorInterface;
class Sensors;
class ActuatorInterface;
class Actuators;
class Decision;
class ControlInterface;
class Controller;


/**
 * @class BasicAgent BasicAgent.h
 * This is a class that can be used to create the basic
 * system for an agent so the user do not have to create and
 * connect all the units herself. Only supply a decision object
 * and maybe change the agent parameters and you are ready to
 * make your own RoboCup basic agent (one that does not use the
 * RoboSoc framework, to use it you must use the FrameworkAgent
 * instead).
 *
 * @todo Make a program that generates a client_constants.h
 *       and server_constants.h from client.conf and server.conf.
 *
 * <!-- @invariant The class invariant is... -->
 *
 * @par Unit:
 *      BasicSystem
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/03 20:41:08 $
 */
class BasicAgent {
public:
  BasicAgent(Decision* decision,
	     const std::string& team_name = CP_team_name,
	     const std::string& host = SP_host,
	     const UInt16 port = SP_port,
	     const UInt32 server_buffer_size = SP_buffer_size,
	     const UInt16 cycle_length = SP_simulator_step,
	     const UInt16 recv_step = SP_recv_step,
	     const UInt16 sense_step = SP_sense_body_step,
	     const UInt16 visual_step = SP_send_step,
	     const UInt16 max_sigalrms = CP_cycles_since_io_before_shutdown,
	     const UInt16 min_ticks = CP_min_ticks_before_sending_command,
	     const UInt16 ticks_before_cmd_warning = CP_ticks_before_command_warning,
	     const UInt16 max_ticks = CP_max_ticks_before_sending_command);

  virtual ~BasicAgent();

  virtual bool start();

protected:
  ServerInterface* serverInterface;
  SensorInterface* sensorInterface;
  Sensors* sensors;
  ActuatorInterface* actuatorInterface;
  Actuators* actuators;
  Decision* decision;
  ControlInterface* controlInterface;
  Controller* controller;
};


RS_END_NAMESPACE


#endif // !BASICAGENT_H
