/* -*- Mode: C++ -*-
 * $Id: ControlInterface.h,v 1.1 2000/09/03 20:41:08 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: ControlInterface.h
 * Unitname: BasicSystem
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/03 20:41:08 $
 *
 *
 * HISTORY
 *
 * $Log: ControlInterface.h,v $
 * Revision 1.1  2000/09/03 20:41:08  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file ControlInterface.h
 * Include file for the class ControlInterface.
 *
 * @par Unit:
 *      BasicSystem
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/03 20:41:08 $
 */


#ifndef CONTROLINTERFACE_H
#define CONTROLINTERFACE_H


#include "ServerCommand.h"
#include "Decision.h"


RS_BEGIN_NAMESPACE


/**
 * @class ControlInterface ControlInterface.h
 * This class is the interface between the controller
 * and the decision maker.
 *
 * <!-- @invariant The class invariant is... -->
 *
 * @par Unit:
 *      BasicSystem
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/03 20:41:08 $
 */
class ControlInterface {
public:
  ControlInterface(Decision* const decision);
  virtual ~ControlInterface();
  
  virtual ServerCommand* initAgent();
  virtual bool start();

  /*
  UInt estimatedTimeToNextSenseBody() const;
  UInt estimatedTimeToNextSee() const;
  UInt estimatedTimeToNextCycle() const;
  UInt estimatedTimeToNextCommandWarning() const;
  UInt estimatedTimeToNextSendCommand() const;
  */
  
  // Event handlers
  virtual void onActuatorSensorData();
  virtual void onAuralSensorData();
  virtual void onInit();
  virtual void onPhysicalSensorData();
  virtual void onSensorError();
  virtual void onVisualSensorData();

  virtual void onBeforeSensorData();
  virtual void onAfterSensorData();
  virtual void onBeforeTick();
  virtual void onAfterTick();

  virtual void onNewCycle(const bool forced);
  virtual bool onEstimatedNewCycle(UInt16 estimated_time, UInt16 ticks);
  virtual void onCommandWarning();

  virtual void onDelayedActions(Int16 dashes, Int16 kicks, Int16 says,
				Int16 turns, Int16 turn_necks);
  virtual void onMissingActions(Int16 dashes, Int16 kicks, Int16 says,
				Int16 turns, Int16 turn_necks);

protected:
  Decision* const theDecisionMaker;

  bool invariant() const;
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "ControlInterface.icc"
#endif


#endif // !CONTROLINTERFACE_H
