/* -*- Mode: C++ -*-
 * $Id: Sensors.h,v 1.2 2000/09/04 08:46:45 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: Sensors.h
 * Unitname: BasicSystem
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/04 08:46:45 $
 *
 *
 * HISTORY
 *
 * $Log: Sensors.h,v $
 * Revision 1.2  2000/09/04 08:46:45  frehe
 * Fixed a naming error in the sensors.
 *
 * Revision 1.1  2000/09/03 20:41:10  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file Sensors.h
 * Include file for the class Sensors.
 *
 * @par Unit:
 *      BasicSystem
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2000/09/04 08:46:45 $
 */


#ifndef SENSORS_H
#define SENSORS_H


#include <queue>

#include "basic_system_types.h"
#include "ServerCommand.h"


RS_BEGIN_NAMESPACE


class Controller;
class SensorInterface;


/**
 * @class Sensors Sensors.h
 * This class is responsible for parsing the server messages
 * and storing them in the sensor interface for further use
 * by the agent.
 *
 * <!-- @invariant The class invariant is... -->
 *
 * @par Unit:
 *      BasicSystem
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2000/09/04 08:46:45 $
 */
class Sensors
{
public:
  Sensors(SensorInterface*);

  void init(Controller*);

  /** Analyze messages from the server to the player. */
  void analyze(char*);
  /** Analyze coach messages. */
  void analyzeCoachMessage(char*);
  /** Analyze feedback from the actuators. */
  void analyze(ServerCommand*);  
  // The feedback should only be updated at the same time as the agents
  // are updated in the server. I.e. Turn neck, change view a.s.o. are
  // updated when they arrive while dash, kick and turn are only updated
  // at the end of the cycle.
  // But isn't that something for the views to take care of?
  // If the sensor interface stores the commands, then it's up to the
  // views when to do the actual update.
  // Major problem that only one command can be stored in the sensor interface.
  // Or maybe not, since only one command is executed last in the cycle
  // that's the only one that needs to be saved. But it could be a problem.

  SensorType getLastSensorType() const throw();
  UInt16 getCurrentTime() const throw();

protected:
  Controller* theController;
  SensorInterface* theSensorInterface;
  SensorType lastSensorType;
  UInt16 currentTime;

  std::queue<CommandType> commandQueue;
  
  enum ObjectType {
    OBJ_LINE,
    OBJ_BALL,
    OBJ_MARKER,
    OBJ_MARKER_BEHIND,
    OBJ_PLAYER
  };
  
  void analyzeInitMessage(char*);
  void analyzeAuralSensor(char*);
  void analyzeVisualSensor(char*);
  void analyzeSenseSensor(char*);
  void analyzeRefereeSound(char*);
  
  Float32 getFloat(char** str_ptr);
  Float32 getFloat(char* str);
  Int32 getInt(char* str);
  Int32 getInt(char** str_ptr);
  void getWord(char** str_ptr);
  void getNextWord(char** str_ptr);
  void advanceTo(char c, char** str_ptr);

  // Hide the copy-constructor and the assignment operator.
  Sensors(const Sensors&);
  Sensors& operator= (const Sensors&);
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "Sensors.icc"
#endif


#endif // !SENSORS_H
