/* -*- Mode: C++ -*-
 * $Id: ServerInterface.h,v 1.1 2000/09/03 20:41:10 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: ServerInterface.h
 * Unitname: BasicSystem
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/03 20:41:10 $
 *
 *
 * HISTORY
 *
 * $Log: ServerInterface.h,v $
 * Revision 1.1  2000/09/03 20:41:10  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file ServerInterface.h
 * Include file for the class ServerInterface.
 *
 * @par Unit:
 *      BasicSystem
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/03 20:41:10 $
 */


#ifndef SERVERINTERFACE_H
#define SERVERINTERFACE_H


#include <string>

#include "Socket.h"


RS_BEGIN_NAMESPACE


/** Used to represent the different errors that
  the server interface can return. */
enum ServerError {
  SERVER_UNKNOWN_ERROR,
  SERVER_NOT_CONNECTED,
  SERVER_NO_DATA,
  SERVER_SOCKET_ERROR,
  SERVER_NO_ERROR,
  SERVER_ALREADY_CONNECTED
};


/** Used to represent the exceptions raised by
  the server interface. */
class ServerException : public RSException {
public:
  ServerException(const std::string& descr,
		  const std::string& file_n = "",
		  const UInt32 line_no = 0) throw()
		    : RSException(descr, file_n, line_no) {};
};


/**
 * @class ServerInterface ServerInterface.h
 * This class is the interface between the controller
 * and the soccer server.
 *
 * <!-- @invariant The class invariant is... -->
 *
 * @par Unit:
 *      BasicSystem
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/03 20:41:10 $
 */
class ServerInterface
{
public:
  ServerInterface(const std::string& host, const UInt16 port,
		  const UInt32 server_buffer_size) throw();

  bool isConnected() const throw();
  bool connect() throw(RSException);
  bool disconnect() throw(RSException);

  bool sendCommand(const std::string& cmd) throw(RSException);
  bool receiveInput(char* const input) throw(RSException);

  ServerError getErrorMessage() const throw();

  UInt32 getServerBufferSize() const throw();

private:
  Socket theSocket;
  bool connected;
  ServerError theErrorMessage;

  ServerInterface(const ServerInterface&);
  ServerInterface& operator= (const ServerInterface&);
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "ServerInterface.icc"
#endif


#endif // !SERVERINTERFACE_H
