/* -*- Mode: C++ -*-
 * $Id: basic_system_types.h,v 1.1 2000/09/03 20:41:11 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: basic_system_types.h
 * Unitname: BasicSystem
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/03 20:41:11 $
 *
 *
 * HISTORY
 *
 * $Log: basic_system_types.h,v $
 * Revision 1.1  2000/09/03 20:41:11  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file basic_system_types.h
 * Include file with the types used by the RoboSoc basic system.
 *
 * @todo Move the server objects to their own files.
 * @todo Find better names for the server object classes.
 *
 * @par Unit:
 *      BasicSystem
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/03 20:41:11 $
 */


#ifndef BASIC_SYSTEM_TYPES_H
#define BASIC_SYSTEM_TYPES_H


#include "library_types_with_unknown.h"
#include "soccer_types.h"


RS_BEGIN_NAMESPACE


/** Used to represent the different types of sensor
  information the agent can receive. */
enum SensorType {
  SENSOR_AURAL,
  SENSOR_VISUAL,
  SENSOR_SENSE,
  SENSOR_ACTUATORS,
  SENSOR_INIT,
  SENSOR_ERROR,
  SENSOR_COACH,
  SENSOR_UNKNOWN
};


/** Used to represent the objects sent by the server
  to the client as see messages. */
class ServerObject {
public:
  FloatUnknown distance;
  AngleDegUnknown direction;

  bool operator< (const ServerObject& obj) const
  { return distance > obj.distance; };
};


/** Used to represent the marker objects sent by the
  server to the client as see messages. */
class ServerMarkerObject : public ServerObject {
public:
  MarkerName markerId;
  bool behind;
};


/** Used to represent the line objects sent by the
  server to the client as see messages. */
class ServerLineObject : public ServerObject {
public:
  SideLineName lineId;
};


/** Used to represent the player objects sent by the
  server to the client as see messages. */
class ServerPlayerObject : public ServerObject {
public:
  Side side;
  Team team;
  UIntUnknown uniformNumber;
  FloatUnknown distanceChange;
  FloatUnknown directionChange;
  AngleDegUnknown bodyDirection;
  AngleDegUnknown faceDirection;
};


/** Used to represent the player objects sent by the
  server to coach clients as see messages. */
class CoachPlayerObject {
public:
  Side side;
  Team team;
  UIntUnknown uniformNumber;
  PointUnknown position;
  VectorUnknown speed;
  AngleDegUnknown bodyDirection;
  AngleDegUnknown neckDirection;
};


RS_END_NAMESPACE


#endif // !BASIC_SYSTEM_TYPES_H
