/* -*- Mode: C++ -*-
 * $Id: BasicAgent.cc,v 1.1 2000/09/03 20:41:12 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: BasicAgent.cc
 * Unitname: BasicSystem
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/03 20:41:12 $
 *
 *
 * HISTORY
 *
 * $Log: BasicAgent.cc,v $
 * Revision 1.1  2000/09/03 20:41:12  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file BasicAgent.cc
 * Implementation file for the class BasicAgent.
 *
 * @par Unit:
 *      BasicSystem
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/03 20:41:12 $
 */


#include "BasicAgent.h"

#include "ServerInterface.h"
#include "SensorInterface.h"
#include "Sensors.h"
#include "ActuatorInterface.h"
#include "Actuators.h"
#include "Decision.h"
#include "ControlInterface.h"
#include "Controller.h"
#include "rs_error.h"


RS_BEGIN_NAMESPACE


BasicAgent::BasicAgent(Decision* decision, const std::string& team_name,
		       const std::string& host, const UInt16 port,
		       const UInt32 server_buffer_size,
		       const UInt16 cycle_length, const UInt16 recv_step,
		       const UInt16 sense_step, const UInt16 visual_step,
		       const UInt16 max_sigalrms, const UInt16 min_ticks,
		       const UInt16 ticks_before_cmd_warning,
		       const UInt16 max_ticks)
  : serverInterface(new ServerInterface(host, port, server_buffer_size)),
    sensorInterface(new SensorInterface(team_name)),
    sensors(new Sensors(sensorInterface)),
    actuatorInterface(new ActuatorInterface()),
    actuators(new Actuators(actuatorInterface)),
    decision(decision),
    controlInterface(new ControlInterface(decision)),
    controller(new Controller(serverInterface, sensors,
			      actuators, controlInterface,
			      cycle_length, recv_step,
			      sense_step, visual_step,
			      max_sigalrms, min_ticks,
			      ticks_before_cmd_warning,
			      max_ticks))
{
};


BasicAgent::~BasicAgent()
{
  delete serverInterface;
  delete sensorInterface;
  delete sensors;
  delete actuatorInterface;
  delete actuators;
  delete controlInterface;
  delete controller;
}


bool BasicAgent::start()
{
  if ( !decision->init(actuatorInterface) ) {
    RS_FATAL("Fatal error in [CBasicAgent.Init]: the decision could not be initialized!");
    return false;
  }
  
  if ( !controller->init() ) {
    RS_FATAL("Fatal error in [CBasicAgent.Init]: the controller could not be initialized!");
    return false;
  }
  
  controller->start();

  return true;
}


RS_END_NAMESPACE
