/* -*- Mode: C++ -*-
 * $Id: Display.cc,v 1.5 2001/06/19 14:01:41 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: Display.cc
 * Unitname: Framework/example/FCFoo
 * $Revision: 1.5 $
 * Created by: Johan Thapper 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2001/06/19 14:01:41 $
 *
 *
 * HISTORY
 *
 * $Log: Display.cc,v $
 * Revision 1.5  2001/06/19 14:01:41  frehe
 * Made minor changes to compile RoboSoc with gcc-3.0
 *
 * Revision 1.4  2000/10/19 00:05:34  frehe
 * Made minor changes.
 *
 * Revision 1.3  2000/10/18 23:07:11  frehe
 * Added a say message and a display of them.
 *
 * Revision 1.2  2000/10/11 18:40:29  frehe
 * Upgraded FCFoo.
 *
 * Revision 1.1  2000/09/05 21:12:54  frehe
 * Made FCFoo compile with RoboSoc 2.5.0
 *
 */

/**
 * @file Display.cc
 * Implementation file for the class Display.
 *
 * @par Unit:
 *      Framework/example/FCFoo
 * $Revision: 1.5 $
 * $Author: frehe $
 * $Date: 2001/06/19 14:01:41 $
 */


#include "Display.h"
#include "view_constants.h"


RS_BEGIN_NAMESPACE


Display::Display(UInt x, UInt y)
  : viewManager(ViewManager::instance()),
    parameterView(NULL),
    gameView(NULL),
    markerView(NULL),
    agentView(NULL),
    ballView(NULL),
    playerView(NULL),
#ifdef RS_USE_COACH_DEBUGGING
    coachView(NULL),
#endif
    col(x),
    row(y)
{
    parameterView = static_cast<ParameterView*>(viewManager->getView(PARAMETER_VIEW_ID));
    if ( parameterView == NULL ) {
        parameterView = new ParameterView(PARAMETER_VIEW_ID);
        viewManager->addView(parameterView, true);
    }
    gameView = static_cast<GameView*>(viewManager->getView(GAME_VIEW_ID));
    if ( gameView == NULL ) {
        gameView = new GameView(GAME_VIEW_ID);
        viewManager->addView(gameView, true);
    }
    markerView = static_cast<MarkerView*>(viewManager->getView(MARKER_VIEW_ID));
    if ( markerView == NULL ) {
        markerView = new MarkerView(MARKER_VIEW_ID, parameterView);
        viewManager->addView(markerView, true);
    }
    agentView = static_cast<AgentView*>(viewManager->getView(AGENT_VIEW_ID));
    if ( agentView == NULL ) {
        agentView = new AgentView(AGENT_VIEW_ID, parameterView);
        viewManager->addView(agentView, true);
    }
    ballView = static_cast<BallView*>(viewManager->getView(BALL_VIEW_ID));
    if ( ballView == NULL ) {
        ballView = new BallView(BALL_VIEW_ID);
        viewManager->addView(ballView, true);
    }
    playerView = static_cast<PlayerView*>(viewManager->getView(PLAYER_VIEW_ID));
    if ( playerView == NULL ) {
        playerView = new PlayerView(PLAYER_VIEW_ID);
        viewManager->addView(playerView, true);
    }
#ifdef RS_USE_COACH_DEBUGGING
    getView(coachView, COACH_VIEW_ID);
#endif
}


Display::~Display()
{
  viewManager->releaseView(PARAMETER_VIEW_ID);
  viewManager->releaseView(GAME_VIEW_ID);
  viewManager->releaseView(MARKER_VIEW_ID);
  viewManager->releaseView(AGENT_VIEW_ID);
  viewManager->releaseView(BALL_VIEW_ID);
  viewManager->releaseView(PLAYER_VIEW_ID);
#ifdef RS_USE_COACH_DEBUGGING
  viewManager->releaseView(COACH_VIEW_ID);
#endif
}


void Display::setLastSkill(const std::string& last)
{
    lastSkill = last;
}


void Display::draw()
{
    CLEAR_SCREEN;
    HOME_CURSOR;

    UInt STAMINA_BAR_SIZE = col - 15;

    // draw stamina bar
    FloatUnknown stamina_unknown = agentView->getStamina(NOW);
    Float stamina = 0;
    if ( stamina_unknown.isKnown() )
      stamina = stamina_unknown.getValue();
    Float percent = stamina/parameterView->SP_stamina_max;
    Int xs = (int)(STAMINA_BAR_SIZE*percent);
    std::cout << "stamina [";
    for (Int i = 0; i < xs; i++) std::cout << "x";
    for (Int i = 0; i < STAMINA_BAR_SIZE-xs; i++) std::cout << ".";
    std::cout << "] " << (Int)(100*percent) << "%\n";

    std::cout << "last coach message: " << agentView->getLastCoachMessage() << "\n";
    std::cout << "last player message: " << agentView->getLastPlayerMessage() << "\n";

#ifdef RS_USE_COACH_DEBUGGING
    //    std::cout << "last coach message time: " << coachView->getCoachMessageTime(NOW) << "\n";
    //    std::cout << "current time: " << gameView->getCurrentTime() << std::"\n";
#endif

    // print last skill name
    std::cout << "last skill: " << lastSkill << "\n";

    // print position
    PointUnknown apos = agentView->getAgentPosition(NOW);
    std::cout << "agent position is " << apos << "\n";
#ifdef RS_USE_COACH_DEBUGGING
    PointUnknown arealpos = coachView->getAgentAbsolutePosition(NOW);
    std::cout << "agent real position is " << arealpos << "\n";
#endif
    
    /*
    if (!apos.isUnknown()) {
        std::cout << "agent pos: (" << apos.getX().getValue() << ", "
	     << apos.getY().getValue() << ")" << "\n";
    } else {
        std::cout << "agent pos: unknown" << "\n";
    }
    */
    
    // print speed
    VectorUnknown aspeed = agentView->getSpeedVector(NOW);
    std::cout << "agent speed is " << aspeed << "\n";
    //std::cout << "             = " << PointUnknown(aspeed) << "\n";
#ifdef RS_USE_COACH_DEBUGGING
    VectorUnknown arealspeed = coachView->getAgentAbsoluteSpeed(NOW);
    std::cout << "agent real speed is " << arealspeed << "\n";
#endif

    // print body, face and neck directions
    AngleDegUCO facedir = agentView->getFaceDirection(NOW);
    AngleDegUCO bodydir = agentView->getBodyDirection(NOW);
    AngleDegUCO neckdir = agentView->getNeckDirection(NOW);
    if ( !facedir.isUnknown() )
      std::cout << "agent face dir: " << facedir.getDeg() << "\n";
    else
      std::cout << "agent face dir: unknown" << "\n";
#ifdef RS_USE_COACH_DEBUGGING
    AngleDegUnknown realfacedir = coachView->getAgentAbsoluteFaceDirection(NOW);
    if ( !realfacedir.isUnknown() )
      std::cout << "real agent face dir: " << realfacedir.getDeg() << "\n";
    else
      std::cout << "real agent face dir: unknown" << "\n";
#endif

    if ( !bodydir.isUnknown() )
      std::cout << "agent body dir: " << bodydir.getDeg() << "\n";
    else
      std::cout << "agent body dir: unknown" << "\n";
#ifdef RS_USE_COACH_DEBUGGING
    AngleDegUnknown realbodydir = coachView->getAgentAbsoluteDirection(NOW);
    if ( !realbodydir.isUnknown() )
      std::cout << "real agent body dir: " << realbodydir.getDeg() << "\n";
    else
      std::cout << "real agent body dir: unknown" << "\n";
#endif

    if ( !neckdir.isUnknown() )
      std::cout << "agent neck dir: " << neckdir.getDeg() << "\n";
    else
      std::cout << "agent neck dir: unknown" << "\n";
#ifdef RS_USE_COACH_DEBUGGING
    AngleDegUnknown realneckdir = coachView->getAgentAbsoluteNeckDirection(NOW);
    if ( !realneckdir.isUnknown() )
      std::cout << "real agent neck dir: " << realneckdir.getDeg() << "\n";
    else
      std::cout << "real agent neck dir: unknown" << "\n";
#endif
    
    if ( agentView->isAgentOutsideField() )
      std::cout << "agent is outside the field" << "\n";


    // print ball info
    AngleDegUCO bdir = ballView->getRelativeBallDirection(NOW);
    FloatUnknown bdist = ballView->getBallDistance(NOW).getValue().getValue();
    PointUnknown bpos = ballView->getBallPosition(NOW);
    VectorUnknown bspeed = ballView->getBallAbsoluteSpeedVector(NOW);

#ifdef RS_USE_COACH_DEBUGGING
    VectorUnknown brealvec = coachView->getRelativeBallVector(NOW);
#endif
    
    if (bdir.isUnknown()) {
      std::cout << "ball direction is not known" << "\n";
    } else {
      std::cout << "ball direction is " << bdir.getDeg() << "\n";
    }
#ifdef RS_USE_COACH_DEBUGGING
    if (brealvec.getAngle().isUnknown()) {
      std::cout << "ball real direction is not known" << "\n";
    } else {
      std::cout << "ball real direction is " << brealvec.getAngle().getDeg() << "\n";
    }
#endif

    if (bdist.isUnknown()) {
      std::cout << "ball distance is not known" << "\n";
    } else {
      std::cout << "ball distance is " << bdist.getValue() << "\n";
    }
#ifdef RS_USE_COACH_DEBUGGING
    if (brealvec.getLength().isUnknown()) {
      std::cout << "ball real distance is not known" << "\n";
    } else {
      std::cout << "ball real distance is " << brealvec.getLength().getValue() << "\n";
    }
#endif

    std::cout << "ball position is " << bpos << "\n";
#ifdef RS_USE_COACH_DEBUGGING
    PointUnknown brealpos = coachView->getBallAbsolutePosition(NOW);
    std::cout << "ball real position is " << brealpos << "\n";
#endif
    std::cout << "ball speed is " << bspeed << "\n";
    //std::cout << "            = " << PointUnknown(bspeed) << "\n";
#ifdef RS_USE_COACH_DEBUGGING
    VectorUnknown brealspeed = coachView->getBallAbsoluteSpeed(NOW);
    std::cout << "ball real speed is " << brealspeed << "\n";
#endif

    std::vector<PlayerObject> teammates = playerView->getTeammates(NOW);
    std::cout << "Saw " << teammates.size() << " teammates." << "\n";
#ifdef RS_USE_COACH_DEBUGGING
    std::cout << "Coach saw " << coachView->getTeammates(NOW).size() << " teammates." << "\n";
#endif
    for ( std::vector<PlayerObject>::const_iterator plr = teammates.begin();
	  plr != teammates.end(); ++plr ) {
      if ( plr->getUniformNumber().isKnown() ) {
	std::cout << "Teammate no " << plr->getUniformNumber().getValue() << "\n";
	std::cout << "pos = " << PointUnknown(plr->getPosition()) << "\n";
#ifdef RS_USE_COACH_DEBUGGING
	if ( coachView->getTeammates(NOW).size() >= teammates.size() )
	  std::cout << "realpos = " << PointUnknown(coachView->getTeammates(NOW)[plr->getUniformNumber().getValue()].getPosition()) << "\n";
#endif
	std::cout << "vector = " << VectorUnknown(plr->getVector()) << "\n";
#ifdef RS_USE_COACH_DEBUGGING
	if ( coachView->getTeammates(NOW).size() >= teammates.size() )
	  std::cout << "realvector = " << VectorUnknown(coachView->getTeammates(NOW)[plr->getUniformNumber().getValue()].getVector()) << "\n";
#endif
      }
    }
    
    /*
    // print closest line info
    LineObject line = markerView->getClosestSeenLine();
    VectorUnknown lvec = line.getVector();

    std::cout << "closest line is " << line.getLineId() << "\n";
    std::cout << "closest line vector is " << lvec << "\n";    
    */

    /*
    // print closest marker info
    MarkerObject flag = markerView->getClosestSeenMarker();
    PointUnknown fpos = flag.getPosition();
    VectorUnknown fvec = flag.getVector();

    std::cout << "closest flag is " << flag.getMarkerId() << "\n";
    std::cout << "closest flag position is " << fpos << "\n";
    std::cout << "closest flag vector is " << fvec << "\n";
    if ( flag.isBehind() )
      std::cout << "closest flag is behind." << "\n";
    */

    std::cout << std::flush;

}


RS_END_NAMESPACE
