/* -*- Mode: C++ -*-
 * $Id: FCFoo.cc,v 1.6 2000/10/24 15:15:46 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: FCFoo.cc
 * Unitname: Framework/example/FCFoo
 * $Revision: 1.6 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/10/24 15:15:46 $
 *
 *
 * HISTORY
 * $Log: FCFoo.cc,v $
 * Revision 1.6  2000/10/24 15:15:46  frehe
 * Made minor updates and tests.
 *
 * Revision 1.5  2000/10/11 18:40:29  frehe
 * Upgraded FCFoo.
 *
 * Revision 1.4  2000/10/04 11:57:03  frehe
 * Removed all the Makefile.in.
 *
 * Revision 1.3  2000/09/13 11:06:19  frehe
 * Fixed a bug in FCFoo that didn't read the command line.
 *
 * Revision 1.2  2000/09/05 21:12:54  frehe
 * Made FCFoo compile with RoboSoc 2.5.0
 *
 */

/**
 * @file FCFoo.cc
 * Implementation file for the example agent FCFoo.
 *
 * @par Unit:
 *      Framework/example/FCFoo
 * $Revision: 1.6 $
 * $Author: frehe $
 * $Date: 2000/10/24 15:15:46 $
 */


#include <fstream>

#include "framework_compiler_directives.h"
#include "ParameterView.h"
#include "view_constants.h"
#include "FrameworkAgent.h"
#include "FCFooDecision.h"
#include "LogStream.h"


RS_BEGIN_NAMESPACE


#if defined(RS_USE_ERROR) && !defined(RS_LOG_ERROR)
extern void rs_warning(const std::string& str,
		       const std::string& file,
		       int line)
{
  std::cerr << "WARNING: " << str
	    << " at line " << line 
	    << " in file " << file
	    << std::endl;
}

extern void rs_error(const std::string& str,
		     const std::string& file,
		     int line)
{
  std::cerr << "ERROR: " << str
	    << " at line " << line 
	    << " in file " << file
	    << std::endl;
}

extern void rs_fatal(const std::string& str,
		     const std::string& file,
		     int line)
{
  std::cerr << "FATAL: " << str
	    << " at line " << line 
	    << " in file " << file
	    << std::endl;
  abort();
}
#endif

#if !defined(RS_USE_EXCEPTION) && !defined(RS_LOG_EXCEPTION)
extern void rs_throw(const std::string& exception,
		     const std::string& msg,
		     const std::string& file,
		     int line)
{
  std::cerr << "RAISED: " << exception << "(\"" << msg << "\")"
	    << " at line " << line 
	    << " in file " << file
	    << std::endl;
}
#endif


RS_END_NAMESPACE


int main(int argc, char** argv)
{
#ifdef RS_USE_NAMESPACE
  using namespace RS_NAMESPACE;
#endif

  std::ofstream logFile("logfile.txt");
  
#ifdef RS_USE_LOG
  LogStream::instance()->addLogArea(LA_ROBOSOC, LA_TOP, true, &std::cerr);
  LogStream::instance()->addLogArea(LA_WARNING, LA_ROBOSOC, true, &logFile, true, &std::cerr);
  LogStream::instance()->addLogArea(LA_ERROR, LA_ROBOSOC, true, &logFile, true, &std::cerr);
  LogStream::instance()->addLogArea(LA_FATAL, LA_ROBOSOC, true, &logFile, true, &std::cerr);
  LogStream::instance()->addLogArea(LA_CONTROLLER, LA_ROBOSOC, false, &logFile);
  LogStream::instance()->addLogArea(LA_ACTUATORS, LA_ROBOSOC, true, &logFile);
  LogStream::instance()->addLogArea(LA_SENSORS, LA_ROBOSOC, true, &logFile);
  LogStream::instance()->addLogArea(LA_SENSOR_INTERFACE, LA_ROBOSOC, false, &logFile);
  LogStream::instance()->addLogArea(LA_AGENT_VIEW, LA_ROBOSOC, true, &logFile);
  LogStream::instance()->addLogArea(LA_BALL_VIEW, LA_ROBOSOC, true, &logFile);
  LogStream::instance()->addLogArea(LA_COMMAND_VIEW, LA_ROBOSOC, true, &logFile);
  LogStream::instance()->addLogArea(LA_DECISION, LA_ROBOSOC, true, &logFile);
  LogStream::instance()->addLogArea(LA_FRAMEWORK_AGENT, LA_ROBOSOC, true, &logFile);
#endif
    
  try {

    std::cerr << "Starting the parameter view...\n";
    ParameterView parameterView(PARAMETER_VIEW_ID,
				argc, argv);
    ViewManager::instance()->addView(&parameterView, true);
    
    std::cerr << "Starting decision...\n";
    FCFooDecision decision;

    std::cerr << "Initializing the agent...\n";
    FrameworkAgent agent(&decision, &parameterView);

    std::cerr << "Starting the agent...\n";
#ifdef RS_USE_COACH_DEBUGGING
    agent.startWithCoach();
#else
    agent.start();
#endif
    
  } catch (RSException &excpt) {
    std::cerr << "Caught an exception: " << excpt.What() << std::endl;
    return 1;
  }
  
  return 0;
  
}

