/* -*- Mode: C++ -*-
 * $Id: FCFooDecision.cc,v 1.12 2001/01/08 13:41:40 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: FCFooDecision.cc
 * Unitname: Framework/example/FCFoo
 * $Revision: 1.12 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2001/01/08 13:41:40 $
 *
 *
 * HISTORY
 *
 * $Log: FCFooDecision.cc,v $
 * Revision 1.12  2001/01/08 13:41:40  frehe
 * Fixed a minor bug in FCFooDecision.
 *
 * Revision 1.11  2001/01/08 13:27:45  frehe
 * Fixed a minor bug in FCFooDecision.
 *
 * Revision 1.10  2000/11/08 09:41:10  frehe
 * Updated the FCFooDecision to use the Score skill.
 *
 * Revision 1.9  2000/11/02 16:33:45  frehe
 * Minor updates.
 *
 * Revision 1.8  2000/10/24 15:15:46  frehe
 * Made minor updates and tests.
 *
 * Revision 1.7  2000/10/19 00:05:35  frehe
 * Made minor changes.
 *
 * Revision 1.6  2000/10/18 23:07:11  frehe
 * Added a say message and a display of them.
 *
 * Revision 1.5  2000/10/11 18:40:29  frehe
 * Upgraded FCFoo.
 *
 * Revision 1.4  2000/10/10 11:55:37  frehe
 * Fixed FCFooDecision to work with the new skills.
 *
 * Revision 1.3  2000/10/08 17:34:11  frehe
 * Fixed FCFoo.
 *
 * Revision 1.2  2000/09/05 21:12:55  frehe
 * Made FCFoo compile with RoboSoc 2.5.0
 *
 * Revision 1.1.1.1  2000/08/31 13:26:05  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file FCFooDecision.cc
 * Implementation of the class FCFooDecision.
 *
 * @par Unit:
 *      Framework/example/FCFoo
 * $Revision: 1.12 $
 * $Author: frehe $
 * $Date: 2001/01/08 13:41:40 $
 */


#include "FCFooDecision.h"

#include <unistd.h>
#include <errno.h>

#include "view_constants.h"
#include "ActuatorInterface.h"
#include "InitCommand.h"
#include "MoveCommand.h"
#include "TurnCommand.h"
#include "TurnNeckCommand.h"
#include "SayCommand.h"
#include "FindBall.h"
#include "GotoBallAvoidingObstacles.h"
#include "MoveToPositionWithBall.h"
#include "Score.h"
#include "rs_assert.h"
#include "rs_log.h"


RS_BEGIN_NAMESPACE


FCFooDecision::FCFooDecision(ActuatorInterface* const actuator_interface)
  : Decision(actuator_interface),
    viewManager(ViewManager::instance()),
    parameterView(NULL),
    gameView(NULL),
    commandView(NULL),
    ballView(NULL),
    agentView(NULL),
    playerView(NULL),
    serverAlive(true)
#ifdef RS_USE_DISPLAY
  , theDisplay()
#endif
#ifdef RS_USE_COACH_DEBUGGING
  , coachView(NULL)
#endif
#ifdef RS_USE_OWN_LOGFILE
  , logFile("FCFoo.log")
#endif
{
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(gameView, GAME_VIEW_ID);
  getView(commandView, COMMAND_VIEW_ID);
  //markerView = RS_GET_VIEW(CMarkerView, MARKER_VIEW_ID);
  getView(ballView, BALL_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID, parameterView);
  getView(playerView, PLAYER_VIEW_ID);
  
#ifdef RS_USE_COACH_DEBUGGING
  getView(coachView, COACH_VIEW_ID);
#endif
}


FCFooDecision::~FCFooDecision()
{
  viewManager->releaseView(PARAMETER_VIEW_ID);
  viewManager->releaseView(GAME_VIEW_ID);
  viewManager->releaseView(COMMAND_VIEW_ID);
  viewManager->releaseView(BALL_VIEW_ID);
  viewManager->releaseView(AGENT_VIEW_ID);
  viewManager->releaseView(PLAYER_VIEW_ID);
#ifdef RS_USE_COACH_DEBUGGING
  viewManager->releaseView(COACH_VIEW_ID);
#endif
}


ServerCommand* FCFooDecision::initAgent()
{
  RS_REQUIRE( parameterView != NULL );
  InitCommand* init_cmd = new InitCommand(parameterView->CP_team_name, 5.5,
					  parameterView->CP_goalie);
  return init_cmd;
}


bool FCFooDecision::start()
{
  RS_REQUIRE( initialized == true );
  while ( serverAlive )
    if ( pause() == -1 && errno != EINTR )
      RS_ERROR("Something went wrong in pause!");
  return true;
}


bool FCFooDecision::decide()
{
  /*
  static ServerCommand* scmd;
  scmd = new TurnCommand(45);
  actuatorInterface->addToPlan(scmd);
  delete scmd;
  return true;
  */
  
  static FindBall findBall;
  static GotoBallAvoidingObstacles gotoBallAvoidingObstacles;
  static MoveToPositionWithBall moveToPositionWithBall(Point(55,0));
  static Score score;
  static int say_counter = 0;

  /*
  std::cout << "My position is: " << agentView->getAgentPosition(NOW)
 	    << " facing: " << agentView->getFaceDirection(NOW)
	    << std::endl;
   */
  
  /*
  if ( playerView->getClosestPlayer() != NULL )
    std::cout << "Closest player is no "
	      << playerView->getClosestPlayer()->getUNumber()
	      << endl;

  if ( playerView->getClosestTeammate() != NULL )
    std::cout << "Closest teammate is no "
	      << playerView->getClosestTeammate()->getUNumber()
	      << endl;

  if ( playerView->getClosestOpponent() != NULL )
    std::cout << "Closest opponent is no "
	      << playerView->getClosestOpponent()->getUNumber()
	      << endl;
  */

  // An agent that only finds the ball
  //actuatorInterface->addPlan(findBall.GeneratePlan());

  // An agent that goes for the ball and tries to score
  Command* cmd = NULL;

  // Get the position of the ball to be used in MoveToPosition(ballPos)
  if ( score.applicable() ) {
    cmd = score.generatePlan().getNextStep().getNextCommand();
    //actuatorInterface->addPlan(score.generatePlan());
  }
  else if ( moveToPositionWithBall.applicable() ) {
    cmd = moveToPositionWithBall.generatePlan().getNextStep().getNextCommand();
  }
  else if ( gotoBallAvoidingObstacles.applicable() ) {
    cmd = gotoBallAvoidingObstacles.generatePlan().getNextStep().getNextCommand();
    //actuatorInterface->addPlan(gotoBallAvoidingObstacles.generatePlan());
  }
  else if ( findBall.applicable() ) {
    cmd = findBall.generatePlan().getNextStep().getNextCommand();
    //actuatorInterface->addPlan(findBall.generatePlan());
  }

  /*
  if ( agentView->getNeckDirection(NOW).isKnown() &&
       ballView->getRelativeBallDirection(NOW).isKnown() &&
       abs(agentView->getNeckDirection(NOW).getDeg()) > 10 ) {
    cmd = new TurnNeckCommand(-agentView->getNeckDirection(NOW).getDeg());
    actuatorInterface->addToPlan(cmd);
    cmd = new TurnCommand(ballView->getRelativeBallDirection(NOW).getDeg() + agentView->getNeckDirection(NOW).getDeg());
  } else if ( findBall.applicable() ) {
    cmd = findBall.generatePlan().getNextStep().getNextCommand();
    //actuatorInterface->addPlan(findBall.generatePlan());
  } else if ( gotoBallAvoidingObstacles.applicable() ) {
    cmd = gotoBallAvoidingObstacles.generatePlan().getNextStep().getNextCommand();
    //actuatorInterface->addPlan(gotoBallAvoidingObstacles.generatePlan());
  } else if ( score.applicable() ) {
    cmd = score.generatePlan().getNextStep().getNextCommand();
    //actuatorInterface->addPlan(score.generatePlan());
  }
  */
  
  if ( cmd != NULL ) {
    actuatorInterface->addToPlan(cmd);

    if ( cmd->getType() != CMD_Turn &&
	 ballView->getRelativeBallDirection(NOW).isKnown() &&
	 abs(ballView->getRelativeBallDirection(NOW)) > 5 ) {
      Command* cmd2 = new TurnNeckCommand(ballView->getRelativeBallDirection(NOW).getDeg());
      actuatorInterface->addToPlan(cmd2);
      delete cmd2;      
    }

    delete cmd;
    cmd = NULL;
  }
  
  if ( cmd != NULL &&
       cmd->getType() != CMD_Turn &&
       ballView->getRelativeBallDirection(NOW).isKnown() &&
       abs(ballView->getRelativeBallDirection(NOW)) > 5 ) {
    Command* cmd = new TurnNeckCommand(ballView->getRelativeBallDirection(NOW).getDeg());
    actuatorInterface->addToPlan(cmd);
    delete cmd;
    cmd = NULL;
  }

  if ( say_counter == 10 ) {
    say_counter = 0;

    if ( agentView->getUniformNumber().isKnown() &&
	 agentView->getAgentPosition(NOW).isKnown() ) {
      Command* cmd = new SayCommand("Player " + toString(agentView->getUniformNumber()) + " my position is " + toString(agentView->getAgentPosition(NOW)));
      actuatorInterface->addToPlan(cmd);
      delete cmd;
      cmd = NULL;
    }
  }
  else {
    ++say_counter;
  }
  
  // To test the quality of the information available to the
  // agent compare the estimated values with the correct values
#ifdef RS_USE_OWN_LOGFILE
  static Float ballXError = 0;
  static Float ballYError = 0;
  static Float ballDirError = 0;
  static Float ballDistError = 0;
  static Float agentXError = 0;
  static Float agentYError = 0;
  static Float agentFaceError = 0;
  static Float agentBodyError = 0;
  static Float agentNeckError = 0;
  static Float agentFaceMaxError = 0;
  static Float agentBodyMaxError = 0;
  static Float agentNeckMaxError = 0;
    
  if ( coachView->getCoachMessageTime(NOW) != gameView->getCurrentTime().getT() ) {
    logFile << "Coach time and current time not equal! "
	    << coachView->getCoachMessageTime(NOW)
	    << " != " << gameView->getCurrentTime().getT() << std::endl;
  } else {
    FloatUnknown ballX = ballView->getBallPosition(NOW).getX();
    FloatUnknown ballY = ballView->getBallPosition(NOW).getY();
    FloatUnknown ballRealX = coachView->getBallAbsolutePosition(NOW).getX();
    FloatUnknown ballRealY = coachView->getBallAbsolutePosition(NOW).getY();
    FloatUnknown ballXDiff = abs(ballX - ballRealX);
    FloatUnknown ballYDiff = abs(ballY - ballRealY);
    AngleDegUnknown ballSpeedDir = ballView->getBallAbsoluteSpeedVector(NOW).getAngle();
    FloatUnknown ballSpeed = ballView->getBallAbsoluteSpeedVector(NOW).getLength();
    AngleDegUnknown ballRealSpeedDir = coachView->getBallAbsoluteSpeed(NOW).getAngle();
    FloatUnknown ballRealSpeed = coachView->getBallAbsoluteSpeed(NOW).getLength();

    if ( ballXDiff.isKnown() )
      ballXError += ballXDiff.getValue();
    if ( ballYDiff.isKnown() )
      ballYError += ballYDiff.getValue();
    

    AngleDegUnknown ballDir = ballView->getRelativeBallDirection(NOW);
    FloatUnknown ballDist = ballView->getBallDistance(NOW);
    AngleDegUnknown ballRealDir = coachView->getRelativeBallVector(NOW).getAngle();
    FloatUnknown ballRealDist = coachView->getRelativeBallVector(NOW).getLength();
    AngleDegUnknown ballDirDiff = abs(ballDir - ballRealDir);
    FloatUnknown ballDistDiff = abs(ballDist - ballRealDist);
    
    if ( ballDirDiff.isKnown() )
      ballDirError += ballDirDiff.getDeg();
    if ( ballDistDiff.isKnown() )
      ballDistError += ballDistDiff.getValue();
    

    FloatUnknown agentX = agentView->getAgentPosition(NOW).getX();
    FloatUnknown agentY = agentView->getAgentPosition(NOW).getY();
    FloatUnknown agentRealX = coachView->getAgentAbsolutePosition(NOW).getX();
    FloatUnknown agentRealY = coachView->getAgentAbsolutePosition(NOW).getY();
    FloatUnknown agentXDiff = abs(agentX - agentRealX);
    FloatUnknown agentYDiff = abs(agentY - agentRealY);
    AngleDegUnknown agentDir = agentView->getSpeedVector(NOW).getAngle();
    FloatUnknown agentSpeed = agentView->getSpeedVector(NOW).getLength();
    AngleDegUnknown agentRealDir = coachView->getAgentAbsoluteSpeed(NOW).getAngle();
    FloatUnknown agentRealSpeed = coachView->getAgentAbsoluteSpeed(NOW).getLength();

    if ( agentXDiff.isKnown() )
      agentXError += agentXDiff.getValue();
    if ( agentYDiff.isKnown() )
      agentYError += agentYDiff.getValue();
    
    AngleDegUnknown agentBody = agentView->getBodyDirection(NOW);
    AngleDegUnknown agentFace = agentView->getFaceDirection(NOW);
    AngleDegUnknown agentNeck = agentView->getNeckDirection(NOW);
    AngleDegUnknown agentRealBody = coachView->getAgentAbsoluteDirection(NOW);
    AngleDegUnknown agentRealFace = coachView->getAgentAbsoluteFaceDirection(NOW);
    AngleDegUnknown agentRealNeck = coachView->getAgentAbsoluteNeckDirection(NOW);
    AngleDegUnknown agentBodyDiff = abs(agentBody - agentRealBody);
    AngleDegUnknown agentFaceDiff = abs(agentFace - agentRealFace);
    AngleDegUnknown agentNeckDiff = abs(agentNeck - agentRealNeck);

    if ( agentBodyDiff.isKnown() ) {
      agentBodyError += agentBodyDiff.getDeg();
      agentBodyMaxError = max(agentBodyDiff.getDeg(), agentBodyMaxError);
    }
    if ( agentFaceDiff.isKnown() ) {
      agentFaceError += agentFaceDiff.getDeg();
      agentFaceMaxError = max(agentFaceDiff.getDeg(), agentFaceMaxError);
    }
    if ( agentNeckDiff.isKnown() ) {
      agentNeckError += agentNeckDiff.getDeg();
      agentNeckMaxError = max(agentNeckDiff.getDeg(), agentNeckMaxError);
    }
    
    logFile << gameView->getCurrentTime() << ": ballPos"
      // << "= ("
      //    << ballX << ", " << ballY << "); ballRealPos = ("
      //    << ballRealX << ", " << ballRealY << "); "
	    << " Diff = (" << ballXDiff << ", " << ballYDiff << ")"
	    << " Sum error = (" << ballXError << ", " << ballYError << ")"
	    << std::endl;
    logFile << gameView->getCurrentTime() << ": ballSpeed"
	    << " = (" << ballSpeedDir << ", " << ballSpeed << "); ballRealSpeed = ("
	    << ballRealSpeedDir << ", " << ballRealSpeed << "); "
	    << " Diff = ("
	    << abs(ballSpeedDir - ballRealSpeedDir) << ", " << abs(ballSpeed - ballRealSpeed) << ")"
	    << std::endl;

    logFile << gameView->getCurrentTime() << ": ballVector"
	    << " = (" << ballDir << ", " << ballDist << ");"
	    << " ballRealVector = (" << ballRealDir << ", " << ballRealDist << ");"
	    << " Diff = (" << ballDirDiff << ", " << ballDistDiff << ");"
	    << " Sum error = (" << ballDirError << ", " << ballDistError << ");"
	    << std::endl;

    logFile << gameView->getCurrentTime() << ": agentPos"
      // << "= ("
      //   << agentX << ", " << agentY << "); agentRealPos = ("
      //    << agentRealX << ", " << agentRealY << "); "
	    << " Diff = (" << agentXDiff << ", " << agentYDiff << ")"
	    << " Sum error = (" << agentXError << ", " << agentYError << ")"
	    << std::endl;
    logFile << gameView->getCurrentTime() << ": agentSpeed"
      //	    << "= ("
      //    << agentDir << ", " << agentSpeed << "); agentRealSpeed = ("
      //    << agentRealDir << ", " << agentRealSpeed << "); "
	    << " Diff = ("
	    << abs(agentDir - agentRealDir) << ", " << abs(agentSpeed - agentRealSpeed) << ")"
	    << std::endl;

    logFile << gameView->getCurrentTime() << ": bodyDir"
	    << "= " << agentBody << " realBodyDir = " << agentRealBody << ";"
	    << " Diff = " << agentBodyDiff
	    << " Sum error = " << agentBodyError
      	    << " Max error = " << agentBodyMaxError 
 	    << std::endl;
    
    logFile << gameView->getCurrentTime() << ": faceDir"
	    << "= " << agentFace << " realFaceDir = " << agentRealFace << ";"
	    << " Diff = " << agentFaceDiff
	    << " Sum error = " << agentFaceError 
      	    << " Max error = " << agentFaceMaxError
 	    << std::endl;
    
    logFile << gameView->getCurrentTime() << ": neckDir"
	    << "= " << agentNeck << " realNeckDir = " << agentRealNeck << ";"
	    << " Diff = " << agentNeckDiff
	    << " Sum error = " << agentNeckError 
	    << " Max error = " << agentNeckMaxError
	    << std::endl;
  }
#endif

  return true;
  
}


void FCFooDecision::onActuatorSensorData()
{
#ifdef RS_USE_DISPLAY
  theDisplay.setLastSkill(commandView->getLastServerString());
#endif
#ifdef RS_USE_OWN_LOGFILE
  logFile << gameView->getCurrentTime() << ": executed action "
	  << commandView->getLastServerString()
	  << std::endl;
  if ( commandView->getActualTurn(NOW) > AngleDeg(0) )
    logFile << gameView->getCurrentTime() << ": actual turn = "
	    << commandView->getActualTurn(NOW)
	    << std::endl;  
#endif
}


void FCFooDecision::onInit()
{
  MoveCommand* move_cmd = new MoveCommand(Point(-10, -10));
  actuatorInterface->addToPlan(move_cmd);
}


void FCFooDecision::onVisualSensorData()
{
  //RS_LOG( LA_DECISION, gameView->getCurrentTime() << ": **saw**" );
}


void FCFooDecision::onAfterSensorData()
{
}


void FCFooDecision::onNewCycle(const bool forced)
{
  if ( forced ) {
    RS_LOG( LA_DECISION,
	    gameView->getCurrentTime() << ": **Forced NewCycle**" );
  }
  else {
    RS_LOG( LA_DECISION, gameView->getCurrentTime() << ": **NewCycle**" );
  }

#ifdef RS_USE_DISPLAY
  theDisplay.draw();
#endif

}


bool FCFooDecision::onEstimatedNewCycle(UInt16 estimated_time, UInt16 ticks)
{
  // We assume that if the sense_body is more than four ticks late
  // then it's lost.
  RS_LOG( LA_DECISION, "onEstimatedNewCycle(" << estimated_time
	  << ", " << ticks << ")" );

  // If more than nine sense_body's are missed in a row then assume that
  // the server is dead.
  if ( estimated_time > 9 )
    serverAlive = false;

  if ( ticks > 4 ) {
    // TODO: change this into sending a control command instead!
    //sensorInterface->ForceNewCycle();
    return true;
  } else
    return false;
}


void FCFooDecision::onCommandWarning()
{
  decide();
}


RS_END_NAMESPACE
