/* -*- Mode: C++ -*-
 * $Id: FCFooDecision.h,v 1.5 2000/10/24 15:15:46 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: FCFooDecision.h
 * Unitname: Framework/example/FCFoo
 * $Revision: 1.5 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/10/24 15:15:46 $
 *
 *
 * HISTORY
 *
 * $Log: FCFooDecision.h,v $
 * Revision 1.5  2000/10/24 15:15:46  frehe
 * Made minor updates and tests.
 *
 * Revision 1.4  2000/10/19 00:05:35  frehe
 * Made minor changes.
 *
 * Revision 1.3  2000/10/11 18:40:30  frehe
 * Upgraded FCFoo.
 *
 * Revision 1.2  2000/09/05 21:12:55  frehe
 * Made FCFoo compile with RoboSoc 2.5.0
 *
 * Revision 1.1.1.1  2000/08/31 13:26:05  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file FCFooDecision.h
 * Include file for the class FCFooDecision.
 *
 * @par Unit:
 *      Framework/example/FCFoo
 * $Revision: 1.5 $
 * $Author: frehe $
 * $Date: 2000/10/24 15:15:46 $
 */


#ifndef FCFOODECISION_H
#define FCFOODECISION_H


#include "basic_system_compiler_directives.h"
#include "Decision.h"
#include "ViewManager.h"
#include "ParameterView.h"
#include "GameView.h"
#include "CommandView.h"
#include "BallView.h"
#include "AgentView.h"
#include "PlayerView.h"

//#define RS_USE_COACH_DEBUGGING
#define RS_USE_DISPLAY

#ifdef RS_USE_DISPLAY
#  include "Display.h"
#endif

//#define RS_USE_OWN_LOGFILE


#ifdef RS_USE_COACH_DEBUGGING
#  include "CoachView.h"
#endif

#ifdef RS_USE_OWN_LOGFILE
#  include <fstream>
#endif


RS_BEGIN_NAMESPACE


/**
 * @class FCFooDecision FCFooDecision.h
 * Example class for what a decision maker might look like.
 *
 * <!-- @todo-->
 *
 * @par Unit:
 *      Library
 * $Revision: 1.5 $
 * $Author: frehe $
 * $Date: 2000/10/24 15:15:46 $
 */
class FCFooDecision : public Decision {
public:
  FCFooDecision(ActuatorInterface* const actuator_interface = NULL);
  virtual ~FCFooDecision();

  virtual ServerCommand* initAgent();
  virtual bool start();
  //virtual bool init(ActuatorInterface* const actuator_interface);
  virtual bool decide();

  // Event handlers
  virtual void onActuatorSensorData();
  //virtual void onAuralSensorData();
  virtual void onInit();
  //virtual void onPhysicalSensorData();
  virtual void onVisualSensorData();

  //virtual void onBeforeSensorData();
  virtual void onAfterSensorData();
  //virtual void onBeforeTick();
  //virtual void onAfterTick();

  virtual void onNewCycle(const bool forced);
  virtual bool onEstimatedNewCycle(UInt16 estimated_time, UInt16 ticks);
  virtual void onCommandWarning();

protected:
  ViewManager* const viewManager;
  ParameterView* parameterView;
  GameView* gameView;
  CommandView* commandView;
  BallView* ballView;
  AgentView* agentView;
  PlayerView* playerView;
  bool serverAlive;

#ifdef RS_USE_DISPLAY
  Display theDisplay;
#endif

#ifdef RS_USE_COACH_DEBUGGING
  CoachView* coachView;
#endif

#ifdef RS_USE_OWN_LOGFILE
  std::ofstream logFile;
#endif
};


RS_END_NAMESPACE


#endif // !FCFOODECISION_H
