/* -*- Mode: C++ -*-
 * $Id: FrameworkAgent.h,v 1.1 2000/09/03 22:08:43 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: FrameworkAgent.h
 * Unitname: Framework
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/03 22:08:43 $
 *
 *
 * HISTORY
 *
 * $Log: FrameworkAgent.h,v $
 * Revision 1.1  2000/09/03 22:08:43  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file FrameworkAgent.h
 * Include file for the class FramworkAgent.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/03 22:08:43 $
 */


#ifndef FRAMEWORKAGENT_H
#define FRAMEWORKAGENT_H


#include <string>

#include "basic_system_types.h"

#if defined(RS_USE_SERVER_CONSTANTS) && defined(RS_USE_CLIENT_CONSTANTS)
#  include "server_constants.h"
#  include "client_constants.h"
#endif


RS_BEGIN_NAMESPACE


class ViewManager;
class ParameterView;

class ServerInterface;
class FrameworkSensorInterface;
class Sensors;
class ActuatorInterface;
class Actuators;
class Decision;
class ControlInterface;
class Controller;


/**
 * @class FrameworkAgent FrameworkAgent.h
 * This is a class that can be used to create the basic system and
 * framework for an agent so the user do not have to create and
 * connect all the units herself. Only supply a decision object
 * and a parameter view pointer or a command line argument with
 * the necessary parameters to start the agent.
 *
 * <!-- @invariant The class invariant is... -->
 *
 * @par Unit:
 *      BasicSystem
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/03 22:08:43 $
 */
class FrameworkAgent {
public:
  FrameworkAgent(Decision* decision,
		 int argc, char** argv);

  FrameworkAgent(Decision* decision,
		 ParameterView* parameterView);

#if defined(RS_USE_SERVER_CONSTANTS) && defined(RS_USE_CLIENT_CONSTANTS)
  FrameworkAgent(Decision* decision);
#endif
  
  virtual ~FrameworkAgent();

  virtual bool start();
  virtual bool startWithCoach();

protected:
  ViewManager* viewManager;
  ParameterView* parameterView;
  
  ServerInterface* serverInterface;
  FrameworkSensorInterface* sensorInterface;
  Sensors* sensors;
  ActuatorInterface* actuatorInterface;
  Actuators* actuators;
  Decision* decision;
  ControlInterface* controlInterface;
  Controller* controller;
};


RS_END_NAMESPACE


#endif // !FRAMEWORKAGENT_H
