/* -*- Mode: C++ -*-
 * $Id: FrameworkSensorInterface.h,v 1.2 2000/09/13 00:04:09 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: FrameworkSensorInterface.h
 * Unitname: Framework
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/13 00:04:09 $
 *
 *
 * HISTORY
 *
 * $Log: FrameworkSensorInterface.h,v $
 * Revision 1.2  2000/09/13 00:04:09  frehe
 * Added forEach methods that takes a method from a View class instead of a global function as argument.
 *
 * Revision 1.1  2000/09/03 22:08:43  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file FrameworkSensorInterface.h
 * Include file for the class FrameworkSensorInterface.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2000/09/13 00:04:09 $
 */


#ifndef FRAMEWORK_SENSOR_INTERFACE_H
#define FRAMEWORK_SENSOR_INTERFACE_H


#include "basic_system_types.h"
#include "SensorInterface.h"
#include "ViewManager.h"


RS_BEGIN_NAMESPACE


class ViewManager;


/**
 * @class FrameworkSensorInterface FrameworkSensorInterface.h
 * This class is the interface between the sensors and the rest
 * of the agent. It is connected to the view manager and also
 * responsible for making sure that the views are updated with the
 * new information from the sensors.
 *
 * <!-- @invariant The class invariant is... -->
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2000/09/13 00:04:09 $
 */
class FrameworkSensorInterface : public SensorInterface {
public:
  FrameworkSensorInterface(const std::string& my_team_name);
  virtual ~FrameworkSensorInterface();

  virtual void setLastSensorType(const SensorType sensor_type);

  void forEachMarker(void (View::*fn)(const ServerMarkerObject&),
		     View* const view) const;
  void forEachLine(void (View::*fn)(const ServerLineObject&),
		   View* const view) const;
  void forEachPlayer(void (View::*fn)(const ServerPlayerObject&),
		     View* const view) const;
  void forEachOpponent(void (View::*fn)(const CoachPlayerObject&),
		       View* const view) const;
  void forEachTeammate(void (View::*fn)(const CoachPlayerObject&),
		       View* const view) const;
  
protected:
  ViewManager* theViewManager;

  virtual void updateBeforeNewCycle();

  //Hide copy-constructor and assignment
  FrameworkSensorInterface(const FrameworkSensorInterface&);
  FrameworkSensorInterface& operator= (const FrameworkSensorInterface&);
};


RS_END_NAMESPACE


#endif // !FRAMEWORK_SENSOR_INTERFACE_H
