/* -*- Mode: C++ -*-
 * $Id: Predicate.h,v 1.4 2000/09/17 20:03:17 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: Predicate.h
 * Unitname: Framework
 * $Revision: 1.4 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/17 20:03:17 $
 *
 *
 * HISTORY
 *
 * $Log: Predicate.h,v $
 * Revision 1.4  2000/09/17 20:03:17  frehe
 * Updated the Predicate base class.
 *
 * Revision 1.3  2000/09/13 03:09:51  frehe
 * Added more documentation to the class.
 *
 * Revision 1.2  2000/09/13 00:03:30  frehe
 * Updated the operator()() in Predicate to return a FuzzyBool instead of a bool.
 *
 * Revision 1.1  2000/09/03 22:08:43  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file Predicate.h
 * Include file for the class Predicate.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.4 $
 * $Author: frehe $
 * $Date: 2000/09/17 20:03:17 $
 */


#ifndef PREDICATE_H
#define PREDICATE_H


#include "ViewManager.h"
#include "FuzzyBool.h"


RS_BEGIN_NAMESPACE


/**
 * @class Predicate Predicate.h
 * This class is the abstract base class for all predicates
 * in the RoboSoc system.
 *
 * <!-- @invariant The class invariant is... -->
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.4 $
 * $Author: frehe $
 * $Date: 2000/09/17 20:03:17 $
 */
class Predicate {
public:
  /** Create a predicate with default behavior. */
  Predicate();

  /** Creates a predicate that is an exact copy of @a obj. */
  Predicate(const Predicate& obj);

  /** Destroys the predicate. */
  virtual ~Predicate();

  
  /** Assignment operator. Assigns @a obj to this object. */
  Predicate& operator= (const Predicate& obj);


  /** Function application operator. Will call evalute(). */
  virtual FuzzyBool operator()();


  /** Should return the truth value of the predicate. */
  virtual FuzzyBool evaluate() = 0;

  
protected:
  /** A pointer to the view manager. */
  ViewManager* viewManager;

  /** The last update number the predicate is updated for. */
  UInt32 lastUpdateNumber;

  
  /** The update function, should be used to update the
    value of the predicate if necessary. */
  virtual void update(const UInt32 update_number) = 0;

  /** Test to see if the predicate needs to be updated. */
  bool needUpdate(const UInt32 update_number) const;
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "Predicate.icc"
#endif


#endif // !PREDICATE_H
