/* -*- Mode: C++ -*-
 * $Id: PredicateWithArguments.h,v 1.1 2000/09/13 03:11:14 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: PredicateWithArguments.h
 * Unitname: Framework
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 2000-09-13
 * Last modified by $Author: frehe $ $Date: 2000/09/13 03:11:14 $
 *
 *
 * HISTORY
 *
 * $Log: PredicateWithArguments.h,v $
 * Revision 1.1  2000/09/13 03:11:14  frehe
 * Added a new class PredicateWithArguments to be used by all predicates using arguments.
 *
 */

/**
 * @file PredicateWithArguments.h
 * Include file for the class PredicateWithArguments.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/13 03:11:14 $
 */


#ifndef PREDICATEWITHARGUMENTS_H
#define PREDICATEWITHARGUMENTS_H


#include "Predicate.h"
#include "TooFewArgumentsException.h"


RS_BEGIN_NAMESPACE


/**
 * @class PredicateWithArguments PredicateWithArguments.h
 * This class is the abstract base class for all predicates
 * that uses arguments.
 *
 * <!-- @invariant The class invariant is... -->
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/13 03:11:14 $
 */
class PredicateWithArguments : public Predicate {
public:
  /** Default constructor, it is ok since the class is abstract
    all subclasses should make sure to get default values for the
    arguments. */
  PredicateWithArguments();

  /** Creates a predicate that is an exact copy of @a obj. */
  PredicateWithArguments(const PredicateWithArguments&);

  /** Destroys the predicate. */
  virtual ~PredicateWithArguments();


  /** Assignment operator. Assigns @a obj to this object. */
  PredicateWithArguments& operator= (const PredicateWithArguments&);

  
  /** Used to set the predicate's arguments in a generic way.
   This method must be overridden by subclasses.
   It should transform the generic vector of floats to
   the actual arguments.
   It will throw TooFewArgumentsException if not enough arguments
   are given. */
  virtual void setArguments(const std::vector<Float>& arguments)
    throw(TooFewArgumentsException) = 0;

  
  /** Should return the truth value of the predicate. */
  virtual FuzzyBool evaluate() = 0;

  /** This method is used to first set the argument of the predicate
    and then evaluate it. It uses the methods setArguments and
    evaluate(), so it should not be overriden unless it should
    made unavailable. */
  virtual FuzzyBool evaluate(const std::vector<Float>& arguments);
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "PredicateWithArguments.icc"
#endif


#endif // !PREDICATEWITHARGUMENTS_H
