/* -*- Mode: C++ -*-
 * $Id: PredicateWithPosition.h,v 1.1 2000/09/13 03:11:41 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: PredicateWithPosition.h
 * Unitname: Framework
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 2000-09-13
 * Last modified by $Author: frehe $ $Date: 2000/09/13 03:11:41 $
 *
 *
 * HISTORY
 *
 * $Log: PredicateWithPosition.h,v $
 * Revision 1.1  2000/09/13 03:11:41  frehe
 * Added predicate class with arguments.
 *
 */

/**
 * @file PredicateWithPosition.h
 * Include file for the predicate class PredicateWithPosition.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/13 03:11:41 $
 */


#ifndef PREDICATEWITHPOSITION_H
#define PREDICATEWITHPOSITION_H


#include "PredicateWithArguments.h"


RS_BEGIN_NAMESPACE


/**
 * @class PredicateWithPosition PredicateWithPosition.h
 * This class is the abstract base class for all predicates
 * that uses a position as argument.
 *
 * <!-- @invariant The class invariant is... -->
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/13 03:11:41 $
 */
class PredicateWithPosition : public PredicateWithArguments {
public:
  /** Constructor taking the default argument for the position
    used as an argument in the predicate. */
  PredicateWithPosition(const Point& pos);

  /** Creates a predicate that is an exact copy of @a obj. */
  PredicateWithPosition(const PredicateWithPosition&);

  /** Destroys the predicate. */
  virtual ~PredicateWithPosition();


  /** Assignment operator. Assigns @a obj to this object. */
  PredicateWithPosition& operator= (const PredicateWithPosition&);

  
  /** Used to set the predicate's arguments in a generic way.
    The first value in the vector will be interpreted as the
    X-coordinate and the second value the Y-coordinate. */
  virtual void setArguments(const std::vector<Float>& arguments)
     throw(TooFewArgumentsException);


  /** Used to set the predicate's position argument to @a pos. */
  virtual void setPosition(const Point& pos);

  /** Return the predicate's position argument. */
  virtual Point getPosition() const;

  
  /** Should return the truth value of the predicate. */
  virtual FuzzyBool evaluate() = 0;

  /** This method is used to first set the argument of the predicate
    and then evaluate it. It uses the methods setPosition and
    evaluate(), so it should not be overriden unless it should
    made unavailable. */
  virtual FuzzyBool evaluate(const Point& pos);


protected:  
  /** The position argument. */
  Point position;
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "PredicateWithPosition.icc"
#endif


#endif // !PREDICATEWITHPOSITION_H
