/* -*- Mode: C++ -*-
 * $Id: Skill.h,v 1.3 2000/09/13 03:09:52 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: Skill.h
 * Unitname: Framework
 * $Revision: 1.3 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/13 03:09:52 $
 *
 *
 * HISTORY
 *
 * $Log: Skill.h,v $
 * Revision 1.3  2000/09/13 03:09:52  frehe
 * Added more documentation to the class.
 *
 * Revision 1.2  2000/09/05 11:09:14  frehe
 * Made the generatePlan method return a Plan instead of a Command*.
 *
 * Revision 1.1  2000/09/03 22:08:44  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file Skill.h
 * Include file for the class Skill.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.3 $
 * $Author: frehe $
 * $Date: 2000/09/13 03:09:52 $
 */


#ifndef SKILL_H
#define SKILL_H


#include "ViewManager.h"
#include "Plan.h"
#include "FuzzyBool.h"


RS_BEGIN_NAMESPACE


/**
 * @class Skill Skill.h
 * This class is the abstract base class for all skills
 * in the RoboSoc system.
 *
 * <!-- @invariant The class invariant is... -->
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.3 $
 * $Author: frehe $
 * $Date: 2000/09/13 03:09:52 $
 */
class Skill {
public:
  /** Constructs a skill with default behavior. */
  Skill();

  /** Destroys the skill. */
  virtual ~Skill();

  
  /** Generate a plan for how to accomplish the goal
    of the skill. */
  virtual Plan generatePlan() = 0;

  
  /** Return @c true (close to 1) if the skill would like
    to be called again. */
  virtual FuzzyBool persistent() = 0;

  /** Return @c true (close to 1) of the skill is applicable
    at the moment, i.e. its preconditions are satisfied. */
  virtual FuzzyBool applicable() = 0;
  

protected:
  /** A pointer to the view manager. */
  ViewManager* const viewManager;

  
private:
  // Hide copy-constructor and assignment
  Skill(const Skill&);
  Skill& operator= (const Skill&);
};


RS_END_NAMESPACE


#endif
