/* -*- Mode: C++ -*-
 * $Id: ViewData.h,v 1.1 2000/09/03 22:08:44 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: ViewData.h
 * Unitname: Framework
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/03 22:08:44 $
 *
 *
 * HISTORY
 *
 * $Log: ViewData.h,v $
 * Revision 1.1  2000/09/03 22:08:44  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file ViewData.h
 * Include file for the class ViewData.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/03 22:08:44 $
 */


#ifndef VIEWDATA_H
#define VIEWDATA_H


#include "machine_dependent_types.h"


RS_BEGIN_NAMESPACE


class View;


/**
 * @class ViewData ViewData.h
 * Used to represent one entry in the ViewManager,
 * it contains one view and some information about
 * it like how many references to it exist in the system.
 * If no references exist and the persistence flag is not
 * set then the view is deleted from the system.
 *
 * <!-- @invariant The class invariant is... -->
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/03 22:08:44 $
 */
class ViewData {

public:
  ViewData(View* const view, bool persist = true);
  ViewData(const ViewData& obj);

  View* getViewPointer();
  //const View* const getViewPointer() const; // Useless?

  bool isUsed() const;
  //bool isPersistent() const; // Useless?
  void setPersistence(const bool new_val);

  //UInt16 getReferenceCount() const; // Useless?
  void increaseReferenceCount();
  void decreaseReferenceCount();

private:
  View* const theView;
  UInt16 referenceCount;
  bool persistent;

  // Hide copy-constructor and assignment.
  ViewData& operator= (const ViewData&);
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "ViewData.icc"
#endif


#endif // !VIEWDATA_H
