/* -*- Mode: C++ -*-
 * $Id: ViewManager.h,v 1.1 2000/09/03 22:08:45 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: ViewManager.h
 * Unitname: Framework
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/03 22:08:45 $
 *
 *
 * HISTORY
 *
 * $Log: ViewManager.h,v $
 * Revision 1.1  2000/09/03 22:08:45  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file ViewManager.h
 * Include file for the class ViewManager.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/03 22:08:45 $
 */


#ifndef VIEWMANAGER_H
#define VIEWMANAGER_H


#include <map>

#include "library_types.h"
#include "ViewData.h"
#include "View.h"


RS_BEGIN_NAMESPACE


/**
 * @class ViewManager ViewManager.h
 * This is the view manager in the RoboSoc system. It is a model of the
 * design pattern Singleton, which means that there is only one instance
 * of this class in the system. This instance can only be accessed from
 * the ViewManager::instance() method, which returns a pointer to the only
 * instance of the class.
 *
 * <!-- @invariant The class invariant is... -->
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/03 22:08:45 $
 */
class ViewManager {
public:
  /** Retreive the only instance of the ViewManager class. */
  static ViewManager* instance();

  bool addView(View* view, const bool persitent = true);
  View* getView(const ViewId v_id);
  void releaseView(const ViewId v_id);

  void setViewPersistence(ViewId v_id, const bool persistence);
  
  UInt32 getCurrentUpdateNumber() const;

  void newCycle();
  void updateAfterCoachMessage();
  void updateAfterInit();
  void updateAfterSee();
  void updateAfterHear();
  void updateAfterSense();
  void updateAfterCommand();

  void forEach(void (*fn)(View*));
  
protected:

  std::map<ViewId, ViewData> theViews;
  UInt32 currentUpdateNumber;

  void forEach(ViewUpdateResult (View::*fn)(const UInt32), const UInt32);

private:
  ViewManager();
  static ViewManager* onlyInstance;

  // Hide copy-constructor and assignment.
  ViewManager(const ViewManager&);
  ViewManager& operator= (const ViewManager&);
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "ViewManager.icc"
#endif


#endif // !VIEWMANAGER_H
