/* -*- Mode: C++ -*-
 * $Id: ViewWithHistory.h,v 1.2 2001/01/10 15:12:58 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: ViewWithHistory.h
 * Unitname: Framework
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 2000-09-12
 * Last modified by $Author: olalu526 $ $Date: 2001/01/10 15:12:58 $
 *
 *
 * HISTORY
 *
 * $Log: ViewWithHistory.h,v $
 * Revision 1.2  2001/01/10 15:12:58  olalu526
 * Added support for WorldModel to some basic classes.
 *
 * Revision 1.1  2000/09/13 00:02:53  frehe
 * Added a new base class called ViewWithHistory for all views that use a history.
 *
 */

/**
 * @file ViewWithHistory.h
 * Include file for the class ViewWithHistory.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.2 $
 * $Author: olalu526 $
 * $Date: 2001/01/10 15:12:58 $
 */


#ifndef VIEWWITHHISTORY_H
#define VIEWWITHHISTORY_H


#include "HistoryDefault.h"
#include "View.h"

RS_BEGIN_NAMESPACE


/**
 * @class ViewWithHistory ViewWithHistory.h
 * This class is the abstract base class for all views that
 * have a history in the RoboSoc system.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.2 $
 * $Author: olalu526 $
 * $Date: 2001/01/10 15:12:58 $
 */
class ViewWithHistory : public View {
public:
  /** Construct a view with ViewId @a id and the history
    length @a hist_len. */
  ViewWithHistory(const ViewId id,
		  const UInt hist_len = DEFAULT_HISTORY_LENGTH);

  /** Destroy the view. */
  virtual ~ViewWithHistory();

  
protected:
  /** Return the actual index to the time step
    @a i steps from the current time step.
    If @a i is -1 then it means the previous time step
    and if @a i is 1 then it means the next time step.
    OBS! Since the history is implemented as a circular
    array getIndex will return a value that between 0 and
    getHistoryLength-1. It also means that getIndex(1) is the last
    element in the array and getIndex(0) = getIndex(getHistoryLength). */
  UInt getIndex(const Int i) const;

  /** Increase the index to point to the next time step. */
  void increaseIndex();

  
  /** Returns the length of the history, which is the maximum
    number of steps in the past that will be saved. */
  UInt getHistoryLength() const;

private:
  /** The length of the history. */
  UInt historyLength;

  /** The current observed time step. */
  UInt currentObservation;

  
  // Hide copy-constructor and assignment.
  ViewWithHistory(const ViewWithHistory&);
  ViewWithHistory& operator= (const ViewWithHistory&);
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "ViewWithHistory.icc"
#endif


#endif // !VIEWWITHHISTORY_H
