/* -*- Mode: C++ -*-
 * $Id: ViewWithSimulation.h,v 1.4 2001/01/12 12:41:24 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: ViewWithSimulation.h
 * Unitname: Framework
 * $Revision: 1.4 $
 * Created by: Ola Lundqvist 2001-01-03
 * Last modified by $Author: olalu526 $
 *
 *
 * HISTORY
 *
 * $Log: ViewWithSimulation.h,v $
 * Revision 1.4  2001/01/12 12:41:24  olalu526
 * Added some documentation.
 *
 * Revision 1.3  2001/01/12 12:40:40  olalu526
 * Added some documentation.
 *
 * Revision 1.2  2001/01/11 22:00:18  olalu526
 * Added the real world model with support for new simulation steps.
 *
 * Revision 1.1  2001/01/10 17:00:25  olalu526
 * Added the world model.
 *
 */

/**
 * @file ViewWithSimulation.h
 * Include file for the class ViewWithSimulation.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.4 $
 * $Author: olalu526 $
 * $Date: 2001/01/12 12:41:24 $
 */


#ifndef VIEWWITHSIMULATION_H
#define VIEWWITHSIMULATION_H


#include "HistoryDefault.h"
#include "View.h"
#include "WorldModelInterface.h"

RS_BEGIN_NAMESPACE


/**
 * @class ViewWithSimulation ViewWithSimulation.h
 * This class is the abstract base class for all views that
 * have a world model in the RoboSoc system.
 *
 * It is basicly just a view with a reference to the world model interface.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.4 $
 * $Author: olalu526 $
 * $Date: 2001/01/12 12:41:24 $
 */
class ViewWithSimulation : public View {
public:
  /** Construct a view with ViewId @a id and the history
    length @a hist_len. */
  ViewWithSimulation(const ViewId id);

  /** Destroy the view. */
  virtual ~ViewWithSimulation();

  
protected:
  /** The reference to the world model interface that the view will get most of
      its information from. */
  WorldModelInterface* worldModel;

private:
  // Hide copy-constructor and assignment.
  ViewWithSimulation(const ViewWithSimulation&);
  ViewWithSimulation& operator= (const ViewWithSimulation&);
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "ViewWithSimulation.icc"
#endif


#endif // !VIEWWITHSIMULATION_H
