/* -*- Mode: C++ -*-
 * $Id: WorldModel.h,v 1.3 2001/01/12 12:40:41 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *                     2001 Ola Lundqvist, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: WorldModel.h
 * Unitname: Framework
 * $Revision: 1.3 $
 * Created by: Ola Lundqvist 2001-01-02
 * Last modified by $Author: olalu526 $
 *
 *
 * HISTORY
 *
 * $Log: WorldModel.h,v $
 * Revision 1.3  2001/01/12 12:40:41  olalu526
 * Added some documentation.
 *
 *
 */

/**
 * @file WorldModel.h
 * Include file for the class WorldModel.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.3 $
 * $Author: olalu526 $
 * $Date: 2001/01/12 12:40:41 $
 */

#ifndef __WORLD_MODEL__
#define __WORLD_MODEL__

#include "WorldModelTypes.h"
#include "HistoryDefault.h"
#include "ParameterView.h"
#include "ViewWithHistory.h"

RS_BEGIN_NAMESPACE

/**
 * @class WorldModel WorldModel.h
 * This class holds most information about the agents surrounding environment.
 * This class should _not_ be used directly. Always use the world model interface
 * to allow world model exchange.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.3 $
 * $Author: olalu526 $
 * $Date: 2001/01/12 12:40:41 $
 */

class WorldModel {
 public:
  /** The constructor that needs a parameter view to set some defaults. */
  WorldModel(ParameterView*, const UInt hist_len = DEFAULT_HISTORY_LENGTH);
  /** The destructor, does it do anything? */
  ~WorldModel();
  
  /** The play mode data, used by for example GameView. */
  PlayModeCollection*	playModeData();
  UInt			getPlayModeIndex(const int) const;
  void			setPlayModeHistoryLength(const int);
  UInt			getPlayModeHistoryLength();
  void			increasePlayModeIndex();
  
  /** The ball data, used by for example BallView. */
  BallCollection*	ballData();
  UInt			getBallIndex(const int) const;
  void			setBallHistoryLength(const int);
  UInt			getBallHistoryLength();
  void			increaseBallIndex();

  /** The opponnets information, used by for example PlayerView. */
  PlayersCollection*	opponentsData();
  UInt			getOpponentsIndex(const int) const;
  void			setOpponentsHistoryLength(const int);
  UInt			getOpponentsHistoryLength();
  void			increaseOpponentsIndex();

  /** The teammates information, used by for example PlayerView. */
  PlayersCollection*	teammatesData();
  UInt			getTeammatesIndex(const int) const;
  void			setTeammatesHistoryLength(const int);
  UInt			getTeammatesHistoryLength();
  void			increaseTeammatesIndex();

  /** The unknown players information, used by for example PlayerView. */
  PlayersCollection*	unknownPlayersData();
  UInt			getUnknownPlayersIndex(const int) const;
  void			setUnknownPlayersHistoryLength(const int);
  UInt			getUnknownPlayersHistoryLength();
  void			increaseUnknownPlayersIndex();

  /** The closest opponent, used by for example PlayerView. */
  UIntUnknownCollection* closestOpponentIndexData();
  UInt			getClosestOpponentIndexIndex(const int) const;
  void			setClosestOpponentIndexHistoryLength(const int);
  UInt			getClosestOpponentIndexHistoryLength();
  void			increaseClosestOpponentIndexIndex();

  /** The closest teammate, used by for example PlayerView. */
  UIntUnknownCollection* closestTeammateIndexData();
  UInt			getClosestTeammateIndexIndex(const int) const;
  void			setClosestTeammateIndexHistoryLength(const int);
  UInt			getClosestTeammateIndexHistoryLength();
  void			increaseClosestTeammateIndexIndex();

  /** The closest unknown player, used by for example PlayerView. */
  UIntUnknownCollection* closestUnknownPlayerIndexData();
  UInt			getClosestUnknownPlayerIndexIndex(const int) const;
  void			setClosestUnknownPlayerIndexHistoryLength(const int);
  UInt			getClosestUnknownPlayerIndexHistoryLength();
  void			increaseClosestUnknownPlayerIndexIndex();

  /** The information about the agent itself, used by for example AgentView. */
  AgentCollection*	agentData();
  UInt			getAgentIndex(const int) const;
  void			setAgentHistoryLength(const int);
  UInt			getAgentHistoryLength();
  void			increaseAgentIndex();

 protected:

  /** The history of the play modes. */
  PlayModeCollection	playModes;
  int currentPlayModeObservation;
  int playModeHistoryLength;

  /** The history of the ball. */
  BallCollection	ball;
  int currentBallObservation;
  int ballHistoryLength;

  /** Stores all opponents seen a cycle.
   No matching is currently done between cycles. */
  PlayersCollection	opponents;
  int currentOpponentsObservation;
  int opponentsHistoryLength;

  /** Stores all teammates seen a cycle.
   No matching is currently done between cycles. */  
  PlayersCollection	teammates;
  int currentTeammatesObservation;
  int teammatesHistoryLength;

  /** Stores all unknown players seen a cycle.
   No matching is currently done between cycles. */  
  PlayersCollection	unknowns;
  int currentUnknownPlayersObservation;
  int unknownPlayersHistoryLength;

  /** The index in @a opponents that is the closest
    of those players. */
  UIntUnknownCollection closestOpponentIndex;
  int currentClosestOpponentIndexObservation;
  int closestOpponentIndexHistoryLength;
  
  /** The index in @a teammates that is the closest
    of those players. */
  UIntUnknownCollection closestTeammateIndex;
  int currentClosestTeammateIndexObservation;
  int closestTeammateIndexHistoryLength;  

  /** The index in @a unknownPlayers that is the closest
    of those players. */
  UIntUnknownCollection closestUnknownPlayerIndex;
  int currentClosestUnknownPlayerIndexObservation;
  int closestUnknownPlayerIndexHistoryLength;

  /** The history of the agent. */
  AgentCollection	agent;
  int currentAgentObservation;
  int agentHistoryLength;
};

RS_END_NAMESPACE

#endif
