/* -*- Mode: C++ -*-
 * $Id: WorldModelInterface.h,v 1.3 2001/01/12 12:40:41 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *                     2001 Ola Lundqvist, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: WorldModelInterface.h
 * Unitname: Framework
 * $Revision: 1.3 $
 * Created by: Ola Lundqvist 2001-01-02
 * Last modified by $Author: olalu526 $
 *
 *
 * HISTORY
 *
 * $Log: WorldModelInterface.h,v $
 * Revision 1.3  2001/01/12 12:40:41  olalu526
 * Added some documentation.
 *
 *
 */

/**
 * @file WorldModelInterface.h
 * Include file for the class WorldModelInterface.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.3 $
 * $Author: olalu526 $
 * $Date: 2001/01/12 12:40:41 $
 */

#ifndef __WORLD_MODEL_INTERFACE__
#define __WORLD_MODEL_INTERFACE__

#include "WorldModelTypes.h"
#include "WorldModel.h"
#include "ViewManager.h"
#include "view_constants.h"
#include "ParameterView.h"

RS_BEGIN_NAMESPACE

class WorldModel;

/**
 * @class WorldModelInterface WorldModelInterface.h
 * This the interface class to the world model. This class is used for all communication
 * with the world model.
 * The world model should _not_ be used directly. Always use the world model interface
 * to allow world model exchange.
 *
 * The interface allows world model exchange that can be _very_ useful in simulation
 * of the behaviour of other agents. One simple way is to run the decision maker on
 * a world model that is a copy of the ordinary one but with the difference that
 * the Agent information is exchanged with some of the players in opponents, teammates
 * or unknown players.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.3 $
 * $Author: olalu526 $
 * $Date: 2001/01/12 12:40:41 $
 */

class WorldModelInterface {
 public:
  /** The constructor, will set up a default world model. */
  WorldModelInterface();
  /** The constructor, will use a predefined world model. */
  WorldModelInterface(WorldModel*);

  /** Desctructor. */
  ~WorldModelInterface();

  /** The method to use to get the world model interface reference. There should
      only be _one_ interface running. */
  static WorldModelInterface* instance();
  
  /** Set the world model to use to a different one.
      OBS! Do not set to a invalid world model (like null) bacause there is no
      protection against that. The program will probably crush instantly.
  */
  void setWorldModel(WorldModel*);
  /** Get the world model interface reference.
      OBS! Do not save this one because it can be changed with the setWorldModel
      and thus you can change things in the wrong world model. So follow the
      principle, do not use the world model directly, use the interface instead.

      This one should only be used for holding the world model for restoring the
      values or to get the difference between different world models.
  */
  WorldModel* getWorldModel();

  /** Exactly the same definition as in WorldModel, see that instead. */
  PlayModeCollection*	playModeData();
  /** Exactly the same definition as in WorldModel, see that instead. */
  UInt			getPlayModeIndex(const int) const;
  /** Exactly the same definition as in WorldModel, see that instead. */
  void			setPlayModeHistoryLength(const int);
  /** Exactly the same definition as in WorldModel, see that instead. */
  UInt			getPlayModeHistoryLength();
  /** Exactly the same definition as in WorldModel, see that instead. */
  void			increasePlayModeIndex();
  /** Exactly the same definition as in WorldModel, see that instead. */
  BallCollection*	ballData();
  /** Exactly the same definition as in WorldModel, see that instead. */
  UInt			getBallIndex(const int) const;
  /** Exactly the same definition as in WorldModel, see that instead. */
  void			setBallHistoryLength(const int);
  /** Exactly the same definition as in WorldModel, see that instead. */
  UInt			getBallHistoryLength();
  /** Exactly the same definition as in WorldModel, see that instead. */
  void			increaseBallIndex();
  /** Exactly the same definition as in WorldModel, see that instead. */
  PlayersCollection*	opponentsData();
  /** Exactly the same definition as in WorldModel, see that instead. */
  UInt			getOpponentsIndex(const int) const;
  /** Exactly the same definition as in WorldModel, see that instead. */
  void			setOpponentsHistoryLength(const int);
  /** Exactly the same definition as in WorldModel, see that instead. */
  UInt			getOpponentsHistoryLength();
  /** Exactly the same definition as in WorldModel, see that instead. */
  void			increaseOpponentsIndex();
  /** Exactly the same definition as in WorldModel, see that instead. */
  PlayersCollection*	teammatesData();
  /** Exactly the same definition as in WorldModel, see that instead. */
  UInt			getTeammatesIndex(const int) const;
  /** Exactly the same definition as in WorldModel, see that instead. */
  void			setTeammatesHistoryLength(const int);
  /** Exactly the same definition as in WorldModel, see that instead. */
  UInt			getTeammatesHistoryLength();
  /** Exactly the same definition as in WorldModel, see that instead. */
  void			increaseTeammatesIndex();
  /** Exactly the same definition as in WorldModel, see that instead. */
  PlayersCollection*	unknownPlayersData();
  /** Exactly the same definition as in WorldModel, see that instead. */
  UInt			getUnknownPlayersIndex(const int) const;
  /** Exactly the same definition as in WorldModel, see that instead. */
  void			setUnknownPlayersHistoryLength(const int);
  /** Exactly the same definition as in WorldModel, see that instead. */
  UInt			getUnknownPlayersHistoryLength();
  /** Exactly the same definition as in WorldModel, see that instead. */
  void			increaseUnknownPlayersIndex();
  /** Exactly the same definition as in WorldModel, see that instead. */
  UIntUnknownCollection* closestOpponentIndexData();
  /** Exactly the same definition as in WorldModel, see that instead. */
  UInt			getClosestOpponentIndexIndex(const int) const;
  /** Exactly the same definition as in WorldModel, see that instead. */
  void			setClosestOpponentIndexHistoryLength(const int);
  /** Exactly the same definition as in WorldModel, see that instead. */
  UInt			getClosestOpponentIndexHistoryLength();
  /** Exactly the same definition as in WorldModel, see that instead. */
  void			increaseClosestOpponentIndexIndex();
  /** Exactly the same definition as in WorldModel, see that instead. */
  UIntUnknownCollection* closestTeammateIndexData();
  /** Exactly the same definition as in WorldModel, see that instead. */
  UInt			getClosestTeammateIndexIndex(const int) const;
  /** Exactly the same definition as in WorldModel, see that instead. */
  void			setClosestTeammateIndexHistoryLength(const int);
  /** Exactly the same definition as in WorldModel, see that instead. */
  UInt			getClosestTeammateIndexHistoryLength();
  /** Exactly the same definition as in WorldModel, see that instead. */
  void			increaseClosestTeammateIndexIndex();
  /** Exactly the same definition as in WorldModel, see that instead. */
  UIntUnknownCollection* closestUnknownPlayerIndexData();
  /** Exactly the same definition as in WorldModel, see that instead. */
  UInt			getClosestUnknownPlayerIndexIndex(const int) const;
  /** Exactly the same definition as in WorldModel, see that instead. */
  void			setClosestUnknownPlayerIndexHistoryLength(const int);
  /** Exactly the same definition as in WorldModel, see that instead. */
  UInt			getClosestUnknownPlayerIndexHistoryLength();
  /** Exactly the same definition as in WorldModel, see that instead. */
  void			increaseClosestUnknownPlayerIndexIndex();
  /** Exactly the same definition as in WorldModel, see that instead. */
  AgentCollection*	agentData();
  /** Exactly the same definition as in WorldModel, see that instead. */
  UInt			getAgentIndex(const int) const;
  /** Exactly the same definition as in WorldModel, see that instead. */
  void			setAgentHistoryLength(const int);
  /** Exactly the same definition as in WorldModel, see that instead. */
  UInt			getAgentHistoryLength();
  /** Exactly the same definition as in WorldModel, see that instead. */
  void			increaseAgentIndex();

 protected:
  /** The static reference container to the only world model interface that should
      be used. */
  static WorldModelInterface* onlyInstance;
  /** The parameter view. This should probably be removed because it is only used
      in the constructor. */
  ParameterView* parameterView;
  /** The world model reference to use. */
  WorldModel* worldModel;
};

RS_END_NAMESPACE

#endif
