/* -*- Mode: C++ -*-
 * $Id: AbleToKickBallToPoint.cc,v 1.4 2001/04/16 11:01:52 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <danbe628@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~danbe628/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: AbleToKickBallToPoint.cc
 * Unitname: Framework/predicate
 * $Revision: 1.4 $
 * Created by: Fredrik Heintz 2000-09-12
 * Last modified by $Author: frehe $ $Date: 2001/04/16 11:01:52 $
 *
 *
 * HISTORY
 *
 * $Log: AbleToKickBallToPoint.cc,v $
 * Revision 1.4  2001/04/16 11:01:52  frehe
 * Made minor changes.
 *
 * Revision 1.3  2000/11/09 18:56:41  marsc818
 * voila
 *
 * Revision 1.2  2000/10/12 03:00:03  marsc818
 * also changed in some way
 *
 * Revision 1.1  2000/09/27 13:52:29  marsc818
 * Added the kick view and abletokickballtopoint predicate.
 *
 * Revision 1.2  2000/09/17 14:31:38  danbe628
 * Made minor updates to the AbleToKickBallToPoint.
 *
 * Revision 1.1  2000/09/13 00:06:00  danbe628
 * Added a predicate teamplate called AbleToKickBallToPoint.
 *
 */

/**
 * @file AbleToKickBallToPoint.cc
 * Implementation file for the predicate class AbleToKickBallToPoint.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.4 $
 * $Author: frehe $
 * $Date: 2001/04/16 11:01:52 $
 */

#include "view_constants.h"

#include "AbleToKickBallToPoint.h"


RS_BEGIN_NAMESPACE


AbleToKickBallToPoint::AbleToKickBallToPoint()
  : PredicateWithArguments(),
    ballView(NULL)
{
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID, parameterView);
  getView(ballView, BALL_VIEW_ID);
  getView(kickView, KICK_VIEW_ID);
}


AbleToKickBallToPoint::AbleToKickBallToPoint(const AbleToKickBallToPoint& obj)
  : PredicateWithArguments(obj),
    parameterView(obj.parameterView),
    agentView(obj.agentView),
    ballView(obj.ballView),
    kickView(obj.kickView),
    point(obj.point)
{
}


AbleToKickBallToPoint::~AbleToKickBallToPoint()
{
  viewManager->releaseView(PARAMETER_VIEW_ID /*parameterView->getId()*/);
  viewManager->releaseView(AGENT_VIEW_ID/*ballView->getId()*/);
  viewManager->releaseView(BALL_VIEW_ID/*agentView->getId()*/);
  viewManager->releaseView(KICK_VIEW_ID/*kickView->getId()*/);
}

void AbleToKickBallToPoint:: setArguments(const std::vector<Float>& arguments)
  throw(TooFewArgumentsException)
{
  point.setX(arguments[0]);
  point.setY(arguments[1]);
}

AbleToKickBallToPoint& AbleToKickBallToPoint::operator= (const AbleToKickBallToPoint& obj)
{
  if ( this != &obj ) {
    Predicate::operator=(obj);

    parameterView = obj.parameterView;
    ballView = obj.ballView;
    agentView = obj.agentView;
    kickView = obj.kickView;
    
    point = obj.point;
  }

  return *this;
}


FuzzyBool AbleToKickBallToPoint::evaluate()
{
  // we check if the ball we kick might reach the point given as parameter for that predicate

  // 1. get distance from agent to point
  PointUnknown agentPos = static_cast<PointUnknown>( agentView->getAgentPosition(NOW) );
  
  if( agentPos.isUnknown() )
    return UNKNOWN;
  
  Float pointDist = sqrt( pow(agentPos.getX().getValue() - point.getX(), 2) 
		     + pow(agentPos.getY().getValue() - point.getY(),2) );

  // 2. get the max kick dist that agent is able to achieve
  Float maxDist = kickView->howFarAgentCanKick();

  // RS_LOG(LA_PREDICATE, "Predicate (AbleToKickBallToPoint);  ptDist = " << pointDist << "  agent far kick = " << maxDist);

  // 3. check if dist is ok
  return maxDist >= pointDist;
}


void AbleToKickBallToPoint::update(const UInt32)
{
}


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "AbleToKickBallToPoint.icc"
#endif
