/* -*- Mode: C++ -*-
 * $Id: AbleToKickBallToPoint.h,v 1.3 2001/04/16 11:01:52 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <danbe628@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~danbe628/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: AbleToKickBallToPoint.h
 * Unitname: Framework/predicate
 * $Revision: 1.3 $
 * Created by: Fredrik Heintz 2000-09-12
 * Last modified by $Author: frehe $ $Date: 2001/04/16 11:01:52 $
 *
 *
 * HISTORY
 *
 *
 */

/**
 * @file AbleToKickBallToPoint.h
 * Include file for the predicate class AbleToKickBallToPoint.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.3 $
 * $Author: frehe $
 * $Date: 2001/04/16 11:01:52 $
 */


#ifndef PREDICATE_AbleToKickBallToPoint_H
#define PREDICATE_AbleToKickBallToPoint_H


#include "ParameterView.h"
#include "AgentView.h"
#include "BallView.h"
#include "KickView.h"

#include "PredicateWithArguments.h"

//#include <library_types.h>


RS_BEGIN_NAMESPACE


/**
 * @class AbleToKickBallToPoint AbleToKickBallToPoint.h
 * The first sentence is a brief description of the skill class SkillStub.
 * The rest is a longer description of the skill class SkillStub.
 * Use it to describe the skill!
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.3 $
 * $Author: frehe $
 * $Date: 2001/04/16 11:01:52 $
 */
class AbleToKickBallToPoint : public PredicateWithArguments {
public:
  /** Create a predicate with default behavior. */
  AbleToKickBallToPoint();

  /** Creates a predicate that is an exact copy of @a obj. */
  AbleToKickBallToPoint(const AbleToKickBallToPoint& obj);

  /** Destroys the predicate. */  
  virtual ~AbleToKickBallToPoint();

  /** Assignment operator. Assigns @a obj to this predicate. */
  AbleToKickBallToPoint& operator= (const AbleToKickBallToPoint& obj);


  virtual void setArguments(const std::vector<Float>& arguments)
    throw(TooFewArgumentsException);


  /** Function application operator. Will call evaluate(). */
  virtual FuzzyBool operator()();


  /** Should return the truth value of the predicate. */
  virtual FuzzyBool evaluate();

  
protected:
  /** The update function, should be used to update the value of the
    predicate if necessary. The method needUpdate(const UInt32 update_number)
    can be used to check if the predicate needs to be updated based on the
    last update number it was updated for. */
  virtual void update(const UInt32 update_number);

private:
  /** used views */
  ParameterView* parameterView;
  AgentView* agentView;
  BallView* ballView;
  KickView* kickView;
  CoordPoint<Float> point;
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "AbleToKickBallToPoint.icc"
#endif


#endif // !PREDICATE_AbleToKickBallToPoint_H
