/* -*- Mode: C++ -*-
 * $Id: AgentInZone.cc,v 1.6 2000/11/07 17:25:37 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: AgentInZone.cc
 * Unitname: Framework/predicate
 * $Revision: 1.6 $
 * Created by: Ola Lundqvist 2000-09-22
 * Last modified by $Author: olalu526 $ $Date: 2000/11/07 17:25:37 $
 *
 *
 * HISTORY
 *
 * $Log: AgentInZone.cc,v $
 * Revision 1.6  2000/11/07 17:25:37  olalu526
 * Generalized the zone concept to handle various type of objects. It now handles a set of rectangles and circles to form an area. This affected quite a lot of skills.
 *
 * Revision 1.5  2000/10/07 19:49:39  olalu526
 * Just minor things.
 *
 * Revision 1.4  2000/09/28 13:32:37  olalu526
 * A lot of changes.
 *
 * Revision 1.3  2000/09/24 10:47:55  olalu526
 * Added PredicateWithZone.cc and all other changes.
 *
 * Revision 1.2  2000/09/23 17:21:13  olalu526
 * Did some changes.
 *
 * Revision 1.1  2000/09/23 12:59:28  olalu526
 * Added AgentInZone prediate.
 *
 */

/**
 * @file AgentInZone.cc
 * Implementation file for the predicate class AgentInZone.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.6 $
 * $Author: olalu526 $
 * $Date: 2000/11/07 17:25:37 $
 */


#include "AgentInZone.h"

RS_BEGIN_NAMESPACE


AgentInZone::AgentInZone(const Zone &z)
  : PredicateWithZone(z),
    parameterView(NULL),
    agentView(NULL)
{
  lastknown = FALSE;
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID, parameterView);
}

AgentInZone::AgentInZone(const AgentInZone& obj)
  : PredicateWithZone(obj),
    parameterView(NULL),
    agentView(NULL)
{
  lastknown = FALSE;
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID, parameterView);
}

AgentInZone::~AgentInZone()
{
  viewManager->releaseView(AGENT_VIEW_ID);
  viewManager->releaseView(PARAMETER_VIEW_ID);
}


AgentInZone& AgentInZone::operator= (const AgentInZone& obj)
{
  if ( this != &obj ) {
    PredicateWithZone::operator=(obj);
  }
  agentView = obj.agentView;
  
  return *this;
}

/* This should be changed! */
FuzzyBool AgentInZone::evaluate()
{
  /* If the position of the agent is inside the zone, return true,
   * else return false.
   */
  if ((agentView->getAgentPosition(NOW)).isKnown()) {
    if (zone.inArea((agentView->getAgentPosition(NOW)).getPoint())) {
      //cout << "IZ!" << flush;
      lastknown = TRUE;
    }
    else {
      //cout << "OZ!" << flush;
      lastknown = FALSE;
    }
  }  
  return lastknown;
}

/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "AgentInZone.icc"
#endif


RS_END_NAMESPACE


