/* -*- Mode: C++ -*-
 * $Id: AgentInZone.h,v 1.5 2000/09/28 13:32:37 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: AgentInZone.h
 * Unitname: Framework/predicate
 * $Revision: 1.5 $
 * Created by: Ola Lundqvist 2000-09-22
 * Last modified by $Author: olalu526 $ $Date: 2000/09/28 13:32:37 $
 *
 *
 * HISTORY
 *
 * $Log: AgentInZone.h,v $
 * Revision 1.5  2000/09/28 13:32:37  olalu526
 * A lot of changes.
 *
 * Revision 1.4  2000/09/25 16:26:08  olalu526
 * Update.
 *
 * Revision 1.3  2000/09/24 10:47:55  olalu526
 * Added PredicateWithZone.cc and all other changes.
 *
 * Revision 1.2  2000/09/23 17:21:13  olalu526
 * Did some changes.
 *
 * Revision 1.1  2000/09/23 12:59:29  olalu526
 * Added AgentInZone prediate.
 *
 * Revision 1.1  2000/09/23 09:08:23  olalu526
 * Added PredicateWithZone and AgentInZone predicates.
 *
 */

/**
 * @file AgentInZone.h
 * Include file for the predicate class AgentInZone.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.5 $
 * $Author: olalu526 $
 * $Date: 2000/09/28 13:32:37 $
 */


#ifndef AGENTINZONE_H
#define AGENTINZONE_H


#include "view_constants.h"
#include "PredicateWithZone.h"
#include "AgentView.h"
#include "ParameterView.h"
#include "../skill/Zone.h"


RS_BEGIN_NAMESPACE


/**
 * @class AgentInZone AgentInZone.h
 * A predicate for asking if the Agent is inside a specific zone.
 * Returns true if the agent is inside the zone, and false elsewhere.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.5 $
 * $Author: olalu526 $
 * $Date: 2000/09/28 13:32:37 $
 */
class AgentInZone : public PredicateWithZone {
public:
  /** Create a predicate with default behavior. */
  AgentInZone(const Zone &z);

  /** Creates a predicate that is an exact copy of @a obj. */
  AgentInZone(const AgentInZone& obj);

  /** Destroys the predicate. */  
  ~AgentInZone();

  /** Assignment operator. Assigns @a obj to this predicate. */
  AgentInZone& operator= (const AgentInZone& obj);

  virtual FuzzyBool evaluate();

protected:
  /** The update function, should be used to update the value of the
    predicate if necessary. The method needUpdate(const UInt32 update_number)
    can be used to check if the predicate needs to be updated based on the
    last update number it was updated for. */
  //virtual void update(const UInt32 update_number);
  ParameterView *parameterView;
  AgentView *agentView;
  FuzzyBool lastknown;
};

RS_END_NAMESPACE

/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "AgentInZone.icc"
#endif

#endif // !AGENTINZONE_H
