/* -*- Mode: C++ -*-
 * $Id: BallDirectedToZone.h,v 1.2 2000/10/10 20:38:50 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: BallDirectedToZone.h
 * Unitname: Framework/predicate
 * $Revision: 1.2 $
 * Created by: Ola Lundqvist 2000-09-22
 * Last modified by $Author: olalu526 $ $Date: 2000/10/10 20:38:50 $
 *
 *
 * HISTORY
 *
 * $Log: BallDirectedToZone.h,v $
 * Revision 1.2  2000/10/10 20:38:50  olalu526
 * Added predicate IsBallDirectedBad and documented BallStopInZone.
 *
 * Revision 1.1  2000/09/28 15:13:59  olalu526
 * Added BallDirectedToZone.
 *
 * Revision 1.4  2000/09/25 16:26:08  olalu526
 * Update.
 *
 * Revision 1.3  2000/09/23 17:21:14  olalu526
 * Did some changes.
 *
 * Revision 1.2  2000/09/23 12:43:32  olalu526
 * Fixed some with PredicateWithZone and BallDirectedToZone to make it more flexible.
 *
 * Revision 1.1  2000/09/23 09:08:23  olalu526
 * Added PredicateWithZone and BallDirectedToZone predicates.
 *
 */

/**
 * @file BallDirectedToZone.h
 * Include file for the predicate class BallDirectedToZone.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.2 $
 * $Author: olalu526 $
 * $Date: 2000/10/10 20:38:50 $
 */


#ifndef BALLDIRECTEDTOZONE_H
#define BALLDIRECTEDTOZONE_H


#include "PredicateWithZone.h"
#include "BallView.h"
#include "../skill/Zone.h"

RS_BEGIN_NAMESPACE


/**
 * @class BallDirectedToZone BallDirectedToZone.h
 * A predicate for asking if the Ball is heading for a specific zone.
 * Returns true if is directed for the zone, and false elsewhere.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.2 $
 * $Author: olalu526 $
 * $Date: 2000/10/10 20:38:50 $
 */
class BallDirectedToZone : public PredicateWithZone {
public:
  /** Create a predicate with default behavior. */
  BallDirectedToZone(const Zone &z);

  /** Creates a predicate that is an exact copy of @a obj. */
  BallDirectedToZone(const BallDirectedToZone& obj);

  /** Destroys the predicate. */  
  virtual ~BallDirectedToZone();

  /** Assignment operator. Assigns @a obj to this predicate. */
  BallDirectedToZone& operator= (const BallDirectedToZone& obj);

  /** Function application operator. Will call evaluate(). */
  virtual FuzzyBool operator()();


  /** Returns true if the ball is heading for the zone. */
  virtual FuzzyBool evaluate();

  
protected:
  /** The update function, should be used to update the value of the
    predicate if necessary. The method needUpdate(const UInt32 update_number)
    can be used to check if the predicate needs to be updated based on the
    last update number it was updated for. */
  virtual void update(const UInt32 update_number);
  BallView *ballView;
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "BallDirectedToZone.icc"
#endif


#endif // !BALLDIRECTEDTOZONE_H
