/* -*- Mode: C++ -*-
 * $Id: BallInZone.cc,v 1.4 2000/11/07 17:25:37 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: BallInZone.cc
 * Unitname: Framework/predicate
 * $Revision: 1.4 $
 * Created by: Ola Lundqvist 2000-09-22
 * Last modified by $Author: olalu526 $ $Date: 2000/11/07 17:25:37 $
 *
 *
 * HISTORY
 *
 * $Log: BallInZone.cc,v $
 * Revision 1.4  2000/11/07 17:25:37  olalu526
 * Generalized the zone concept to handle various type of objects. It now handles a set of rectangles and circles to form an area. This affected quite a lot of skills.
 *
 * Revision 1.3  2000/09/28 13:32:38  olalu526
 * A lot of changes.
 *
 * Revision 1.2  2000/09/23 12:30:08  olalu526
 * Updated BallInZone. It should now work, but is not tested yet.
 *
 * Revision 1.1  2000/09/23 09:08:22  olalu526
 * Added PredicateWithZone and BallInZone predicates.
 *
 * Revision 1.2  2000/09/17 14:31:38 olalu526 
 * Made minor updates to the BallInZone.
 *
 * Revision 1.1  2000/09/13 00:06:00 olalu526 
 * Added a predicate teamplate called BallInZone.
 *
 */

/**
 * @file BallInZone.cc
 * Implementation file for the predicate class BallInZone.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.4 $
 * $Author: olalu526 $
 * $Date: 2000/11/07 17:25:37 $
 */


#include "BallInZone.h"
#include "view_constants.h"

RS_BEGIN_NAMESPACE


BallInZone::BallInZone(const Zone &z)
  : PredicateWithZone(z),
    ballView(NULL)
{
  getView(ballView, BALL_VIEW_ID);
}

BallInZone::BallInZone(const BallInZone& obj)
  : PredicateWithZone(obj),
    ballView(NULL)
{
  getView(ballView, BALL_VIEW_ID);
}


BallInZone::~BallInZone()
{
  viewManager->releaseView(BALL_VIEW_ID);
}


BallInZone& BallInZone::operator= (const BallInZone& obj)
{
  if ( this != &obj ) {
    PredicateWithZone::operator=(obj);
  }
  ballView = obj.ballView;
  
  return *this;
}


/* This should be changed! */
FuzzyBool BallInZone::evaluate()
{
  /* If the position of the ball is inside the zone, return true,
   * else return false.
   */
  if ((ballView->getBallPosition(NOW)).isKnown()) {
    if (zone.inArea((ballView->getBallPosition(NOW)).getPoint())) {
      return TRUE;
    }
  }
  return FALSE;
}


void BallInZone::update(const UInt32)
{
}


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "BallInZone.icc"
#endif
