/* -*- Mode: C++ -*-
 * $Id: BallKickedToMe.cc,v 1.1 2000/11/01 17:34:12 mikbr212 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: BallKickedToMe.cc
 * Unitname: Framework/predicate
 * $Revision: 1.1 $
 *
 *
 * HISTORY
 *
 * $Log: BallKickedToMe.cc,v $
 * Revision 1.1  2000/11/01 17:34:12  mikbr212
 * created
 *
 */

/**
 * @file BallKickedToMe.cc
 * Implementation file for the predicate class BallKickedToMe.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.1 $
 * $Author: mikbr212 $
 * $Date: 2000/11/01 17:34:12 $
 */


#include "BallKickedToMe.h"


RS_BEGIN_NAMESPACE


BallKickedToMe::BallKickedToMe()
  : Predicate(),
    parameterView(NULL),
    agentView(NULL),
    kickBallView(NULL),
    time(10)
{
  init();
  range = parameterView->SP_kickable_area * 2.0;
}

BallKickedToMe::BallKickedToMe(Float r, int t)
  : Predicate(),
    parameterView(NULL),
    agentView(NULL),
    kickBallView(NULL),
    range(r),
    time(t)
{
  init();
}


BallKickedToMe::BallKickedToMe(const BallKickedToMe& obj)
  : Predicate(obj),
    parameterView(NULL),
    agentView(NULL),
    kickBallView(NULL),
    range()
{
  init();

  range = obj.getRange();
  time = obj.getTime();
}

void BallKickedToMe::init() {
    BallView *ballView = NULL;
  CommandView *commandView = NULL;

  getView(parameterView, PARAMETER_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID, parameterView);
  getView(ballView, BALL_VIEW_ID);
  getView(commandView, COMMAND_VIEW_ID);
  getView(kickBallView, KICKBALL_VIEW_ID, parameterView,
	  ballView, agentView, commandView);
}

BallKickedToMe::~BallKickedToMe()
{
  viewManager->releaseView(PARAMETER_VIEW_ID);
  viewManager->releaseView(COMMAND_VIEW_ID);
  viewManager->releaseView(BALL_VIEW_ID);
  viewManager->releaseView(AGENT_VIEW_ID);
  viewManager->releaseView(KICKBALL_VIEW_ID);
}


BallKickedToMe& BallKickedToMe::operator= (const BallKickedToMe& obj)
{
  if ( this != &obj ) {
    Predicate::operator=(obj);
  }
  range = obj.getRange();
  time = obj.getTime();
  return *this;
}


FuzzyBool BallKickedToMe::evaluate()
{
  Vector ballPos, ballSpeed;
  if(!kickBallView->initBall(ballSpeed, ballPos))
    return false;

  ballSpeed *= (1 - pow(parameterView->SP_ball_decay, time)) /
    (1 - parameterView->SP_ball_decay);

  return Line(Point(ballPos),ballSpeed).isThroughCircle(Point(0,0),range);
}

int BallKickedToMe::getTime() const {
  return time;
}

void BallKickedToMe::setTime(int t) {
  time = t;
}

Float BallKickedToMe::getRange() const {
  return range;
}

void BallKickedToMe::setRange(Float r) {
  range = r;
}

void BallKickedToMe::update(const UInt32)
{
}


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "BallKickedToMe.icc"
#endif
