/* -*- Mode: C++ -*-
 * $Id: BallKickedToMe.h,v 1.1 2000/11/01 17:34:13 mikbr212 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: BallKickedToMe.h
 * Unitname: Framework/predicate
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 2000-09-12
 * Last modified by $Author: mikbr212 $ $Date: 2000/11/01 17:34:13 $
 *
 *
 * HISTORY
 *
 * $Log: BallKickedToMe.h,v $
 * Revision 1.1  2000/11/01 17:34:13  mikbr212
 * created
 *
 */

/**
 * @file BallKickedToMe.h
 * Include file for the predicate class BallKickedToMe.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.1 $
 * $Author: mikbr212 $
 * $Date: 2000/11/01 17:34:13 $
 */


#ifndef BALLKICKEDTOME_H
#define BALLKICKEDTOME_H


#include "Predicate.h"
#include "AgentView.h"
#include "ParameterView.h"
#include "KickBallView.h"
#include "view_constants.h"

RS_BEGIN_NAMESPACE


/**
 * @class BallKickedToMe BallKickedToMe.h
 * Returns true if the ball is moving toward the agent (within a specified time).
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.1 $
 * $Author: mikbr212 $
 * $Date: 2000/11/01 17:34:13 $
 */
class BallKickedToMe : public Predicate {
public:
  /** Create a predicate with default behavior. */
  BallKickedToMe();

  /** Constructor */
  BallKickedToMe(Float, int);

  /** Creates a predicate that is an exact copy of @a obj. */
  BallKickedToMe(const BallKickedToMe& obj);

  /** Destroys the predicate. */  
  virtual ~BallKickedToMe();


  /** Assignment operator. Assigns @a obj to this predicate. */
  BallKickedToMe& operator= (const BallKickedToMe& obj);


  /** Function application operator. Will call evaluate(). */
  virtual FuzzyBool operator()();


  /** Should return the truth value of the predicate. */
  virtual FuzzyBool evaluate();


  /** Sets the range defining what's close. */
  virtual void setRange(Float);

  /** Gets the range defining what's close. */
  virtual Float getRange() const;
  

  /** Sets the time. */
  virtual void setTime(int);

  /** Gets the time. */
  virtual int getTime() const;
  
protected:
  /** The update function, should be used to update the value of the
    predicate if necessary. The method needUpdate(const UInt32 update_number)
    can be used to check if the predicate needs to be updated based on the
    last update number it was updated for. */
  virtual void update(const UInt32 update_number);
  
  void init();

  ParameterView *parameterView;
  AgentView *agentView;
  KickBallView *kickBallView;
  Float range;
  int time;
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "BallKickedToMe.icc"
#endif


#endif // !BALLKICKEDTOME_H
