/* -*- Mode: C++ -*-
 * $Id: BallStopInZone.cc,v 1.2 2000/11/07 17:25:37 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: BallStopInZone.cc
 * Unitname: Framework/predicate
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 2000-09-12
 * Last modified by $Author: olalu526 $ $Date: 2000/11/07 17:25:37 $
 *
 *
 * HISTORY
 *
 * $Log: BallStopInZone.cc,v $
 * Revision 1.2  2000/11/07 17:25:37  olalu526
 * Generalized the zone concept to handle various type of objects. It now handles a set of rectangles and circles to form an area. This affected quite a lot of skills.
 *
 * Revision 1.1  2000/10/10 19:49:42  olalu526
 * Added BallStopInZone predicate.
 *
 *
 */

/**
 * @file BallStopInZone.cc
 * Implementation file for the predicate class BallStopInZone.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.2 $
 * $Author: olalu526 $
 * $Date: 2000/11/07 17:25:37 $
 */


#include "BallStopInZone.h"


RS_BEGIN_NAMESPACE


BallStopInZone::BallStopInZone(Zone z)
  : PredicateWithZone(z),
    ballInFuture()
{
  getView(ballInFuture, FUTUREBALL_VIEW_ID);
}


BallStopInZone::BallStopInZone(const BallStopInZone& obj)
  : PredicateWithZone(obj),
    ballInFuture()
{
  getView(ballInFuture, FUTUREBALL_VIEW_ID);
}


BallStopInZone::~BallStopInZone()
{
  viewManager->releaseView(FUTUREBALL_VIEW_ID);
}


BallStopInZone& BallStopInZone::operator= (const BallStopInZone& obj)
{
  if ( this != &obj ) {
    Predicate::operator=(obj);
  }

  return *this;
}


FuzzyBool BallStopInZone::evaluate()
{
  Point p = ballInFuture->fastBallStopPosition();
  if (getZone().inArea(p)) {
    return TRUE;
  }
  return FALSE;
}

void BallStopInZone::update(const UInt32)
{
}

RS_END_NAMESPACE


