/* -*- Mode: C++ -*-
 * $Id: BallStopInZone.h,v 1.2 2000/10/10 20:38:51 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: BallStopInZone.h
 * Unitname: Framework/predicate
 * $Revision: 1.2 $
 * Created by: Ola Lundqvist 2000-10-10
 * Last modified by $Author: olalu526 $ $Date: 2000/10/10 20:38:51 $
 *
 *
 * HISTORY
 *
 * $Log: BallStopInZone.h,v $
 * Revision 1.2  2000/10/10 20:38:51  olalu526
 * Added predicate IsBallDirectedBad and documented BallStopInZone.
 *
 * Revision 1.1  2000/10/10 19:49:42  olalu526
 * Added BallStopInZone predicate.
 *
 */

/**
 * @file BallStopInZone.h
 * Include file for the predicate class BallStopInZone.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.2 $
 * $Author: olalu526 $
 * $Date: 2000/10/10 20:38:51 $
 */


#ifndef BALLSTOPINZONE_H
#define BALLSTOPINZONE_H


#include "PredicateWithZone.h"
#include "view_constants.h"
#include "FutureBallView.h"

RS_BEGIN_NAMESPACE


/**
 * @class BallStopInZone BallStopInZone.h
 * Returns true if the ball will (or is estimated to) stop inside the zone.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.2 $
 * $Author: olalu526 $
 * $Date: 2000/10/10 20:38:51 $
 */
class BallStopInZone : public PredicateWithZone {
public:
  /** Create a predicate with default behavior. */
  BallStopInZone(Zone);

  /** Creates a predicate that is an exact copy of @a obj. */
  BallStopInZone(const BallStopInZone& obj);

  /** Destroys the predicate. */  
  virtual ~BallStopInZone();


  /** Assignment operator. Assigns @a obj to this predicate. */
  BallStopInZone& operator= (const BallStopInZone& obj);


  /** Should return true if the ball will (or is estimated to) stop inside the zone. */
  virtual FuzzyBool evaluate();

  
protected:
  /** The update function, should be used to update the value of the
    predicate if necessary. The method needUpdate(const UInt32 update_number)
    can be used to check if the predicate needs to be updated based on the
    last update number it was updated for. */
  virtual void update(const UInt32 update_number);
  FutureBallView* ballInFuture;
};


RS_END_NAMESPACE


#endif // !BALLSTOPINZONE_H
