/* -*- Mode: C++ -*-
 * $Id: CanShootToVector.cc,v 1.2 2000/11/01 13:10:38 mikbr212 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: CanShootToVector.cc
 * Unitname: Framework/predicate
 * $Revision: 1.2 $
 * Created by: 
 * Last modified by $Author: mikbr212 $ $Date: 2000/11/01 13:10:38 $
 *
 *
 * HISTORY
 *
 * $Log: CanShootToVector.cc,v $
 * Revision 1.2  2000/11/01 13:10:38  mikbr212
 * fixed evaluate
 *
 * Revision 1.1  2000/11/01 12:41:36  mikbr212
 * Created (used in KickBallToSpeedVectorMulti)
 *
 */

/**
 * @file CanShootToVector.cc
 * Implementation file for the predicate class CanShootToVector.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.2 $
 * $Author: mikbr212 $
 * $Date: 2000/11/01 13:10:38 $
 */


#include "CanShootToVector.h"

RS_BEGIN_NAMESPACE


CanShootToVector::CanShootToVector(const Vector& vec)
  : PredicateWithVector(vec),
    ballView(NULL),
    parameterView(NULL)
{
  getView(ballView, BALL_VIEW_ID);
  getView(parameterView, PARAMETER_VIEW_ID);
}


CanShootToVector::CanShootToVector(const CanShootToVector& obj)
  : PredicateWithVector(obj),
    ballView(NULL),
    parameterView(NULL)
{
    getView(ballView, BALL_VIEW_ID);
    getView(parameterView, PARAMETER_VIEW_ID);
}


CanShootToVector::~CanShootToVector()
{
  viewManager->releaseView(BALL_VIEW_ID);
  viewManager->releaseView(PARAMETER_VIEW_ID);
}

CanShootToVector& CanShootToVector::operator= (const CanShootToVector& obj)
{
  if ( this != &obj ) {
    Predicate::operator=(obj);
  }
  ballView = obj.ballView;
  
  return *this;
}

FuzzyBool CanShootToVector::evaluate()
{
  if(ballView->getRelativeBallVector(NOW).isUnknown())
    return false;

  Line straightLine = Line(Point(ballView->getRelativeBallVector(NOW).getVector()),
			   vector);
  return !straightLine.isThroughCircle(Point(0,0),parameterView->SP_player_size +
				       parameterView->SP_ball_size);
}


void CanShootToVector::update(const UInt32)
{
}


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "CanShootToVector.icc"
#endif
