/* -*- Mode: C++ -*-
 * $Id: CanShootToVector.h,v 1.1 2000/11/01 12:41:37 mikbr212 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: CanShootToVector.h
 * Unitname: Framework/predicate
 * $Revision: 1.1 $
 * Last modified by $Author: mikbr212 $ $Date: 2000/11/01 12:41:37 $
 *
 *
 * HISTORY
 *
 * $Log: CanShootToVector.h,v $
 * Revision 1.1  2000/11/01 12:41:37  mikbr212
 * Created (used in KickBallToSpeedVectorMulti)
 *
 */

/**
 * @file CanShootToVector.h
 * Include file for the predicate class CanShootToVector.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.1 $
 * $Author: mikbr212 $
 * $Date: 2000/11/01 12:41:37 $
 */


#ifndef CANSHOOTTOVECTOR_H
#define CANSHOOTTOVECTOR_H


#include "PredicateWithVector.h"
#include "BallView.h"
#include "ParameterView.h"
#include "Line.h"
#include "view_constants.h"

RS_BEGIN_NAMESPACE


/**
 * @class CanShootToVector CanShootToVector.h
 * Is the ball position known?
 * This predicate check if the ball position is known.
 *
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.1 $
 * $Author: mikbr212 $
 * $Date: 2000/11/01 12:41:37 $
 */
class CanShootToVector : public PredicateWithVector {
public:
  /** Create a predicate with default behavior. */
  CanShootToVector(const Vector& vec);

  /** Creates a predicate that is an exact copy of @a obj. */
  CanShootToVector(const CanShootToVector& obj);

  /** Destroys the predicate. */  
  virtual ~CanShootToVector();


  /** Assignment operator. Assigns @a obj to this predicate. */
  CanShootToVector& operator= (const CanShootToVector& obj);


  /** Function application operator. Will call evaluate(). */
  virtual FuzzyBool operator()();

  virtual FuzzyBool operator()(const Vector& vec);

  /** Should return the truth value of the predicate. */
  virtual FuzzyBool evaluate();


  
protected:
  /** The update function, should be used to update the value of the
    predicate if necessary. The method needUpdate(const UInt32 update_number)
    can be used to check if the predicate needs to be updated based on the
    last update number it was updated for. */
  virtual void update(const UInt32 update_number);

  /** A view that contain information about the ball. */
  BallView *ballView;
  ParameterView *parameterView;
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "CanShootToVector.icc"
#endif


#endif // !CANSHOOTTOVECTOR_H
