/* -*- Mode: C++ -*-
 * $Id: IAmClosestToBall.cc,v 1.2 2000/11/08 17:54:20 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: IAmClosestToBall.cc
 * Unitname: Framework/predicate
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 2000-09-20
 * Last modified by $Author: olalu526 $ $Date: 2000/11/08 17:54:20 $
 *
 *
 * HISTORY
 *
 * $Log: IAmClosestToBall.cc,v $
 * Revision 1.2  2000/11/08 17:54:20  olalu526
 * This fixes a LOT of problems.
 *
 * Revision 1.1  2000/09/20 14:42:48  frehe
 * Added the predicate IAmClosestToBall that will return true iff the agent is the closest player to the ball.
 *
 */

/**
 * @file IAmClosestToBall.cc
 * Implementation file for the predicate class IAmClosestToBall.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.2 $
 * $Author: olalu526 $
 * $Date: 2000/11/08 17:54:20 $
 */


#include "IAmClosestToBall.h"

#include "BallView.h"
#include "PlayerView.h"
#include "view_constants.h"



RS_BEGIN_NAMESPACE


IAmClosestToBall::IAmClosestToBall()
  : Predicate(),
    ballView(NULL),
    playerView(NULL),
    closerPlayers(0)
{
  getView(ballView, BALL_VIEW_ID);
  getView(playerView, PLAYER_VIEW_ID);
}


IAmClosestToBall::IAmClosestToBall(const IAmClosestToBall& obj)
  : Predicate(obj),
    ballView(obj.ballView),
    playerView(obj.playerView),
    closerPlayers(obj.closerPlayers)
{
}


IAmClosestToBall::~IAmClosestToBall()
{
  viewManager->releaseView(ballView->getId());
  viewManager->releaseView(playerView->getId());
}


IAmClosestToBall& IAmClosestToBall::operator= (const IAmClosestToBall& obj)
{
  if ( this != &obj ) {
    Predicate::operator=(obj);
    ballView = obj.ballView;
    playerView = obj.playerView;
    closerPlayers = obj.closerPlayers;
  }

  return *this;
}


FuzzyBool IAmClosestToBall::evaluate()
{
  FloatUnknown myDistToBall = ballView->getBallDistance(NOW);
  FloatUnknown distToBall;
  std::vector<PlayerObject> teammates;
  std::vector<PlayerObject>::iterator p;
  int nearerPlayers = 0;

  // If cant see the ball
  if (myDistToBall.isUnknown())
    return FALSE;

  teammates = playerView->getTeammates(NOW);
  //pout.Printf("ClosestToBall: myDist=%f\n", myDistToBall.GetValue());
  //pout.Printf("ClosestToBall: players=%d\n", teammates.size());
  for ( p = teammates.begin(); p != teammates.end(); ++p ) {
    distToBall = p->getDistanceTo(ballView->getBallPosition(NOW));
    //if (distToBall.isKnown())
    //  pout.Printf("ClosestToBall: heDist=%f\n", distToBall.getValue());
    if ((distToBall.isKnown()) && (distToBall < myDistToBall))
      nearerPlayers++;
  }

  //pout.Printf("ClosestToBall: nearer players=%d\n", nearerPlayers);
  //cerr << "ClosestToBall: nearer players=" << nearerPlayers << std::endl;

  if (nearerPlayers <= closerPlayers)
    return TRUE;

  return FALSE;
}


/*
void CImClosestToBall::SetArguments(std::vector<t_float> arguments)
{
  
  if (arguments.size() >= 1) {
    closerPlayers = (int)arguments[0];
  }

  if (arguments.size() >= 2) {
    // TODO: a = (int)arguments[1];
  }

}
*/


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "IAmClosestToBall.icc"
#endif
