/* -*- Mode: C++ -*-
 * $Id: IAmClosestToBall.h,v 1.1 2000/09/20 14:42:48 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: IAmClosestToBall.h
 * Unitname: Framework/predicate
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 2000-09-20
 * Last modified by $Author: frehe $ $Date: 2000/09/20 14:42:48 $
 *
 *
 * HISTORY
 *
 * $Log: IAmClosestToBall.h,v $
 * Revision 1.1  2000/09/20 14:42:48  frehe
 * Added the predicate IAmClosestToBall that will return true iff the agent is the closest player to the ball.
 *
 */

/**
 * @file IAmClosestToBall.h
 * Include file for the predicate class IAmClosestToBall.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/20 14:42:48 $
 */


#ifndef IAMCLOSESTTOBALL_H
#define IAMCLOSESTTOBALL_H


#include "Predicate.h"


RS_BEGIN_NAMESPACE


class BallView;
class PlayerView;


/**
 * @class IAmClosestToBall IAmClosestToBall.h
 * This predicate returns @c true iff the agent is the closest player
 * to the ball.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/20 14:42:48 $
 */
class IAmClosestToBall : public Predicate {
public:
  /** Create a predicate with default behavior. */
  IAmClosestToBall();

  /** Creates a predicate that is an exact copy of @a obj. */
  IAmClosestToBall(const IAmClosestToBall& obj);

  /** Destroys the predicate. */  
  virtual ~IAmClosestToBall();


  /** Assignment operator. Assigns @a obj to this predicate. */
  IAmClosestToBall& operator= (const IAmClosestToBall& obj);


  /** Function application operator. Will call evaluate(). */
  virtual FuzzyBool operator()();


  /** Will return @c true iff the agent is the closest player to the ball. */
  virtual FuzzyBool evaluate();

  
protected:
  /** A pointer to the ball view. */
  BallView* ballView;

  /** A pointer to the player view. */
  PlayerView* playerView;


  /** The update function, should be used to update the value of the
    predicate if necessary. The method needUpdate(const UInt32 update_number)
    can be used to check if the predicate needs to be updated based on the
    last update number it was updated for. */
  virtual void update(const UInt32 update_number);

  
  /** The number of closer players allowed. */
  UInt closerPlayers;
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "IAmClosestToBall.icc"
#endif


#endif // !IAMCLOSESTTOBALL_H
