/* -*- Mode: C++ -*-
 * $Id: IAmGoalie.cc,v 1.2 2000/09/20 14:41:55 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: IAmGoalie.cc
 * Unitname: Framework/predicate
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 2000-09-18
 * Last modified by $Author: frehe $ $Date: 2000/09/20 14:41:55 $
 *
 *
 * HISTORY
 *
 * $Log: IAmGoalie.cc,v $
 * Revision 1.2  2000/09/20 14:41:55  frehe
 * Fixed a bug, got ball view instead of agent view.
 *
 * Revision 1.1  2000/09/19 13:56:07  frehe
 * Added the predicate IAmGoalie which will return true iff the agent is the goal keeper.
 *
 */

/**
 * @file IAmGoalie.cc
 * Implementation file for the predicate class IAmGoalie.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2000/09/20 14:41:55 $
 */


#include "IAmGoalie.h"
#include "AgentView.h"
#include "ParameterView.h"
#include "view_constants.h"


RS_BEGIN_NAMESPACE


IAmGoalie::IAmGoalie()
  : Predicate(),
    agentView(NULL),
    parameterView(NULL)
{
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID, parameterView);
}


IAmGoalie::IAmGoalie(const IAmGoalie& obj)
  : Predicate(obj),
    agentView(obj.agentView),
    parameterView(obj.parameterView)
{
}


IAmGoalie::~IAmGoalie()
{
  viewManager->releaseView(agentView->getId());
  viewManager->releaseView(parameterView->getId());
}


IAmGoalie& IAmGoalie::operator= (const IAmGoalie& obj)
{
  if ( this != &obj ) {
    Predicate::operator=(obj);
    agentView = obj.agentView;
    parameterView = obj.parameterView;
  }

  return *this;
}


FuzzyBool IAmGoalie::evaluate()
{
  return FuzzyBool(agentView->isGoalie());
}


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "IAmGoalie.icc"
#endif
