/* -*- Mode: C++ -*-
 * $Id: IAmGoalie.h,v 1.1 2000/09/19 13:56:07 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: IAmGoalie.h
 * Unitname: Framework/predicate
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 2000-09-18
 * Last modified by $Author: frehe $ $Date: 2000/09/19 13:56:07 $
 *
 *
 * HISTORY
 *
 * $Log: IAmGoalie.h,v $
 * Revision 1.1  2000/09/19 13:56:07  frehe
 * Added the predicate IAmGoalie which will return true iff the agent is the goal keeper.
 *
 */

/**
 * @file IAmGoalie.h
 * Include file for the predicate class IAmGoalie.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/19 13:56:07 $
 */


#ifndef IAMGOALIE_H
#define IAMGOALIE_H


#include "Predicate.h"


RS_BEGIN_NAMESPACE


class ParameterView;
class AgentView;


/**
 * @class IAmGoalie IAmGoalie.h
 * This predicate returns @c true iff the agent is the goal keeper.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/19 13:56:07 $
 */
class IAmGoalie : public Predicate {
public:
  /** Create a predicate with default behavior. */
  IAmGoalie();

  /** Creates a predicate that is an exact copy of @a obj. */
  IAmGoalie(const IAmGoalie& obj);

  /** Destroys the predicate. */  
  virtual ~IAmGoalie();


  /** Assignment operator. Assigns @a obj to this predicate. */
  IAmGoalie& operator= (const IAmGoalie& obj);


  /** Function application operator. Will call evaluate(). */
  virtual FuzzyBool operator()();


  /** Will return @c true iff the agent is the goal keeper. */
  virtual FuzzyBool evaluate();

  
protected:
  /** A pointer to the agent view. */
  AgentView* agentView;

  /** A pointer to the parameter view. */
  ParameterView* parameterView;

  
  /** The update function, should be used to update the value of the
    predicate if necessary. The method needUpdate(const UInt32 update_number)
    can be used to check if the predicate needs to be updated based on the
    last update number it was updated for. */
  virtual void update(const UInt32 update_number);
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "IAmGoalie.icc"
#endif


#endif // !IAMGOALIE_H

