/* -*- Mode: C++ -*-
 * $Id: InGoalArea.cc,v 1.1 2000/09/26 16:24:32 bjoli035 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: InGoalArea.cc
 * Unitname: Framework/predicate
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 2000-09-12
 * Last modified by $Author: bjoli035 $ $Date: 2000/09/26 16:24:32 $
 *
 *
 * HISTORY
 *
 * $Log: InGoalArea.cc,v $
 * Revision 1.1  2000/09/26 16:24:32  bjoli035
 * Predicate used in the skill ReturnToGoalArea
 *
 * Revision 1.2  2000/09/17 14:31:38  frehe
 * Made minor updates to the InGoalArea.
 *
 * Revision 1.1  2000/09/13 00:06:00  frehe
 * Added a predicate teamplate called InGoalArea.
 *
 */

/**
 * @file InGoalArea.cc
 * Implementation file for the predicate class InGoalArea.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.1 $
 * $Author: bjoli035 $
 * $Date: 2000/09/26 16:24:32 $
 */


#include "InGoalArea.h"
#include "AgentView.h"
#include "MarkerView.h"
#include "view_constants.h"
#include "ParameterView.h"


RS_BEGIN_NAMESPACE


InGoalArea::InGoalArea()
  : Predicate(),
    parameterView(NULL),
    agentView(NULL),
    markerView(NULL)

{
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID, parameterView);
  getView(markerView, MARKER_VIEW_ID, parameterView);
}


InGoalArea::InGoalArea(const InGoalArea& obj)
  : Predicate(obj),
    parameterView(obj.parameterView),
    agentView(obj.agentView),
    markerView(obj.markerView)
{

}


InGoalArea::~InGoalArea()
{
  viewManager->releaseView(parameterView->getId());
  viewManager->releaseView(agentView->getId());
  viewManager->releaseView(markerView->getId());
}


InGoalArea& InGoalArea::operator= (const InGoalArea& obj)
{
  if ( this != &obj ) {
    Predicate::operator=(obj);
    parameterView = obj.parameterView;
    agentView = obj.agentView;
    markerView = obj.markerView;
  }

  return *this;
}


FuzzyBool InGoalArea::evaluate()
{
  PointUCO UnknownPos; 
  Point Pos;
  const Float MIN_X_IN_GOAL_AREA = -52.5;
  const Float MAX_X_IN_GOAL_AREA = -36;
  const Float MIN_Y_IN_GOAL_AREA = -20.16;
  const Float MAX_Y_IN_GOAL_AREA = 20.16;

  UnknownPos= agentView->getAgentPosition(NOW);
  if (UnknownPos.isKnown())
    Pos = UnknownPos.getPoint();
  else
      return FALSE;
  
  return (((MIN_X_IN_GOAL_AREA <= Pos.getX()) && 
	   (Pos.getX() <= MAX_X_IN_GOAL_AREA)) &&
	  ((MIN_Y_IN_GOAL_AREA <= Pos.getY()) &&
	   (Pos.getY() <= MAX_Y_IN_GOAL_AREA)));
    
}


void InGoalArea::update(const UInt32)
{
}


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "InGoalArea.icc"
#endif
