/* -*- Modex: C++ -*-
 * $Id: InPenaltyArea.cc,v 1.1 2000/10/08 11:34:53 olale507 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: InPenaltyArea.cc
 * Unitname: Framework/predicate
 * $Revision: 1.1 $
 * Created by: Bjrn, Ola och Jenny 2000-09-12
 * Last modified by $Author: olale507 $ $Date: 2000/10/08 11:34:53 $
 *
 *
 * HISTORY
 *
 */

/**
 * @file InPenaltyArea.cc
 * Implementation file for the predicate class InPenaltyArea.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.1 $
 * $Author: olale507 $
 * $Date: 2000/10/08 11:34:53 $
 */


#include "InPenaltyArea.h"
#include "AgentView.h"
#include "MarkerView.h"
#include "view_constants.h"
#include "ParameterView.h"


RS_BEGIN_NAMESPACE


InPenaltyArea::InPenaltyArea()
  : Predicate(),
    parameterView(NULL),
    agentView(NULL),
    markerView(NULL)
{
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID, parameterView);
  getView(markerView, MARKER_VIEW_ID, parameterView);
}

InPenaltyArea::InPenaltyArea(const InPenaltyArea& obj)
  : Predicate(obj),
    parameterView(obj.parameterView),
    agentView(obj.agentView),
    markerView(obj.markerView)
{
}


InPenaltyArea::~InPenaltyArea()
{
  viewManager->releaseView(parameterView->getId());
  viewManager->releaseView(agentView->getId());
  viewManager->releaseView(markerView->getId());
}


InPenaltyArea& InPenaltyArea::operator= (const InPenaltyArea& obj)
{
  if ( this != &obj ) {
    Predicate::operator=(obj);
    parameterView = obj.parameterView;
    agentView = obj.agentView;
    markerView = obj.markerView;
  }
  
  return *this;
}


FuzzyBool InPenaltyArea::evaluate()
{
  PointUCO UnknownPos; 
  Point Pos;
  // These are the edges of the penaltyarea.
  const Float MIN_X_IN_PENALTY_AREA = -52.5;
  const Float MAX_X_IN_PENALTY_AREA = -36;
  const Float MIN_Y_IN_PENALTY_AREA = -20.16;
  const Float MAX_Y_IN_PENALTY_AREA = 20.16;
  
  UnknownPos= agentView->getAgentPosition(NOW);
  if (UnknownPos.isKnown())
    Pos = UnknownPos.getPoint();
  else
    return FALSE;
  
  return (((MIN_X_IN_PENALTY_AREA <= Pos.getX()) && 
	   (Pos.getX() <= MAX_X_IN_PENALTY_AREA)) &&
	  ((MIN_Y_IN_PENALTY_AREA <= Pos.getY()) &&
	   (Pos.getY() <= MAX_Y_IN_PENALTY_AREA)));
}


void InPenaltyArea::update(const UInt32)
{
}

RS_END_NAMESPACE
