/* -*- Mode: C++ -*-
 * $Id: InPenaltyArea.h,v 1.1 2000/10/08 11:34:52 olale507 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: InPenaltyArea.h
 * Unitname: Framework/predicate
 * $Revision: 1.1 $
 * Created by: Bjrn, Ola och Jenny 2000-09-12
 * Last modified by $Author: olale507 $ $Date: 2000/10/08 11:34:52 $
 *
 *
 * HISTORY
 *
 *
 */

/**
 * @file InPenaltyArea.h
 * Include file for the predicate class InPenaltyArea.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.1 $
 * $Author: olale507 $
 * $Date: 2000/10/08 11:34:52 $
 */


#ifndef INPENALTYAREA_H
#define INPENALTYAREA_H


#include "Predicate.h"

RS_BEGIN_NAMESPACE

class ParameterView;
class AgentView;
class MarkerView;


/**
 * @class InPenaltyArea InPenaltyArea.h
 * Checks if the agent is in the penalty area.
 * If the agents position is known a check is run to see if it's
 * inside the penalty area. If it's unknown the result is false.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.1 $
 * $Author: olale507 $
 * $Date: 2000/10/08 11:34:52 $ */
class InPenaltyArea : public Predicate {
public:
  /** Create a predicate with default behavior. */
  InPenaltyArea();

  /** Creates a predicate that is an exact copy of @a obj. */
  InPenaltyArea(const InPenaltyArea& obj);

  /** Destroys the predicate. */  
  virtual ~InPenaltyArea();


  /** Assignment operator. Assigns @a obj to this predicate. */
  InPenaltyArea& operator= (const InPenaltyArea& obj);


  /** Function application operator. Will call evaluate(). */
  //  virtual FuzzyBool operator()();


  /** Should return the truth value of the predicate. */
  virtual FuzzyBool evaluate();

  
protected:
  /** A pointer to the parameter view. */
  ParameterView* parameterView;

  /** A pointer to the agent view. */
  AgentView* agentView;

  /** A pointer to the agent view. */
  MarkerView* markerView;


  /** The update function, should be used to update the value of the
    predicate if necessary. The method needUpdate(const UInt32 update_number)
    can be used to check if the predicate needs to be updated based on the
    last update number it was updated for. */
  virtual void update(const UInt32 update_number);
};


RS_END_NAMESPACE



#endif // !INPENALTYAREA_H
