/* -*- Mode: C++ -*-
 * $Id: IsBallDirectedBad.cc,v 1.1 2000/10/10 20:38:51 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: IsBallDirectedBad.cc
 * Unitname: Framework/predicate
 * $Revision: 1.1 $
 * Created by: Ola Lundqvist 2000-10-10
 * Last modified by $Author: olalu526 $ $Date: 2000/10/10 20:38:51 $
 *
 *
 * HISTORY
 *

 *
 */

/**
 * @file IsBallDirectedBad.cc
 * Implementation file for the predicate class IsBallDirectedBad.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.1 $
 * $Author: olalu526 $
 * $Date: 2000/10/10 20:38:51 $
 */


#include "IsBallDirectedBad.h"


RS_BEGIN_NAMESPACE


IsBallDirectedBad::IsBallDirectedBad()
  : Predicate(),
    ballView(NULL)
{
  getView(ballView, BALL_VIEW_ID);
}


IsBallDirectedBad::IsBallDirectedBad(const IsBallDirectedBad& obj)
  : Predicate(obj),
    ballView(NULL)
{
  getView(ballView, BALL_VIEW_ID);
}


IsBallDirectedBad::~IsBallDirectedBad()
{
  viewManager->releaseView(BALL_VIEW_ID);
}


IsBallDirectedBad& IsBallDirectedBad::operator= (const IsBallDirectedBad& obj)
{
  if ( this != &obj ) {
    Predicate::operator=(obj);
  }

  return *this;
}


FuzzyBool IsBallDirectedBad::evaluate()
{
  AngleDegUCO a = ballView->getAbsoluteBallDirection(NOW);
  if (a.isKnown()) {
    AngleDeg aa = a.getAngle().getAngle().getAngle();
    if (aa + AngleDeg(Float(90)) < AngleDeg(Float(0))) {
      return TRUE;
    }
  }
  return FALSE;
}


void IsBallDirectedBad::update(const UInt32)
{
}


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "IsBallDirectedBad.icc"
#endif
