/* -*- Mode: C++ -*-
 * $Id: IsBallDirectedBad.h,v 1.1 2000/10/10 20:38:51 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: IsBallDirectedBad.h
 * Unitname: Framework/predicate
 * $Revision: 1.1 $
 * Created by: Ola Lundqvist 2000-10-10
 * Last modified by $Author: olalu526 $ $Date: 2000/10/10 20:38:51 $
 *
 *
 * HISTORY
 *
 * $Log: IsBallDirectedBad.h,v $
 * Revision 1.1  2000/10/10 20:38:51  olalu526
 * Added predicate IsBallDirectedBad and documented BallStopInZone.
 *
 *
 */

/**
 * @file IsBallDirectedBad.h
 * Include file for the predicate class IsBallDirectedBad.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.1 $
 * $Author: olalu526 $
 * $Date: 2000/10/10 20:38:51 $
 */


#ifndef ISBALLDIRECTEDBAD_H
#define ISBALLDIRECTEDBAD_H


#include "Predicate.h"
#include "BallView.h"
#include "view_constants.h"


RS_BEGIN_NAMESPACE


/**
 * @class IsBallDirectedBad IsBallDirectedBad.h
 * Returns true if the ball is directed to the agents team side.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.1 $
 * $Author: olalu526 $
 * $Date: 2000/10/10 20:38:51 $
 */
class IsBallDirectedBad : public Predicate {
public:
  /** Create a predicate with default behavior. */
  IsBallDirectedBad();

  /** Creates a predicate that is an exact copy of @a obj. */
  IsBallDirectedBad(const IsBallDirectedBad& obj);

  /** Destroys the predicate. */  
  virtual ~IsBallDirectedBad();


  /** Assignment operator. Assigns @a obj to this predicate. */
  IsBallDirectedBad& operator= (const IsBallDirectedBad& obj);


  /** Function application operator. Will call evaluate(). */
  virtual FuzzyBool operator()();


  /** Should return the truth value of the predicate. */
  virtual FuzzyBool evaluate();

  
protected:
  /** The update function, should be used to update the value of the
    predicate if necessary. The method needUpdate(const UInt32 update_number)
    can be used to check if the predicate needs to be updated based on the
    last update number it was updated for. */
  virtual void update(const UInt32 update_number);
  BallView *ballView;
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "IsBallDirectedBad.icc"
#endif


#endif // !ISBALLDIRECTEDBAD_H
