/* -*- Mode: C++ -*-
 * $Id: IsBallDirectedGood.cc,v 1.2 2000/10/11 10:39:29 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: IsBallDirectedGood.cc
 * Unitname: Framework/predicate
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 2000-09-12
 * Last modified by $Author: olalu526 $ $Date: 2000/10/11 10:39:29 $
 *
 *
 * HISTORY
 *
 * $Log: IsBallDirectedGood.cc,v $
 * Revision 1.2  2000/10/11 10:39:29  olalu526
 * Final fixes.
 *
 * Revision 1.1  2000/10/10 20:09:22  olalu526
 * Added predicate IsBallDirectedGood.
 *
 * Revision 1.2  2000/09/17 14:31:38 olalu526 
 * Made minor updates to the IsBallDirectedGood.
 *
 * Revision 1.1  2000/09/13 00:06:00 olalu526 
 * Added a predicate teamplate called IsBallDirectedGood.
 *
 */

/**
 * @file IsBallDirectedGood.cc
 * Implementation file for the predicate class IsBallDirectedGood.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.2 $
 * $Author: olalu526 $
 * $Date: 2000/10/11 10:39:29 $
 */


#include "IsBallDirectedGood.h"


RS_BEGIN_NAMESPACE


IsBallDirectedGood::IsBallDirectedGood()
  : Predicate(),
    ballView(NULL)
{
  getView(ballView, BALL_VIEW_ID);
}


IsBallDirectedGood::IsBallDirectedGood(const IsBallDirectedGood& obj)
  : Predicate(obj),
    ballView(NULL)
{
  getView(ballView, BALL_VIEW_ID);
}


IsBallDirectedGood::~IsBallDirectedGood()
{
  viewManager->releaseView(BALL_VIEW_ID);
}


IsBallDirectedGood& IsBallDirectedGood::operator= (const IsBallDirectedGood& obj)
{
  if ( this != &obj ) {
    Predicate::operator=(obj);
  }

  return *this;
}


FuzzyBool IsBallDirectedGood::evaluate()
{
  AngleDegUCO a = ballView->getAbsoluteBallDirection(NOW);
  if (a.isKnown()) {
    AngleDeg aa = a.getAngle().getAngle().getAngle();
    if (aa > AngleDeg(Float(-90)) &&
	aa < AngleDeg(Float(90))) {
      return TRUE;
    }
  }
  return FALSE;
}


void IsBallDirectedGood::update(const UInt32)
{
}


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "IsBallDirectedGood.icc"
#endif
