/* -*- Mode: C++ -*-
 * $Id: IsBallKnown.cc,v 1.2 2001/01/22 13:25:09 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: IsBallKnown.cc
 * Unitname: Framework/predicate
 * $Revision: 1.2 $
 * Created by: Mattias Karlsson & Per Olof Pettersson 2000-10-07
 * Last modified by $Author: olalu526 $ $Date: 2001/01/22 13:25:09 $
 *
 *
 * HISTORY
 *
 * $Log: IsBallKnown.cc,v $
 * Revision 1.2  2001/01/22 13:25:09  olalu526
 * Fixed is ball known so that it can believe in a position for 20 cycles and not just 3 cycles.
 *
 * Revision 1.1  2000/10/07 14:28:26  matka134
 * Added the IsBallKnown predicate
 *
 */

/**
 * @file IsBallKnown.cc
 * Implementation file for the predicate class IsBallKnown.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.2 $
 * $Author: olalu526 $
 * $Date: 2001/01/22 13:25:09 $
 */


#include "IsBallKnown.h"
#include "view_constants.h"

RS_BEGIN_NAMESPACE


IsBallKnown::IsBallKnown()
  : Predicate(),
    ballView(NULL)
{
  getView(ballView, BALL_VIEW_ID);
}


IsBallKnown::IsBallKnown(const IsBallKnown& obj)
  : Predicate(obj),
    ballView(NULL),
    gameView(NULL)
{
    getView(ballView, BALL_VIEW_ID);
    getView(gameView, GAME_VIEW_ID);
}


IsBallKnown::~IsBallKnown()
{
  viewManager->releaseView(BALL_VIEW_ID);
  viewManager->releaseView(GAME_VIEW_ID);
}


IsBallKnown& IsBallKnown::operator= (const IsBallKnown& obj)
{
  if ( this != &obj ) {
    Predicate::operator=(obj);
  }
  ballView = obj.ballView;
  
  return *this;
}


FuzzyBool IsBallKnown::evaluate()
{
  TimeUnknown t = ballView->getLastBallObservationTime();
  if ( ballView->getRelativeBallDirection(NOW).isUnknown() ||
       ballView->getBallPosition(NOW).isUnknown()) {
    return FALSE;
  }
  else if (t.isUnknown()) {
    return FALSE;
  }
  // This value should be possible to modify.
  else if (t.getValue() < gameView->getCurrentTime() - 20) {
    return FALSE;
  }
  else {
    return TRUE;
  }
}

void IsBallKnown::update(const UInt32)
{
}


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "IsBallKnown.icc"
#endif
