/* -*- Mode: C++ -*-
 * $Id: IsBallKnown.h,v 1.2 2000/10/10 20:07:06 matka134 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: IsBallKnown.h
 * Unitname: Framework/predicate
 * $Revision: 1.2 $
 * Created by: Mattias Karlsson & Per Olof Pettersson 2000-10-07
 * Last modified by $Author: matka134 $ $Date: 2000/10/10 20:07:06 $
 *
 *
 * HISTORY
 *
 * $Log: IsBallKnown.h,v $
 * Revision 1.2  2000/10/10 20:07:06  matka134
 * Just added a few comments
 *
 * Revision 1.1  2000/10/07 14:28:27  matka134
 * Added the IsBallKnown predicate
 *
 */

/**
 * @file IsBallKnown.h
 * Include file for the predicate class IsBallKnown.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.2 $
 * $Author: matka134 $
 * $Date: 2000/10/10 20:07:06 $
 */


#ifndef PREDICATESTUB_H
#define PREDICATESTUB_H


#include "Predicate.h"
#include "BallView.h"
#include "GameView.h"

RS_BEGIN_NAMESPACE


/**
 * @class IsBallKnown IsBallKnown.h
 * Is the ball position known?
 * This predicate check if the ball position is known.
 *
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.2 $
 * $Author: matka134 $
 * $Date: 2000/10/10 20:07:06 $
 */
class IsBallKnown : public Predicate {
public:
  /** Create a predicate with default behavior. */
  IsBallKnown();

  /** Creates a predicate that is an exact copy of @a obj. */
  IsBallKnown(const IsBallKnown& obj);

  /** Destroys the predicate. */  
  virtual ~IsBallKnown();


  /** Assignment operator. Assigns @a obj to this predicate. */
  IsBallKnown& operator= (const IsBallKnown& obj);


  /** Function application operator. Will call evaluate(). */
  virtual FuzzyBool operator()();


  /** Should return the truth value of the predicate. */
  virtual FuzzyBool evaluate();


  
protected:
  /** The update function, should be used to update the value of the
    predicate if necessary. The method needUpdate(const UInt32 update_number)
    can be used to check if the predicate needs to be updated based on the
    last update number it was updated for. */
  virtual void update(const UInt32 update_number);

  /** A view that contain information about the ball. */
  BallView *ballView;
  /** A view that contain information about the game */
  GameView *gameView;
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "IsBallKnown.icc"
#endif


#endif // !PREDICATESTUB_H
