/* -*- Mode: C++ -*-
 * $Id: IsOpponentClose.cc,v 1.3 2000/10/11 06:12:16 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: IsOpponentClose.cc
 * Unitname: Framework/predicate
 * $Revision: 1.3 $
 * Created by: Fredrik Heintz 2000-09-12
 * Last modified by $Author: olalu526 $ $Date: 2000/10/11 06:12:16 $
 *
 *
 * HISTORY
 *
 * $Log: IsOpponentClose.cc,v $
 * Revision 1.3  2000/10/11 06:12:16  olalu526
 * Implementation of IsOppononentClose done, hopefully.
 *
 * Revision 1.2  2000/09/17 14:31:38 olalu526 
 * Made minor updates to the IsOpponentClose.
 *
 * Revision 1.1  2000/09/13 00:06:00 olalu526 
 * Added a predicate teamplate called IsOpponentClose.
 *
 */

/**
 * @file IsOpponentClose.cc
 * Implementation file for the predicate class IsOpponentClose.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.3 $
 * $Author: olalu526 $
 * $Date: 2000/10/11 06:12:16 $
 */


#include "IsOpponentClose.h"


RS_BEGIN_NAMESPACE


IsOpponentClose::IsOpponentClose()
  : Predicate(),
    playerView(NULL),
    parameterView(NULL),
    agentView(NULL),
    range(5)
{
  getView(playerView, PLAYER_VIEW_ID);
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID, parameterView);
}

IsOpponentClose::IsOpponentClose(Float r)
  : Predicate(),
    playerView(NULL),
    parameterView(NULL),
    agentView(NULL),
    range(r)
{
  getView(playerView, PLAYER_VIEW_ID);
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID, parameterView);
}


IsOpponentClose::IsOpponentClose(const IsOpponentClose& obj)
  : Predicate(obj),
    playerView(NULL),
    parameterView(NULL),
    agentView(NULL),
    range()
{
  getView(playerView, PLAYER_VIEW_ID);
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID, parameterView);
  range = obj.getRange();
}


IsOpponentClose::~IsOpponentClose()
{
  viewManager->releaseView(PLAYER_VIEW_ID);
  viewManager->releaseView(PARAMETER_VIEW_ID);
  viewManager->releaseView(AGENT_VIEW_ID);
}


IsOpponentClose& IsOpponentClose::operator= (const IsOpponentClose& obj)
{
  if ( this != &obj ) {
    Predicate::operator=(obj);
  }
  range = obj.getRange();
  return *this;
}


FuzzyBool IsOpponentClose::evaluate()
{
  PointUCO ap = agentView->getAgentPosition(NOW);
  if (ap.isUnknown()) {
    return TRUE;
  }
  PlayerObject *p = playerView->getClosestOpponent(NOW);
  if (p != NULL) {
    VectorUCO po = p->getRelativeVectorTo(ap.getPoint());
    if (po.isKnown()) {
      if (po.getLength() < getRange()) {
	return TRUE;
      }
    }
  }
  return FALSE;
}

Float IsOpponentClose::getRange() const {
  return range;
}

void IsOpponentClose::setRange(Float r) {
  range = r;
}

void IsOpponentClose::update(const UInt32)
{
}


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "IsOpponentClose.icc"
#endif
