/* -*- Mode: C++ -*-
 * $Id: IsOpponentClose.h,v 1.3 2000/10/11 10:39:29 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: IsOpponentClose.h
 * Unitname: Framework/predicate
 * $Revision: 1.3 $
 * Created by: Fredrik Heintz 2000-09-12
 * Last modified by $Author: olalu526 $ $Date: 2000/10/11 10:39:29 $
 *
 *
 * HISTORY
 *
 * $Log: IsOpponentClose.h,v $
 * Revision 1.3  2000/10/11 10:39:29  olalu526
 * Final fixes.
 *
 * Revision 1.2  2000/10/11 06:04:32  olalu526
 * Added the rest of the interface for this predicate.
 *
 * Revision 1.3  2000/09/17 18:03:18 olalu526 
 * Fixed an error in the IsOpponentClose.
 *
 * Revision 1.2  2000/09/17 14:31:38 olalu526 
 * Made minor updates to the IsOpponentClose.
 *
 * Revision 1.1  2000/09/13 00:06:00 olalu526 
 * Added a predicate teamplate called IsOpponentClose.
 *
 */

/**
 * @file IsOpponentClose.h
 * Include file for the predicate class IsOpponentClose.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.3 $
 * $Author: olalu526 $
 * $Date: 2000/10/11 10:39:29 $
 */


#ifndef ISOPPONENTCLOSE_H
#define ISOPPONENTCLOSE_H


#include "Predicate.h"
#include "PlayerView.h"
#include "AgentView.h"
#include "ParameterView.h"
#include "view_constants.h"

RS_BEGIN_NAMESPACE


/**
 * @class IsOpponentClose IsOpponentClose.h
 * Returns true if an opponend is inside the range.
 * If there is no opponent known it returns false.
 * If the opponent vector is unknown it returns true, because that
 * this often happens if the opponent stand so that the agent can not see.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.3 $
 * $Author: olalu526 $
 * $Date: 2000/10/11 10:39:29 $
 */
class IsOpponentClose : public Predicate {
public:
  /** Create a predicate with default behavior. */
  IsOpponentClose();

  /** Constructor that sets a close radius. */
  IsOpponentClose(Float);

  /** Creates a predicate that is an exact copy of @a obj. */
  IsOpponentClose(const IsOpponentClose& obj);

  /** Destroys the predicate. */  
  virtual ~IsOpponentClose();


  /** Assignment operator. Assigns @a obj to this predicate. */
  IsOpponentClose& operator= (const IsOpponentClose& obj);


  /** Function application operator. Will call evaluate(). */
  virtual FuzzyBool operator()();


  /** Should return the truth value of the predicate. */
  virtual FuzzyBool evaluate();


  /** Sets the range defining what's close. */
  virtual void setRange(Float);

  /** Gets the range defining what's close. */
  virtual Float getRange() const;
  
protected:
  /** The update function, should be used to update the value of the
    predicate if necessary. The method needUpdate(const UInt32 update_number)
    can be used to check if the predicate needs to be updated based on the
    last update number it was updated for. */
  virtual void update(const UInt32 update_number);
  PlayerView *playerView;
  ParameterView *parameterView;
  AgentView *agentView;
  Float range;
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "IsOpponentClose.icc"
#endif


#endif // !ISOPPONENTCLOSE_H
