/* -*- Mode: C++ -*-
 * $Id: NeedPositionate.cc,v 1.1 2000/09/28 16:07:14 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: NeedPositionate.cc
 * Unitname: Framework/predicate
 * $Revision: 1.1 $
 * Created by: Ola Lundqvist 2000-09-22
 * Last modified by $Author: olalu526 $ $Date: 2000/09/28 16:07:14 $
 *
 *
 * HISTORY
 *
 * $Log: NeedPositionate.cc,v $
 * Revision 1.1  2000/09/28 16:07:14  olalu526
 * Fixed for compilation errors.
 *
 *
 */

/**
 * @file NeedPositionate.cc
 * Implementation file for the predicate class NeedPositionate.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.1 $
 * $Author: olalu526 $
 * $Date: 2000/09/28 16:07:14 $
 */


#include "NeedPositionate.h"

RS_BEGIN_NAMESPACE


NeedPositionate::NeedPositionate(const Zone& z)
  : PredicateWithZone(z),
    positionateView(NULL),
    parameterView(NULL),
    agentView(NULL)
{
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID, parameterView);
  getView(positionateView, ZONEDEFENCEDEFAULTPOSITION_VIEW_ID);
  positionateView->setZone(z);
}

NeedPositionate::NeedPositionate(const NeedPositionate& obj)
  : PredicateWithZone(obj),
    positionateView(NULL),
    parameterView(NULL),
    agentView(NULL)
{
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID, parameterView);
  getView(positionateView, ZONEDEFENCEDEFAULTPOSITION_VIEW_ID);
  positionateView->setZone(obj.zone);
}

NeedPositionate::~NeedPositionate()
{
  viewManager->releaseView(AGENT_VIEW_ID);
  viewManager->releaseView(PARAMETER_VIEW_ID);
  viewManager->releaseView(ZONEDEFENCEDEFAULTPOSITION_VIEW_ID);
}


NeedPositionate& NeedPositionate::operator= (const NeedPositionate& obj)
{
  if ( this != &obj ) {
    PredicateWithZone::operator=(obj);
  }
  agentView = obj.agentView;
  
  return *this;
}

/* This should be changed! */
FuzzyBool NeedPositionate::evaluate()
{
  Point p = positionateView->getPosition();
  /* If the position of the agent is inside the zone, return true,
   * else return false.
   */
  if ((agentView->getAgentPosition(NOW)).isKnown()) {
    Point ap = (agentView->getAgentPosition(NOW)).getPoint();
    if (ap.getDistanceTo(p) > 1) {
      //      cout << "NP!" << flush;
      return TRUE;
    }
  }  
  return FALSE;
}

/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "NeedPositionate.icc"
#endif


RS_END_NAMESPACE


