/* -*- Mode: C++ -*-
 * $Id: NeedPositionate.h,v 1.2 2000/09/28 16:17:21 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: NeedPositionate.h
 * Unitname: Framework/predicate
 * $Revision: 1.2 $
 * Created by: Ola Lundqvist 2000-09-28
 * Last modified by $Author: olalu526 $ $Date: 2000/09/28 16:17:21 $
 *
 *
 * HISTORY
 *
 * $Log: NeedPositionate.h,v $
 * Revision 1.2  2000/09/28 16:17:21  olalu526
 * Fixed ifdef, define problem.
 *
 *
 */

/**
 * @file NeedPositionate.h
 * Include file for the predicate class NeedPositionate.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.2 $
 * $Author: olalu526 $
 * $Date: 2000/09/28 16:17:21 $
 */


#ifndef NEEDPOSITIONATE_H
#define NEEDPOSITIONATE_H


#include "view_constants.h"
#include "PredicateWithZone.h"
#include "AgentView.h"
#include "ParameterView.h"
#include "../skill/Zone.h"
#include "ZoneDefenceDefaultPosition.h"


RS_BEGIN_NAMESPACE


/**
 * @class NeedPositionate NeedPositionate.h
 * A predicate for asking if the Agent is inside a specific zone.
 * Returns true if the agent is inside the zone, and false elsewhere.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.2 $
 * $Author: olalu526 $
 * $Date: 2000/09/28 16:17:21 $
 */
class NeedPositionate : public PredicateWithZone {
public:
  /** Create a predicate with default behavior. */
  NeedPositionate(const Zone& z);

  /** Creates a predicate that is an exact copy of @a obj. */
  NeedPositionate(const NeedPositionate& obj);

  /** Destroys the predicate. */  
  ~NeedPositionate();

  /** Assignment operator. Assigns @a obj to this predicate. */
  NeedPositionate& operator= (const NeedPositionate& obj);

  virtual FuzzyBool evaluate();

protected:
  /** The update function, should be used to update the value of the
    predicate if necessary. The method needUpdate(const UInt32 update_number)
    can be used to check if the predicate needs to be updated based on the
    last update number it was updated for. */
  //virtual void update(const UInt32 update_number);
  ZoneDefenceDefaultPosition* positionateView;
  ParameterView *parameterView;
  AgentView *agentView;
};

RS_END_NAMESPACE

/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "NeedPositionate.icc"
#endif

#endif // !NEEDPOSITIONATE_H
