/* -*- Mode: C++ -*-
 * $Id: PlayModePredicate.h,v 1.1 2000/09/17 18:02:48 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: PlayModePredicate.h
 * Unitname: Framework/predicate
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 2000-09-17
 * Last modified by $Author: frehe $ $Date: 2000/09/17 18:02:48 $
 *
 *
 * HISTORY
 *
 * $Log: PlayModePredicate.h,v $
 * Revision 1.1  2000/09/17 18:02:48  frehe
 * Added the PlayModePredicate that will test if it is a certain play mode and PlayModes which is several instantiations of the PlayModePredicate template.
 *
 */

/**
 * @file PlayModePredicate.h
 * Include file for the template predicate class PlayModePredicate<pm>.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/17 18:02:48 $
 */


#ifndef PLAYMODEPREDICATE_H
#define PLAYMODEPREDICATE_H


#include "Predicate.h"
#include "soccer_types.h"
#include "GameView.h"
#include "view_constants.h"


RS_BEGIN_NAMESPACE


/**
 * @class PlayModePredicate PlayModePredicate.h
 * This predicate returns true iff it is play mode @a pm
 * specified by the template argument to the class.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/17 18:02:48 $
 */
template<PlayMode pm>
class PlayModePredicate : public Predicate {
public:
  /** Create a predicate with default behavior. */
  PlayModePredicate();

  /** Creates a predicate that is an exact copy of @a obj. */
  PlayModePredicate(const PlayModePredicate<pm>& obj);

  /** Destroys the predicate. */  
  virtual ~PlayModePredicate();


  /** Assignment operator. Assigns @a obj to this predicate. */
  PlayModePredicate<pm>& operator= (const PlayModePredicate<pm>& obj);


  /** Function application operator. Will call evaluate(). */
  virtual FuzzyBool operator()();


  /** Will return @c true iff it is PlayMode @a pm. */
  virtual FuzzyBool evaluate();

  
protected:
  /** A pointer to the GameView. */
  GameView* gameView;

  
  /** The update function, should be used to update the value of the
    predicate if necessary. The method needUpdate(const UInt32 update_number)
    can be used to check if the predicate needs to be updated based on the
    last update number it was updated for. */
  virtual void update(const UInt32 update_number);
};


template<PlayMode pm>
inline FuzzyBool PlayModePredicate<pm>::operator()()
{
  return evaluate();
}


template<PlayMode pm>
inline void PlayModePredicate<pm>::update(const UInt32)
{
}


template<PlayMode pm>
PlayModePredicate<pm>::PlayModePredicate()
  : Predicate(),
    gameView(NULL)
{
  getView(gameView, GAME_VIEW_ID);
}


template<PlayMode pm>
PlayModePredicate<pm>::PlayModePredicate(const PlayModePredicate<pm>& obj)
  : Predicate(obj),
    gameView(obj.gameView)
{
}


template<PlayMode pm>
PlayModePredicate<pm>::~PlayModePredicate()
{
  viewManager->releaseView(gameView->getId());
}


template<PlayMode pm>
PlayModePredicate<pm>& PlayModePredicate<pm>::operator= (const PlayModePredicate<pm>& obj)
{
  if ( this != &obj ) {
    Predicate::operator=(obj);
    gameView = obj.gameView;
  }

  return *this;
}


template<PlayMode pm>
FuzzyBool PlayModePredicate<pm>::evaluate()
{
  if ( gameView->getPlayMode() == PM_UNKNOWN )
    return UNKNOWN;
  else
    return gameView->getPlayMode() == pm;
}


RS_END_NAMESPACE


#endif // !PLAYMODEPREDICATE_H
