/* -*- Mode: C++ -*-
 * $Id: PlayModes.h,v 1.1 2000/09/17 18:02:49 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: PlayModes.h
 * Unitname: Framework/predicate
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 2000-09-17
 * Last modified by $Author: frehe $ $Date: 2000/09/17 18:02:49 $
 *
 *
 * HISTORY
 *
 * $Log: PlayModes.h,v $
 * Revision 1.1  2000/09/17 18:02:49  frehe
 * Added the PlayModePredicate that will test if it is a certain play mode and PlayModes which is several instantiations of the PlayModePredicate template.
 *
 * Revision 1.1  2000/09/17 14:30:33  frehe
 * Added the PlayModes predicate.
 *
 */

/**
 * @file PlayModes.h
 * Include file for the predicate class PlayModes.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/17 18:02:49 $
 */


#ifndef PLAYMODES_H_
#define PLAYMODES_H_


#include "PlayModePredicate.h"


RS_BEGIN_NAMESPACE


/** This predicate is true iff the play mode is before kick off. */
typedef PlayModePredicate<PM_BEFORE_KICK_OFF> BeforeKickOff;

/** This predicate is true iff the play mode is time over. */
typedef PlayModePredicate<PM_TIME_OVER> TimeOver;

/** This predicate is true iff the play mode is play on. */
typedef PlayModePredicate<PM_PLAY_ON> PlayOn;

/** This predicate is true iff the play mode is our offside kick. */
typedef PlayModePredicate<PM_OUR_OFFSIDE_KICK> OurOffsideKick;

/** This predicate is true iff the play mode is their offside kick. */
typedef PlayModePredicate<PM_THEIR_OFFSIDE_KICK> TheirOffsideKick;

/** This predicate is true iff the play mode is half time. */
typedef PlayModePredicate<PM_HALF_TIME> HalfTime;

/** This predicate is true iff the play mode is time up. */
typedef PlayModePredicate<PM_TIME_UP> TimeUp;

/** This predicate is true iff the play mode is extended time. */
typedef PlayModePredicate<PM_EXTENDED_TIME> ExtendedTime;

/** This predicate is true iff the play mode is out kick off. */
typedef PlayModePredicate<PM_OUR_KICK_OFF> OurKickOff;

/** This predicate is true iff the play mode is their kick off. */
typedef PlayModePredicate<PM_THEIR_KICK_OFF> TheirKickOff;

/** This predicate is true iff the play mode is our kick in. */
typedef PlayModePredicate<PM_OUR_KICK_IN> OurKickIn;

/** This predicate is true iff the play mode is their kick in. */
typedef PlayModePredicate<PM_THEIR_KICK_IN> TheirKickIn;

/** This predicate is true iff the play mode is our free kick. */
typedef PlayModePredicate<PM_OUR_FREE_KICK> OurFreeKick;

/** This predicate is true iff the play mode is their free kick. */
typedef PlayModePredicate<PM_THEIR_FREE_KICK> TheirFreeKick;

/** This predicate is true iff the play mode is our corner kick. */
typedef PlayModePredicate<PM_OUR_CORNER_KICK> OurCornerKick;

/** This predicate is true iff the play mode is their corner kick. */
typedef PlayModePredicate<PM_THEIR_CORNER_KICK> TheirCornerKick;

/** This predicate is true iff the play mode is our goal kick. */
typedef PlayModePredicate<PM_OUR_GOAL_KICK> OurGoalKick;

/** This predicate is true iff the play mode is their goal kick. */
typedef PlayModePredicate<PM_THEIR_GOAL_KICK> TheirGoalKick;

/** This predicate is true iff the play mode is our goal. */
typedef PlayModePredicate<PM_OUR_GOAL> OurGoal;

/** This predicate is true iff the play mode is their goal. */
typedef PlayModePredicate<PM_THEIR_GOAL> TheirGoal;

/** This predicate is true iff the play mode is our goalie got ball. */
typedef PlayModePredicate<PM_OUR_GOALIE_GOT_BALL> OurGoalieGotBall;

/** This predicate is true iff the play mode is their  goalie got ball. */
typedef PlayModePredicate<PM_THEIR_GOALIE_GOT_BALL> TheirGoalieGotBall;


RS_END_NAMESPACE


#endif // !PLAYMODES_H

