/* -*- Mode: C++ -*-
 * $Id: PredicateWithZone.cc,v 1.1 2000/09/24 10:47:56 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: PredicateWithZone.icc
 * Unitname: Framework/predicate
 * $Revision: 1.1 $
 * Created by: Ola Lundqvist 2000-09-22
 * Last modified by $Author: olalu526 $ $Date: 2000/09/24 10:47:56 $
 *
 *
 * HISTORY
 *
 * $Log: PredicateWithZone.cc,v $
 * Revision 1.1  2000/09/24 10:47:56  olalu526
 * Added PredicateWithZone.cc and all other changes.
 *
 * Revision 1.4  2000/09/23 17:21:15  olalu526
 * Did some changes.
 *
 * Revision 1.3  2000/09/23 12:43:33  olalu526
 * Fixed some with PredicateWithZone and BallInZone to make it more flexible.
 *
 * Revision 1.2  2000/09/23 12:30:08  olalu526
 * Updated BallInZone. It should now work, but is not tested yet.
 *
 * Revision 1.1  2000/09/23 09:08:24  olalu526
 * Added PredicateWithZone and BallInZone predicates.
 *
 */

/**
 * @file PredicateWithZone.icc
 * Inline definitions for the predicate class PredicateWithZone.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.1 $
 * $Author: olalu526 $
 * $Date: 2000/09/24 10:47:56 $
 */


#include "machine_dependent_compiler_directives.h"
#include "PredicateWithZone.h"

RS_BEGIN_NAMESPACE


PredicateWithZone::PredicateWithZone(const Zone z)
  : PredicateWithArguments(),
    zone(z)
{
}


PredicateWithZone::PredicateWithZone(const PredicateWithZone& obj)
  : PredicateWithArguments(),
    zone(obj.zone)
{
}

PredicateWithZone::~PredicateWithZone() {
}

void PredicateWithZone::setArguments(const std::vector<Float>& arguments) throw(TooFewArgumentsException)
{
  if (arguments.size() < 2) {
    RS_THROW(TooFewArgumentsException,
	     "PredicateWithZone got less than three arguments.");
  }
  else {
    zone = Zone(arguments[0], arguments[1], arguments[2]);
  }
}


void PredicateWithZone::setZone(const Zone& z)
{
  zone = z;
}


Zone PredicateWithZone::getZone() const
{
  return zone;
}

PredicateWithZone& PredicateWithZone::operator= (const PredicateWithZone& obj)
{
  if ( this != &obj ) {
    PredicateWithArguments::operator=(obj);
  }
  zone = obj.zone;
  
  return *this;
}

PredicateWithZone& PredicateWithZone::operator= (const Zone& z)
{
  setZone(z);
  return *this;
}

FuzzyBool PredicateWithZone::operator()()
{
  return evaluate();
}


FuzzyBool PredicateWithZone::evaluate(const Zone&z ) {
  setZone(z);
  return evaluate();
}

void PredicateWithZone::update(const UInt32 update_number) {
}


RS_END_NAMESPACE
