/* -*- Mode: C++ -*-
 * $Id: PredicateWithZone.h,v 1.5 2000/09/25 16:26:09 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: PredicateWithZone.h
 * Unitname: Framework/predicate
 * $Revision: 1.5 $
 * Created by: Ola Lundqvist 2000-09-22
 * Last modified by $Author: olalu526 $ $Date: 2000/09/25 16:26:09 $
 *
 *
 * HISTORY
 *
 * $Log: PredicateWithZone.h,v $
 * Revision 1.5  2000/09/25 16:26:09  olalu526
 * Update.
 *
 * Revision 1.4  2000/09/24 10:47:57  olalu526
 * Added PredicateWithZone.cc and all other changes.
 *
 * Revision 1.3  2000/09/23 17:21:14  olalu526
 * Did some changes.
 *
 * Revision 1.2  2000/09/23 12:43:33  olalu526
 * Fixed some with PredicateWithZone and BallInZone to make it more flexible.
 *
 * Revision 1.1  2000/09/23 09:08:24  olalu526
 * Added PredicateWithZone and BallInZone predicates.
 *
 */

/**
 * @file PredicateWithZone.h
 * Include file for the predicate class PredicateWithZone.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.5 $
 * $Author: olalu526 $
 * $Date: 2000/09/25 16:26:09 $
 */


#ifndef PREDICATEWITHZONE_H
#define PREDICATEWITHZONE_H


#include "PredicateWithArguments.h"
#include "rs_throw.h"
#include "../skill/Zone.h"

RS_BEGIN_NAMESPACE

/**
 * @class PredicateWithZone PredicateWithZone.h
 * Base class for all predicates that needs a zone as argument.
 * Use this class if it is a predicate and if you need a zone in it.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.5 $
 * $Author: olalu526 $
 * $Date: 2000/09/25 16:26:09 $
 */
class PredicateWithZone : public PredicateWithArguments {
public:
  /** Create a predicate with default behavior. */
  PredicateWithZone(const Zone);

  /** Creates a predicate that is an exact copy of @a obj. */
  PredicateWithZone(const PredicateWithZone&);

  /** Destroys the predicate. */  
  virtual ~PredicateWithZone();

  /** Used to set the predicate's arguments in a generic way.
    The first value in the vector will be interpreted as the
    X-coordinate and the second value the Y-coordinate. */
  virtual void setArguments(const std::vector<Float>& arguments)
     throw(TooFewArgumentsException);


  /** Used to set the predicate's position argument to @a pos. */
  virtual void setZone(const Zone& pos);

  /** Return the predicate's position argument. */
  virtual Zone getZone() const;

  /** Assignment operator. Assigns @a obj to this predicate. */
  PredicateWithZone& operator= (const PredicateWithZone&);
  PredicateWithZone& operator= (const Zone&);

  FuzzyBool operator()();

  /** Should return the truth value of the predicate. */
  virtual FuzzyBool evaluate(const Zone&);

  virtual FuzzyBool evaluate() = 0;
protected:
  /** The update function, should be used to update the value of the
    predicate if necessary. The method needUpdate(const UInt32 update_number)
    can be used to check if the predicate needs to be updated based on the
    last update number it was updated for. */
  virtual void update(const UInt32 update_number);
  Zone zone;
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

//#ifdef RS_USE_INLINE
//#  include "PredicateWithZone.icc"
//#endif


#endif // !PREDICATEWITHZONE_H
