/* -*- Mode: C++ -*-
 * $Id: SawBall.cc,v 1.2 2000/09/17 18:02:06 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: SawBall.cc
 * Unitname: Framework/predicate
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 2000-09-17
 * Last modified by $Author: frehe $ $Date: 2000/09/17 18:02:06 $
 *
 *
 * HISTORY
 *
 * $Log: SawBall.cc,v $
 * Revision 1.2  2000/09/17 18:02:06  frehe
 * Updated the SawBall predicate.
 *
 * Revision 1.1  2000/09/17 14:30:33  frehe
 * Added the SawBall predicate.
 *
 */

/**
 * @file SawBall.cc
 * Implementation file for the predicate class SawBall.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2000/09/17 18:02:06 $
 */


#include "SawBall.h"
#include "AgentView.h"
#include "BallView.h"
#include "ParameterView.h"
#include "view_constants.h"


RS_BEGIN_NAMESPACE


SawBall::SawBall()
  : Predicate(),
    agentView(NULL),
    ballView(NULL),
    parameterView(NULL)
{
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(agentView, BALL_VIEW_ID, parameterView);
  getView(ballView, BALL_VIEW_ID);
}


SawBall::SawBall(const SawBall& obj)
  : Predicate(obj),
    agentView(obj.agentView),
    ballView(obj.ballView),
    parameterView(obj.parameterView)
{
}


SawBall::~SawBall()
{
  viewManager->releaseView(agentView->getId());
  viewManager->releaseView(ballView->getId());
  viewManager->releaseView(parameterView->getId());
}


SawBall& SawBall::operator= (const SawBall& obj)
{
  if ( this != &obj ) {
    Predicate::operator=(obj);
    agentView = obj.agentView;
    ballView = obj.ballView;
    parameterView = obj.parameterView;
  }

  return *this;
}


FuzzyBool SawBall::evaluate()
{
  if (ballView->getLastBallObservationTime().isUnknown()) {
    return FALSE;
  }
  else {
    // If the ball was seen the last see message than this is true.
    return agentView->getLastSeeTime() == ballView->getLastBallObservationTime().getValue();
  }
}


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "SawBall.icc"
#endif
