/* -*- Mode: C++ -*-
 * $Id: SawBall.h,v 1.2 2000/09/17 18:02:05 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: SawBall.h
 * Unitname: Framework/predicate
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 2000-09-17
 * Last modified by $Author: frehe $ $Date: 2000/09/17 18:02:05 $
 *
 *
 * HISTORY
 *
 * $Log: SawBall.h,v $
 * Revision 1.2  2000/09/17 18:02:05  frehe
 * Updated the SawBall predicate.
 *
 * Revision 1.1  2000/09/17 14:30:33  frehe
 * Added the SawBall predicate.
 *
 */

/**
 * @file SawBall.h
 * Include file for the predicate class SawBall.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2000/09/17 18:02:05 $
 */


#ifndef SAWBALL_H
#define SAWBALL_H


#include "Predicate.h"


RS_BEGIN_NAMESPACE


class AgentView;
class BallView;
class ParameterView;


/**
 * @class SawBall SawBall.h
 * This predicate returns true iff the ball was seen in the last
 * see message, otherwise false.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2000/09/17 18:02:05 $
 */
class SawBall : public Predicate {
public:
  /** Create a predicate with default behavior. */
  SawBall();

  /** Creates a predicate that is an exact copy of @a obj. */
  SawBall(const SawBall& obj);

  /** Destroys the predicate. */  
  virtual ~SawBall();


  /** Assignment operator. Assigns @a obj to this predicate. */
  SawBall& operator= (const SawBall& obj);


  /** Function application operator. Will call evaluate(). */
  virtual FuzzyBool operator()();


  /** Will return @c true iff the ball was seen in the last
    see message, otherwise @c false. */
  virtual FuzzyBool evaluate();

  
protected:
  /** A pointer to the agent view. */
  AgentView* agentView;

  /** A pointer to the ball view. */
  BallView* ballView;

  /** A pointer to the parameter view. */
  ParameterView* parameterView;

  
  /** The update function, should be used to update the value of the
    predicate if necessary. The method needUpdate(const UInt32 update_number)
    can be used to check if the predicate needs to be updated based on the
    last update number it was updated for. */
  virtual void update(const UInt32 update_number);
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "SawBall.icc"
#endif


#endif // !SAWBALL_H
