/* -*- Mode: C++ -*-
 * $Id: CatchBall.cc,v 1.2 2000/10/08 11:36:19 olale507 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: CatchBall.cc
 * Unitname: Framework/skill
 * $Revision: 1.2 $
 * Created by: Bjrn, Ola och Jenny
 * Last modified by $Author: olale507 $ $Date: 2000/10/08 11:36:19 $
 *
 *
 * HISTORY
 *
 *
 */

/**
 * @file CatchBall.cc
 * This is a definition file for the skill CatchBall.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.2 $
 * $Author: olale507 $
 * $Date: 2000/10/08 11:36:19 $
 */

#include "BallView.h"
#include "CatchBall.h"
#include "view_constants.h"
#include "CatchCommand.h"
#include "TurnCommand.h"
#include "AgentView.h"
#include "ParameterView.h"


RS_BEGIN_NAMESPACE


// --- CONSTRUCTORS ---

CatchBall::CatchBall()
  : Skill(),
    parameterView(NULL),
    ballView(NULL),
    agentView(NULL)

{
  getView(parameterView, PARAMETER_VIEW_ID);  
  getView(ballView, BALL_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID, parameterView);
}

// --- DESTRUCTOR ---

CatchBall::~CatchBall()
{
    viewManager->releaseView(BALL_VIEW_ID);
    viewManager->releaseView(AGENT_VIEW_ID);
    viewManager->releaseView(PARAMETER_VIEW_ID);
}


// --- METHODS ---

Plan CatchBall::generatePlan()
{
  static Command* cmd;
  static Plan plan;
  static AngleDegUnknown ang;

  ang = ballView->getRelativeBallDirection(NOW);

  cmd = new CatchCommand(ang.getDeg());

  plan = Plan(Step(cmd));
  
  delete cmd;
  cmd = NULL;
  
  return plan;
}


FuzzyBool CatchBall::persistent()
{
  return FALSE;
}


FuzzyBool CatchBall::applicable()
{
  return  ( agentView->isGoalie() &&
	    ballView->getRelativeBallDirection(NOW).isKnown() &&
	    agentView->getAgentPosition(NOW).isKnown() &&
	    ballView->getBallPosition(NOW).isKnown() &&
	    ballView->isBallCatchable(NOW) );
}


RS_END_NAMESPACE
