/* -*- Mode: C++ -*-
 * $Id: CatchBall.h,v 1.3 2000/11/08 15:37:55 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: CatchBall.h
 * Unitname: Framework/skill
 * $Revision: 1.3 $
 * Created by: Bjrn, Ola och Jenny 2000-09-07
 * Last modified by $Author: olalu526 $ $Date: 2000/11/08 15:37:55 $
 *
 *
 * HISTORY
 *
 */

/**
 * @file CatchBall.h
 * This is the definitionfile of the skill CatchBall.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.3 $
 * $Author: olalu526 $
 * $Date: 2000/11/08 15:37:55 $ */


#ifndef CATCHBALL_H
#define CATCHBALL_H

#include "machine_dependent_compiler_directives.h"
#include "Skill.h"
#include "ParameterView.h"
#include "BallView.h"
#include "AgentView.h"

RS_BEGIN_NAMESPACE


/**
 * @class CatchBall CatchBall.h
 * The skill will try to catch the ball.
 * If the ball is catchable, this skill will produce a catchcommand in
 * the appropriate angle.
 *
 * @pre The preconditions for the skill are:
 * The agent is the goalie and
 * the relative balldirection, agentposition, ballposition are known and
 * the ball is catchable.
 * @post The intended postconditions for the skill are:
 * The ball has been catched.
 *
 * @todo Things that should be done:
 * Perhaps make the skill persistent if the ball wasn't catched.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.3 $
 * $Author: olalu526 $
 * $Date: 2000/11/08 15:37:55 $
 */
class CatchBall : public Skill {
public:
  /** Default constructor. Here you can initialize the skill. */
  CatchBall();

  /** Destructor should clean up after the skill. */
  virtual ~CatchBall();

  
  /** This method is used to generate a plan for how the goal of
    the skill should be obtained by the agent. You can assume
    that the preconditions of the skill are true (this can be tested
    by calling the applicable() method, after you have implemented it). */
  virtual Plan generatePlan();

  /** This method should return a FuzzyBool telling the caller
    if the skill wants to be called again. */
  virtual FuzzyBool persistent();

  /** This method should implement the precondition specified in
   the description of the skill. */
  virtual FuzzyBool applicable();

  
private:
  /** Description of this attribute. */
  //  TheType attributeName;

  
  /** Hide copy-constructor. */
  CatchBall(const CatchBall&);
  /** Hide assignment operator. */
  CatchBall& operator= (const CatchBall&);

protected:
  ParameterView* parameterView;
  BallView* ballView;
  AgentView* agentView;


};


RS_END_NAMESPACE

#endif // !CATCHBALL_H
