/* -*- Mode: C++ -*-
 * $Id: ClearBall.h,v 1.8 2000/10/16 12:27:01 govme133 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: ClearBall.h
 * Unitname: Framework/skill
 * $Revision: 1.8 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: govme133 $ $Date: 2000/10/16 12:27:01 $
 *
 *
 * HISTORY
 * $Log: ClearBall.h,v $
 * Revision 1.8  2000/10/16 12:27:01  govme133
 * *** empty log message ***
 *
 * Revision 1.7  2000/10/09 14:06:49  govme133
 * final version
 *
 * Revision 1.6  2000/10/07 14:35:55  govme133
 * version using the angles between opponents
 *
 * Revision 1.5  2000/09/27 13:39:05  govme133
 * nearly finished skill
 *
 * Revision 1.4  2000/09/24 20:21:31  govme133
 * *** empty log message ***
 *
 * Revision 1.3  2000/09/24 19:13:39  govme133
 * *** empty log message ***
 *
 * Revision 1.1  2000/09/05 21:12:02  frehe
 * #include rs_log.h
 *
 */

/**
 * @file ClearBall.h
 * Include file for the skill class ClearBall.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.8 $
 * $Author: govme133 $
 * $Date: 2000/10/16 12:27:01 $
 */


#ifndef CLEARBALL_H
#define CLEARBALL_H


#include "Skill.h"


RS_BEGIN_NAMESPACE


class ParameterView;
class MarkerView;
class AgentView;
class BallView;
class PlayerView;


/**
 * @class ClearBall ClearBall.h
 * This class encapsulates a ClearBall skill.
 * It's called in an emergency (probably in front of the own goal) 
 * and it results in kicking the ball to an open area on 
 * the field, preferably towards the enemy side of the field.
 *
 * <!--
 * @pre The preconditions for the skill are...
 *  The player needs to be in kicking distance to the ball
 *  It is impossible to give a good pass to a friendly player
 *  It is to dangerous to try to dribble (e.g. if the enemy players get the ball they will score)
 * @post The postconditions for the skill are...
 *  The ball is kicked to an empty area on the field if an appropriate area exists.
 *  if no good area exists within the view of the player he tries to kick it in the direction 
 *  of the enemy goal.
 *  The ball is NEVER kicked in the direction of the own side of the field (because this could 
 *  cause a more dangerous situation).
 *  The ball is only kicked outside of the field if there is no better solution.
 * @invariant The class invariant is...
 *   what is an invariant :)
 * @todo This needs to be done...
 * --> nothing
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.8 $
 * $Author: govme133 $
 * $Date: 2000/10/16 12:27:01 $
 */
class ClearBall : public Skill {
public:
  ClearBall();
  virtual ~ClearBall();

  virtual float calculateValue(float width,float angle, 
                             float distanceA, float distanceB, float goaldir);
  virtual float theirGoal();
  virtual float calculateBestAngle();
  virtual Plan generatePlan();

  virtual FuzzyBool persistent();
  virtual FuzzyBool applicable();

protected:
  ParameterView* parameterView;
  MarkerView* markerView;
  AgentView* agentView;
  BallView* ballView;
  PlayerView* playerView;
};


RS_END_NAMESPACE


#endif // !CLEARBALL_H
