/* -*- Mode: C++ -*-
 * $Id: CoordArea.h,v 1.2 2001/06/19 14:01:42 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: CoordArea.h
 * Unitname: Library
 * $Revision: 1.2 $
 * Created by: Ola Lundqvist 1999-09-22
 * Last modified by $Author: frehe $ $Date: 2001/06/19 14:01:42 $
 *
 *
 * HISTORY
 *
 * $Log: CoordArea.h,v $
 * Revision 1.2  2001/06/19 14:01:42  frehe
 * Made minor changes to compile RoboSoc with gcc-3.0
 *
 * Revision 1.1  2000/11/07 17:24:00  olalu526
 * Generalized the zone concept to handle various type of objects. It now handles a set of rectangles and circles to form an area. This affected quite a lot of skills.
 *
 * Revision 1.3  2000/11/02 22:56:36  olalu526
 * Added a good header.
 *
 * Revision 1.2  2000/09/24 10:44:33  olalu526
 * Now ZoneDefenceToDefend works (had to remove a non woring file in Makefile.am, sorry.
 *
 * Revision 1.1  2000/09/22 19:10:47  olalu526
 * Generated new ZoneDefenceDefender files and checked in Zone.h, Area.h and CoordArea.h needed by that.
 *
 */

/**
 * @file CoordArea.h
 * Include file for the template CoordArea.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2001/06/19 14:01:42 $
 */


#ifndef COORDAREA_H
#define COORDAREA_H


#include <iostream>

#include "FuzzyBool.h"
#include "library_types.h"
#include "math_utils.h"
#include "AngleDegrees.h"
#include "CoordVector.h"
#include "CoordPoint.h"


/** String to use as the header when outputing a CoordArea. */
#define AREA_HEAD std::string("#Area")


RS_BEGIN_NAMESPACE


//template <class T> class CoordVector;
template <class T> class CoordPoint;


/**
 * @class CoordArea CoordArea.h
 * Template class for representing a cartesian two dimensional
 * coordinate with a radiues, forming a area.
 * The X, Y and R values are of type T.
 *
 * <!-- @todo -->
 *
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2001/06/19 14:01:42 $
 */
template<class T> class CoordArea {
public:
  CoordArea(char);
  char type;
  /*
  virtual T getArea() = 0;
  virtual T getCircumference() = 0;
  virtual void setArea(const T&) = 0;
  */
  virtual T getDistanceTo(const CoordPoint<T>& pt);

  virtual FuzzyBool inArea(const CoordPoint<T>& pt);
  //virtual FuzzyBool throughArea(const CoordPoint<T>& pt);
  
protected:
private:
};

typedef CoordArea<Float> Area;

template<class T>
inline CoordArea<T>::CoordArea<T>(char c) {
  type = c;
}

/*
template<class T>
inline T CoordArea<T>::getArea() {
  return T(0);
}

template<class T>
inline T CoordArea<T>::getCircumference() {
  return T(0);
}

template<class T>
inline void CoordArea<T>::setArea(const T&) {
}

*/
template<class T>
inline T CoordArea<T>::getDistanceTo(const CoordPoint<T>& pt) {
  std::cerr << "CoordArea::getDistanceTo should not run " << pt
	    << "!" << type << "!" << std::endl;
  return T(0);
}

template<class T>
inline FuzzyBool CoordArea<T>::inArea(const CoordPoint<T>& pt) {
  std::cerr << "CoordArea::inArea should not run " << pt
	    << "!" << type << "!" << std::endl;
  return FALSE;
}

RS_END_NAMESPACE

#endif // COORDAREA_H
