/* -*- Mode: C++ -*-
 * $Id: CoordCircle.h,v 1.5 2001/06/19 14:01:42 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: CoordCircle.h
 * Unitname: Library
 * $Revision: 1.5 $
 * Created by: Ola Lundqvist 1999-09-22
 * Last modified by $Author: frehe $ $Date: 2001/06/19 14:01:42 $
 *
 *
 * HISTORY
 *
 * $Log: CoordCircle.h,v $
 * Revision 1.5  2001/06/19 14:01:42  frehe
 * Made minor changes to compile RoboSoc with gcc-3.0
 *
 * Revision 1.4  2000/11/07 17:27:15  olalu526
 * Generalized the zone concept to handle various type of objects. It now handles a set of rectangles and circles to form an area. This affected quite a lot of skills.
 *
 * Revision 1.3  2000/11/02 22:56:36  olalu526
 * Added a good header.
 *
 * Revision 1.2  2000/09/24 10:44:33  olalu526
 * Now ZoneDefenceToDefend works (had to remove a non woring file in Makefile.am, sorry.
 *
 * Revision 1.1  2000/09/22 19:10:47  olalu526
 * Generated new ZoneDefenceDefender files and checked in Zone.h, Circle.h and CoordCircle.h needed by that.
 *
 */

/**
 * @file CoordCircle.h
 * Include file for the template CoordCircle.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.5 $
 * $Author: frehe $
 * $Date: 2001/06/19 14:01:42 $
 */


#ifndef COORDCIRCLE_H
#define COORDCIRCLE_H


#include <iostream>

#include "library_types.h"
#include "math_utils.h"
#include "AngleDegrees.h"
#include "CoordVector.h"
#include "CoordPoint.h"
#include "CoordArea.h"


/** String to use as the header when outputing a CoordCircle. */
#define CIRCLE_HEAD std::string("#Circle")


RS_BEGIN_NAMESPACE


template <class T> class CoordVector;
template <class T> class CoordPoint;
template <class T> class CoordArea;

/**
 * @class CoordCircle CoordCircle.h
 * Template class for representing a cartesian two dimensional
 * coordinate with a radiues, forming a circle.
 * The X, Y and R values are of type T.
 *
 * <!-- @todo -->
 *
 * @par Unit:
 *      Library
 * $Revision: 1.5 $
 * $Author: frehe $
 * $Date: 2001/06/19 14:01:42 $
 */
template<class T> class CoordCircle : public CoordArea<T> {
public:
  CoordCircle();
  CoordCircle(const T&, const T&, const T&);
  CoordCircle(const CoordCircle<T>&);
  CoordCircle(const T&, const CoordPoint<T>&);
  CoordCircle(const CoordPoint<T>&, const T&);
  CoordCircle(const T&, const CoordVector<T>&);
  CoordCircle(const CoordVector<T>&, const T&);
  //explicit CoordCircle(const CoordVector<T>&);

  virtual CoordCircle<T> operator - () const;
  virtual CoordCircle<T> operator - (const CoordCircle<T>&) const;
  virtual CoordCircle<T> operator - (const CoordVector<T>&) const;
  virtual CoordCircle<T> operator - (const CoordPoint<T>&) const;
  virtual CoordCircle<T> operator + (const CoordCircle<T>&) const;
  virtual CoordCircle<T> operator + (const CoordVector<T>&) const;
  virtual CoordCircle<T> operator + (const CoordPoint<T>&) const;
  virtual CoordCircle<T> operator * (const T&) const;
  virtual CoordCircle<T> operator * (const CoordCircle<T>&) const;
  virtual CoordCircle<T> operator / (const T&) const;
  virtual CoordCircle<T> operator / (const CoordCircle<T>&) const;

  virtual CoordCircle<T>& operator =(const CoordCircle<T>&);

  virtual CoordCircle<T>& operator -=(const CoordCircle<T>&);
  virtual CoordCircle<T>& operator -=(const CoordVector<T>&);
  virtual CoordCircle<T>& operator -=(const CoordPoint<T>&);
  virtual CoordCircle<T>& operator +=(const CoordCircle<T>&);
  virtual CoordCircle<T>& operator +=(const CoordVector<T>&);
  virtual CoordCircle<T>& operator +=(const CoordPoint<T>&);
  virtual CoordCircle<T>& operator *=(const T&);
  virtual CoordCircle<T>& operator *=(const CoordCircle<T>&);
  virtual CoordCircle<T>& operator /=(const T&);
  virtual CoordCircle<T>& operator /=(const CoordCircle<T>&);
  
  virtual bool operator ! () const;
  virtual bool operator ==(const CoordCircle<T>&) const;
  virtual bool operator !=(const CoordCircle<T>&) const;

  virtual T getX() const;
  virtual T getY() const;
  virtual T getR() const;
  virtual CoordCircle<T>& addX(const T& pt_x);
  virtual CoordCircle<T>& addY(const T& pt_y);
  virtual CoordCircle<T>& addR(const T& pt_r);
  virtual CoordCircle<T>& addXY(const T& pt_x, const T& pt_y);
  virtual CoordCircle<T>& addXYR(const T& pt_x, const T& pt_y, const T& pt_r);
  virtual CoordCircle<T>& setX(const T& pt_x);
  virtual CoordCircle<T>& setY(const T& pt_y);
  virtual CoordCircle<T>& setR(const T& pt_y);
  virtual CoordCircle<T>& set (const T& pt_x, const T& pt_y, const T& pt_r);
  
  virtual CoordVector<T> getVectorTo(const CoordCircle<T>& pt) const;
  virtual CoordPoint<T> getPointOf() const;
  virtual AngleDegrees<T> getDirectionTo(const CoordCircle<T>& pt) const;
  virtual AngleDegrees<T> getDirectionTo(const CoordPoint<T>& pt) const;
  virtual T getDistanceTo(const CoordCircle<T>& pt) const;
  virtual T getDistanceTo(const CoordPoint<T>& pt) const;

  virtual FuzzyBool inArea(const CoordPoint<T>& pt);
  //FuzzyBool throughArea(const CoordPoint<T>& pt);
  
protected:
  T x;
  T y;
  T r;
};

typedef CoordCircle<Float> Circle;

template<class T>
inline FuzzyBool CoordCircle<T>::inArea(const CoordPoint<T> & pt) {
  //cerr << "Circle::inArea" << endl;
  T dist = abs(pt.getDistanceTo(getPointOf()));
  //cerr << "inCArea: (" << x << ", " << y << ", " << r << ") " << pt;
  if (r > dist) {
    //cerr << " TRUE!" << endl << flush;
    return TRUE;
  }
  //cerr << " FALSE!" << endl << flush;
  return FALSE;
}


/*
 * X, Y is inverted (negated) but the radius remains unchanged.
 */
template<class T>
inline CoordCircle<T> CoordCircle<T>::operator -() const
{
  return CoordCircle<T>(-x,-y,r);
}

template<class T>
inline CoordCircle<T> CoordCircle<T>::operator -(const CoordCircle<T>& pt) const
{
  return CoordCircle<T>(x - pt.x, y - pt.y, abs(r - pt.r));
}

template<class T>
inline CoordCircle<T> CoordCircle<T>::operator -(const CoordVector<T>& v) const
{
  return CoordCircle<T>(x - Point(v).getX(), y - Point(v).getY(), r);
}

template<class T>
inline CoordCircle<T> CoordCircle<T>::operator -(const CoordPoint<T>& pt) const
{
  return CoordCircle<T>(x - pt.getX(), y - pt.getY(), r);
}

template<class T>
inline CoordCircle<T> CoordCircle<T>::operator +(const CoordVector<T>& v) const
{
  return CoordCircle<T>(x + Point(v).getX(), y + Point(v).getY(), r);
}

template<class T>
inline CoordCircle<T> CoordCircle<T>::operator +(const CoordPoint<T>& pt) const
{
  return CoordCircle<T>(x + pt.getX(), y + pt.getY(), r);
}

template<class T>
inline CoordCircle<T> CoordCircle<T>::operator *(const T& a) const
{
  return CoordCircle<T>(x * a, y * a, r * a);
}

template<class T>
inline CoordCircle<T> CoordCircle<T>::operator *(const CoordCircle<T>& pt) const
{
  return CoordCircle<T>(x * pt.x, y * pt.y, r * pt.r);
}

template<class T>
inline CoordCircle<T> operator *(const T& a, const CoordCircle<T>& pt)
{
  return pt * a;
}

template<class T>
inline CoordCircle<T> CoordCircle<T>::operator /(const T& a) const
{
#if CHECK_DIVISION_BY_ZERO == 1
  if (a == T(0))
    return COORDCIRCLE_DIVISION_BY_ZERO_RESULT;
  else
#endif
    return CoordCircle<T>(x / a, y / a, r / a);
}

template<class T>
inline CoordCircle<T> CoordCircle<T>::operator /(const CoordCircle<T>& pt) const
{
#if CHECK_DIVISION_BY_ZERO == 1
  if ( pt.x == T(0) ) {
    if ( pt.y == T(0) ) {
      return CoordCircle_DIVISION_BY_ZERO_RESULT;
    } else {
      return CoordCircle<T>(T_DIVISION_BY_ZERO_RESULT, y / pt.y);
    }
  } else {
    if ( pt.y == T(0) ) {
      return CoordCircle<T>(x / pt.x, T_DIVISION_BY_ZERO_RESULT);
    } else {
#endif
      return CoordCircle<T>(x / pt.x, y / pt.y, r / pt.r);
#if CHECK_DIVISION_BY_ZERO == 1      
    }
  }
#endif
}

template<class T>
inline CoordCircle<T> CoordCircle<T>::operator +(const CoordCircle<T>& pt) const
{
  return CoordCircle<T>(x + pt.x, y + pt.y, r + pt.r);
}

template<class T>
inline CoordCircle<T>& CoordCircle<T>::operator-=(const CoordCircle<T>& pt)
{
  x -= pt.x;
  y -= pt.y;
  r -= pt.r;
  return *this;
}

template<class T>
inline CoordCircle<T>& CoordCircle<T>::operator-=(const CoordPoint<T>& pt)
{
  x -= pt.getX();
  y -= pt.getY();
  return *this;
}

template<class T>
inline CoordCircle<T>& CoordCircle<T>::operator-=(const CoordVector<T>& vec)
{
  *this = *this - vec;
  return *this;
}

template<class T>
inline CoordCircle<T>& CoordCircle<T>::operator+=(const CoordCircle<T>& pt)
{
  x += pt.x;
  y += pt.y;
  r += pt.r;
  return *this;
}

template<class T>
inline CoordCircle<T>& CoordCircle<T>::operator+=(const CoordPoint<T>& pt)
{
  x += pt.getX();
  y += pt.getY();
  return *this;
}

template<class T>
inline CoordCircle<T>& CoordCircle<T>::operator+=(const CoordVector<T>& vec)
{
  *this = *this + vec;
  return *this;
}

template<class T>
inline CoordCircle<T>& CoordCircle<T>::operator*=(const T& a)
{
  x *= a;
  y *= a;
  r *= a;
  return *this;
}

template<class T>
inline CoordCircle<T>& CoordCircle<T>::operator*=(const CoordCircle<T>& pt)
{
  x *= pt.x;
  y *= pt.y;
  r *= pt.r;
  return *this;
}

template<class T>
inline CoordCircle<T>& CoordCircle<T>::operator/=(const T& a)
{
#if CHECK_DIVISION_BY_ZERO == 1
  if ( a == T(0)) {
    x = T_DIVISION_BY_ZERO_RESULT;
    y = T_DIVISION_BY_ZERO_RESULT;
  } else {
#endif
    x /= a;
    y /= a;
    r /= a;
#if CHECK_DIVISION_BY_ZERO == 1
  }
#endif
  return *this;
}

template<class T>
inline CoordCircle<T>& CoordCircle<T>::operator/=(const CoordCircle<T>& pt)
{
#if CHECK_DIVISION_BY_ZERO == 1
  if ( pt.x == T(0) )
    x = T_DIVISION_BY_ZERO_RESULT;
  else
#endif
    x /= pt.x;
  
#if CHECK_DIVISION_BY_ZERO == 1
  if ( pt.y == T(0) )
    y = T_DIVISION_BY_ZERO_RESULT;
  else
#endif
    y /= pt.y;

#if CHECK_DIVISION_BY_ZERO == 1
  if ( pt.r == T(0) )
    r = T_DIVISION_BY_ZERO_RESULT;
  else
#endif
    r /= pt.r;

  return *this;
}

template<class T>
CoordCircle<T>& CoordCircle<T>::operator =(const CoordCircle<T>& pt)
{
  if ( this != &pt ) {
    y = pt.y;
    x = pt.x;
    r = pt.r;
  }
  return *this;
}

template<class T>
inline bool CoordCircle<T>::operator !() const
{
  return (!y) && (!x) && (!r);
}

template<class T>
inline bool CoordCircle<T>::operator ==(const CoordCircle<T>& pt) const
{
  return (y == pt.y) && (x == pt.x) && (r == pt.r);
}

template<class T>
inline bool CoordCircle<T>::operator !=(const CoordCircle<T>& pt) const
{
  return (y != pt.y) || (x != pt.x) || (r != pt.r);
}

/*
inline bool CoordCircle<T>::InFront(const CoordCircle<T>& pt) const
{
  return (x > pt.x);
}

inline bool CoordCircle<T>::Behind(const CoordCircle<T>& pt) const
{
  return (x < pt.x);
}
*/

/*
inline bool CoordCircle<T>::LeftOf(const CoordCircle<T>& pt) const
{
  return (x > pt.x);
}

inline bool CoordCircle<T>::RightOf(const CoordCircle<T>& pt) const
{
  return (x < pt.x);
}

inline bool CoordCircle<T>::Above(const CoordCircle<T>& pt) const
{
  return (y < pt.y);
}

inline bool CoordCircle<T>::Below(const CoordCircle<T>& pt) const
{
  return (y > pt.y);
}
*/

template<class T>
inline CoordCircle<T>& CoordCircle<T>::addX (const T& pt_x)
{
  x += pt_x;
  return *this;
}

template<class T>
inline CoordCircle<T>& CoordCircle<T>::addY (const T& pt_y)
{
  y += pt_y;
  return *this;
}

template<class T>
inline CoordCircle<T>& CoordCircle<T>::addR (const T& pt_r)
{
  r += pt_r;
  return *this;
}

template<class T>
inline CoordCircle<T>& CoordCircle<T>::addXY (const T& pt_x, const T& pt_y)
{
  x += pt_x;
  y += pt_y;
  return *this;
}

template<class T>
inline CoordCircle<T>& CoordCircle<T>::addXYR (const T& pt_x, const T& pt_y, const T& pt_r)
{
  x += pt_x;
  y += pt_y;
  r += pt_r;
  return *this;
}

template<class T>
inline T CoordCircle<T>::getX() const
{
  return x;
}

template<class T>
inline T CoordCircle<T>::getY() const
{
  return y;
}

template<class T>
inline T CoordCircle<T>::getR() const
{
  return r;
}

template<class T>
inline CoordCircle<T>& CoordCircle<T>::setX(const T& pt_x)
{
  x = pt_x;
  return *this;
}

template<class T>
inline CoordCircle<T>& CoordCircle<T>::setY(const T& pt_y)
{
  y = pt_y;
  return *this;
}

template<class T>
inline CoordCircle<T>& CoordCircle<T>::setR(const T& pt_r)
{
  r = pt_r;
  return *this;
}

template<class T>
inline CoordCircle<T>& CoordCircle<T>::set(const T& pt_x, const T& pt_y, const T& pt_r)
{
  x = pt_x;
  y = pt_y;
  r = pt_r;
  return *this;
}

template<class T>
inline AngleDegrees<T> CoordCircle<T>::getDirectionTo(const CoordCircle<T>& pt) const
{
  return ATan2((pt.y - y), (pt.x - x));
}

template<class T>
inline AngleDegrees<T> CoordCircle<T>::getDirectionTo(const CoordPoint<T>& pt) const
{
  //  cout << "AngleDegrees: " << pt.getY() << " " << pt.getX() << " "
  //     << " : " << y << " " << x
  //     << "R: " << ATan2((pt.getY() - y), (pt.getX() - x)) << endl;
  return ATan2((pt.getY() - y), (pt.getX() - x));
}

template<class T>
inline T CoordCircle<T>::getDistanceTo(const CoordCircle<T>& pt) const
{
  return sqrt(sqr(pt.x - x) + sqr(pt.y - y));
}

template<class T>
inline T CoordCircle<T>::getDistanceTo(const CoordPoint<T>& pt) const
{
  return sqrt(sqr(pt.getX() - x) + sqr(pt.getY() - y));
}

template<class T>
inline std::ostream& operator<<(std::ostream& os, const CoordCircle<T>& pt)
{
#ifdef CIRCLE_HEAD
  return os << CIRCLE_HEAD << "(" << pt.getX() << ", " << pt.getY() << ", " << pt.getR() << ")";
#else
  return os << "(" << pt.getX() << ", " << pt.getY() << ", " << pt.getR() << ")";
#endif
}

template<class T>
inline const std::string ToString(const CoordCircle<T>& pt)
{
#ifdef CIRCLE_HEAD
  return CIRCLE_HEAD + "(" + ToString(pt.getX()) + ", " + ToString(pt.getY()) + ", " + ToString(pt.getR()) + ")";
#else
  return "(" + ToString(pt.getX()) + ", " + ToString(pt.getY()) + ", " + ToString(pt.getR()) + ")";
#endif
}

template<class T>
CoordCircle<T>::CoordCircle()
  : CoordArea<T>('c'),
    x(),
    y(),
    r()
{
}

template<class T>
CoordCircle<T>::CoordCircle(const T& pt_x, const T& pt_y, const T& pt_r)
  : CoordArea<T>('c'),
    x(pt_x),
    y(pt_y),
    r(pt_r)
{
}

template<class T>
CoordCircle<T>::CoordCircle(const CoordCircle<T>& pt)
  : CoordArea<T>('c'),
    x(pt.x),
    y(pt.y),
    r(pt.r)
{
}

/*CoordCircle<T>::CoordCircle(const T& pt_r, const CoordPoint<T>& pt)
{
  x = pt.getX();
  y = pt.getY();
  r = pt_r;
}
*/
template<class T>
CoordCircle<T>::CoordCircle(const CoordPoint<T>& pt, const T& pt_r) :
  CoordArea<T>('c')
{
  x = pt.getX();
  y = pt.getY();
  r = pt_r;
}

template<class T>
CoordCircle<T>::CoordCircle(const CoordVector<T>& v, const T& pt_r) :
  CoordArea<T>('c')
{
  x = v.getLength() * cos(v.getAngle());
  y = v.getLength() * sin(v.getAngle());
  r = pt_r;
}

template<class T>
CoordCircle<T>::CoordCircle(const T& pt_r, const CoordVector<T>& v) :
  CoordArea<T>('c')
{
  x = v.getLength() * cos(v.getAngle());
  y = v.getLength() * sin(v.getAngle());
  r = pt_r;
}
/*
template<class T>
CoordCircle<T> CoordCircle<T>::operator-(const CoordVector<T>& v) const
{
  return (-v) + *this;
}

template<class T>
CoordCircle<T> CoordCircle<T>::operator+(const CoordVector<T>& v) const
{
  return v+*this;
}
*/
template<class T>
CoordPoint<T> CoordCircle<T>::getPointOf() const
{
  return CoordPoint<T>(x,y);
}


template<class T>
CoordVector<T> CoordCircle<T>::getVectorTo(const CoordCircle<T>& z) const
{
  return CoordVector<T>(z.getPointOf() - this->getPointOf());
}

template<class T>
std::istream& operator>>(std::istream& is, CoordCircle<T>& pt)
{
  /* Input formats: f is a T
	(f,f) = (f,f)
  */

  T pt_x;
  T pt_y;
  T pt_r;
  char c = 0;

  is >> std::ws >> c;
  if (c == '(') {
    is >> pt_x >> c;
    if (c == ',') is >> pt_y >> c;
    if (c != ',') is.clear(std::ios::badbit); //error...;
    if (c == ',') is >> pt_r >> c;
    if (c != ')') is.clear(std::ios::badbit); //error...;
  }
  else {
    is.clear(std::ios::badbit); //error...;
  }

  if (is) pt = CoordCircle<T>(pt_x, pt_y, pt_r);

  return is;
}


RS_END_NAMESPACE


#endif // COORDCIRCLE_H
