/* -*- Mode: C++ -*-
 * $Id: CutOffGoalAngle.cc,v 1.2 2000/09/24 15:21:01 krini782 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: CutOffGoalAngle.cc
 * Unitname: Framework/skill
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: krini782 $ $Date: 2000/09/24 15:21:01 $
 *
 *
 * HISTORY
 */

/**
 * @file CutOffGoalAngle.cc
 * Implementation file for the skill class CutOffGoalAngle.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.2 $
 * $Author: krini782 $
 * $Date: 2000/09/24 15:21:01 $
 */


#include "CutOffGoalAngle.h"
#include "view_constants.h"
#include "TurnCommand.h"
#include "DashCommand.h"
#include "ParameterView.h"
#include "MarkerView.h"
#include "AgentView.h"
#include "BallView.h"
#include "rs_log.h"


RS_BEGIN_NAMESPACE


CutOffGoalAngle::CutOffGoalAngle()
  : Skill(),
    parameterView(NULL),
    markerView(NULL),
    agentView(NULL),
    ballView(NULL)
{
  parameterView = static_cast<ParameterView*>(viewManager->getView(PARAMETER_VIEW_ID));
  if ( parameterView == NULL ) {
    parameterView = new ParameterView(PARAMETER_VIEW_ID);
    viewManager->addView(parameterView);
  }

  markerView = static_cast<MarkerView*>(viewManager->getView(MARKER_VIEW_ID));
  if ( markerView == NULL ) {
    markerView = new MarkerView(MARKER_VIEW_ID, parameterView);
    viewManager->addView(markerView, true);
  }

  agentView = static_cast<AgentView*>(viewManager->getView(AGENT_VIEW_ID));
  if ( agentView == NULL ) {
    agentView = new AgentView(AGENT_VIEW_ID, parameterView);
    viewManager->addView(agentView, true);
  }

  ballView = static_cast<BallView*>(viewManager->getView(BALL_VIEW_ID));
  if ( ballView == NULL ) {
    ballView = new BallView(BALL_VIEW_ID);
    viewManager->addView(ballView, true);
  }
} 


CutOffGoalAngle::~CutOffGoalAngle()
{
  viewManager->releaseView(PARAMETER_VIEW_ID);
  viewManager->releaseView(MARKER_VIEW_ID);
  viewManager->releaseView(AGENT_VIEW_ID);
  viewManager->releaseView(BALL_VIEW_ID);
}


Plan CutOffGoalAngle::generatePlan()
{
  static Command* cmd;
  static VectorUCO goalVec, relBallVec, ballVec, scoreVec;
  static AngleDegUCO ang;
  static PointUCO dest;
  static Plan plan;

  PointUCO pt = agentView->getAgentPosition(NOW);
  goalVec = VectorUCO(pt, markerView->getOurGoal(NOW).getPosition());
  relBallVec = ballView->getRelativeBallVector(NOW);
  scoreVec = goalVec - relBallVec;

  /*RS_LOG( LA_CUTOFFGOALANGLE,
	  "\nplayer pos = "
	  << PointUnknown(pt)
	  << "\ngoal pos = "
	  << PointUnknown(markerView->getTheirGoal(NOW).getPosition())
	  << "\ngoal vector = "
	  << VectorUnknown(vec)
	  << "\nbody dir = "
	  << AngleDegUnknown(agentView->getBodyDirection(NOW))
	  << "\nface dir = "
	  << AngleDegUnknown(agentView->getFaceDirection(NOW)) );
  */

  if ( scoreVec.isUnknown() ) {
    cmd = NULL;
    RS_LOG( LA_SCORE, "Score vector unknown!\n" );
  }
  else {
    dest = ballView->getBallPosition(NOW) + 0.5 * scoreVec;
    ang = (agentView->getAgentObject(NOW)).getRelativeDirectionTo(dest);
    //    RS_LOG( LA_SCORE, "ang = " << ang );

    if ( abs(ang) > 30 )
      cmd = new TurnCommand(ang.getDeg());
    else
      cmd = new DashCommand(50);
  }

  /*else if ( agentView->getBodyDirection(NOW).isUnknown() ) {
    cmd = new KickCommand(100, vec.getAngle().getDeg());
    RS_LOG( LA_SCORE, "kick dir = " << vec.getAngle().getDeg() );
  }
  else {
    cmd = new KickCommand(100, vec.getAngle().getDeg() -
			  agentView->getBodyDirection(NOW).getDeg());
    RS_LOG( LA_SCORE, "kick dir = "
	    << vec.getAngle().getDeg() - agentView->getBodyDirection(NOW).getDeg() );
	    }*/
	    
  plan = Plan(Step(cmd));
  delete cmd;
  cmd = NULL;
  
  return plan;
}


FuzzyBool CutOffGoalAngle::persistent()
{
  return FALSE;
}


FuzzyBool CutOffGoalAngle::applicable()
{
  /* Kolla om det varit Kick Off ocks... */

  static VectorUCO goalVec, ballVec, scoreVec, relBallVec;

  PointUCO pt = agentView->getAgentPosition(NOW);
  goalVec = VectorUCO(pt, markerView->getOurGoal(NOW).getPosition());
  relBallVec = VectorUCO(pt, ballView->getBallPosition(NOW));
  scoreVec = goalVec - relBallVec;
  if ( ballView->getBallPosition(2).isKnown() && 
       ballView->getBallPosition(NOW).isKnown() ) {
    ballVec = VectorUCO(ballView->getBallPosition(2), ballView->getBallPosition(NOW));
    
    /* ndra sen till att skapa positiv vektor och kolla nedanfr */
    
    //    if ( abs(ballVec.getAngle()) < 90 ||  ballVec.getLength() == 0 ) return FALSE;
  }

  if ( scoreVec.isUnknown() || scoreVec.getLength() > 30 )
    return FALSE;
  else 
    return TRUE; 
}


RS_END_NAMESPACE
