/* -*- Mode: C++ -*-
 * $Id: CutOffGoalAngleStamina.h,v 1.1 2000/11/07 18:05:35 mikbr212 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: CutOffGoalAngleStamina.h
 * Unitname: Framework/skill
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: mikbr212 $ $Date: 2000/11/07 18:05:35 $
 *
 *
 * HISTORY
 * $Log: CutOffGoalAngleStamina.h,v $
 * Revision 1.1  2000/11/07 18:05:35  mikbr212
 * Created
 *
 * Revision 1.4  2000/10/25 08:39:47  krini782
 * Added pre- and postconditions.
 *
 * Revision 1.3  2000/10/09 08:33:49  krini782
 * Added doxygen comments.
 *
 * Revision 1.2  2000/09/24 15:21:02  krini782
 * Added skill CutOffPassAngle.
 *
 * Revision 1.1  2000/09/20 09:39:47  marma102
 * Added CutOffGoalAngleStamina skill.
 *
 * Revision 1.1  2000/09/05 21:12:02  frehe
 * #include rs_log.h
 *
 */

/**
 * @file CutOffGoalAngleStamina.h
 * Include file for the skill class CutOffGoalAngleStamina.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.1 $
 * $Author: mikbr212 $
 * $Date: 2000/11/07 18:05:35 $
 */


#ifndef CUTOFFGOALANGLESTAMINA_H
#define CUTOFFGOALANGLESTAMINA_H


#include "Skill.h"
#include "KickBallView.h"
#include "ParameterView.h"
#include "AgentView.h"
#include "BallView.h"
#include "Line.h"
#include "MoveToPositionFacingBall.h"

RS_BEGIN_NAMESPACE


/**
 * @class CutOffGoalAngleStamina CutOffGoalAngleStamina.h
 * This class encapsulates a cutoffgoalangle:ing skill.
 *
 * @pre The ball is moving towards the own goal, and is close to the goal.
 * @post The agent is between the ball and the goal.
 *
 * @todo Use skill MoveToPositionFacingDirection.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.1 $
 * $Author: mikbr212 $
 * $Date: 2000/11/07 18:05:35 $
 */

class CutOffGoalAngleStamina : public Skill {
public:
  CutOffGoalAngleStamina();
  virtual ~CutOffGoalAngleStamina();

  virtual Plan generatePlan();

  virtual FuzzyBool persistent();
  virtual FuzzyBool applicable();

protected:
  Point getOptimum(const Point&);
  PointUCO getAbsoluteBallPosition();

  Line goalLine;
  float maxDistFromGoal;
  float blockRadius;
  Point pos;
  Time timeStamp;
  MoveToPositionFacingBall goSkill;

  ParameterView* parameterView;
  MarkerView* markerView;
  AgentView* agentView;
  BallView* ballView;
  KickBallView* kickBallView;
};


RS_END_NAMESPACE


#endif // !CUTOFFGOALANGLESTAMINA_H
