/* -*- Mode: C++ -*-
 * $Id: CutOffPassAngle.cc,v 1.4 2000/11/01 16:17:56 krini782 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: CutOffPassAngle.cc
 * Unitname: Framework/skill
 * $Revision: 1.4 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: krini782 $ $Date: 2000/11/01 16:17:56 $
 *
 *
 * HISTORY
 */

/**
 * @file CutOffPassAngle.cc
 * Implementation file for the skill class CutOffPassAngle.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.4 $
 * $Author: krini782 $
 * $Date: 2000/11/01 16:17:56 $
 */


/* Fungerar skitbra! Ej testad dock... */

#include "CutOffPassAngle.h"
#include "view_constants.h"
#include "TurnCommand.h"
#include "DashCommand.h"
#include "ParameterView.h"
#include "MarkerView.h"
#include "AgentView.h"
#include "BallView.h"
#include "PlayerView.h"
#include "rs_log.h"


RS_BEGIN_NAMESPACE


CutOffPassAngle::CutOffPassAngle()
  : Skill(),
    parameterView(NULL),
    markerView(NULL),
    agentView(NULL),
    ballView(NULL),
    playerView(NULL)
{
  parameterView = static_cast<ParameterView*>(viewManager->getView(PARAMETER_VIEW_ID));
  if ( parameterView == NULL ) {
    parameterView = new ParameterView(PARAMETER_VIEW_ID);
    viewManager->addView(parameterView);
  }

  markerView = static_cast<MarkerView*>(viewManager->getView(MARKER_VIEW_ID));
  if ( markerView == NULL ) {
    markerView = new MarkerView(MARKER_VIEW_ID, parameterView);
    viewManager->addView(markerView, true);
  }

  agentView = static_cast<AgentView*>(viewManager->getView(AGENT_VIEW_ID));
  if ( agentView == NULL ) {
    agentView = new AgentView(AGENT_VIEW_ID, parameterView);
    viewManager->addView(agentView, true);
  }

  ballView = static_cast<BallView*>(viewManager->getView(BALL_VIEW_ID));
  if ( ballView == NULL ) {
    ballView = new BallView(BALL_VIEW_ID);
    viewManager->addView(ballView, true);
  }
 
  playerView = static_cast<PlayerView*>(viewManager->getView(PLAYER_VIEW_ID));
  if ( playerView == NULL ) {
    playerView = new PlayerView(PLAYER_VIEW_ID);
    viewManager->addView(playerView, true);
  }
} 


CutOffPassAngle::~CutOffPassAngle()
{
  viewManager->releaseView(PARAMETER_VIEW_ID);
  viewManager->releaseView(MARKER_VIEW_ID);
  viewManager->releaseView(AGENT_VIEW_ID);
  viewManager->releaseView(BALL_VIEW_ID);
  viewManager->releaseView(PLAYER_VIEW_ID);
}


Plan CutOffPassAngle::generatePlan()
{
  static Command* cmd;
  static Plan plan;
  std::vector<PlayerObject> opponentList;
  std::vector<PlayerObject> teammateList;
  typedef std::vector<PlayerObject> Container;
  Container::iterator i;
  Container::iterator j;
  Container::iterator k;
  PointUCO dest;
  AngleDegUCO ang;

  /*
   * Search for the opponent closest to the ball.
   */

  opponentList = playerView->getOpponents(NOW);
  i = opponentList.begin();
  j = opponentList.begin();
  while ( i != opponentList.end() ) {
    if ( VectorUCO( (*i).getPosition(), ballView->getBallPosition(NOW) ).getLength() <
	 VectorUCO( (*j).getPosition(), ballView->getBallPosition(NOW) ).getLength() ) {
      j = i;
    }
    i++;
  }

  /* 
   * Check which of the opponents is most likely to get the pass. 
   * (I.e. the one most in the facing direction of the player in possesion of the ball.)
   */

  i = opponentList.begin();
  k = opponentList.begin();
  while ( i != opponentList.end() ) {
    if ( abs ( VectorUCO( (*j).getPosition(), (*i).getPosition() ).getAngle() -
	       (*j).getNeckDirection() ) <
	 abs ( VectorUCO( (*j).getPosition(), (*k).getPosition() ).getAngle() -
	       (*j).getNeckDirection() ))
      k = i;
    i++;
  }

  /*
   * Destination point: Halfway between the opponent in possesion of the ball and the
   * opponent most likely to recieve the pass.
   */

  dest = (*j).getPosition() + ( 0.5 * VectorUCO( (*j).getPosition(),
						(*k).getPosition() ));

  ang = (agentView->getAgentObject(NOW)).getRelativeDirectionTo(dest);

  if ( abs(ang) > 30 )
    cmd = new TurnCommand(ang.getDeg());
  else
    cmd = new DashCommand(50);

  plan = Plan(Step(cmd));
  delete cmd;
  cmd = NULL;
  
  return plan;
}


FuzzyBool CutOffPassAngle::persistent()
{
  return FALSE;
}


FuzzyBool CutOffPassAngle::applicable()
{
  /* Kolla om det varit Kick Off ocks... */

  std::vector<PlayerObject> opponentList;
  std::vector<PlayerObject> teammateList;
  typedef std::vector<PlayerObject> Container;
  Container::iterator i;
  Container::iterator j;
  Container::iterator k;
  PointUCO dest;

  /*
   * Search for the opponent closest to the ball.
   */

  opponentList = playerView->getOpponents(NOW);
  if ( opponentList.empty() ) return FALSE;
  i = opponentList.begin();
  j = opponentList.begin();
  while ( i != opponentList.end() ) {
    if ( VectorUCO( (*i).getPosition(), ballView->getBallPosition(NOW) ).getLength() <
	 VectorUCO( (*j).getPosition(), ballView->getBallPosition(NOW) ).getLength() )
      j = i;
    i++;
  }

  /*
   * Check if a teammate is closer to the ball than the opponents.
   */

  teammateList = playerView->getTeammates(NOW);
  i = teammateList.begin();
  while ( i != teammateList.end() ) {
    if ( VectorUCO( (*i).getPosition(), ballView->getBallPosition(NOW) ).getLength() <
	 VectorUCO( (*j).getPosition(), ballView->getBallPosition(NOW) ).getLength() )
      return FALSE;
    i++;
  }

  /* 
   * Check which of the opponents is most likely to get the pass. 
   * (I.e. the one most in the facing direction of the player in possesion of the ball.)
   */

  i = opponentList.begin();
  k = opponentList.begin();
  while ( i != opponentList.end() ) {
    if ( abs ( VectorUCO( (*j).getPosition(), (*i).getPosition() ).getAngle() -
	       (*j).getNeckDirection() ) <
	 abs ( VectorUCO( (*j).getPosition(), (*k).getPosition() ).getAngle() -
	       (*j).getNeckDirection() ))
      k = i;
    i++;
  }

  dest = (*j).getPosition() + ( 0.5 * VectorUCO( (*j).getPosition(),
						(*k).getPosition() ));


  /*
   * Check if a teammate is closer to the cut off point.
   */

  i = teammateList.begin();
  while ( i != teammateList.end() ) {
    if ( VectorUCO( (*i).getPosition(), dest ).getLength() <
	 VectorUCO( agentView->getAgentObject(NOW).getPosition(), dest ).getLength() )
      return FALSE;
    i++;
  }

  return TRUE;
  
}

RS_END_NAMESPACE

