/* -*- Mode: C++ -*-
 * $Id: CutOffPassAngle.h,v 1.2 2000/10/25 08:39:48 krini782 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: Score.h
 * Unitname: Framework/skill
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: krini782 $ $Date: 2000/10/25 08:39:48 $
 *
 *
 * HISTORY
 * $Log: CutOffPassAngle.h,v $
 * Revision 1.2  2000/10/25 08:39:48  krini782
 * Added pre- and postconditions.
 *
 * Revision 1.1  2000/09/24 15:21:02  krini782
 * Added skill CutOffPassAngle.
 *
 * Revision 1.1  2000/09/20 09:39:47  marma102
 * Added CutOffPassAngle skill.
 *
 * Revision 1.1  2000/09/05 21:12:02  frehe
 * #include rs_log.h
 *
 */

/**
 * @file CutOffPassAngle.h
 * Include file for the skill class CutOffPassAngle.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.2 $
 * $Author: krini782 $
 * $Date: 2000/10/25 08:39:48 $
 */


#ifndef CUTOFFPASSANGLE_H
#define CUTOFFPASSANGLE_H


#include "Skill.h"


RS_BEGIN_NAMESPACE


class ParameterView;
class MarkerView;
class AgentView;
class BallView;
class PlayerView;


/**
 * @class CutOffPassAngle CutOffPassAngle.h
 * This class encapsulates a cutoffpassangle:ing skill.
 *
 * @pre An opponent has the ball and another opponent is available for
 * a pass. No teammate is closer to the cutoff point.
 * @post The player is between the ball and the opponent most likely
 * to recieve the pass. 
 *
 * @todo Use skill MoveToPositionFacingDirection. Test the skill... 
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.2 $
 * $Author: krini782 $
 * $Date: 2000/10/25 08:39:48 $
 */
class CutOffPassAngle : public Skill {
public:
  CutOffPassAngle();
  virtual ~CutOffPassAngle();

  virtual Plan generatePlan();

  virtual FuzzyBool persistent();
  virtual FuzzyBool applicable();

protected:
  ParameterView* parameterView;
  MarkerView* markerView;
  AgentView* agentView;
  BallView* ballView;
  PlayerView* playerView;
};


RS_END_NAMESPACE


#endif // !CUTOFFPASSANGLE_H
