/* -*- Mode: C++ -*-
 * $Id: DashToSpeedVector.cc,v 1.2 2001/06/19 14:01:43 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: DashToSpeedVector.cc
 * Unitname: Framework/skill
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 2000-09-07
 * Last modified by $Author: frehe $ $Date: 2001/06/19 14:01:43 $
 *
 *
 * HISTORY
 *
 * $Log: DashToSpeedVector.cc,v $
 * Revision 1.2  2001/06/19 14:01:43  frehe
 * Made minor changes to compile RoboSoc with gcc-3.0
 *
 * Revision 1.1  2000/09/23 20:47:45  perpe427
 * Added the DashToSpeedVector skill
 *
 * Revision 1.3  2000/09/16 18:39:39  frehe
 * Fixed the log messages.
 *
 * Revision 1.2  2000/09/16 18:36:48  frehe
 * Made minor fixes the skill stubs.
 *
 * Revision 1.1  2000/09/07 16:10:34  frehe
 * Added stubs for the skills.
 *
 */

/**
 * @file DashToSpeedVector.cc
 * Implementation file for the skill class DashToSpeedVector.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2001/06/19 14:01:43 $
 */


#include "DashToSpeedVector.h"
#include "AgentView.h"
#include "ParameterView.h"
#include "view_constants.h"
#include "math_utils.h"
#include "TurnCommand.h"
#include "DashCommand.h"


RS_BEGIN_NAMESPACE


// --- CONSTRUCTORS ---

DashToSpeedVector::DashToSpeedVector(const Vector& v, AngleDeg maxDeviation = 5)
  // This is an initializer list
  // which means that all the objects inside the
  // class can be initialized after the colon.
  // You initialize the variable myVar with
  // myVar(constructor arguments).
  // You should also initialize the base classes, but
  // only the classes that you directly inherit from.
  // If you do not initialize a variable its default
  // constructor will be called.
  : SkillWithVector(v)
{
  DashToSpeedVector::maxDeviation = maxDeviation;
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID, parameterView);
  std::cerr << "DashToSpeedVector::Constructor: vector=" << vector <<  std::endl;
}


// --- DESTRUCTOR ---

DashToSpeedVector::~DashToSpeedVector()
{
  viewManager->releaseView(AGENT_VIEW_ID);
  viewManager->releaseView(PARAMETER_VIEW_ID);
}


// --- METHODS ---

Plan DashToSpeedVector::generatePlan()
{
  static Float maxPower = parameterView->SP_maxpower;
  static Float powerRate = parameterView->SP_dash_power_rate;
  static Float decay = parameterView->SP_player_decay;
  static Float inertia = parameterView->SP_inertia_moment;

  Command* cmd = NULL;
  Plan plan;
  
  std::cerr << "DashToSpeedVector: ";

  if (agentView->getBodyDirection(NOW).isKnown() ) {
    AngleDeg bodyDirection = AngleDegUnknown(agentView->getBodyDirection(NOW)).getAngle();
    
    Float v;
    if(agentView->getSpeed(NOW).isKnown()) {
      v = FloatUnknown(agentView->getSpeed(NOW)).getValue();
    } else {
      v = 0;
    }
    
    std::cerr << 
      "v.ang=" << vector.getAngle() << 
      ", bodyDir=" << bodyDirection <<
      ", maxDev=" << maxDeviation <<
      ", abs=" << abs(vector.getAngle() - bodyDirection) <<
      ", ";

    // Return an empty plan.
    if(abs(vector.getAngle() - bodyDirection) >= maxDeviation) {
      std::cerr << "turn=" << vector.getAngle() - bodyDirection << " ";
      cmd = new TurnCommand((vector.getAngle() - bodyDirection) *
			    (1.0 + inertia * v));
    } else {
      std::cerr << "dash=" << min(maxPower,(vector.getLength()-v)/powerRate) << " ";
      cmd = new DashCommand(min(maxPower,(vector.getLength()-v)/powerRate));
    } 
  } else {
    cmd = NULL;
  }
  std::cerr << std::endl;
  plan = Plan(Step(cmd));
  delete cmd;
  return plan;
}


FuzzyBool DashToSpeedVector::persistent()
{
  // Never persistent.
  return FALSE;
}


FuzzyBool DashToSpeedVector::applicable()
{
  // Allways applicable.
  return TRUE;
}


RS_END_NAMESPACE


// Remove from here
/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "DashToSpeedVector.icc"
#endif
// To here if you do not have any inline definitions!

