/* -*- Mode: C++ -*-
 * $Id: DashToSpeedVector.h,v 1.2 2000/09/27 12:17:01 perpe427 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: DashToSpeedVector.h
 * Unitname: Framework/skill
 * $Revision: 1.2 $
 * Created by: Per Olof Pettersson 2000-09-23
 * Last modified by $Author: perpe427 $ $Date: 2000/09/27 12:17:01 $
 *
 *
 * HISTORY
 *
 * $Log: DashToSpeedVector.h,v $
 * Revision 1.2  2000/09/27 12:17:01  perpe427
 * Renamed InterceptBallPoM as InterceptBallStamina
 *
 * Revision 1.1  2000/09/23 20:47:46  perpe427
 * Added the DashToSpeedVector skill
 *
 */

/**
 * @file DashToSpeedVector.h
 * Include file for the skill class GotoBall.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.2 $
 * $Author: perpe427 $
 * $Date: 2000/09/27 12:17:01 $
 */


#ifndef DASHTOSPEEDVECTOR_H
#define DASHTOSPEEDVECTOR_H


#include "SkillWithVector.h"

RS_BEGIN_NAMESPACE

class AgentView;
class ParameterView;

/**
 * @class DashToSpeedVector DashToSpeedVector.h
 * Makes one, 
 * The rest is a longer description of the skill class DashToSpeedVector.
 * Use it to describe the skill!
 *
 * @pre The preconditions for the skill are ...
 * @post The intended postconditions for the skill are...
 *
 * @todo Things that should be done...
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.2 $
 * $Author: perpe427 $
 * $Date: 2000/09/27 12:17:01 $
 */
class DashToSpeedVector : public SkillWithVector {
public:
  /** Default constructor. Here you can initialize the skill. */
  DashToSpeedVector(const Vector& v,AngleDeg maxDeviation = 5);

  /** Destructor should clean up after the skill. */
  virtual ~DashToSpeedVector();


  /** Used to set the skills's arguments in a generic way.
    The first value in the vector will be interpreted as the
    distance and the second value as the direction in degrees. */
  virtual void setArguments(const std::vector<Float>& arguments)
     throw(TooFewArgumentsException);


  /** Used to set the skill's vector argument to @a vec. */
  virtual void setMaxDeviation(const AngleDeg& maxDeviation);

  /** Return the skill's vector argument. */
  virtual AngleDeg getMaxDeviation() const;

  
  /** This method is used to generate a plan for how the goal of
    the skill should be obtained by the agent. You can assume
    that the preconditions of the skill are true (this can be tested
    by calling the applicable() method, after you have implemented it). */
  virtual Plan generatePlan();

  /** This method should return a FuzzyBool telling the caller
    if the skill wants to be called again. */
  virtual FuzzyBool persistent();

  /** This method should implement the precondition specified in
   the description of the skill. */
  virtual FuzzyBool applicable();

protected:
  AgentView* agentView;
  ParameterView* parameterView;
  AngleDeg maxDeviation;
  
private:
  /** Hide copy-constructor. */
  DashToSpeedVector(const DashToSpeedVector&);
  /** Hide assignment operator. */
  DashToSpeedVector& operator= (const DashToSpeedVector&);
};


RS_END_NAMESPACE


// Remove from here 
/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "DashToSpeedVector.icc"
#endif
// To here if you do not have any inline definitions!

#endif // !DASHTOSPEEDVECTOR_H
