/* -*- Mode: C++ -*-
 * $Id: DashToSpeedVectorFacingDirection.cc,v 1.5 2001/06/19 14:01:43 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: DashToSpeedVectorFacingDirection.cc
 * Unitname: Framework/skill
 * $Revision: 1.5 $
 * Created by: Ola Lundqvist 2000-09-27
 * Last modified by $Author: frehe $ $Date: 2001/06/19 14:01:43 $
 *
 *
 * HISTORY
 *
 * $Log: DashToSpeedVectorFacingDirection.cc,v $
 * Revision 1.5  2001/06/19 14:01:43  frehe
 * Made minor changes to compile RoboSoc with gcc-3.0
 *
 * Revision 1.4  2001/01/23 22:18:49  olalu526
 * Reorderd to make it work better.
 *
 * Revision 1.3  2001/01/11 22:00:19  olalu526
 * Added the real world model with support for new simulation steps.
 *
 * Revision 1.2  2000/11/08 14:49:55  olalu526
 * Added view width checking support instead of just use 45 degrees.
 *
 * Revision 1.1  2000/11/08 10:16:40  olalu526
 * Fixing corruptness in dash to speedvector.
 *
 * Revision 1.26  2000/11/07 17:42:07  olalu526
 * Patched the dash to speedvector so that it looks in different available angles so that it will get more information about the world :)
 *
 * Revision 1.25  2000/10/26 07:12:02  perpe427
 * Commented out some debug-msgs
 *
 * Revision 1.24  2000/10/22 15:10:51  perpe427
 * Made it possible to set minimum turn angle + made generatePlan(...) call DTSVFD::generatePlan() rather than allowing polymorphism
 *
 * Revision 1.23  2000/10/11 10:39:14  olalu526
 * Final fixes.
 *
 * Revision 1.22  2000/10/10 11:33:25  olalu526
 * Fixed segfault.
 *
 * Revision 1.19  2000/10/10 10:29:33  olalu526
 * Removed typo error.
 *
 * Revision 1.18  2000/10/10 10:11:35  olalu526
 * Added documentation.
 *
 * Revision 1.17  2000/10/09 21:03:31  olalu526
 * Fixed bug in nextStamina.
 *
 * Revision 1.16  2000/10/09 21:00:32  olalu526
 * Fixed merge.
 *
 * Revision 1.15  2000/10/09 19:18:49  perpe427
 * Changed a max(a,b) to a min(a,b) so that the dash isn't allways 100 as it was before
 *
 * Revision 1.14  2000/10/09 09:04:38  olalu526
 * Added a ifdef, endif on cout.
 *
 * Revision 1.13  2000/10/08 21:50:22  olalu526
 * Structurated and make sure that the right generatePlan is used.
 *
 * Revision 1.12  2000/10/07 19:41:34  olalu526
 * Fixed some minor issues.
 *
 * Revision 1.8  2000/10/07 17:09:45  olalu526
 * Fixed compile errors.
 *
 * Revision 1.5  2000/10/07 14:28:23  olalu526
 * Added some skill files and fixed things.
 *
 * Revision 1.4  2000/10/01 13:05:53  olalu526
 * Updated FacingDirection, fixing bugs and stuff.
 *
 * Revision 1.3  2000/09/30 15:35:15  olalu526
 * Bug fixed MoveToPositionFacingDirection, now works really fine.
 *
 * Revision 1.2  2000/09/29 21:57:58  olalu526
 * Created MoveToPositionFacingBall and let all zone-dependent skills use it, and to make this work, movetopositionfacing_x had to be fixed. Not perfect but almost.
 *
 * Revision 1.1  2000/09/28 13:32:39  olalu526
 * A lot of changes.
 *
 */

/**
 * @file DashToSpeedVectorFacingDirection.cc
 * Implementation file for the skill class DashToSpeedVectorFacingDirection.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.5 $
 * $Author: frehe $
 * $Date: 2001/06/19 14:01:43 $
 */


#include "DashToSpeedVectorFacingDirection.h"
#define MAXUSESTAM 100
#define EMAXUSESTAM 100

RS_BEGIN_NAMESPACE


// --- CONSTRUCTORS ---


DashToSpeedVectorFacingDirection::DashToSpeedVectorFacingDirection()
  : SkillWithArguments(),
    emergency(FALSE),
    maxSpeed(),
    emergencyMaxUseStaminaPerCycle(),
    maxUseStaminaPerCycle(),
    emergencyMinStamina(),
    minStamina(),
    minTurnAngle(-0.5),
    parameterView(NULL),
    agentView(NULL),
    speedAndDirection(),
    angle()
{
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID, parameterView);
  defaults();
}


DashToSpeedVectorFacingDirection::DashToSpeedVectorFacingDirection(const Vector &v)
  : SkillWithArguments(),
    emergency(FALSE),
    maxSpeed(),
    emergencyMaxUseStaminaPerCycle(),
    maxUseStaminaPerCycle(),
    emergencyMinStamina(),
    minStamina(),
    minTurnAngle(-0.5),
    parameterView(NULL),
    agentView(NULL),
    speedAndDirection(v),
    angle()
{
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID, parameterView);
  defaults();
}

DashToSpeedVectorFacingDirection::DashToSpeedVectorFacingDirection(const AngleDeg &a)
  : SkillWithArguments(),
    emergency(FALSE),
    maxSpeed(),
    emergencyMaxUseStaminaPerCycle(),
    maxUseStaminaPerCycle(),
    emergencyMinStamina(),
    minStamina(),
    minTurnAngle(-0.5),
    parameterView(NULL),
    agentView(NULL),
    speedAndDirection(),
    angle(a)
{
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID, parameterView);
  defaults();
}

DashToSpeedVectorFacingDirection::DashToSpeedVectorFacingDirection(const Vector &v, const AngleDeg &a)
  : SkillWithArguments(),
    emergency(FALSE),
    maxSpeed(),
    emergencyMaxUseStaminaPerCycle(),
    maxUseStaminaPerCycle(),
    emergencyMinStamina(),
    minStamina(),
    minTurnAngle(-0.5),
    parameterView(NULL),
    agentView(NULL),
    speedAndDirection(v),
    angle(a)
{
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID, parameterView);
  defaults();
}

void DashToSpeedVectorFacingDirection::defaults() {
  maxSpeed = Float(1.0);
  emergencyMaxUseStaminaPerCycle = Float(EMAXUSESTAM);
  maxUseStaminaPerCycle = Float(MAXUSESTAM);
  emergencyMinStamina = looseRecoverLimit();
  minStamina = lowerEffortLimit();
  turnHeadCount = 0;
}

// --- DESTRUCTOR ---

DashToSpeedVectorFacingDirection::~DashToSpeedVectorFacingDirection()
{
  viewManager->releaseView(PARAMETER_VIEW_ID);
  viewManager->releaseView(AGENT_VIEW_ID);
}

// --- METHODS ---

/* EMERGENCY AND KEEPSTAMINA METHODS */

void DashToSpeedVectorFacingDirection::setEmergency(FuzzyBool e) {
  emergency = e;
}

FuzzyBool DashToSpeedVectorFacingDirection::getEmergency() const {
  return emergency;
}

void DashToSpeedVectorFacingDirection::setMaxSpeed(Float e) {
  maxSpeed = e;
}

Float DashToSpeedVectorFacingDirection::getMaxSpeed() const {
  return maxSpeed;
}

void DashToSpeedVectorFacingDirection::setMaxUseStaminaPerCycle(Float e) {
  maxUseStaminaPerCycle = e;
}

Float DashToSpeedVectorFacingDirection::getMaxUseStaminaPerCycle() const {
  return maxUseStaminaPerCycle;
}

void DashToSpeedVectorFacingDirection::setEmergencyMaxUseStaminaPerCycle(Float e) {
  emergencyMaxUseStaminaPerCycle = e;
}

Float DashToSpeedVectorFacingDirection::getEmergencyMaxUseStaminaPerCycle() const {
  return emergencyMaxUseStaminaPerCycle;
}

void DashToSpeedVectorFacingDirection::setMaxUseStamina(Float e) {
  setMinStamina(getCurStamina() - e);
}

Float DashToSpeedVectorFacingDirection::getMaxUseStamina() {
  return getCurStamina() - getMinStamina();
}

void DashToSpeedVectorFacingDirection::setMinStamina(Float e) {
  minStamina = e;
}

Float DashToSpeedVectorFacingDirection::getMinStamina() const {
  return minStamina;
}

void DashToSpeedVectorFacingDirection::setEmergencyMinStamina(Float e) {
  emergencyMinStamina = e;
}

Float DashToSpeedVectorFacingDirection::getEmergencyMinStamina() const {
  return emergencyMinStamina;
}

Float DashToSpeedVectorFacingDirection::getCurStamina() {
  FloatUCO su = agentView->getStamina(NOW);
  if (su.isKnown()) {
    return su.getValue().getValue().getValue();
  }
  return parameterView->SP_stamina_max;
}
  

Float DashToSpeedVectorFacingDirection::lowerEffortLimit() {
  return parameterView->SP_effort_inc_thr * parameterView->SP_stamina_max;
}

Float DashToSpeedVectorFacingDirection::looseRecoverLimit() {
  return parameterView->SP_recover_dec_thr * parameterView->SP_stamina_max;
}

void 
DashToSpeedVectorFacingDirection::setMinTurnAngle(const AngleDeg& a)
{
   minTurnAngle = a;
}

AngleDeg
DashToSpeedVectorFacingDirection::getMinTurnAngle()
{
   return minTurnAngle;
}

/* POWER FUNCTIONS */
Float DashToSpeedVectorFacingDirection::powerWith() {
#ifdef OPALDEBUG 
  std::cout << "NeedPower " << power << " used " << usedPower() << " " << endl<< std::flush;
#endif
  if (getEmergency()) {
    std::cout << "EM:" << std::flush;
    return  max(Float(0),
		min(emergencyLeftStamina() + recoverStamina(),
		    min(emergencyAvailableStamina(),
			needPower())));
  }
//  cout << "NE:" << flush;
  return max(Float(0),
	     min(leftStamina() + recoverStamina(),
		 min(availableStamina(),
		     needPower())));
}

Float DashToSpeedVectorFacingDirection::leftStamina() {
  return getCurStamina() - getMinStamina();
}
Float DashToSpeedVectorFacingDirection::emergencyLeftStamina() {
  return getCurStamina() - getEmergencyMinStamina();
}


Float DashToSpeedVectorFacingDirection::nextAbsSpeed() {
  FloatUCO su = agentView->getSpeed(NOW);
  if (su.isKnown()) {
    return abs(su.getValue().getValue().getValue() * parameterView->SP_player_decay);
  }
  return Float(0);
}

Float DashToSpeedVectorFacingDirection::availableSpeed() {
  return min(getMaxSpeed(), abs(getSpeedVector().getLength()));
}

Float DashToSpeedVectorFacingDirection::availableAbsSpeed() {
  return abs(availableSpeed());
}

Float DashToSpeedVectorFacingDirection::needAcceleration() {
  return max(Float(0), abs(speedAndDirection.getLength()) - nextAbsSpeed());
}

Float DashToSpeedVectorFacingDirection::needPower() {
  Float p = needAcceleration() / parameterView->SP_dash_power_rate;
  // If it is not an emergency situation we can have luck and get the speed anyway.
  return min(p, Float(100));
}

Float DashToSpeedVectorFacingDirection::usedPower() {
  if (willBack()) {
    return  abs(needPower() * 2);
  }
  return needPower();
}

Float DashToSpeedVectorFacingDirection::availableStamina() {
  return max(Float(0), recoverStamina() + getMaxUseStaminaPerCycle());
}

Float DashToSpeedVectorFacingDirection::emergencyAvailableStamina() {
  return max(Float(0), recoverStamina() + getEmergencyMaxUseStaminaPerCycle());
}

Float DashToSpeedVectorFacingDirection::recoverStamina() {
  // It should be agentView->getRecover() but it does now seems to exist...
  return parameterView->SP_recover_max * parameterView->SP_stamina_inc_max;
}


/* SET ARGUMENTS */

void DashToSpeedVectorFacingDirection::setArguments(const std::vector<Float>& arguments)
  throw(TooFewArgumentsException)
{
  if (arguments.size() < 2) {
    RS_THROW(TooFewArgumentsException,
	     "PredicateWithZone got less than three arguments.");
  }
  else {
    setSpeedVector(Vector(arguments[0], arguments[1]));
    setFaceAngle(AngleDeg(arguments[2]));
  }
}

void DashToSpeedVectorFacingDirection::setFaceAngle(const AngleDeg &a) {
  if (angle != a) {
    turnHeadCount = 0;
  }
  angle = a;
}

AngleDeg DashToSpeedVectorFacingDirection::getFaceAngle() {
  return angle;
}

void DashToSpeedVectorFacingDirection::setSpeedVector(const Vector &v) {
  speedAndDirection = v;
}

Vector DashToSpeedVectorFacingDirection::getSpeedVector() {
  return speedAndDirection;
}

/*
 * If the difference between the angles are to high the agent have to back.
 */
FuzzyBool DashToSpeedVectorFacingDirection::willBack() {
  if (abs(angle - speedAndDirection.getAngle()) > 90) {
    return TRUE;
  }
  return FALSE;
}

FuzzyBool DashToSpeedVectorFacingDirection::willBack(const Vector& v) {
  setSpeedVector(v);
  return willBack();
}


FuzzyBool DashToSpeedVectorFacingDirection::willBack(const AngleDeg& a) {
  setFaceAngle(a);
  return willBack();
}

FuzzyBool DashToSpeedVectorFacingDirection::willBack(const Vector& v, const AngleDeg& a) {
  setSpeedVector(v);
  setFaceAngle(a);
  return willBack();
}


Plan DashToSpeedVectorFacingDirection::generatePlan(const Vector& v) {
  setSpeedVector(v);
  return DashToSpeedVectorFacingDirection::generatePlan();
}


Plan DashToSpeedVectorFacingDirection::generatePlan(const AngleDeg& a) {
  setFaceAngle(a);
  return DashToSpeedVectorFacingDirection::generatePlan();
}


Plan DashToSpeedVectorFacingDirection::generatePlan(const Vector& v, const AngleDeg& a) {
  setSpeedVector(v);
  setFaceAngle(a);
  return DashToSpeedVectorFacingDirection::generatePlan();
}

Plan DashToSpeedVectorFacingDirection::generatePlan()
{
  static Command* cmd;
  static Plan plan;
  static TurnDirection turn;
  static FaceDirection face;

  turnHeadCount = (turnHeadCount+1)%8;

  if(minTurnAngle.getDeg() > -0.5) { // if minTurnAngle is modified
     turn.setMinTurnAngle(minTurnAngle);  // set it
  }  // otherwise use default for TurnDirection

  plan.makeEmpty();
  AngleDeg bodyturn = AngleDeg(Float(0));
  AngleDeg turnto = speedAndDirection.getAngle();
  Float dashpow = powerWith();
  if (willBack()) {
    //    cout << "Backward:" << endl << flush;
    dashpow = - dashpow;
    turnto = turnto + Float(180);
  }
  /*  else {
    // nothing
    //cout << "Forward:" << endl << flush;
    }*/
  if (turn.applicable(turnto)) {
    plan = turn.generatePlan();
    bodyturn = turn.getLastTurnAngle();
    //cout << "DashToSp: turn: " << bodyturn.getDeg() << endl << flush;
  }
  else {
    cmd = new DashCommand(dashpow);
    plan.addStep(Step(cmd));
    delete cmd;
    cmd = NULL;
    //cout << "DashToSp: dash: " << dashpow
    //	 << " bt: " << bodyturn.getDeg() << endl << flush;
  }

  AngleDeg fangle = angle;
  // Now calculate some so that the agent really looks around some.
  AngleDegUnknown vw = agentView->getViewAngle(NOW);
  if (vw.isKnown()) {
    AngleDeg viewWidth = max(abs(vw.getAngle()) - 5, AngleDeg(0));
    AngleDeg vv = angle - turnto;
    AngleDeg a = AngleDeg(90) - abs(vv);
    AngleDeg r(0);
    AngleDeg l(0);
    // If look at right.
    if (vv > 0) {
      r = min(a,viewWidth);
      l = min(AngleDeg(90),viewWidth);
    }
    // If look at left.
    if (vv < 0) {
      r = max(AngleDeg(-90),-viewWidth);
      l = max(-a,-viewWidth);
    }
    // turn left this time.
    if (turnHeadCount == 2 ||
	turnHeadCount == 3) {
      fangle += l;
    }
    // turn right this time.
    if (turnHeadCount == 6 ||
	turnHeadCount == 7) {
      fangle += r;
    }
  }
    // If I have to face an other direction, use.
  if (face.applicable(fangle, bodyturn)) {
    //    cout << "DashToSp: face: " << endl << flush;
    Plan p2 = plan;
    plan = face.generatePlan();
    plan.addStep(p2.getNextStep());
  }
  
  return plan;
}


FuzzyBool DashToSpeedVectorFacingDirection::persistent()
{
  // Never persistent.
  return FALSE;
}


FuzzyBool DashToSpeedVectorFacingDirection::applicable()
{
  // Almost always applicable.
  if ((agentView->getAgentPosition(NOW)).isKnown() &&
      (agentView->getBodyDirection(NOW)).isKnown() &&
      (agentView->getNeckDirection(NOW)).isKnown() ) {
    return TRUE;
  }
  return FALSE;
}


RS_END_NAMESPACE


// Remove from here
/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "DashToSpeedVectorFacingDirection.icc"
#endif
// To here if you do not have any inline definitions!
