/* -*- Mode: C++ -*-
 * $Id: DashToSpeedVectorFacingDirection.h,v 1.1 2000/11/08 10:16:40 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: DashToSpeedVectorFacingDirection.h
 * Unitname: Framework/skill
 * $Revision: 1.1 $
 * Created by: Ola Lundqvist 2000-09-27
 * Last modified by $Author: olalu526 $ $Date: 2000/11/08 10:16:40 $
 *
 *
 * HISTORY
 *
 * $Log: DashToSpeedVectorFacingDirection.h,v $
 * Revision 1.1  2000/11/08 10:16:40  olalu526
 * Fixing corruptness in dash to speedvector.
 *
 * Revision 1.14  2000/11/07 17:42:07  olalu526
 * Patched the dash to speedvector so that it looks in different available angles so that it will get more information about the world :)
 *
 * Revision 1.13  2000/10/22 15:10:51  perpe427
 * Made it possible to set minimum turn angle + made generatePlan(...) call DTSVFD::generatePlan() rather than allowing polymorphism
 *
 * Revision 1.12  2000/10/10 19:10:54  olalu526
 * Added documentation.
 *
 * Revision 1.11  2000/10/10 11:33:25  olalu526
 * Fixed segfault.
 *
 * Revision 1.10  2000/10/10 10:11:35  olalu526
 * Added documentation.
 *
 * Revision 1.9  2000/10/08 21:50:22  olalu526
 * Structurated and make sure that the right generatePlan is used.
 *
 * Revision 1.8  2000/10/07 17:53:25  olalu526
 * Fixing more things.
 *
 * Revision 1.5  2000/10/07 15:11:12  olalu526
 * Fixed errors.
 *
 * Revision 1.4  2000/10/07 14:28:23  olalu526
 * Added some skill files and fixed things.
 *
 * Revision 1.3  2000/10/01 13:05:54  olalu526
 * Updated FacingDirection, fixing bugs and stuff.
 *
 * Revision 1.1  2000/09/28 13:32:40  olalu526
 * A lot of changes.
 *
 *
 */

/**
 * @file DashToSpeedVectorFacingDirection.h
 * Include file for the skill class GotoBall.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.1 $
 * $Author: olalu526 $
 * $Date: 2000/11/08 10:16:40 $
 */


#ifndef DASHTOSPEEDVECTORFACINGDIRECTION_H
#define DASHTOSPEEDVECTORFACINGDIRECTION_H


#include "SkillWithArguments.h"
#include "AgentView.h"
#include "ParameterView.h"
#include "TurnDirection.h"
#include "FaceDirection.h"
#include "DashCommand.h"
//#include "Command.h"


RS_BEGIN_NAMESPACE


/**
 * @class DashToSpeedVectorFacingDirection DashToSpeedVectorFacingDirection.h
 * This skill makes the agent move in a director while facing an other one.
 * If the angle between the body and the face is too high the agent
 * will turn around and back. This is not very efficient so you can use the
 * willBack() method to check if that will happen or not.
 *
 * The skill do also take care of a number of limits:
 *	- How much stamina you can use loose in each cycle.
 *	- The maximum allowed speed.
 *	- The minimum stamina allowed to reach.
 *	- If it is an emgergency situation or not.
 *
 * OBS! Negative speed is treated as a positive one. It makes no difference.
 *
 * @pre The agent should know its own face and body direction,
 *	and its position.
 * @post The agent is heading for that direction at the right speed.
 *
 * @todo Implement it:
 *	- Write generatePlan();
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.1 $
 * $Author: olalu526 $
 * $Date: 2000/11/08 10:16:40 $
 */
class DashToSpeedVectorFacingDirection : public SkillWithArguments {
public:
  /** Default constructor. Here you can initialize the skill. */
  DashToSpeedVectorFacingDirection();
  /** Constructor, that also sets the speed vector. */
  DashToSpeedVectorFacingDirection(const Vector&);
  DashToSpeedVectorFacingDirection(const AngleDeg&);
  /** Constructor, that also sets the speed vector and face angle. */
  DashToSpeedVectorFacingDirection(const Vector&, const AngleDeg&);

  /** Destructor cleans up after the skill. */
  virtual ~DashToSpeedVectorFacingDirection();

  /** Sets the face angle to use. */
  virtual void setFaceAngle(const AngleDeg&);
  /** Gets the face angle it uses. */
  virtual AngleDeg getFaceAngle();
  /** Sets the speed vector to use. */
  virtual void setSpeedVector(const Vector&);
  /** Gets the intended speed vector. */
  virtual Vector getSpeedVector();
  /** Whether or not going backwards is needed. */
  FuzzyBool willBack();
  /** Whether or not going backwards is needed, using this speed vector. */
  FuzzyBool willBack(const Vector&);
  /** Whether or not going backwards is needed, using this face angle. */
  FuzzyBool willBack(const AngleDeg&);
  /** Whether or not going backwards is needed, using this speed vector and face angle. */
  FuzzyBool willBack(const Vector&, const AngleDeg&);

  /** This method is used to generate a plan for how the goal of
    the skill should be obtained by the agent. You can assume
    that the preconditions of the skill are true (this can be tested
    by calling the applicable() method, after you have implemented it). */
  virtual Plan generatePlan();
  /** Constructor, that also sets the speed vector. */
  virtual Plan generatePlan(const Vector&);
  /** Constructor, that also sets the face angle. */
  virtual Plan generatePlan(const AngleDeg&);
  /** Constructor, that also sets the speed vector and face angle. */
  virtual Plan generatePlan(const Vector&, const AngleDeg&);

  /** This method should return a FuzzyBool telling the caller
    if the skill wants to be called again. */
  virtual FuzzyBool persistent();

  /** This method should implement the precondition specified in
   the description of the skill. */
  virtual FuzzyBool applicable();

  /** Sets the arguments in a generic way. */
  virtual void setArguments(const std::vector<Float>&)
    throw(TooFewArgumentsException);
  
  /** How much the skill will use. */
  virtual Float powerWith();

  /** Sets the maximum speed the agent can run. */
  virtual void setMaxSpeed(Float);

  /** Gets the maximum speed the agent can run. */
  virtual Float getMaxSpeed() const;

  /** Sets emergency mode. */
  virtual void setEmergency(FuzzyBool);

  /** Gets whether or not it is emergency mode. */
  virtual FuzzyBool getEmergency() const;

  /** Sets the maximum stamina used per cycle. */
  virtual void setMaxUseStaminaPerCycle(Float);

  /** Gets the maximum stamina used per cycle. */
  virtual Float getMaxUseStaminaPerCycle() const;

  /** Sets the maximum stamina used per cycle in emergency mode. */
  virtual void setEmergencyMaxUseStaminaPerCycle(Float);

  /** Gets the maximum stamina used per cycle in emergency mode. */
  virtual Float getEmergencyMaxUseStaminaPerCycle() const;

  /** Sets the maximum amount of stamina to use. */
  virtual void setMaxUseStamina(Float);

  /** Gets the maximum amount of stamina to use. */
  virtual Float getMaxUseStamina();

  /** Sets the minumum level of stamina in non emergency mode. */
  virtual void setMinStamina(Float);

  /** Gets the minumum level of stamina in non emergency mode. */
  virtual Float getMinStamina() const;

  /** Sets the minumum level of stamina in emergency mode. */
  virtual void setEmergencyMinStamina(Float);

  /** Gets the minumum level of stamina in emergency mode. */
  virtual Float getEmergencyMinStamina() const;

  /** Get the default min stamina level. */
  Float lowerEffortLimit();

  /** Get the default emergency min stamina level. */
  Float looseRecoverLimit();

  /** Set the minimum turn angle. */
  void setMinTurnAngle(const AngleDeg&);

  /** Get the minimum turn angle. */
  AngleDeg getMinTurnAngle();

protected:
  /** Defaults to set for all constructors. */
  void defaults();

  /** How much speed the agent can get (min maxspeed, requestedspeed).*/
  Float availableSpeed();
  /** The absolute value of availableSpeed. */
  Float availableAbsSpeed();
  /** The absolute speed the agent will have next turn. */
  Float nextAbsSpeed();
  /** How much the agent needs to accelerate to get the requested speed. */
  Float needAcceleration();
  /** How much power it needs to use to get the requested speed. */
  Float needPower();
  /** How much stamina it needs to use. (needpower with care taken for willback). */
  Float usedPower();
  /** How much stamina the agent will recover to the next turn. */
  Float recoverStamina();
  /** How much stamina there is available for this turn. */
  Float availableStamina();
  /** How much (emergency situation) stamina there is available for this turn. */
  Float emergencyAvailableStamina();
  /** How much stamina there is left to the min stamina limit. */
  Float leftStamina();
  
  /** How much (emergency situation) stamina there is left to the min stamina limit. */
  Float emergencyLeftStamina();
  /** Get the agents current stamina. If unknown it sets it to max. */
  Float getCurStamina();

  /** Whether or not it is a emergenccy situation. */
  FuzzyBool emergency;
  /** The maximum speed allowed. */
  Float maxSpeed;
  /** The maximum stamina to use each cycle (in emergency situation). */
  Float emergencyMaxUseStaminaPerCycle;
  /** The maximum stamina to use each cycle. */
  Float maxUseStaminaPerCycle;
  /** The manimum stamina allowed to reach. */
  Float emergencyMinStamina;
  /** The minimum stamina allowed to reach (in emergency situation). */
  Float minStamina;
  AngleDeg minTurnAngle;
  
  ParameterView *parameterView;
  AgentView *agentView;
  Vector speedAndDirection;
  /** The face angle. */
  AngleDeg angle;
  /** Hide copy-constructor. */
  DashToSpeedVectorFacingDirection(const DashToSpeedVectorFacingDirection&);
  /** Hide assignment operator. */
  DashToSpeedVectorFacingDirection& operator= (const DashToSpeedVectorFacingDirection&);

  int turnHeadCount;
};


RS_END_NAMESPACE


// Remove from here 
/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "DashToSpeedVectorFacingDirection.icc"
#endif
// To here if you do not have any inline definitions!

#endif // !DASHTOSPEEDVECTORFACINGDIRECTION_H
