/* -*- Mode: C++ -*-
 * $Id: DribbleMM.h,v 1.6 2000/12/13 10:21:25 magin846 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: DribbleMM.h
 * Unitname: Framework/skill
 * $Revision: 1.6 $
 * Created by: Magnus Ingmarsson and Magnus Wahlstrm 2000-09-17
 * Last modified by $Author: magin846 $ $Date: 2000/12/13 10:21:25 $
 *
 *
 * HISTORY
 *
 *
 */

/**
 * @file DribbleMM.h
 * Include file for the skill class GotoBall.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.6 $
 * $Author: magin846 $
 * $Date: 2000/12/13 10:21:25 $
 */


#ifndef DRIBBLEMM_H
#define DRIBBLEMM_H

 // Some required files

#include <math.h>

#include "SkillWithDirection.h"
#include "BallView.h"
#include "PlayerView.h"
#include "AgentView.h"
#include "MarkerView.h"
#include "GameView.h"
#include "view_constants.h"
#include "KickCommand.h"
//#include "KickBallToSpeedVector.h"

RS_BEGIN_NAMESPACE


class BallView;
class AgentView;
class PlayerView;
class ParameterView;
class MarkerView;
class GameView;

/**
 * @class DribbleMM DribbleMM.h 
 * Goal: Dribble in a given direction while avoiding opponents.
 * Goal: The agent will try to dribble in the direction given, while
 * taking some detours if neccessary to avoid opponents.
 * 
 * @pre The agent must be in a position to kick the ball
 * @post Whatever (The ball has hopefully been kicked....by _our_ team :-) )
 *
 * @todo One known bug so far, (apart from the Agent being a bit
 * stupid): When the agent is standing by the ball and the ball is
 * located so that the agent is facing towards it's own goal, the
 * agent does not kick the ball due to an error in the function that
 * we use to calculate in which diorection to kick the ball.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.6 $
 * $Author: magin846 $
 * $Date: 2000/12/13 10:21:25 $ */
class DribbleMM : public SkillWithDirection {
public:
  /** Initializer for DribbleMM */
  DribbleMM();
  DribbleMM(AngleDeg dir);

  /** Destructor should clean up after the skill. */
  virtual ~DribbleMM();

  /** This would take a direction (absolute angle in degrees) and a
      'control' argument (1: careless and fast, 9: carefull) if we
      knew how to call it.

      Use setDirection and setCare instead.*/
  //  virtual void setArguments(const std::vector<Float>& arguments);
    //    throw(TooFewArgumentsException);

  //  void setDirection(const AngleDegrees angle);
  
  /** Set the Dribble precision (currently only controls paranoia).
      An integer 1-9, where 9 means try to keep a lot of space between
      you and the opponents, even if you have to back up to do it, and 
      1 means get dangerously close to the opponents. Default 5.*/
  void setPrecision(int prec);

  /** Directly set the distances used when choosing a direction.
      First argument: Enemies closer than this occupy 5/8 of the view
      Second argument: Enemies closer than this occupy 3/8 of the view
      Thid argument: Enemies further away than this might be ignored. */
  void setDistances(Float vclose, Float close, Float far);


  /** This method is used to generate a plan for how the goal of
    the skill should be obtained by the agent. You can assume
    that the preconditions of the skill are true (this can be tested
    by calling the applicable() method, after you have implemented it). */
  Plan generatePlan();
  Plan generatePlan(AngleDeg dir);

  /** DribbleMM is so far never persistent */
  virtual FuzzyBool persistent();

  /** DribbleMM is only applicable when the Agent is within kickable range of the ball. */
  virtual FuzzyBool applicable();

  
private:
  /** Hide copy-constructor. */
  DribbleMM(const DribbleMM&);
  /** Hide assignment operator. */
  DribbleMM& operator= (const DribbleMM&);
  
protected:
  BallView* ballView;
  ParameterView* parameterView;
  PlayerView* playerView;
  AgentView* agentView;
  MarkerView* markerView;  
  GameView* gameView;
  Float kickpower;
  Float d_safe;
  Float d_close;
  Float d_horror;
  int sector(Float dir);
  int bestSector(int a, int b);
  bool safeSector(int sec);
  int pickSector();
  int pickSector(int left);
  VectorUCO *worst(VectorUCO *a, VectorUCO *b);
  VectorUCO *dangers[8];
  VectorUCO *edgeVector(Float dir, PointUCO pos);

};


RS_END_NAMESPACE



#endif // !DRIBBLEMM_H
