

 /* -*- Mode: C++ -*-
 * $Id: DribbleToPosition.cc,v 1.6 2001/06/19 14:01:43 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: DribbleToPosition.cc
 * Unitname: Framework/skill
 * $Revision: 1.6 $
 * Created by: Micael Sjlund & Bjrn Tikkanen, 2000-10-10
 * Last modified by $Author: frehe $ $Date: 2001/06/19 14:01:43 $
 *
 *
 * HISTORY
 *
 * $Log: DribbleToPosition.cc,v $
 * Revision 1.6  2001/06/19 14:01:43  frehe
 * Made minor changes to compile RoboSoc with gcc-3.0
 *
 * Revision 1.5  2001/01/24 07:00:39  olalu526
 * Fine tuning and fixing.
 *
 * Revision 1.4  2001/01/11 22:00:19  olalu526
 * Added the real world model with support for new simulation steps.
 *
 * Revision 1.3  2001/01/11 12:28:00  olalu526
 * Updated with some improvements from the first competition.
 *
 * Revision 1.2  2000/11/08 23:57:48  olalu526
 * Now uses intercept instead.
 *
 * Revision 1.1  2000/10/11 09:19:24  bjoti341
 * And it's dribbling, finally
 *
 * Revision 1.2  2000/10/09 09:38:19  bjoti341
 * First working version
 *
 * Revision 1.1  2000/09/27 13:36:59  bjoti341
 * Added skill DribbleToPosition.
 *
 */

/**
 * @file DribbleToPosition.cc
 * Implementation file for the skill class DribbleToPosition
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.6 $
 * $Author: frehe $
 * $Date: 2001/06/19 14:01:43 $
 */



#include "DribbleToPosition.h"
#include "view_constants.h"
#include "PlayerView.h"
#include "ParameterView.h"
#include "MarkerView.h"
#include "AgentView.h"
#include "BallView.h"
#include "rs_log.h"
#include "KickCommand.h"
#include "GotoBall.h"
#include "InterceptBallStamina.h"
#include "IsOpponentClose.h"
#include "IsBallKnown.h"

RS_BEGIN_NAMESPACE

// --- CONSTRUCTORS ---

DribbleToPosition::DribbleToPosition(Point p)
  : SkillWithPosition(p),
    ballView(NULL),
    agentView(NULL),
    parameterView(NULL),
    playerView(NULL), 
    position(p)
{
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(ballView, BALL_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID, parameterView);
  getView(playerView, PLAYER_VIEW_ID);
}


// --- DESTRUCTOR ---

DribbleToPosition::~DribbleToPosition()
{
  viewManager->releaseView(BALL_VIEW_ID);
  viewManager->releaseView(AGENT_VIEW_ID);
  viewManager->releaseView(PARAMETER_VIEW_ID);
  viewManager->releaseView(PLAYER_VIEW_ID);
}


// --- METHODS ---

void DribbleToPosition::setArguments(Point p)
{
  position = (PointUCO) p;
}

Plan DribbleToPosition::generatePlan()
{
  std::cout << "(D," << std::flush;
  static IsOpponentClose isOpponentClose;
  Command* cmd = NULL;
  Plan plan = Plan();
  ballPoint = ballView->getBallPosition(NOW);
  //GotoBall* gotoBall = new GotoBall();
  static InterceptBallStamina intercept;
  PointUCO agentPosition = agentView->getAgentPosition(NOW);
  // agentPosition = agentView->getAgentPosition(NOW);
  PlayerObject agent = static_cast<PlayerObject>(agentView->getAgentObject(NOW));
  //agent = static_cast<PlayerObject>(agentView->getAgentObject(NOW));
  PlayerObject* closestPlayer = playerView->getClosestPlayer(NOW);
  int kickPower = 10;
  bool kickable = ballView->isBallKickable(NOW);
  AngleDegUCO directionToClosestPlayer;
  FloatUCO distanceToObstacle;
  std::cout << "I," << std::flush;

  /* are we there yet and "holding" the ball? */
  if (((agentPosition.getDistanceTo(position)) < 3) && 
      (ballPoint.getDistanceTo(position) < 3))
    {
      std::cout << "NIL)" << std::flush;
      cmd = NULL; 
      return plan = Plan(Step(cmd));
    }
  
  else if (closestPlayer != NULL)
    {
      std::cout << "C," << std::flush;

      PointUCO closestPlayerPos = closestPlayer->getPosition().getPoint();
      AngleDegUnknown dirToClosestPlayer = agent.getRelativeVectorTo(closestPlayerPos).getAngle().getDeg();
      if (dirToClosestPlayer.isKnown()) {
	std::cout << "K," << std::flush;
	directionToClosestPlayer = dirToClosestPlayer;
      }
      else directionToClosestPlayer = 0;
      
      FloatUnknown disToObstacle =  closestPlayer->getDistance();
      std::cout << "OD," << std::flush;
      if (disToObstacle.isKnown())
	{
	  std::cout << "OK," << std::flush;
	  distanceToObstacle = disToObstacle;
	}
      else distanceToObstacle = 10;
      
      /* Within what distance do we dribble */
      float obstacleRange = 5;
    
      /* Are we facing an obstacle? (some player)
       * Obstacle is player within an certain area between
       * agent and its "goto-position"
       * Kick the ball to the side of obstacle */
      if ((kickable)  && 
	  (abs(directionToClosestPlayer.getDeg()) < 45) &&
	  (dirToClosestPlayer.isKnown()) &&
	  (distanceToObstacle < obstacleRange))
	{
	  std::cout << "Obstacle," << std::flush;
	  /* Obstacle to left */
	  if (directionToClosestPlayer < 0)
	    {
	      cmd = new KickCommand(kickPower, directionToClosestPlayer.getDeg() + 45); 
	      plan = Plan(Step(cmd));
	      delete cmd;
	      cmd = NULL;
	      std::cout << "OLeft)" << std::flush;
	      return plan;
	    }
	  /* Obstacle in the middle or right  */
	  else 
	    {
	      cmd = new KickCommand(kickPower,directionToClosestPlayer.getDeg() - 45); 
	      plan = Plan(Step(cmd));
	      delete cmd;
	      cmd = NULL;
	      std::cout << "ORight)" << std::flush;
	      return plan;
	    }
	}
    }
  
  
  /* Can we kick the ball? -> kick IT! */
  if (kickable)
    {
      std::cout << "Kickable" << std::flush;
      /* Different kickpowers for different distances from position */
      if (agentPosition.getDistanceTo(position) < 5)
	{
	  kickPower = 1;
	}
      else 
	kickPower = 4;
      
      AngleDeg relativeAngleToBall = agent.getRelativeDirectionTo(position).getAngle().getDeg();
      AngleDeg nonKickableAngle = 110;
      AngleDeg maxKickableAngle = 90;
      
      
      /* Agent can't kick into itself... */
      if (relativeAngleToBall > nonKickableAngle)
	relativeAngleToBall = maxKickableAngle;
      if (relativeAngleToBall < -nonKickableAngle)
	relativeAngleToBall = -maxKickableAngle;
      
      cmd = new KickCommand(kickPower, relativeAngleToBall);
      plan = Plan(Step(cmd));
    }
  
  /* otherwise -> go to the ball (or atleast to the position of the ball...) */
  else 
    {
      std::cout << "Else intercept" << std::flush;
      intercept.setEmergency(FALSE);
      if (isOpponentClose()) {
	intercept.setEmergency(TRUE);
      }
      plan = intercept.generatePlan();
    }
  
  delete cmd;
  cmd = NULL;
  std::cout << "!)" << std::flush;
  return plan;
  
}


FuzzyBool DribbleToPosition::persistent()
{
  return FALSE;
}

FuzzyBool DribbleToPosition::applicable()
{
  static IsBallKnown isBallKnown;
  if (isBallKnown()) // && ballView->getRelativeBallDirection(NOW).isKnown())
    return TRUE;
  else
    return FALSE;
}


RS_END_NAMESPACE

