/* -*- Mode: C++ -*-
 * $Id: FaceDirection.h,v 1.2 2000/10/10 19:10:54 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: FaceDirection.h
 * Unitname: Framework/skill
 * $Revision: 1.2 $
 * Created by: Ola Lundqvist 2000-09-27
 * Last modified by $Author: olalu526 $ $Date: 2000/10/10 19:10:54 $
 *
 *
 * HISTORY
 *
 * $Log: FaceDirection.h,v $
 * Revision 1.2  2000/10/10 19:10:54  olalu526
 * Added documentation.
 *
 * Revision 1.1  2000/09/29 17:55:01  olalu526
 * Added FaceDirection, fixed some in FindBall (now works as expected), fixed segmentation fault in TurnDirection and more.
 *
 * Revision 1.3  2000/09/28 20:39:06  olalu526
 * Updated for testing, segmentation fault...
 *
 * Revision 1.2  2000/09/28 20:17:37  olalu526
 * Update FaceDirection.
 *
 * Revision 1.1  2000/09/28 20:12:27  olalu526
 * Added FaceDirection.
 *
 *
 */

/**
 * @file FaceDirection.h
 * Include file for the skill class GotoBall.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.2 $
 * $Author: olalu526 $
 * $Date: 2000/10/10 19:10:54 $
 */


#ifndef FACEDIRECTION_H
#define FACEDIRECTION_H


#include "SkillWithArguments.h"
#include "AgentView.h"
#include "ViewManager.h"
#include "ParameterView.h"
#include "view_constants.h"
#include "TurnNeckCommand.h"


RS_BEGIN_NAMESPACE


/**
 * @class FaceDirection FaceDirection.h
 * This skill will make the agent face a specific direction.
 *
 * @pre The agent body direction is known and the diff angle is high enough.
 * @post The agent is heading for that direction at the right speed.
 *
 * @todo Implement it:
 *	- Write generatePlan();
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.2 $
 * $Author: olalu526 $
 * $Date: 2000/10/10 19:10:54 $
 */
class FaceDirection : public SkillWithArguments {
public:
  /** Default constructor. Here you can initialize the skill. */
  FaceDirection();
  /** Constructor with an angle to face. */
  FaceDirection(const AngleDeg&);

  /** Destructor should clean up after the skill. */
  virtual ~FaceDirection();

  /** Sets the face angle. */
  void setAngle(const AngleDeg&);

  /** This method is used to generate a plan for how the goal of
    the skill should be obtained by the agent. You can assume
    that the preconditions of the skill are true (this can be tested
    by calling the applicable() method, after you have implemented it). */
  virtual Plan generatePlan();
  /* Generates a plan with the help of an absolute angle to turn to.*/
  virtual Plan generatePlan(const AngleDeg&);
  /* Generates a plan with the help of an absolute angle to turn to,
     and how much the body will turn from other commands in this cycle. */
  virtual Plan generatePlan(const AngleDeg&, const AngleDeg&);

  /** This method should reface a FuzzyBool telling the caller
    if the skill wants to be called again. */
  virtual FuzzyBool persistent();
  
  /** Gets the difference between the intended face direction and the one that
      the agent have right now. */
  AngleDeg getDiffAngle() const;

  /** Returns if the skill is applicable. See preconditions for more
      information. Also sets the angle to face, and how much the body will
      turn from other commands in this cycle. */
  virtual FuzzyBool applicable(const AngleDeg&, const AngleDeg&);

  /** Returns if the skill is applicable. See preconditions for more
      information. Also sets the angle to face. */
  virtual FuzzyBool applicable(const AngleDeg&);
  /** Returns if the skill is applicable. See preconditions for more
      information. */
  virtual FuzzyBool applicable();
  /** Sets the arguments in a generic way. */
  virtual void setArguments(const std::vector<Float>&)
    throw(TooFewArgumentsException);

  /** Clears the body turn angle. */
  virtual void noBodyTurn();
  /** Sets how much the body will turn from other commands in this cycle. */
  virtual void setBodyTurn(const AngleDeg&);
  /** Gets how much the body will turn from other commands in this cycle. */
  virtual AngleDeg getBodyTurn();

private:
  /** The face angle. */
  AngleDeg angle;
  /** The bodyturn angle. */
  AngleDeg bodyturn;
  /** Hide copy-constructor. */
  FaceDirection(const FaceDirection&);
  /** Hide assignment operator. */
  FaceDirection& operator= (const FaceDirection&);
  AgentView *agentView;
  ParameterView *parameterView;
};


RS_END_NAMESPACE


// Remove from here 
/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "FaceDirection.icc"
#endif
// To here if you do not have any inline definitions!

#endif // !FACEDIRECTION_H
