/* -*- Mode: C++ -*-
 * $Id: FindBall.cc,v 1.7 2001/01/24 07:00:39 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: FindBall.cc
 * Unitname: Framework/skill
 * $Revision: 1.7 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: olalu526 $ $Date: 2001/01/24 07:00:39 $
 *
 *
 * HISTORY
 * $Log: FindBall.cc,v $
 * Revision 1.7  2001/01/24 07:00:39  olalu526
 * Fine tuning and fixing.
 *
 * Revision 1.6  2001/01/11 22:00:19  olalu526
 * Added the real world model with support for new simulation steps.
 *
 * Revision 1.5  2000/10/07 15:07:36  matka134
 * Now using the predicate IsBallKnown
 *
 * Revision 1.4  2000/09/29 17:55:02  olalu526
 * Added FaceDirection, fixed some in FindBall (now works as expected), fixed segmentation fault in TurnDirection and more.
 *
 * Revision 1.3  2000/09/28 18:52:17  olalu526
 * Now the agent works quite fine.
 *
 * Revision 1.2  2000/09/13 01:38:59  frehe
 * Updated the skills to use getView instead of static_cast<...>.
 *
 * Revision 1.1  2000/09/05 21:12:01  frehe
 * #include rs_log.h
 *
 */

/**
 * @file FindBall.cc
 * Implementation file for the skill class FindBall.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.7 $
 * $Author: olalu526 $
 * $Date: 2001/01/24 07:00:39 $
 */


#include "FindBall.h"
#include "BallView.h"
#include "view_constants.h"
#include "TurnCommand.h"
#include "IsBallKnown.h"

RS_BEGIN_NAMESPACE


FindBall::FindBall()
  : Skill(),
    ballView(NULL),
    gameView(NULL)
{
  getView(ballView, BALL_VIEW_ID);
  getView(gameView, GAME_VIEW_ID);
} 


FindBall::~FindBall()
{
  viewManager->releaseView(BALL_VIEW_ID);
  viewManager->releaseView(GAME_VIEW_ID);
}


Plan FindBall::generatePlan()
{
  static Command* cmd;
  static AngleDegUnknown ang;
  static Plan plan;

  ang = ballView->getRelativeBallDirection(NOW);
  if ( ang.isUnknown() ) {
    cmd = new TurnCommand(45);
  }
  else
    cmd = new TurnCommand(ang.getDeg());

  plan = Plan(Step(cmd));
  delete cmd;
  cmd = NULL;
  
  return plan;
}


FuzzyBool FindBall::persistent()
{
  return FALSE;
}


FuzzyBool FindBall::applicable()
{
  /*
  TimeUnknown t = ballView->getLastBallObservationTime();
  if ( ballView->getRelativeBallDirection(NOW).isUnknown() ||
       ballView->getBallPosition(NOW).isUnknown()) {
    return TRUE;
  }
  else if (t.isUnknown()) {
    return TRUE;
  }
  else if (t.getValue() < gameView->getCurrentTime() -2) {
    return TRUE;
  }
  else {
    return FALSE;
    }*/

  static IsBallKnown isBallKnown;
  return !isBallKnown();
}


RS_END_NAMESPACE
