/* -*- Mode: C++ -*-
 * $Id: InterceptBall.cc,v 1.6 2000/10/10 08:45:40 johbu627 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: InterceptBall.cc
 * Unitname: Framework/skill
 * $Revision: 1.6 $
 * Created by: Rufus & Martin 2000-09-16
 * Last modified by $Author: johbu627 $ $Date: 2000/10/10 08:45:40 $
 *
 *
 * HISTORY
 *
 * $Log: InterceptBall.cc,v $
 * Revision 1.6  2000/10/10 08:45:40  johbu627
 * forgot the factor 100
 *
 * Revision 1.5  2000/10/09 21:36:24  johbu627
 * *** empty log message ***
 *
 * Revision 1.4  2000/09/24 14:14:32  johbu627
 * Minor changes, because of changes in SkillWithSpeed
 *
 * Revision 1.3  2000/09/19 19:16:21  johbu627
 * Some minor changes in InterceptBall.cc
 *
 * Revision 1.2  2000/09/17 22:17:28  johbu627
 * changed InterceptBall to be a child of SkillWithSpeed
 *
 */

/**
 * @file InterceptBall.cc
 * Implementation file for the skill class InterceptBall.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.6 $
 * $Author: johbu627 $
 * $Date: 2000/10/10 08:45:40 $
 */
#include "SkillWithSpeed.h"

#include "InterceptBall.h"

#include "BallView.h"
#include "ParameterView.h"

#include "view_constants.h"
#include "math_utils.h"

#include "TurnCommand.h"
#include "DashCommand.h"

// #include <cmath>


RS_BEGIN_NAMESPACE


// --- CONSTRUCTORS ---

 InterceptBall::InterceptBall(const Float spd)
  :SkillWithSpeed(spd),
   ballView(NULL),
   parameterView(NULL),
   agentView(NULL)
{
  std::cout << "bin im Konstruktor" << std::endl;
  getView(ballView, BALL_VIEW_ID);
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID, parameterView);

  ballDecay = parameterView->SP_ball_decay;
  playerDecay = parameterView->SP_player_decay;
  powerRate = parameterView->SP_dash_power_rate;
  recoveryMin = parameterView->SP_recover_min;
  staminaIncMax = parameterView->SP_stamina_inc_max;
  staminaMax = parameterView->SP_stamina_max;

  effortMin = parameterView->SP_effort_min;
  effortDecThr = parameterView->SP_effort_dec_thr;
}




// --- DESTRUCTOR ---

InterceptBall::~InterceptBall()
{
  viewManager->releaseView(BALL_VIEW_ID);
  viewManager->releaseView(PARAMETER_VIEW_ID);
  viewManager->releaseView(AGENT_VIEW_ID);
}




// --- METHODS ---

PointUCO InterceptBall::ExtrapolateBallPosition(PointUCO BallNow, VectorUCO SpeedNow, FloatUCO cycel)
{
  return (BallNow + SpeedNow * (power(ballDecay, cycel.getValue().getValue().getValue() + 1) -1) / (ballDecay - 1)) ;
}


Float32 InterceptBall::CalculatePower(Float32 desiredSpeed, Float32 effectivity)
{
  Float32 temp = desiredSpeed * (1 - playerDecay) / powerRate / effectivity;

  if (temp > 100)
  {
    return 100;
  }
  else
  {
    return temp;
  }
}


Float32 InterceptBall::CalculateSpeed(Float32 usedPower, Float32 stamina, Float32 effectivity)
{
  if (usedPower < stamina) {
    return usedPower * powerRate / (1 - playerDecay) * effectivity;
  }
  else {
    return staminaIncMax * effortMin * recoveryMin;
  }
}


Float32 InterceptBall::CalculateTime(Float32 desiredSpeed, Float32 dist)
{
  stamina = (agentView->getStamina(NOW)).getValue().getValue().getValue();

  Float32 effort = (agentView->getEffort(NOW)).getValue().getValue().getValue();
  Float32 neededPower = CalculatePower(desiredSpeed, effort);
  Float32 vReached = CalculateSpeed(neededPower, stamina, effort);

  // cycels above effortDecThr
  Float32 c1 = (stamina - effortDecThr * staminaMax) / neededPower;
  if (c1 < 0)
    {
      c1 = 0;
    }

  if (dist / vReached > c1)
    {
      dist = dist - vReached * c1;
      neededPower = CalculatePower(desiredSpeed, effortMin);
      if (stamina > effortDecThr * staminaMax)
        {
          stamina = effortDecThr * staminaMax;
        }

      vReached = CalculateSpeed(neededPower, stamina, effortMin);
      Float32 c2 = stamina / neededPower;
      if ( (dist / vReached) > c2)
        {
          dist = dist - vReached * c2;
          vReached = CalculateSpeed(neededPower, staminaIncMax * recoveryMin, effortMin);
          Float32 c3 = dist / vReached;
          return c1 +c2 +c3;
        }
      else
        {
          return c1+ c2;
        }
    }

  else
    {
      return c1;
    }

}


PointUCO InterceptBall::InterceptBallPosition(Float32 v) {
  PointUCO startPoint, endPoint, guessedPoint, truePoint;
  FloatUCO distGuessed, distGuessedBall, distTrueBall, timePlayer;
  bool toLong;

  VectorUCO relBallVector = ballView->getAbsoluteBallVector(NOW);
 
  PointUCO relBallPosition = PointUCO( relBallVector );
  VectorUCO ballSpeed = ballView->getBallAbsoluteSpeedVector(NOW);

  endPoint = ballSpeed * (1 / ( 1 - ballDecay )) + relBallPosition;  // Limes

  startPoint = relBallPosition;

  PointUCO origo;
  origo.setX(0);
  origo.setY(0);

  for (int i = 0; i < 10; i++) {  // The main iteration of the routine
    guessedPoint = (endPoint - startPoint) * 2/3 + startPoint;  // guesse Point, 2/3 of the line from Start- to Endpoint
    distGuessed = guessedPoint.getDistanceTo(origo);
    timePlayer = CalculateTime(v, distGuessed.getValue().getValue().getValue() );  // Time the Player needs to the Endpoint

    distGuessedBall = guessedPoint.getDistanceTo(relBallPosition);

    truePoint = ExtrapolateBallPosition(relBallPosition, ballSpeed, timePlayer);  // the true position of the ball in the moment, the player arrives the guessed point

    distTrueBall = truePoint.getDistanceTo(relBallPosition);  // True Point --> Ball

    toLong = (distTrueBall < distGuessedBall);

    if (toLong) {  // now we shorten the intervall
      endPoint = guessedPoint;
    }
    else {
      startPoint = guessedPoint;
    }
  }
  return guessedPoint;
}


Plan InterceptBall::generatePlan(const Float spd)
{
  PointUCO interceptPoint;

  static Command* cmd;
  static AngleDegUnknown angBall, angPlayer, ang;
  static Plan plan;

  interceptPoint = InterceptBallPosition(spd); // calculates the ideal InterceptPoint


  PointUCO origo;
  origo.setX(0);
  origo.setY(0);

  angBall = origo.getDirectionTo(interceptPoint); // the angle between Player an InterceptPoint
  angPlayer = agentView->getBodyDirection(NOW);

  ang = angBall - angPlayer;  // angle between Ball and Player

  if ( abs(ang) > 5 ) { // if the angle is to big,
    cmd = new TurnCommand(ang.getDeg());  // the player must turn
  }
  else {
    stamina = (agentView->getStamina(NOW)).getValue().getValue().getValue();
    //    std::cout << "Stamina: " << stamina << std::endl;
    //    std::cout << "gewnschter Speed: " << spd * 100 << std::endl;
    Float32 effort = (agentView->getEffort(NOW)).getValue().getValue().getValue();
    //    std::cout << "Effort: " << effort << std::endl;
    Float32 power = CalculatePower(spd, effort);
    //    std::cout << "Power: " << power << std::endl;
    //    std::cout << std::endl;
    cmd = new DashCommand(power);  // otherwise he can dash
    // cmd = new DashCommand(spd * 100);  // otherwise he can dash
  }

  plan = Plan(Step(cmd));
  delete cmd;
  cmd = NULL;

  return plan;
}


Plan InterceptBall::generatePlan()
{
  return Plan();
}


FuzzyBool InterceptBall::persistent()
{
  if ( (ballView->isBallKickable(NOW) ) ) {
    return FALSE;
  }
  else {
    return TRUE;
  }
}


FuzzyBool InterceptBall::applicable()
{
  if ( (ballView->getRelativeBallVector(NOW).isKnown())
       && (ballView->getBallAbsoluteSpeedVector(NOW).isKnown())
       && (agentView->getBodyDirection(NOW).isKnown()) )
    return TRUE;
  else
    return FALSE;
}


RS_END_NAMESPACE
