/* -*- Mode: C++ -*-
 * $Id: InterceptBallStamina.cc,v 1.9 2001/01/21 19:15:32 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: InterceptBallStamina.cc
 * Unitname: Framework/skill
 * $Revision: 1.9 $
 * Created by: Mattias Karlsson and Per Olof Pettersson 2000/09/19
 * Last modified by $Author: olalu526 $ $Date: 2001/01/21 19:15:32 $
 *
 *
 * HISTORY
 *
 * $Log: InterceptBallStamina.cc,v $
 * Revision 1.9  2001/01/21 19:15:32  olalu526
 * Fixed stall bug.
 *
 * Revision 1.8  2001/01/11 22:00:20  olalu526
 * Added the real world model with support for new simulation steps.
 *
 * Revision 1.7  2001/01/11 12:28:01  olalu526
 * Updated with some improvements from the first competition.
 *
 * Revision 1.6  2000/11/08 20:58:19  mikbr212
 * Changed persistent so that this skill won't be persistent
 * if the agent is losing the ball (ball moving further away)
 * or when no progress (in chasing the ball) is made.
 * These changes can be disabled by setting neverGiveUp to true (default).
 *
 * Added setNeverGiveUp
 * Added some attributes
 *
 * Changed Persistent
 * Changed generatePlan
 * Changed setArguments
 * Changed constructor
 *
 * Revision 1.5  2000/11/07 19:56:28  perpe427
 * Updated to last version after cvs-problems
 *
 * Revision 1.7  2000/11/04 12:45:30  matka134
 * Using default MinTurnAngle
 *
 * Revision 1.6  2000/11/02 15:55:32  perpe427
 * merged versions (removed unused comments)
 *
 * Revision 1.5  2000/11/02 15:05:45  matka134
 * Some internal changes, it now uses a lower default value for minStamina and uses emergency
 *
 * Revision 1.4  2000/10/22 15:10:52  perpe427
 * Made it possible to set minimum turn angle + made generatePlan(...) call DTSVFD::generatePlan() rather than allowing polymorphism
 *
 * Revision 1.3  2000/10/10 15:12:32  perpe427
 * Removed alot of functions that are not used any more or have been transfered to FutureBallView and MoveToPositionFacingDirection. Documented code.
 *
 * Revision 1.2  2000/10/09 22:29:46  matka134
 * Major rewrite of the code in InterceptBallStamina. Moste of the code has been moved from this class to some MoveToPosition classes. It now uses MoveToPositinFacingBall to do most of the calulations.
 * It will hopefully work better now that we take the direction of the neck into account and such stuff. I havn't been testing it much, but it seem to work quite ok.
 *
 * Revision 1.1  2000/09/27 12:17:01  perpe427
 * Renamed InterceptBallPoM as InterceptBallStamina
 *
 */

/**
 * @file InterceptBallStamina.cc
 * Implementation file for the skill class InterceptBallStamina.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.9 $
 * $Author: olalu526 $
 * $Date: 2001/01/21 19:15:32 $
 */

#include "FindBall.h"
#include "InterceptBallStamina.h"
#include "BallView.h"
#include "AgentView.h"
#include "view_constants.h"
#include "math_utils.h"
#include "TurnCommand.h"
#include "DashCommand.h"
#include "IsBallKnown.h"
#include "FutureBallView.h"

RS_BEGIN_NAMESPACE


// --- CONSTRUCTORS ---

InterceptBallStamina::InterceptBallStamina(Float minStamina = -1,
					   bool _neverGiveUp = true)
  : SkillWithArguments(),
    ballView(NULL),
    parameterView(NULL),
    agentView(NULL),
    futureBallView(NULL),
    moveToPos(NULL),
    em(TRUE)
{
  getView(ballView, BALL_VIEW_ID);
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID, parameterView);
  getView(futureBallView, FUTUREBALL_VIEW_ID);

  moveToPos = new MoveToPositionFacingBall( Point(0,0) );
  //moveToPos->setMinTurnAngle(15);
  moveToPos->setEmergency(em); // default??
  if(minStamina > -0.5) {
    setMinStamina(minStamina);
  }
  
  setNeverGiveUp(_neverGiveUp);
}

void
InterceptBallStamina::setEmergency(const FuzzyBool& e) {
  em = e;
}

void 
InterceptBallStamina::setMinStamina(Float minStamina)
{
  //moveToPos->setMinStaminaWhenFinished(minStamina);
  moveToPos->setEmergencyMinStaminaWhenFinished(minStamina); // right??
}



// --- DESTRUCTOR ---

InterceptBallStamina::~InterceptBallStamina()
{
  viewManager->releaseView(BALL_VIEW_ID);
  viewManager->releaseView(PARAMETER_VIEW_ID);
  viewManager->releaseView(AGENT_VIEW_ID);
  viewManager->releaseView(FUTUREBALL_VIEW_ID);

  delete moveToPos;
}


// --- METHODS ---

Plan InterceptBallStamina::generatePlan()
{

  //cout << "Ball angle: " <<  ballView->getBallAbsoluteSpeedDirection(NOW) << " Ball speed: " <<  ballView->getBallSpeed(NOW) << endl;
  //cout << "Ball end pos: " << futureBallView->fastBallStopPosition() << endl;


  // *** ADDED BY MICKE ***
  if(!neverGiveUp) {
    if(Time::currentTime != timeStamp + 1) {
      // skill wasn't called last cycle -> reset vars
      closestBallDistance = HUGE;
    } 
    timeStamp = Time::currentTime;
    
    if(ballView->getBallDistance(NOW).isKnown()) {
      float dist = ballView->getBallDistance(NOW).getValue().getValue().getValue();
      if(dist < closestBallDistance) {
	closestBallDistance = dist;
	closestTime = Time::currentTime;
      }
    }
  } // !neverGiveUp
  // *** END ***
  
  UInt step = 16;
  UInt cycles = 0;
  FuzzyBool reachable;
  Point ballPos;

  // Check to see if the ball is reachable in a certain time. Increase the time in 'step' cycles
  // in each iteration.
  do {
    cycles += step;
    ballPos = futureBallView->fastBallPosition( Int(cycles) );
    reachable = moveToPos->reachable(ballPos, cycles);
  } while ( reachable.isFalse() && (cycles < 1000) );
  // If cycles => 1000 the agent is probalby too tired or something has gone wrong.
  // The result will be that the agent will run towards the position where
  // the ball will stop (or at least it will almost stop in 1000 cycles).


  // Get a better value for the time it will take to intecept the ball.
  do {
    step = step /2;
    if(reachable.isTrue())
      cycles -= step;
    else
      cycles += step;
    ballPos = futureBallView->fastBallPosition( Int(cycles) );
    reachable = moveToPos->reachable(ballPos, cycles);
  } while( step > 1 );
  
  if(reachable.isFalse()) {
    cycles++;
    ballPos = futureBallView->fastBallPosition( Int(cycles) );
  }
  return moveToPos->generatePlan( ballPos );
}


FuzzyBool InterceptBallStamina::persistent()
{
  // *** ADDED BY MICKE ***
  if(!neverGiveUp &&
     ballView->getBallDistance(NOW).isKnown()) {
    float dist = ballView->getBallDistance(NOW).getValue().getValue().getValue();
    if(dist > closestBallDistance + giveUpDistance || // ball has moved x m away
       Time::currentTime > closestTime + giveUpTime) // no progress in t cycles
      return false;
  }
  // *** END ***

  if ( !ballView->isBallKickable(NOW) ) {
    return TRUE;
  } else {
    return FALSE;
  }
}


FuzzyBool InterceptBallStamina::applicable()
{
  IsBallKnown isBallKnown;
  
  if ( isBallKnown().isFalse() ||
       ballView->isBallKickable(NOW) )
    return FALSE;
  else
    return TRUE;
}

void InterceptBallStamina::setArguments(const std::vector<Float>& arguments) throw(TooFewArgumentsException)
{
  //  if (arguments.size() < 1) {
  //  RS_THROW(TooFewArgumentsException,
  //	     "SkillWithSpeed got less than one argument.");
  //}
  if (arguments.size() >= 1) {
    moveToPos->setMinStaminaWhenFinished( arguments[0] );
  }
  // *** ADDED BY MICKE ***
  if(arguments.size() >= 2) 
    neverGiveUp = (arguments[1] != 0);
  if(arguments.size() >= 3) 
    giveUpDistance = arguments[2];
  if(arguments.size() >= 4) 
    giveUpTime = (int)arguments[3]; 
  // *** END ***
}

void InterceptBallStamina::setNeverGiveUp(bool flag,
					  float _giveUpDist,
					  int _giveUpTime) {
  neverGiveUp = flag;
  giveUpDistance = _giveUpDist;
  giveUpTime = _giveUpTime;
}

RS_END_NAMESPACE


// Remove from here
/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "InterceptBallStamina.icc"
#endif
// To here if you do not have any inline definitions!
