/* -*- Mode: C++ -*-
 * $Id: KickBallInDirection.h,v 1.9 2001/06/19 14:01:44 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: GotoBall.h
 * Unitname: Framework/skill
 * $Revision: 1.9 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2001/06/19 14:01:44 $
 *
 *
 * HISTORY
 * $Log: KickBallInDirection.h,v $
 * Revision 1.9  2001/06/19 14:01:44  frehe
 * Made minor changes to compile RoboSoc with gcc-3.0
 *
 * Revision 1.8  2000/11/21 19:58:37  marsc818
 * better adjustment and possibility to use max power kick now
 *
 * Revision 1.7  2000/11/09 18:58:48  marsc818
 * some improvements
 *
 * Revision 1.6  2000/11/01 16:17:57  krini782
 * CutOffPassAngle should not segfault anymore when there are no opponents.
 *
 * Revision 1.5  2000/10/26 07:29:39  danbe628
 * Html-uncommented comments
 *
 * Revision 1.4  2000/10/11 23:03:05  danbe628
 * KickBallInDirection now knows it's limits and don't try to kick the ball harder than it can resulting in a bad direction.
 *
 * Revision 1.3  2000/10/11 16:51:38  danbe628
 * KickBallInDirection now handles moving balls
 * Some fixes to KickView
 *
 * Revision 1.2  2000/09/21 10:06:27  danbe628
 * Started work on the Score and KickallInDirection skills.
 *
 * Revision 1.1  2000/09/20 10:21:58  danbe628
 * Added the beginning of the skill KickBallInDirection.
 *
 * Revision 1.1  2000/09/05 21:12:02  frehe
 * #include rs_log.h
 *
 */

/*
 * @file GotoBall.h
 * Include file for the skill class KickBallInDirection.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.9 $
 * $Author: frehe $
 * $Date: 2001/06/19 14:01:44 $
 */


#ifndef KICKBALLINDIRECTION_H
#define KICKBALLINDIRECTION_H


#include "SkillWithArguments.h"


RS_BEGIN_NAMESPACE


class BallView;
class PlayerView;
class AgentView;
class KickView;

/**
 * @class KickBallInDirection KickBallInDirection.h
 * A skill for kicking the ball in a certain direction
 * no matter how the ball currently moves.
 *
 * @pre The precondition for the skill are that the ball is kickable (within the kickable area)
 * @post There are no postconditions. Once you have kicked the ball is off and there is nothing to do about it (for a primitive skill).
 * @invariant The class invariant is...
 *
 * @todo Maybe say that it's not applicable when the ball is moving so fast that there is no way to change to the desired direction.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.9 $
 * $Author: frehe $
 * $Date: 2001/06/19 14:01:44 $
 */


class KickBallInDirection : public SkillWithArguments {
public:
  /**
     \arg \c eps is Espilon
     the approximation value used for comparisons
  */
  KickBallInDirection(Float eps = 0.1, bool UseMaxPower = false);
  virtual ~KickBallInDirection();
  /**
     \arg \c args Vector of two elements
     - the direction
     - The power, as the power to kick with if the ball had been still. Not guaranteed, direction has priority.
  */
  virtual void setArguments(const std::vector<Float> & args)
    throw(RS::TooFewArgumentsException);
  
  /**
     \arg \c bUseMaxPower
     boolean specifying whether to use the maximum power when kicking or not (false by default)
   */
  virtual Plan generatePlan();

  virtual FuzzyBool persistent();
  virtual FuzzyBool applicable();

  void setUseMaxPower(const bool UseMaxPower = true) { bUseMaxPower = UseMaxPower; }
  bool getUseMaxPower() const { return bUseMaxPower; }
  

protected:

  // computes the angle and power needed
  VectorUnknown computeKickSpeedVector(Float desiredPower);
  VectorUnknown findBestKickSpeedVector(void);

  Float Epsilon;
  
  bool bUseMaxPower;

  // args
  Float power;
  AngleDeg angle;

  // vector
  VectorUCO ballSpeedVector; // ball speed vector at that time, will be updated in the constructor

  // views
  BallView* ballView;
  PlayerView* playerView;
  AgentView* agentView;
  ParameterView* parameterView;
  KickView* kickView;
};


RS_END_NAMESPACE


#endif // !KICKBALLINDIRECTION_H
