/* -*- Mode: C++ -*-
 * $Id: KickBallToAbsoluteSpeedVector.cc,v 1.3 2001/06/19 14:01:44 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: KickBallToAbsoluteSpeedVector.cc
 * Unitname: Framework/skill
 * $Revision: 1.3 $
 * Created by: Ola Lundqvist 2001-01-23
 * Last modified by $Author: frehe $ $Date: 2001/06/19 14:01:44 $
 *
 *
 * HISTORY
 *
 * $Log: KickBallToAbsoluteSpeedVector.cc,v $
 * Revision 1.3  2001/06/19 14:01:44  frehe
 * Made minor changes to compile RoboSoc with gcc-3.0
 *
 * Revision 1.2  2001/01/24 07:00:39  olalu526
 * Fine tuning and fixing.
 *
 * Revision 1.1  2001/01/23 17:18:16  olalu526
 * Added kick to absoulute vector
 *
 */

/**
 * @file KickBallToAbsoluteSpeedVector.cc
 * Implementation file for the skill class KickBallToAbsoluteSpeedVector.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.3 $
 * $Author: frehe $
 * $Date: 2001/06/19 14:01:44 $
 */


#include "KickBallToAbsoluteSpeedVector.h"


RS_BEGIN_NAMESPACE

/**
 * Constructor.
 */
KickBallToAbsoluteSpeedVector::KickBallToAbsoluteSpeedVector(const Vector& vec)
  : SkillWithVector(vec),
    vect(vec),
    kick(Vector(0,0)),
    parameterView(NULL),
    agentView(NULL)
{
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID, parameterView);
} 

/**
 * Destructor.
 */
KickBallToAbsoluteSpeedVector::~KickBallToAbsoluteSpeedVector()
{
  viewManager->releaseView(PARAMETER_VIEW_ID);
  viewManager->releaseView(AGENT_VIEW_ID);
}

void KickBallToAbsoluteSpeedVector::setVector(const Vector& vec) {
  vect = vec;
}

Vector KickBallToAbsoluteSpeedVector::getVector() const {
  return vect;
}

/**
 * Generates a plan.
 */
Plan KickBallToAbsoluteSpeedVector::generatePlan(const Vector& vec) {
  setVector(vec);
  return generatePlan();
}

/**
 * Generates a plan.
 */
Plan KickBallToAbsoluteSpeedVector::generatePlan() {
  std::cout << "(TO GOAL KICK " << vect << ")" << std::flush;
  AngleDegUnknown angu = agentView->getFaceDirection(NOW);
  std::cout << angu.getAngle() << std::flush;
  angu = agentView->getBodyDirection(NOW);
  std::cout << angu.getAngle() << std::flush;
  if (angu.isKnown()) {
    kick.setVector(Vector(vect.getAngle() - angu.getAngle(), vect.getLength()));
  }
  return kick.generatePlan();
}

/**
 * Checks if this skill should be called again (this cycle).
 */
FuzzyBool KickBallToAbsoluteSpeedVector::persistent()
{
  return FALSE; // This is a one-step skill
}

/**
 * Checks if the preconds are true.
 */
FuzzyBool KickBallToAbsoluteSpeedVector::applicable(const Vector& vec) {
  setVector(vec);
  return applicable();
}

/**
 * Checks if the preconds are true.
 */
FuzzyBool KickBallToAbsoluteSpeedVector::applicable() {
  AngleDegUnknown angu = agentView->getFaceDirection(NOW);
  if (angu.isKnown()) {
    kick.setVector(Vector(vect.getAngle() - angu.getAngle(), vect.getLength()));
  }
  return kick.applicable() && FuzzyBool(angu.isKnown());
}

RS_END_NAMESPACE
