/* -*- Mode: C++ -*-
 * $Id: KickBallToAbsoluteSpeedVector.h,v 1.1 2001/01/23 17:18:17 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: TurnBall.h
 * Unitname: Framework/skill
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: olalu526 $ $Date: 2001/01/23 17:18:17 $
 *
 *
 * HISTORY
 * $Log: KickBallToAbsoluteSpeedVector.h,v $
 * Revision 1.1  2001/01/23 17:18:17  olalu526
 * Added kick to absoulute vector
 *
 */

#ifndef KICKBALLTOABSOLUTESPEEDVECTOR_H
#define KICKBALLTOABSOLUTESPEEDVECTOR_H


#include "SkillWithVector.h"
#include "ParameterView.h"
#include "AgentView.h"
#include "rs_log.h"
#include "view_constants.h"
#include "KickBallToSpeedVector.h"


RS_BEGIN_NAMESPACE


/**
 * @class KickBallToAbsoluteSpeedVector KickBallToAbsoluteSpeedVector.h
 * This class encapsulates a kicking skill.
 *
 * 
 * @pre The preconditions for the skill are:
 *  Same as the relative kick skill + that the agent has to know where
 *  it is.
 *
 * @post The postconditions for the skill are...
 * 
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.1 $
 * $Author: olalu526 $
 * $Date: 2001/01/23 17:18:17 $
 */
class KickBallToAbsoluteSpeedVector : public SkillWithVector {
public:
  KickBallToAbsoluteSpeedVector(const Vector& vec);
  virtual ~KickBallToAbsoluteSpeedVector();

  virtual Plan generatePlan();
  virtual Plan generatePlan(const Vector& vec);

  virtual void setVector(const Vector& vec);
  virtual Vector getVector() const;

  virtual FuzzyBool persistent();

  virtual FuzzyBool applicable();
  virtual FuzzyBool applicable(const Vector& vec);

protected:
  Vector vect;
  KickBallToSpeedVector kick;
  ParameterView* parameterView;
  AgentView* agentView;
};


RS_END_NAMESPACE


#endif // !KICKBALLTOABSOLUTESPEEDVECTOR_H


