/* -*- Mode: C++ -*-
 * $Id: KickBallToSpeedVector.cc,v 1.8 2000/11/08 19:32:07 mikbr212 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: KickBallToSpeedVector.cc
 * Unitname: Framework/skill
 * $Revision: 1.8 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: mikbr212 $ $Date: 2000/11/08 19:32:07 $
 *
 *
 * HISTORY
 *
 * $Log: KickBallToSpeedVector.cc,v $
 * Revision 1.8  2000/11/08 19:32:07  mikbr212
 * Fixed mem-leak in generatePlan
 *
 * Revision 1.7  2000/11/07 18:10:43  mikbr212
 * changed persistent
 *
 * Revision 1.6  2000/10/30 18:32:45  mikbr212
 * added comments
 *
 */

/**
 * @file KickBallToSpeedVector.cc
 * Implementation file for the skill class KickBallToSpeedVector.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.8 $
 * $Author: mikbr212 $
 * $Date: 2000/11/08 19:32:07 $
 */


#include "KickBallToSpeedVector.h"


RS_BEGIN_NAMESPACE

/**
 * Constructor.
 */
KickBallToSpeedVector::KickBallToSpeedVector(const Vector& vec)
  : SkillWithVector(vec),
    ballView(NULL),
    kickBallView(NULL)
{
  ParameterView* parameterView = NULL;
  AgentView* agentView = NULL;
  CommandView* commandView = NULL;
  
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(ballView, BALL_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID, parameterView);
  getView(commandView, COMMAND_VIEW_ID);
  getView(kickBallView, KICKBALL_VIEW_ID, parameterView,
	  ballView, agentView, commandView);
} 

/**
 * Destructor.
 */
KickBallToSpeedVector::~KickBallToSpeedVector()
{
  viewManager->releaseView(PARAMETER_VIEW_ID);
  viewManager->releaseView(BALL_VIEW_ID);
  viewManager->releaseView(AGENT_VIEW_ID);
  viewManager->releaseView(COMMAND_VIEW_ID);
  viewManager->releaseView(KICKBALL_VIEW_ID);
}

/**
 * Sets vector.
 */
void KickBallToSpeedVector::setVector(const Vector& vec) {
  vector = vec;
}

/**
 * Retrieves vector.
 */
Vector KickBallToSpeedVector::getVector() const {
  return vector;
}

/**
 * Generates a plan.
 */
Plan KickBallToSpeedVector::generatePlan(const Vector& vec) {
  setVector(vec);
  return generatePlan();
}

/**
 * Generates a plan.
 */
Plan KickBallToSpeedVector::generatePlan() {
  static Command* cmd = NULL;

  if(cmd != NULL) {  
    delete cmd; 
    cmd = NULL;
  }

  if(!ballView->isBallKickable(NOW)) 
    return Plan(); // empty plan

  Vector kickVec = kickBallView->getKickVector(vector);
  
  cmd = new KickCommand(kickVec.getLength()/kickBallView->kickPowerRate(),
			kickVec.getAngle().getDeg());
  return Plan(Step(cmd));
}

/**
 * Checks if this skill should be called again (this cycle).
 */
FuzzyBool KickBallToSpeedVector::persistent()
{
  return FALSE; // This is a one-step skill
}

/**
 * Checks if the preconds are true.
 */
FuzzyBool KickBallToSpeedVector::applicable(const Vector& vec) {
  setVector(vec);
  return applicable();
}

/**
 * Checks if the preconds are true.
 */
FuzzyBool KickBallToSpeedVector::applicable() {
  return kickBallView->canShootToSpeedVector(vector);
}





RS_END_NAMESPACE
