/* -*- Mode: C++ -*-
 * $Id: KickBallToSpeedVector.h,v 1.4 2000/10/09 18:48:09 mikbr212 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: TurnBall.h
 * Unitname: Framework/skill
 * $Revision: 1.4 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: mikbr212 $ $Date: 2000/10/09 18:48:09 $
 *
 *
 * HISTORY
 * $Log: KickBallToSpeedVector.h,v $
 * Revision 1.4  2000/10/09 18:48:09  mikbr212
 * Added comments
 *
 * Revision 1.3  2000/10/07 14:26:55  mikbr212
 * *** empty log message ***
 *
 * Revision 1.2  2000/09/27 14:29:59  mikbr212
 * fixed ballspeed bug
 *
 * Revision 1.1  2000/09/24 19:30:19  andbo928
 * Various skills.
 *
 * Revision 1.1  2000/09/05 21:12:02  frehe
 * #include rs_log.h
 *
 */

#ifndef KICKBALLTOSPEEDVECTOR_H
#define KICKBALLTOSPEEDVECTOR_H


#include "SkillWithVector.h"
#include "CoordVector.h"
#include "CoordPoint.h"
#include "Line.h"
#include "KickCommand.h"
#include "ParameterView.h"
#include "BallView.h"
#include "AgentView.h"
#include "KickBallView.h"
#include "rs_log.h"
#include "view_constants.h"



RS_BEGIN_NAMESPACE


/**
 * @class KickBallToSpeedVector KickBallToSpeedVector.h
 * This class encapsulates a kicking skill.
 *
 * 
 * @pre The preconditions for the skill are:
 * 1) ball in kickable_area
 * 2) no obstacle on straight line from current ball position to next ball position 
 * @post The postconditions for the skill are...
 * 
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.4 $
 * $Author: mikbr212 $
 * $Date: 2000/10/09 18:48:09 $
 */
class KickBallToSpeedVector : public SkillWithVector {
public:
  KickBallToSpeedVector(const Vector& vec);
  virtual ~KickBallToSpeedVector();

  //  virtual void setArguments(const Vector&) 
  //    throw(TooFewArgumentsException);
  virtual void setVector(const Vector& vec);
  virtual Vector getVector() const;
  
  virtual Plan generatePlan();
  virtual Plan generatePlan(const Vector& vec);

  virtual FuzzyBool persistent();

  virtual FuzzyBool applicable();
  virtual FuzzyBool applicable(const Vector& vec);

protected:

  ParameterView* parameterView;
  BallView* ballView;
  AgentView* agentView;
  KickBallView* kickBallView;
  //Vector vector; inherited
  
};


RS_END_NAMESPACE


#endif // !KICKBALLTOSPEEDVECTOR_H


