/* -*- Mode: C++ -*-
 * $Id: KickBallToSpeedVectorMulti.h,v 1.5 2000/11/01 16:38:59 mikbr212 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: TurnBall.h
 * Unitname: Framework/skill
 * $Revision: 1.5 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: mikbr212 $ $Date: 2000/11/01 16:38:59 $
 *
 *
 * HISTORY
 * $Log: KickBallToSpeedVectorMulti.h,v $
 * Revision 1.5  2000/11/01 16:38:59  mikbr212
 * fixed constructor
 *
 * Revision 1.4  2000/11/01 16:01:40  mikbr212
 * changed constructor
 *
 * Revision 1.3  2000/11/01 12:43:00  mikbr212
 * Added possibility to do a fast kick
 *
 * Revision 1.2  2000/10/28 20:29:30  mikbr212
 * generatePlan now uses time stamps and phases; Fixed wrong-calculations in recalcDesPos; Finally: made a working (?) skill!
 *
 * Revision 1.1  2000/10/07 14:26:56  mikbr212
 * *** empty log message ***
 *
 * Revision 1.2  2000/09/27 14:29:59  mikbr212
 * fixed ballspeed bug
 *
 * Revision 1.1  2000/09/24 19:30:19  andbo928
 * Various skills.
 *
 * Revision 1.1  2000/09/05 21:12:02  frehe
 * #include rs_log.h
 *
 */

#ifndef KICKBALLTOSPEEDVECTORMULTI_H
#define KICKBALLTOSPEEDVECTORMULTI_H


#include "SkillWithVector.h"
#include "CoordVector.h"
#include "CoordPoint.h"
#include "Line.h"
#include "ParameterView.h"
#include "BallView.h"
#include "AgentView.h"
#include "Time.h"

#include "KickBallView.h"
#include "KickBallToSpeedVector.h"
#include "TurnBall.h"

#include "IsOpponentClose.h"
#include "CanShootToVector.h"

#include "rs_log.h"
#include "view_constants.h"



RS_BEGIN_NAMESPACE

/**
 * @class KickBallToSpeedVectorMulti KickBallToSpeedVectorMulti.h
 * This class encapsulates a scoring skill.
 *
 * <!--
 * @pre The preconditions for the skill are...
 * @post The postconditions for the skill are...
 * @invariant The class invariant is...
 *
 * @todo This needs to be done...
 * -->
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.5 $
 * $Author: mikbr212 $
 * $Date: 2000/11/01 16:38:59 $
 */
class KickBallToSpeedVectorMulti : public SkillWithVector {
public:
  KickBallToSpeedVectorMulti(const Vector& vec, bool fast=true, bool adaptive=false);

  virtual ~KickBallToSpeedVectorMulti();

  //  virtual void setArguments(const Vector&) 
  //    throw(TooFewArgumentsException);
  virtual void setVector(const Vector& vec);
  virtual Vector getVector() const;
  
  virtual bool setPlayerMargin(double x = 15.0);
  virtual bool setKickableMargin(double x = 15.0);

  virtual void setFastKick(bool fast, bool adaptive = false);

  virtual Plan generatePlan();
  virtual Plan generatePlan(const Vector& vec);

  virtual FuzzyBool persistent();

  virtual FuzzyBool applicable();
  virtual FuzzyBool applicable(const Vector& vec);

protected:
  void init();
  
  virtual void recalcDesPos();

  double bufR; //Buffer radius 
  double kickR; //Kickable area radius 

  double bufMin;
  double kickMax;

  static const int turnPhase = 1;
  static const int backKickPhase = 2;
  static const int simpleKickPhase = 3;
  int phase;
  Time timeStamp;

  bool doFast;
  bool doAdaptive;

  Vector desBallPosR;
  Vector desBallPosL;
  
  IsOpponentClose emergency;
  CanShootToVector canKick;
  
  ParameterView* parameterView;
  BallView* ballView;
  AgentView* agentView;
  KickBallView* kickBallView;
  
  KickBallToSpeedVector kickSkill;
  TurnBall turnSkill;
  //Vector vector; inherited
  
};


RS_END_NAMESPACE


#endif // !KICKBALLTOSPEEDVECTORMULTI_H
