/* -*- Mode: C++ -*-
 * $Id: KickForGoal.cc,v 1.2 2001/06/19 14:01:44 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: KickForGoal.cc
 * Unitname: Framework/skill
 * $Revision: 1.2 $
 * Created by: Ola Lundqvist 2001-01-23
 * Last modified by $Author: frehe $ $Date: 2001/06/19 14:01:44 $
 *
 *
 * HISTORY
 *
 * $Log: KickForGoal.cc,v $
 * Revision 1.2  2001/06/19 14:01:44  frehe
 * Made minor changes to compile RoboSoc with gcc-3.0
 *
 * Revision 1.1  2001/01/23 22:17:39  olalu526
 * Fixed kick against goal.
 *
 *
 */

/**
 * @file KickForGoal.cc
 * Implementation file for the skill class KickForGoal.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2001/06/19 14:01:44 $
 */


#include "KickForGoal.h"
#include "math_utils.h"
#include "InterceptBallStamina.h"
#include "IAmClosestToBall.h"
#include "IsOpponentClose.h"
#include "FindBall.h"
#include "KickBallToAbsoluteSpeedVector.h"

RS_BEGIN_NAMESPACE


// --- CONSTRUCTORS ---

KickForGoal::KickForGoal()
  : Skill(),
    kick(Vector(0,0)),
    ballView(NULL),
    parameterView(NULL),
    agentView(NULL)
{
  getView(ballView, BALL_VIEW_ID);
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID, parameterView);
}

// --- DESTRUCTOR ---

KickForGoal::~KickForGoal()
{
  viewManager->releaseView(BALL_VIEW_ID);
  viewManager->releaseView(PARAMETER_VIEW_ID);
  viewManager->releaseView(AGENT_VIEW_ID);
}


// --- METHODS ---

Plan KickForGoal::generatePlan()
{
  AngleDeg dir = AngleDeg(0);
  if (agentView->getAgentPosition(NOW).isKnown()) {
    Point p = agentView->getAgentPosition(NOW).getPoint();
    dir = p.getDirectionTo(Point(54,0));
  }

  Vector kv = Vector(dir,Float(2.4));
  kick.setVector(kv);
  if (kick.applicable()) {
    return kick.generatePlan();
  }
  kv = Vector(dir,Float(2));
  kick.setVector(kv);
  if (kick.applicable()) {
    return kick.generatePlan();
  }
  kv = Vector(dir,Float(1.6));
  kick.setVector(kv);
  if (kick.applicable()) {
    return kick.generatePlan();
  }
  kv = Vector(dir,Float(1.4));
  kick.setVector(kv);
  if (kick.applicable()) {
    return kick.generatePlan();
  }
  kv = Vector(dir,Float(1));
  kick.setVector(kv);
  if (kick.applicable()) {
    return kick.generatePlan();
  }
  kv = Vector(dir,Float(.6));
  kick.setVector(kv);
  if (kick.applicable()) {
    return kick.generatePlan();
  }
  std::cout << "KickForGoal: This should NEVER happen!" << std::flush;
  return Plan();
}

FuzzyBool KickForGoal::persistent()
{
  // Never persistent.
  return FALSE;
}


FuzzyBool KickForGoal::applicable()
{
  AngleDeg dir = AngleDeg(0);
  if (agentView->getAgentPosition(NOW).isKnown()) {
    Point p = agentView->getAgentPosition(NOW).getPoint();
    dir = p.getDirectionTo(Point(54,0));
  }
  Vector kv = Vector(dir,Float(.6));
  kick.setVector(kv);
  return kick.applicable();
}


RS_END_NAMESPACE


// Remove from here
/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "KickForGoal.icc"
#endif
// To here if you do not have any inline definitions!
