/* -*- Mode: C++ -*-
 * $Id: KickForGoal.h,v 1.1 2001/01/23 22:17:40 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: KickForGoal.h
 * Unitname: Framework/skill
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 2000-09-22
 * Last modified by $Author: olalu526 $ $Date: 2001/01/23 22:17:40 $
 *
 *
 * HISTORY
 *
 * $Log: KickForGoal.h,v $
 * Revision 1.1  2001/01/23 22:17:40  olalu526
 * Fixed kick against goal.
 *
 */

/**
 * @file KickForGoal.h
 * Include file for the skill class GotoBall.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.1 $
 * $Author: olalu526 $
 * $Date: 2001/01/23 22:17:40 $
 */


#ifndef KICKFORGOAL_H
#define KICKFORGOAL_H


#include "Skill.h"
#include "KickBallToAbsoluteSpeedVector.h"
#include "BallView.h"
#include "ParameterView.h"
#include "AgentView.h"

RS_BEGIN_NAMESPACE

class BallView;
class ParameterView;
class AgentView;

/**
 * @class KickForGoal KickForGoal.h
 * The skill to defend a zone.
 * This skill is used to defend a zone. With this skill the agent should
 * (in the future) be able to positionate, intercept the ball, and kick
 * the ball out of the zone (in the right direction) or pass an other
 * friendly player.
 *
 * @pre The preconditions for the skill are: (and)
 *	- If the ball is inside the zone or will stop inside the zone.
 *	- Not if the ball is kickable and no opponent is near.
 *	- Ball is not heading for friendly players.
 *	  This is implemented as quick as cutoffpassangle does not segfault.
 * @post The intended postconditions for the skill are:
 *	- To get the ball out of the zone (on the right side).
 *	- The ball is kickable and no opponent is near.
 *
 * @todo There are some things todo left:
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.1 $
 * $Author: olalu526 $
 * $Date: 2001/01/23 22:17:40 $
 */

class KickForGoal : public Skill {
public:
  /** Here you can initialize the skill, with a zone to defend. */
  KickForGoal();

  /** Destructor should clean up after the skill. */
  ~KickForGoal();

  
  /** This method is used to generate a plan for how the goal of
      the skill should be obtained by the agent.
      For more information see the code it should be quite self explained.
  */
  virtual Plan generatePlan();

  /** This method should return a FuzzyBool telling the caller
      if the skill wants to be called again. Never persistent. */
  virtual FuzzyBool persistent();

  /** This method implements the precondition specified in
      the description of the skill. */
  virtual FuzzyBool applicable();

  
private:
  KickBallToAbsoluteSpeedVector kick;
  BallView* ballView;
  ParameterView* parameterView;
  AgentView* agentView;
};

RS_END_NAMESPACE


// Remove from here 
/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#endif // !KICKFORGOAL_H
