/* -*- Mode: C++ -*-
 * $Id: MoveToPosition.cc,v 1.10 2000/11/16 16:36:21 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: MoveToPosition.cc
 * Unitname: Framework/skill
 * $Revision: 1.10 $
 * Created by: Jan Axelson and Christer Rosenquist 2000-09-24
 * Last modified by $Author: frehe $ $Date: 2000/11/16 16:36:21 $
 *
 *
 * HISTORY
 * */

/**
 * @file MoveToPosition.cc
 *  
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.10 $
 * $Author: frehe $
 * $Date: 2000/11/16 16:36:21 $
 */


#include "MoveToPosition.h"
#include "view_constants.h"
#include "Command.h"
#include "TurnCommand.h"
#include "DashCommand.h"
#include "Step.h"

RS_BEGIN_NAMESPACE


// --- CONSTRUCTORS ---

MoveToPosition::MoveToPosition()
  :  SkillWithPosition(Point(0,0)),
     agentView(NULL)
{
  agentView = static_cast<AgentView*>(viewManager->getView(AGENT_VIEW_ID));
  setDash(50); // default dash power.
}

MoveToPosition::MoveToPosition(const Point &position)
  : SkillWithPosition(position),
    agentView(NULL)
{
  agentView = static_cast<AgentView*>(viewManager->getView(AGENT_VIEW_ID));
  setDash(50);// default dash power.
}

MoveToPosition::MoveToPosition(const int dash)
  : SkillWithPosition(Point(0,0)),
    agentView(NULL)
{
  agentView = static_cast<AgentView*>(viewManager->getView(AGENT_VIEW_ID));
  setDash(dash);
}

MoveToPosition::MoveToPosition(const Point &position, const int dash)
  : SkillWithPosition(position),
    agentView(NULL)
{
  agentView = static_cast<AgentView*>(viewManager->getView(AGENT_VIEW_ID));
  setDash(dash);
}

// --- DESTRUCTOR ---

MoveToPosition::~MoveToPosition()
{
  viewManager->releaseView(BALL_VIEW_ID);
}


// --- METHODS ---

Plan MoveToPosition::generatePlan()
{
  static Command* cmd;
  static Plan plan;
  static PointUnknown agentPosition;
  static AngleDegUnknown toDirection, bodyDirection;
  static AngleDeg angle;
  static FloatUnknown distanceToPosition;

  if(applicable().isFalse())
    cmd = new DashCommand(0);
  else if(applicable().isTrue()){
    //setHeadDirectionToPosition();
    agentPosition = agentView->getAgentPosition(NOW);
    toDirection = agentPosition.getDirectionTo(getPosition());
    if((bodyDirection = agentView->getBodyDirection(NOW)).isKnown() &&
       ((abs(angle = toDirection.getAngle() - bodyDirection.getAngle()) > 5)))
      cmd = new TurnCommand(angle);
    else
      cmd = new DashCommand(getDash()); 
  }
  // If no command has been choosen, that is, applicable is UNKNOWN.
  if(cmd == NULL) cmd = new DashCommand(25); //better to do than not
  plan = Plan(Step(cmd));
  delete cmd;
  cmd = NULL;

  return plan;
}

// Setting new position and returning a 'Plan' to obtain that goal. 
Plan MoveToPosition::generatePlan(const Point& position){
  setPosition(position);
  return generatePlan();
}

Plan MoveToPosition::generatePlan(const int dash){
  
  setDash(dash);
  return generatePlan();
}

Plan MoveToPosition::generatePlan(const Point &position, const int dash){
  setPosition(position);
  setDash(dash);
  return generatePlan();
}

FuzzyBool MoveToPosition::persistent()
{
  // Never persistent.
  return FALSE;
}

// If the agent's position is known and the agent's distance to the wanna
// go to position is greater than 0.2, the method returns TRUE.
// If the agent's position is unknown the method returns UNKNOWN,
// else FALSE (meaning the agent's distance to the wanna go to position is 
// less than 0.2).
FuzzyBool MoveToPosition::applicable()
{
  FuzzyBool  applicable;
  if((agentView->getAgentPosition(NOW)).isKnown())
    applicable = (((agentView->getAgentPosition(NOW)).getDistanceTo(getPosition())).getValue() > 0.2) ?
      TRUE : FALSE;
  else
    applicable = UNKNOWN;

  return applicable;
}

/*
void MoveToPosition::setHeadDirectionToPosition(){
  static AgentObject ao;

  ao = agentView->getAgentObject(NOW);
  if(ao.getBodyDirection().isKnown() && ao.getNeckDirection().isKnown()){
    ao.setNeckDirection(ao.getRelativeDirectionTo(getPosition()));
  }
}
*/

void MoveToPosition::setDash(int d){
  dash = d;
}

int MoveToPosition::getDash(){
  return dash;
}

RS_END_NAMESPACE



