/* -*- Mode: C++ -*-
 * $Id: MoveToPosition.h,v 1.1 2000/11/16 16:36:20 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: MoveToPosition.h
 * Unitname: Framework/skill
 * $Revision: 1.1 $
 * Created by: Jan Axelson and Christer Rosenquist 2000-09-24
 * Last modified by $Author: frehe $ $Date: 2000/11/16 16:36:20 $
 *
 *
 * HISTORY
 *
 * $Log: MoveToPosition.h,v $
 * Revision 1.1  2000/11/16 16:36:20  frehe
 * Fixed an inconsistency in MoveToPosition.
 *
 * Revision 1.5  2000/11/15 17:02:06  chrro564
 * *** empty log message ***
 *
 * Revision 1.4  2000/10/11 19:09:57  chrro564
 * *** empty log message ***
 *
 * Revision 1.3  2000/10/09 22:13:02  chrro564
 * Updated MoveToPositioin skill.
 *
 * Revision 1.2  2000/10/07 14:28:24  olalu526
 * Added some skill files and fixed things.
 *
 * Revision 1.1  2000/09/25 11:51:28  chrro564
 * Added the MoveToPosition skill.
 *
 *
 */

/**
 * @file MoveToPosition.h
 * This skill takes the agent to a position.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/11/16 16:36:20 $
 */


#ifndef MOVETOPOSITION_H
#define MOVETOPOSITION_H

#include "machine_dependent_compiler_directives.h"
#include "SkillWithPosition.h"
#include "AgentView.h"

//#include "library_compiler_directives.h"
//#include "basic_system_compiler_directives.h"
//#include "framework_compiler_directives.h"
//#include "TheType.h"
#include "MoveToPositionFacingBall.h"

RS_BEGIN_NAMESPACE

class BallView;

/**
 *
 * @class MoveToPosition MoveToPosition.h
 * This skill takes the agent to a position.
 *
 *
 * @pre The agent's position should be known and the centre of the agent should
 *      not be closer than 0.2 meter from the position to move to, in the later
 *      case the agent is considered to be at the desired position.
 *
 * @post The agent's distance to position is =< 'distance before to position'
 *
 * @todo Things that should be done...
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/11/16 16:36:20 $
 */

class MoveToPosition : public SkillWithPosition
{
public:

  MoveToPosition();
  MoveToPosition(const Point &pos);

  MoveToPosition(const int dash);
  MoveToPosition(const Point &pos, const int dash);
  virtual ~MoveToPosition();

  virtual Plan generatePlan();

  // Set new position and return a 'Plan' to obtain that goal.
  virtual Plan generatePlan(const Point &position);

  // Set new dash and return a 'Plan' to obtain that goal.
  virtual Plan generatePlan(const int dash);

  // Set new position and new dash. Return a 'Plan'.
  virtual Plan generatePlan(const Point &position, const int dash);

  virtual FuzzyBool persistent();

  // If the agent's position is known and the agent's distance to the wanna
  // go to position is greater than 0.2, the method returns TRUE.
  // If the agent's position is unknown the method returns UNKNOWN,
  // else FALSE (meaning the agent's distance to the wanna go to position is 
  // less than 0.2).
  virtual FuzzyBool applicable();
  virtual void setDash(int d);
  virtual int getDash();

protected:

  ParameterView* parameterView;
  AgentView* agentView;
  int dash;

  /** Hide copy-constructor. */
  //MoveToPosition(const MoveToPosition&);
  /** Hide assignment operator. */
  // MoveToPosition& operator= (const MoveToPosition&);
};

RS_END_NAMESPACE

#endif // !MOVETOPOSITION_H



