/* -*- Mode: C++ -*-
 * $Id: MoveToPositionAvoidingObstacles.h,v 1.4 2000/10/11 19:09:58 chrro564 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: MoveToPositionAvoidingObstacles.h
 * Unitname: Framework/skill
 * $Revision: 1.4 $
 * Created by: Jan Axelson and Christer Rosenquist 2000-09-24
 * Last modified by $Author: chrro564 $ $Date: 2000/10/11 19:09:58 $
 *
 *
 * HISTORY
 *
 * $Log: MoveToPositionAvoidingObstacles.h,v $
 * Revision 1.4  2000/10/11 19:09:58  chrro564
 * *** empty log message ***
 *
 * Revision 1.3  2000/10/10 08:58:40  chrro564
 * *** empty log message ***
 *
 * Revision 1.2  2000/10/09 22:13:03  chrro564
 * Updated MoveToPositioin skill.
 *
 * Revision 1.1  2000/09/27 16:31:14  chrro564
 * Added the MoveToPositionAvoidingObstacles skill.
 *
 * Revision 1.1  2000/09/07 16:10:34  frehe
 * Added stubs for the skills.
 *
 */

/**
 * @file MoveToPositionAvoidingObstacles.h
 * This is a stub for skills that can be used as the basis for a skill.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.4 $
 * $Author: chrro564 $
 * $Date: 2000/10/11 19:09:58 $
 */


#ifndef MOVETOPOSITIONAVOIDINGOBSTACLES_H
#define MOVETOPOSITIONAVOIDINGOBSTACLES_H

#include "machine_dependent_compiler_directives.h"
#include "MoveToPosition.h"
#include "AgentView.h"
#include "PlayerView.h"
#include "PlayerObject.h"

//#include "library_compiler_directives.h"
//#include "basic_system_compiler_directives.h"
//#include "framework_compiler_directives.h"
//#include "TheType.h"

RS_BEGIN_NAMESPACE

class PlayerView;

/**
 * @class MoveToPositionAvoidingObstacles MoveToPositionAvoidingObstacles.h
 *        Takes the agent to a position avoiding obstacles.
 * 
 * The agent's strategy is a greedy one. She First gets hold of the closest
 * player object, then checks if it's in a corridor, wich is located betwen
 * the agent and the position she is heading for. The length of the corridor
 * is max 3 meters or the distace betwen the agent and the position. The width
 * of the corridor is 2 times the radius of the agent. If an obstacle is 
 * detected in the corridor, the agent chooses the shortest path around the
 * obstacle.
 * In the case where it's more than one obstacle to pass (hopefully enemy agents, 
 * and if not, one might consider ones own team mates strategy...),
 * it wouldn't, perhaps,  be such a bad idea if the agent didn't pass the
 * 'obstacles' so quickly. 
 * 
 *
 * @pre The agent's position should be known and the centre of the agent should
 *      not be closer than 0.2 meter from the position to move to, in the later
 *      case the agent is considered to be at the desired position.
 *
 * @post The agent returns a plan to get closer to the position.
 *
 * @todo 
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.4 $
 * $Author: chrro564 $
 * $Date: 2000/10/11 19:09:58 $
 */

class MoveToPositionAvoidingObstacles : public MoveToPosition
{
public:

  MoveToPositionAvoidingObstacles();
  MoveToPositionAvoidingObstacles(const Point &pos);
  MoveToPositionAvoidingObstacles(const int dash);
  MoveToPositionAvoidingObstacles(const Point &pos, const int dash);
  virtual ~MoveToPositionAvoidingObstacles();

  virtual Plan generatePlan();

  /* This should not be needed, but it doesn't without them... */
  // Set new position and return a 'Plan' to obtain that goal.
  virtual Plan generatePlan(const Point &position);

  // Set new dash and return a 'Plan' to obtain that goal.
  virtual Plan generatePlan(const int dash);

  // Set new position and new dash. Return a 'Plan'.
  virtual Plan generatePlan(const Point &position, const int dash);

protected:

  // Determinates if the obstacle is in a 'corridor'.
  // The corridor is 2 * (the agent radie) wide and the distance between the
  //  agent and either the distance to the goal position long,
  //  or at max 3 meters long.
  //  The corridor is situated between the agent and the goal position.
  bool inTheCorridor(PlayerObject* obstacle);

  // Determine if the obstacle is to left of a straight line between
  // the agent and the goal position.
  bool toLeft(PlayerObject* obstacle);

  // The created point is located 2 times the radius of the agent + MARGINAL beside
  // the obstacle and at the same distance from the agent that the obstacle is. 
  // Wich side of the obstacle the created point is situated at, is determined by
  // the closest way to the wanna go to position.
  Point createPoint(PlayerObject* obstacle);

  // Returns a 'Plan' to get closer to the toPosition.
  Plan goToXY(Point toPosition);

  PlayerView* playerView;

  /** Description of this attribute. */

  /** Hide copy-constructor. */
  //MoveToPositionAvoidingObstacles(const MoveToPositionAvoidingObstacles&);
  /** Hide assignment operator. */
  // MoveToPositionAvoidingObstacles& operator= (const MoveToPositionAvoidingObstacles&);
};

RS_END_NAMESPACE
 
#endif // !MOVETOPOSITIONAVOIDINGOBSTACLES_H
