/* -*- Mode: C++ -*-
 * $Id: MoveToPositionFacingBall.cc,v 1.9 2001/06/19 14:01:44 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: MoveToPositionFacingBall.cc
 * Unitname: Framework/skill
 * $Revision: 1.9 $
 * Created by: Ola Lundqvist 2000-09-29
 * Last modified by $Author: frehe $ $Date: 2001/06/19 14:01:44 $
 *
 *
 * HISTORY
 *
 */

/**
 * @file MoveToPositionFacingBall.cc
 *  
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.9 $
 * $Author: frehe $
 * $Date: 2001/06/19 14:01:44 $
 */


#include "MoveToPositionFacingBall.h"

RS_BEGIN_NAMESPACE


// --- CONSTRUCTORS ---

MoveToPositionFacingBall::MoveToPositionFacingBall()
  : MoveToPositionFacingDirectionAvoidingObstacles(Point(Float(0),Float(0))),
    ballView(NULL)
{
  getView(ballView, BALL_VIEW_ID);
}

MoveToPositionFacingBall::MoveToPositionFacingBall(const Point &position)
  : MoveToPositionFacingDirectionAvoidingObstacles(position),
    ballView(NULL)
{
  getView(ballView, BALL_VIEW_ID);
}


// --- DESTRUCTOR ---

MoveToPositionFacingBall::~MoveToPositionFacingBall()
{
  viewManager->releaseView(BALL_VIEW_ID);
}


// --- METHODS ---


Plan MoveToPositionFacingBall::generatePlan()
{
  PointUCO ap = agentView->getAgentPosition(NOW);
  if (ap.isKnown()) {
    PointUCO p = ballView->getBallPosition(NOW);
    if (p.isKnown()) {
      /*      std::cout << "\n-------\nMoveToFa: " << ap.getDirectionTo(p.getPoint()).getDeg()
	      << std::endl << std::flush;*/
      return MoveToPositionFacingDirectionAvoidingObstacles::generatePlan(ap.getDirectionTo(p.getPoint()).getDeg());
    }
    /*    std::cout << "\n----\nMoveToFa: ballunknown!" << endl << std::flush;*/
    return MoveToPositionFacingDirectionAvoidingObstacles::generatePlan();
  }
  std::cout << "MoveToPositionFacingBall: This should NEVER happen!" << std::flush;
  return Plan();
}

// Setting new position and returning a 'Plan' to obtain that goal. 
Plan MoveToPositionFacingBall::generatePlan(const Point& position){
  setPosition(position);
  return MoveToPositionFacingBall::generatePlan();
}


RS_END_NAMESPACE



