/* -*- Mode: C++ -*-
 * $Id: MoveToPositionFacingBall.h,v 1.8 2000/10/11 10:39:14 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: MoveToPositionFacingBall.h
 * Unitname: Framework/skill
 * $Revision: 1.8 $
 * Created by: Ola Lundqvist 2000-09-29
 * Last modified by $Author: olalu526 $ $Date: 2000/10/11 10:39:14 $
 *
 *
 * HISTORY
 *
 * $Log: MoveToPositionFacingBall.h,v $
 * Revision 1.8  2000/10/11 10:39:14  olalu526
 * Final fixes.
 *
 * Revision 1.7  2000/10/10 19:10:54  olalu526
 * Added documentation.
 *
 * Revision 1.6  2000/10/09 11:15:00  olalu526
 * Added ballView (that really should be there).
 *
 * Revision 1.5  2000/10/09 11:07:48  olalu526
 * Made so that GotoBall inherits from MoveToPositionFacingBall.
 *
 * Revision 1.4  2000/10/07 18:02:33  perpe427
 * Restructured inheritage for MoveToPositionFacing*-skills
 *
 * Revision 1.3  2000/10/07 14:28:24  olalu526
 * Added some skill files and fixed things.
 *
 * Revision 1.2  2000/10/01 13:05:55  olalu526
 * Updated FacingDirection, fixing bugs and stuff.
 *
 * Revision 1.1  2000/09/29 21:58:00  olalu526
 * Created MoveToPositionFacingBall and let all zone-dependent skills use it, and to make this work, movetopositionfacing_x had to be fixed. Not perfect but almost.
 *
 *
 */

/**
 * @file MoveToPositionFacingBall.h
 * This skill takes the agent to a position.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.8 $
 * $Author: olalu526 $
 * $Date: 2000/10/11 10:39:14 $
 */


#ifndef MOVETOPOSITIONFACINGBALL_H
#define MOVETOPOSITIONFACINGBALL_H

#include "machine_dependent_compiler_directives.h"
#include "view_constants.h"
#include "SkillWithPosition.h"
#include "BallView.h"
#include "ParameterView.h"
#include "AgentView.h"
#include "MoveToPositionFacingDirectionAvoidingObstacles.h"

RS_BEGIN_NAMESPACE

class BallView;

/**
 * @class MoveToPositionFacingBall MoveToPositionFacingBall.h
 * This skill takes the agent to a position.
 *
 * To instanciate this class use one argument in the constructor. This
 * argument should be of the type Position.
 * To change the position the agent is heading to, use: object.setPosition().
 * Or use o.generatePlan(with argumen of type Position).
 * The default dash is 50. To change this do: o.setDash(prefered dash).
 *
 *
 * @pre See MoveToPositionFacingDirection and the ballposition has to be known.
 * @post The agent's position is at the point specified.
 *
 * @todo Nothing.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.8 $
 * $Author: olalu526 $
 * $Date: 2000/10/11 10:39:14 $
 */

class MoveToPositionFacingBall : public MoveToPositionFacingDirectionAvoidingObstacles
{
public:
  /** Default constructor. */
  MoveToPositionFacingBall();
  /** Constructor that sets the position too. */
  MoveToPositionFacingBall(const Point &pos);
  /** Cleans up after skill. */
  virtual ~MoveToPositionFacingBall();

  /** Gnerates the plan that moves the agent closer to the position. */
  virtual Plan generatePlan();
  /** Generates a plan efter setting the position to move to. */
  virtual Plan generatePlan(const Point& position);

protected:

  /** Hide copy-constructor. */
  //MoveToPositionFacingBall(const MoveToPositionFacingBall&);
  /** Hide assignment operator. */
  // MoveToPositionFacingBall& operator= (const MoveToPositionFacingBall&);
  BallView *ballView;
};

RS_END_NAMESPACE

#endif // !MOVETOPOSITIONFACINGBALL_H



