/* -*- Mode: C++ -*-
 * $Id: MoveToPositionFacingDirection.cc,v 1.21 2000/11/07 17:41:03 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: MoveToPositionFacingDirection.cc
 * Unitname: Framework/skill
 * $Revision: 1.21 $
 * Created by: Ola Lundqvist 2000-09-27
 * Last modified by $Author: olalu526 $ $Date: 2000/11/07 17:41:03 $
 *
 *
 * HISTORY
 *
 * $Log: MoveToPositionFacingDirection.cc,v $
 * Revision 1.21  2000/11/07 17:41:03  olalu526
 * Removed some annoying log lines.
 *
 * Revision 1.20  2000/10/26 07:12:03  perpe427
 * Commented out some debug-msgs
 *
 * Revision 1.19  2000/10/09 21:00:33  olalu526
 * Fixed merge.
 *
 * Revision 1.18  2000/10/09 19:58:33  perpe427
 * Added calcSpeed() and implemented estimatedCycles() and reachable()
 *
 * Revision 1.17  2000/10/09 12:19:51  olalu526
 * Changed the default value for inPointRange.
 *
 * Revision 1.16  2000/10/09 11:59:49  olalu526
 * Fixed applicable references (so that it points to the one that it inherits from.
 *
 * Revision 1.15  2000/10/09 09:10:15  olalu526
 * Fixed so that setPosition do NOT call itself.
 *
 * Revision 1.14  2000/10/09 09:04:39  olalu526
 * Added a ifdef, endif on cout.
 *
 * Revision 1.13  2000/10/08 21:50:23  olalu526
 * Structurated and make sure that the right generatePlan is used.
 *
 * Revision 1.12  2000/10/08 10:15:19  perpe427
 * SegFault-fix number 3.. now done the correct way rather than a hack to get things working
 *
 * Revision 1.11  2000/10/08 10:07:37  perpe427
 * SegFault-fix number 2.. now the skill seems to work.. but is not extensively tested
 *
 * Revision 1.10  2000/10/07 23:54:28  perpe427
 * Fixed segfault in MoveToPositionFacingDirection
 *
 * Revision 1.9  2000/10/07 19:41:35  olalu526
 * Fixed some minor issues.
 *
 * Revision 1.8  2000/10/07 18:02:34  perpe427
 * Restructured inheritage for MoveToPositionFacing*-skills
 *
 * Revision 1.7  2000/10/07 14:28:25  olalu526
 * Added some skill files and fixed things.
 *
 * Revision 1.6  2000/10/01 13:05:56  olalu526
 * Updated FacingDirection, fixing bugs and stuff.
 *
 * Revision 1.5  2000/09/30 15:35:16  olalu526
 * Bug fixed MoveToPositionFacingDirection, now works really fine.
 *
 * Revision 1.4  2000/09/29 21:58:00  olalu526
 * Created MoveToPositionFacingBall and let all zone-dependent skills use it, and to make this work, movetopositionfacing_x had to be fixed. Not perfect but almost.
 *
 * Revision 1.3  2000/09/29 18:20:00  olalu526
 * Changed the applicable function in MoveToPositionFacingDirection.
 *
 * Revision 1.2  2000/09/29 18:02:31  olalu526
 * Added some more applicable functions with new arguments.
 *
 * Revision 1.1  2000/09/28 13:32:44  olalu526
 * A lot of changes.
 *
 *
 */

/**
 * @file MoveToPositionFacingDirection.cc
 * Implementation file for the skill class MoveToPositionFacingDirection.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.21 $
 * $Author: olalu526 $
 * $Date: 2000/11/07 17:41:03 $
 */


#include "MoveToPositionFacingDirection.h"
#define INPR 0.00

RS_BEGIN_NAMESPACE


// --- CONSTRUCTORS ---


MoveToPositionFacingDirection::MoveToPositionFacingDirection()
  : DashToSpeedVectorFacingDirection(),
    position(Float(0), Float(0)),
    minStaminaWhenFinished(lowerEffortLimit()),
    emergencyMinStaminaWhenFinished(looseRecoverLimit()),
    inPointRange(Float(INPR))
{
}


MoveToPositionFacingDirection::MoveToPositionFacingDirection(const Point &p)
  : DashToSpeedVectorFacingDirection(),
    position(p),
    minStaminaWhenFinished(lowerEffortLimit()),
    emergencyMinStaminaWhenFinished(looseRecoverLimit()),
    inPointRange(Float(INPR))
{
}


MoveToPositionFacingDirection::MoveToPositionFacingDirection(const AngleDeg &a)
  : DashToSpeedVectorFacingDirection(a),
    position(Float(0), Float(0)),
    minStaminaWhenFinished(lowerEffortLimit()),
    emergencyMinStaminaWhenFinished(looseRecoverLimit()),
    inPointRange(Float(INPR))
{
}


MoveToPositionFacingDirection::MoveToPositionFacingDirection(const Point &p, const AngleDeg &a)
  : DashToSpeedVectorFacingDirection(a),
    position(p),
    minStaminaWhenFinished(lowerEffortLimit()),
    emergencyMinStaminaWhenFinished(looseRecoverLimit()),
    inPointRange(Float(INPR))
{
}


// --- DESTRUCTOR ---

MoveToPositionFacingDirection::~MoveToPositionFacingDirection()
{
}


// --- METHODS ---

int MoveToPositionFacingDirection::estimatedCycles() const {
  /*  if (! agentView->getAgentPosition(NOW).isKnown()) {
    return -1;
  }
  int t = position.getDistanceTo(agentView->getAgentPosition(NOW).getPointOf();
  */
  if ((agentView->getAgentPosition(NOW)).isKnown()) {
    Float distance = (agentView->getAgentPosition(NOW)).getPoint().getDistanceTo(position);
    return int(distance/calcSpeed() + 0.999);
  } else {
    return 50; // A guess as good as any :) -- should maybe return IntUnknown
  }
}

void MoveToPositionFacingDirection::setArguments(const std::vector<Float>& arguments)
  throw(TooFewArgumentsException)
{
  if (arguments.size() < 2) {
    RS_THROW(TooFewArgumentsException,
	     "PredicateWithZone got less than three arguments.");
  }
  else {
    setPosition(Point(arguments[0], arguments[1]));
    setFaceAngle(AngleDeg(arguments[2]));
  }
}

void MoveToPositionFacingDirection::setPosition(const Point p) {
  position = p;
  updateVector();
}

Point
MoveToPositionFacingDirection::getPosition() const {
  return position;
}

FuzzyBool MoveToPositionFacingDirection::willBack() {
  updateVector();
  return DashToSpeedVectorFacingDirection::willBack();
}


FuzzyBool MoveToPositionFacingDirection::willBack(const Point& p) {
  setPosition(p);
  return willBack();
}


FuzzyBool MoveToPositionFacingDirection::willBack(const AngleDeg& a) {
  setFaceAngle(a);
  return willBack();
}


FuzzyBool MoveToPositionFacingDirection::willBack(const Point& p,
						  const AngleDeg& a) {
  setPosition(p);
  setFaceAngle(a);
  return willBack();
}


Plan MoveToPositionFacingDirection::generatePlan(const Point p) {
  setPosition(p);
  return MoveToPositionFacingDirection::generatePlan();
}


Plan MoveToPositionFacingDirection::generatePlan(const AngleDeg a) {
  setFaceAngle(a);
  return MoveToPositionFacingDirection::generatePlan();
}


Plan MoveToPositionFacingDirection::generatePlan(const Point p, const AngleDeg a) {
  setPosition(p);
  setFaceAngle(a);
  return MoveToPositionFacingDirection::generatePlan();
}


Plan MoveToPositionFacingDirection::generatePlan()
{
  updateVector();
  return DashToSpeedVectorFacingDirection::generatePlan();
}


FuzzyBool MoveToPositionFacingDirection::persistent()
{
  // Never persistent.
  return FALSE;
}

FuzzyBool MoveToPositionFacingDirection::applicable()
{
  //cerr << "applicable: ";
  // Almost always applicable.
  if ((agentView->getAgentPosition(NOW)).isKnown() &&
      (agentView->getBodyDirection(NOW)).isKnown() &&
      (agentView->getNeckDirection(NOW)).isKnown() ) {
    if (abs(position.getDistanceTo(agentView->getAgentPosition(NOW).getPoint())) > inPointRange) {
      //cerr << "true" << endl;
      return TRUE;
    }
  }
  //cerr << "false" << endl;
  return FALSE;
}

FuzzyBool MoveToPositionFacingDirection::applicable(const Point p) {
  setPosition(p);
  return MoveToPositionFacingDirection::applicable();
}


FuzzyBool MoveToPositionFacingDirection::applicable(const AngleDeg a) {
  setFaceAngle(a);
  return MoveToPositionFacingDirection::applicable();
}


FuzzyBool MoveToPositionFacingDirection::applicable(const Point p, const AngleDeg a) {
  setPosition(p);
  setFaceAngle(a);
  return MoveToPositionFacingDirection::applicable();
}


void MoveToPositionFacingDirection::setInPointRange(Float r) {
  inPointRange = r;
}


Float MoveToPositionFacingDirection::getInPointRange() {
  return inPointRange;
}


void MoveToPositionFacingDirection::updateVector() {
//  cerr << "updateVector(): " << endl;
  if ((agentView->getAgentPosition(NOW)).isKnown()) {
    Point ap = (agentView->getAgentPosition(NOW)).getPoint();
    Vector v = ap.getVectorTo(position);
    if (v.getLength() > Float(1)) {
      //v.setLength(getSpeedVector().getLength());
      v.setLength(calcSpeed());
    }
    //cout << "MoveBDir: " << v.getAngle().getDeg() << endl << flush;*/
    setSpeedVector(v);
    //    cout << "speedvector:" << getSpeedVector().getLength() << endl;
  }
}

Float 
MoveToPositionFacingDirection::calcSpeed() const
{
  //  cout << "calcSpeed: ";
  if ((agentView->getAgentPosition(NOW)).isKnown()) {
    Float distance = (agentView->getAgentPosition(NOW)).getPoint().getDistanceTo(position);
    return calcSpeed(distance);
  } else {
    return Float(0.0);
  }  
}


Float 
MoveToPositionFacingDirection::calcSpeed(Float distance) const
{
  //float v0;  // Should be used for calculations
  Float minStamina = getEmergency() ? emergencyMinStaminaWhenFinished : minStaminaWhenFinished;
  Float stamina = agentView->getStamina(NOW).isKnown() ?
    agentView->getStamina(NOW).getValue().getValue().getValue() - minStamina :
    parameterView->SP_stamina_max - minStamina;
  static Float power_rate = parameterView->SP_dash_power_rate;
  static Float decay = parameterView->SP_player_decay;
  static Float max_speed = parameterView->SP_player_speed_max;
  static Float max_power = parameterView->SP_maxpower;
  static Float stamina_inc_max = parameterView->SP_stamina_inc_max;
  Float stamina_inc = stamina_inc_max;  // Should implement AgentView->getRecovery()
  Float v,n;
  
  if(max_power*distance/max_speed < stamina + stamina_inc*distance/max_speed) {
    v = max_speed;
    n = distance/max_speed;
  } else {
    /*
      v = acc + decay*v  =>  acc = (1-decay)/v  =>
      =>  power = (1-decay)/power_rate*distance/n  =>
      =>  stamina/n + stamina_inc = (1-decay)/power_rate*distance/n  =>
      =>  stamina + n*stamina_inc = (1-decay)/power_rate*distance  =>
      =>  n = ((1-decay)/power_rate*distance - stamina)/stamina_inc 
    */
    n = (((1.0-decay)/power_rate)*distance - stamina)/stamina_inc;
    v = distance/n;
  }
  //    cout << "d=" << distance << ", v=" << v << ", n=" << n <<
  //  ", s=" << stamina << endl;
  return v;
}
  
void 
MoveToPositionFacingDirection::setMinStaminaWhenFinished(Float s)
{
  minStaminaWhenFinished = s;
}

Float
MoveToPositionFacingDirection::getMinStaminaWhenFinished() const
{
  return minStaminaWhenFinished;
}

void
MoveToPositionFacingDirection::setMaxStaminaUsedWhenFinished(Float s)
{
  if(agentView->getStamina(NOW).isKnown()) {
    setMinStaminaWhenFinished(
       agentView->getStamina(NOW).getValue().getValue().getValue() - s);
  } else {
    setMinStaminaWhenFinished(parameterView->SP_stamina_max - s);    
  }
}

Float
MoveToPositionFacingDirection::getMaxStaminaUsedWhenFinished() const
{
  if(agentView->getStamina(NOW).isKnown()) {
    return agentView->getStamina(NOW).getValue().getValue().getValue() 
      - getMinStaminaWhenFinished();
  } else {
    return parameterView->SP_stamina_max - getMinStaminaWhenFinished();
  }
}

void 
MoveToPositionFacingDirection::setEmergencyMinStaminaWhenFinished(Float s)
{
  emergencyMinStaminaWhenFinished = s;
}

Float
MoveToPositionFacingDirection::getEmergencyMinStaminaWhenFinished() const
{
  return emergencyMinStaminaWhenFinished;
}

void
MoveToPositionFacingDirection::setEmergencyMaxStaminaUsedWhenFinished(Float s)
{
  if(agentView->getStamina(NOW).isKnown()) {
    setEmergencyMinStaminaWhenFinished(
       agentView->getStamina(NOW).getValue().getValue().getValue() - s);
  } else {
    setEmergencyMinStaminaWhenFinished(parameterView->SP_stamina_max - s);    
  }
}

Float
MoveToPositionFacingDirection::getEmergencyMaxStaminaUsedWhenFinished() const
{
  if(agentView->getStamina(NOW).isKnown()) {
    return agentView->getStamina(NOW).getValue().getValue().getValue() 
      - getEmergencyMinStaminaWhenFinished();
  } else {
    return parameterView->SP_stamina_max - getEmergencyMinStaminaWhenFinished();
  }
}


FuzzyBool
MoveToPositionFacingDirection::reachable(UInt n)
{
  return reachable(position,n);
}

FuzzyBool
MoveToPositionFacingDirection::reachable(Point p, UInt n)
{
  setPosition(p);
  if ((agentView->getAgentPosition(NOW)).isKnown()) {
    Float distance = (agentView->getAgentPosition(NOW)).getPoint().getDistanceTo(position);
    return FuzzyBool(distance < calcSpeed(distance)*n);
  } else {
    return FALSE;
  }
}

RS_END_NAMESPACE


// Remove from here
/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "MoveToPositionFacingDirection.icc"
#endif
// To here if you do not have any inline definitions!
