/* -*- Mode: C++ -*-
 * $Id: MoveToPositionFacingDirection.h,v 1.12 2000/10/10 20:38:24 perpe427 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: MoveToPositionFacingDirection.h
 * Unitname: Framework/skill
 * $Revision: 1.12 $
 * Created by: Ola Lundqvist 2000-09-27
 * Last modified by $Author: perpe427 $ $Date: 2000/10/10 20:38:24 $
 *
 *
 * HISTORY
 *
 * $Log: MoveToPositionFacingDirection.h,v $
 * Revision 1.12  2000/10/10 20:38:24  perpe427
 * Edited some documentation
 *
 * Revision 1.11  2000/10/10 19:10:54  olalu526
 * Added documentation.
 *
 * Revision 1.10  2000/10/10 10:11:36  olalu526
 * Added documentation.
 *
 * Revision 1.9  2000/10/09 19:58:34  perpe427
 * Added calcSpeed() and implemented estimatedCycles() and reachable()
 *
 * Revision 1.8  2000/10/07 18:02:34  perpe427
 * Restructured inheritage for MoveToPositionFacing*-skills
 *
 * Revision 1.7  2000/10/07 14:28:25  olalu526
 * Added some skill files and fixed things.
 *
 * Revision 1.6  2000/10/01 13:05:56  olalu526
 * Updated FacingDirection, fixing bugs and stuff.
 *
 * Revision 1.5  2000/09/30 15:35:16  olalu526
 * Bug fixed MoveToPositionFacingDirection, now works really fine.
 *
 * Revision 1.4  2000/09/29 21:58:00  olalu526
 * Created MoveToPositionFacingBall and let all zone-dependent skills use it, and to make this work, movetopositionfacing_x had to be fixed. Not perfect but almost.
 *
 * Revision 1.3  2000/09/29 18:20:00  olalu526
 * Changed the applicable function in MoveToPositionFacingDirection.
 *
 * Revision 1.1  2000/09/28 13:32:44  olalu526
 * A lot of changes.
 *
 *
 */

/**
 * @file MoveToPositionFacingDirection.h
 * Include file for the skill class GotoBall.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.12 $
 * $Author: perpe427 $
 * $Date: 2000/10/10 20:38:24 $
 */


#ifndef MOVETOPOSITIONFACINGDIRECTION_H
#define MOVETOPOSITIONFACINGDIRECTION_H


#include "view_constants.h"
#include "ParameterView.h"
#include "AgentView.h"
#include "DashToSpeedVectorFacingDirection.h"

RS_BEGIN_NAMESPACE


/**
 * @class MoveToPositionFacingDirection MoveToPositionFacingDirection.h
 * This skill makes the agent move to a position while facing an other one.
 * If the angle between the body and the face is too high the agent
 * will turn around and back. This is not very efficient so you can use the
 * willBack() method to check if that will happen or not.
 *x
 * @pre The agent should know its own face and body direction and position.
 * @post The agent is on the position.
 *
 * @todo Implement it:
 *	- Write willBack.
 *	- Start using WithArguments.
 *	- Write the standard methods.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.12 $
 * $Author: perpe427 $
 * $Date: 2000/10/10 20:38:24 $
 */
class MoveToPositionFacingDirection : public DashToSpeedVectorFacingDirection {
public:
  /** Default constructor. Here you can initialize the skill. */
  MoveToPositionFacingDirection();
  /** Constructor that sets the point to move to. */
  MoveToPositionFacingDirection(const Point&);
  /** Constructor that sets the face direction. */
  MoveToPositionFacingDirection(const AngleDeg&);
  /** Constructor that sets the point to move to, and the face direction. */
  MoveToPositionFacingDirection(const Point&, const AngleDeg&);

  /** Destructor should clean up after the skill. */
  virtual ~MoveToPositionFacingDirection();

  /** Sets the position to move to. */
  virtual void setPosition(const Point);
  /** Gets the position to move to. */
  virtual Point getPosition() const;

  /** This method is used to generate a plan for how the goal of
    the skill should be obtained by the agent. You can assume
    that the preconditions of the skill are true (this can be tested
    by calling the applicable() method, after you have implemented it). */
  virtual Plan generatePlan();
  /** Planner that first sets the point to move to. */
  virtual Plan generatePlan(const Point);
  /** Planner that first sets the face direction. */
  virtual Plan generatePlan(const AngleDeg);
  /** Planner that first sets the point to move to and the face direction. */
  virtual Plan generatePlan(const Point, const AngleDeg);
  /** Returns true if the angle is such that the agent have to back. */
  virtual FuzzyBool willBack();
  /** First sets the position to move to and then returns willback. */
  virtual FuzzyBool willBack(const Point&);
  /** First sets the face angle and then returns willback. */
  virtual FuzzyBool willBack(const AngleDeg&);
  /** First sets both the position to move to and the angle to face and then
      return willback. */
  virtual FuzzyBool willBack(const Point&, const AngleDeg&);


  /** This method should return a FuzzyBool telling the caller
    if the skill wants to be called again. */
  virtual FuzzyBool persistent();

  /** This method implements the precondition specified in
   the description of the skill. */
  virtual FuzzyBool applicable();
  /** First sets the point to move to and then return applicable. */
  virtual FuzzyBool applicable(const Point);
  /** First sets the face angle and then returns applicable. */
  virtual FuzzyBool applicable(const AngleDeg);
  /** First sets both the position to move to and the angle to face and then
      return applicable. */
  virtual FuzzyBool applicable(const Point, const AngleDeg);

  /** Get the inPointRange variable. (depricated) */
  virtual Float getInPointRange();
  /** Set the inPointRange variable. (depricated) */
  virtual void setInPointRange(Float);

  /** Set the arguments in a general way. (x, y) */
  virtual void setArguments(const std::vector<Float>&)
    throw(TooFewArgumentsException);
  
  /** The estimated number of cycles it will take to go to that position. */
  virtual int estimatedCycles() const;

  /** Sets the minimum stamina when having reached the position. */
  virtual void setMinStaminaWhenFinished(Float);

  /** Gets the minimum stamina when having reached the position. */
  virtual Float getMinStaminaWhenFinished() const;

  /** Sets the maximum amount of stamina to use to reach the position. */
  virtual void setMaxStaminaUsedWhenFinished(Float);

  /** Gets the maximum amount of stamina to use to reach the position. */
  virtual Float getMaxStaminaUsedWhenFinished() const;

  /** Sets the minimum stamina when having reached the position when emergency. */
  virtual void setEmergencyMinStaminaWhenFinished(Float);

  /** Gets the minimum stamina when having reached the position when emergency. */
  virtual Float getEmergencyMinStaminaWhenFinished() const;

  /** Sets the maximum amount of stamina to use to reach the position under emergency. */
  virtual void setEmergencyMaxStaminaUsedWhenFinished(Float);

  /** Gets the maximum amount of stamina to use to reach the position under emergency. */
  virtual Float getEmergencyMaxStaminaUsedWhenFinished() const;

  /** Is the position reachable in the specified number of cycles. */
  virtual FuzzyBool reachable(UInt);

  /** Is the specified position reachable in the specified number of cycles. */
  virtual FuzzyBool reachable(Point, UInt);

  /** Calculates speed to use in order to reach the position using the
      appropriate amount of stamina depending on if emergency is set
      or not. */
  virtual Float calcSpeed() const;

  /** Calculates speed to run distance using the appropriate amount of
      stamina depending on if emergency is set or not. */
  virtual Float calcSpeed(Float distance) const;

private:
  /** The goal position. */
  Point position;

  /** The minimum stamina allowed when the agent has reached the goal. */
  Float minStaminaWhenFinished;  
  /** The minimum stamina allowed (in emergency situation) when the agent has
      reached the goal. */
  Float emergencyMinStaminaWhenFinished;  

  /** Hide copy-constructor. */
  MoveToPositionFacingDirection(const MoveToPositionFacingDirection&);
  /** Hide assignment operator. */
  MoveToPositionFacingDirection& operator= (const MoveToPositionFacingDirection&);
  /** The inPointRange. (depricated). */
  Float inPointRange;
  /** How fast the agent is allowed to run. */
  Float maxdash;

  /** The function that updates the speedvector to the dash command. */
  virtual void updateVector();
};


RS_END_NAMESPACE


// Remove from here 
/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "MoveToPositionFacingDirection.icc"
#endif
// To here if you do not have any inline definitions!

#endif // !MOVETOPOSITIONFACINGDIRECTION_H
