/* -*- Mode: C++ -*-
 * $Id: MoveToPositionFacingDirectionAvoidingObstacles.cc,v 1.4 2000/11/07 17:38:13 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Jan Axelson and
 * Christer Rosenquist.
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: MoveToPositionFacingDirectionAvoidingObstacles.cc
 * Unitname: Framework/skill
 * $Revision: 1.4 $
 * Created by: Jan Axelson and Christer Rosenquist 2000-09-24
 * Last modified by $Author: olalu526 $ $Date: 2000/11/07 17:38:13 $
 *
 *
 * HISTORY
 *
 */

/**
 * @file MoveToPositionFacingDirectionAvoidingObstacles.cc
 *  
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.4 $
 * $Author: olalu526 $
 * $Date: 2000/11/07 17:38:13 $
 */


#include "MoveToPositionFacingDirectionAvoidingObstacles.h"
#include "view_constants.h"
#include "Command.h"
#include "TurnCommand.h"
#include "DashCommand.h"
#include "Step.h"
#include "PlayerObject.h"
#include "ObjectCollection.h"
#include "OwnMath.h"

RS_BEGIN_NAMESPACE

const float MARGINAL = 2;

// --- CONSTRUCTORS ---

MoveToPositionFacingDirectionAvoidingObstacles::MoveToPositionFacingDirectionAvoidingObstacles()
  : MoveToPositionFacingDirection(),
    playerView(NULL),
    subgoto()
{
  getView(playerView, PLAYER_VIEW_ID);
}

MoveToPositionFacingDirectionAvoidingObstacles::MoveToPositionFacingDirectionAvoidingObstacles(const AngleDeg &a)
  : MoveToPositionFacingDirection(a),
    playerView(NULL),
    subgoto()
{
  getView(playerView, PLAYER_VIEW_ID);
}

MoveToPositionFacingDirectionAvoidingObstacles::MoveToPositionFacingDirectionAvoidingObstacles(const Point &position)
  : MoveToPositionFacingDirection(position),
    playerView(NULL),
    subgoto()
{
  getView(playerView, PLAYER_VIEW_ID);
}

MoveToPositionFacingDirectionAvoidingObstacles::MoveToPositionFacingDirectionAvoidingObstacles(const Point &position, const AngleDeg &a)
  : MoveToPositionFacingDirection(position, a),
    playerView(NULL),
    subgoto()
{
  getView(playerView, PLAYER_VIEW_ID);
}

// --- DESTRUCTOR ---

MoveToPositionFacingDirectionAvoidingObstacles::~MoveToPositionFacingDirectionAvoidingObstacles()
{
  viewManager->releaseView(PLAYER_VIEW_ID);
}


// --- METHODS ---

void MoveToPositionFacingDirectionAvoidingObstacles::setFaceAngle(const AngleDeg a) {
  DashToSpeedVectorFacingDirection::setFaceAngle(a);
  subgoto.setFaceAngle(a);
}

Plan MoveToPositionFacingDirectionAvoidingObstacles::generatePlan(const AngleDeg& a) {
  setFaceAngle(a);
  return MoveToPositionFacingDirectionAvoidingObstacles::generatePlan();
}

Plan MoveToPositionFacingDirectionAvoidingObstacles::generatePlan(const Point& p) {
  setPosition(p);
  return MoveToPositionFacingDirectionAvoidingObstacles::generatePlan();
}

Plan MoveToPositionFacingDirectionAvoidingObstacles::generatePlan(const Point& p, const AngleDeg& a) {
  setFaceAngle(a);
  setPosition(p);
  return MoveToPositionFacingDirectionAvoidingObstacles::generatePlan();
}

Plan MoveToPositionFacingDirectionAvoidingObstacles::generatePlan()
{
  // If obstacle between Agent and obstacle. 
  Point p = createPoint();
  if (p != getPosition()) {
    return subgoto.generatePlan(p);
  }
  // Move straight ahead to position.
  return  MoveToPositionFacingDirection::generatePlan();
}

Point MoveToPositionFacingDirectionAvoidingObstacles::createPoint(){
  static int turnedTime = 0;
  if (turnedTime > 0) {
    turnedTime--;
    return getPosition();
  }
  PointUCO uap = agentView->getAgentPosition(NOW);
  if (!uap.isKnown()) {
    //cerr << "(AP unknown)" << flush;
    turnedTime = 0;
    return getPosition();
  }
  Point ap = uap.getPoint();
  std::vector<PlayerObject> ol = playerView->getOpponents(NOW);
  std::vector<PlayerObject> tm = playerView->getTeammates(NOW);
  for (std::vector<PlayerObject>::const_iterator addo = tm.begin();
       addo != tm.end(); ++addo ) {
    ol.push_back(*addo);
  }
  std::vector<PlayerObject> up = playerView->getTeammates(NOW);
  for (std::vector<PlayerObject>::const_iterator addo = up.begin();
       addo != up.end(); ++addo ) {
    ol.push_back(*addo);
  }
  Point rPoint = getPosition();
  Point lPoint = getPosition();
  AngleDeg rA = 0;
  AngleDeg lA = 0;
  VectorUCO uv;
  // Go through objects.
  for (std::vector<PlayerObject>::const_iterator obj = ol.begin();
       obj != ol.end(); ++obj ) {
    uv = -(obj->getRelativeVectorTo(ap));
    //   cout << "(OV uv" << uv << ")" << flush;
    if (uv.isKnown()) {
      // Ser till att dr vi str blir origo.
      Vector v = uv.getVector();
      Float m = 1;
      Point op = Point(v);
      Point dp = getPosition() - ap;
      if (Vector(dp).getLength() < MARGINAL) {
	turnedTime = 2;
	return getPosition();
      }
      // Is the object too far away? Further than the goal or further than 30 m?
      if (Vector(op).getLength() < Vector(dp).getLength() &&
	  Vector(op).getLength() < 30) {
	// Can this object be in the way?
	AngleDeg tang = abs(Vector(op).getAngle() - Vector(dp).getAngle());
	if (tang < AngleDeg(90) ||
	    abs(Vector(op).getLength()) < 2*MARGINAL) {
	  for (Float i = 0; i<m; i++) {
	    Vector av = Vector(dp);
	    av.setLength(i);
	    //	    op += av;
	    //cout << "(MI corridor)" << flush;
	    
	    Point ppar = (scalarMult(op,dp)/scalarMult(dp,dp))*dp;
	    Vector vort = ppar.getVectorTo(op);
	    // is this object really in the corridor?
	    if (vort.getLength() < 1.5*MARGINAL) {
	      // So now start the real calculations!!!
	      //cout << "(IC " << ppar << vort.getLength() << " " << vort << ")" << flush;
	      Vector nov = vort;
	      nov.setLength(MARGINAL);
	      Point rP = op + nov;
	      Point lP = op - nov;
	      AngleDeg la = abs(Vector(lPoint).getAngle() - Vector(dp).getAngle());
	      AngleDeg ra = abs(Vector(rPoint).getAngle() - Vector(dp).getAngle());
	      if (ra > rA) {
		//cout << "(L " << ra << " " << rP << ")" << flush;
		rA = ra;
		rPoint = ap + rP;
	      }
	      if (la > lA) {
		//cout << "(R " << la << " " << lP << ")" << flush;
		lA = la;
		lPoint = ap + lP;
	      }
	    }
	  }
	}
      }
    }
  }
  if (rA < lA) {
    //cout << "(CR)" << flush;
    if (rPoint == getPosition()) {
      turnedTime = 2;
    }
    return rPoint;
  }
  //cout << "(CL)" << flush;
  if (lPoint == getPosition()) {
    turnedTime = 2;
  }
  return lPoint;
}

RS_END_NAMESPACE



