/* -*- Mode: C++ -*-
 * $Id: MoveToPositionFacingDirectionAvoidingObstacles.h,v 1.2 2000/11/03 23:03:34 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: MoveToPositionFacingDirectionAvoidingObstacles.h
 * Unitname: Framework/skill
 * $Revision: 1.2 $
 * Created by: Jan Axelson and Christer Rosenquist 2000-09-24
 * Last modified by $Author: olalu526 $ $Date: 2000/11/03 23:03:34 $
 *
 *
 * HISTORY
 *
 * $Log: MoveToPositionFacingDirectionAvoidingObstacles.h,v $
 * Revision 1.2  2000/11/03 23:03:34  olalu526
 * Rewrote most of it so that it will actually work now.
 *
 * Revision 1.1  2000/10/11 08:12:57  olalu526
 * Added MoveToPositionFacingDirectionAvoidingObstacles, most code if from MoveToPositionAvoidingObstacles but there are some changes to improve it.
 *
 * Revision 1.3  2000/10/10 08:58:40  chrro564
 * *** empty log message ***
 *
 * Revision 1.2  2000/10/09 22:13:03  chrro564
 * Updated MoveToPositioin skill.
 *
 * Revision 1.1  2000/09/27 16:31:14  chrro564
 * Added the MoveToPositionFacingDirectionAvoidingObstacles skill.
 *
 * Revision 1.1  2000/09/07 16:10:34  frehe
 * Added stubs for the skills.
 *
 */

/**
 * @file MoveToPositionFacingDirectionAvoidingObstacles.h
 * This is a stub for skills that can be used as the basis for a skill.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.2 $
 * $Author: olalu526 $
 * $Date: 2000/11/03 23:03:34 $
 */


#ifndef MOVETOPOSITIONFACINGDIRECTIONAVOIDINGOBSTACLES_H
#define MOVETOPOSITIONFACINGDIRECTIONAVOIDINGOBSTACLES_H

#include "machine_dependent_compiler_directives.h"
#include "MoveToPositionFacingDirection.h"
#include "AgentView.h"
#include "PlayerView.h"
#include "PlayerObject.h"

//#include "library_compiler_directives.h"
//#include "basic_system_compiler_directives.h"
//#include "framework_compiler_directives.h"
//#include "TheType.h"

RS_BEGIN_NAMESPACE

class PlayerView;

/**
 * @class MoveToPositionFacingDirectionAvoidingObstacles MoveToPositionFacingDirectionAvoidingObstacles.h
 * This code is taken from MoveToPositionAvoidingObstacles.
 * 
 * @pre The agent's position should be known and the centre of the agent should
 *      not be closer than 0.2 meter from the position to move to, in the later
 *      case the agent is considered to be at the desired position.
 *
 * @post The agent returns a plan to get closer to the position.
 *
 * @todo 
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.2 $
 * $Author: olalu526 $
 * $Date: 2000/11/03 23:03:34 $
 */

class MoveToPositionFacingDirectionAvoidingObstacles : public MoveToPositionFacingDirection
{
public:
  /** Default constructor. */
  MoveToPositionFacingDirectionAvoidingObstacles();
  /** Default constructor to set the face angle. */
  MoveToPositionFacingDirectionAvoidingObstacles(const AngleDeg &);
  /** Default constructor to set the position to go to. */
  MoveToPositionFacingDirectionAvoidingObstacles(const Point &);
  /** Default constructor to set both the postition to go to and the face angle. */
  MoveToPositionFacingDirectionAvoidingObstacles(const Point &, const AngleDeg &);

  /** Cleans up after it. */
  virtual ~MoveToPositionFacingDirectionAvoidingObstacles();

  /** Generates the plan. See MoveToPositionAvoidingObstacles. */
  virtual Plan generatePlan();
  /** First sets the face angle and then generates the plan. */
  virtual Plan generatePlan(const AngleDeg&);
  /** First sets the point to go to and then generates the plan. */
  virtual Plan generatePlan(const Point&);
  /** First sets both the position to go to and the face angle and then generates the plan. */
  virtual Plan generatePlan(const Point&, const AngleDeg&);
  /** Sets the face angle. */
  virtual void setFaceAngle(const AngleDeg);

  /** Creates a sub point. */
  virtual Point createPoint();

protected:
  /** The players view. */
  PlayerView* playerView;
  /** A sub planner. */
  MoveToPositionFacingDirection subgoto;
};




RS_END_NAMESPACE
 

#endif // !MOVETOPOSITIONFACINGDIRECTIONAVOIDINGOBSTACLES_H
