/* -*- Mode: C++ -*-
 * $Id: MoveToPositionFacingPosition.h,v 1.1 2000/11/04 12:58:50 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: MoveToPositionFacingPosition.h
 * Unitname: Framework/skill
 * $Revision: 1.1 $
 * Created by: Ola Lundqvist 2000-09-29
 * Last modified by $Author: olalu526 $ $Date: 2000/11/04 12:58:50 $
 *
 *
 * HISTORY
 *
 * $Log: MoveToPositionFacingPosition.h,v $
 * Revision 1.1  2000/11/04 12:58:50  olalu526
 * Added MoveToPositionFacingPosition.
 *
 *
 */

/**
 * @file MoveToPositionFacingPosition.h
 * This skill takes the agent to a position.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.1 $
 * $Author: olalu526 $
 * $Date: 2000/11/04 12:58:50 $
 */


#ifndef MOVETOPOSITIONFACINGPOSITION_H
#define MOVETOPOSITIONFACINGPOSITION_H

#include "machine_dependent_compiler_directives.h"
#include "view_constants.h"
#include "SkillWithPosition.h"
#include "ParameterView.h"
#include "AgentView.h"
#include "MoveToPositionFacingDirectionAvoidingObstacles.h"

RS_BEGIN_NAMESPACE

/**
 * @class MoveToPositionFacingPosition MoveToPositionFacingPosition.h
 * This skill takes the agent to a position.
 *
 * To instanciate this class use one argument in the constructor. This
 * argument should be of the type Position.
 * To change the position the agent is heading to, use: object.setPosition().
 * Or use o.generatePlan(with argumen of type Position).
 * The default dash is 50. To change this do: o.setDash(prefered dash).
 *
 *
 * @pre See MoveToPositionFacingDirection and the positionposition has to be known.
 * @post The agent's position is at the point specified.
 *
 * @todo Nothing.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.1 $
 * $Author: olalu526 $
 * $Date: 2000/11/04 12:58:50 $
 */

class MoveToPositionFacingPosition : public MoveToPositionFacingDirectionAvoidingObstacles
{
public:
  /** Default constructor. */
  MoveToPositionFacingPosition();
  /** Constructor that sets the position too. */
  MoveToPositionFacingPosition(const Point &pos);
  /** Cleans up after skill. */
  virtual ~MoveToPositionFacingPosition();

  /** Gnerates the plan that moves the agent closer to the position. */
  virtual Plan generatePlan();
  /** Generates a plan efter setting the position to move to. */
  virtual Plan generatePlan(const Point& position);

protected:
  /** Hide copy-constructor. */
  //MoveToPositionFacingPosition(const MoveToPositionFacingPosition&);
  /** Hide assignment operator. */
  // MoveToPositionFacingPosition& operator= (const MoveToPositionFacingPosition&);
};

RS_END_NAMESPACE

#endif // !MOVETOPOSITIONFACINGPOSITION_H



