

 /* -*- Mode: C++ -*-
 * $Id: MoveToPositionWithBall.cc,v 1.3 2001/01/17 15:26:04 magin846 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: MoveToPositionWithBall.cc
 * Unitname: Framework/skill
 * $Revision: 1.3 $
 * Created by: Micael Sjlund & Bjrn Tikkanen, 2000-09-19
 * Last modified by $Author: magin846 $ $Date: 2001/01/17 15:26:04 $
 *
 *
 * HISTORY
 *
 * $Log: MoveToPositionWithBall.cc,v $
 * Revision 1.3  2001/01/17 15:26:04  magin846
 * Fixed bug in applicable. (Agent could die with signal SIGABORT.)
 * Added missing tests.
 *
 * Revision 1.2  2000/10/09 09:38:19  bjoti341
 * First working version
 *
 * Revision 1.1  2000/09/27 13:36:59  bjoti341
 * Added skill MoveToPositionWithBall.
 *
 */

/**
 * @file MoveToPositionWithBall.cc
 * Implementation file for the skill class MoveToPositionWithBall
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.3 $
 * $Author: magin846 $
 * $Date: 2001/01/17 15:26:04 $
 */



#include "MoveToPositionWithBall.h"
#include "view_constants.h"
#include "ParameterView.h"
#include "MarkerView.h"
#include "AgentView.h"
#include "BallView.h"
#include "rs_log.h"
#include "KickCommand.h"


RS_BEGIN_NAMESPACE

// --- CONSTRUCTORS ---

MoveToPositionWithBall::MoveToPositionWithBall(Point p)
  : SkillWithPosition(p),
    ballView(NULL),
    agentView(NULL),
    parameterView(NULL),
    position(p)
{
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(ballView, BALL_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID, parameterView);
}


// --- DESTRUCTOR ---

MoveToPositionWithBall::~MoveToPositionWithBall()
{
  viewManager->releaseView(BALL_VIEW_ID);
  viewManager->releaseView(AGENT_VIEW_ID);
  viewManager->releaseView(PARAMETER_VIEW_ID);
}


// --- METHODS ---

void MoveToPositionWithBall::setArguments(Point p)
{
  position = (PointUCO) p;
}


Plan MoveToPositionWithBall::generatePlan()
{
  Command* cmd = NULL;
  Plan plan;
  ballPoint = ballView->getBallPosition(NOW);
  moveToPos = new MoveToPositionFacingBall(ballPoint.getPoint());
  PointUCO agentPosition;
  agentPosition = agentView->getAgentPosition(NOW);
  PlayerObject agent;
  agent = static_cast<PlayerObject>(agentView->getAgentObject(NOW));

  int kickPower = 10;
  bool kickable = ballView->isBallKickable(NOW);
  
  plan = Plan();
  
  /* are we there yet and "holding" the ball? */
  if (((agentPosition.getDistanceTo(position)) < 3) && 
      (ballPoint.getDistanceTo(position) < 3))
						  
    {
      cmd = NULL; 
      return plan = Plan(Step(cmd));
    }
  
  /* Can we kick the ball? -> kick IT! */
  else if (kickable)
      {

	/* Different kickpowers for different distances from position */
	if (agentPosition.getDistanceTo(position) < 5)
	  {
	    kickPower = 2;
	  }
	else 
	  kickPower = 4;

	AngleDeg relativeAngleToBall = agent.getRelativeDirectionTo(position).getAngle().getDeg();
	AngleDeg nonKickableAngle = 110;
	AngleDeg maxKickableAngle = 90;
	
	
	if (relativeAngleToBall > nonKickableAngle)
	  relativeAngleToBall = maxKickableAngle;
	if (relativeAngleToBall < -nonKickableAngle)
	  relativeAngleToBall = -maxKickableAngle;
		
	
	cmd = new KickCommand(kickPower, relativeAngleToBall);
	plan = Plan(Step(cmd));
      }
  
  /* otherwise -> go to the ball (or atleast to the position of the ball...) */
  else 
      plan = moveToPos->generatePlan();
  
  delete cmd;
  
  return plan;
}


FuzzyBool MoveToPositionWithBall::persistent()
{
  return FALSE;
}

FuzzyBool MoveToPositionWithBall::applicable()
{
  if (ballView->getRelativeBallDirection(NOW).isKnown() && (ballView->getBallPosition(NOW).isKnown() && agentView->getAgentPosition(NOW).isKnown()))
    return TRUE;
  else
    return FALSE;
}


RS_END_NAMESPACE

