/* -*- Mode: C++ -*-
 * $Id: MoveToPositionWithBall.h,v 1.2 2000/10/09 09:38:31 bjoti341 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: MoveToPositionWithBall.h
 * Unitname: Framework/skill
 * $Revision: 1.2 $
 * Created by: Micael Sjlund & Bjrn Tikkanen 2000/09/19
 * Last modified by $Author: bjoti341 $ $Date: 2000/10/09 09:38:31 $
 *
 *
 * HISTORY
 *
 * $Log: MoveToPositionWithBall.h,v $
 * Revision 1.2  2000/10/09 09:38:31  bjoti341
 * First working version
 *
 * Revision 1.1  2000/09/27 13:36:55  bjoti341
 * Added skill MoveToPositionWithBall.
 *
 * 2000-09-20 20.30.00 Succeeded in compiling test file!
 */

/**
 * @file MoveToPositionWithBall.h
 * 
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.2 $
 * $Author: bjoti341 $
 * $Date: 2000/10/09 09:38:31 $
 */


#ifndef MoveToPositionWithBall_H
#define MoveToPositionWithBall_H


// Include the system header which definitions you use.
//#include <...>
//#include <...>

// Include the header files which definitions you use.
#include "MoveToPositionFacingBall.h"
#include "machine_dependent_compiler_directives.h"
#include "SkillWithPosition.h"
//#include "library_compiler_directives.h"
//#include "basic_system_compiler_directives.h"
//#include "framework_compiler_directives.h"
//#include "TheType.h"

RS_BEGIN_NAMESPACE
class BallView;
class AgentView;

/**
 * @class MoveToPositionWithBall MoveToPositionWithBall.h
 * The agent runs with the ball to a specified position.
 * It uses the skill MoveToPositionFacingBall and a regular kick command. It kicks
 * hard when it's far from the position, and not so hard when closer.
 *
 * @pre The Relative Ball Direction is known
 * @post The agent has reached the wanted position with the ball close to him.
 *
 * @todo Kick the ball towards the position. Run to the ball. Repeat until postcondition fulfilled.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.2 $
 * $Author: bjoti341 $
 * $Date: 2000/10/09 09:38:31 $
 */
class MoveToPositionWithBall : public SkillWithPosition {
public:
  /** Default constructor. Here you can initialize the skill. */
  MoveToPositionWithBall(Point p);

  /** Destructor should clean up after the skill. */
  virtual ~MoveToPositionWithBall();

  void setArguments(Point p);
  
  /** This method is used to generate a plan for how the goal of
    the skill should be obtained by the agent. You can assume
    that the preconditions of the skill are true (this can be tested
    by calling the applicable() method, after you have implemented it). */
  virtual Plan generatePlan();

  /** This method should return a FuzzyBool telling the caller
    if the skill wants to be called again. */
  virtual FuzzyBool persistent();

  /** This method should implement the precondition specified in
   the description of the skill. */
  virtual FuzzyBool applicable();


private:
  /** Description of this attribute. */
  // TheType attributeName;

  
  /** Hide copy-constructor. */
  MoveToPositionWithBall(const MoveToPositionWithBall&);
  /** Hide assignment operator. */
  MoveToPositionWithBall& operator= (const MoveToPositionWithBall&);

protected:

  PointUnknown ballPoint;
  MoveToPositionFacingBall* moveToPos;
  BallView* ballView;
  AgentView* agentView;
  ParameterView* parameterView;
  PointUCO position;
 };


RS_END_NAMESPACE


#endif // !MoveToPositionWithBall_H
