/* -*- C++ -*- */
   
#ifndef PASS_H
#define PASS_H


// Include the system header which definitions you use.
//#include <...>
//#include <...>

// Include the header files which definitions you use.
#include "Skill.h"
#include "SkillWithArguments.h"

#include "CoordVector.h"
#include "CoordPoint.h"
#include "view_constants.h"
#include "ParameterView.h"
#include "MarkerView.h"
#include "AgentView.h"
#include "BallView.h"
#include "TerritoryControlView.h"
#include "PlayerView.h"
#include "rs_log.h"

RS_BEGIN_NAMESPACE

//class ParameterView;
//class MarkerView;
//class AgentView;
//class BallView;

/**
 * @class Pass Pass.h
 * This class implements the pass skill
 *
 * <!--
 * @pre HaveBall, visible players.
 * @post Ball hopefully at the closest passable player.
 * @invariant ?!?!
 *
 * @todo Finetune the skill.
 * -->
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.7 $
 * $Author: frehe $
 * $Date: 2001/06/19 14:01:45 $
 */

class Pass : public SkillWithArguments {
public:
  /** Default constructor. Here you can initialize the skill. */
  Pass();

  /** Destructor should clean up after the skill. */
  virtual ~Pass();

  
  /** This method is used to generate a plan for how the goal of
    the skill should be obtained by the agent. You can assume
    that the preconditions of the skill are true (this can be tested
    by calling the applicable() method, after you have implemented it). */
  virtual Plan generatePlan();

  /** This method should return a FuzzyBool telling the caller
    if the skill wants to be called again. */
  virtual FuzzyBool persistent();

  /** This method should implement the precondition specified in
   the description of the skill. */
  virtual FuzzyBool applicable();
  
  /** Used to set the skill's arguments in a generic way.
   This method must be overridden by subclasses.
   It should transform the generic vector of floats to
   the actual arguments.
   It will throw TooFewArgumentsException if not enough arguments
   are given. */
  virtual void setArguments(const std::vector<Float>& arguments)
    throw(RS::TooFewArgumentsException);


  /** This method is used to first set the arguments of the skill
    and then generate a plan for how the goal of the skill should
    be obtained by the agent. It uses the methods setArguments and
    generatePlan(), so it should not be overriden unless it should
    made unavailable. */
  virtual Plan generatePlan(const std::vector<Float>& arguments);
  virtual FuzzyBool applicable(const std::vector<Float>& arguments);
  Vector calculatePassVector();

protected:

  PlayerObject findClosestTeammate();
  ParameterView* parameterView;
  //  MarkerView* markerView;
  AgentView* agentView;
  BallView* ballView;
  PlayerView* playerView;
  TerritoryControlView* territoryControlView;
  std::vector<PlayerObject> Pass::seenVictims();
  std::vector<int> victims;

  
  /** Hide copy-constructor. */
  Pass(const Pass&);
  /** Hide assignment operator. */
  Pass& operator= (const Pass&);
};


RS_END_NAMESPACE

#endif // !PASS_H
