/* -*- Mode: C++ -*-
 * $Id: PlanContainer.cc,v 1.2 2000/11/19 11:30:28 andbo928 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: PlanContainer.cc
 * Unitname: Framework/skill
 * $Revision: 1.2 $
 *
 *
 * HISTORY
 * $Log: PlanContainer.cc,v $
 * Revision 1.2  2000/11/19 11:30:28  andbo928
 * Fixed a bug.
 *
 * Revision 1.1  2000/10/31 12:48:55  mikbr212
 * created
 *
 */

/**
 * @file PlanContainer.cc
 * Implementation file for the skill class PlanContainer.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.2 $
 * $Author: andbo928 $
 * $Date: 2000/11/19 11:30:28 $
 */

#include "PlanContainer.h"

RS_BEGIN_NAMESPACE


PlanContainer::PlanContainer()
  : plan(Plan()),
    consumePlan(true)
{
} 

PlanContainer::~PlanContainer()
{
}

/**
 * Inserts (replaces) a plan into this skill.
 */
void PlanContainer::setPlan(const Plan& thePlan) {
  plan = thePlan;
}

/**
 * Choose if the plan should be consumed (made empty) after generatePlan
 * has been called.
 * @param flag Set to true if plan should be consumed after call to generatePlan.
 */
void PlanContainer::setConsumePlan(bool flag) {
  consumePlan = flag;
}

/**
 * Retrieves the stored plan.
 * @see setConsumePlan(bool)
 */
Plan PlanContainer::generatePlan()
{
  Plan retVal = plan;
  if(consumePlan)
    plan.makeEmpty(); 
  return retVal;
}

/**
 * Always returns false..
 */
FuzzyBool PlanContainer::persistent()
{
  return FALSE;
}

/**
 * Applicable really have no meaning in this skill, now it checks if the skill 
 * contains a non-empty plan. Maybe a predicate could be provided?
 */
FuzzyBool PlanContainer::applicable()
{
  return !plan.empty();
}


RS_END_NAMESPACE


