/* -*- Mode: C++ -*-
 * $Id: ReturnToGoalArea.cc,v 1.3 2000/10/08 11:36:20 olale507 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: ReturnToGoalArea.cc
 * Unitname: Framework/skill
 * $Revision: 1.3 $
 * Created by: Bjrn, Ola och Jenny 2000-09-07
 * Last modified by $Author: olale507 $ $Date: 2000/10/08 11:36:20 $
 *
 *
 * HISTORY
 *
 * $Log: ReturnToGoalArea.cc,v $
 * Revision 1.3  2000/10/08 11:36:20  olale507
 * Commented the code.
 *
 * Revision 1.2  2000/09/24 11:37:08  olale507
 * Updated ReturnToGoalArea-skill.
 *
 * Revision 1.1  2000/09/23 13:37:12  olale507
 * Added the skill ReturnToGoalArea
 *
 *
 */

/**
 * @file ReturnToGoalArea.cc
 * Implementation file for the skill class ReturnToGoalArea.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.3 $
 * $Author: olale507 $
 * $Date: 2000/10/08 11:36:20 $
 */


#include "ReturnToGoalArea.h"
#include "InPenaltyArea.h"
#include "ParameterView.h"
#include "AgentView.h"
#include "MarkerView.h"
#include "DashCommand.h"
#include "TurnCommand.h"

#include "SayCommand.h"

#include "view_constants.h"

RS_BEGIN_NAMESPACE


// --- CONSTRUCTORS ---

ReturnToGoalArea::ReturnToGoalArea()
  : Skill(),
    parameterView(NULL),
    agentView(NULL),
    markerView(NULL)
{
  getView(parameterView, PARAMETER_VIEW_ID);  
  getView(agentView, AGENT_VIEW_ID, parameterView);
  getView(markerView, MARKER_VIEW_ID, parameterView);
}


// --- DESTRUCTOR ---

ReturnToGoalArea::~ReturnToGoalArea()
{
    viewManager->releaseView(PARAMETER_VIEW_ID);
    viewManager->releaseView(AGENT_VIEW_ID);
    viewManager->releaseView(MARKER_VIEW_ID);
}


// --- METHODS ---

Plan ReturnToGoalArea::generatePlan()
{
  static InPenaltyArea* inPenaltyArea = new InPenaltyArea();
  static PointUCO here,goal;
  static Command* cmd;
  static MarkerObject ourGoal = markerView->getOurGoal(NOW);
  static AngleDegUnknown ang;
  static Plan plan;

  here = agentView->getAgentPosition(NOW);
  goal = ourGoal.getPosition();

  ang = VectorUCO(here,goal).getAngle().getDeg();
  
  /* If the agent is turned too wrong, change direction. */
  if(ang.getDeg() - agentView->getBodyDirection(NOW).getDeg() > 0)
    cmd = new TurnCommand(agentView->getBodyDirection(NOW).getDeg() 
			  - ang.getDeg());
  /* Dash forward really fast.*/
  else
    cmd = new DashCommand(100);
  
  plan = Plan(Step(cmd));
  delete cmd;
  cmd = NULL;
  return plan; 
}


/* Persistent if the goalie is outside the goal area */
FuzzyBool ReturnToGoalArea::persistent()
{
  static InPenaltyArea* inPenaltyArea;
  inPenaltyArea = new InPenaltyArea();
  
  return !inPenaltyArea->evaluate();
}

/* Only for goalies who know where they are, their direction, etc. */
FuzzyBool ReturnToGoalArea::applicable()
{
  InPenaltyArea* inPenaltyArea;
  inPenaltyArea = new InPenaltyArea();

  if (agentView->isGoalie() && 
      !inPenaltyArea->evaluate())
  {
    inPenaltyArea->~InPenaltyArea();
    if (agentView->getAgentPosition(NOW).isKnown())
      if (agentView->getBodyDirection(NOW).isKnown())
	if (markerView->getOurGoal(NOW).getAbsoluteDirectionFrom(agentView->getAgentPosition(NOW)).isKnown())
	  return TRUE;
    return FALSE;
  }
  else {
    inPenaltyArea->~InPenaltyArea();
    return FALSE;
  }
}


RS_END_NAMESPACE

