/* -*- Mode: C++ -*-
 * $Id: ReturnToGoalArea.h,v 1.2 2000/10/08 11:36:20 olale507 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: ReturnToGoalArea.h
 * Unitname: Framework/skill
 * $Revision: 1.2 $
 * Created by: Bjrn, Ola och Jenny 2000-09-07
 * Last modified by $Author: olale507 $ $Date: 2000/10/08 11:36:20 $
 *
 *
 * HISTORY
 *
 * $Log: ReturnToGoalArea.h,v $
 * Revision 1.2  2000/10/08 11:36:20  olale507
 * Commented the code.
 *
 * Revision 1.1  2000/09/23 13:37:12  olale507
 * Added the skill ReturnToGoalArea
 *
 *
 */

/**
 * @file ReturnToGoalArea.h
 * Include file for the skill class ReturnToGoalArea.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.2 $
 * $Author: olale507 $
 * $Date: 2000/10/08 11:36:20 $
 */


#ifndef RETURNTOGOALAREA_H
#define RETURNTOGOALAREA_H


#include "Skill.h"

RS_BEGIN_NAMESPACE


class ParameterView;
class AgentView;
class MarkerView;

/**
 * @class ReturnToGoalArea ReturnToGoalArea.h
 * Runs back to the penalty area.
 * The agent runs back to the penalty area.
 * @pre The preconditions for the skill are:
 * The agent is the goalie and the agent is outside the penalty area
 * and agentPosition, bodyDirection, the distance to our goal are all
 * known.
 * @post The intended postconditions for the skill are:
 * The agent is inside the penalty area.
 * @todo Things that should be done:
 * The name should be changed to ReturnToPenaltyArea.
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.2 $
 * $Author: olale507 $
 * $Date: 2000/10/08 11:36:20 $ */
class ReturnToGoalArea : public Skill {
public:
  /** Default constructor. Here you can initialize the skill. */
  ReturnToGoalArea();

  /** Destructor should clean up after the skill. */
  virtual ~ReturnToGoalArea();

  
  /** This method is used to generate a plan for how the goal of
    the skill should be obtained by the agent. You can assume
    that the preconditions of the skill are true (this can be tested
    by calling the applicable() method, after you have implemented it). */
  virtual Plan generatePlan();

  /** This method should return a FuzzyBool telling the caller
    if the skill wants to be called again. */
  virtual FuzzyBool persistent();

  /** This method should implement the precondition specified in
   the description of the skill. */
  virtual FuzzyBool applicable();

  
private:
  /** A pointer to the parameter view. */
  ParameterView* parameterView;

  /** A pointer to the agent view. */
  AgentView* agentView;

  /** A pointer to the agent view. */
  MarkerView* markerView;

  /** Hide copy-constructor. */
  ReturnToGoalArea(const ReturnToGoalArea&);

  /** Hide assignment operator. */
  ReturnToGoalArea& operator= (const ReturnToGoalArea&);

};


RS_END_NAMESPACE


#endif // !STAYINGOALAREASTUB_H
