/* -*- Mode: C++ -*-
 * $Id: Rush.cc,v 1.2 2000/11/08 19:11:51 mikbr212 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: Rush.cc
 * Unitname: Framework/skill
 * $Revision: 1.2 $
 * Created by: Bjrn, Ola och Jenny 
 * Last modified by $Author: mikbr212 $ $Date: 2000/11/08 19:11:51 $
 *
 *
 * HISTORY
 * $Log: Rush.cc,v $
 * Revision 1.2  2000/11/08 19:11:51  mikbr212
 * Fixed memory leakage in generatePlan and introduced timeStamp in applicable and generatePlan.
 *
 */

/**
 * @file Rush.cc
 * This is the implemention file for the skill Rush.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.2 $
 * $Author: mikbr212 $
 * $Date: 2000/11/08 19:11:51 $
 */


#include "Rush.h"

#include "ParameterView.h"
#include "AgentView.h"
#include "BallView.h"

#include "DashCommand.h"
#include "CatchBall.h"
#include "MoveToPosition.h"
#include "view_constants.h"
#include "GoalieView.h"

RS_BEGIN_NAMESPACE


// --- CONSTRUCTORS ---

Rush::Rush()
  : Skill(),
    parameterView(NULL),
    agentView(NULL),
    ballView(NULL),
    goalieView(NULL)
{
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID, parameterView);
  getView(ballView, BALL_VIEW_ID);
  getView(goalieView, GOALIE_VIEW_ID);  
}


// --- DESTRUCTOR ---

Rush::~Rush()
{
  viewManager->releaseView(PARAMETER_VIEW_ID);
  viewManager->releaseView(AGENT_VIEW_ID);
  viewManager->releaseView(BALL_VIEW_ID);
  viewManager->releaseView(GOALIE_VIEW_ID);
}


// --- METHODS ---

// This simple function was first used to test the rest of the skill
/* Point Rush::interceptBallPos()
{
  PointUCO pos;

  pos = ballView->getBallPosition(NOW);

  if(pos.isKnown())    
    if(inPenaltyArea(pos.getPoint()))
      return pos.getPoint();
  pos.setX(-47);
  pos.setY(0);
  return pos.getPoint();
}
*/


FuzzyBool Rush::inPenaltyArea(Point pos)
{

  // This test should be done in the predicate InPenaltyArea in
  // the future.
  const Float MIN_X_IN_GOAL_AREA = -52.5;
  const Float MAX_X_IN_GOAL_AREA = -36;
  const Float MIN_Y_IN_GOAL_AREA = -20.16;
  const Float MAX_Y_IN_GOAL_AREA = 20.16;
  
  
  return (((MIN_X_IN_GOAL_AREA <= pos.getX()) && 
	   (pos.getX() <= MAX_X_IN_GOAL_AREA)) &&
	  ((MIN_Y_IN_GOAL_AREA <= pos.getY()) &&
	   (pos.getY() <= MAX_Y_IN_GOAL_AREA)));
}

Plan Rush::generatePlan()
{
  //  CatchBall* catchBall = new CatchBall();
  //  MoveToPosition* moveToPosition;
  static CatchBall catchBall;
  static MoveToPosition moveToPosition;

  // Micke: I'm not sure if the intention (by the author) was to recalculate
  // pos each turn, but now it does (to avoid running to a ghost-ball).
  if(timeStamp < Time::currentTime) // in case applicable wasn't run this cycle
    pos = goalieView->interceptPosition();

  if(catchBall.applicable())
    return catchBall.generatePlan();
  else {
    if(pos.isKnown())
      {
	//	moveToPosition = new MoveToPosition(pos.getPoint());
	//	moveToPosition->setDash(100);
	//	return moveToPosition->generatePlan();
	moveToPosition.setPosition(pos.getPoint());
	moveToPosition.setDash(100);
	return moveToPosition.generatePlan();
      }
    else 
      { // getPoint() is not healthy when point is not known!

	//	moveToPosition = new MoveToPosition(pos.getPoint());
	//	moveToPosition->setDash(100);
	//	return moveToPosition->generatePlan();

	return Plan(); // empty plan
      } 
  }
}

// The skill is persistent for as long as we're not "there yet".
FuzzyBool Rush::persistent()
{
  return !((agentView->getAgentObject(NOW).getDistanceFrom(pos) < 
	    parameterView->SP_catchable_area_l) ||
	   (ballView->getBallPosition(NOW).isKnown() &&
	    ballView->isBallCatchable(NOW)));
}

// The skill is applicable whenever there's and interceptposition to
// be found inside the penalty area.
FuzzyBool Rush::applicable()
{    
  pos = goalieView->interceptPosition();
  timeStamp = Time::currentTime;
  if ( pos.isKnown() )
    return inPenaltyArea(pos.getPoint());
  return FALSE;
}


RS_END_NAMESPACE
