/* -*- Mode: C++ -*-
 * $Id: Rush.h,v 1.2 2000/11/08 19:11:51 mikbr212 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: Rush.h
 * Unitname: Framework/skill
 * $Revision: 1.2 $
 * Created by: Bjrn, Ola och Jenny 2000-09-07
 * Last modified by $Author: mikbr212 $ $Date: 2000/11/08 19:11:51 $
 *
 *
 * HISTORY
 *
 *
 */

/**
 * @file Rush.h
 * Include file for the skill class Rush.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.2 $
 * $Author: mikbr212 $
 * $Date: 2000/11/08 19:11:51 $
 */


#ifndef RUSH_H
#define RUSH_H


#include "Skill.h"
#include "BallView.h"
#include "GoalieView.h"


RS_BEGIN_NAMESPACE


/**
 * @class Rush Rush.h
 * Rushes the goalie to intercept and catch the ball.
 * First an intercept point is calculated (in the
 * applicable-method). Then steps are generated to get to that
 * point. While the agent is on his way the skill is persistent.
 *
 * @pre The preconditions for the skill are:
 * There is an interceptposition inside the penalty area.
 * @post The intended postconditions for the skill are:
 * The ball is catchable.
 * @todo Things that should be done:
 * The skill should be applicable only if the new position is better
 * than the old one, to avoid useless dashing.
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.2 $
 * $Author: mikbr212 $
 * $Date: 2000/11/08 19:11:51 $ */
class Rush : public Skill {
public:
  /** Default constructor. Here you can initialize the skill. */
  Rush();

  /** Destructor should clean up after the skill. */
  virtual ~Rush();

  
  /** This method is used to generate a plan for how the goal of
    the skill should be obtained by the agent. You can assume
    that the preconditions of the skill are true (this can be tested
    by calling the applicable() method, after you have implemented it). */
  virtual Plan generatePlan();

  /** This method should return a FuzzyBool telling the caller
    if the skill wants to be called again. */
  virtual FuzzyBool persistent();

  /** This method should implement the precondition specified in
   the description of the skill. */
  // When this is called, pos is updated!
  virtual FuzzyBool applicable();

  
private:
  /** Hide copy-constructor. */
  Rush(const Rush&);
  /** Hide assignment operator. */
  Rush& operator= (const Rush&);

  // This isn't used any more, it's the corresponding method in
  // GoalieView that's used now.
  Point interceptBallPos();

  // *****this method will be replaced with a more general later.****
  FuzzyBool inPenaltyArea(Point pos);

  // This is the position to rush too. It's declared here so that both
  // generateplan and persistant can use it.
  PointUCO pos;
  Time timeStamp; // to check wether pos is valid

protected:
  ParameterView* parameterView;
  AgentView* agentView;
  BallView* ballView;
  GoalieView* goalieView;

};


RS_END_NAMESPACE

#endif // !RUSH_H







