/* -*- Mode: C++ -*-
 * $Id: Score.h,v 1.2 2000/10/12 02:49:21 marsc818 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: Score.h
 * Unitname: Framework/skill
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: marsc818 $ $Date: 2000/10/12 02:49:21 $
 *
 *
 * HISTORY
 * $Log: Score.h,v $
 * Revision 1.2  2000/10/12 02:49:21  marsc818
 * Mostly working although far from perfect Score skill
 *
 * Revision 1.1  2000/09/05 21:12:02  frehe
 * #include rs_log.h
 *
 */

/**
 * @file Score.h
 * Include file for the skill class Score.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.2 $
 * $Author: marsc818 $
 * $Date: 2000/10/12 02:49:21 $
 */


#ifndef SCORE_H
#define SCORE_H

#include <PlayerObject.h>

#include "Skill.h"


RS_BEGIN_NAMESPACE


class ParameterView;
class MarkerView;
class AgentView;
class BallView;
class PlayerView;


/** WeightenShootAngle struct: used for storing scoring directions and scoring dir.'s "weight" */
struct WeightenShootAngle
{
  AngleDegrees<Float> angle;
  Float weight;
};
/** WeightenShootAngleList type */
typedef std::vector<WeightenShootAngle> WeightenShootAngleList;

/** EqFormula structure: used for y=ax+b equations representation */
struct EqFormula
{
  /** a = the slope */
  Float a;
  /** b = the Y coordinate when X=0 */
  Float b;
};

/** EqFormulaList type: a vector of EqFormula structures */
typedef std::vector<EqFormula> EqFormulaList;

/** WeightenMidPoint struct: used fot storing a point and his "weight" */
struct WeightenMidPoint
{
  Point midPoint;
  Float weight;
};

/** Epsilon is a constant used as approximating factor for the calculations */
const Float Epsilon = 0.001;

/**
 * @class Score Score.h
 * This class encapsulates a scoring skill.
 *
 * <!--
 * @pre The preconditions for the skill are...
 * @post The postconditions for the skill are...
 * @invariant The class invariant is...
 *
 * @todo This needs to be done...
 * -->
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.2 $
 * $Author: marsc818 $
 * $Date: 2000/10/12 02:49:21 $
 */
class Score : public Skill {
public:
  Score();
  virtual ~Score();

  virtual Plan generatePlan();

  virtual FuzzyBool persistent();
  virtual FuzzyBool applicable();

private:
  void findScoringDirections(void);

  bool findBestScoringDirection(AngleDeg& ang);

protected:
  // members
  /** players in the triangle field */
  std::vector<PlayerObject> playersInField;

  /** possible scoring angles to the goal */
  WeightenShootAngleList shootAngles;

  // views
  ParameterView* parameterView;
  MarkerView* markerView;
  AgentView* agentView;
  BallView* ballView;
  PlayerView* playerView;
};


RS_END_NAMESPACE


#endif // !SCORE_H
