/* -*- Mode: C++ -*-
 * $Id: SkillWithZone.cc,v 1.12 2001/01/23 17:47:59 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: SkillWithZone.cc
 * Unitname: Framework/skill
 * $Revision: 1.12 $
 * Created by: Ola Lundqvist 2000-09-24
 * Last modified by $Author: olalu526 $ $Date: 2001/01/23 17:47:59 $
 *
 *
 * HISTORY
 *
 * $Log: SkillWithZone.cc,v $
 * Revision 1.12  2001/01/23 17:47:59  olalu526
 * Implemented some sort of avoid offide possibility to zone defence defender.
 *
 * Revision 1.11  2000/10/11 16:53:30  olalu526
 * Added ignoration of balls going to friendly players.
 *
 * Revision 1.10  2000/10/11 06:52:19  olalu526
 * Fixing documentation and precondition for ZoneDefenceDefender, fixed also the planner (removed some stuff and added clearball).
 *
 * Revision 1.8  2000/10/10 20:33:28  olalu526
 * Fixed some documentation and post, precondition for ZoneDefenceRest.
 *
 * Revision 1.7  2000/10/10 10:11:36  olalu526
 * Added documentation.
 *
 * Revision 1.6  2000/09/28 16:21:07  olalu526
 * Changed SkillWithZone (and friends) to use a new predicate and view ...
 *
 * Revision 1.5  2000/09/28 13:32:48  olalu526
 * A lot of changes.
 *
 * Revision 1.4  2000/09/27 19:18:50  olalu526
 * Some changes, now ZoneDefence actually defend the zone. Added ZoneDefenceRest.
 *
 * Revision 1.3  2000/09/25 21:25:28  olalu526
 * Changed SkillWithZone (added positionate) and changed ZoneDefenceToDefend to use MoveToPosition.
 *
 * Revision 1.2  2000/09/25 16:25:14  olalu526
 * Added FutureBallView to SkillWithZone.
 *
 * Revision 1.1  2000/09/24 16:51:36  olalu526
 * Added SkillWithZone and changed ZoneDefenceDefender to use it.
 *
 */

/**
 * @file SkillWithZone.cc
 * Implementation file for the skill class SkillWithZone.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.12 $
 * $Author: olalu526 $
 * $Date: 2001/01/23 17:47:59 $
 */


#include "SkillWithZone.h"
#include "math_utils.h"


RS_BEGIN_NAMESPACE

// --- CONSTRUCTORS ---

SkillWithZone::SkillWithZone(const Zone &z)
  : SkillWithArguments(),
    zone(z),
    ballInZonePredicate(z),
    agentInZonePredicate(z),
    needPositionatePredicate(z),
    ballDirectedToZone(z),
    ballInGoodDirection(),
    ballInBadDirection(),
    isOpponentClose(),
    ballStopInZone(z),
    gotop(z.getPointOf()),
    clearBall(),
    cutoffpassangle(),
    ballView(NULL),
    futureBallView(NULL),
    positionateView(NULL),
    parameterView(NULL),
    agentView(NULL),
    markerView(NULL),
    playerView(NULL)
{
  getView(ballView, BALL_VIEW_ID);
  getView(futureBallView, FUTUREBALL_VIEW_ID);
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID, parameterView);
  getView(markerView, MARKER_VIEW_ID, parameterView);
  getView(playerView, PLAYER_VIEW_ID);
  getView(positionateView, ZONEDEFENCEDEFAULTPOSITION_VIEW_ID);
  positionateView->setZone(z);
}

// --- DESTRUCTOR ---

SkillWithZone::~SkillWithZone()
{
  viewManager->releaseView(BALL_VIEW_ID);
  viewManager->releaseView(FUTUREBALL_VIEW_ID);
  viewManager->releaseView(PARAMETER_VIEW_ID);
  viewManager->releaseView(AGENT_VIEW_ID);
  viewManager->releaseView(MARKER_VIEW_ID);
  viewManager->releaseView(PLAYER_VIEW_ID);
}

// --- METHODS ---

void SkillWithZone::setArguments(const std::vector<Float>& arguments)
  throw(TooFewArgumentsException)
{
  if (arguments.size() < 2) {
    RS_THROW(TooFewArgumentsException,
	     "PredicateWithZone got less than three arguments.");
  }
  else {
    setZone(Zone(arguments[0], arguments[1], arguments[2]));
  }
}

// Ska nog in lite kvadrater hr!!!

/* Function to calculate the default position of the agent. */
// Klar!!
void SkillWithZone::positionate() {
  // Here we should add some check for opponents.
  Point pos = positionateView->maxOffsidePosition(positionateView->getPosition());
  gotop.setPosition(pos);
}

Point SkillWithZone::getPosition() const {
  return gotop.getPosition();
}


void SkillWithZone::setZone(const Zone& z)
{
  zone = z;
  ballStopInZone.setZone(z);
  ballDirectedToZone.setZone(z);
  positionateView->setZone(z);
}


Zone SkillWithZone::getZone() const
{
  return zone;
}

RS_END_NAMESPACE


// Remove from here
/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

//#ifndef RS_USE_INLINE
//#  include "SkillWithZone.icc"
//#endif
// To here if you do not have any inline definitions!
