/* -*- Mode: C++ -*-
 * $Id: SkillWithZone.h,v 1.15 2001/01/23 17:47:59 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <olalu526@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: SkillWithZone.h
 * Unitname: Framework/skill
 * $Revision: 1.15 $
 * Created by: Fredrik Heintz 2000-09-22
 * Last modified by $Author: olalu526 $ $Date: 2001/01/23 17:47:59 $
 *
 *
 * HISTORY
 *
 * $Log: SkillWithZone.h,v $
 * Revision 1.15  2001/01/23 17:47:59  olalu526
 * Implemented some sort of avoid offide possibility to zone defence defender.
 *
 * Revision 1.14  2000/10/11 16:53:31  olalu526
 * Added ignoration of balls going to friendly players.
 *
 * Revision 1.13  2000/10/11 06:52:20  olalu526
 * Fixing documentation and precondition for ZoneDefenceDefender, fixed also the planner (removed some stuff and added clearball).
 *
 * Revision 1.11  2000/10/10 20:33:28  olalu526
 * Fixed some documentation and post, precondition for ZoneDefenceRest.
 *
 * Revision 1.10  2000/10/10 10:25:31  olalu526
 * Now using FacingBall instead.
 *
 * Revision 1.8  2000/10/10 10:11:36  olalu526
 * Added documentation.
 *
 * Revision 1.7  2000/10/07 14:28:28  olalu526
 * Added some skill files and fixed things.
 *
 * Revision 1.6  2000/09/29 21:58:00  olalu526
 * Created MoveToPositionFacingBall and let all zone-dependent skills use it, and to make this work, movetopositionfacing_x had to be fixed. Not perfect but almost.
 *
 * Revision 1.5  2000/09/28 16:21:07  olalu526
 * Changed SkillWithZone (and friends) to use a new predicate and view ...
 *
 * Revision 1.4  2000/09/28 13:32:48  olalu526
 * A lot of changes.
 *
 * Revision 1.3  2000/09/25 21:25:29  olalu526
 * Changed SkillWithZone (added positionate) and changed ZoneDefenceToDefend to use MoveToPosition.
 *
 * Revision 1.2  2000/09/25 16:25:15  olalu526
 * Added FutureBallView to SkillWithZone.
 *
 * Revision 1.1  2000/09/24 16:51:36  olalu526
 * Added SkillWithZone and changed ZoneDefenceDefender to use it.
 *
 */

/**
 * @file SkillWithZone.h
 * Include file for the skill base class SkillWithZone.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.15 $
 * $Author: olalu526 $
 * $Date: 2001/01/23 17:47:59 $
 */


#ifndef SKILLWITHZONE_H
#define SKILLWITHZONE_H


#include "SkillWithArguments.h"
#include "view_constants.h"
#include "GotoBall.h"
#include "Zone.h"
#include "AgentView.h"
#include "ParameterView.h"
#include "BallView.h"
#include "FutureBallView.h"
#include "MarkerView.h"
#include "BallInZone.h"
#include "BallDirectedToZone.h"
#include "IsBallDirectedGood.h"
#include "IsBallDirectedBad.h"
#include "IsOpponentClose.h"
#include "ClearBall.h"
#include "AgentInZone.h"
#include "MoveToPositionFacingBall.h"
#include "ZoneDefenceDefaultPosition.h"
#include "NeedPositionate.h"
#include "BallStopInZone.h"
#include "CutOffPassAngle.h"
#include "PlayerView.h"
#include "TooFewArgumentsException.h"

RS_BEGIN_NAMESPACE

class BallView;
class ParameterView;
class AgentView;
class MarkerView;
class PlayerView;


/**
 * @class SkillWithZone SkillWithZone.h
 * Base class for all zone defence skills.
 * Use this class if you need to pass a zone to it.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.15 $
 * $Author: olalu526 $
 * $Date: 2001/01/23 17:47:59 $
 */

class SkillWithZone : public SkillWithArguments {
public:
  /** Default constructor that takes a zone to defend. */
  SkillWithZone(const Zone &);

  /** Destructor should clean up after the skill. */
  ~SkillWithZone();

  
  /** Set the arguments in a generic way. */
  virtual void setArguments(const std::vector<Float>&)
    throw(TooFewArgumentsException);

  /** Sets the zone to defend. */
  virtual void setZone(const Zone&);

  /** Gets the zone that the skill defends. */
  virtual Zone getZone() const;

  /** Virtual function */
  virtual Plan generatePlan() = 0;

  /** Virtual function */
  virtual FuzzyBool persistent() = 0;

  /** Virtual function */
  virtual FuzzyBool applicable() = 0;

  /** Sets the default position to use. */
  virtual void positionate();

  /** Gets where the default position is. */
  Point getPosition() const;

protected:
  Zone zone;
  BallInZone   ballInZonePredicate;
  AgentInZone agentInZonePredicate;
  NeedPositionate needPositionatePredicate;
  BallDirectedToZone ballDirectedToZone;
  IsBallDirectedGood ballInGoodDirection;
  IsBallDirectedBad ballInBadDirection;
  IsOpponentClose isOpponentClose;
  BallStopInZone ballStopInZone;
  MoveToPositionFacingBall gotop;
  ClearBall clearBall;
  CutOffPassAngle cutoffpassangle;
  
  BallView*           ballView;
  FutureBallView* futureBallView;
  ZoneDefenceDefaultPosition* positionateView;
  ParameterView* parameterView;
  AgentView*         agentView;
  MarkerView*       markerView;
  PlayerView*		playerView;
  /** Hide copy-constructor. */
  SkillWithZone(const SkillWithZone&);
  /** Hide assignment operator. */
  SkillWithZone& operator= (const SkillWithZone&);
};

RS_END_NAMESPACE

#endif // !SKILLWITHZONE_H
