/* -*- Mode: C++ -*-
 * $Id: Triangle.h,v 1.1 2000/10/12 03:12:11 marsc818 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: Triangle.h
 * Unitname: Framework/skill
 * $Revision: 1.1 $
 * Created by: Schlienger Marc 2000-10-08
 * Last modified by $Author: marsc818 $ $Date: 2000/10/12 03:12:11 $
 *
 *
 * HISTORY
 * $Log: Triangle.h,v $
 * Revision 1.1  2000/10/12 03:12:11  marsc818
 * generic triangle class for mathematical related functions
 *
 * Revision 1.0  2000/10/11 21:12:02  
 * #include rs_log.h
 *
 */

/**
 * @file Triangle.h
 * Include file for the utility class Triangle.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.1 $
 * $Author: marsc818 $
 * $Date: 2000/10/12 03:12:11 $
 */


#ifndef TRIANGLE_H
#define TRIANGLE_H

#include <library_types.h>

RS_BEGIN_NAMESPACE


/**
 * @class Triangle Triangle.h
 * This class encapsulates a mathematical Triangle.
 * Used for calculations on triangle typical functions.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.1 $
 * $Author: marsc818 $
 * $Date: 2000/10/12 03:12:11 $
 */
class Triangle
{
 public:
  /** Default Constructor */
  Triangle() : Pt1(0.0,0.0), Pt2(0.0,0.0), Pt3(0.0,0.0) {}
 
  /** Constructor with parameters */
  Triangle( CoordPoint<Float>& pt1, CoordPoint<Float>& pt2, CoordPoint<Float>& pt3 ) : Pt1(pt1), Pt2(pt2), Pt3(pt3) {}

  ~Triangle() {}

  /** Returns the area of this triangle */
  Float myArea() { return abs(0.5 * ((Pt3.getX() - Pt2.getX()) * (Pt2.getY() - Pt1.getY()) -  (Pt2.getX() - Pt1.getX()) * (Pt3.getY() - Pt2.getY()))); }
 
  /** Returns the area of the triangle defined by the given CoordPoints */
  Float area(CoordPoint<Float>& pt1, CoordPoint<Float>& pt2, CoordPoint<Float>& pt3) { return abs(0.5 * ((pt3.getX() - pt2.getX()) * (pt2.getY() - pt1.getY()) -  (pt2.getX() - pt1.getX()) * (pt3.getY() - pt2.getY()))); }
 
  /** Returns true if the given point is in this triangle */
  bool isPointInTriangle(CoordPoint<Float> p, Float epsilon);


 private:
  /** the 3 points forming the triangle */
  CoordPoint<Float> Pt1, Pt2, Pt3;
};

RS_END_NAMESPACE


#endif // !TRIANGLE_H
