/* -*- Mode: C++ -*-
 * $Id: TurnBall.h,v 1.8 2000/10/30 18:00:14 mikbr212 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: TurnBall.h
 * Unitname: Framework/skill
 * $Revision: 1.8 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: mikbr212 $ $Date: 2000/10/30 18:00:14 $
 *
 *
 * HISTORY
 * $Log: TurnBall.h,v $
 * Revision 1.8  2000/10/30 18:00:14  mikbr212
 * Fixed persistent() (added timestamps...)
 *
 * Revision 1.7  2000/10/28 20:24:34  mikbr212
 * updated because constructor of KickBallView changed
 *
 * Revision 1.6  2000/10/10 13:34:50  mikbr212
 * more tweaked default values
 *
 * Revision 1.5  2000/10/09 18:47:36  mikbr212
 * Fixed stuff in generatePlan
 *
 * Revision 1.4  2000/10/09 10:18:56  mikbr212
 * minor tweaking
 *
 * Revision 1.3  2000/10/08 20:03:34  mikbr212
 * fixed some extra cases in generatePlan
 *
 * Revision 1.2  2000/10/07 14:26:57  mikbr212
 * *** empty log message ***
 *
 * Revision 1.1  2000/09/24 19:30:18  andbo928
 * Various skills.
 *
 * Revision 1.1  2000/09/05 21:12:02  frehe
 * #include rs_log.h
 *
 */

/**
 * @file TurnBall.h
 * Include file for the skill class TurnBall.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.8 $
 * $Author: mikbr212 $
 * $Date: 2000/10/30 18:00:14 $
 */


#ifndef TURNBALL_H
#define TURNBALL_H


#include "SkillWithVector.h"
#include "CoordVector.h"
#include "CoordPoint.h"
#include "Line.h"
#include "ParameterView.h"
#include "BallView.h"
#include "rs_log.h"
#include "view_constants.h"
#include "KickBallView.h"
#include "AgentView.h"
#include "KickCommand.h"


RS_BEGIN_NAMESPACE

/**
 * @class TurnBall TurnBall.h
 * This class encapsulates a turnball skill.
 *
 * @pre The preconditions for the skill are:
 * 1) ball is kickable
 * @post The postconditions for the skill are:
 * 1) ball is is near the wanted position
 * @invariant The class invariant is...
 *
 * @todo A lot!
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.8 $
 * $Author: mikbr212 $
 * $Date: 2000/10/30 18:00:14 $
 */
class TurnBall : public SkillWithVector {
public:
  TurnBall(const Vector& vec);
  virtual ~TurnBall();

  //  virtual void setArguments(const Vector&) 
  //    throw(TooFewArgumentsException);
  virtual void setVector(const Vector& vec);
  virtual Vector getVector() const;

  virtual bool setAccuracy(double x = 0.3); 
  virtual bool setPlayerMargin(double x = 15.0);
  virtual bool setKickableMargin(double x = 40.0);
  virtual bool setMaxAcceleration(double x = 0.15);

  virtual Plan generatePlan();
  virtual Plan generatePlan(const Vector& vec);

  virtual FuzzyBool persistent();
  
  virtual FuzzyBool applicable();
  virtual FuzzyBool applicable(const Vector& vec);

protected:
  Plan getKickPlan(Vector vec);
  
  double bufR; //Buffer radius
  double kickR; //Kickable area radius
  double accR; // accuracy radius
  double maxAccelerate;

  double kickMax;
  double bufMin;

  Time timeStamp;
  Vector newBallPos;

  ParameterView* parameterView;
  BallView* ballView;
  KickBallView* kickBallView;
  //Vector vector; inherited
};


RS_END_NAMESPACE


#endif // !TURNBALL_H
