/* -*- Mode: C++ -*-
 * $Id: TurnDirection.cc,v 1.9 2001/06/19 14:01:45 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: TurnDirection.cc
 * Unitname: Framework/skill
 * $Revision: 1.9 $
 * Created by: Ola Lundqvist 2000-09-27
 * Last modified by $Author: frehe $ $Date: 2001/06/19 14:01:45 $
 *
 *
 * HISTORY
 *
 * $Log: TurnDirection.cc,v $
 * Revision 1.9  2001/06/19 14:01:45  frehe
 * Made minor changes to compile RoboSoc with gcc-3.0
 *
 * Revision 1.8  2000/10/22 15:09:45  perpe427
 * Made it possible to set minimum turn angle
 *
 * Revision 1.7  2000/10/01 13:05:56  olalu526
 * Updated FacingDirection, fixing bugs and stuff.
 *
 * Revision 1.6  2000/09/30 15:35:16  olalu526
 * Bug fixed MoveToPositionFacingDirection, now works really fine.
 *
 * Revision 1.5  2000/09/29 21:58:01  olalu526
 * Created MoveToPositionFacingBall and let all zone-dependent skills use it, and to make this work, movetopositionfacing_x had to be fixed. Not perfect but almost.
 *
 * Revision 1.4  2000/09/29 17:55:03  olalu526
 * Added FaceDirection, fixed some in FindBall (now works as expected), fixed segmentation fault in TurnDirection and more.
 *
 * Revision 1.3  2000/09/28 20:39:05  olalu526
 * Updated for testing, segmentation fault...
 *
 * Revision 1.2  2000/09/28 20:17:37  olalu526
 * Update TurnDirection.
 *
 * Revision 1.1  2000/09/28 20:12:27  olalu526
 * Added TurnDirection.
 *
 *
 */

/**
 * @file TurnDirection.cc
 * Implementation file for the skill class TurnDirection.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.9 $
 * $Author: frehe $
 * $Date: 2001/06/19 14:01:45 $
 */


#include "TurnDirection.h"


RS_BEGIN_NAMESPACE


// --- CONSTRUCTORS ---


TurnDirection::TurnDirection()
  : SkillWithArguments(),
    angle(),
    agentView(),
    parameterView(),
    turnAngle(),
    minTurnAngle(5)
{
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID, parameterView);
}


TurnDirection::TurnDirection(const AngleDeg &a)
  : SkillWithArguments(),
    angle(a),
    agentView(),
    parameterView(),
    turnAngle()
{
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID, parameterView);
}


// --- DESTRUCTOR ---

TurnDirection::~TurnDirection()
{
  viewManager->releaseView(PARAMETER_VIEW_ID);
  viewManager->releaseView(AGENT_VIEW_ID);
}


// --- METHODS ---

void TurnDirection::setAngle(const AngleDeg &a) {
  angle = a;
}

void TurnDirection::setArguments(const std::vector<Float>& arguments)
  throw(TooFewArgumentsException)
{
  if (arguments.size() < 1) {
    RS_THROW(TooFewArgumentsException,
	     "TurnDirection got less than three arguments.");
  }
  else {
    setAngle(AngleDeg(arguments[0]));
  }
}


Plan TurnDirection::generatePlan(const AngleDeg& a) {
  setAngle(a);
  return generatePlan();
}

AngleDeg TurnDirection::getDiffAngle() const {
    AngleDeg agd = (agentView->getBodyDirection(NOW)).getDeg();
    /*    std::cout << "TurDDiff: " << (angle-agd).getDeg()
	 << " angle: " << angle.getDeg()
	 << " agbod: " << agd.getDeg() << endl << std::flush;*/
    return angle - agd;
}

AngleDeg TurnDirection::getLastTurnAngle() const {
  return turnAngle;
}

Plan TurnDirection::generatePlan()
{
  // Return an empty plan.
  static Command* cmd;
  static Plan plan;
  plan.makeEmpty();
  
  if ((agentView->getBodyDirection(NOW)).isKnown() ) {    
    turnAngle = getDiffAngle();
    cmd = new TurnCommand(turnAngle.getDeg());
    plan.addStep(Step(cmd));
    delete cmd;
    cmd = NULL;
    return plan;
  }
  std::cout << "TurnDirection: This should NEVER happen!" << std::flush;
  return Plan();
}


FuzzyBool TurnDirection::persistent()
{
  // Never persistent.
  return FALSE;
}


FuzzyBool TurnDirection::applicable(const AngleDeg& a) {
  setAngle(a);
  return applicable();
}


FuzzyBool TurnDirection::applicable()
{
  // Almost always applicable.
  if ((agentView->getBodyDirection(NOW)).isKnown()) {
    if (abs(getDiffAngle()) > minTurnAngle) {
      return TRUE;
    }
  }
  return FALSE;
}

void 
TurnDirection::setMinTurnAngle(const AngleDeg& a)
{
   minTurnAngle = a;
}

AngleDeg
TurnDirection::getMinTurnAngle()
{
   return minTurnAngle;
}
 
RS_END_NAMESPACE


// Remove from here
/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "TurnDirection.icc"
#endif
// To here if you do not have any inline definitions!
