/* -*- Mode: C++ -*-
 * $Id: TurnDirection.h,v 1.7 2000/10/22 15:09:46 perpe427 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: TurnDirection.h
 * Unitname: Framework/skill
 * $Revision: 1.7 $
 * Created by: Ola Lundqvist 2000-09-27
 * Last modified by $Author: perpe427 $ $Date: 2000/10/22 15:09:46 $
 *
 *
 * HISTORY
 *
 * $Log: TurnDirection.h,v $
 * Revision 1.7  2000/10/22 15:09:46  perpe427
 * Made it possible to set minimum turn angle
 *
 * Revision 1.6  2000/10/10 19:10:54  olalu526
 * Added documentation.
 *
 * Revision 1.5  2000/09/29 21:58:01  olalu526
 * Created MoveToPositionFacingBall and let all zone-dependent skills use it, and to make this work, movetopositionfacing_x had to be fixed. Not perfect but almost.
 *
 * Revision 1.4  2000/09/29 17:55:03  olalu526
 * Added FaceDirection, fixed some in FindBall (now works as expected), fixed segmentation fault in TurnDirection and more.
 *
 * Revision 1.3  2000/09/28 20:39:06  olalu526
 * Updated for testing, segmentation fault...
 *
 * Revision 1.2  2000/09/28 20:17:37  olalu526
 * Update TurnDirection.
 *
 * Revision 1.1  2000/09/28 20:12:27  olalu526
 * Added TurnDirection.
 *
 *
 */

/**
 * @file TurnDirection.h
 * Include file for the skill class GotoBall.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.7 $
 * $Author: perpe427 $
 * $Date: 2000/10/22 15:09:46 $
 */


#ifndef TURNDIRECTION_H
#define TURNDIRECTION_H


#include "SkillWithArguments.h"
#include "AgentView.h"
#include "ViewManager.h"
#include "ParameterView.h"
#include "view_constants.h"
#include "TurnCommand.h"


RS_BEGIN_NAMESPACE


/**
 * @class TurnDirection TurnDirection.h
 * This skill makes the agent turn to a specific direction.
 *
 * @pre The agent body direction is known and the diff angle is high enough.
 * @post The agent is turned to that direction.
 *
 * @todo Works, so nothing more to do.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.7 $
 * $Author: perpe427 $
 * $Date: 2000/10/22 15:09:46 $
 */
class TurnDirection : public SkillWithArguments {
public:
  /** Default constructor. Here you can initialize the skill. */
  TurnDirection();
  /** Constructor with an angle to turn to. */
  TurnDirection(const AngleDeg&);

  /** Destructor should clean up after the skill. */
  virtual ~TurnDirection();

  /** Set the absolute direction to turn to. */
  void setAngle(const AngleDeg&);

  /** This method is used to generate a plan for how the goal of
    the skill should be obtained by the agent. You can assume
    that the preconditions of the skill are true (this can be tested
    by calling the applicable() method, after you have implemented it). */
  virtual Plan generatePlan();
  virtual Plan generatePlan(const AngleDeg&);

  /** This method should return a FuzzyBool telling the caller
    if the skill wants to be called again. */
  virtual FuzzyBool persistent();
  
  /** Gets the angle to turn. */
  AngleDeg getDiffAngle() const;
  /** Get how much the agent have turned the last time. Useful for passing to
      facedirection if you use them in the same cycle. */
  AngleDeg getLastTurnAngle() const;

  /** Set the minimum turn angle for which the skill is applicable */
  void setMinTurnAngle(const AngleDeg&);

  /** Get the minimum turn angle for which the skill is applicable */
  AngleDeg getMinTurnAngle();

  /** This method implements the precondition specified in
   the description of the skill. Also sets a new angle. */
  virtual FuzzyBool applicable(const AngleDeg&);
  /** This method implements the precondition specified in
   the description of the skill. */
  virtual FuzzyBool applicable();
  /** Sets the arguments in a general way. */
  virtual void setArguments(const std::vector<Float>&)
    throw(TooFewArgumentsException);

  
private:
  /** The absolute angle to turn to. */
  AngleDeg angle;
  /** Hide copy-constructor. */
  TurnDirection(const TurnDirection&);
  /** Hide assignment operator. */
  TurnDirection& operator= (const TurnDirection&);
  AgentView *agentView;
  ParameterView *parameterView;
  AngleDeg turnAngle;
  AngleDeg minTurnAngle;
};


RS_END_NAMESPACE


// Remove from here 
/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "TurnDirection.icc"
#endif
// To here if you do not have any inline definitions!

#endif // !TURNDIRECTION_H
