/**
 * @file Zone.h
 * Include file for the Zone datatype.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2001/06/19 14:01:45 $
 */

#include "Zone.h"

RS_BEGIN_NAMESPACE

/** @class Zone Zone.h
 * Zone is (right now) a circular area. Not any longer.
 * Zone is a circular area with a center position and a radius.
 *
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2001/06/19 14:01:45 $
 */
Zone::Zone() :
  coordCircles(),
  coordRectangles(),
  point()
{
}

Zone::Zone(const Float &x, const Float &y, const Float &r) :
  coordCircles(),
  coordRectangles(),
  point(x,y)
{
  addArea(Circle(x,y,r));
}

Zone::Zone(const Float &x1, const Float &y1, const Float &x2, const Float &y2) :
  coordCircles(),
  coordRectangles(),
  point()
{
  Rectangle rec(x1,y1,x2,y2);
  addArea(rec);
  point = rec.getPointOf();
}

Zone::Zone(const Circle &c) :
  coordCircles(),
  coordRectangles(),
  point()
{
  point = c.getPointOf();
  addArea(c);
}

Zone::Zone(const Rectangle &rec) :
  coordCircles(),
  coordRectangles(),
  point()
{
  point = rec.getPointOf();
  addArea(rec);
}

Zone::Zone(const Point &p, const CircleContainer &c, const RectangleContainer& r) :
  coordCircles(c),
  coordRectangles(r),
  point(p)
{
}

Zone::~Zone() {
}

void Zone::addArea(const Circle& a) {
  coordCircles.push_back(a);
}

void Zone::addArea(const Rectangle& a) {
  coordRectangles.push_back(a);
}

FuzzyBool Zone::inArea(const Point& p) {
  for (CircleContainer::iterator i = coordCircles.begin();
       i != coordCircles.end();
       ++i) {
    if (i->inArea(p)) {
      return TRUE;
    }
  }
  for (RectangleContainer::iterator i = coordRectangles.begin();
       i != coordRectangles.end();
       ++i) {
    if (i->inArea(p)) {
      return TRUE;
    }
  }
  return FALSE;
}

//FuzzyBool troughArea(Point &,Angle &);

Point Zone::getPointOf() const {
  return point;
}

std::istream& operator>>(std::istream& is, Zone& z)
{
  CircleContainer cvec;
  RectangleContainer rvec;
  Circle circ;
  Rectangle rect;
  Point p;

  char c = 0;

  is >> std::ws >> c;
  if (c != 'z' && c != 'Z') {
    is.clear(std::ios::badbit); //error...;
  }
  is >> c;
  if (c == '(') {
    int cont = 1;
    is >> std::ws >> p;
    while (cont) {
      is >> std::ws >> c;
      if (c == 'c' || c == 'C') {
	is >> std::ws >> circ;
	if (is.fail()) {
	  std::cerr << "Circle unreadable." << std::endl;
	  cont = 0;
	}
	else {
	  // Add to vector too!
	  cvec.push_back(circ);
	  std::cerr << "C";
	}
      }
      else if (c == 'r' || c == 'R') {
	is >> std::ws >> rect;
	if (is.fail()) {
	  std::cerr << "Rect unreadable." << std::endl;
	  cont = 0;
	}
	else {
	  // Add to vector too!
	  rvec.push_back(rect);
	  std::cerr << "R";
	}
      }
      else if (c == ')') {
	cont = 0;
      }
      else {
	std::cerr << "Unknown error, unknown caracter." << std::endl;
	is.clear(std::ios::badbit); //error...;
	cont = 0;
      }
    }
  }
  else {
    is.clear(std::ios::badbit); //error...;
  }

  if (is) z = Zone(p, cvec, rvec);

  return is;
}

RS_END_NAMESPACE
