#ifndef ZONE_H
#define ZONE_H

/**
 * @file Zone.h
 * Include file for the Zone datatype.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.3 $
 * $Author: olalu526 $
 * $Date: 2000/11/07 17:20:06 $
 */

#include "library_types.h"
#include "CoordArea.h"
#include "CoordCircle.h"
#include "CoordRectangle.h"

RS_BEGIN_NAMESPACE

//typedef std::vector<Area> AreaContainer;
typedef std::vector<Circle> CircleContainer;
typedef std::vector<Rectangle> RectangleContainer;

/** @class Zone Zone.h
 * Zone is (right now) a circular area.
 * Zone is a circular area with a center position and a radius.
 *
 * $Revision: 1.3 $
 * $Author: olalu526 $
 * $Date: 2000/11/07 17:20:06 $
 */
//typedef Circle Zone;


class Zone {
 public:
  Zone();
  Zone(const Float &, const Float &, const Float &);
  Zone(const Float &, const Float &, const Float &, const Float&);
  Zone(const CoordCircle<Float> &);
  Zone(const CoordRectangle<Float> &);
  Zone(const Point &, const CircleContainer &, const RectangleContainer &);
  ~Zone();

  void addArea(const Rectangle&);
  void addArea(const Circle&);

  FuzzyBool inArea(const Point&);
  //FuzzyBool troughArea(Point &,Angle &);

  Point getPointOf() const;

 protected:

 private:
  CircleContainer coordCircles;
  RectangleContainer coordRectangles;
  Point point;
};

std::istream& operator>>(std::istream& is, Zone& z);

RS_END_NAMESPACE

#endif // ZONE_H
