/* -*- Mode: C++ -*-
 * $Id: ZoneDefence.cc,v 1.14 2001/06/19 14:01:46 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: ZoneDefence.cc
 * Unitname: Framework/skill
 * $Revision: 1.14 $
 * Created by: Ola Lundqvist 2000-09-24
 * Last modified by $Author: frehe $ $Date: 2001/06/19 14:01:46 $
 *
 *
 * HISTORY
 *
 * $Log: ZoneDefence.cc,v $
 * Revision 1.14  2001/06/19 14:01:46  frehe
 * Made minor changes to compile RoboSoc with gcc-3.0
 *
 * Revision 1.13  2000/11/08 10:16:41  olalu526
 * Fixing corruptness in dash to speedvector.
 *
 * Revision 1.12  2000/10/11 10:39:15  olalu526
 * Final fixes.
 *
 * Revision 1.11  2000/10/11 07:34:02  olalu526
 * Added documentation and added setZone function that should be there.
 *
 * Revision 1.10  2000/10/11 06:33:13  olalu526
 * Added the final documentation and fixed the final things in applicable.
 *
 * Revision 1.9  2000/10/10 11:17:36  olalu526
 * Set better defaults and debugged dashtospeedvector.
 *
 * Revision 1.8  2000/10/10 10:11:36  olalu526
 * Added documentation.
 *
 * Revision 1.7  2000/10/01 13:05:56  olalu526
 * Updated FacingDirection, fixing bugs and stuff.
 *
 * Revision 1.6  2000/09/28 20:12:28  olalu526
 * Added TurnDirection.
 *
 * Revision 1.5  2000/09/28 18:52:17  olalu526
 * Now the agent works quite fine.
 *
 * Revision 1.4  2000/09/28 16:21:07  olalu526
 * Changed SkillWithZone (and friends) to use a new predicate and view ...
 *
 * Revision 1.3  2000/09/27 19:18:50  olalu526
 * Some changes, now ZoneDefence actually defend the zone. Added ZoneDefenceRest.
 *
 * Revision 1.2  2000/09/25 16:25:16  olalu526
 * Added FutureBallView to SkillWithZone.
 *
 * Revision 1.1  2000/09/24 18:37:38  olalu526
 * Added ZoneDefence and changed ZoneDefenceToDefend to use SkillWithZone.
 *
 *
 */

/**
 * @file ZoneDefence.cc
 * Implementation file for the skill class ZoneDefence.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.14 $
 * $Author: frehe $
 * $Date: 2001/06/19 14:01:46 $
 */


#include "ZoneDefence.h"

RS_BEGIN_NAMESPACE


// --- CONSTRUCTORS ---

ZoneDefence::ZoneDefence(const Zone &z)
  : SkillWithZone(z),
    defender(z),
    todefend(z),
    positionate(z),
    rest(z)
{
}

// --- DESTRUCTOR ---

ZoneDefence::~ZoneDefence()
{
}


// --- METHODS ---

void ZoneDefence::setZone(const Zone &z) {
  SkillWithZone::setZone(z);
  defender.setZone(zone);
  todefend.setZone(zone);
  positionate.setZone(zone);
  rest.setZone(zone);
}

Plan ZoneDefence::generatePlan()
{
  PointUCO p = agentView->getAgentPosition(NOW);
  if (p.isKnown()) {
    std::cout << "A:" << p.getPoint() << std::flush;
  }
  p = ballView->getBallPosition(NOW);
  if (p.isKnown()) {
    std::cout << "B:" << p.getPoint() << std::endl << std::flush;
  }
  // Return an empty plan.
  if (defender.applicable()) {
    std::cout << "D," << std::flush;
    return defender.generatePlan();
  }
  else if (todefend.applicable()) {
    std::cout << "T," << std::flush;
    return todefend.generatePlan();
  }
  else if (positionate.applicable()) {
    std::cout << "P," << std::flush;
    return positionate.generatePlan();
  }
  else if (rest.applicable()) {
    std::cout << "R," << std::flush;
    return rest.generatePlan();
  }
  else {
    std::cout << "NOTHING! This should never happen!\n" << std::flush;
  }
  return Plan();
}

FuzzyBool ZoneDefence::persistent()
{
  // Never persistent.
  return FALSE;
}


FuzzyBool ZoneDefence::applicable()
{
  if (clearBall.applicable() && ! isOpponentClose()) {
    return FALSE;
  }
  return (rest.applicable() ||
	  positionate.applicable() ||
	  todefend.applicable() ||
	  defender.applicable());
}


RS_END_NAMESPACE


// Remove from here
/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "ZoneDefence.icc"
#endif
// To here if you do not have any inline definitions!
