/* -*- Mode: C++ -*-
 * $Id: ZoneDefence.h,v 1.6 2000/10/11 06:33:13 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: ZoneDefence.h
 * Unitname: Framework/skill
 * $Revision: 1.6 $
 * Created by: Fredrik Heintz 2000-09-24
 * Last modified by $Author: olalu526 $ $Date: 2000/10/11 06:33:13 $
 *
 *
 * HISTORY
 *
 * $Log: ZoneDefence.h,v $
 * Revision 1.6  2000/10/11 06:33:13  olalu526
 * Added the final documentation and fixed the final things in applicable.
 *
 * Revision 1.5  2000/10/10 10:11:36  olalu526
 * Added documentation.
 *
 * Revision 1.4  2000/09/28 18:52:17  olalu526
 * Now the agent works quite fine.
 *
 * Revision 1.3  2000/09/27 19:18:50  olalu526
 * Some changes, now ZoneDefence actually defend the zone. Added ZoneDefenceRest.
 *
 * Revision 1.2  2000/09/25 16:25:16  olalu526
 * Added FutureBallView to SkillWithZone.
 *
 * Revision 1.1  2000/09/24 18:37:38  olalu526
 * Added ZoneDefence and changed ZoneDefenceToDefend to use SkillWithZone.
 *
 */

/**
 * @file ZoneDefence.h
 * Include file for the skill class GotoBall.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.6 $
 * $Author: olalu526 $
 * $Date: 2000/10/11 06:33:13 $
 */


#ifndef ZONEDEFENCE_H
#define ZONEDEFENCE_H


#include "SkillWithZone.h"
#include "ZoneDefenceDefender.h"
#include "ZoneDefenceToDefend.h"
#include "ZoneDefencePositionate.h"
#include "ZoneDefenceRest.h"

RS_BEGIN_NAMESPACE

class BallView;
class ParameterView;
class AgentView;
class MarkerView;

/**
 * @class ZoneDefence ZoneDefence.h
 * This skill makes it possible for an agent to defend a zone.
 * The agent will defend the zone in the following ways:
 *	- Run (or walk) to a good position in the zone.
 *	- Intercept the ball if it is inside the zone (or near).
 *	- Rest and just follow the ball if it is a calm situation.
 *
 * @pre The preconditions for the skill are:
 *	- If the ball is kickable and no opponent is near then it is not
 *	  applicable, else it is possible to defend the zone.
 * @post The intended postconditions for the skill are:
 *	- To get the ball out of the zone (on the right side).
 *	- Or if the ball is kickable and no opponents near.
 *
 * @todo
 *	- Add the condition that if the ball is on the way to a friendly
 *	  player (and not to a opponent) then the goal is fullfilled.
 *	
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.6 $
 * $Author: olalu526 $
 * $Date: 2000/10/11 06:33:13 $
 */

class ZoneDefence : public SkillWithZone {
public:
  /** Here you can initialize the skill with a zone to defend. */
  ZoneDefence(const Zone &);

  /** Destructor should clean up after the skill. */
  ~ZoneDefence();

  
  /** This sets the zone for all subskills. */
  virtual void setZone(const Zone&);

  /** The funcction that generates the plan. The plan can be from 0 to 2 actions.*/
  virtual Plan generatePlan();

  /** This method should return a FuzzyBool telling the caller
    if the skill wants to be called again. Always false. */
  virtual FuzzyBool persistent();

  /** This method implements the precondition specified in
   the description of the skill. */
  virtual FuzzyBool applicable();

  
private:
  /** This skill knows how to defend. */
  ZoneDefenceDefender defender;
  /** Also knows how to get to the zone of defence. */
  ZoneDefenceToDefend todefend;
  /** Knows how to place itself on the default position. */
  ZoneDefencePositionate positionate;
  /** And how to rest, doing almost nothing. Maybe turn the head. */
  ZoneDefenceRest rest;
};

RS_END_NAMESPACE


// Remove from here 
/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "ZoneDefence.icc"
#endif
// To here if you do not have any inline definitions!

#endif // !ZONEDEFENCE_H
