/* -*- Mode: C++ -*-
 * $Id: ZoneDefenceDefender.cc,v 1.29 2001/06/19 14:01:46 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: ZoneDefenceDefender.cc
 * Unitname: Framework/skill
 * $Revision: 1.29 $
 * Created by: Ola Lundqvist 2000-09-22
 * Last modified by $Author: frehe $ $Date: 2001/06/19 14:01:46 $
 *
 *
 * HISTORY
 *
 * $Log: ZoneDefenceDefender.cc,v $
 * Revision 1.29  2001/06/19 14:01:46  frehe
 * Made minor changes to compile RoboSoc with gcc-3.0
 *
 * Revision 1.28  2001/01/23 22:17:40  olalu526
 * Fixed kick against goal.
 *
 * Revision 1.27  2001/01/23 17:47:59  olalu526
 * Implemented some sort of avoid offide possibility to zone defence defender.
 *
 * Revision 1.26  2001/01/22 11:57:54  olalu526
 * Removed unnecessary output.
 *
 * Revision 1.25  2001/01/22 11:56:14  olalu526
 * Fixed segfault problem and added findball.
 *
 * Revision 1.24  2001/01/11 12:28:02  olalu526
 * Updated with some improvements from the first competition.
 *
 * Revision 1.23  2000/11/07 17:35:41  olalu526
 * Removed redundant code and added so that the agent only tries to get the ball if he/she is closest.
 *
 * Revision 1.22  2000/10/30 11:51:49  olalu526
 * Recommitted this fix so that I do not call a function that segfaults.
 *
 * Revision 1.21  2000/10/11 16:53:31  olalu526
 * Added ignoration of balls going to friendly players.
 *
 * Revision 1.20  2000/10/11 10:39:15  olalu526
 * Final fixes.
 *
 * Revision 1.19  2000/10/11 06:52:20  olalu526
 * Fixing documentation and precondition for ZoneDefenceDefender, fixed also the planner (removed some stuff and added clearball).
 *
 * Revision 1.18  2000/10/11 06:33:13  olalu526
 * Added the final documentation and fixed the final things in applicable.
 *
 * Revision 1.17  2000/10/10 19:10:55  olalu526
 * Added documentation.
 *
 * Revision 1.16  2000/10/10 11:17:37  olalu526
 * Set better defaults and debugged dashtospeedvector.
 *
 * Revision 1.15  2000/10/10 10:11:36  olalu526
 * Added documentation.
 *
 * Revision 1.14  2000/10/09 11:07:49  olalu526
 * Made so that GotoBall inherits from MoveToPositionFacingBall.
 *
 * Revision 1.13  2000/10/01 13:05:57  olalu526
 * Updated FacingDirection, fixing bugs and stuff.
 *
 * Revision 1.12  2000/09/30 15:35:17  olalu526
 * Bug fixed MoveToPositionFacingDirection, now works really fine.
 *
 * Revision 1.11  2000/09/29 17:55:03  olalu526
 * Added FaceDirection, fixed some in FindBall (now works as expected), fixed segmentation fault in TurnDirection and more.
 *
 * Revision 1.10  2000/09/28 20:39:06  olalu526
 * Updated for testing, segmentation fault...
 *
 * Revision 1.9  2000/09/28 18:52:18  olalu526
 * Now the agent works quite fine.
 *
 * Revision 1.8  2000/09/28 16:21:08  olalu526
 * Changed SkillWithZone (and friends) to use a new predicate and view ...
 *
 * Revision 1.7  2000/09/28 13:32:49  olalu526
 * A lot of changes.
 *
 * Revision 1.6  2000/09/27 19:18:51  olalu526
 * Some changes, now ZoneDefence actually defend the zone. Added ZoneDefenceRest.
 *
 * Revision 1.5  2000/09/25 16:25:17  olalu526
 * Added FutureBallView to SkillWithZone.
 *
 * Revision 1.4  2000/09/24 16:51:37  olalu526
 * Added SkillWithZone and changed ZoneDefenceDefender to use it.
 *
 * Revision 1.3  2000/09/23 17:21:15  olalu526
 * Did some changes.
 *
 * Revision 1.2  2000/09/22 19:10:48  olalu526
 * Generated new ZoneDefenceDefender files and checked in Zone.h, Circle.h and CoordCircle.h needed by that.
 *
 */

/**
 * @file ZoneDefenceDefender.cc
 * Implementation file for the skill class ZoneDefenceDefender.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.29 $
 * $Author: frehe $
 * $Date: 2001/06/19 14:01:46 $
 */


#include "ZoneDefenceDefender.h"
#include "math_utils.h"
#include "InterceptBallStamina.h"
#include "IAmClosestToBall.h"
#include "IsOpponentClose.h"
#include "FindBall.h"
#include "KickBallToAbsoluteSpeedVector.h"
#include "KickForGoal.h"

RS_BEGIN_NAMESPACE


// --- CONSTRUCTORS ---

ZoneDefenceDefender::ZoneDefenceDefender(const Zone &z)
  : SkillWithZone(z)
{
}

// --- DESTRUCTOR ---

ZoneDefenceDefender::~ZoneDefenceDefender()
{
}


// --- METHODS ---

Plan ZoneDefenceDefender::generatePlan()
{
  static InterceptBallStamina interceptball;
  static Score score;
  static GotoBall gotoball;
  static IAmClosestToBall iAmClosestToBall;
  static TurnDirection turn;
  static IsOpponentClose isOpponentClose(6);
  static FindBall findBall;
  static KickBallToAbsoluteSpeedVector kick(Vector(0,0));
  static KickForGoal toGoalKick;

  //if (clearBall.applicable() &&
  //    isOpponentClose()) {
  //  return clearBall.generatePlan();
  //}
  if (toGoalKick.applicable()) {
    return toGoalKick.generatePlan();
  }
  if (interceptball.applicable()) {
    //std::cout << "I!";
    interceptball.setEmergency(FALSE);
    if (isOpponentClose()) {
      interceptball.setEmergency(TRUE);
    }
    return interceptball.generatePlan();
  }
  else if (gotoball.applicable() && iAmClosestToBall()) {
    gotoball.setEmergency(TRUE);
    //std::cout << "G!" << std::flush;
    return gotoball.generatePlan();
  }
  //std::cout << "t?" << std::flush;
  if (ballView->getRelativeBallDirection(NOW).isKnown()) {
    if (turn.applicable(ballView->getRelativeBallDirection(NOW).getDeg())) {
      //std::cout << "TURN!!" << std::flush;
      return turn.generatePlan();
    }
  }
  if (findBall.applicable()) {
    return findBall.generatePlan();
  }
  std::cout << "ZoneDefenceDefender: This should NEVER happen!" << std::flush;
  return Plan();
}

FuzzyBool ZoneDefenceDefender::persistent()
{
  // Never persistent.
  return FALSE;
}


FuzzyBool ZoneDefenceDefender::applicable()
{
  //if (clearBall.applicable() && ! isOpponentClose()) {
  //  return FALSE;
  //}
  /*if (! cutoffpassangle.applicable()) {
    return FALSE;
    }*/
  if (ballInZonePredicate() || ballStopInZone()) {
    return TRUE;
  }
  return FALSE;
}


RS_END_NAMESPACE


// Remove from here
/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "ZoneDefenceDefender.icc"
#endif
// To here if you do not have any inline definitions!
