/* -*- Mode: C++ -*-
 * $Id: ZoneDefenceDefender.h,v 1.16 2000/10/30 12:00:52 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: ZoneDefenceDefender.h
 * Unitname: Framework/skill
 * $Revision: 1.16 $
 * Created by: Fredrik Heintz 2000-09-22
 * Last modified by $Author: olalu526 $ $Date: 2000/10/30 12:00:52 $
 *
 *
 * HISTORY
 *
 * $Log: ZoneDefenceDefender.h,v $
 * Revision 1.16  2000/10/30 12:00:52  olalu526
 * Oops I did not commit this one, or recommitted.
 *
 * Revision 1.15  2000/10/11 16:53:31  olalu526
 * Added ignoration of balls going to friendly players.
 *
 * Revision 1.14  2000/10/11 06:53:02  olalu526
 * Added postcondition.
 *
 * Revision 1.12  2000/10/10 19:10:55  olalu526
 * Added documentation.
 *
 * Revision 1.11  2000/10/10 10:11:36  olalu526
 * Added documentation.
 *
 * Revision 1.10  2000/09/28 20:39:06  olalu526
 * Updated for testing, segmentation fault...
 *
 * Revision 1.9  2000/09/28 18:52:18  olalu526
 * Now the agent works quite fine.
 *
 * Revision 1.8  2000/09/28 13:32:49  olalu526
 * A lot of changes.
 *
 * Revision 1.7  2000/09/27 19:18:51  olalu526
 * Some changes, now ZoneDefence actually defend the zone. Added ZoneDefenceRest.
 *
 * Revision 1.6  2000/09/25 16:25:18  olalu526
 * Added FutureBallView to SkillWithZone.
 *
 * Revision 1.5  2000/09/24 16:51:37  olalu526
 * Added SkillWithZone and changed ZoneDefenceDefender to use it.
 *
 * Revision 1.4  2000/09/24 10:44:33  olalu526
 * Now ZoneDefenceToDefend works (had to remove a non woring file in Makefile.am, sorry.
 *
 * Revision 1.3  2000/09/23 17:21:16  olalu526
 * Did some changes.
 *
 * Revision 1.2  2000/09/22 19:10:48  olalu526
 * Generated new ZoneDefenceDefender files and checked in Zone.h, Circle.h and CoordCircle.h needed by that.
 *
 */

/**
 * @file ZoneDefenceDefender.h
 * Include file for the skill class GotoBall.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.16 $
 * $Author: olalu526 $
 * $Date: 2000/10/30 12:00:52 $
 */


#ifndef ZONEDEFENCEDEFENDER_H
#define ZONEDEFENCEDEFENDER_H


#include "SkillWithZone.h"
#include "GotoBall.h"
#include "BallInZone.h"
#include "AgentInZone.h"
#include "InterceptBall.h"
#include "TurnCommand.h"
#include "Pass.h"
#include "Score.h"
#include "IAmClosestToBall.h"
#include "FindBall.h"
#include "TurnDirection.h"

RS_BEGIN_NAMESPACE

class BallView;
class ParameterView;
class AgentView;
class MarkerView;

/**
 * @class ZoneDefenceDefender ZoneDefenceDefender.h
 * The skill to defend a zone.
 * This skill is used to defend a zone. With this skill the agent should
 * (in the future) be able to positionate, intercept the ball, and kick
 * the ball out of the zone (in the right direction) or pass an other
 * friendly player.
 *
 * @pre The preconditions for the skill are: (and)
 *	- If the ball is inside the zone or will stop inside the zone.
 *	- Not if the ball is kickable and no opponent is near.
 *	- Ball is not heading for friendly players.
 *	  This is implemented as quick as cutoffpassangle does not segfault.
 * @post The intended postconditions for the skill are:
 *	- To get the ball out of the zone (on the right side).
 *	- The ball is kickable and no opponent is near.
 *
 * @todo There are some things todo left:
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.16 $
 * $Author: olalu526 $
 * $Date: 2000/10/30 12:00:52 $
 */

class ZoneDefenceDefender : public SkillWithZone {
public:
  /** Here you can initialize the skill, with a zone to defend. */
  ZoneDefenceDefender(const Zone &);

  /** Destructor should clean up after the skill. */
  ~ZoneDefenceDefender();

  
  /** This method is used to generate a plan for how the goal of
      the skill should be obtained by the agent.
      For more information see the code it should be quite self explained.
  */
  virtual Plan generatePlan();

  /** This method should return a FuzzyBool telling the caller
      if the skill wants to be called again. Never persistent. */
  virtual FuzzyBool persistent();

  /** This method implements the precondition specified in
      the description of the skill. */
  virtual FuzzyBool applicable();

  
private:
  IAmClosestToBall iAmClosestToBall;
};

RS_END_NAMESPACE


// Remove from here 
/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "ZoneDefenceDefender.icc"
#endif
// To here if you do not have any inline definitions!

#endif // !ZONEDEFENCEDEFENDER_H
