/* -*- Mode: C++ -*-
 * $Id: ZoneDefencePositionate.cc,v 1.10 2001/06/19 14:01:46 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: ZoneDefencePositionate.cc
 * Unitname: Framework/skill
 * $Revision: 1.10 $
 * Created by: Ola Lundqvist 2000-09-22
 * Last modified by $Author: frehe $ $Date: 2001/06/19 14:01:46 $
 *
 *
 * HISTORY
 *
 * $Log: ZoneDefencePositionate.cc,v $
 * Revision 1.10  2001/06/19 14:01:46  frehe
 * Made minor changes to compile RoboSoc with gcc-3.0
 *
 * Revision 1.9  2000/11/08 23:58:09  olalu526
 * fixes so that it will not become that tired.
 *
 * Revision 1.8  2000/10/11 10:39:16  olalu526
 * Final fixes.
 *
 * Revision 1.7  2000/10/11 07:04:29  olalu526
 * Added post, precondition documentation and removed unnecessary stuff.
 *
 * Revision 1.6  2000/10/10 10:11:37  olalu526
 * Added documentation.
 *
 * Revision 1.5  2000/10/07 14:28:28  olalu526
 * Added some skill files and fixed things.
 *
 * Revision 1.4  2000/10/01 13:05:57  olalu526
 * Updated FacingDirection, fixing bugs and stuff.
 *
 * Revision 1.3  2000/09/28 17:51:30  olalu526
 * Fixed compilation errors. Sorry for those.
 *
 * Revision 1.2  2000/09/28 16:21:08  olalu526
 * Changed SkillWithZone (and friends) to use a new predicate and view ...
 *
 * Revision 1.1  2000/09/28 13:32:50  olalu526
 * A lot of changes.
 *
 *
 */

/**
 * @file ZoneDefencePositionate.cc
 * Implementation file for the skill class ZoneDefencePositionate.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.10 $
 * $Author: frehe $
 * $Date: 2001/06/19 14:01:46 $
 */


#include "ZoneDefencePositionate.h"
#include "math_utils.h"

RS_BEGIN_NAMESPACE


// --- CONSTRUCTORS ---

ZoneDefencePositionate::ZoneDefencePositionate(const Zone &z)
  : SkillWithZone(z)
{
}

// --- DESTRUCTOR ---

ZoneDefencePositionate::~ZoneDefencePositionate()
{
}


// --- METHODS ---

Plan ZoneDefencePositionate::generatePlan()
{
  gotop.setMaxUseStaminaPerCycle(Float(0));
  if (needPositionatePredicate()) {
    positionate();
    if (ballInGoodDirection()) {
      gotop.setMaxUseStaminaPerCycle(Float(100));
      gotop.setEmergency(FALSE);
    }
    else if (ballInBadDirection()) {
      gotop.setEmergency(FALSE);
    }
    //std::cout << "P," << std::flush;
    if (gotop.applicable()) {
      //std::cout << "G!" << std::flush;
      return gotop.generatePlan();
    }
  }
  std::cout << "ZoneDefencePositionate: This should NEVER happen!" << std::flush;
  return Plan();
}


FuzzyBool ZoneDefencePositionate::persistent()
{
  // Never persistent.
  return FALSE;
}


FuzzyBool ZoneDefencePositionate::applicable()
{
  // Allways applicable.
  return needPositionatePredicate() && gotop.applicable();
}


RS_END_NAMESPACE


// Remove from here
/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

//#ifndef RS_USE_INLINE
//#  include "ZoneDefencePositionate.icc"
//#endif
// To here if you do not have any inline definitions!
