/* -*- Mode: C++ -*-
 * $Id: ZoneDefencePositionate.h,v 1.3 2000/10/11 10:39:16 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: ZoneDefencePositionate.h
 * Unitname: Framework/skill
 * $Revision: 1.3 $
 * Created by: Fredrik Heintz 2000-09-22
 * Last modified by $Author: olalu526 $ $Date: 2000/10/11 10:39:16 $
 *
 *
 * HISTORY
 *
 * $Log: ZoneDefencePositionate.h,v $
 * Revision 1.3  2000/10/11 10:39:16  olalu526
 * Final fixes.
 *
 * Revision 1.2  2000/10/11 07:04:30  olalu526
 * Added post, precondition documentation and removed unnecessary stuff.
 *
 * Revision 1.1  2000/09/28 13:32:51  olalu526
 * A lot of changes.
 *
 *
 */

/**
 * @file ZoneDefencePositionate.h
 * Include file for the skill class GotoBall.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.3 $
 * $Author: olalu526 $
 * $Date: 2000/10/11 10:39:16 $
 */


#ifndef ZONEDEFENCEPOSITIONATE_H
#define ZONEDEFENCEPOSITIONATE_H


#include "SkillWithZone.h"
#include "GotoBall.h"
#include "BallInZone.h"
#include "AgentInZone.h"
#include "BallDirectedToZone.h"

#include "InterceptBall.h"

#include "TurnCommand.h"
#include "TurnNeckCommand.h"
#include "DashCommand.h"

RS_BEGIN_NAMESPACE

class BallView;
class ParameterView;
class AgentView;
class MarkerView;

/**
 * @class ZoneDefencePositionate ZoneDefencePositionate.h
 * Finds and walks to a good base position in the zone.
 * The position is calculated from the center of the zone and the agent
 * will move closer and closer to the edge if the ball is closer and closer
 * to the zone.
 *
 * I have one more positionate rule but it is not implemented yet.
 * It uses the goal position instead of the center of the zone and makes
 * is possible to use any type of zone.
 *
 * @pre The preconditions for the skill are:
 *	- It is possible to go the calculated position.
 *	- The position is far enough, so that there is a reason to move. 
 * @post The intended postconditions for the skill are:
 *	- The agent finds (and is on) a good position.
 *
 * @todo There are some more things to do:
 *	- Implement the new positionate routine.
 *	  This works quite good but there is a new one that might work better.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.3 $
 * $Author: olalu526 $
 * $Date: 2000/10/11 10:39:16 $
 */

class ZoneDefencePositionate : public SkillWithZone {
public:
  /** Default constructor. Here you can initialize the skill. */
  //ZoneDefencePositionate();
  ZoneDefencePositionate(const Zone &);

  /** Destructor should clean up after the skill. */
  ~ZoneDefencePositionate();

  
  /** This method is used to generate a plan for how the goal of
    the skill should be obtained by the agent. You can assume
    that the preconditions of the skill are true (this can be tested
    by calling the applicable() method, after you have implemented it). */
  virtual Plan generatePlan();

  /** This method should return a FuzzyBool telling the caller
    if the skill wants to be called again. */
  virtual FuzzyBool persistent();

  /** This method should implement the precondition specified in
   the description of the skill. */
  virtual FuzzyBool applicable();

private:
};

RS_END_NAMESPACE


// Remove from here 
/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

//#ifdef RS_USE_INLINE
//#  include "ZoneDefencePositionate.icc"
//#endif
// To here if you do not have any inline definitions!

#endif // !ZONEDEFENCETODEFEND_H
