/* -*- Mode: C++ -*-
 * $Id: ZoneDefenceRest.cc,v 1.8 2001/06/19 14:01:46 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: ZoneDefenceRest.cc
 * Unitname: Framework/skill
 * $Revision: 1.8 $
 * Created by: Ola Lundqvist 2000-09-22
 * Last modified by $Author: frehe $ $Date: 2001/06/19 14:01:46 $
 *
 *
 * HISTORY
 *
 * $Log: ZoneDefenceRest.cc,v $
 * Revision 1.8  2001/06/19 14:01:46  frehe
 * Made minor changes to compile RoboSoc with gcc-3.0
 *
 * Revision 1.7  2000/10/10 20:33:28  olalu526
 * Fixed some documentation and post, precondition for ZoneDefenceRest.
 *
 * Revision 1.6  2000/10/10 10:11:37  olalu526
 * Added documentation.
 *
 * Revision 1.5  2000/09/29 17:55:04  olalu526
 * Added FaceDirection, fixed some in FindBall (now works as expected), fixed segmentation fault in TurnDirection and more.
 *
 * Revision 1.4  2000/09/28 20:39:07  olalu526
 * Updated for testing, segmentation fault...
 *
 * Revision 1.3  2000/09/28 20:12:28  olalu526
 * Added TurnDirection.
 *
 * Revision 1.2  2000/09/28 18:52:18  olalu526
 * Now the agent works quite fine.
 *
 * Revision 1.1  2000/09/27 19:18:52  olalu526
 * Some changes, now ZoneDefence actually defend the zone. Added ZoneDefenceRest.
 *
 *
 */

/**
 * @file ZoneDefenceRest.cc
 * Implementation file for the skill class ZoneDefenceRest.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.8 $
 * $Author: frehe $
 * $Date: 2001/06/19 14:01:46 $
 */


#include "ZoneDefenceRest.h"
#include "math_utils.h"


RS_BEGIN_NAMESPACE


// --- CONSTRUCTORS ---

ZoneDefenceRest::ZoneDefenceRest(const Zone &z)
  : SkillWithZone(z)
{
}

// --- DESTRUCTOR ---

ZoneDefenceRest::~ZoneDefenceRest()
{
}


// --- METHODS ---

Plan ZoneDefenceRest::generatePlan()
{
  static FindBall findball;
  static TurnDirection turn;
  
  if (findball.applicable()) {
    std::cout << "FB!" << std::flush;
    return findball.generatePlan();
  }
  /*  std::cout << "Tu," << std::flush;
      AngleDeg d = ballView->getRelativeBallDirection(NOW).getDeg();
      std::cout << "Gd," << std::flush;
      if (turn.applicable(d)) {
      std::cout << "T!" << std::flush;
      return turn.generatePlan();
      }*/
  //  std::cout << "R!" << ballView->getBallPosition(NOW).getPoint() << std::flush;
  return Plan();
}

FuzzyBool ZoneDefenceRest::persistent()
{
  // Never persistent.
  return FALSE;

}


FuzzyBool ZoneDefenceRest::applicable()
{
  // Allways applicable.
  if (! ballInZonePredicate() &&
      ballInGoodDirection() &&
      ! ballDirectedToZone()) {
    std::cout << "A," << std::flush;
    return TRUE;
  }
  return FALSE;
}


RS_END_NAMESPACE


// Remove from here
/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "ZoneDefenceRest.icc"
#endif
// To here if you do not have any inline definitions!
