/* -*- Mode: C++ -*-
 * $Id: ZoneDefenceRest.h,v 1.4 2000/10/10 20:33:29 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: ZoneDefenceRest.h
 * Unitname: Framework/skill
 * $Revision: 1.4 $
 * Created by: Fredrik Heintz 2000-09-22
 * Last modified by $Author: olalu526 $ $Date: 2000/10/10 20:33:29 $
 *
 *
 * HISTORY
 *
 * $Log: ZoneDefenceRest.h,v $
 * Revision 1.4  2000/10/10 20:33:29  olalu526
 * Fixed some documentation and post, precondition for ZoneDefenceRest.
 *
 * Revision 1.3  2000/09/28 20:39:07  olalu526
 * Updated for testing, segmentation fault...
 *
 * Revision 1.2  2000/09/28 18:52:18  olalu526
 * Now the agent works quite fine.
 *
 * Revision 1.1  2000/09/27 19:18:52  olalu526
 * Some changes, now ZoneDefence actually defend the zone. Added ZoneDefenceRest.
 *
 *
 */

/**
 * @file ZoneDefenceRest.h
 * Include file for the skill class GotoBall.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.4 $
 * $Author: olalu526 $
 * $Date: 2000/10/10 20:33:29 $
 */


#ifndef ZONEDEFENCEREST_H
#define ZONEDEFENCEREST_H


#include "SkillWithZone.h"
#include "GotoBall.h"
#include "BallInZone.h"
#include "FindBall.h"
#include "AgentInZone.h"
#include "InterceptBall.h"
#include "TurnCommand.h"
#include "Pass.h"
#include "Score.h"
#include "TurnDirection.h"

RS_BEGIN_NAMESPACE

class BallView;
class ParameterView;
class AgentView;
class MarkerView;

/**
 * @class ZoneDefenceRest ZoneDefenceRest.h
 * The skill to rest while defending a zone.
 * This skill is used to rest while defending a zone. With this skill the
 * agent should be able to take a rest if the ball is not too close.
 *
 * @pre The preconditions for the skill are:
 *	- The agent should be inside the zone.
 *	- The ball should not be directed to the zone.
 *	- The ball should be also directed in a good (to opponent side)
 *	  direction.
 * @post The intended postconditions for the skill are:
 *	- The agent should have got some better stamina.
 *
 * @todo Nothing.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.4 $
 * $Author: olalu526 $
 * $Date: 2000/10/10 20:33:29 $
 */

class ZoneDefenceRest : public SkillWithZone {
public:
  /** Default constructor. Here you can initialize the skill.
      It needs a zone. */
  ZoneDefenceRest(const Zone &);

  /** Destructor should clean up after the skill. */
  ~ZoneDefenceRest();

  
  /** This method is used to generate a plan for how the goal of
    the skill should be obtained by the agent. You can assume
    that the preconditions of the skill are true (this can be tested
    by calling the applicable() method, after you have implemented it). */
  virtual Plan generatePlan();

  /** This method should return a FuzzyBool telling the caller
    if the skill wants to be called again. Never persistent. */
  virtual FuzzyBool persistent();

  /** This method implements the precondition specified in
      the description of the skill. */
  virtual FuzzyBool applicable();

  
private:
};

RS_END_NAMESPACE


// Remove from here 
/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "ZoneDefenceRest.icc"
#endif
// To here if you do not have any inline definitions!

#endif // !ZONEDEFENCEREST_H
