/* -*- Mode: C++ -*-
 * $Id: ZoneDefenceToDefend.cc,v 1.14 2001/06/19 14:01:46 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: ZoneDefenceToDefend.cc
 * Unitname: Framework/skill
 * $Revision: 1.14 $
 * Created by: Ola Lundqvist 2000-09-22
 * Last modified by $Author: frehe $ $Date: 2001/06/19 14:01:46 $
 *
 *
 * HISTORY
 *
 * $Log: ZoneDefenceToDefend.cc,v $
 * Revision 1.14  2001/06/19 14:01:46  frehe
 * Made minor changes to compile RoboSoc with gcc-3.0
 *
 * Revision 1.13  2000/11/07 17:34:06  olalu526
 * Fixed the to defend applicable function to avoid empty plans.
 *
 * Revision 1.12  2000/10/11 10:39:16  olalu526
 * Final fixes.
 *
 * Revision 1.11  2000/10/11 07:34:02  olalu526
 * Added documentation and added setZone function that should be there.
 *
 * Revision 1.10  2000/10/10 10:11:37  olalu526
 * Added documentation.
 *
 * Revision 1.9  2000/10/07 14:28:28  olalu526
 * Added some skill files and fixed things.
 *
 * Revision 1.8  2000/10/01 13:05:57  olalu526
 * Updated FacingDirection, fixing bugs and stuff.
 *
 * Revision 1.7  2000/09/28 17:51:30  olalu526
 * Fixed compilation errors. Sorry for those.
 *
 * Revision 1.6  2000/09/28 16:21:11  olalu526
 * Changed SkillWithZone (and friends) to use a new predicate and view ...
 *
 * Revision 1.5  2000/09/28 13:32:54  olalu526
 * A lot of changes.
 *
 * Revision 1.4  2000/09/25 21:25:29  olalu526
 * Changed SkillWithZone (added positionate) and changed ZoneDefenceToDefend to use MoveToPosition.
 *
 * Revision 1.3  2000/09/25 16:25:19  olalu526
 * Added FutureBallView to SkillWithZone.
 *
 * Revision 1.2  2000/09/24 18:37:39  olalu526
 * Added ZoneDefence and changed ZoneDefenceToDefend to use SkillWithZone.
 *
 * Revision 1.1  2000/09/24 10:44:34  olalu526
 * Now ZoneDefenceToDefend works (had to remove a non woring file in Makefile.am, sorry.
 *
 * Revision 1.3  2000/09/23 17:21:15  olalu526
 * Did some changes.
 *
 * Revision 1.2  2000/09/22 19:10:48  olalu526
 * Generated new ZoneDefenceToDefend files and checked in Zone.h, Circle.h and CoordCircle.h needed by that.
 *
 */

/**
 * @file ZoneDefenceToDefend.cc
 * Implementation file for the skill class ZoneDefenceToDefend.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.14 $
 * $Author: frehe $
 * $Date: 2001/06/19 14:01:46 $
 */


#include "ZoneDefenceToDefend.h"
#include "math_utils.h"

RS_BEGIN_NAMESPACE


// --- CONSTRUCTORS ---

ZoneDefenceToDefend::ZoneDefenceToDefend(const Zone &z)
  : SkillWithZone(z),
    ballInZonePredicate(z),
    agentInZonePredicate(z),
    defender(z),
    positionate(z)
{
}

// --- DESTRUCTOR ---

ZoneDefenceToDefend::~ZoneDefenceToDefend()
{
}


// --- METHODS ---

void ZoneDefenceToDefend::setZone(Zone z) {
  SkillWithZone::setZone(z);
  defender.setZone(z);
  positionate.setZone(z);
}

Plan ZoneDefenceToDefend::generatePlan()
{
  // The following is exactly the same as in the positionating code in
  // positionate.
  static FindBall findball;

  if (findball.applicable()) {
    //std::cout << "FB!" << std::flush;
    return findball.generatePlan();
  }
  if (positionate.applicable()) {
    return positionate.generatePlan();
  }
  std::cout << "ZoneDefenceToDefend: This should NEVER happen!" << std::flush;
  
  return Plan();
}

FuzzyBool ZoneDefenceToDefend::persistent()
{
  // Never persistent.
  return FALSE;
}


FuzzyBool ZoneDefenceToDefend::applicable()
{
  // Allways applicable.
  return ((! ballInZonePredicate()) &&
	  (! agentInZonePredicate()) &&
	  (! defender.applicable()) &&
	  (needPositionatePredicate()));
}


RS_END_NAMESPACE


// Remove from here
/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

//#ifndef RS_USE_INLINE
//#  include "ZoneDefenceToDefend.icc"
//#endif
// To here if you do not have any inline definitions!
