/* -*- Mode: C++ -*-
 * $Id: ZoneDefenceToDefend.h,v 1.7 2000/10/11 07:34:02 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: ZoneDefenceToDefend.h
 * Unitname: Framework/skill
 * $Revision: 1.7 $
 * Created by: Fredrik Heintz 2000-09-22
 * Last modified by $Author: olalu526 $ $Date: 2000/10/11 07:34:02 $
 *
 *
 * HISTORY
 *
 * $Log: ZoneDefenceToDefend.h,v $
 * Revision 1.7  2000/10/11 07:34:02  olalu526
 * Added documentation and added setZone function that should be there.
 *
 * Revision 1.6  2000/10/01 13:05:57  olalu526
 * Updated FacingDirection, fixing bugs and stuff.
 *
 * Revision 1.5  2000/09/28 18:52:19  olalu526
 * Now the agent works quite fine.
 *
 * Revision 1.4  2000/09/28 16:21:11  olalu526
 * Changed SkillWithZone (and friends) to use a new predicate and view ...
 *
 * Revision 1.3  2000/09/25 16:25:19  olalu526
 * Added FutureBallView to SkillWithZone.
 *
 * Revision 1.2  2000/09/24 18:37:40  olalu526
 * Added ZoneDefence and changed ZoneDefenceToDefend to use SkillWithZone.
 *
 * Revision 1.1  2000/09/24 10:44:34  olalu526
 * Now ZoneDefenceToDefend works (had to remove a non woring file in Makefile.am, sorry.
 *
 * Revision 1.3  2000/09/23 17:21:16  olalu526
 * Did some changes.
 *
 * Revision 1.2  2000/09/22 19:10:48  olalu526
 * Generated new ZoneDefenceToDefend files and checked in Zone.h, Circle.h and CoordCircle.h needed by that.
 *
 */

/**
 * @file ZoneDefenceToDefend.h
 * Include file for the skill class GotoBall.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.7 $
 * $Author: olalu526 $
 * $Date: 2000/10/11 07:34:02 $
 */


#ifndef ZONEDEFENCETODEFEND_H
#define ZONEDEFENCETODEFEND_H


#include "SkillWithZone.h"
#include "GotoBall.h"
#include "BallInZone.h"
#include "AgentInZone.h"
#include "FindBall.h"

#include "InterceptBall.h"

#include "TurnCommand.h"
#include "TurnNeckCommand.h"
#include "DashCommand.h"
#include "ZoneDefenceDefender.h"
#include "ZoneDefencePositionate.h"


RS_BEGIN_NAMESPACE

class BallView;
class ParameterView;
class AgentView;
class MarkerView;

/**
 * @class ZoneDefenceToDefend ZoneDefenceToDefend.h
 * Run to a zone to defend.
 * ZoneDefenceToDefend is a skill to as quick and efficient as possible run
 * to a zone that it later should (be able to) defend.
 *
 * If the ball is directed in a good direction it is not emergency.
 * If the ball is directed in a bad direction it is not emergency.
 *
 * @pre The preconditions for the skill are:
 *	- The agent is outside the zone.
 *	- ZoneDefenceDefender is not applicable.
 * @post The intended postconditions for the skill are:
 *	- The agent is inside the zone.
 *
 * @todo There are some more things to do:
 *	- Just debugging.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.7 $
 * $Author: olalu526 $
 * $Date: 2000/10/11 07:34:02 $
 */

class ZoneDefenceToDefend : public SkillWithZone {
public:
  /** Default constructor. Here you can initialize the skill. */
  //ZoneDefenceToDefend();
  ZoneDefenceToDefend(const Zone &);

  /** Destructor should clean up after the skill. */
  ~ZoneDefenceToDefend();

  
  /** This method is used to generate a plan for how the goal of
    the skill should be obtained by the agent. You can assume
    that the preconditions of the skill are true (this can be tested
    by calling the applicable() method, after you have implemented it). */
  virtual Plan generatePlan();

  /** This method should return a FuzzyBool telling the caller
    if the skill wants to be called again. */
  virtual FuzzyBool persistent();

  /** This method should implement the precondition specified in
   the description of the skill. */
  virtual FuzzyBool applicable();

  virtual void setZone(Zone);
  
private:
  BallInZone   ballInZonePredicate;
  AgentInZone agentInZonePredicate;  
  ZoneDefenceDefender defender;
  ZoneDefencePositionate positionate;
};

RS_END_NAMESPACE


// Remove from here 
/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

//#ifdef RS_USE_INLINE
//#  include "ZoneDefenceToDefend.icc"
//#endif
// To here if you do not have any inline definitions!

#endif // !ZONEDEFENCETODEFEND_H
