/* -*- Mode: C++ -*-
 * $Id: FrameworkSensorInterface.cc,v 1.2 2000/09/13 00:04:15 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: FrameworkSensorInterface.cc
 * Unitname: Framework
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/13 00:04:15 $
 *
 *
 * HISTORY
 *
 * $Log: FrameworkSensorInterface.cc,v $
 * Revision 1.2  2000/09/13 00:04:15  frehe
 * Added forEach methods that takes a method from a View class instead of a global function as argument.
 *
 * Revision 1.1  2000/09/03 22:08:46  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file FrameworkSensorInterface.cc
 * Implementation for the class FrameworkSensorInterface.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2000/09/13 00:04:15 $
 */


#include "FrameworkSensorInterface.h"
#include "ViewManager.h"
#include "rs_error.h"


RS_BEGIN_NAMESPACE


FrameworkSensorInterface::FrameworkSensorInterface(const std::string& my_team_name)
  : SensorInterface(my_team_name),
    theViewManager(ViewManager::instance())
{
}


FrameworkSensorInterface::~FrameworkSensorInterface()
{
}


void FrameworkSensorInterface::updateBeforeNewCycle()
{
  theViewManager->newCycle();
  SensorInterface::updateBeforeNewCycle();
}


/*
UInt16 FrameworkSensorInterface::estimatedTimeToNextSenseBody() const
{
  return 100 - (getCurrentTimeMSec()-lastSenseBodyTime);
}

UInt16 FrameworkSensorInterface::estimatedTimeToNextSee() const
{
  return 150 - (getCurrentTimeMSec()-lastSeeTime);
}

UInt16 FrameworkSensorInterface::estimatedTimeToNextCycle() const
{
  return 100 - (getCurrentTimeMSec()-lastSenseBodyTime);
}

UInt16 FrameworkSensorInterface::estimatedTimeToNextAction() const
{
  return 70 - (getCurrentTimeMSec()-lastSenseBodyTime);
}
*/


void FrameworkSensorInterface::setLastSensorType(const SensorType sensor_type)
{
  //std::cerr << "In SetLastSensorType...\n";
  
  // This method is called *after* the whole server message is parsed.
  switch ( sensor_type ) {
  case SENSOR_AURAL:
    theViewManager->updateAfterHear();
    break;
  case SENSOR_VISUAL:
    theViewManager->updateAfterSee();
    break;
  case SENSOR_SENSE:
    theViewManager->updateAfterSense();
    break;
  case SENSOR_ACTUATORS:
    theViewManager->updateAfterCommand();
    break;
  case SENSOR_INIT:
    theViewManager->updateAfterInit();
    break;
  case SENSOR_COACH:
    theViewManager->updateAfterCoachMessage();
    break;
  default:
    RS_ERROR("Unknown sensor type in [FrameworkSensorInterface]:SetLastSensorType.");
  }
}


void FrameworkSensorInterface::forEachMarker(void (View::*fn)(const ServerMarkerObject&),
					     View* const view) const
{
  std::priority_queue<ServerMarkerObject> tmp_markers(markers);
  while ( !tmp_markers.empty() ) {
    (view->*fn)(tmp_markers.top());
    tmp_markers.pop();
  }
}


void FrameworkSensorInterface::forEachLine(void (View::*fn)(const ServerLineObject&),
					   View* const view) const
{
  std::priority_queue<ServerLineObject> tmp_lines(lines);
  while ( !tmp_lines.empty() ) {
    (view->*fn)(tmp_lines.top());
    tmp_lines.pop();
  }
}


void FrameworkSensorInterface::forEachPlayer(void (View::*fn)(const ServerPlayerObject&),
					     View* const view) const
{
  for ( std::vector<ServerPlayerObject>::const_iterator i = players.begin();
	i != players.end(); ++i )
    (view->*fn)(*i);
}


void FrameworkSensorInterface::forEachOpponent(void (View::*fn)(const CoachPlayerObject&),
					       View* const view) const
{
  for ( std::vector<CoachPlayerObject>::const_iterator i = opponents.begin();
	i != opponents.end(); ++i )
    (view->*fn)(*i);
}


void FrameworkSensorInterface::forEachTeammate(void (View::*fn)(const CoachPlayerObject&),
					       View* const view) const
{
  for ( std::vector<CoachPlayerObject>::const_iterator i = teammates.begin();
	i != teammates.end(); ++i )
    (view->*fn)(*i);
}


RS_END_NAMESPACE
