/* -*- Mode: C++ -*-
 * $Id: GameView.cc,v 1.3 2001/01/11 22:00:20 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: GameView.cc
 * Unitname: Framework
 * $Revision: 1.3 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: olalu526 $ $Date: 2001/01/11 22:00:20 $
 *
 *
 * HISTORY
 *
 * $Log: GameView.cc,v $
 * Revision 1.3  2001/01/11 22:00:20  olalu526
 * Added the real world model with support for new simulation steps.
 *
 * Revision 1.2  2001/01/10 15:12:59  olalu526
 * Added support for WorldModel to some basic classes.
 *
 * Revision 1.1  2000/09/21 13:46:46  frehe
 * Move the RoboSoc views from rsviews to the rsframework library.
 *
 * Revision 1.2  2000/09/13 00:08:08  frehe
 * Updated the views to inherit from the ViewWithHistory instead and also changed those views that used a static pointer to itself to do process the data in the sensor interface to use the new forEach methods implemented in the FrameworkSensorInterface.
 *
 * Revision 1.1  2000/09/04 15:27:23  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file GameView.cc
 * Implementation file for the class GameView.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.3 $
 * $Author: olalu526 $
 * $Date: 2001/01/11 22:00:20 $
 */


#include "GameView.h"


RS_BEGIN_NAMESPACE


GameView::GameView(const ViewId id,
		   UInt16 hist_len)
  : ViewWithSimulation(id)
{
  worldModel->setPlayModeHistoryLength(hist_len);
}


GameView::~GameView()
{
}
   

void GameView::myNewCycle()
{
  // Update the current observation.
  increaseIndex();
  
  // Copy the play mode
  playModes[getIndex(NOW)] = playModes[getIndex(PREV)];
}


void GameView::myUpdateAfterHear()
{
  // Save the current play mode.
  playModes[getIndex(NOW)] = sensorInterface->getPlayMode();
}


void GameView::myUpdateAfterInit()
{
  // Set the play mode to the initial play mode.
  playModes[getIndex(NOW)] = sensorInterface->getPlayMode();
}


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "GameView.icc"
#endif
