/* -*- Mode: C++ -*-
 * $Id: ParameterView.cc,v 1.1 2000/09/21 13:46:48 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: ParameterView.cc
 * Unitname: Framework/view
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/21 13:46:48 $
 *
 *
 * HISTORY
 *
 * $Log: ParameterView.cc,v $
 * Revision 1.1  2000/09/21 13:46:48  frehe
 * Move the RoboSoc views from rsviews to the rsframework library.
 *
 * Revision 1.1  2000/09/04 15:27:25  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file ParameterView.cc
 * Implementation file for the class ParameterView.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/21 13:46:48 $
 */


#include <cstdio>

#include "ParameterView.h"
#include "rs_error.h"
#include "rs_assert.h"


RS_BEGIN_NAMESPACE


ParameterView::ParameterView(const ViewId id)
  : View(id)
{
  parameterInit();
  initOptions();
}


ParameterView::ParameterView(const ViewId id,
			     int argc, char** argv, int verbose)
  : View(id)
{
  parameterInit();
  initOptions();
  readCommandLine(argc, argv, verbose);
}


ParameterView::~ParameterView()
{
}


void ParameterView::parameterInit()
{
  // Special option
  CP_team_name                  = "RoboSoc";
  CP_goalie                     = false;
  CP_cycles_since_io_before_shutdown  = 10;
  CP_min_ticks_before_sending_command = 2;
  CP_ticks_before_command_warning     = 5;
  CP_max_ticks_before_sending_command = 9;
  
  // no option flags for these
  SP_pitch_length    		= 105.0;
  SP_pitch_width 	 	= 68.0;
  SP_pitch_margin	 	= 5.0;
  SP_penalty_area_length 	= 16.5;
  SP_penalty_area_width		= 40.32;
  SP_goal_area_length		= 5.5;
  SP_goal_area_width		= 18.32;
  SP_penalty_spot_dist		= 11.0;
  SP_corner_arc_r		= 1.0;
  SP_free_kick_buffer		= 9.15;
  SP_after_goal_wait  		= 50;
  SP_feel_distance 		= 3.0;
  SP_num_lines     		= 4;
  SP_num_markers   		= 55;
  SP_unum_far_length		= 20.0;
  SP_unum_too_far_length	= 40.0;
  SP_team_far_length		= 40.0;
  SP_team_too_far_length	= 60.0;
  SP_version	 		= 5.06;
  SP_team_size     		= 11;
  SP_half	    		= 1;        
  SP_host                       = "localhost";
  SP_buffer_size                = 2048;
  
  // server.conf starts here        
  SP_goal_width    		= 14.02;       
  SP_player_size   		= 0.3;
  SP_player_decay  		= 0.4;
  SP_player_rand 		= 0.1;
  SP_player_weight		= 60.0;
  SP_player_speed_max		= 1.0;
  SP_stamina_min                = 0.0;
  SP_stamina_max                = 3500.0;
  SP_stamina_inc_max		= 20.0;
  SP_recover_dec_thr		= 0.3;
  SP_recover_dec   		= 0.002;
  SP_recover_min   		= 0.5;
  SP_recover_max   		= 1.0;
  SP_effort_dec_thr		= 0.3;
  SP_effort_dec    		= 0.005;	
  SP_effort_inc_thr		= 0.6;	
  SP_effort_inc    		= 0.01;
  SP_effort_min    		= 0.6;
  SP_effort_max    		= 1.0;
  SP_hear_max                   = 2;
  SP_hear_inc                   = 1;
  SP_hear_decay    		= 2;
  SP_inertia_moment		= 5.0;
  SP_sense_body_step            = 100;
  SP_catchable_area_l  		= 2.0;	
  SP_catchable_area_w  		= 1.0;	
  SP_catch_probability		= 1.0;
  SP_catch_ban_cycle		= 5;        
  SP_ball_size     		= 0.085;		
  SP_ball_decay    		= 0.94;
  SP_ball_rand     		= 0.05;		
  SP_ball_weight   		= 0.2;
  SP_ball_speed_max		= 2.7;
  SP_wind_dir		        = 0.0;	
  SP_wind_force    		= 0.0;
  SP_wind_rand     		= 0.0;	
  SP_wind_none     		= false;
  SP_wind_random   		= false;
  SP_kickable_margin		= 0.7;	
  SP_control_radius             = 2.0;
  SP_ckick_margin               = 1.0;
  SP_dash_power_rate		= 0.006;        
  SP_kick_power_rate		= 0.016;
  SP_visible_angle 		= 90.0;
  SP_visible_distance           = 3.0;
  SP_audio_cut_dist		= 50.0;	
  SP_quantize_step    		= 0.1;
  SP_quantize_step_l    	= 0.01;
  SP_maxpower     		= 100;
  SP_minpower     		= -100;
  SP_maxmoment    		= 180;
  SP_minmoment    		= -180;
  SP_maxneckang                 = 90;
  SP_minneckang                 = -90;
  SP_maxneckmoment              = 180;
  SP_minneckmoment              = -180;
  SP_port	    		= 6000;
  SP_coach_port    		= 6001;
  SP_olcoach_port               = 6002;
  SP_say_coach_cnt_max          = 128;
  SP_say_coach_msg_size         = 128;
  SP_send_vi_step               = 100;
  SP_simulator_step		= 100;
  SP_send_step     		= 150;
  SP_recv_step     		= 10;
  SP_half_time     		= 300;
  SP_say_msg_size  		= 512;
  SP_use_offside   		= true;
  SP_offside_active_area_size   = 9.15;
  SP_forbid_kick_off_offside    = true;
  SP_verbose	 		= false;
  SP_record_version             = 2;
  SP_record_log                 = false;
  SP_send_log                   = true;
  SP_log_file                   = "";
  SP_record                     = "";
  SP_replay                     = "";
  SP_coach                      = false;
  SP_coach_w_referee            = false;
  SP_old_coach_hear             = false;

  SP_kickable_area = 1.0; //SP_kickable_margin + SP_ball_size + SP_player_size;
}


void ParameterView::initOptions()
{
  // variable types and names
  OptionType opt[] =
  {{ "team_name",                 (void *) &CP_team_name,                  V_STRING },
   { "goalie",                    (void *) &CP_goalie,                     V_BOOL },
   { "cycles_since_io_before_shutdown", (void *) &CP_cycles_since_io_before_shutdown, V_UINT16 },
   { "min_ticks_before_sending_command", (void *) &CP_min_ticks_before_sending_command, V_UINT16 },
   { "ticks_before_command_warning", (void *) &CP_ticks_before_command_warning, V_UINT16 },
   { "max_ticks_before_sending_command", (void *) &CP_max_ticks_before_sending_command, V_UINT16 },

   { "pitch_length",              (void *) &SP_pitch_length,               V_FLOAT32 },
   { "pitch_width",               (void *) &SP_pitch_width,                V_FLOAT32 },
   { "pitch_margin",              (void *) &SP_pitch_margin,               V_FLOAT32 },
   { "penalty_area_length",       (void *) &SP_penalty_area_length,        V_FLOAT32 },
   { "penalty_area_width",        (void *) &SP_penalty_area_width,         V_FLOAT32 },
   { "goal_area_length",          (void *) &SP_goal_area_length,           V_FLOAT32 },
   { "goal_area_width",           (void *) &SP_goal_area_width,            V_FLOAT32 },
   { "penalty_spot_dist",         (void *) &SP_penalty_spot_dist,          V_FLOAT32 },
   { "corner_arc_r",              (void *) &SP_corner_arc_r,               V_FLOAT32 },
   { "free_kick_buffer",          (void *) &SP_free_kick_buffer,           V_FLOAT32 },
   { "after_goal_wait",           (void *) &SP_after_goal_wait,            V_INT32 },
   { "feel_distance",             (void *) &SP_feel_distance,              V_FLOAT32 },
   { "num_lines",                 (void *) &SP_num_lines,                  V_INT32 },
   { "num_markers",               (void *) &SP_num_markers,                V_INT32 },
   { "unum_far_length",           (void *) &SP_unum_far_length,            V_FLOAT32 },
   { "unum_too_far_length",       (void *) &SP_unum_too_far_length,        V_FLOAT32 },
   { "team_far_length",           (void *) &SP_team_far_length,            V_FLOAT32 },
   { "team_too_far_length",       (void *) &SP_team_too_far_length,        V_FLOAT32 },
   { "version",                   (void *) &SP_version,                    V_FLOAT32 },
   { "team_size",                 (void *) &SP_team_size,                  V_INT32 },
   { "half",                      (void *) &SP_half,                       V_INT32 },
   { "host",                      (void *) &SP_host,                       V_STRING },
   { "buffer_size",               (void *) &SP_buffer_size,                V_INT32 },
   { "goal_width",                (void *) &SP_goal_width,                 V_FLOAT32 },
   { "player_size",               (void *) &SP_player_size,                V_FLOAT32 },
   { "player_decay",              (void *) &SP_player_decay,               V_FLOAT32 },
   { "player_rand",               (void *) &SP_player_rand,                V_FLOAT32 },
   { "player_weight",             (void *) &SP_player_weight,              V_FLOAT32 },
   { "player_speed_max",          (void *) &SP_player_speed_max,           V_FLOAT32 },
   { "stamina_min",               (void *) &SP_stamina_min,                V_FLOAT32 },
   { "stamina_max",               (void *) &SP_stamina_max,                V_FLOAT32 },
   { "stamina_inc_max",           (void *) &SP_stamina_inc_max,            V_FLOAT32 },
   { "recover_dec_thr",           (void *) &SP_recover_dec_thr,            V_FLOAT32 },
   { "recover_dec",               (void *) &SP_recover_dec,                V_FLOAT32 },
   { "recover_min",               (void *) &SP_recover_min,                V_FLOAT32 },
   { "recover_max",               (void *) &SP_recover_max,                V_FLOAT32 },
   { "effort_dec_thr",            (void *) &SP_effort_dec_thr,             V_FLOAT32 },
   { "effort_dec",                (void *) &SP_effort_dec,                 V_FLOAT32 },
   { "effort_inc_thr",            (void *) &SP_effort_inc_thr,             V_FLOAT32 },
   { "effort_inc",                (void *) &SP_effort_inc,                 V_FLOAT32 },
   { "effort_min",                (void *) &SP_effort_min,                 V_FLOAT32 },
   { "effort_max",                (void *) &SP_effort_max,                 V_FLOAT32 },
   { "hear_max",                  (void *) &SP_hear_max,                   V_INT32 },
   { "hear_inc",                  (void *) &SP_hear_inc,                   V_INT32 },
   { "hear_decay",                (void *) &SP_hear_decay,                 V_INT32 },
   { "inertia_moment",            (void *) &SP_inertia_moment,             V_FLOAT32 },
   { "sense_body_step",           (void *) &SP_sense_body_step,            V_INT32 },
 
   { "catchable_area_l",          (void *) &SP_catchable_area_l,           V_FLOAT32 },
   { "catchable_area_w",          (void *) &SP_catchable_area_w,           V_FLOAT32 },
   { "catch_probability",         (void *) &SP_catch_probability,          V_FLOAT32 },
   { "catch_ban_cycle",           (void *) &SP_catch_ban_cycle,            V_INT32 },
 
   { "ball_size",                 (void *) &SP_ball_size,                  V_FLOAT32 },
   { "ball_decay",                (void *) &SP_ball_decay,                 V_FLOAT32 },
   { "ball_rand",                 (void *) &SP_ball_rand,                  V_FLOAT32 },
   { "ball_weight",               (void *) &SP_ball_weight,                V_FLOAT32 },
   { "ball_speed_max",            (void *) &SP_ball_speed_max,             V_FLOAT32 },
 
   { "wind_force",                (void *) &SP_wind_force,                 V_FLOAT32 },
   { "wind_dir",                  (void *) &SP_wind_dir,                   V_FLOAT32 },
   { "wind_rand",                 (void *) &SP_wind_rand,                  V_FLOAT32 },
   { "wind_none",                 (void *) &SP_wind_none,                  V_BOOL },
   { "wind_random",               (void *) &SP_wind_random,                V_BOOL },
   { "kickable_margin",           (void *) &SP_kickable_margin,            V_FLOAT32 },
   { "control_radius",            (void *) &SP_control_radius,             V_FLOAT32 },
   { "ckick_margin",              (void *) &SP_ckick_margin,               V_FLOAT32 },
   { "dash_power_rate",           (void *) &SP_dash_power_rate,            V_FLOAT32 },
   { "kick_power_rate",           (void *) &SP_kick_power_rate,            V_FLOAT32 },
   { "visible_angle",             (void *) &SP_visible_angle,              V_FLOAT32 },
   { "visible_distance",          (void *) &SP_visible_distance,           V_FLOAT32 },
   { "audio_cut_dist",            (void *) &SP_audio_cut_dist,             V_FLOAT32 },
   { "quantize_step",             (void *) &SP_quantize_step,              V_FLOAT32 },
   { "quantize_step_l",           (void *) &SP_quantize_step_l,            V_FLOAT32 },
   { "maxpower",                  (void *) &SP_maxpower,                   V_FLOAT32 },
   { "minpower",                  (void *) &SP_minpower,                   V_FLOAT32 },
   { "maxmoment",                 (void *) &SP_maxmoment,                  V_FLOAT32 },
   { "minmoment",                 (void *) &SP_minmoment,                  V_FLOAT32 },
   { "maxneckang",                (void *) &SP_maxneckang,                 V_FLOAT32 },
   { "minneckang",                (void *) &SP_minneckang,                 V_FLOAT32 },
   { "maxneckmoment",             (void *) &SP_maxneckmoment,              V_FLOAT32 },
   { "minneckmoment",             (void *) &SP_minneckmoment,              V_FLOAT32 },
   { "port",                      (void *) &SP_port,                       V_INT32 },
   { "coach_port",                (void *) &SP_coach_port,                 V_INT32 },
   { "olcoach_port",              (void *) &SP_olcoach_port,               V_INT32 },
   { "say_coach_cnt_max",         (void *) &SP_say_coach_cnt_max,          V_INT32 },
   { "say_coach_msg_size",        (void *) &SP_say_coach_msg_size,         V_INT32 },
   { "send_vi_step",              (void *) &SP_send_vi_step,               V_INT32 },
   { "simulator_step",            (void *) &SP_simulator_step,             V_INT32 },
   { "send_step",                 (void *) &SP_send_step,                  V_INT32 },
   { "recv_step",                 (void *) &SP_recv_step,                  V_INT32 },
   { "half_time",                 (void *) &SP_half_time,                  V_INT32 },
   { "say_msg_size",              (void *) &SP_say_msg_size,               V_INT32 },
   { "use_offside",               (void *) &SP_use_offside,                V_ONOFF },
   { "offside_active_area_size",  (void *) &SP_offside_active_area_size,   V_FLOAT32 },
   { "forbid_kick_off_offside",   (void *) &SP_forbid_kick_off_offside,    V_ONOFF },
   { "verbose",                   (void *) &SP_verbose,                    V_ONOFF },
   { "record_version",            (void *) &SP_record_version,             V_INT32 },
   { "record_log",                (void *) &SP_record_log,                 V_BOOL },
   { "send_log",                  (void *) &SP_send_log,                   V_BOOL },
   { "log_file",                  (void *) &SP_log_file,                   V_STRING },
   { "record",                    (void *) &SP_record,                     V_STRING },
   { "replay",                    (void *) &SP_replay,                     V_STRING },
 
   { "coach",                     (void *) &SP_coach,                      V_BOOL },
   { "coach_w_referee",           (void *) &SP_coach_w_referee,            V_BOOL },
   { "old_coach_hear",            (void *) &SP_old_coach_hear,             V_BOOL },
   // server.conf ends here

   { "kickable_area",             (void *) &SP_kickable_area,              V_FLOAT32 },
   { "\000", 0, V_NONE }
  };

  int i = 0;
  while (opt[i].vptr != 0 && i < optsize) {
    strcpy(options[i].name, opt[i].name);
    options[i].vptr = opt[i].vptr;
    options[i].vsize = opt[i].vsize;
    ++i;
  }

  RS_ASSERT(opt[i].vptr == 0, "Option array too small!");
  
  i = (i < optsize - 1) ? i : optsize - 1;
  options[i].vptr = 0;
}


void ParameterView::readCommandLine(int argc, char** argv, int verbose)
{
  // skip command name
  argv++ ; argc-- ;

  // first, search for option '-f' that's read a configuration file
  for(int i = 0 ; i < argc ; ++i) {
    if (!strcmp(*(argv + i),"-f")) {
      readOptions(*(argv+i+1), verbose);
    }
  }

  // next, analyze command line option
  while (argc) {
    if (!strcmp(*argv, "-f")) {
      argv += 2 ;
      argc -= 2 ;
      continue ;
    }

    if ( readOption(*argv + 1, *(argv + 1)) ) {
      argv += 2 ;
      argc -= 2 ;
    } else {
      argv++ ;
      argc-- ;
    }
  }
}


void ParameterView::readOptions(const std::string& filename, int verbose)
{

  FILE *file;

  file = fopen(filename.c_str(), "r");

  if (file == NULL) {
    if (verbose > 0) {
      fprintf(stderr, "can't open (%s)\n",
	      filename.c_str());
	}
    else {
      RS_WARNING("Can't open file " + filename + " in ParameterView::readOptions!");
    }
	  
    return;
  }

  static char linebuf[256];
  char *t;
  int i = 0;

  while (fgets(linebuf, 256, file) != 0) {
    ++i;
    if (verbose == 1 && i % 10 == 0)
      fprintf(stderr, ".");
                
    if (verbose > 2) {
      fprintf(stderr,linebuf);
      fflush(stderr);
    }

    if (linebuf[0] == '#'
	|| linebuf[0] == '\n'
	|| linebuf[0] == '\0')
      continue;

    t = linebuf;
    // replace the ':' by space
    while (*t != '\0') {
      if (*t == ':') *t = ' ';
      t++;
    }
    readOption(linebuf);
    
  }
}


bool ParameterView::readOption(const char* linebuf, int verbose, const OptionType* opts)
{
  int n;
  static char com[64];
  static char val[64];

  n = sscanf(linebuf,"%s %s", com, val) ;

  if (n < 1) {
    if (verbose > 0) {
      fprintf(stderr, "Read illegal line:  (%s)\n", linebuf);
    }
    else {
      RS_WARNING("Read illegal line: " + std::string(linebuf) + " in ParameterView::readOption!");
      return false;
    }
  }

  return readOption(com, val, opts);

}


bool ParameterView::readOption(const char* option, const char* value, const OptionType* opts)
{
  int p;
  for (p = 0 ; opts[p].vptr != NULL ; p++) {
    if (strcmp(option, opts[p].name))
      continue ;

    switch(opts[p].vsize) {
    case V_UINT16:
      *((UInt16 *)opts[p].vptr) = atoi(value) ;
      break ;

    case V_INT32:
      *((Int32 *)opts[p].vptr) = atoi(value) ;
      break ;

    case V_STRING:
      *((std::string *)opts[p].vptr) = value ;
      break ;

    case V_FLOAT32:
      *((Float32 *)opts[p].vptr) = atof(value) ;
      break ;

    case V_BOOL:
      *((bool *)opts[p].vptr) = (!strcmp(value, "false")) ? false : true ;
      break ;

    case V_ONOFF:
      *((bool *)opts[p].vptr) = (!strcmp(value, "on")) ? true : false ;
      break ;
    case V_NONE:
      break ;
    }
    break ;
  }

  if (opts[p].vptr == NULL) {
    RS_ERROR("Unrecognized option : " + std::string(option)
	     + " in ParameterView::readOption");
    return false;
  }

  return true;
}


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "ParameterView.icc"
#endif
