/* -*- Mode: C++ -*-
 * $Id: View.cc,v 1.2 2000/09/13 00:05:00 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: View.cc
 * Unitname: Framework
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/13 00:05:00 $
 *
 *
 * HISTORY
 *
 * $Log: View.cc,v $
 * Revision 1.2  2000/09/13 00:05:00  frehe
 * Added process-methods that corresponds to the forEach methods in the FrameworkSensorInterface.
 *
 * Revision 1.1  2000/09/03 22:08:47  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file View.cc
 * Implementation file for the class View.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2000/09/13 00:05:00 $
 */


#include "View.h"
#include "ViewManager.h"
#include "FrameworkSensorInterface.h"
#include "rs_error.h"


RS_BEGIN_NAMESPACE


const FrameworkSensorInterface* View::sensorInterface = NULL;


View::View(const ViewId id)
  : viewManager(ViewManager::instance()),
    initialized(false),
    id(id),
    lastUpdateNumber(0),
    updateBegun(false)
{
}


View::~View()
{
}


ViewUpdateResult View::newCycle(const UInt32 update_no)
{
  if ( updateBegun ) {
    RS_FATAL("Circular updates in the views!!!");
    return UPDATE_ERROR;
  } else if ( update_no > lastUpdateNumber ) {
    currentUpdateNumber = update_no;
    updateBegun = true;
    myNewCycle();
    updateBegun = false;
    if ( update_no - lastUpdateNumber > 0 ) {
      lastUpdateNumber = update_no;
      return UPDATE_MISSING_UPDATES;
    } else {
      lastUpdateNumber = update_no;
      return UPDATE_OK;
    }
  } else
    return UPDATE_ALREADY_MADE;
}


ViewUpdateResult View::updateAfterCoachMessage(const UInt32 update_no)
{
  if ( updateBegun ) {
    RS_FATAL("[UpdateAfterCoachMessage] Circular updates in the views!!!");
    return UPDATE_ERROR;
  } else if ( update_no > lastUpdateNumber ) {
    currentUpdateNumber = update_no;
    updateBegun = true;
    myUpdateAfterCoachMessage();
    updateBegun = false;
    if ( update_no - lastUpdateNumber > 0 ) {
      lastUpdateNumber = update_no;
      return UPDATE_MISSING_UPDATES;
    } else {
      lastUpdateNumber = update_no;
      return UPDATE_OK;
    }
  } else
    return UPDATE_ALREADY_MADE;
}


ViewUpdateResult View::updateAfterInit(const UInt32 update_no)
{
  if ( updateBegun ) {
    RS_FATAL("[UpdateAfterInit] Circular updates in the views!!!");
    return UPDATE_ERROR;
  } else if ( update_no > lastUpdateNumber ) {
    currentUpdateNumber = update_no;
    updateBegun = true;
    myUpdateAfterInit();
    updateBegun = false;
    if ( update_no - lastUpdateNumber > 0 ) {
      lastUpdateNumber = update_no;
      return UPDATE_MISSING_UPDATES;
    } else {
      lastUpdateNumber = update_no;
      return UPDATE_OK;
    }
  } else
    return UPDATE_ALREADY_MADE;
}


ViewUpdateResult View::updateAfterSee(const UInt32 update_no)
{
  if ( updateBegun ) {
    RS_FATAL("[UpdateAfterSee] Circular updates in the views!!!");
    return UPDATE_ERROR;
  } else if ( update_no > lastUpdateNumber ) {
    currentUpdateNumber = update_no;
    updateBegun = true;
    myUpdateAfterSee();
    updateBegun = false;
    if ( update_no - lastUpdateNumber > 0 ) {
      lastUpdateNumber = update_no;
      return UPDATE_MISSING_UPDATES;
    } else {
      lastUpdateNumber = update_no;
      return UPDATE_OK;
    }
  } else
    return UPDATE_ALREADY_MADE;
}


ViewUpdateResult View::updateAfterHear(const UInt32 update_no)
{
  if ( updateBegun ) {
    RS_FATAL("[UpdateAfterHear] Circular updates in the views!!!");
    return UPDATE_ERROR;
  } else if ( update_no > lastUpdateNumber ) {
    currentUpdateNumber = update_no;
    updateBegun = true;
    myUpdateAfterHear();
    updateBegun = false;
    if ( update_no - lastUpdateNumber > 0 ) {
      lastUpdateNumber = update_no;
      return UPDATE_MISSING_UPDATES;
    } else {
      lastUpdateNumber = update_no;
      return UPDATE_OK;
    }
  } else
    return UPDATE_ALREADY_MADE;
}


ViewUpdateResult View::updateAfterSense(const UInt32 update_no)
{
  if ( updateBegun ) {
    RS_FATAL("[UpdateAfterSense] Circular updates in the views!!!");
    return UPDATE_ERROR;
  } else if ( update_no > lastUpdateNumber ) {
    currentUpdateNumber = update_no;
    updateBegun = true;
    myUpdateAfterSense();
    updateBegun = false;
    if ( update_no - lastUpdateNumber > 0 ) {
      lastUpdateNumber = update_no;
      return UPDATE_MISSING_UPDATES;
    } else {
      lastUpdateNumber = update_no;
      return UPDATE_OK;
    }
  } else
    return UPDATE_ALREADY_MADE;
}


ViewUpdateResult View::updateAfterCommand(const UInt32 update_no)
{
  if ( updateBegun ) {
    RS_FATAL("[UpdateAfterCommand] Circular updates in the views!!!");
    return UPDATE_ERROR;
  } else if ( update_no > lastUpdateNumber ) {
    currentUpdateNumber = update_no;
    updateBegun = true;
    myUpdateAfterCommand();
    updateBegun = false;
    if ( update_no - lastUpdateNumber > 0 ) {
      lastUpdateNumber = update_no;
      return UPDATE_MISSING_UPDATES;
    } else {
      lastUpdateNumber = update_no;
      return UPDATE_OK;
    }
  } else
    return UPDATE_ALREADY_MADE;
}


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "View.icc"
#endif
