/* -*- Mode: C++ -*-
 * $Id: ViewManager.cc,v 1.1 2000/09/03 22:08:47 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: ViewManager.cc
 * Unitname: Framework
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/03 22:08:47 $
 *
 *
 * HISTORY
 *
 * $Log: ViewManager.cc,v $
 * Revision 1.1  2000/09/03 22:08:47  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file ViewManager.cc
 * Implementation file for the class ViewManager.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/03 22:08:47 $
 */


#include "ViewManager.h"


RS_BEGIN_NAMESPACE


ViewManager* ViewManager::onlyInstance(NULL);


ViewManager* ViewManager::instance()
{
  if ( onlyInstance == NULL ) {
    onlyInstance = new ViewManager();
  }
  return onlyInstance;
}


bool ViewManager::addView(View* view, const bool persistent)
{
  // AddView returns true if the view was added, otherwise false.
  
  std::map<ViewId, ViewData>::value_type item(view->getId(),
					      ViewData(view, persistent));

  std::pair<std::map<ViewId, ViewData>::iterator, bool> r;
  r = theViews.insert(item);

  return r.second;

  // Returns a pointer to the view with the same id as the
  // view supplied.
  // return theViews.insert(std::map<ViewId, ViewData>::value_type(view->GetID(), ViewData(view, persistent))).first->second.GetViewPointer();

}  


View* ViewManager::getView(const ViewId v_id)
{
  // Returns a pointer to the wanted view or NULL if the
  // view doesn't exist.
  // Also increment the reference counter.
  
  std::map<ViewId, ViewData>::iterator r = theViews.find(v_id);

  if ( r != theViews.end() ) {
    r->second.increaseReferenceCount();
    return r->second.getViewPointer();
  } else {
    return NULL;
  }
  
}


void ViewManager::releaseView(const ViewId v_id)
{
  // Decrement reference count if the view exist.
  // If the view is no longer in use delete it.

  std::map<ViewId, ViewData>::iterator r = theViews.find(v_id);

  if ( r != theViews.end() ) {
    r->second.decreaseReferenceCount();

    if ( !r->second.isUsed() )
      theViews.erase(r);
  }

}

void ViewManager::setViewPersistence(ViewId v_id, const bool persistence)
{
  // If the view exist change its persistence.
  std::map<ViewId, ViewData>::iterator r = theViews.find(v_id);

  if ( r != theViews.end() ) {
    r->second.setPersistence(persistence);

    if ( !r->second.isUsed() )
      theViews.erase(r);
  }
}
  

ViewManager::ViewManager()
  : theViews(),
    currentUpdateNumber(0)
{
}


void ViewManager::forEach(void (*fn)(View*))
{
  // Apply fn to all views.
  for ( std::map<ViewId, ViewData>::iterator i = theViews.begin();
	i != theViews.end(); ++i )
    fn(i->second.getViewPointer());
}


void ViewManager::forEach(ViewUpdateResult (View::*fn)(const UInt32),
			   const UInt32 val)
{
  // Apply fn to all views.
  for ( std::map<ViewId, ViewData>::iterator i = theViews.begin();
	i != theViews.end(); ++i )
    (i->second.getViewPointer()->*fn)(val);
}


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "ViewManager.icc"
#endif
