/* -*- Mode: C++ -*-
 * $Id: WorldModelInterface.cc,v 1.3 2001/01/12 12:40:34 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *                     2001 Ola Lundqvist, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: WorldModelInterface.cc
 * Unitname: Framework
 * $Revision: 1.3 $
 * Created by: Ola Lundqvist 2001-01-02
 * Last modified by $Author: olalu526 $ $Date: 2001/01/12 12:40:34 $
 *
 *
 * HISTORY
 *
 * $Log: WorldModelInterface.cc,v $
 * Revision 1.3  2001/01/12 12:40:34  olalu526
 * Added some documentation.
 *
 *
 */

/**
 * @file WorldModelInterface.cc
 * Implementation file for the class WorldModelInterface
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.3 $
 * $Author: olalu526 $
 * $Date: 2001/01/12 12:40:34 $
 */

#include "WorldModelInterface.h"

RS_BEGIN_NAMESPACE

WorldModelInterface::WorldModelInterface() :
  parameterView(NULL)
{
  getView(parameterView, PARAMETER_VIEW_ID);
  worldModel = new WorldModel(parameterView);
}

WorldModelInterface::~WorldModelInterface() {
  ViewManager::instance()->releaseView(PARAMETER_VIEW_ID);
  delete worldModel;
}

WorldModelInterface::WorldModelInterface(WorldModel* wm) :
  parameterView(NULL)
{
  getView(parameterView, PARAMETER_VIEW_ID);
  worldModel = wm;
}

WorldModelInterface* WorldModelInterface::onlyInstance(0);

WorldModelInterface* WorldModelInterface::instance() {
  if (onlyInstance == 0) {
    onlyInstance = new WorldModelInterface;
  }
  return onlyInstance;
}

void WorldModelInterface::setWorldModel(WorldModel* wm) {
  worldModel = wm;
}

WorldModel* WorldModelInterface::getWorldModel() {
  return worldModel;
}

/* ===== */


PlayModeCollection* WorldModelInterface::playModeData() {
  return worldModel->playModeData();
}

UInt WorldModelInterface::getPlayModeIndex(const int i) const {
  return worldModel->getPlayModeIndex(i);
}

void WorldModelInterface::setPlayModeHistoryLength(const int i) {
  worldModel->setPlayModeHistoryLength(i);
}

UInt WorldModelInterface::getPlayModeHistoryLength() {
  return worldModel->getPlayModeHistoryLength();
}

void WorldModelInterface::increasePlayModeIndex() {
  worldModel->increasePlayModeIndex();
}

/* ===== */

BallCollection* WorldModelInterface::ballData() {
  return worldModel->ballData();
}

UInt WorldModelInterface::getBallIndex(const int i) const {
  return worldModel->getBallIndex(i);
}

void WorldModelInterface::setBallHistoryLength(const int i) {
  worldModel->setBallHistoryLength(i);
}

UInt WorldModelInterface::getBallHistoryLength() {
  return worldModel->getBallHistoryLength();
}

void WorldModelInterface::increaseBallIndex() {
  worldModel->increaseBallIndex();
}

/* ===== */

PlayersCollection* WorldModelInterface::opponentsData() {
  return worldModel->opponentsData();
}

UInt WorldModelInterface::getOpponentsIndex(const int i) const {
  return worldModel->getOpponentsIndex(i);
}

void WorldModelInterface::setOpponentsHistoryLength(const int i) {
  worldModel->setOpponentsHistoryLength(i);
}

UInt WorldModelInterface::getOpponentsHistoryLength() {
  return worldModel->getOpponentsHistoryLength();
}

void WorldModelInterface::increaseOpponentsIndex() {
  worldModel->increaseOpponentsIndex();
}

/* ===== */

PlayersCollection* WorldModelInterface::teammatesData() {
  return worldModel->teammatesData();
}

UInt WorldModelInterface::getTeammatesIndex(const int i) const {
  return worldModel->getTeammatesIndex(i);
}

void WorldModelInterface::setTeammatesHistoryLength(const int i) {
  worldModel->setTeammatesHistoryLength(i);
}

UInt WorldModelInterface::getTeammatesHistoryLength() {
  return worldModel->getTeammatesHistoryLength();
}

void WorldModelInterface::increaseTeammatesIndex() {
  worldModel->increaseTeammatesIndex();
}

/* ===== */

PlayersCollection* WorldModelInterface::unknownPlayersData() {
  return worldModel->unknownPlayersData();
}

UInt WorldModelInterface::getUnknownPlayersIndex(const int i) const {
  return worldModel->getUnknownPlayersIndex(i);
}

void WorldModelInterface::setUnknownPlayersHistoryLength(const int i) {
  worldModel->setUnknownPlayersHistoryLength(i);
}

UInt WorldModelInterface::getUnknownPlayersHistoryLength() {
  return worldModel->getUnknownPlayersHistoryLength();
}

void WorldModelInterface::increaseUnknownPlayersIndex() {
  worldModel->increaseUnknownPlayersIndex();
}

/* ===== */

UIntUnknownCollection* WorldModelInterface::closestOpponentIndexData() {
  return worldModel->closestOpponentIndexData();
}

UInt WorldModelInterface::getClosestOpponentIndexIndex(const int i) const {
  return worldModel->getClosestOpponentIndexIndex(i);
}

void WorldModelInterface::setClosestOpponentIndexHistoryLength(const int i) {
  worldModel->setClosestOpponentIndexHistoryLength(i);
}

UInt WorldModelInterface::getClosestOpponentIndexHistoryLength() {
  return worldModel->getClosestOpponentIndexHistoryLength();
}

void WorldModelInterface::increaseClosestOpponentIndexIndex() {
  worldModel->increaseClosestOpponentIndexIndex();
}

/* ===== */

UIntUnknownCollection* WorldModelInterface::closestTeammateIndexData() {
  return worldModel->closestTeammateIndexData();
}

UInt WorldModelInterface::getClosestTeammateIndexIndex(const int i) const {
  return worldModel->getClosestTeammateIndexIndex(i);
}

void WorldModelInterface::setClosestTeammateIndexHistoryLength(const int i) {
  worldModel->setClosestTeammateIndexHistoryLength(i);
}

UInt WorldModelInterface::getClosestTeammateIndexHistoryLength() {
  return worldModel->getClosestTeammateIndexHistoryLength();
}

void WorldModelInterface::increaseClosestTeammateIndexIndex() {
  worldModel->increaseClosestTeammateIndexIndex();
}

/* ===== */

UIntUnknownCollection* WorldModelInterface::closestUnknownPlayerIndexData() {
  return worldModel->closestUnknownPlayerIndexData();
}

UInt WorldModelInterface::getClosestUnknownPlayerIndexIndex(const int i) const {
  return worldModel->getClosestUnknownPlayerIndexIndex(i);
}

void WorldModelInterface::setClosestUnknownPlayerIndexHistoryLength(const int i) {
  worldModel->setClosestUnknownPlayerIndexHistoryLength(i);
}

UInt WorldModelInterface::getClosestUnknownPlayerIndexHistoryLength() {
  return worldModel->getClosestUnknownPlayerIndexHistoryLength();
}

void WorldModelInterface::increaseClosestUnknownPlayerIndexIndex() {
  worldModel->increaseClosestUnknownPlayerIndexIndex();
}

/* ===== */

AgentCollection* WorldModelInterface::agentData() {
  return worldModel->agentData();
}

UInt WorldModelInterface::getAgentIndex(const int i) const {
  return worldModel->getAgentIndex(i);
}

void WorldModelInterface::setAgentHistoryLength(const int i) {
  worldModel->setAgentHistoryLength(i);
}

UInt WorldModelInterface::getAgentHistoryLength() {
  return worldModel->getAgentHistoryLength();
}

void WorldModelInterface::increaseAgentIndex() {
  worldModel->increaseAgentIndex();
}

RS_END_NAMESPACE

