/* -*- Mode: C++ -*-
 * $Id: PredicateStub.h,v 1.3 2000/09/17 18:03:18 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: PredicateStub.h
 * Unitname: Framework/predicate
 * $Revision: 1.3 $
 * Created by: Fredrik Heintz 2000-09-12
 * Last modified by $Author: frehe $ $Date: 2000/09/17 18:03:18 $
 *
 *
 * HISTORY
 *
 * $Log: PredicateStub.h,v $
 * Revision 1.3  2000/09/17 18:03:18  frehe
 * Fixed an error in the PredicateStub.
 *
 * Revision 1.2  2000/09/17 14:31:38  frehe
 * Made minor updates to the PredicateStub.
 *
 * Revision 1.1  2000/09/13 00:06:00  frehe
 * Added a predicate teamplate called PredicateStub.
 *
 */

/**
 * @file PredicateStub.h
 * Include file for the predicate class PredicateStub.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.3 $
 * $Author: frehe $
 * $Date: 2000/09/17 18:03:18 $
 */


#ifndef PREDICATESTUB_H
#define PREDICATESTUB_H


#include "Predicate.h"


RS_BEGIN_NAMESPACE


/**
 * @class PredicateStub PredicateStub.h
 * The first sentence is a brief description of the skill class SkillStub.
 * The rest is a longer description of the skill class SkillStub.
 * Use it to describe the skill!
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.3 $
 * $Author: frehe $
 * $Date: 2000/09/17 18:03:18 $
 */
class PredicateStub : public Predicate {
public:
  /** Create a predicate with default behavior. */
  PredicateStub();

  /** Creates a predicate that is an exact copy of @a obj. */
  PredicateStub(const PredicateStub& obj);

  /** Destroys the predicate. */  
  virtual ~PredicateStub();


  /** Assignment operator. Assigns @a obj to this predicate. */
  PredicateStub& operator= (const PredicateStub& obj);


  /** Function application operator. Will call evaluate(). */
  virtual FuzzyBool operator()();


  /** Should return the truth value of the predicate. */
  virtual FuzzyBool evaluate();

  
protected:
  /** The update function, should be used to update the value of the
    predicate if necessary. The method needUpdate(const UInt32 update_number)
    can be used to check if the predicate needs to be updated based on the
    last update number it was updated for. */
  virtual void update(const UInt32 update_number);
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "PredicateStub.icc"
#endif


#endif // !PREDICATESTUB_H
