/* -*- Mode: C++ -*-
 * $Id: Team.cc,v 1.1 2000/09/16 18:37:02 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: Team.cc
 * Unitname: Team
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 2000-09-16
 * Last modified by $Author: frehe $ $Date: 2000/09/16 18:37:02 $
 *
 *
 * HISTORY
 * $Log: Team.cc,v $
 * Revision 1.1  2000/09/16 18:37:02  frehe
 * Add a team that can be used as the basis for new stand alone teams.
 *
 */

/**
 * @file Team.cc
 * Implementation file for the agent Team.
 *
 * @par Unit:
 *      Team
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/16 18:37:02 $
 */


#include <fstream>

#include "framework_compiler_directives.h"
#include "ParameterView.h"
#include "view_constants.h"
#include "FrameworkAgent.h"
#include "TeamDecision.h"
#include "LogStream.h"


RS_BEGIN_NAMESPACE


#if defined(RS_USE_ERROR) && !defined(RS_LOG_ERROR)
extern void rs_warning(const std::string& str,
		       const std::string& file,
		       int line)
{
  std::cerr << "WARNING: " << str
	    << " at line " << line 
	    << " in file " << file
	    << std::endl;
}

extern void rs_error(const std::string& str,
		     const std::string& file,
		     int line)
{
  std::cerr << "ERROR: " << str
	    << " at line " << line 
	    << " in file " << file
	    << std::endl;
}

extern void rs_fatal(const std::string& str,
		     const std::string& file,
		     int line)
{
  std::cerr << "FATAL: " << str
	    << " at line " << line 
	    << " in file " << file
	    << std::endl;
  abort();
}
#endif

#if !defined(RS_USE_EXCEPTION) && !defined(RS_LOG_EXCEPTION)
extern void rs_throw(const std::string& exception,
		     const std::string& msg,
		     const std::string& file,
		     int line)
{
  std::cerr << "RAISED: " << exception << "(\"" << msg << "\")"
	    << " at line " << line 
	    << " in file " << file
	    << std::endl;
}
#endif


RS_END_NAMESPACE


int main(int argc, char** argv)
{
#ifdef RS_USE_NAMESPACE
  using namespace RS_NAMESPACE;
#endif

  ofstream logFile("logfile.txt");
  
#ifdef RS_USE_LOG
  LogStream::instance()->addLogArea(LA_ROBOSOC, LA_TOP, true, &std::cerr);
  LogStream::instance()->addLogArea(LA_CONTROLLER, LA_ROBOSOC, true, &logFile);
  LogStream::instance()->addLogArea(LA_ACTUATORS, LA_ROBOSOC, false, &logFile);
  LogStream::instance()->addLogArea(LA_SENSORS, LA_ROBOSOC, false, &logFile);
  LogStream::instance()->addLogArea(LA_SENSOR_INTERFACE, LA_ROBOSOC, false, &logFile);
  LogStream::instance()->addLogArea(LA_AGENT_VIEW, LA_ROBOSOC, true, &logFile);
  LogStream::instance()->addLogArea(LA_BALL_VIEW, LA_ROBOSOC, true, &logFile);
  LogStream::instance()->addLogArea(LA_COMMAND_VIEW, LA_ROBOSOC, true, &logFile);
#endif
    
  try {

    std::cerr << "Starting the parameter view...\n";
    ParameterView parameterView(PARAMETER_VIEW_ID,
				argc, argv);
    ViewManager::instance()->addView(&parameterView, true);
    
    std::cerr << "Starting decision...\n";
    TeamDecision decision;

    std::cerr << "Initializing the agent...\n";
    FrameworkAgent agent(&decision, &parameterView);

    std::cerr << "Starting the agent...\n";
    agent.start();
  
  } catch (RSException &excpt) {
    std::cerr << "Caught an exception: " << excpt.What() << std::endl;
    return 1;
  }
  
  return 0;
  
}

