/* -*- Mode: C++ -*-
 * $Id: TeamDecision.h,v 1.1 2000/09/16 18:37:03 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: TeamDecision.h
 * Unitname: Team
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 2000-09-16
 * Last modified by $Author: frehe $ $Date: 2000/09/16 18:37:03 $
 *
 *
 * HISTORY
 *
 * $Log: TeamDecision.h,v $
 * Revision 1.1  2000/09/16 18:37:03  frehe
 * Add a team that can be used as the basis for new stand alone teams.
 *
 */

/**
 * @file TeamDecision.h
 * Include file for the class TeamDecision.
 *
 * @par Unit:
 *      Team
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/16 18:37:03 $
 */


#ifndef TEAMDECISION_H
#define TEAMDECISION_H


//#define RS_USE_DISPLAY
//#define RS_USE_COACH_DEBUGGING


#include "basic_system_compiler_directives.h"
#include "Decision.h"
#include "ViewManager.h"
#include "ParameterView.h"
#include "GameView.h"
#include "CommandView.h"
#include "BallView.h"
#include "AgentView.h"
#include "PlayerView.h"

#ifdef RS_USE_DISPLAY
#  include "Display.h"
#endif

#ifdef RS_USE_COACH_DEBUGGING
#  include "CoachView.h"
#endif


RS_BEGIN_NAMESPACE


/**
 * @class TeamDecision TeamDecision.h
 * This is the decision maker for Team.
 *
 * <!-- @todo-->
 *
 * @par Unit:
 *      Team
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/16 18:37:03 $
 */
class TeamDecision : public Decision {
public:
  TeamDecision(ActuatorInterface* const actuator_interface = NULL);
  virtual ~TeamDecision();

  virtual ServerCommand* initAgent();
  virtual bool start();
  //virtual bool init(ActuatorInterface* const actuator_interface);
  virtual bool decide();

  // Event handlers
  virtual void onActuatorSensorData();
  //virtual void onAuralSensorData();
  virtual void onInit();
  //virtual void onPhysicalSensorData();
  virtual void onVisualSensorData();

  //virtual void onBeforeSensorData();
  virtual void onAfterSensorData();
  //virtual void onBeforeTick();
  //virtual void onAfterTick();

  virtual void onNewCycle(const bool forced);
  virtual bool onEstimatedNewCycle(UInt16 estimated_time, UInt16 ticks);
  virtual void onCommandWarning();

protected:
  ViewManager* const viewManager;
  ParameterView* parameterView;
  GameView* gameView;
  CommandView* commandView;
  BallView* ballView;
  AgentView* agentView;
  PlayerView* playerView;
  bool serverAlive;

#ifdef RS_USE_DISPLAY
  Display theDisplay;
#endif

#ifdef RS_USE_COACH_DEBUGGING
  CoachView* coachView;
#endif
};


RS_END_NAMESPACE


#endif // !TEAMDECISION_H
