/* -*- Mode: C++ -*-
 * $Id: BallDataImprover.cc,v 1.4 2001/06/19 14:01:47 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: BallDataImprover.cc
 * Unitname: Framework/view
 * $Revision: 1.4 $
 * Created by: Ola Lundqvist 2000-09-24
 * Last modified by $Author: frehe $ $Date: 2001/06/19 14:01:47 $
 *
 *
 * HISTORY
 *
 * $Log: BallDataImprover.cc,v $
 * Revision 1.4  2001/06/19 14:01:47  frehe
 * Made minor changes to compile RoboSoc with gcc-3.0
 *
 * Revision 1.3  2001/01/24 12:49:21  olalu526
 * Some changes to hand in.
 *
 * Revision 1.2  2001/01/23 17:17:13  olalu526
 * Added ballsimulator and improved the ball data improver.
 *
 * Revision 1.1  2001/01/22 10:24:20  olalu526
 * Added the Ball Data Improver.
 *
 *
 */

/**
 * @file BallDataImprover.cc
 * Implementation file for the view class BallDataImprover.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.4 $
 * $Author: frehe $
 * $Date: 2001/06/19 14:01:47 $
 */


#include "BallDataImprover.h"


RS_BEGIN_NAMESPACE


BallDataImprover::BallDataImprover(const ViewId id)
  : ViewWithSimulation(id),
    ballView(NULL)
{
  getView(ballView, BALL_VIEW_ID);
}


BallDataImprover::~BallDataImprover()
{
  viewManager->releaseView(BALL_VIEW_ID);
}

void BallDataImprover::myNewCycle()
{
  ballView->newCycle(currentUpdateNumber);
  update();
}

inline void BallDataImprover::myUpdateAfterSee()
{
  ballView->updateAfterSee(currentUpdateNumber);
  update();
}

void BallDataImprover::update() {
  //std::cout << "(BALL " << worldModel->getBallIndex(NOW) << ", " << worldModel->getBallIndex(PREV) << ")" << std::flush;

  PointUCO nowB = theBall[worldModel->getBallIndex(NOW)].getPosition();
  PointUCO lastB = theBall[worldModel->getBallIndex(PREV)].getPosition();

  if (lastB.isKnown() &&
      ! nowB.isKnown()) {
    std::cout << "(BALL disappeared)" << std::flush;
    //theBall.copy(worldModel->getBallIndex(PREV), worldModel->getBallIndex(NOW));
    PointUCO p = theBall[worldModel->getBallIndex(PREV)].getPosition();
    if (p.isKnown()) {
      VectorUCO v = theBall[worldModel->getBallIndex(PREV)].getAbsoluteSpeedVector();
      theBall[worldModel->getBallIndex(NOW)].setPosition(p+v.getVector());
    }
  }
}

RS_END_NAMESPACE

/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "BallDataImprover.icc"
#endif
