/* -*- Mode: C++ -*-
 * $Id: BallDataImprover.h,v 1.2 2001/01/23 17:17:14 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: BallDataImprover.h
 * Unitname: Framework/view
 * $Revision: 1.2 $
 * Created by: Ola Lundqvist 2000-09-24
 * Last modified by $Author: olalu526 $ $Date: 2001/01/23 17:17:14 $
 *
 *
 * HISTORY
 *
 * $Log: BallDataImprover.h,v $
 * Revision 1.2  2001/01/23 17:17:14  olalu526
 * Added ballsimulator and improved the ball data improver.
 *
 * Revision 1.1  2001/01/22 10:24:20  olalu526
 * Added the Ball Data Improver.
 *
 *
 */

/**
 * @file BallDataImprover.h
 * Include file for the view class BallDataImprover.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.2 $
 * $Author: olalu526 $
 * $Date: 2001/01/23 17:17:14 $
 */


#ifndef BALLDATAIMPROVER_H
#define BALLDATAIMPROVER_H


#include "view_constants.h"
#include "ViewWithSimulation.h"
#include "BallView.h"

RS_BEGIN_NAMESPACE


/**
 * @class BallDataImprover BallDataImprover.h
 * This is a base class for Views that only calculates the future and
 * therefore do not store anything and do not get any data from sensors.
 *
 * It should just (but can include more than) include functions that
 * calculates the fucture from other views.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.2 $
 * $Author: olalu526 $
 * $Date: 2001/01/23 17:17:14 $
 */

class BallDataImprover : public ViewWithSimulation {
public:
  /** Constructor. */
  BallDataImprover(const ViewId id);
  
  /** Destroys the view. */
  ~BallDataImprover();

protected:
  BallView* ballView;
  virtual void myNewCycle();
  virtual void myUpdateAfterCoachMessage();
  virtual void myUpdateAfterInit();
  virtual void myUpdateAfterSee();
  virtual void myUpdateAfterHear();
  virtual void myUpdateAfterSense();
  virtual void myUpdateAfterCommand();
  
private:
  // Hide copy-constructor and assignment.
  void update();
  BallDataImprover(const BallDataImprover&);
  BallDataImprover& operator= (const BallDataImprover&);  
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "BallDataImprover.icc"
#endif


#endif // !BALLDATAIMPROVER_H
