/* -*- Mode: C++ -*-
 * $Id: BallSimulator.cc,v 1.2 2001/01/24 12:49:21 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: BallSimulator.cc
 * Unitname: Framework/view
 * $Revision: 1.2 $
 * Created by: Ola Lundqvist 2001-01-23
 * Last modified by $Author: olalu526 $ $Date: 2001/01/24 12:49:21 $
 *
 *
 * HISTORY
 *
 * $Log: BallSimulator.cc,v $
 * Revision 1.2  2001/01/24 12:49:21  olalu526
 * Some changes to hand in.
 *
 * Revision 1.1  2001/01/23 17:17:15  olalu526
 * Added ballsimulator and improved the ball data improver.
 *
 *
 */

/**
 * @file BallSimulator.cc
 * Implementation file for the view class BallSimulator.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.2 $
 * $Author: olalu526 $
 * $Date: 2001/01/24 12:49:21 $
 */


#include "BallSimulator.h"
#include "ViewManager.h"

RS_BEGIN_NAMESPACE

BallSimulator::BallSimulator(const ViewId id)
   : ViewForSimulation(id),
     ballView(NULL),
     parameterView(NULL)
{
  getView(ballView, BALL_VIEW_ID);
  getView(parameterView, PARAMETER_VIEW_ID);
}

BallSimulator::~BallSimulator()
{
  viewManager->releaseView(BALL_VIEW_ID);
  viewManager->releaseView(PARAMETER_VIEW_ID);
}

void BallSimulator::step() {
  // curRelSimCycle...
  int oldsim = curRelSimCycle;
  curRelSimCycle++;
  /**
   * Check for future allocation.
   * And if not enough are allocated you should allocate it.
   *
   * I have not implemented a easy method to increase the size of the buffer where
   * the data is stored. But it is quite easy:
   *  - increse the size.
   *  - copy data from the old indexes to the new ones.
   *    Just make sure that you do not destroy anything.
   * This stuff should be implemented in the world model or in the view for simulation.
   *
   * But it will (probably) work even if you do not increse the size but then you
   * can not simulate very many values (default size = 5, I think).
   */
  theBall[worldModel->getBallIndex(curRelSimCycle)].
    setPosition(theBall[worldModel->getBallIndex(oldsim)].getPosition() +
		theBall[worldModel->getBallIndex(oldsim)].getAbsoluteSpeedVector());
  theBall[worldModel->getBallIndex(curRelSimCycle)].
    setSpeed(theBall[worldModel->getBallIndex(oldsim)].getSpeed() * parameterView->SP_ball_decay);
}

RS_END_NAMESPACE
