#include "ConeBlob.h"

/* The cone looks like a fez (turkish hat) where the innerRadius
   is the top and the outerRadius is the bottom. outerRadius
   must of course be larger than (or equal to, to create a cylinder)
   than the innerRadius. To get an ordinary cone choose innerRadius
   to zero.
*/

RS_BEGIN_NAMESPACE

ConeBlob::ConeBlob(const float innerRadiusR, const float outerRadiusR,
		   const float innerRadiusTh, const float outerRadiusTh)
  : Blob(2*outerRadiusR, 2*outerRadiusTh),
    innerR(innerRadiusR), outerR(outerRadiusR),
    innerTh(innerRadiusTh), outerTh(outerRadiusTh)
{
  if(innerR < 0)
    innerR = 0;
  if(outerR < innerR)
    outerR = innerR;
  if(innerTh < 0)
    innerTh = 0;
  if(outerTh < innerTh)
    outerTh = innerTh;
}

ConeBlob::~ConeBlob()
{
}

float ConeBlob::getValueAt(float rDepth, float angle /*Vector v*/)
{
//   float rDepth, angle;
//   rDepth = v.getLength();
//   angle = v.getAngle().getDeg();
//   if(angle < 0.0)
//   {
//     rDepth = -rDepth;
//     angle += 180.0;
//   }
  float iradius = pow(rDepth/innerR, 2) + pow(angle/innerTh, 2);
  float oradius = pow(rDepth/outerR, 2) + pow(angle/outerTh, 2);
  if (iradius < 1.0)
    return 1.0;
  else
    return max(1.0 - sqrt(oradius), 0.0);
}

RS_END_NAMESPACE
