/* -*- Mode: C++ -*-
 * $Id: DashView.h,v 1.1 2000/09/17 18:01:38 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: DashView.h
 * Unitname: Framework/view
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 2000-09-17
 * Last modified by $Author: frehe $ $Date: 2000/09/17 18:01:38 $
 *
 *
 * HISTORY
 *
 * $Log: DashView.h,v $
 * Revision 1.1  2000/09/17 18:01:38  frehe
 * Added a (so far empty) dash view, which should be used to make calculations needed by the dash command.
 *
 */

/**
 * @file DashView.h
 * Include file for the view class DashView.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/17 18:01:38 $
 */


#ifndef DASHVIEW_H
#define DASHVIEW_H


#include "View.h"


RS_BEGIN_NAMESPACE


/**
 * @class DashView DashView.h
 * A view for making calculations needed by the dash command.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/17 18:01:38 $
 */
class DashView : public View {
public:
  /** Construct a view with the ViewId @a id. */
  DashView(const ViewId id);

  /** Destroys the view. */
  ~DashView();

protected:
  virtual void myNewCycle();
  virtual void myUpdateAfterCoachMessage();
  virtual void myUpdateAfterInit();
  virtual void myUpdateAfterSee();
  virtual void myUpdateAfterHear();
  virtual void myUpdateAfterSense();
  virtual void myUpdateAfterCommand();

  
private:
  // Hide copy-constructor and assignment.
  DashView(const DashView&);
  DashView& operator= (const DashView&);  
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "DashView.icc"
#endif


#endif // !DASHVIEW_H
