/* -*- Mode: C++ -*-
 * $Id: FutureBallView.cc,v 1.5 2001/01/16 20:54:10 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: FutureBallView.cc
 * Unitname: Framework/view
 * $Revision: 1.5 $
 * Created by: Ola Lundqvist 2000-09-24
 * Last modified by $Author: olalu526 $ $Date: 2001/01/16 20:54:10 $
 *
 *
 * HISTORY
 *
 * $Log: FutureBallView.cc,v $
 * Revision 1.5  2001/01/16 20:54:10  olalu526
 * Added a interface for simulation, and fixed a bug in future ball view.
 *
 * Revision 1.4  2000/10/07 15:08:28  matka134
 * fixed a bug in fastBallPosition
 *
 * Revision 1.3  2000/10/04 08:12:07  olalu526
 * Added some documentation.
 *
 * Revision 1.2  2000/09/25 16:24:14  olalu526
 * Updated FutureBallView and fixed link-error in ViewWithFuture.
 *
 * Revision 1.1  2000/09/24 21:18:28  olalu526
 * Added ViewWithFuture and FutureBallView.
 *
 *
 */

/**
 * @file FutureBallView.cc
 * Implementation file for the view class FutureBallView.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.5 $
 * $Author: olalu526 $
 * $Date: 2001/01/16 20:54:10 $
 */


#include "FutureBallView.h"
#include "ViewManager.h"

RS_BEGIN_NAMESPACE


FutureBallView::FutureBallView(const ViewId id)
   : ViewWithFuture(id),
     ballView(NULL),
     parameterView(NULL)
{
  getView(ballView, BALL_VIEW_ID);
  getView(parameterView, PARAMETER_VIEW_ID);
}


FutureBallView::~FutureBallView()
{
  viewManager->releaseView(BALL_VIEW_ID);
  viewManager->releaseView(PARAMETER_VIEW_ID);
}

Point FutureBallView::fastBallStopPosition() const {
  const Float R = parameterView->SP_ball_decay;

  Int i = NOW;

  // First go back in history and check for the first known position
  // And velocity.
  while (! (ballView->getBallPosition(i)).isKnown() ||
	 ! (ballView->getBallAbsoluteSpeedVector(i)).isKnown() ) {
    i--;
    if (i < -10) {
      return Point(Float(0), Float(0));
    }
  }

  // Get that point. (The first known).
  Point p = (ballView->getBallPosition(i)).getPoint();
  // And that speed. (The first known).
  Vector speed = (ballView->getBallAbsoluteSpeedVector(i)).getVector();

  // A special case (avoiding division by zero below.
  // Will now result in a too big number!!!!!
  // This will probably never happen... Only if the server is buggy :)
  if (R == 1) {
    return p + speed * (Float)1010000001010102012012012021012012012021012.;
  }
  // Now calculate the future, using the above function.
  // Let m->everything
  // Analysis gives, while letting the time goes to a very high value :)
  return  p + (-1)/(R - 1) * speed;

}

/** Calculate the ball position with a fast function without
    using any extrapolation */
Point FutureBallView::fastBallPosition(const Int fut) const {
  const Float R = parameterView->SP_ball_decay;
  /*
   * Pn+1 = Pn + Vn+1 = P0 + V0 + R*V0 + R'2*V0 + ... =
   * P0 + (R'm+1 - 1)/(R + 1)*V0
   */

  // If requested is before "going back found" then return the positon in
  // the ballView instead (the one that is reqeusted!).
  Int now = NOW;
  if (fut < NOW) {
    if ((ballView->getBallPosition(NOW)).isKnown()) {
      return (ballView->getBallPosition(NOW)).getPoint();
    }
    now = fut;
  }
  Int i = now;

  // First go back in history and check for the first known position
  // And velocity.
  while (! (ballView->getBallPosition(i)).isKnown() ||
	 ! (ballView->getBallAbsoluteSpeedVector(i)).isKnown() ) {
    i--;
    if (i < -10) {
      return Point(Float(0), Float(0));
    }
  }

  /* How many times to step... */
  // I changed the code here. I think it fixed the bug without introducing some more ;)
  //  Int m = now - i;
  Int m = fut - i;
  
  // If that position was known then just use it!
  if (! m) {
    return (ballView->getBallPosition(i)).getPoint();
  }

  // Get that point.
  Point p = (ballView->getBallPosition(i)).getPoint();
  // Get that speed.
  Vector speed = (ballView->getBallAbsoluteSpeedVector(i)).getVector();

  // A special case (avoiding division by zero below.
  // This will only happen when the server is buggy :)
  if (R == 1) {
    return p + speed * (Float)m;
  }
  // Now calculate the future, using the above function.
  // Analysis :)
  return  p + (power(R, (unsigned)m) -1)/(R - 1) * speed;
}


RS_END_NAMESPACE
