/* -*- Mode: C++ -*-
 * $Id: FutureBallView.h,v 1.4 2000/10/11 07:35:36 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: FutureBallView.h
 * Unitname: Framework/view
 * $Revision: 1.4 $
 * Created by: Ola Lundqvist 2000-09-24
 * Last modified by $Author: olalu526 $ $Date: 2000/10/11 07:35:36 $
 *
 *
 * HISTORY
 *
 * $Log: FutureBallView.h,v $
 * Revision 1.4  2000/10/11 07:35:36  olalu526
 * Added minor documentation.
 *
 * Revision 1.3  2000/10/04 08:12:07  olalu526
 * Added some documentation.
 *
 * Revision 1.2  2000/09/25 16:24:14  olalu526
 * Updated FutureBallView and fixed link-error in ViewWithFuture.
 *
 * Revision 1.1  2000/09/24 21:18:28  olalu526
 * Added ViewWithFuture and FutureBallView.
 *
 *
 */

/**
 * @file FutureBallView.h
 * Include file for the view class FutureBallView.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.4 $
 * $Author: olalu526 $
 * $Date: 2000/10/11 07:35:36 $
 */


#ifndef FUTUREBALLVIEW_H
#define FUTUREBALLVIEW_H


#include "ViewWithFuture.h"
#include "view_constants.h"
#include "ParameterView.h"
#include "BallView.h"
#include "CoordPoint.h"


RS_BEGIN_NAMESPACE

class BallView;
class ParameterView;

/**
 * @class FutureBallView FutureBallView.h
 * A view that calculates where it believes the ball will be in the future.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.4 $
 * $Author: olalu526 $
 * $Date: 2000/10/11 07:35:36 $
 */
class FutureBallView : public ViewWithFuture {
public:
  /** Default constructor. */
  FutureBallView(const ViewId id);

  /** Destroys the view. */
  ~FutureBallView();

  /* Methods */

  /** Calculates the ball position in the future. Returns the point in the
      future and needs a integer of how many cycles in the future it should
      check */
  Point fastBallPosition(const Int) const;
  /** Calculates where the ball will stop. Returns a point. */
  Point fastBallStopPosition() const;

private:
  BallView* ballView;
  ParameterView* parameterView;
};


RS_END_NAMESPACE

#endif // !FUTUREBALLVIEW_H
