/* -*- Mode: C++ -*-
 * $Id: GoalieView.cc,v 1.2 2000/11/14 10:27:15 bjoli035 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: GoalieView.cc
 * Unitname: Framework/view
 * $Revision: 1.2 $
 * Created by: Bjrn, Ola och Jenny
 * Last modified by $Author: bjoli035 $ $Date: 2000/11/14 10:27:15 $
 *
 *
 * HISTORY
 *
 *
 */

/**
 * @file ViewStub.cc
 * Implementation file for the view class GoalieView.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.2 $
 * $Author: bjoli035 $
 * $Date: 2000/11/14 10:27:15 $
 */

#include "view_constants.h"
#include "GoalieView.h"
#include "ParameterView.h"
#include "AgentView.h"
#include "BallView.h"
#include "ViewManager.h"
#include "GameView.h"
#include "PlayerView.h"
#include "CatchBall.h"

#include <math.h>
#include <iostream.h>

RS_BEGIN_NAMESPACE


GoalieView::GoalieView(const ViewId id)
  : View(id),
    parameterView(NULL),
    agentView(NULL),
    ballView(NULL),
    gameView(NULL),
    playerView(NULL)
{
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID,parameterView);
  getView(ballView, BALL_VIEW_ID);
  getView(gameView, GAME_VIEW_ID);
  getView(playerView, PLAYER_VIEW_ID);

  ourGoalie = false;
  theirGoalie = false;

}


GoalieView::~GoalieView()
{
  viewManager->releaseView(parameterView->getId());
  viewManager->releaseView(agentView->getId());
  viewManager->releaseView(ballView->getId());
}

// Compute the position to intercept the ball in.
PointUCO GoalieView::interceptPosition()
{
  VectorUCO ballVec = ballView->getBallAbsoluteSpeedVector(NOW);
  PointUCO ballPos = ballView->getBallPosition(NOW);
  PointUCO agentPos = agentView->getAgentPosition(NOW);
  
  if( ballVec.isKnown() && ballPos.isKnown() && agentPos.isKnown())
    {
      return numericIterativeGuessPoint(agentPos.getPoint(),
					ballPos.getPoint(),
					ballVec.getVector());
    }
  return ballPos;
}

// Compute the distance between two points.
Float GoalieView::distance(Point one, Point two)
{
   return abs(one.getVectorTo(two).getLength());
}

// Find the point that is closest to get to.
Point GoalieView::numericIterativeGuessPoint(Point playerPos, Point ballPos, Vector speedVector)

{
  Float32 ballDecay = parameterView->SP_ball_decay;
  speedVector=speedVector * 1/(1-ballDecay);
  Point oldpoint = ballPos;
  Point newpoint = ballPos + speedVector;
  while(distance(newpoint,playerPos) < distance(oldpoint,playerPos))
    {
      oldpoint = newpoint;
      speedVector=speedVector * 1/(1-ballDecay);
      newpoint = newpoint + speedVector;
    }
  return oldpoint;
}

bool GoalieView::ourGoalieGotBall()
{
  return ourGoalie;
}

bool GoalieView::theirGoalieGotBall()
{
  return theirGoalie;
}
// These are not used.
void GoalieView::myNewCycle()
{}

void GoalieView::myUpdateAfterCoachMessage()
{}

void GoalieView::myUpdateAfterInit() 
{}

void GoalieView::myUpdateAfterSee() 
{}

void GoalieView::myUpdateAfterHear() 
{
  //Kolla om ngn mlvakt har fngat bollen
  if ( gameView->getPlayMode(NOW) == PM_OUR_GOALIE_GOT_BALL )
    ourGoalie = true;
  if( gameView->getPlayMode(NOW) == PM_THEIR_GOALIE_GOT_BALL )
    theirGoalie = true;

  //Om bollen var fngad frrut s betyder play_on att den inte r det
  //lngre
  if(ourGoalie && gameView->getPlayMode(NOW)==PM_PLAY_ON)
    ourGoalie = false;
  if(theirGoalie && gameView->getPlayMode(NOW)==PM_PLAY_ON)
    theirGoalie = false;

  /* std::cout << std::endl << std::endl << std::endl << "Vrdet p ourGoalieGotBall = " << ourGoalieGotBall << endl;
     std::cout << std::endl << std::endl << std::endl << "Vrdet p playmode r " << gameView->getPlayMode(NOW)  << std::endl << std::endl;
  */
}

void GoalieView::myUpdateAfterSense() 
{}
	 
      

void GoalieView::myUpdateAfterCommand()
{}

RS_END_NAMESPACE
