/* -*- Mode: C++ -*-
 * $Id: GoalieView.h,v 1.2 2000/11/14 10:27:16 bjoli035 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: GoalieView.h
 * Unitname: Framework/view
 * $Revision: 1.2 $
 * Created by: Bjrn, Ola och Jenny
 * Last modified by $Author: bjoli035 $ $Date: 2000/11/14 10:27:16 $
 *
 *
 * HISTORY
 *
 */

/**
 * @file GoalieView.h
 * Include file for the view class GoalieView.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.2 $
 * $Author: bjoli035 $
 * $Date: 2000/11/14 10:27:16 $
 */


#ifndef GOALIEVIEW_H
#define GOALIEVIEW_H


#include "View.h"


RS_BEGIN_NAMESPACE


/**
 * @class GoalieView GoalieView.h This view has a method to calculate
 * an intercept point that the goalie can rush to and later try to
 * catch the ball in.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.2 $
 * $Author: bjoli035 $
 * $Date: 2000/11/14 10:27:16 $ */
// If you do not want any history then inherit from View instead.

class ParameterView;
class AgentView;
class BallView;
class GameView;
class PlayerView;

class GoalieView : public View {
//class GoalieView : public ViewWithHistory {
public:
  // If you do not want history then remove the second argument.
  // Don't forget to update the description and GoalieView.cc as well.
  GoalieView(const ViewId id);
  /** Construct a view with the ViewId @a id and the history length
    @a hist_len. */
  // GoalieView(const ViewId id,
  //	   const UInt hist_len = DEFAULT_HISTORY_LENGTH);

  /** Destroys the view. */
  ~GoalieView();

  bool ourGoalieGotBall();
  bool theirGoalieGotBall();

  PointUCO interceptPosition();
  
  
protected:
  virtual void myNewCycle();
  virtual void myUpdateAfterCoachMessage();
  virtual void myUpdateAfterInit();
  virtual void myUpdateAfterSee();
  virtual void myUpdateAfterHear();
  virtual void myUpdateAfterSense();
  virtual void myUpdateAfterCommand();


  ParameterView* parameterView;
  AgentView* agentView;
  BallView* ballView; 
  GoalieView* goalieView;
  GameView* gameView;
  PlayerView* playerView;
  
private:
  // Hide copy-constructor and assignment.
  GoalieView(const GoalieView&);
  GoalieView& operator= (const GoalieView&);  
  Float distance(Point one, Point two);
  Point numericIterativeGuessPoint(Point here, 
				   Point there, 
				   Vector speedVector);
  bool ourGoalie;
  bool theirGoalie;
  
};


RS_END_NAMESPACE


#endif // !GOALIEVIEW_H
