/* -*- Mode: C++ -*-
 * $Id: KickBallView.h,v 1.2 2000/10/28 20:22:03 mikbr212 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: KickBallView.h
 * Unitname: Framework/view
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 2000-09-12
 * Last modified by $Author: mikbr212 $ $Date: 2000/10/28 20:22:03 $
 *
 *
 * HISTORY
 *
 * $Log: KickBallView.h,v $
 * Revision 1.2  2000/10/28 20:22:03  mikbr212
 * changed initBall: now ballSpeed is retrieved from commandView if not known in ballView
 *
 * Revision 1.1  2000/10/07 14:29:26  mikbr212
 * created
 *
 * Revision 1.2  2000/09/17 14:17:32  frehe
 * Made minor updates to the KickBallView.
 *
 * Revision 1.1  2000/09/13 00:06:21  frehe
 * Added a view template called KickBallView.
 *
 */

/**
 * @file KickBallView.h
 * Include file for the view class KickBallView.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.2 $
 * $Author: mikbr212 $
 * $Date: 2000/10/28 20:22:03 $
 */


#ifndef KICKBALLVIEW_H
#define KICKBALLVIEW_H


#include "View.h"
#include "CoordVector.h"
#include "CoordPoint.h"
#include "ParameterView.h"
#include "BallView.h"
#include "AgentView.h"
#include "CommandView.h"
#include "view_constants.h"
#include "ViewManager.h"
#include "Line.h"

RS_BEGIN_NAMESPACE

/**
 * @class KickBallView KickBallView.h
 * A stub class for views that can be used as a template
 * for creating new views. It uses history.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.2 $
 * $Author: mikbr212 $
 * $Date: 2000/10/28 20:22:03 $
 */
// If you do not want any history then inherit from View instead.
//class KickBallView : public View {
class KickBallView : public View {
public:
  // If you do not want history then remove the second argument.
  // Don't forget to update the description and KickBallView.cc as well.
  // KickBallView(const ViewId id);
  /** Construct a view with the ViewId @a id and the history length
    @a hist_len. */
  KickBallView(const ViewId id,
	       ParameterView* _parameterView,
	       BallView* _ballView,
	       AgentView* _agentView,
	       CommandView* _commandView);

  /** Destroys the view. */
  ~KickBallView();

  double kickPowerRate();
  double kickPowerRate(const Vector& ballPos);
  
  Vector getKickVector(const Vector& desBallSpeed,
		       double maxKick = -1.0);
  Vector getKickVector(const Vector& ballSpeed,
		       const Vector& ballPos,
		       const Vector& desBallSpeed,
		       double maxKick = -1.0);

  bool canShootToSpeedVector(const Vector& desBallSpeed,
			     double maxKick = -1.0);
  bool canShootToSpeedVector(const Vector& ballSpeed,
			     const Vector& ballPos,
			     const Vector& desBallSpeed,
			     double maxKick = -1.0);

  bool initBall(Vector& ballSpeed, Vector& ballPos, bool assume = false);
protected:
  
  virtual void myNewCycle();
  virtual void myUpdateAfterCoachMessage();
  virtual void myUpdateAfterInit();
  virtual void myUpdateAfterSee();
  virtual void myUpdateAfterHear();
  virtual void myUpdateAfterSense();
  virtual void myUpdateAfterCommand();

  ParameterView* parameterView;
  BallView* ballView;
  AgentView* agentView;
  CommandView* commandView;

private:
  // Hide copy-constructor and assignment.
  KickBallView(const KickBallView&);
  KickBallView& operator= (const KickBallView&);  
};

void getView(KickBallView*& tmp, ViewId view_id, 
	     ParameterView* paramView,
	     BallView* ballView,
	     AgentView* agentView,
	     CommandView* commandView);


RS_END_NAMESPACE


#endif // !KICKBALLVIEW_H
