/* -*- Mode: C++ -*-
 * $Id: KickView.h,v 1.3 2000/11/09 18:59:37 marsc818 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <danbe628@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~danbe628/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: KickView.h
 * Unitname: Framework/view
 * $Revision: 1.3 $
 * Created by: Fredrik Heintz 2000-09-12
 * Last modified by $Author: marsc818 $ $Date: 2000/11/09 18:59:37 $
 *
 *
 * HISTORY
 *
 * $Log: KickView.h,v $
 * Revision 1.3  2000/11/09 18:59:37  marsc818
 * some improvements
 *
 * Revision 1.2  2000/10/11 16:51:51  danbe628
 * KickBallInDirection now handles moving balls
 * Some fixes to KickView
 *
 * Revision 1.1  2000/09/27 13:52:37  marsc818
 * Added the kick view and abletokickballtopoint predicate.
 *
 * Revision 1.2  2000/09/17 14:17:32  danbe628
 * Made minor updates to the KickView.
 *
 * Revision 1.1  2000/09/13 00:06:21  danbe628
 * Added a view template called KickView.
 *
 */

/**
 * @file KickView.h
 * Include file for the view class KickView.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.3 $
 * $Author: marsc818 $
 * $Date: 2000/11/09 18:59:37 $
 */


#ifndef KICKVIEW_H
#define KICKVIEW_H


#include "View.h"
//#include "ViewWithHistory.h"

class ParameterView;
class BallView;
class AgentView;

RS_BEGIN_NAMESPACE



// remove this line before commiting !!!!!!!!
//ok! //static const ViewId KICK_VIEW_ID = 12;


/**
 * @class KickView KickView.h
 * A view for getting kick related information.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.3 $
 * $Author: marsc818 $
 * $Date: 2000/11/09 18:59:37 $
 */
// If you do not want any history then inherit from View instead.
//class KickView : public View {
class KickView : public View/*WithHistory*/ {
public:
  // If you do not want history then remove the second argument.
  // Don't forget to update the description and KickView.cc as well.
  // KickView(const ViewId id);
  /** Construct a view with the ViewId @a id and the history length
    @a hist_len. */
  KickView(const ViewId id//,
	   /*const UInt hist_len = DEFAULT_HISTORY_LENGTH*/);

  /** Destroys the view. */
  ~KickView();


  /** Initialize the view.
   Do not have to implemented, can be removed! */
  virtual bool init();
  /** The maximum kick power */
  Float howHardAgentCanKick();
  /** How far the agent can kick the ball */
  Float howFarAgentCanKick();
  /** Returns the kick power rate (depending on the distance and direction towards the ball) */
  Float kickPowerRate();
 
protected:
  virtual void myNewCycle();
  virtual void myUpdateAfterCoachMessage();
  virtual void myUpdateAfterInit();
  virtual void myUpdateAfterSee();
  virtual void myUpdateAfterHear();
  virtual void myUpdateAfterSense();
  virtual void myUpdateAfterCommand();

  AgentView* agentView;
  ParameterView* parameterView;
  BallView* ballView;
  
private:
  // Hide copy-constructor and assignment.
  KickView(const KickView&);
  KickView& operator= (const KickView&);  
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "KickView.icc"
#endif


#endif // !KICKVIEW_H
