/* -*- Mode: C++ -*-
 * $Id: PlayerDataImprover.cc,v 1.3 2001/06/19 14:01:47 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: PlayerDataImprover.cc
 * Unitname: Framework/view
 * $Revision: 1.3 $
 * Created by: Ola Lundqvist 2000-09-24
 * Last modified by $Author: frehe $ $Date: 2001/06/19 14:01:47 $
 *
 *
 * HISTORY
 *
 * $Log: PlayerDataImprover.cc,v $
 * Revision 1.3  2001/06/19 14:01:47  frehe
 * Made minor changes to compile RoboSoc with gcc-3.0
 *
 * Revision 1.2  2001/01/24 12:49:21  olalu526
 * Some changes to hand in.
 *
 * Revision 1.1  2001/01/21 19:14:01  olalu526
 * Added the PlayerDataImprover to improve player data and fixed a compile bug in TerritoryControlView.
 *
 * Revision 1.1  2001/01/16 20:54:10  olalu526
 * Added a interface for simulation, and fixed a bug in future ball view.
 *
 *
 */

/**
 * @file PlayerDataImprover.cc
 * Implementation file for the view class PlayerDataImprover.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.3 $
 * $Author: frehe $
 * $Date: 2001/06/19 14:01:47 $
 */


#include "PlayerDataImprover.h"


RS_BEGIN_NAMESPACE


PlayerDataImprover::PlayerDataImprover(const ViewId id)
  : ViewWithSimulation(id),
    playerView(NULL)
{
  getView(playerView, PLAYER_VIEW_ID);
}


PlayerDataImprover::~PlayerDataImprover()
{
  viewManager->releaseView(PLAYER_VIEW_ID);
}

inline void PlayerDataImprover::myNewCycle() {
  // Now check the data in the world model.

  /*
   * First build a index of known players.
   * Indexed by shirt number - 1.
   */
  // Shirt known (this cycle)
  int t_ko[11]; // opponents
  int t_kt[11]; // teammates
  // Position known (this cycle)
  int t_po[11];
  int t_pt[11];
  // Shirt known (last cycle)
  int l_ko[11]; // opponents
  int l_kt[11]; // teammates
  // Position known (last cycle)
  int l_po[11];
  int l_pt[11];
  // index in player list (this)
  int t_io[11];
  int t_it[11];
  // index in player list (last)
  int l_io[11];
  int l_it[11];
  // Clear
  for (int j = 0; j < 11; j++) {
    t_ko[j] = 0;
    t_kt[j] = 0;
    t_po[j] = 0;
    t_pt[j] = 0;
    l_ko[j] = 0;
    l_kt[j] = 0;
    l_po[j] = 0;
    l_pt[j] = 0;
    t_io[j] = -1;
    t_it[j] = -1;
    l_io[j] = -1;
    l_it[j] = -1;
  }
  // Now build the indexes.
  std::vector<PlayerObject>::const_iterator o;
  std::vector<PlayerObject> ol = theOpponents[worldModel->getOpponentsIndex(PREV)];
  int i;
  // prev opponents
  for (i = 0, o = ol.begin();
       o != ol.end(); ++o,i++ ) {
    if (o->getUniformNumber().isKnown()) {
      int v = o->getUniformNumber().getValue() - 1;
      l_io[v] = i;
      if (o->getPosition().isKnown()) {
	l_po[v] = 1;
      }
      else {
	l_po[v] = 0;
      }
      l_ko[v] = 1;
    }
  }
  // prev teammates
  for (i = 0, o = ol.begin();
       o != ol.end(); ++o,i++ ) {
    if (o->getUniformNumber().isKnown()) {
      int v = o->getUniformNumber().getValue() - 1;
      l_it[v] = i;
      if (o->getPosition().isKnown()) {
	l_pt[v] = 1;
      }
      else {
	l_pt[v] = 0;
      }
      l_kt[v] = 1;
    }
  }
  // this opponents
  for (i = 0, o = ol.begin();
       o != ol.end(); ++o,i++ ) {
    if (o->getUniformNumber().isKnown()) {
      int v = o->getUniformNumber().getValue() - 1;
      t_io[v] = i;
      if (o->getPosition().isKnown()) {
	t_po[v] = 1;
      }
      else {
	t_po[v] = 0;
      }
      t_ko[v] = 1;
    }
  }
  // this teammates
  for (i = 0, o = ol.begin();
       o != ol.end(); ++o,i++ ) {
    if (o->getUniformNumber().isKnown()) {
      int v = o->getUniformNumber().getValue() - 1;
      t_it[v] = i;
      if (o->getPosition().isKnown()) {
	t_pt[v] = 1;
      }
      else {
	t_pt[v] = 0;
      }
      t_kt[v] = 1;
    }
  }
  /*
   * So now we should check for what information that can be improved.
   */
  for (i = 0; i < 11; i++) {
    if (l_ko[i] && ! t_ko[i]) {
      // This one have disapeared.
      std::cout << "(DPI,o: t " << i << " disappeared)" << std::flush;
    }
    if (l_ko[i] &&
	t_ko[i] &&
	l_po[i] &&
	! t_po[i]) {
      // The position have disapeared.
      std::cout << "(DPI,o: p " << i << " disappeared)" << std::flush;
    }
  }
  for (i = 0; i < 11; i++) {
    if (l_kt[i] && ! t_ko[i]) {
      // This one have disapeared.
      std::cout << "(DPI,t: t " << i << " disappeared)" << std::flush;
    }
    if (l_kt[i] &&
	t_kt[i] &&
	l_pt[i] &&
	! t_pt[i]) {
      // The position have disapeared.
      std::cout << "(DPI,t: p " << i << " disappeared)" << std::flush;
    }
  }
}

RS_END_NAMESPACE

/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "PlayerDataImprover.icc"
#endif
