/* -*- Mode: C++ -*-
 * $Id: PlayerDataImprover.h,v 1.2 2001/01/21 19:54:24 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: PlayerDataImprover.h
 * Unitname: Framework/view
 * $Revision: 1.2 $
 * Created by: Ola Lundqvist 2000-09-24
 * Last modified by $Author: olalu526 $ $Date: 2001/01/21 19:54:24 $
 *
 *
 * HISTORY
 *
 * $Log: PlayerDataImprover.h,v $
 * Revision 1.2  2001/01/21 19:54:24  olalu526
 * Fixed error.
 *
 * Revision 1.1  2001/01/21 19:14:01  olalu526
 * Added the PlayerDataImprover to improve player data and fixed a compile bug in TerritoryControlView.
 *
 * Revision 1.1  2001/01/16 20:54:11  olalu526
 * Added a interface for simulation, and fixed a bug in future ball view.
 *
 * Revision 1.1  2000/09/24 21:18:29  olalu526
 * Added PlayerDataImprover and FutureBallView.
 *
 *
 */

/**
 * @file PlayerDataImprover.h
 * Include file for the view class PlayerDataImprover.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.2 $
 * $Author: olalu526 $
 * $Date: 2001/01/21 19:54:24 $
 */


#ifndef PLAYERDATAIMPROVER_H
#define PLAYERDATAIMPROVER_H


#include "view_constants.h"
#include "ViewWithSimulation.h"
#include "PlayerView.h"

RS_BEGIN_NAMESPACE


/**
 * @class PlayerDataImprover PlayerDataImprover.h
 * This is a base class for Views that only calculates the future and
 * therefore do not store anything and do not get any data from sensors.
 *
 * It should just (but can include more than) include functions that
 * calculates the fucture from other views.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.2 $
 * $Author: olalu526 $
 * $Date: 2001/01/21 19:54:24 $
 */

class PlayerDataImprover : public ViewWithSimulation {
public:
  /** Constructor. */
  PlayerDataImprover(const ViewId id);
  
  /** Destroys the view. */
  ~PlayerDataImprover();

protected:
  PlayerView* playerView;
  virtual void myNewCycle();
  virtual void myUpdateAfterCoachMessage();
  virtual void myUpdateAfterInit();
  virtual void myUpdateAfterSee();
  virtual void myUpdateAfterHear();
  virtual void myUpdateAfterSense();
  virtual void myUpdateAfterCommand();
  
private:
  // Hide copy-constructor and assignment.
  PlayerDataImprover(const PlayerDataImprover&);
  PlayerDataImprover& operator= (const PlayerDataImprover&);  
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "PlayerDataImprover.icc"
#endif


#endif // !PLAYERDATAIMPROVER_H
