/* -*- Mode: C++ -*-
 * $Id: SituationView.cc,v 1.7 2001/06/19 14:01:47 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: SituationView.cc
 * Unitname: Framework/view
 * $Revision: 1.7 $
 * Created by: Fredrik Heintz 2000-09-12
 * Last modified by $Author: frehe $ $Date: 2001/06/19 14:01:47 $
 *
 *
 * HISTORY
 *
 */

/**
 * @file SituationView.cc
 * Implementation file for the view class SituationView.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.7 $
 * $Author: frehe $
 * $Date: 2001/06/19 14:01:47 $
 */


#include "IsBallKnown.h"
#include "SituationView.h"
#include <stdio.h>

RS_BEGIN_NAMESPACE


// If you do not want history then remove the second argument,
// and initialize View(id) instead of ViewWithHistory(id, hist_len).
SituationView::SituationView(const ViewId id)
  : ViewWithFuture(id),
    goalieView(NULL),
    parameterView(NULL),
    agentView(NULL),
    ballView(NULL),
    markerView(NULL),
    shirtView(NULL),
    gameView(NULL),
    playerView(NULL)
{
  getView(goalieView, GOALIE_VIEW_ID);
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID, parameterView);
  getView(ballView, BALL_VIEW_ID);
  getView(markerView, MARKER_VIEW_ID, parameterView);
  getView(shirtView, VIRTUALSHIRT_VIEW_ID);
  getView(gameView, GAME_VIEW_ID);
  getView(playerView, PLAYER_VIEW_ID);
  std::cout << "Creating situation view." << std::flush << std::endl;
  for (int i = 0; i < MAX_SITUATIONS; i++) {
    for (int j = 0; j < 11; j++) {
      area[i][j] = Zone();
      role[i][j] = 0;
      passAngle[i][j] = 0;
      passWidth[i][j] = 0;
    }
    enabled[i] = 0;
  }
  parseConfigs();
}

SituationView::~SituationView()
{
  viewManager->releaseView(GOALIE_VIEW_ID);
  viewManager->releaseView(PARAMETER_VIEW_ID);
  viewManager->releaseView(AGENT_VIEW_ID);
  viewManager->releaseView(BALL_VIEW_ID);
  viewManager->releaseView(MARKER_VIEW_ID);
  viewManager->releaseView(VIRTUALSHIRT_VIEW_ID);
  viewManager->releaseView(GAME_VIEW_ID);
  viewManager->releaseView(PLAYER_VIEW_ID);
}

//public:


UInt SituationView::getSituation() const {
  // This have to be much more advanced!!!
  PlayMode pm = gameView->getPlayMode();
  static int situation = 0;
  UInt lastSituation = situation;
  int calc = 0;
  PointUCO bp = ballView->getBallPosition(NOW);
  // 0 - nocalc
  // 1 - normal calc
  switch (pm) {
    /*
      PM_UNKNOWN,
      PM_BEFORE_KICK_OFF,
      PM_HALF_TIME,
      PM_TIME_UP,
      PM_TIME_OVER,
      PM_EXTENDED_TIME,
      PM_PLAY_ON,
      PM_DROP_BALL,
      PM_OFFSIDE_KICK,
      PM_OUR_OFFSIDE_KICK,
      PM_THEIR_OFFSIDE_KICK,
      PM_KICK_OFF,
      PM_OUR_KICK_OFF,
      PM_THEIR_KICK_OFF,
      PM_KICK_IN,
      PM_OUR_KICK_IN,
      PM_THEIR_KICK_IN,
      PM_FREE_KICK,
      PM_OUR_FREE_KICK,
      PM_THEIR_FREE_KICK,
      PM_CORNER_KICK,
      PM_OUR_CORNER_KICK,
      PM_THEIR_CORNER_KICK,
      PM_GOAL_KICK,
      PM_OUR_GOAL_KICK,
      PM_THEIR_GOAL_KICK,
      PM_GOAL,
      PM_OUR_GOAL,
      PM_THEIR_GOAL,
      PM_GOALIE_GOT_BALL,
      PM_OUR_GOALIE_GOT_BALL,
      PM_THEIR_GOALIE_GOT_BALL
    */
  case PM_UNKNOWN:
    break;

  case PM_BEFORE_KICK_OFF:
    if (gameView->getOurSide() == S_LEFT) {
      situation = SITUATION_BEFORE_OUR_KICKOFF;
    }
    else {
      situation = SITUATION_BEFORE_FOE_KICKOFF;
    }
    break;
    
  case PM_HALF_TIME:
    if (gameView->getOurSide() == S_RIGHT) {
      situation = SITUATION_BEFORE_OUR_KICKOFF;
    }
    else {
      situation = SITUATION_BEFORE_FOE_KICKOFF;
    }
    break;

  case PM_TIME_UP:
    situation = SITUATION_END;
    break;

  case PM_TIME_OVER:
    situation = SITUATION_END;
    break;

  case PM_EXTENDED_TIME:
    // situation = SITUATION_;
    calc = 1;
    break;

  case PM_PLAY_ON:
    // situation = SITUATION_;
    calc = 1;
    break;

  case PM_DROP_BALL:
    // situation = SITUATION_;
    calc = 1;
    // What is this?
    break;

    /*  case PM_OFFSIDE_KICK:
	situation = SITUATION_;
	break;
    */
  case PM_OUR_OFFSIDE_KICK:
    if (situation != SITUATION_OUR_GOALIE_BALL) {
      situation = SITUATION_OUR_FREE_KICK;
    }
    break;

  case PM_THEIR_OFFSIDE_KICK:
    if (situation != SITUATION_FOE_GOALIE_BALL) {
      situation = SITUATION_FOE_FREE_KICK;
    }
    break;

    /*
      case PM_KICK_OFF:
      situation = SITUATION_;
      break;
    */

  case PM_OUR_KICK_OFF:
    situation = SITUATION_OUR_KICKOFF;
    break;

  case PM_THEIR_KICK_OFF:
    situation = SITUATION_FOE_KICKOFF;
    break;

    /*  case PM_KICK_IN:
	situation = SITUATION_OUR_THROW;
	break;
    */

  case PM_OUR_KICK_IN:
    situation = SITUATION_OUR_THROW;
    break;

  case PM_THEIR_KICK_IN:
    situation = SITUATION_FOE_THROW;
    break;

    /*  case PM_FREE_KICK:
	situation = SITUATION_;
	break;
    */	

  case PM_OUR_FREE_KICK:
    situation = SITUATION_OUR_FREE_KICK;
    break;

  case PM_THEIR_FREE_KICK:
    situation = SITUATION_FOE_FREE_KICK;
    break;

  case PM_CORNER_KICK:
    if (bp.isKnown()) {
      if (bp.getPoint().getX() > 0) {
	// on the other side, our corner kick :)
	if (bp.getPoint().getY() > 0) {
	  situation = SITUATION_OUR_RCORNER;
	}
	else {
	  situation = SITUATION_OUR_LCORNER;
	}
      }
      else {
	// Foe corner kick.
	if (bp.getPoint().getY() > 0) {
	  situation = SITUATION_FOE_RCORNER;
	}
	else {
	  situation = SITUATION_FOE_LCORNER;
	}
      }
    }
    else {
      situation = SITUATION_UNKNOWN_BALL;
    }
    break;

  case PM_OUR_CORNER_KICK:
    // left of right?
    if (bp.isKnown()) {
      if (bp.getPoint().getY() > 0) {
	situation = SITUATION_OUR_RCORNER;
      }
      else {
	situation = SITUATION_OUR_LCORNER;
      }
    }
    else {
      situation = SITUATION_OUR_BALL;
    }
    break;

  case PM_THEIR_CORNER_KICK:
    if (bp.isKnown()) {
      if (bp.getPoint().getY() > 0) {
	situation = SITUATION_FOE_RCORNER;
      }
      else {
	situation = SITUATION_FOE_LCORNER;
      }
    }
    else {
      situation = SITUATION_FOE_BALL;
    }
    break;

    /*
      case PM_GOAL_KICK:
      situation = SITUATION_;
      break;
    */
    
  case PM_OUR_GOAL_KICK:
    situation = SITUATION_OUR_GOALIE_KICK;
    break;

  case PM_THEIR_GOAL_KICK:
    situation = SITUATION_FOE_GOALIE_KICK;
    break;

    /*
      case PM_GOAL:
      situation = SITUATION_;
      break;
    */
    
  case PM_OUR_GOAL:
    situation = SITUATION_BEFORE_FOE_KICKOFF;
    break;

  case PM_THEIR_GOAL:
    situation = SITUATION_BEFORE_OUR_KICKOFF;
    break;

    /*  case PM_GOALIE_GOT_BALL:
	situation = SITUATION_;
	break;
    */

  case PM_OUR_GOALIE_GOT_BALL:
    situation = SITUATION_OUR_GOALIE_BALL;
    break;

  case PM_THEIR_GOALIE_GOT_BALL:
    situation = SITUATION_FOE_GOALIE_BALL;
    break;
    
  default:
    std::cerr << "Can not resolve what situation there is now." << std::endl;
    break;
  }
  if (calc == 1) {
    if (!bp.isKnown()) {
      return SITUATION_UNKNOWN_BALL;
    }
    static IsBallKnown isBallKnown;
    if (!isBallKnown()) {
      return SITUATION_UNKNOWN_BALL;
    }
    std::vector<PlayerObject> op = playerView->getOpponents(NOW);
    std::vector<PlayerObject> tp = playerView->getTeammates(NOW);
    std::vector<PlayerObject> up = playerView->getUnknownPlayers(NOW);
    Float cop = 500;
    Float ctp = 500;
    PointUCO p = agentView->getAgentPosition(NOW);
    if (p.isKnown()) {
      ctp = p.getPoint().getDistanceTo(bp.getPoint());
    }
    VectorUCO v;
    Float cup = 500;
    Float t;
    // Go through opponents.
    for (std::vector<PlayerObject>::const_iterator plr = op.begin();
	 plr != op.end(); ++plr ) {
      v = plr->getRelativeVectorTo(bp);
      if (v.isKnown()) {
	t = v.getVector().getLength();
	if (t < cop) {
	  cop = t;
	}
      }
    }
    // Go through teammates.
    for (std::vector<PlayerObject>::const_iterator plr = tp.begin();
	 plr != tp.end(); ++plr ) {
      v = plr->getRelativeVectorTo(bp);
      if (v.isKnown()) {
	t = v.getVector().getLength();
	if (t < ctp) {
	  ctp = t;
	}
      }
    }
    // Go through unknownes.
    for (std::vector<PlayerObject>::const_iterator plr = up.begin();
	 plr != up.end(); ++plr ) {
      v = plr->getRelativeVectorTo(bp);
      if (v.isKnown()) {
	t = v.getVector().getLength();
	if (t < cup) {
	  cup = t;
	}
      }
    }
    // So what is the situation?
    t = min(cup, min(cop, ctp));
    if (t == cop) {
      if (abs(cop - cup) < Float(2)) {
	situation = SITUATION_UNKNOWN_BALL;
      }
      else {
	situation = SITUATION_FOE_BALL;
      }
    }
    else if (t == ctp) {
      if (abs(cop - cup) < Float(2)) {
	situation = SITUATION_UNKNOWN_BALL;
      }
      else {
	situation = SITUATION_OUR_BALL;
      }
    }
    else {//if (t == cup) {
      situation = SITUATION_UNKNOWN_BALL;
    }
  }
  //if (goalieView->ourGoalieGotBall()) {
  //  situation = SITUATION_OUR_GOALIE_BALL;
  //}
  //else if (goalieView->theirGoalieGotBall()){
  //  situation = SITUATION_FOE_GOALIE_BALL;
  //}


  //std::cout << "(SIT: " << situation << ")" << std::endl;
  if (!enabled[situation]) {
    situation = lastSituation;
  }
  std::cerr << "(S" << situation << ")" << std::flush;
  return situation;
}

UInt SituationView::getRole() {
  return getRoleOf(shirtView->getVirt());
}

UInt SituationView::getRoleOf(UInt n) {
  return getRoleInOf(getSituation(),n);
}

UInt SituationView::getRoleIn(UInt n) {
  return getRoleInOf(n, shirtView->getVirt());
}

UInt SituationView::getRoleInOf(UInt n, UInt m) {
  return role[n][m];
}

Zone SituationView::getArea() {
  return getAreaOf(shirtView->getVirt());
}

Zone SituationView::getAreaOf(UInt n) {
  return getAreaInOf(getSituation(),n);
}

Zone SituationView::getAreaIn(UInt n) {
  return getAreaInOf(n, shirtView->getVirt());
}

Zone SituationView::getAreaInOf(UInt n, UInt m) {
  return area[n][m];
}

AngleDeg SituationView::getPassAngle() {
  return getPassAngleOf(shirtView->getVirt());
}

AngleDeg SituationView::getPassAngleOf(UInt n) {
  return getPassAngleInOf(getSituation(),n);
}

AngleDeg SituationView::getPassAngleIn(UInt n) {
  return getPassAngleInOf(n, shirtView->getVirt());
}

AngleDeg SituationView::getPassAngleInOf(UInt n, UInt m) {
  return passAngle[n][m];
}

AngleDeg SituationView::getPassWidth() {
  return getPassWidthOf(shirtView->getVirt());
}

AngleDeg SituationView::getPassWidthOf(UInt n) {
  return getPassWidthInOf(getSituation(),n);
}

AngleDeg SituationView::getPassWidthIn(UInt n) {
  return getPassWidthInOf(n, shirtView->getVirt());
}

AngleDeg SituationView::getPassWidthInOf(UInt n, UInt m) {
  return passWidth[n][m];
}


//private:

void SituationView::parseConfigs() {
  for (UInt i = 0; i<MAX_SITUATIONS; i++) {
    enabled[i] = parseFile(i);
  }
}

UInt SituationView::parseFile(UInt n) {
  Zone z;
  CoordPoint<UInt> p;
  AngleDeg a;
  AngleDeg w;
  UInt shirt = 0;
  UInt trole = 0;
  ifstream inf;
  char filename[30];
  sprintf((char*)filename, "situations/%d", n);
  inf.open(filename);
  std::cout << "Reading situation file " << n << "." << std::flush;
  if (inf.fail()) {
    std::cerr << "Can not read file situations/" << n << "." << std::endl;
    return 0;
  }
  int j = 1;
  while (!inf.eof() && !inf.fail()) {
    std::cout << "," << std::flush;
    inf >> std::ws >> p >> std::ws;
    if (inf.fail()) {
      std::cerr << "Failed reading point on line " << j
	   << " in file " << filename << "." << std::endl;
      return 0;
    }
    inf >> std::ws >> z >> std::ws;
    if (inf.fail()) {
      std::cerr << "Failed reading area on line " << j
	   << " in file " << filename << "." << std::endl;
      return 0;
    }
    char c;
    inf >> std::ws >> c;
    if (c == '(') {
      inf >> std::ws >> a >> std::ws >> c;
      if (c == ',') inf >> std::ws >> w >> std::ws >> c;
      if (c != ')') inf.clear(std::ios::badbit); //error...;
    }
    else {
      inf.clear(std::ios::badbit); //error...;
    }
    if (inf.fail()) {
      std::cerr << "Failed reading pass direction on line " << j
	   << " in file " << filename << "." << std::endl;
      return 0;
    }
    shirt = p.getX();
    trole = p.getY();
    // Now store.
    area[n][shirt] = z;
    role[n][shirt] = trole;
    passAngle[n][shirt] = a;
    passWidth[n][shirt] = w;
    inf >> std::ws;
    j++;
  }
  std::cout << std::endl;
  inf.close();
  return 1;
}

RS_END_NAMESPACE
