/* -*- Mode: C++ -*-
 * $Id: SituationView.h,v 1.2 2001/01/23 17:17:17 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: SituationView.h
 * Unitname: Framework/view
 * $Revision: 1.2 $
 * Created by: Ola Lundqvist 2000-11-01
 * Last modified by $Author: olalu526 $ $Date: 2001/01/23 17:17:17 $
 *
 *
 * HISTORY
 *
 * $Log: SituationView.h,v $
 * Revision 1.2  2001/01/23 17:17:17  olalu526
 * Added ballsimulator and improved the ball data improver.
 *
 * Revision 1.1  2000/11/02 22:55:10  olalu526
 * Added situation view.
 *
 * Revision 1.1  2000/11/01 18:31:18  olalu526
 * Added SituationView for Virtual to Physical shirt mapping.
 *
 *
 */

/**
 * @file SituationView.h
 * Include file for the view class SituationView.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.2 $
 * $Author: olalu526 $
 * $Date: 2001/01/23 17:17:17 $
 */


#ifndef SITUATIONVIEW_H
#define SITUATIONVIEW_H


#include "ViewWithFuture.h"
#include "view_constants.h"
#include "ViewManager.h"
#include "ParameterView.h"
#include "AgentView.h"
#include "BallView.h"
#include "MarkerView.h"
#include "Situations.h"
#include "VirtualShirtView.h"
#include "GameView.h"
#include "PlayerView.h"
#include "Zone.h"
#include <fstream.h>
#include "GoalieView.h"

RS_BEGIN_NAMESPACE


/**
 * @class SituationView SituationView.h
 * A stub class for views that can be used as a template
 * for creating new views. It uses history.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.2 $
 * $Author: olalu526 $
 * $Date: 2001/01/23 17:17:17 $
 */
// If you do not want any history then inherit from View instead.
//class SituationView : public View {
class SituationView : public ViewWithFuture {
public:
  // If you do not want history then remove the second argument.
  // Don't forget to update the description and SituationView.cc as well.
  SituationView(const ViewId id);

  /** Destroys the view. */
  ~SituationView();

  UInt getSituation() const;
  UInt getRole();
  UInt getRoleOf(UInt);
  UInt getRoleIn(UInt);
  UInt getRoleInOf(UInt, UInt);
  Zone getArea();
  Zone getAreaOf(UInt);
  Zone getAreaIn(UInt);
  Zone getAreaInOf(UInt,UInt);
  AngleDeg getPassAngle();
  AngleDeg getPassAngleOf(UInt);
  AngleDeg getPassAngleIn(UInt);
  AngleDeg getPassAngleInOf(UInt, UInt);
  AngleDeg getPassWidth();
  AngleDeg getPassWidthOf(UInt);
  AngleDeg getPassWidthIn(UInt);
  AngleDeg getPassWidthInOf(UInt, UInt);

private:
  void parseConfigs();
  UInt parseFile(const UInt n);

  // Hide copy-constructor and assignment.
  SituationView(const SituationView&);
  SituationView& operator= (const SituationView&);  
  GoalieView* goalieView;
  ParameterView* parameterView;
  AgentView*     agentView;
  BallView*      ballView;
  MarkerView*    markerView;
  VirtualShirtView* shirtView;
  GameView*      gameView;
  PlayerView*    playerView;

  // Everything are indexed by situation, virshirt
  Zone area[MAX_SITUATIONS][11];
  UInt role[MAX_SITUATIONS][11];
  AngleDeg passAngle[MAX_SITUATIONS][11];
  AngleDeg passWidth[MAX_SITUATIONS][11];
  UInt enabled[MAX_SITUATIONS];
};


RS_END_NAMESPACE

#endif // !SITUATIONVIEW_H
