/* -*- Mode: C++ -*-
 * $Id: ViewForSimulation.cc,v 1.2 2001/06/19 14:01:48 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: ViewForSimulation.cc
 * Unitname: Framework/view
 * $Revision: 1.2 $
 * Created by: Ola Lundqvist 2000-09-24
 * Last modified by $Author: frehe $ $Date: 2001/06/19 14:01:48 $
 *
 *
 * HISTORY
 *
 * $Log: ViewForSimulation.cc,v $
 * Revision 1.2  2001/06/19 14:01:48  frehe
 * Made minor changes to compile RoboSoc with gcc-3.0
 *
 * Revision 1.1  2001/01/16 20:54:10  olalu526
 * Added a interface for simulation, and fixed a bug in future ball view.
 *
 *
 */

/**
 * @file ViewForSimulation.cc
 * Implementation file for the view class ViewForSimulation.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2001/06/19 14:01:48 $
 */


#include "ViewForSimulation.h"


RS_BEGIN_NAMESPACE


ViewForSimulation::ViewForSimulation(const ViewId id)
  : ViewWithSimulation(id),
    autoNewCycle(true),
    curRelSimCycle(0)
{
}


ViewForSimulation::~ViewForSimulation()
{
}

void ViewForSimulation::step(const int c) {
  for (int i = 0; i < c; i++) {
    step();
  }
}

void ViewForSimulation::stepTo(const int i) {
  while (curRelSimCycle < i) {
    int test = curRelSimCycle;
    step();
    if (test + 1 != curRelSimCycle) {
      std::cerr << "Error in ViewForSimulation::stepTo(" << i << ");\n"
		<< "\tstep() do not increase curRelSimCycle as it should. Old ("
		<< test << "), new (" << curRelSimCycle << ").\n"
		<< "Exiting the stepTo loop." << std::endl;
      continue;
    }
  }
}

void ViewForSimulation::newCycle() {
  curRelSimCycle = 0;
}

int ViewForSimulation::setRelativeSimulationCycle(const int i) {
  if (i > 0) {
    curRelSimCycle = i;
  }
  return curRelSimCycle;
}

int ViewForSimulation::getRelativeSimulationCycle() const {
  return curRelSimCycle;
}

void ViewForSimulation::setAutoNewCycle(const bool b) {
  autoNewCycle = b;
}

bool ViewForSimulation::getAutoNewCycle() const {
  return autoNewCycle;
}

RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "ViewForSimulation.icc"
#endif
