/* -*- Mode: C++ -*-
 * $Id: ViewForSimulation.h,v 1.2 2001/01/23 17:17:18 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: ViewForSimulation.h
 * Unitname: Framework/view
 * $Revision: 1.2 $
 * Created by: Ola Lundqvist 2000-09-24
 * Last modified by $Author: olalu526 $ $Date: 2001/01/23 17:17:18 $
 *
 *
 * HISTORY
 *
 * $Log: ViewForSimulation.h,v $
 * Revision 1.2  2001/01/23 17:17:18  olalu526
 * Added ballsimulator and improved the ball data improver.
 *
 * Revision 1.1  2001/01/16 20:54:11  olalu526
 * Added a interface for simulation, and fixed a bug in future ball view.
 *
 * Revision 1.1  2000/09/24 21:18:29  olalu526
 * Added ViewForSimulation and FutureBallView.
 *
 *
 */

/**
 * @file ViewForSimulation.h
 * Include file for the view class ViewForSimulation.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.2 $
 * $Author: olalu526 $
 * $Date: 2001/01/23 17:17:18 $
 */


#ifndef VIEWFORSIMULATION_H
#define VIEWFORSIMULATION_H


#include "ViewWithSimulation.h"


RS_BEGIN_NAMESPACE


/**
 * @class ViewForSimulation ViewForSimulation.h
 * This is a base class for Views that only calculates the future and
 * therefore do not store anything and do not get any data from sensors.
 *
 * It should just (but can include more than) include functions that
 * calculates the fucture from other views.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.2 $
 * $Author: olalu526 $
 * $Date: 2001/01/23 17:17:18 $
 */

class ViewForSimulation : public ViewWithSimulation {
public:
  /** Constructor. */
  ViewForSimulation(const ViewId id);
  
  /** Destroys the view. */
  ~ViewForSimulation();

  virtual void step() = 0;
  virtual void step(const int);
  virtual void stepTo(const int);

  virtual void newCycle();
  virtual int setRelativeSimulationCycle(const int);
  virtual int getRelativeSimulationCycle() const;
  virtual void setAutoNewCycle(const bool);
  virtual bool getAutoNewCycle() const;

protected:
  virtual void myNewCycle();
  virtual void myUpdateAfterCoachMessage();
  virtual void myUpdateAfterInit();
  virtual void myUpdateAfterSee();
  virtual void myUpdateAfterHear();
  virtual void myUpdateAfterSense();
  virtual void myUpdateAfterCommand();
  
  bool autoNewCycle;
  int curRelSimCycle;

private:
  // Hide copy-constructor and assignment.
  ViewForSimulation(const ViewForSimulation&);
  ViewForSimulation& operator= (const ViewForSimulation&);  
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "ViewForSimulation.icc"
#endif


#endif // !VIEWWITHFUTURE_H
