/* -*- Mode: C++ -*-
 * $Id: ViewWithFuture.h,v 1.1 2000/09/24 21:18:29 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: ViewWithFuture.h
 * Unitname: Framework/view
 * $Revision: 1.1 $
 * Created by: Ola Lundqvist 2000-09-24
 * Last modified by $Author: olalu526 $ $Date: 2000/09/24 21:18:29 $
 *
 *
 * HISTORY
 *
 * $Log: ViewWithFuture.h,v $
 * Revision 1.1  2000/09/24 21:18:29  olalu526
 * Added ViewWithFuture and FutureBallView.
 *
 *
 */

/**
 * @file ViewWithFuture.h
 * Include file for the view class ViewWithFuture.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.1 $
 * $Author: olalu526 $
 * $Date: 2000/09/24 21:18:29 $
 */


#ifndef VIEWWITHFUTURE_H
#define VIEWWITHFUTURE_H


#include "View.h"


RS_BEGIN_NAMESPACE


/**
 * @class ViewWithFuture ViewWithFuture.h
 * This is a base class for Views that only calculates the future and
 * therefore do not store anything and do not get any data from sensors.
 *
 * It should just (but can include more than) include functions that
 * calculates the fucture from other views.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.1 $
 * $Author: olalu526 $
 * $Date: 2000/09/24 21:18:29 $
 */

// If you do not want any history then inherit from View instead.
class ViewWithFuture : public View {
  //class ViewWithFuture : public ViewWithHistory {
public:
  // If you do not want history then remove the second argument.
  // Don't forget to update the description and ViewWithFuture.cc as well.
  
  ViewWithFuture(const ViewId id);
  
  /** Destroys the view. */
  ~ViewWithFuture();

protected:
  virtual void myNewCycle();
  virtual void myUpdateAfterCoachMessage();
  virtual void myUpdateAfterInit();
  virtual void myUpdateAfterSee();
  virtual void myUpdateAfterHear();
  virtual void myUpdateAfterSense();
  virtual void myUpdateAfterCommand();
  
private:
  // Hide copy-constructor and assignment.
  ViewWithFuture(const ViewWithFuture&);
  ViewWithFuture& operator= (const ViewWithFuture&);  
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "ViewWithFuture.icc"
#endif


#endif // !VIEWWITHFUTURE_H
