/* -*- Mode: C++ -*-
 * $Id: VirtualShirtView.cc,v 1.3 2001/06/19 14:01:48 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: VirtualShirtView.cc
 * Unitname: Framework/view
 * $Revision: 1.3 $
 * Created by: Fredrik Heintz 2000-09-12
 * Last modified by $Author: frehe $ $Date: 2001/06/19 14:01:48 $
 *
 *
 * HISTORY
 *
 */

/**
 * @file VirtualShirtView.cc
 * Implementation file for the view class VirtualShirtView.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.3 $
 * $Author: frehe $
 * $Date: 2001/06/19 14:01:48 $
 */


#include "VirtualShirtView.h"


RS_BEGIN_NAMESPACE


// If you do not want history then remove the second argument,
// and initialize View(id) instead of ViewWithHistory(id, hist_len).
VirtualShirtView::VirtualShirtView(const ViewId id)
  : ViewWithFuture(id),
    parameterView(NULL),
    agentView(NULL)
{
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID, parameterView);
  shirts = new int[11];
  for (int i=0;i<11;i++) {
    shirts[i] = i;
  }
}

VirtualShirtView::~VirtualShirtView()
{
  viewManager->releaseView(PARAMETER_VIEW_ID);
  viewManager->releaseView(AGENT_VIEW_ID);
}

void VirtualShirtView::switchShirt(const int s1, const int s2) {
  shirts[s1] = s2;
  shirts[s2] = s1;
}

int VirtualShirtView::getVirt() {
  UIntUnknown a = agentView->getUniformNumber();
  if (a.isKnown()) {
    //cout << "My shirt number is " << a.getValue() << "!!" << flush << endl;
    return getVirt(a.getValue() -1);
  }
  return getVirt(0);
}

int VirtualShirtView::getVirt(const int s) {
  return shirts[s];
}

int VirtualShirtView::getReal(const int s) {
  for (int i=0;i<11;i++) {
    if (shirts[i] == s) {
      return i;
    }
  }
  std::cerr << "VirtualShirtView corrupted!" << std::endl;
  return -1;
}

RS_END_NAMESPACE
