/* -*- Mode: C++ -*-
 * $Id: VirtualShirtView.h,v 1.1 2000/11/01 18:31:18 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: VirtualShirtView.h
 * Unitname: Framework/view
 * $Revision: 1.1 $
 * Created by: Ola Lundqvist 2000-11-01
 * Last modified by $Author: olalu526 $ $Date: 2000/11/01 18:31:18 $
 *
 *
 * HISTORY
 *
 * $Log: VirtualShirtView.h,v $
 * Revision 1.1  2000/11/01 18:31:18  olalu526
 * Added VirtualShirtView for Virtual to Physical shirt mapping.
 *
 *
 */

/**
 * @file VirtualShirtView.h
 * Include file for the view class VirtualShirtView.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.1 $
 * $Author: olalu526 $
 * $Date: 2000/11/01 18:31:18 $
 */


#ifndef VIRTUALSHIRTVIEW_H
#define VIRTUALSHIRTVIEW_H


#include "ViewWithFuture.h"
#include "view_constants.h"
#include "ViewManager.h"
#include "ParameterView.h"
#include "AgentView.h"

RS_BEGIN_NAMESPACE


/**
 * @class VirtualShirtView VirtualShirtView.h
 * A stub class for views that can be used as a template
 * for creating new views. It uses history.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.1 $
 * $Author: olalu526 $
 * $Date: 2000/11/01 18:31:18 $
 */
// If you do not want any history then inherit from View instead.
//class VirtualShirtView : public View {
class VirtualShirtView : public ViewWithFuture {
public:
  // If you do not want history then remove the second argument.
  // Don't forget to update the description and VirtualShirtView.cc as well.
  VirtualShirtView(const ViewId id);

  /** Destroys the view. */
  ~VirtualShirtView();

  void switchShirt(const int, const int);
  int getVirt();
  int getVirt(const int);
  int getReal(const int);

private:
  // Hide copy-constructor and assignment.
  VirtualShirtView(const VirtualShirtView&);
  VirtualShirtView& operator= (const VirtualShirtView&);  
  ParameterView* parameterView;
  AgentView* agentView;
  int *shirts;
};


RS_END_NAMESPACE

#endif // !VIRTUALSHIRTVIEW_H
