/* -*- Mode: C++ -*-
 * $Id: ZoneDefenceDefaultPosition.cc,v 1.8 2001/06/19 14:01:48 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: ZoneDefenceDefaultPosition.cc
 * Unitname: Framework/view
 * $Revision: 1.8 $
 * Created by: Ola Lundqvist 2000-09-28
 * Last modified by $Author: frehe $ $Date: 2001/06/19 14:01:48 $
 *
 *
 * HISTORY
 *
 *
 */

/**
 * @file ZoneDefenceDefaultPosition.cc
 * Implementation file for the view class ZoneDefenceDefaultPosition.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.8 $
 * $Author: frehe $
 * $Date: 2001/06/19 14:01:48 $
 */


#include "ZoneDefenceDefaultPosition.h"


RS_BEGIN_NAMESPACE


// If you do not want history then remove the second argument,
// and initialize View(id) instead of ViewWithHistory(id, hist_len).
ZoneDefenceDefaultPosition::ZoneDefenceDefaultPosition(const ViewId id)
  : ViewWithFuture(id),
    zone(Zone(Float(0),Float(0),Float(0))),
    ballView(NULL),
    playerView(NULL)
{
  getView(ballView, BALL_VIEW_ID);
  getView(playerView, PLAYER_VIEW_ID);
}


ZoneDefenceDefaultPosition::~ZoneDefenceDefaultPosition()
{
  viewManager->releaseView(BALL_VIEW_ID);
  viewManager->releaseView(PLAYER_VIEW_ID);
}


Point ZoneDefenceDefaultPosition::getPosition() {
  /* P = zp + R/|v| * v - R*v*R/|v||v| = Pz + R * v * ((((1 - )))R/|v|) / |v| =
   * The ((( x ))) should not be there... Have thought wrong somewhere...
   */
  
  Point zp = zone.getPointOf();
  Float K = Float(0.95);
  if ((ballView->getBallPosition(NOW)).isKnown()) {
    // We should know that |v| < R
    Point bp = (ballView->getBallPosition(NOW)).getPoint();
    Vector v = zp.getVectorTo(bp);
    Float dv = zp.getDistanceTo(bp);
    Float R = 0;//zone.getR();
    Point ap = zp;
    while (R < 100 && zone.inArea(Point((R/dv)*v)+ap)) {
      R++;
    }
    if (R>0) {
      R--;
    }
    std::cerr << "Found R as : " << R << "." << std::endl;
    if (dv > R) {
      Point p = zp + K * R * v * (R/dv) / dv;
      //      std::cout << "Setpos: " << p << " bp " << bp << " zp " << zp << " R: " << R << " dv " << dv;
      return p;
    }
    else {
      //std::cout << "The ball is inside the zone, you should not positionate!" << std::endl;
      return zp;
    }
  }

  // If nothing else works...
  std::cout << "Do not know where the ball is... So go back to the middle!"<< std::endl;
  return zp;
}

void ZoneDefenceDefaultPosition::setZone(const Zone& z)
{
  zone = z;
}

Point ZoneDefenceDefaultPosition::maxOffsidePosition(const Point p) {
  Point pp = p;
  std::vector<PlayerObject>::const_iterator o;
  std::vector<PlayerObject> ol = playerView->getOpponents(NOW);
  for (o = ol.begin();
       o != ol.end(); ++o) {
    if (o->getPosition().isKnown()) {
      Point op = o->getPosition().getPoint();
      if (op.getX() > pp.getX()) {
	pp.setX(op.getX()-0.5);
      }
    }
  }
  return pp;
}

Zone ZoneDefenceDefaultPosition::getZone() const
{
  return zone;
}


RS_END_NAMESPACE



/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "ZoneDefenceDefaultPosition.icc"
#endif
