/* -*- Mode: C++ -*-
 * $Id: ZoneDefenceDefaultPosition.h,v 1.4 2001/01/23 17:47:32 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: ZoneDefenceDefaultPosition.h
 * Unitname: Framework/view
 * $Revision: 1.4 $
 * Created by: Ola Lundqvist 2000-09-28
 * Last modified by $Author: olalu526 $ $Date: 2001/01/23 17:47:32 $
 *
 *
 * HISTORY
 *
 * $Log: ZoneDefenceDefaultPosition.h,v $
 * Revision 1.4  2001/01/23 17:47:32  olalu526
 * Implemented some sort of avoid offide possibility to zone defence defender.
 *
 * Revision 1.3  2000/11/07 17:54:34  olalu526
 * Generalized the zone concept to handle various type of objects. It now handles a set of rectangles and circles to form an area. This affected quite a lot of skills.
 *
 * Revision 1.2  2000/09/28 16:00:16  olalu526
 * Fixed typo errors.
 *
 * Revision 1.1  2000/09/28 15:30:55  olalu526
 * Added ZondDefenceDefaultPosition.
 *
 *
 */

/**
 * @file ZoneDefenceDefaultPosition.h
 * Include file for the view class ZoneDefenceDefaultPosition.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.4 $
 * $Author: olalu526 $
 * $Date: 2001/01/23 17:47:32 $
 */


#ifndef ZONEDEFENCEDEFAULTPOSITION_H
#define ZONEDEFENCEDEFAULTPOSITION_H


#include "ViewWithFuture.h"
#include "BallView.h"
#include "PlayerView.h"
#include "../skill/Zone.h"
#include "view_constants.h"
#include "ViewManager.h"


RS_BEGIN_NAMESPACE

class BallView;

/**
 * @class ZoneDefenceDefaultPosition ZoneDefenceDefaultPosition.h
 * A view for calculating the default position and to make some
 * checking on it.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.4 $
 * $Author: olalu526 $
 * $Date: 2001/01/23 17:47:32 $
 */
class ZoneDefenceDefaultPosition : public ViewWithFuture {
public:
  // ZoneDefenceDefaultPosition(const ViewId id);
  /** Construct a view with the ViewId @a id and the history length
    @a hist_len. */
  ZoneDefenceDefaultPosition(const ViewId);

  /** Destroys the view. */
  ~ZoneDefenceDefaultPosition();

  virtual void setZone(const Zone&);
  virtual Zone getZone() const;
  Point getPosition();
  Point maxOffsidePosition(const Point);

protected:
  
private:
  // Hide copy-constructor and assignment.
  ZoneDefenceDefaultPosition(const ZoneDefenceDefaultPosition&);
  ZoneDefenceDefaultPosition& operator= (const ZoneDefenceDefaultPosition&);  
  Zone zone;
  BallView *ballView;
  PlayerView *playerView;
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "ZoneDefenceDefaultPosition.icc"
#endif


#endif // !ZONEDEFENCEDEFAULTPOSITION_H
