/* -*- Mode: C++ -*-
 * $Id: view_constants.h,v 1.12 2001/01/22 10:24:21 olalu526 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: view_constants.h
 * Unitname: Framework
 * $Revision: 1.12 $
 * Created by: Fredrik Heintz 2000-09-03
 * Last modified by $Author: olalu526 $ $Date: 2001/01/22 10:24:21 $
 *
 *
 * HISTORY
 *
 * $Log: view_constants.h,v $
 * Revision 1.12  2001/01/22 10:24:21  olalu526
 * Added the Ball Data Improver.
 *
 * Revision 1.11  2001/01/21 19:47:12  olalu526
 * Added the player data improver constant.
 *
 * Revision 1.10  2000/11/19 11:39:13  mikbr212
 * added FuzzyBallView
 *
 * Revision 1.9  2000/11/02 22:55:11  olalu526
 * Added situation view.
 *
 * Revision 1.8  2000/11/01 18:31:18  olalu526
 * Added VirtualShirtView for Virtual to Physical shirt mapping.
 *
 * Revision 1.7  2000/10/07 14:29:58  mikbr212
 * KickBallView added
 *
 * Revision 1.6  2000/10/05 13:21:03  olale507
 * Added GOALIE_VIEW_ID
 *
 * Revision 1.5  2000/10/04 10:00:27  andsi988
 * Added Territory control view.
 *
 * Revision 1.4  2000/09/28 15:57:20  olalu526
 * Fixed the name of the zonedefence... view.
 *
 * Revision 1.3  2000/09/28 15:17:07  frehe
 * Moved view_constants from Framework/include to Framework/view.
 *
 * Revision 1.4  2000/09/28 15:12:09  olalu526
 * Added view id for zonedefencenormalposition.
 *
 * Revision 1.3  2000/09/27 13:52:28  marsc818
 * Added the kick view and abletokickballtopoint predicate.
 *
 * Revision 1.2  2000/09/24 21:21:11  olalu526
 * Added a view id.
 *
 * Revision 1.1  2000/09/21 13:46:44  frehe
 * Move the RoboSoc views from rsviews to the rsframework library.
 *
 * Revision 1.1  2000/09/04 15:27:26  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file view_constants.h
 * Include file with symbolic constants for the view id's of
 * the views in RoboSoc.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.12 $
 * $Author: olalu526 $
 * $Date: 2001/01/22 10:24:21 $
 */


#ifndef VIEW_CONSTANTS_H
#define VIEW_CONSTANTS_H


#include "machine_dependent_types.h"


RS_BEGIN_NAMESPACE


typedef UInt16 ViewId;


/** Symbolic name for the id of the parameter view. */
static const ViewId PARAMETER_VIEW_ID = 1;
/** Symbolic name for the id of the game view. */
static const ViewId GAME_VIEW_ID      = 2;
/** Symbolic name for the id of the command view. */
static const ViewId COMMAND_VIEW_ID   = 3;
/** Symbolic name for the id of the marker view. */
static const ViewId MARKER_VIEW_ID    = 4;
/** Symbolic name for the id of the agent view. */
static const ViewId AGENT_VIEW_ID     = 5;
/** Symbolic name for the id of the ball view. */
static const ViewId BALL_VIEW_ID      = 6;
/** Symbolic name for the id of the player view. */
static const ViewId PLAYER_VIEW_ID    = 7;
/** Symbolic name for the id of the coach view. */
static const ViewId COACH_VIEW_ID     = 8;
/** Symbolic name for the id of the control view. */
static const ViewId CONTROL_VIEW_ID   = 9;
/** Symbolic name for the id of the dash view. */
static const ViewId DASH_VIEW_ID   = 10;
/** Symbolic name for the id of the FutureBall view */
static const ViewId FUTUREBALL_VIEW_ID = 11;
/** Symbolic name for the id of the kick view */
static const ViewId KICK_VIEW_ID = 12;
/** Symbolic name for the id of the zonedefencenormalposition view */
static const ViewId ZONEDEFENCEDEFAULTPOSITION_VIEW_ID = 13;
/** Symbolic name for the id of the Goalie view */
static const ViewId GOALIE_VIEW_ID = 14;
/** Symbolic name for the id of the TerritoryControl view */
static const ViewId TERRITORY_CONTROL_VIEW_ID = 15;
/** Symbolic name for the if of the KickBall view */
static const ViewId KICKBALL_VIEW_ID = 16;
/** Symbolic name for the if of the VitualShirt view */
static const ViewId VIRTUALSHIRT_VIEW_ID = 17;
/** Symbolic name for the if of the Situation view */
static const ViewId SITUATION_VIEW_ID = 18;
/** Symbolic name for the if of the Fuzzy Ball view */
static const ViewId FUZZY_BALL_VIEW_ID = 19;
/** Symbolic name for the if of the Player Data Improver */
static const ViewId PLAYERDATAIMPROVER_ID = 20;
/** Symbolic name for the if of the Ball Data Improver */
static const ViewId BALLDATAIMPROVER_ID = 21;


RS_END_NAMESPACE


#endif // VIEW_CONSTANTS_H
