/* -*- Mode: C++ -*-
 * $Id: AgentObject.h,v 1.2 2000/10/18 23:05:36 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: AgentObject.h
 * Unitname: Library
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/10/18 23:05:36 $
 *
 *
 * HISTORY
 *
 * $Log: AgentObject.h,v $
 * Revision 1.2  2000/10/18 23:05:36  frehe
 * Added lastCoachMessage and lastPlayerMessage.
 *
 * Revision 1.1  2000/09/01 03:24:30  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file AgentObject.h
 * Include file for the class AgentObject.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2000/10/18 23:05:36 $
 */


#ifndef AGENTOBJECT_H
#define AGENTOBJECT_H


#include "PlayerObject.h"
#include "ServerCommand.h"


RS_BEGIN_NAMESPACE


/**
 * @class AgentObject AgentObject.h
 * Class used to represent the agent.
 *
 * <!-- @invariant The class invariant is... -->
 *
 * @todo Make a toString and operator<<
 *
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2000/10/18 23:05:36 $
 */
class AgentObject : public PlayerObject
{
public:
  AgentObject(const Team def_team = T_UNKNOWN,
	       const Side def_side = S_UNKNOWN,
	       const UIntUnknown def_u_number = UIntUnknown(),
	       const AngleDegUCO& def_neck_dir = AngleDegUCO(),
	       const AngleDegUCO& def_face_dir = AngleDegUCO(),
	       const PointUCO& def_point = PointUCO(),
	       const VectorUCO& def_vector = VectorUCO(),
	       const FloatUCO def_dist_changed = FloatUCO(),
	       const AngleDegUCO& def_dir_changed = AngleDegUCO(),
	       const VectorUCO& def_speed = VectorUCO(),
	       const bool def_goalie = false,
	       const FloatUCO def_stamina = FloatUCO(),
	       const FloatUCO def_effort = FloatUCO(),
	       const FloatUCO def_recover = FloatUCO(),
	       //const UInt def_dashes = 0,
	       //const UInt def_kicks = 0,
	       //const UInt def_says = 0,
	       //const UInt def_turns = 0,
	       //const UInt def_turn_necks = 0,
	       const UIntUnknown def_hear_capacity = UIntUnknown(),
	       const ViewWidth def_view_width = VW_UNKNOWN,
	       const ViewQuality def_view_quality = VQ_UNKNOWN);
  AgentObject(const AgentObject& obj);
  virtual ~AgentObject();

  AgentObject& operator= (const AgentObject& obj);

  void reset();

  void setGoalie(const bool g);
  bool isGoalie() const;
  
  void setStamina(const FloatUCO new_stamina);
  FloatUCO getStamina() const;

  void setEffort(const FloatUCO new_effort);
  FloatUCO getEffort() const;

  void setRecover(const FloatUCO new_recover);
  FloatUCO getRecover() const;

  void setDashes(const UInt new_dashes);
  UInt getDashes() const;

  void setKicks(const UInt new_kicks);
  UInt getKicks() const;

  void setSays(const UInt new_says);
  UInt getSays() const;

  void setTurns(const UInt new_turns);
  UInt getTurns() const;

  void setTurnNecks(const UInt new_turn_necks);
  UInt getTurnNecks() const;

  void setHearCapacityFromTeammate(const UIntUnknown new_hear_capacity);
  UIntUnknown getHearCapacityFromTeammate() const;

  void setHearCapacityFromOpponent(const UIntUnknown new_hear_capacity);
  UIntUnknown getHearCapacityFromOpponent() const;

  void setCyclesSinceCatch(const UInt new_cycs_since_catch);
  UInt getCyclesSinceCatch() const;

  void setCyclesSinceSay(const UInt new_cycs_since_say);
  UInt getCyclesSinceSay() const;

  void setViewWidth(const ViewWidth new_view_width);
  ViewWidth getViewWidth() const;
  
  void setViewQuality(const ViewQuality new_view_quality);
  ViewQuality getViewQuality() const;

  /*
  void setLastSensetype(const t_sense new_last_sense_type);
  t_sense getLastSensetype() const;
  
  void setLastSensetime(const Time& new_last_sense_time);
  Time getLastSensetime() const;
  
  void setLastSightTime(const Time& new_last_sight_time);
  Time getLastSightTime() const;

  void setLastHearTime(const Time& new_last_hear_time);
  Time getLastHearTime() const;
  
  void setLastHearTeammateTime(const Time& new_last_hear_time);
  Time getLastHearTeammateTime() const;
  
  void setLastHearOpponentTime(const Time& new_last_hear_time);
  Time getLastHearOpponentTime() const;
  
  void setLastHearRefereeTime(const Time& new_last_hear_time);
  Time getLastHearRefereeTime() const;
  
  void setLastActTime(const Time& new_last_act_time);
  Time getLastActTime() const;

  void setLastCommand(const CCommand* const new_last_command);
  const CCommand* const getLastCommand() const;
  
  void setLastExecutedCommand(const CCommand* const new_last_exec_cmd);
  const CCommand* const getLastExecutedCommand() const;

  void setNextTurnNeck(const CTurnNeckCommand& new_turn_neck);
  CTurnNeckCommand getNextTurnNeck() const;
  
  const CMessages& SentMessages() const;
  CMessages& SentMessages();
  */

  const std::string& getLastCoachMessage() const;
  void setLastCoachMessage(const std::string& msg);

  const std::string& getLastPlayerMessage() const;
  void setLastPlayerMessage(const std::string& msg);

  
protected:
  bool outside;
  bool goalie;

  FloatUCO stamina;
  FloatUCO effort;
  FloatUCO recover;
  UInt dashes;
  UInt kicks;
  UInt says;
  UInt turns;
  UInt turnNecks;
  ViewWidth viewWidth;
  ViewQuality viewQuality;

  UIntUnknown hearCapacityFromTeammate;
  UIntUnknown hearCapacityFromOpponent;
  UInt cyclesSinceCatch;
  UInt cyclesSinceSay;

  bool defaultGoalie;
  FloatUCO defaultStamina;
  FloatUCO defaultEffort;
  FloatUCO defaultRecover;
  UIntUnknown defaultHearCapacity;
  ViewWidth defaultViewWidth;
  ViewQuality defaultViewQuality;

  std::string lastCoachMessage;
  std::string lastPlayerMessage;
  
  /*
  t_sense lastSenseType;
  Time lastSenseTime;
  Time lastSightTime;
  Time lastHearTime;
  Time lastHearTeammateTime;
  Time lastHearOpponentTime;
  Time lastHearRefereeTime;
  Time lastActTime;
  Time lastNeckMovement;
  bool doTurnNeck;
  
  CMessages sentMessages;
  CMessages receivedMessages;
  */
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "AgentObject.icc"
#endif


#endif // !AGENTOBJECT_H
