/* -*- Mode: C++ -*-
 * $Id: AngleDegreesUnknown.h,v 1.4 2000/09/18 15:20:04 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: AngleDegreesUnknown.h
 * Unitname: Library
 * $Revision: 1.4 $
 * Created by: Fredrik Heintz 2000-08-31
 * Last modified by $Author: frehe $ $Date: 2000/09/18 15:20:04 $
 *
 *
 * HISTORY
 *
 * $Log: AngleDegreesUnknown.h,v $
 * Revision 1.4  2000/09/18 15:20:04  frehe
 * Due to a problem with namespaces the name for atan2, atan, acos and asin is changed to ATan2, ATan, ASin and ACos.
 *
 * Revision 1.3  2000/09/04 15:21:48  frehe
 * Changed the name of the trigonometric functions to the same as in the standard math library (sin, cos, tan, asin, acos, atan, and atan2).
 *
 * Revision 1.2  2000/09/01 03:23:51  frehe
 * Made minor changes.
 *
 * Revision 1.1  2000/09/01 01:24:36  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file AngleDegreesUnknown.h
 * Include file for the template AngleDegreesUnknown.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.4 $
 * $Author: frehe $
 * $Date: 2000/09/18 15:20:04 $
 */


#ifndef ANGLEDEGREESUNKNOWN_H
#define ANGLEDEGREESUNKNOWN_H


#include "AngleDegrees.h"
#include "WithUnknown.h"
#include "UnknownValueException.h"
#include "rs_throw.h"


#define DEGUNKNOWN_HEAD std::string("#DegUnknown")


RS_BEGIN_NAMESPACE


/**
 * @class AngleDegreesUnknown AngleDegreesUnknown.h
 * Template class for representing angles in degrees
 * that can be unknown.
 * The template argument only defines what type the
 * angles should be represented by.
 * The only demands on the template argument T is that the trigonometric
 * and arithmetic functions are defined on it.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.4 $
 * $Author: frehe $
 * $Date: 2000/09/18 15:20:04 $
 */
template<class T> class AngleDegreesUnknown : private AngleDegrees<T> {
public:
  AngleDegreesUnknown();
  AngleDegreesUnknown(const T& ang);
  AngleDegreesUnknown(const WithUnknown<T>& ang);
  AngleDegreesUnknown(const AngleDegrees<T>& ang);
  AngleDegreesUnknown(const AngleDegreesUnknown<T>& ang);
  AngleDegreesUnknown(const AngleRadians<T>& ang);

  AngleDegreesUnknown<T>& makeUnknown();
  bool isKnown() const;
  bool isUnknown() const;
  
  AngleDegrees<T> getAngle() const throw(UnknownValueException);
  T getRad() const throw(UnknownValueException);
  T getDeg() const throw(UnknownValueException);
  //WithUnknown<T> getRad() const;
  //WithUnknown<T> getDeg() const;

  // I don't think I have to define all the operators for T too!
  // No but the efficiency is a little bit better.
  AngleDegreesUnknown<T> operator -() const;
  AngleDegreesUnknown<T> operator +(const AngleDegreesUnknown<T>&) const;
  AngleDegreesUnknown<T> operator +(const T&) const;
  AngleDegreesUnknown<T> operator -(const AngleDegreesUnknown<T>&) const;
  AngleDegreesUnknown<T> operator -(const T&) const;
  AngleDegreesUnknown<T> operator *(const AngleDegreesUnknown<T>&) const;
  AngleDegreesUnknown<T> operator *(const T&) const;
  AngleDegreesUnknown<T> operator /(const AngleDegreesUnknown<T>&) const;
  AngleDegreesUnknown<T> operator /(const T&) const;
  
  AngleDegreesUnknown<T>&  operator  =(const AngleDegreesUnknown<T>&);
  AngleDegreesUnknown<T>&  operator  =(const AngleRadians<T>&);
  AngleDegreesUnknown<T>&  operator  =(const T&);

  AngleDegreesUnknown<T>&  operator ++();
  AngleDegreesUnknown<T>&  operator ++(int);
  AngleDegreesUnknown<T>&  operator --();
  AngleDegreesUnknown<T>&  operator --(int);
  AngleDegreesUnknown<T>&  operator +=(const AngleDegreesUnknown<T>&);
  AngleDegreesUnknown<T>&  operator +=(const T&);
  AngleDegreesUnknown<T>&  operator -=(const AngleDegreesUnknown<T>&);
  AngleDegreesUnknown<T>&  operator -=(const T&);
  AngleDegreesUnknown<T>&  operator *=(const AngleDegreesUnknown<T>&);
  AngleDegreesUnknown<T>&  operator *=(const T&);
  AngleDegreesUnknown<T>&  operator /=(const AngleDegreesUnknown<T>&);
  AngleDegreesUnknown<T>&  operator /=(const T&);
  
  bool operator ! () const;
  bool operator ==(const AngleDegreesUnknown<T>&) const;
  bool operator ==(const T&) const;
  bool operator !=(const AngleDegreesUnknown<T>&) const;
  bool operator !=(const T&) const;
  bool operator > (const AngleDegreesUnknown<T>&) const;
  bool operator > (const T&) const;
  bool operator >=(const AngleDegreesUnknown<T>&) const;
  bool operator >=(const T&) const;
  bool operator < (const AngleDegreesUnknown<T>&) const;
  bool operator < (const T&) const;
  bool operator <=(const AngleDegreesUnknown<T>&) const;
  bool operator <=(const T&) const;
  
protected:
  bool unknown;
};


template <class T> 
inline AngleDegreesUnknown<T>& AngleDegreesUnknown<T>::makeUnknown()
{
  unknown = true;
  return *this;
}


template <class T> 
inline bool AngleDegreesUnknown<T>::isKnown() const
{
  return !unknown;
}


template <class T> 
inline bool AngleDegreesUnknown<T>::isUnknown() const
{
  return unknown;
}


template<class T>
inline AngleDegrees<T> AngleDegreesUnknown<T>::getAngle() const throw(UnknownValueException)
{
  if ( unknown ) {
    RS_THROW(UnknownValueException, "Tried to take the value of an unknown AngleDegrees");
    return AngleDegrees<T>();
  } else
    return AngleDegrees<T>(*this);
}


template<class T>
inline T AngleDegreesUnknown<T>::getRad() const throw(UnknownValueException)
{
  if ( unknown ) {
    RS_THROW(UnknownValueException, "Tried to take the value of an unknown AngleDegrees");
    return T();
  } else
    return AngleDegrees<T>::getRad();
}


template<class T>
inline T AngleDegreesUnknown<T>::getDeg() const throw(UnknownValueException)
{
  if ( unknown ) {
    RS_THROW(UnknownValueException, "Tried to take the value of an unknown AngleDegrees");
    return T();
  } else
    return AngleDegrees<T>::getDeg();
}


template<class T>
inline AngleDegreesUnknown<T> AngleDegreesUnknown<T>::operator -() const
{
  if ( unknown )
    return AngleDegreesUnknown<T>();
  else
    return AngleDegreesUnknown<T>(AngleDegrees<T>::operator-());
}


template<class T>
inline AngleDegreesUnknown<T> AngleDegreesUnknown<T>::operator +(const AngleDegreesUnknown<T>& ang) const
{
  if ( unknown || ang.unknown )
    return AngleDegreesUnknown<T>();
  else
    return AngleDegreesUnknown<T>(AngleDegrees<T>::operator+(ang));
}


template<class T>
inline AngleDegreesUnknown<T> AngleDegreesUnknown<T>::operator +(const T& ang) const
{
  if ( unknown )
    return AngleDegreesUnknown<T>();
  else
    return AngleDegreesUnknown<T>(AngleDegrees<T>::operator+(ang));
}


template<class T>
inline AngleDegreesUnknown<T> AngleDegreesUnknown<T>::operator -(const AngleDegreesUnknown<T>& ang) const
{
  if ( unknown || ang.unknown )
    return AngleDegreesUnknown<T>();
  else
    return AngleDegreesUnknown<T>(AngleDegrees<T>::operator-(ang));
}


template<class T>
inline AngleDegreesUnknown<T> AngleDegreesUnknown<T>::operator -(const T& ang) const
{
  if ( unknown )
    return AngleDegreesUnknown<T>();
  else
    return AngleDegreesUnknown<T>(AngleDegrees<T>::operator-(ang));
}


template<class T>
inline AngleDegreesUnknown<T> AngleDegreesUnknown<T>::operator *(const AngleDegreesUnknown<T>& ang) const
{
  if ( unknown || ang.unknown )
    return AngleDegreesUnknown<T>();
  else
    return AngleDegreesUnknown<T>(AngleDegrees<T>::operator*(ang));
}


template<class T>
inline AngleDegreesUnknown<T> AngleDegreesUnknown<T>::operator *(const T& ang) const
{
  if ( unknown )
    return AngleDegreesUnknown<T>();
  else
    return AngleDegreesUnknown<T>(AngleDegrees<T>::operator*(ang));
}


template<class T>
inline AngleDegreesUnknown<T> AngleDegreesUnknown<T>::operator /(const AngleDegreesUnknown<T>& ang) const
{
  if ( unknown || ang.unknown )
    return AngleDegreesUnknown<T>();
  else
    return AngleDegreesUnknown<T>(AngleDegrees<T>::operator/(ang));
}


template<class T>
inline AngleDegreesUnknown<T> AngleDegreesUnknown<T>::operator /(const T& ang) const
{
  if ( unknown )
    return AngleDegreesUnknown<T>();
  else
    return AngleDegreesUnknown<T>(AngleDegrees<T>::operator/(ang));
}


template<class T>
inline AngleDegreesUnknown<T> operator +(const T& op1, const AngleDegreesUnknown<T>& op2)
{
  return op2 + op1;
}


template<class T>
inline AngleDegreesUnknown<T> operator -(const T& op1, const AngleDegreesUnknown<T>& op2)
{
  return (-op2) + op1;
}


template<class T>
inline AngleDegreesUnknown<T> operator *(const T& op1, const AngleDegreesUnknown<T>& op2)
{
  return op2 * op1;
}


template<class T>
inline AngleDegreesUnknown<T> operator /(const T& op1, const AngleDegreesUnknown<T>& op2)
{
  if ( unknown || op2.unknown )
    return AngleDegreesUnknown<T>();
  else
    return AngleDegreesUnknown<T>(op1 / op2.getAngle());
}


template<class T>
inline AngleDegreesUnknown<T> operator +(const AngleDegrees<T>& op1, const AngleDegreesUnknown<T>& op2)
{
  return op2 + op1;
}


template<class T>
inline AngleDegreesUnknown<T> operator -(const AngleDegrees<T>& op1, const AngleDegreesUnknown<T>& op2)
{
  return (-op2) + op1;
}


template<class T>
inline AngleDegreesUnknown<T> operator *(const AngleDegrees<T>& op1, const AngleDegreesUnknown<T>& op2)
{
  return op2 * op1;
}


template<class T>
inline AngleDegreesUnknown<T> operator /(const AngleDegrees<T>& op1, const AngleDegreesUnknown<T>& op2)
{
  if ( unknown || op2.unknown )
    return AngleDegreesUnknown<T>();
  else
    return AngleDegreesUnknown<T>(op1 / op2.getAngle());
}


// assign, no range check
template<class T>
inline AngleDegreesUnknown<T>& AngleDegreesUnknown<T>::operator =(const AngleDegreesUnknown<T>& ang)
{
  if ( &ang != this ) {
    AngleDegrees<T>::operator=(ang);
    unknown = ang.unknown;
  }
  
  return *this;
}


// assign, range check
template<class T>
inline AngleDegreesUnknown<T>& AngleDegreesUnknown<T>::operator =(const T& d)
{
  AngleDegrees<T>::operator=(d);
  unknown = false;
  return *this;
}


template<class T>
inline AngleDegreesUnknown<T>& AngleDegreesUnknown<T>::operator ++()
{
  AngleDegrees<T>::operator++();
  return *this;
}


template<class T>
inline AngleDegreesUnknown<T>& AngleDegreesUnknown<T>::operator ++(int i)
{
  AngleDegrees<T>::operator++(i);
  return *this;
}


template<class T>
inline AngleDegreesUnknown<T>& AngleDegreesUnknown<T>::operator --()
{ 
  AngleDegrees<T>::operator--();
  return *this;
}


template<class T>
inline AngleDegreesUnknown<T>& AngleDegreesUnknown<T>::operator --(int i)
{
  AngleDegrees<T>::operator--(i);
  return *this;
}


template<class T>
inline AngleDegreesUnknown<T>& AngleDegreesUnknown<T>::operator +=(const AngleDegreesUnknown<T>& ang)
{
  AngleDegrees<T>::operator+=(ang);
  unknown = unknown && ang.unknown;
  return *this;
}


template<class T>
inline AngleDegreesUnknown<T>& AngleDegreesUnknown<T>::operator +=(const T& ang)
{
  AngleDegrees<T>::operator+=(ang);
  unknown = unknown;
  return *this;
}


template<class T>
inline AngleDegreesUnknown<T>& AngleDegreesUnknown<T>::operator -=(const AngleDegreesUnknown<T>& ang)
{
  AngleDegrees<T>::operator-=(ang);
  unknown = unknown && ang.unknown;
  return *this;
}


template<class T>
inline AngleDegreesUnknown<T>& AngleDegreesUnknown<T>::operator -=(const T& ang)
{
  AngleDegrees<T>::operator-=(ang);
  unknown = unknown;
  return *this;
}


template<class T>
inline AngleDegreesUnknown<T>& AngleDegreesUnknown<T>::operator *=(const AngleDegreesUnknown<T>& ang)
{
  AngleDegrees<T>::operator*=(ang);
  unknown = unknown && ang.unknown;
  return *this;
}


template<class T>
inline AngleDegreesUnknown<T>& AngleDegreesUnknown<T>::operator *=(const T& ang)
{
  AngleDegrees<T>::operator*=(ang);
  unknown = unknown;
  return *this;
}


template<class T>
inline AngleDegreesUnknown<T>& AngleDegreesUnknown<T>::operator /=(const AngleDegreesUnknown<T>& ang)
{
  AngleDegrees<T>::operator/=(ang);
  unknown = unknown && ang.unknown;
  return *this;
}


template<class T>
inline AngleDegreesUnknown<T>& AngleDegreesUnknown<T>::operator /=(const T& ang)
{
  AngleDegrees<T>::operator/=(ang);
  unknown = unknown;
  return *this;
}


template<class T>
inline bool AngleDegreesUnknown<T>::operator !() const
{
  return (unknown || !a);
}


template<class T>
inline bool AngleDegreesUnknown<T>::operator ==(const AngleDegreesUnknown<T>& ang) const
{
  return ( ( unknown && ang.unknown ) ||
	   ( !unknown && ang.unknown && a == ang.a ) );
}


template<class T>
inline bool AngleDegreesUnknown<T>::operator ==(const T& ang) const
{
  return ( !unknown && a == ang );
}


template<class T>
inline bool AngleDegreesUnknown<T>::operator !=(const AngleDegreesUnknown<T>& ang) const
{
  return !( a == ang );
}


template<class T>
inline bool AngleDegreesUnknown<T>::operator !=(const T& ang) const
{
  return !( a == ang );
}


template<class T>
inline bool AngleDegreesUnknown<T>::operator >(const AngleDegreesUnknown<T>& ang) const
{
  if ( unknown && !ang.unknown )
    return true;
  else if ( !unknown && !ang.unknown )
    return (a > ang.a);
  else
    return false;
}


template<class T>
inline bool AngleDegreesUnknown<T>::operator >(const T& ang) const
{
  if ( unknown )
    return true;
  else
    return (a > ang);
}


template<class T>
inline bool AngleDegreesUnknown<T>::operator >=(const AngleDegreesUnknown<T>& ang) const
{
  return !( a < ang );
}


template<class T>
inline bool AngleDegreesUnknown<T>::operator >=(const T& ang) const
{
  return !( a < ang );
}


template<class T>
inline bool AngleDegreesUnknown<T>::operator <(const AngleDegreesUnknown<T>& ang) const
{
  if ( !unknown && ang.unknown )
    return true;
  else if ( !unknown && !ang.unknown )
    return (a < ang.a);
  else
    return false;
}


template<class T>
inline bool AngleDegreesUnknown<T>::operator <(const T& ang) const
{
  if ( unknown )
    return false;
  else
    return (a < ang);
}


template<class T>
inline bool AngleDegreesUnknown<T>::operator <=(const AngleDegreesUnknown<T>& ang) const
{
  return !( a > ang );
}


template<class T>
inline bool AngleDegreesUnknown<T>::operator <=(const T& ang) const
{
  return !( a > ang );
}


template<class T>
inline bool operator ==(const T& op1, const AngleDegreesUnknown<T>& op2)
{
  return (op2 == op1);
}


template<class T>
inline bool operator !=(const T& op1, const AngleDegreesUnknown<T>& op2)
{
  return (op2 != op1);
}


template<class T>
inline bool operator <(const T& op1, const AngleDegreesUnknown<T>& op2)
{
  return (op2 > op1);
}


template<class T>
inline bool operator <=(const T& op1, const AngleDegreesUnknown<T>& op2)
{
  return (op2 >= op1);
}


template<class T>
inline bool operator >(const T& op1, const AngleDegreesUnknown<T>& op2) 
{
  return (op2 < op1);
}


template<class T>
inline bool operator >=(const T& op1, const AngleDegreesUnknown<T>& op2)
{
  return (op2 <= op1);
}


template<class T>
inline std::ostream& operator <<(std::ostream& os, const AngleDegreesUnknown<T>& ang)
{
  if ( ang.isUnknown() )
    return os << "UNKNOWN(AngleDegrees)";
  else {
#ifdef DEGUNKNOWN_HEAD
    return os << DEGUNKNOWN_HEAD << "(" << ang.getDeg() << ")";
#else
    return os << ang.getDeg();
#endif
  }
}


template<class T>
inline std::istream& operator >>(std::istream& is, AngleDegreesUnknown<T>& ang)
{
  T inp;
  return is >> inp;
  if (is) ang = AngleDegreesUnknown<T>(inp);
}


template<class T>
inline const std::string ToString(const AngleDegreesUnknown<T>& ang)
{
  if ( ang.isUnknown() )
    return "UNKNOWN(AngleDegrees)";
  else {
#ifdef DEGUNKNOWN_HEAD
    return DEGUNKNOWN_HEAD + "(" + ToString(ang.getDeg()) + ")";
#else
    return ToString(ang.getDeg());
#endif
  }
}


template<class T>
AngleDegreesUnknown<T>::AngleDegreesUnknown()
  : AngleDegrees<T>(),
    unknown(true)
{
}


template<class T>
AngleDegreesUnknown<T>::AngleDegreesUnknown(const T& ang)
  : AngleDegrees<T>(ang),
    unknown(false)
{
}


template<class T>
AngleDegreesUnknown<T>::AngleDegreesUnknown(const WithUnknown<T>& ang)
  : AngleDegrees<T>(),
    unknown(ang.isUnknown())
{
  if ( !unknown )
    a = ang.getValue();
}


template<class T>
AngleDegreesUnknown<T>::AngleDegreesUnknown(const AngleDegrees<T>& ang)
  : AngleDegrees<T>(ang),
    unknown(false)
{
}


template<class T>
AngleDegreesUnknown<T>::AngleDegreesUnknown(const AngleDegreesUnknown<T>& ang)
  : AngleDegrees<T>(ang),
    unknown(ang.unknown)
{
}


template<class T>
AngleDegreesUnknown<T>::AngleDegreesUnknown(const AngleRadians<T>& ang)
  : AngleDegrees<T>(ang),
    unknown(false)
{
}


// assign, no range check
template<class T>
AngleDegreesUnknown<T>& AngleDegreesUnknown<T>::operator =(const AngleRadians<T>& ang)
{
  AngleDegrees<T>::operator=(ang);
  unknown = false;
  return *this;
}


template<class T>
inline WithUnknown<T> sin(const AngleDegreesUnknown<T>& x) 
{
  if ( x.isUnknown() )
    return WithUnknown<T>();
  else
    return WithUnknown<T>(sin(x.getAngle()));
}


template<class T>
inline WithUnknown<T> cos(const AngleDegreesUnknown<T>& x)
{
  if ( x.isUnknown() )
    return WithUnknown<T>();
  else
  return WithUnknown<T>(cos(x.getAngle()));
}


template<class T>
inline WithUnknown<T> tan(const AngleDegreesUnknown<T>& x)
{
  if ( x.isUnknown() )
    return WithUnknown<T>();
  else
    return WithUnknown<T>(tan(x.getAngle()));
}


#ifdef RS_TRIG_IN_DEG


template<class T>
AngleDegreesUnknown<T> ASin(const WithUnknown<T>& op)
{
  if ( op.isUnknown() )
    return AngleDegreesUnknown<T>();
  else
    return AngleDegreesUnknown<T>(ASin(op.getValue()));
}


template<class T>
AngleDegreesUnknown<T> ACos(const WithUnknown<T>& op)
{
  if ( op.isUnknown() )
    return AngleDegreesUnknown<T>();
  else
    return AngleDegreesUnknown<T>(ACos(op.getValue()));
}


template<class T>
AngleDegreesUnknown<T> ATan(const WithUnknown<T>& op)
{
  if ( op.isUnknown() )
    return AngleDegreesUnknown<T>();
  else
    return AngleDegreesUnknown<T>(ATan(op.getValue()));
}


template<class T>
AngleDegreesUnknown<T> ATan2(const WithUnknown<T>& op1, const WithUnknown<T>& op2)
{
  if ( op1.isUnknown() ||
       op2.isUnknown() )
    return AngleDegreesUnknown<T>();
  else
    return AngleDegreesUnknown<T>(ATan2(op1.getValue(), op2.getValue()));
}


#endif // RS_TRIG_IN_DEG


/*
#ifdef RS_TRIG_IN_RAD

template<class T>
WithUnknown< AngleRadians<T>> ASin(const WithUnknown<T>& op)
{
  if ( op.isUnknown() )
    return WithUnknown< AngleRadians<T>>();
  else
    return WithUnknown< AngleRadians<T>>(ASin(op.getValue()));
}


template<class T>
WithUnknown< AngleRadians<T>> ACos(const WithUnknown<T>& op)
{
  if ( op.isUnknown() )
    return WithUnknown< AngleRadians<T>>();
  else
    return WithUnknown< AngleRadians<T>>(ACos(op.getValue()));
}


template<class T>
WithUnknown< AngleRadians<T>> ATan(const WithUnknown<T>& op)
{
  if ( op.isUnknown() )
    return WithUnknown< AngleRadians<T>>();
  else
    return WithUnknown< AngleRadians<T>>(ATan(op.getValue()));
}


template<class T>
WithUnknown< AngleRadians<T>> ATan2(const WithUnknown<T>& op1, const WithUnknown<T>& op2)
{
  if ( op1.isUnknown() ||
       op2.isUnknown() )
    return WithUnknown< AngleRadians<T> >();
  else
    return WithUnknown< AngleRadians<T>>(ATan2(op1.getValue(), op2.getValue()));
}

#endif // RS_TRIG_IN_RAD
*/


RS_END_NAMESPACE


#endif // !ANGLEDEGREESUNKNOWN_H
