/* -*- Mode: C++ -*-
 * $Id: BallObject.h,v 1.1 2000/09/01 03:24:30 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: BallObject.h
 * Unitname: Library
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/01 03:24:30 $
 *
 *
 * HISTORY
 *
 * $Log: BallObject.h,v $
 * Revision 1.1  2000/09/01 03:24:30  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file BallObject.h
 * Include file for the class BallObject.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/01 03:24:30 $
 */


#ifndef BALLOBJECT_H
#define BALLOBJECT_H


#include "MobileObject.h"


RS_BEGIN_NAMESPACE


/**
 * @class BallObject BallObject.h
 * Class used to represent the ball.
 *
 * <!-- @invariant The class invariant is... -->
 *
 * @todo Make a toString and operator<<
 *
 * @par Unit:
 *      Library
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/01 03:24:30 $
 */
class BallObject : public MobileObject {
public:
  BallObject(const PointUCO& def_point = PointUCO(),
	     const VectorUCO& def_vector = VectorUCO(),
	     const FloatUCO def_dist_changed = FloatUCO(),
	     const AngleDegUCO& def_dir_changed = AngleDegUCO(),
	     const VectorUCO& def_speed = VectorUCO(),
	     const PointUnknown def_caught_ball_position = PointUnknown(),
	     const VectorUnknown def_relative_ball_position = VectorUnknown(),
	     const TimeUnknown def_last_seen_moving_time = TimeUnknown());
  BallObject(const BallObject& obj);

  BallObject& operator= (const BallObject& obj);

  void reset();

  void setCaughtBallPosition(const PointUnknown& pt);
  PointUnknown getCaughtBallPosition();
  
  void setRelativeBallPosition(const VectorUnknown& vec);
  VectorUnknown getRelativeBallPosition() const;

  void setLastSeenMovingTime(const TimeUnknown& tm);
  TimeUnknown getLastSeenMovingTime() const;

private:
  PointUnknown caughtBallPosition;
  VectorUnknown relativeBallPosition;
  TimeUnknown lastSeenMovingTime;

  PointUnknown defaultCaughtBallPosition;
  VectorUnknown defaultRelativeBallPosition;
  TimeUnknown defaultLastSeenMovingTime;
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "BallObject.icc"
#endif


#endif // !BALLOBJECT_H
