/* -*- Mode: C++ -*-
 * $Id: Command.h,v 1.5 2000/09/09 11:49:44 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: Command.h
 * Unitname: Library
 * $Revision: 1.5 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/09 11:49:44 $
 *
 *
 * HISTORY
 *
 * $Log: Command.h,v $
 * Revision 1.5  2000/09/09 11:49:44  frehe
 * Added a new level in the command hieararchy, namely ImmediateCommand and PeriodicCommand.
 *
 * Revision 1.4  2000/09/03 11:10:26  frehe
 * Made the clone method constant.
 *
 * Revision 1.3  2000/09/01 11:43:00  frehe
 * Made minor changes in documentation.
 *
 * Revision 1.2  2000/09/01 01:24:37  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file Command.h
 * Include file for the class Command.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.5 $
 * $Author: frehe $
 * $Date: 2000/09/09 11:49:44 $
 */


#ifndef COMMAND_H
#define COMMAND_H


#include <string>
#include <iostream>
#include "machine_dependent_compiler_directives.h"
#include "library_types.h"


RS_BEGIN_NAMESPACE


class ServerCommand;
class ControlCommand;


/**
 * @class Command Command.h
 * Class used as the base class of all commands defined in
 * RoboSoc.
 *
 * <!-- @invariant The class invariant is... -->
 *
 * <!-- @todo -->
 *
 * @par Unit:
 *      Library
 * $Revision: 1.5 $
 * $Author: frehe $
 * $Date: 2000/09/09 11:49:44 $
 */
class Command {
public:
  /** Create a command of the type @a c_type. */
  Command(const CommandType c_type);

  /** Create a command that is a copy of @a obj. */
  Command(const Command& obj);

  /** Destroys the command. */
  virtual ~Command();

  
  /** Assignment operator. Assign @a obj to the command. */
  Command& operator =(const Command& obj);

  
  /** Returns a ServerCommand pointer if the object is
    actually a ServerCommand otherwise a NULL pointer. */
  virtual ServerCommand* getServerCommand() = 0;

  /** Returns a ControlCommand pointer if the object is
    actually a ControlCommand otherwise a NULL pointer. */
  virtual ControlCommand* getControlCommand() = 0;

  
  /** Returns @c true iff the Command is a ServerCommand. */
  virtual bool isServerCommand() const = 0;

  /** Returns @c true iff the Command is a ControlCommand. */
  virtual bool isControlCommand() const = 0;

  
  /** Get the type of the command. Should be made obsolete in
   future versions of RoboSoc. */
  CommandType getType() const;

  /** Get a string description of the command. */
  virtual const std::string getDescription() const = 0;

  
  /** Returns a new copy of the command.
    REMEMBER TO DELETE IT, OTHERWISE YOU WILL HAVE A MEMORY LEAK!!! */
  virtual Command* clone() const = 0;

protected:
  /** Used to copy the information stored in a command. */
  void copyCommand(const Command& obj);

private:
  CommandType cmdType;
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "Command.icc"
#endif


#endif // !COMMAND_H
