/* -*- Mode: C++ -*-
 * $Id: ControlCommand.h,v 1.6 2000/09/05 09:36:40 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: ControlCommand.h
 * Unitname: Library
 * $Revision: 1.6 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/05 09:36:40 $
 *
 *
 * HISTORY
 *
 * $Log: ControlCommand.h,v $
 * Revision 1.6  2000/09/05 09:36:40  frehe
 * Made minor changes to improve the documentation.
 *
 * Revision 1.5  2000/09/03 11:10:26  frehe
 * Made the clone method constant.
 *
 * Revision 1.4  2000/09/02 15:29:02  frehe
 * Made more minor changes.
 *
 * Revision 1.3  2000/09/01 11:43:00  frehe
 * Made minor changes in documentation.
 *
 * Revision 1.2  2000/09/01 01:24:37  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file ControlCommand.h
 * Include file for the class ControlCommand.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.6 $
 * $Author: frehe $
 * $Date: 2000/09/05 09:36:40 $
 */


#ifndef CONTROLCOMMAND_H
#define CONTROLCOMMAND_H


#include "Command.h"


RS_BEGIN_NAMESPACE


class SetCyclesSinceIOBeforeShutdown;
class SetMinTicksBeforeSendingCommand;
class SetMaxTicksBeforeSendingCommand;
class SetTicksBeforeCommandWarning;
class SetTicksBetweenCycles;
class SetTicksBetweenSenses;
class SetTicksBetweenVisuals;


/**
 * @class ControlCommand ControlCommand.h
 * Class used as the base class of all control commands
 * defined in RoboSoc.
 *
 * <!-- @invariant The class invariant is... -->
 *
 * <!-- @todo -->
 *
 * @par Unit:
 *      Library
 * $Revision: 1.6 $
 * $Author: frehe $
 * $Date: 2000/09/05 09:36:40 $
 */
class ControlCommand : public Command
{
public:
  ControlCommand(const CommandType);
  ControlCommand(const ControlCommand& obj);
  virtual ~ControlCommand();

  ControlCommand& operator =(const ControlCommand& obj);

  virtual ServerCommand* getServerCommand();
  virtual ControlCommand* getControlCommand();

  virtual bool isServerCommand() const;
  virtual bool isControlCommand() const;
  
  /** Returns a new copy of the command.
    REMEMBER TO DELETE IT, OTHERWISE YOU WILL HAVE A MEMORY LEAK!!! */
  virtual ControlCommand* clone() const = 0;

  // Since these commands should not change very often
  // I include get methods for them all.
  virtual SetCyclesSinceIOBeforeShutdown* getSetCyclesSinceIOBeforeShutdown();
  virtual SetMinTicksBeforeSendingCommand* getSetMinTicksBeforeSendingCommand();
  virtual SetMaxTicksBeforeSendingCommand* getSetMaxTicksBeforeSendingCommand();
  virtual SetTicksBeforeCommandWarning* getSetTicksBeforeCommandWarning();
  virtual SetTicksBetweenCycles* getSetTicksBetweenCycles();
  virtual SetTicksBetweenSenses* getSetTicksBetweenSenses();
  virtual SetTicksBetweenVisuals* getSetTicksBetweenVisuals();
  
protected:
  //void copyCommand(const ControlCommand& obj);
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "ControlCommand.icc"
#endif


#endif // !CONTROLCOMMAND_H
