/* -*- Mode: C++ -*-
 * $Id: CoordPointUC.h,v 1.4 2000/09/18 15:20:04 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: CoordPointUC.h
 * Unitname: Library
 * $Revision: 1.4 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/18 15:20:04 $
 *
 *
 * HISTORY
 *
 * $Log: CoordPointUC.h,v $
 * Revision 1.4  2000/09/18 15:20:04  frehe
 * Due to a problem with namespaces the name for atan2, atan, acos and asin is changed to ATan2, ATan, ASin and ACos.
 *
 * Revision 1.3  2000/09/04 15:21:49  frehe
 * Changed the name of the trigonometric functions to the same as in the standard math library (sin, cos, tan, asin, acos, atan, and atan2).
 *
 * Revision 1.2  2000/09/01 03:23:52  frehe
 * Made minor changes.
 *
 * Revision 1.1  2000/09/01 01:24:38  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 *
 */

/**
 * @file CoordPointUC.h
 * Include file for the template CoordPointUC.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.4 $
 * $Author: frehe $
 * $Date: 2000/09/18 15:20:04 $
 */


#ifndef COORDPOINTUC_H
#define COORDPOINTUC_H


#include "WithUC.h"
#include "AngleDegreesUC.h"
#include "CoordPointUnknown.h"
#include "CoordVectorUC.h"


#define POINTUC_HEAD std::string("#PointUC")


RS_BEGIN_NAMESPACE


template<class T> class CoordVectorUC;


/**
 * @class CoordPointUC CoordPointUC.h
 * Template class for representing a cartesian two dimensional
 * coordinate that can be unknown and have a confidence factor.
 * The X and Y values are of type WithUC<T>.
 *
 * <!-- @todo -->
 *
 * @par Unit:
 *      Library
 * $Revision: 1.4 $
 * $Author: frehe $
 * $Date: 2000/09/18 15:20:04 $
 */
template<class T> class CoordPointUC {
public:
  CoordPointUC();
  CoordPointUC(const T&, const T&);
  CoordPointUC(const WithUC<T>&, const WithUC<T>&);
  CoordPointUC(const CoordPoint<T>&);
  CoordPointUC(const CoordPointUnknown<T>&);
  CoordPointUC(const CoordPointUC<T>&);
  CoordPointUC(const CoordVectorUnknown<T>&, const CoordPoint<T>&);
  CoordPointUC(const CoordVectorUnknown<T>&, const CoordPointUnknown<T>&);
  CoordPointUC(const CoordVectorUC<T>&, const CoordPointUC<T>&);
  CoordPointUC(const CoordVectorUnknown<T>&, const CoordPointUC<T>&);
  CoordPointUC(const CoordPointUnknown<T>&, const CoordVectorUnknown<T>&);
  CoordPointUC(const CoordPointUC<T>&, const CoordVectorUC<T>&);
  explicit CoordPointUC(const CoordVectorUnknown<T>&);
  explicit CoordPointUC(const CoordVectorUC<T>&);

  bool isKnown() const;
  bool isUnknown() const;
  CoordPointUC<T>& makeUnknown();
  CoordPoint<T> getPoint() const throw(UnknownValueException);

  operator CoordPointUnknown<T>();
  
  CoordPointUC<T> operator - () const;
  CoordPointUC<T> operator - (const CoordPointUC<T>&) const;
  CoordPointUC<T> operator - (const CoordVector<T>&) const;
  CoordPointUC<T> operator - (const CoordVectorUC<T>&) const;
  CoordPointUC<T> operator + (const CoordPointUC<T>&) const;
  CoordPointUC<T> operator + (const CoordVector<T>&) const;
  CoordPointUC<T> operator + (const CoordVectorUC<T>&) const;
  CoordPointUC<T> operator * (const WithUC<T>&) const;
  CoordPointUC<T> operator * (const CoordPointUC<T>&) const;
  CoordPointUC<T> operator / (const WithUC<T>&) const;
  CoordPointUC<T> operator / (const CoordPointUC<T>&) const;

  CoordPointUC<T>& operator =(const CoordPointUC<T>&);
  CoordPointUC<T>& operator =(const CoordPointUnknown<T>&);
  CoordPointUC<T>& operator =(const CoordPoint<T>&);
  CoordPointUC<T>& operator =(const CoordVectorUC<T>&);
  CoordPointUC<T>& operator =(const CoordVectorUnknown<T>&);

  CoordPointUC<T>& operator -=(const CoordPointUC<T>&);
  CoordPointUC<T>& operator -=(const CoordVector<T>&);
  CoordPointUC<T>& operator -=(const CoordVectorUC<T>&);
  CoordPointUC<T>& operator +=(const CoordPointUC<T>&);
  CoordPointUC<T>& operator +=(const CoordVector<T>&);
  CoordPointUC<T>& operator +=(const CoordVectorUC<T>&);
  CoordPointUC<T>& operator *=(const WithUC<T>&);
  CoordPointUC<T>& operator *=(const CoordPointUC<T>&);
  CoordPointUC<T>& operator /=(const WithUC<T>&);
  CoordPointUC<T>& operator /=(const CoordPointUC<T>&);

  bool operator ! () const;
  bool operator ==(const CoordPointUC<T>&) const;
  bool operator !=(const CoordPointUC<T>&) const;

  /* Define these in a separate file. They need not be methods.
  // These use the coordinate system of the soccer field
  bool InFront(const CoordPointUC<T>&) const;  // Equal to LeftOf
  bool Behind(const CoordPointUC<T>&)  const;   // Equal to RightOf
  // These use standar coordinate system
  bool LeftOf(const CoordPointUC<T>&)  const;
  bool RightOf(const CoordPointUC<T>&) const;
  boolbove(const CoordPointUC<T>&)     const;
  bool Below(const CoordPointUC<T>&)   const;
  */
  
  WithUC<T> getX() const;
  WithUC<T> getY() const;
  CoordPointUC<T>& addX(const WithUC<T>& pt_x);
  CoordPointUC<T>& addY(const WithUC<T>& pt_y);
  CoordPointUC<T>& addXY(const WithUC<T>& pt_x,
			      const WithUC<T>& pt_y);
  CoordPointUC<T>& setX(const WithUC<T>& pt_x);
  CoordPointUC<T>& setY(const WithUC<T>& pt_y);
  CoordPointUC<T>& set (const WithUC<T>& pt_x, const WithUC<T>& pt_y);
  
  CoordVectorUC<T> getVectorTo(const CoordPointUC<T>& pt) const;
  AngleDegreesUC<T> getDirectionTo(const CoordPointUC<T>& pt) const;
  WithUC<T> getDistanceTo(const CoordPointUC<T>& pt) const;
  
protected:
  WithUC<T> x;
  WithUC<T> y;  
};

/*
#if CHECK_DIVisION_BY_ZERO == 1
static const CoordPointUC<T> CoordPointUC<T>_DIVisION_BY_ZERO_RESULT(T_DIVisION_BY_ZERO_RESULT,
						   T_DIVisION_BY_ZERO_RESULT);
#endif CHECK_DIVisION_BY_ZERO
*/

template<class T>
CoordPointUC<T>::CoordPointUC()
  : x(),
    y()
{
}

template<class T>
CoordPointUC<T>::CoordPointUC(const T& pt_x, const T& pt_y)
  : x(pt_x),
    y(pt_y)
{
}

template<class T>
CoordPointUC<T>::CoordPointUC(const WithUC<T>& pt_x, const WithUC<T>& pt_y)
  : x(pt_x),
    y(pt_y)
{
}

template<class T>
CoordPointUC<T>::CoordPointUC(const CoordPoint<T>& pt)
  : x(pt.getX()),
    y(pt.getY())
{
}

template<class T>
CoordPointUC<T>::CoordPointUC(const CoordPointUnknown<T>& pt)
  : x(pt.getX()),
    y(pt.getY())
{
}

template<class T>
CoordPointUC<T>::CoordPointUC(const CoordPointUC<T>& pt)
  : x(pt.x),
    y(pt.y)
{
}

template<class T>
CoordPointUC<T>::CoordPointUC(const CoordVectorUnknown<T>& v,
			      const CoordPoint<T>& pt)
{
  *this = v + pt;
}

template<class T>
CoordPointUC<T>::CoordPointUC(const CoordVectorUnknown<T>& v,
			      const CoordPointUnknown<T>& pt)
{
  *this = v + pt;
}

template<class T>
CoordPointUC<T>::CoordPointUC(const CoordVectorUnknown<T>& v,
			      const CoordPointUC<T>& pt)
{
  *this = CoordVectorUC<T>(v) + pt;
}

template<class T>
CoordPointUC<T>::CoordPointUC(const CoordVectorUC<T>& v,
			      const CoordPointUC<T>& pt)
{
  *this = v + pt;
}

template<class T>
CoordPointUC<T>::CoordPointUC(const CoordPointUnknown<T>& pt,
			      const CoordVectorUnknown<T>& v)
{
  *this = v + pt;
}

template<class T>
CoordPointUC<T>::CoordPointUC(const CoordPointUC<T>& pt,
			      const CoordVectorUC<T>& v)
{
  *this = v + pt;
}

template<class T>
CoordPointUC<T>::CoordPointUC(const CoordVectorUnknown<T>& v)
{
  x = v.getLength() * cos(v.getAngle());
  y = v.getLength() * sin(v.getAngle());
}

template<class T>
CoordPointUC<T>::CoordPointUC(const CoordVectorUC<T>& v)
{
  x = v.getLength() * cos(v.getAngle());
  y = v.getLength() * sin(v.getAngle());
}

template<class T>
inline bool CoordPointUC<T>::isKnown() const
{
  return x.isKnown() && y.isKnown();
}

template<class T>
inline bool CoordPointUC<T>::isUnknown() const
{
  return x.isUnknown() && y.isUnknown();
}

template<class T>
inline CoordPointUC<T>& CoordPointUC<T>::makeUnknown()
{
  x.makeUnknown();
  y.makeUnknown();
  return *this;
}

template <class T> 
inline CoordPoint<T> CoordPointUC<T>::getPoint() const throw(UnknownValueException)
{
  if ( isUnknown() ) {
    RS_THROW(UnknownValueException, "Tried to take the value of an unknown point");
    return CoordPoint<T>();
  }  else
    return CoordPoint<T>(x.getValue().getValue(),
			 y.getValue().getValue());
}

template<class T>
inline CoordPointUC<T>::operator CoordPointUnknown<T>()
{
  return CoordPointUnknown<T>(x,y);
}

template<class T>
CoordPointUC<T>& CoordPointUC<T>::operator =(const CoordPoint<T>& pt)
{
  y = pt.getY();
  x = pt.getX();
  return *this;
}

template<class T>
CoordPointUC<T>& CoordPointUC<T>::operator =(const CoordPointUnknown<T>& pt)
{
  y = pt.getY();
  x = pt.getX();
  return *this;
}

template<class T>
CoordPointUC<T>& CoordPointUC<T>::operator =(const CoordPointUC<T>& pt)
{
  if ( this != &pt ) {
    y = pt.y;
    x = pt.x;
  }
  return *this;
}

template<class T>
inline CoordPointUC<T>& CoordPointUC<T>::operator =(const CoordVectorUnknown<T>& vec)
{
  *this = CoordPointUC<T>(vec);
  return *this;
}

template<class T>
inline CoordPointUC<T>& CoordPointUC<T>::operator =(const CoordVectorUC<T>& vec)
{
  *this = CoordPointUC<T>(vec);
  return *this;
}

template<class T>
inline CoordPointUC<T> CoordPointUC<T>::operator -() const
{
  return CoordPointUC<T>(-x,-y);
}

template<class T>
inline CoordPointUC<T> CoordPointUC<T>::operator -(const CoordPointUC<T>& pt) const
{
  return CoordPointUC<T>(x - pt.x, y - pt.y);
}

template<class T>
inline CoordPointUC<T> CoordPointUC<T>::operator -(const CoordVector<T>& v) const
{
  return (-v) + *this;
}

template<class T>
inline CoordPointUC<T> CoordPointUC<T>::operator -(const CoordVectorUC<T>& v) const
{
  return (-v) + *this;
}

template<class T>
inline CoordPointUC<T> operator -(const CoordPoint<T>& pt1,
				  const CoordPointUC<T>& pt2)
{
  return (-pt2) + pt1;
}

template<class T>
inline CoordPointUC<T> operator -(const CoordPointUnknown<T>& pt1,
				  const CoordPointUC<T>& pt2)
{
  return (-pt2) + pt1;
}

template<class T>
inline CoordPointUC<T> CoordPointUC<T>::operator +(const CoordPointUC<T>& pt) const
{
  return CoordPointUC<T>(x + pt.x, y + pt.y);
}

template<class T>
inline CoordPointUC<T> CoordPointUC<T>::operator+(const CoordVector<T>& v) const
{
  return CoordVectorUC<T>(v)+*this;
}

template<class T>
inline CoordPointUC<T> CoordPointUC<T>::operator+(const CoordVectorUC<T>& v) const
{
  return v+*this;
}

template<class T>
inline CoordPointUC<T> operator +(const CoordPoint<T>& pt1,
				  const CoordPointUC<T>& pt2)
{
  return pt2 + pt1;
}

template<class T>
inline CoordPointUC<T> operator +(const CoordPointUnknown<T>& pt1,
				  const CoordPointUC<T>& pt2)
{
  return pt2 + pt1;
}

template<class T>
inline CoordPointUC<T> operator +(const CoordVector<T>& pt1,
				  const CoordPointUC<T>& pt2)
{
  return CoordPointUC<T>(pt1) + pt2;
}

template<class T>
inline CoordPointUC<T> CoordPointUC<T>::operator *(const WithUC<T>& a) const
{
  return CoordPointUC<T>(x * a, y * a);
}

template<class T>
inline CoordPointUC<T> CoordPointUC<T>::operator *(const CoordPointUC<T>& pt) const
{
  return CoordPointUC<T>(x * pt.x, y * pt.y);
}

template<class T>
inline CoordPointUC<T> operator *(const T& a, const CoordPointUC<T>& pt)
{
  return pt * a;
}

template<class T>
inline CoordPointUC<T> operator *(const WithUC<T>& a, const CoordPointUC<T>& pt)
{
  return pt * a;
}

template<class T>
inline CoordPointUC<T> operator *(const CoordPoint<T>& pt1,
				  const CoordPointUC<T>& pt2)
{
  return pt2 * pt1;
}

template<class T>
inline CoordPointUC<T> CoordPointUC<T>::operator /(const WithUC<T>& a) const
{
#if CHECK_DIVisION_BY_ZERO == 1
  if (a == T(0))
    return CoordPointUC_DIVisION_BY_ZERO_RESULT;
  else
#endif
    return CoordPointUC<T>(x / a, y / a);
}

template<class T>
inline CoordPointUC<T> CoordPointUC<T>::operator /(const CoordPointUC<T>& pt) const
{
#if CHECK_DIVisION_BY_ZERO == 1
  if ( pt.x == T(0) ) {
    if ( pt.y == T(0) ) {
      return CoordPointUC_DIVisION_BY_ZERO_RESULT;
    } else {
      return CoordPointUC<T>(T_DIVisION_BY_ZERO_RESULT, y / pt.y);
    }
  } else {
    if ( pt.y == T(0) ) {
      return CoordPointUC<T>(x / pt.x, T_DIVisION_BY_ZERO_RESULT);
    } else {
#endif
      return CoordPointUC<T>(x / pt.x, y / pt.y);
#if CHECK_DIVisION_BY_ZERO == 1      
    }
  }
#endif
}

template<class T>
CoordPointUC<T> operator /(const CoordPoint<T>& pt1, const CoordPointUC<T>& pt2)
{
  if ( pt2.isUnknown() )
    return CoordPointUC<T>();
  else
    return pt1 / pt2.getPoint();
}

template<class T>
inline CoordPointUC<T>& CoordPointUC<T>::operator-=(const CoordPointUC<T>& pt)
{
  x -= pt.x;
  y -= pt.y;
  return *this;
}

template<class T>
inline CoordPointUC<T>& CoordPointUC<T>::operator-=(const CoordVector<T>& vec)
{
  *this = *this - vec;
  return *this;
}

template<class T>
inline CoordPointUC<T>& CoordPointUC<T>::operator-=(const CoordVectorUC<T>& vec)
{
  *this = *this - vec;
  return *this;
}

template<class T>
inline CoordPointUC<T>& CoordPointUC<T>::operator+=(const CoordVector<T>& vec)
{
  *this = *this + CoordPointUC<T>(vec);
  return *this;
}

template<class T>
inline CoordPointUC<T>& CoordPointUC<T>::operator+=(const CoordVectorUC<T>& vec)
{
  *this = *this + vec;
  return *this;
}

template<class T>
inline CoordPointUC<T>& CoordPointUC<T>::operator+=(const CoordPointUC<T>& pt)
{
  x += pt.x;
  y += pt.y;
  return *this;
}

template<class T>
inline CoordPointUC<T>& CoordPointUC<T>::operator*=(const WithUC<T>& a)
{
  x *= a;
  y *= a;
  return *this;
}

template<class T>
inline CoordPointUC<T>& CoordPointUC<T>::operator*=(const CoordPointUC<T>& pt)
{
  x *= pt.x;
  y *= pt.y;
  return *this;
}

template<class T>
CoordPointUC<T>& CoordPointUC<T>::operator/=(const WithUC<T>& a)
{
#if CHECK_DIVisION_BY_ZERO == 1
  if ( a == T(0)) {
    x = T_DIVisION_BY_ZERO_RESULT;
    y = T_DIVisION_BY_ZERO_RESULT;
  } else {
#endif
    x /= a;
    y /= a;
#if CHECK_DIVisION_BY_ZERO == 1
  }
#endif
  return *this;
}

template<class T>
inline CoordPointUC<T>& CoordPointUC<T>::operator/=(const CoordPointUC<T>& pt)
{
#if CHECK_DIVisION_BY_ZERO == 1
  if ( pt.x == T(0) )
    x = T_DIVisION_BY_ZERO_RESULT;
  else
#endif
    x /= pt.x;
  
#if CHECK_DIVisION_BY_ZERO == 1
  if ( pt.y == T(0) )
    y = T_DIVisION_BY_ZERO_RESULT;
  else
#endif
    y /= pt.y;

  return *this;
}

template<class T>
inline bool CoordPointUC<T>::operator !() const
{
  return (!y) && (!x);
}

template<class T>
inline bool CoordPointUC<T>::operator ==(const CoordPointUC<T>& pt) const
{
  return (y == pt.y) && (x == pt.x);
}

template<class T>
inline bool operator ==(const CoordPoint<T>& pt1,
			const CoordPointUC<T>& pt2)
{
  return pt2 == pt1;
}

template<class T>
inline bool CoordPointUC<T>::operator !=(const CoordPointUC<T>& pt) const
{
  return (y != pt.y) || (x != pt.x);
}

template<class T>
inline bool operator !=(const CoordPoint<T>& pt1,
			const CoordPointUC<T>& pt2)
{
  return pt2 != pt1;
}

/*
inline bool CoordPointUC<T>::InFront(const CoordPointUC<T>& pt) const
{
  return (x > pt.x);
}

inline bool CoordPointUC<T>::Behind(const CoordPointUC<T>& pt) const
{
  return (x < pt.x);
}
*/

/*
inline bool CoordPointUC<T>::LeftOf(const CoordPointUC<T>& pt) const
{
  return (x > pt.x);
}

inline bool CoordPointUC<T>::RightOf(const CoordPointUC<T>& pt) const
{
  return (x < pt.x);
}

inline bool CoordPointUC<T>::Above(const CoordPointUC<T>& pt) const
{
  return (y < pt.y);
}

inline bool CoordPointUC<T>::Below(const CoordPointUC<T>& pt) const
{
  return (y > pt.y);
}
*/

template<class T>
inline CoordPointUC<T>& CoordPointUC<T>::addX (const WithUC<T>& pt_x)
{
  x += pt_x;
  return *this;
}

template<class T>
inline CoordPointUC<T>& CoordPointUC<T>::addY (const WithUC<T>& pt_y)
{
  y += pt_y;
  return *this;
}

template<class T>
inline CoordPointUC<T>& CoordPointUC<T>::addXY (const WithUC<T>& pt_x,
						const WithUC<T>& pt_y)
{
  x += pt_x;
  y += pt_y;
  return *this;
}

template<class T>
inline WithUC<T> CoordPointUC<T>::getX() const
{
  return x;
}

template<class T>
inline WithUC<T> CoordPointUC<T>::getY() const
{
  return y;
}

template<class T>
inline CoordPointUC<T>& CoordPointUC<T>::setX(const WithUC<T>& pt_x)
{
  x = pt_x;
  return *this;
}

template<class T>
inline CoordPointUC<T>& CoordPointUC<T>::setY(const WithUC<T>& pt_y)
{
  y = pt_y;
  return *this;
}

template<class T>
inline CoordPointUC<T>& CoordPointUC<T>::set(const WithUC<T>& pt_x,
					     const WithUC<T>& pt_y)
{
  x = pt_x;
  y = pt_y;
  return *this;
}

template<class T>
AngleDegreesUC<T> CoordPointUC<T>::getDirectionTo(const CoordPointUC<T>& pt) const
{
  if ( pt.isUnknown() )
    return AngleDegreesUC<T>();
  else
    return ATan2((pt.y - y), (pt.x - x));
}

template<class T>
inline WithUC<T> CoordPointUC<T>::getDistanceTo(const CoordPointUC<T>& pt) const
{
  return sqrt(sqr(pt.x - x) + sqr(pt.y - y));
}

template<class T>
inline std::ostream& operator<<(std::ostream& os, const CoordPointUC<T>& pt)
{
#ifdef POINTUC_HEAD
  return os << POINTUC_HEAD << "(" << pt.getX() << ", " << pt.getY() << ")";
#else
  return os << "(" << pt.getX() << ", " << pt.getY() << ")";
#endif
}

template<class T>
inline const std::string toString(const CoordPointUC<T>& pt)
{
#ifdef POINTUC_HEAD
  return POINTUC_HEAD + "(" + toString(pt.getX()) + ", " + toString(pt.getY()) + ")";
#else
  return "(" + toString(pt.getX()) + ", " + toString(pt.getY()) + ")";
#endif
}

template<class T>
inline CoordVectorUC<T> CoordPointUC<T>::getVectorTo(const CoordPointUC<T>& pt) const
{
  return CoordVectorUC<T>(*this, pt);
}

template<class T>
std::istream& operator>>(std::istream& is, CoordPointUC<T>& pt)
{
  /* Input formats: f is a T
	(f,f) = (f,f)
  */

  WithUC<T> pt_x;
  WithUC<T> pt_y;
  char c = 0;

  is >> c;
  if (c == '(') {
    is >> pt_x >> c;
    if (c == ',') is >> pt_y >> c;
    if (c != ')') is.clear(std::ios::badbit); //error...;
  }
  else {
    is.clear(std::ios::badbit); //error...;
  }

  if (is) pt = CoordPointUC<T>(pt_x, pt_y);

  return is;
}

// Extend this to be the point on the line between pt1 and pt2 
// with a certain ratio 1:1 is the middle point?
template<class T>
inline CoordPointUC<T> middlePoint(const CoordPointUC<T>& pt1,
					const CoordPointUC<T>& pt2)
{
  return pt1 + CoordPointUC<T>((pt2.getX() - pt1.getX())/2,
			       (pt2.getY() - pt1.getY())/2);
}


RS_END_NAMESPACE


#endif // !COORDPOINTUC_H
