/* -*- Mode: C++ -*-
 * $Id: CoordPointUCO.h,v 1.5 2000/09/18 15:20:05 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: CoordPointUCO.h
 * Unitname: Library
 * $Revision: 1.5 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/18 15:20:05 $
 *
 *
 * HISTORY
 *
 * $Log: CoordPointUCO.h,v $
 * Revision 1.5  2000/09/18 15:20:05  frehe
 * Due to a problem with namespaces the name for atan2, atan, acos and asin is changed to ATan2, ATan, ASin and ACos.
 *
 * Revision 1.4  2000/09/04 15:21:49  frehe
 * Changed the name of the trigonometric functions to the same as in the standard math library (sin, cos, tan, asin, acos, atan, and atan2).
 *
 * Revision 1.3  2000/09/01 11:43:00  frehe
 * Made minor changes in documentation.
 *
 * Revision 1.2  2000/09/01 03:23:52  frehe
 * Made minor changes.
 *
 * Revision 1.1  2000/09/01 01:24:38  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 *
 */

/**
 * @file CoordPointUCO.h
 * Include file for the template CoordPointUCO.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.5 $
 * $Author: frehe $
 * $Date: 2000/09/18 15:20:05 $
 */


#ifndef COORDPOINTUCO_H
#define COORDPOINTUCO_H


#include "WithUCO.h"
#include "AngleDegreesUCO.h"
#include "CoordPointUC.h"
#include "CoordVectorUCO.h"


#define POINTUCO_HEAD std::string("#PointUCO")


RS_BEGIN_NAMESPACE


template<class T, class Time> class CoordVectorUCO;


/**
 * @class CoordPointUCO CoordPointUCO.h
 * Template class for representing a cartesian two dimensional
 * coordinate that can be unknown and have a confidence factor,
 * and an observation time.
 * The X and Y values are of type WithUCO<T, Time>.
 * The restrictions on T are...
 * The restrictions on Time are...
 *
 * <!-- @todo -->
 *
 * @par Unit:
 *      Library
 * $Revision: 1.5 $
 * $Author: frehe $
 * $Date: 2000/09/18 15:20:05 $
 */
template<class T, class Time> class CoordPointUCO {
public:
  CoordPointUCO();
  CoordPointUCO(const T&, const T&);
  CoordPointUCO(const WithUCO<T, Time>&, const WithUCO<T, Time>&);
  CoordPointUCO(const CoordPoint<T>&);
  CoordPointUCO(const CoordPointUnknown<T>&);
  CoordPointUCO(const CoordPointUC<T>&);
  CoordPointUCO(const CoordPointUCO<T, Time>&);
  CoordPointUCO(const CoordVectorUnknown<T>&, const CoordPoint<T>&);
  CoordPointUCO(const CoordVectorUnknown<T>&, const CoordPointUnknown<T>&);
  CoordPointUCO(const CoordVectorUC<T>&, const CoordPointUC<T>&);
  CoordPointUCO(const CoordVectorUCO<T, Time>&, const CoordPointUCO<T, Time>&);
  CoordPointUCO(const CoordVectorUnknown<T>&, const CoordPointUCO<T, Time>&);
  CoordPointUCO(const CoordPointUnknown<T>&, const CoordVectorUnknown<T>&);
  CoordPointUCO(const CoordPointUC<T>&, const CoordVectorUC<T>&);
  CoordPointUCO(const CoordPointUCO<T, Time>&, const CoordVectorUCO<T, Time>&);
  explicit CoordPointUCO(const CoordVectorUnknown<T>&);
  explicit CoordPointUCO(const CoordVectorUC<T>&);
  explicit CoordPointUCO(const CoordVectorUCO<T, Time>&);

  bool isKnown() const;
  bool isUnknown() const;
  CoordPointUCO<T, Time>& makeUnknown();
  CoordPoint<T> getPoint() const throw(UnknownValueException);

  operator CoordPointUnknown<T>();
  //operator CoordPointUC<T>();
  
  CoordPointUCO<T, Time> operator - () const;
  CoordPointUCO<T, Time> operator - (const CoordPointUCO<T, Time>&) const;
  CoordPointUCO<T, Time> operator - (const CoordVector<T>&) const;
  CoordPointUCO<T, Time> operator - (const CoordVectorUCO<T, Time>&) const;
  CoordPointUCO<T, Time> operator + (const CoordPointUCO<T, Time>&) const;
  CoordPointUCO<T, Time> operator + (const CoordVector<T>&) const;
  CoordPointUCO<T, Time> operator + (const CoordVectorUCO<T, Time>&) const;
  CoordPointUCO<T, Time> operator * (const WithUCO<T, Time>&) const;
  CoordPointUCO<T, Time> operator * (const CoordPointUCO<T, Time>&) const;
  CoordPointUCO<T, Time> operator / (const WithUCO<T, Time>&) const;
  CoordPointUCO<T, Time> operator / (const CoordPointUCO<T, Time>&) const;

  CoordPointUCO<T, Time>& operator =(const CoordPointUCO<T, Time>&);
  CoordPointUCO<T, Time>& operator =(const CoordPointUC<T>&);
  CoordPointUCO<T, Time>& operator =(const CoordPointUnknown<T>&);
  CoordPointUCO<T, Time>& operator =(const CoordPoint<T>&);
  CoordPointUCO<T, Time>& operator =(const CoordVectorUCO<T, Time>&);
  CoordPointUCO<T, Time>& operator =(const CoordVectorUC<T>&);
  CoordPointUCO<T, Time>& operator =(const CoordVectorUnknown<T>&);

  CoordPointUCO<T, Time>& operator -=(const CoordPointUCO<T, Time>&);
  CoordPointUCO<T, Time>& operator -=(const CoordVector<T>&);
  CoordPointUCO<T, Time>& operator -=(const CoordVectorUCO<T, Time>&);
  CoordPointUCO<T, Time>& operator +=(const CoordPointUCO<T, Time>&);
  CoordPointUCO<T, Time>& operator +=(const CoordVector<T>&);
  CoordPointUCO<T, Time>& operator +=(const CoordVectorUCO<T, Time>&);
  CoordPointUCO<T, Time>& operator *=(const WithUCO<T, Time>&);
  CoordPointUCO<T, Time>& operator *=(const CoordPointUCO<T, Time>&);
  CoordPointUCO<T, Time>& operator /=(const WithUCO<T, Time>&);
  CoordPointUCO<T, Time>& operator /=(const CoordPointUCO<T, Time>&);

  bool operator ! () const;
  bool operator ==(const CoordPointUCO<T, Time>&) const;
  bool operator !=(const CoordPointUCO<T, Time>&) const;

  /* Define these in a separate file. They need not be methods.
  // These use the coordinate system of the soccer field
  bool InFront(const CoordPointUCO<T, Time>&) const;  // Equal to LeftOf
  bool Behind(const CoordPointUCO<T, Time>&)  const;   // Equal to RightOf
  // These use standar coordinate system
  bool LeftOf(const CoordPointUCO<T, Time>&)  const;
  bool RightOf(const CoordPointUCO<T, Time>&) const;
  boolbove(const CoordPointUCO<T, Time>&)     const;
  bool Below(const CoordPointUCO<T, Time>&)   const;
  */
  
  WithUCO<T, Time> getX() const;
  WithUCO<T, Time> getY() const;
  CoordPointUCO<T, Time>& addX(const WithUCO<T, Time>& pt_x);
  CoordPointUCO<T, Time>& addY(const WithUCO<T, Time>& pt_y);
  CoordPointUCO<T, Time>& addXY(const WithUCO<T, Time>& pt_x,
			      const WithUCO<T, Time>& pt_y);
  CoordPointUCO<T, Time>& setX(const WithUCO<T, Time>& pt_x);
  CoordPointUCO<T, Time>& setY(const WithUCO<T, Time>& pt_y);
  CoordPointUCO<T, Time>& set (const WithUCO<T, Time>& pt_x, const WithUCO<T, Time>& pt_y);
  
  CoordVectorUCO<T, Time> getVectorTo(const CoordPointUCO<T, Time>& pt) const;
  AngleDegreesUC<T> getDirectionTo(const CoordPointUCO<T, Time>& pt) const;
  WithUCO<T, Time> getDistanceTo(const CoordPointUCO<T, Time>& pt) const;
  
protected:
  WithUCO<T, Time> x;
  WithUCO<T, Time> y;  
};

/*
#if CHECK_DIVisION_BY_ZERO == 1
static const CoordPointUCO<T, Time> CoordPointUCO<T, Time>_DIVisION_BY_ZERO_RESULT(T_DIVisION_BY_ZERO_RESULT,
						   T_DIVisION_BY_ZERO_RESULT);
#endif CHECK_DIVisION_BY_ZERO
*/

template<class T, class Time>
CoordPointUCO<T, Time>::CoordPointUCO()
  : x(),
    y()
{
}

template<class T, class Time>
CoordPointUCO<T, Time>::CoordPointUCO(const T& pt_x, const T& pt_y)
  : x(pt_x),
    y(pt_y)
{
}

template<class T, class Time>
CoordPointUCO<T, Time>::CoordPointUCO(const WithUCO<T, Time>& pt_x, const WithUCO<T, Time>& pt_y)
  : x(pt_x),
    y(pt_y)
{
}

template<class T, class Time>
CoordPointUCO<T, Time>::CoordPointUCO(const CoordPoint<T>& pt)
  : x(pt.getX()),
    y(pt.getY())
{
}

template<class T, class Time>
CoordPointUCO<T, Time>::CoordPointUCO(const CoordPointUnknown<T>& pt)
  : x(pt.getX()),
    y(pt.getY())
{
}

template<class T, class Time>
CoordPointUCO<T, Time>::CoordPointUCO(const CoordPointUC<T>& pt)
  : x(pt.getX()),
    y(pt.getY())
{
}

template<class T, class Time>
CoordPointUCO<T, Time>::CoordPointUCO(const CoordPointUCO<T, Time>& pt)
  : x(pt.x),
    y(pt.y)
{
}

template<class T, class Time>
CoordPointUCO<T, Time>::CoordPointUCO(const CoordVectorUnknown<T>& v,
			      const CoordPoint<T>& pt)
{
  *this = v + pt;
}

template<class T, class Time>
CoordPointUCO<T, Time>::CoordPointUCO(const CoordVectorUnknown<T>& v,
				      const CoordPointUnknown<T>& pt)
{
  *this = v + pt;
}

template<class T, class Time>
CoordPointUCO<T, Time>::CoordPointUCO(const CoordVectorUC<T>& v,
				      const CoordPointUC<T>& pt)
{
  *this = v + pt;
}

template<class T, class Time>
CoordPointUCO<T, Time>::CoordPointUCO(const CoordVectorUnknown<T>& v,
				      const CoordPointUCO<T, Time>& pt)
{
  *this = CoordVectorUCO<T, Time>(v) + pt;
}

template<class T, class Time>
CoordPointUCO<T, Time>::CoordPointUCO(const CoordVectorUCO<T, Time>& v,
				      const CoordPointUCO<T, Time>& pt)
{
  *this = v + pt;
}

template<class T, class Time>
CoordPointUCO<T, Time>::CoordPointUCO(const CoordPointUnknown<T>& pt,
				      const CoordVectorUnknown<T>& v)
{
  *this = v + pt;
}

template<class T, class Time>
CoordPointUCO<T, Time>::CoordPointUCO(const CoordPointUC<T>& pt,
				      const CoordVectorUC<T>& v)
{
  *this = v + pt;
}

template<class T, class Time>
CoordPointUCO<T, Time>::CoordPointUCO(const CoordPointUCO<T, Time>& pt,
				      const CoordVectorUCO<T, Time>& v)
{
  *this = v + pt;
}

template<class T, class Time>
CoordPointUCO<T, Time>::CoordPointUCO(const CoordVectorUnknown<T>& v)
{
  x = v.getLength() * cos(v.getAngle());
  y = v.getLength() * sin(v.getAngle());
}

template<class T, class Time>
CoordPointUCO<T, Time>::CoordPointUCO(const CoordVectorUC<T>& v)
{
  x = v.getLength() * cos(v.getAngle());
  y = v.getLength() * sin(v.getAngle());
}

template<class T, class Time>
CoordPointUCO<T, Time>::CoordPointUCO(const CoordVectorUCO<T, Time>& v)
{
  x = v.getLength() * cos(v.getAngle());
  y = v.getLength() * sin(v.getAngle());
}

template<class T, class Time>
inline bool CoordPointUCO<T, Time>::isKnown() const
{
  return x.isKnown() && y.isKnown();
}

template<class T, class Time>
inline bool CoordPointUCO<T, Time>::isUnknown() const
{
  return x.isUnknown() && y.isUnknown();
}

template<class T, class Time>
inline CoordPointUCO<T, Time>& CoordPointUCO<T, Time>::makeUnknown()
{
  x.makeUnknown();
  y.makeUnknown();
  return *this;
}

template <class T, class Time> 
inline CoordPoint<T> CoordPointUCO<T, Time>::getPoint() const throw(UnknownValueException)
{
  if ( isUnknown() ) {
    RS_THROW(UnknownValueException, "Tried to take the value of an unknown point");
    return CoordPoint<T>();
  } else
    return CoordPoint<T>(x.getValue().getValue().getValue(),
			 y.getValue().getValue().getValue());
}

template<class T, class Time>
inline CoordPointUCO<T, Time>::operator CoordPointUnknown<T>()
{
  return CoordPointUnknown<T>(x,y);
}

/*
template<class T, class Time>
inline CoordPointUCO<T, Time>::operator CoordPointUC<T>()
{
  return CoordPointUC<T>(x,y);
}
*/

template<class T, class Time>
CoordPointUCO<T, Time>& CoordPointUCO<T, Time>::operator =(const CoordPoint<T>& pt)
{
  y = pt.getY();
  x = pt.getX();
  return *this;
}

template<class T, class Time>
CoordPointUCO<T, Time>& CoordPointUCO<T, Time>::operator =(const CoordPointUnknown<T>& pt)
{
  y = pt.getY();
  x = pt.getX();
  return *this;
}

template<class T, class Time>
CoordPointUCO<T, Time>& CoordPointUCO<T, Time>::operator =(const CoordPointUC<T>& pt)
{
  y = pt.getY();
  x = pt.getX();
  return *this;
}

template<class T, class Time>
CoordPointUCO<T, Time>& CoordPointUCO<T, Time>::operator =(const CoordPointUCO<T, Time>& pt)
{
  if ( this != &pt ) {
    y = pt.y;
    x = pt.x;
  }
  return *this;
}

template<class T, class Time>
inline CoordPointUCO<T, Time>& CoordPointUCO<T, Time>::operator =(const CoordVectorUnknown<T>& vec)
{
  *this = CoordPointUCO<T, Time>(vec);
  return *this;
}

template<class T, class Time>
inline CoordPointUCO<T, Time>& CoordPointUCO<T, Time>::operator =(const CoordVectorUC<T>& vec)
{
  *this = CoordPointUCO<T, Time>(vec);
  return *this;
}

template<class T, class Time>
inline CoordPointUCO<T, Time>& CoordPointUCO<T, Time>::operator =(const CoordVectorUCO<T, Time>& vec)
{
  *this = CoordPointUCO<T, Time>(vec);
  return *this;
}

template<class T, class Time>
inline CoordPointUCO<T, Time> CoordPointUCO<T, Time>::operator -() const
{
  return CoordPointUCO<T, Time>(-x,-y);
}

template<class T, class Time>
inline CoordPointUCO<T, Time> CoordPointUCO<T, Time>::operator -(const CoordPointUCO<T, Time>& pt) const
{
  return CoordPointUCO<T, Time>(x - pt.x, y - pt.y);
}

template<class T, class Time>
inline CoordPointUCO<T, Time> CoordPointUCO<T, Time>::operator -(const CoordVector<T>& v) const
{
  return (-v) + *this;
}

template<class T, class Time>
inline CoordPointUCO<T, Time> CoordPointUCO<T, Time>::operator -(const CoordVectorUCO<T, Time>& v) const
{
  return (-v) + *this;
}

template<class T, class Time>
inline CoordPointUCO<T, Time> operator -(const CoordPoint<T>& pt1,
				  const CoordPointUCO<T, Time>& pt2)
{
  return (-pt2) + pt1;
}

template<class T, class Time>
inline CoordPointUCO<T, Time> operator -(const CoordPointUnknown<T>& pt1,
				  const CoordPointUCO<T, Time>& pt2)
{
  return (-pt2) + pt1;
}

/*
template<class T, class Time>
inline CoordPointUCO<T, Time> CoordPointUCO<T, Time>::operator +(const CoordPointUC<T>& pt) const
{
  return CoordPointUCO<T, Time>(x + pt.x, y + pt.y);
}
*/

template<class T, class Time>
inline CoordPointUCO<T, Time> CoordPointUCO<T, Time>::operator +(const CoordPointUCO<T, Time>& pt) const
{
  return CoordPointUCO<T, Time>(x + pt.x, y + pt.y);
}

template<class T, class Time>
inline CoordPointUCO<T, Time> CoordPointUCO<T, Time>::operator+(const CoordVector<T>& v) const
{
  return CoordVectorUCO<T, Time>(v)+*this;
}

template<class T, class Time>
inline CoordPointUCO<T, Time> CoordPointUCO<T, Time>::operator+(const CoordVectorUCO<T, Time>& v) const
{
  return v+*this;
}

template<class T, class Time>
inline CoordPointUCO<T, Time> operator +(const CoordPoint<T>& pt1,
					 const CoordPointUCO<T, Time>& pt2)
{
  return pt2 + pt1;
}

template<class T, class Time>
inline CoordPointUCO<T, Time> operator +(const CoordPointUnknown<T>& pt1,
				  const CoordPointUCO<T, Time>& pt2)
{
  return pt2 + pt1;
}

template<class T, class Time>
inline CoordPointUCO<T, Time> operator +(const CoordVector<T>& pt1,
					 const CoordPointUCO<T, Time>& pt2)
{
  return CoordPointUCO<T, Time>(pt1) + pt2;
}

template<class T, class Time>
inline CoordPointUCO<T, Time> CoordPointUCO<T, Time>::operator *(const WithUCO<T, Time>& a) const
{
  return CoordPointUCO<T, Time>(x * a, y * a);
}

template<class T, class Time>
inline CoordPointUCO<T, Time> CoordPointUCO<T, Time>::operator *(const CoordPointUCO<T, Time>& pt) const
{
  return CoordPointUCO<T, Time>(x * pt.x, y * pt.y);
}

template<class T, class Time>
inline CoordPointUCO<T, Time> operator *(const T& a, const CoordPointUCO<T, Time>& pt)
{
  return pt * a;
}

template<class T, class Time>
inline CoordPointUCO<T, Time> operator *(const WithUnknown<T>& a, const CoordPointUCO<T, Time>& pt)
{
  return pt * a;
}

template<class T, class Time>
inline CoordPointUCO<T, Time> operator *(const WithUC<T>& a, const CoordPointUCO<T, Time>& pt)
{
  return pt * a;
}

template<class T, class Time>
inline CoordPointUCO<T, Time> operator *(const WithUCO<T, Time>& a, const CoordPointUCO<T, Time>& pt)
{
  return pt * a;
}

template<class T, class Time>
inline CoordPointUCO<T, Time> operator *(const CoordPoint<T>& pt1,
					 const CoordPointUCO<T, Time>& pt2)
{
  return pt2 * pt1;
}

template<class T, class Time>
inline CoordPointUCO<T, Time> operator *(const CoordPointUnknown<T>& pt1,
					 const CoordPointUCO<T, Time>& pt2)
{
  return pt2 * pt1;
}

template<class T, class Time>
inline CoordPointUCO<T, Time> operator *(const CoordPointUC<T>& pt1,
					 const CoordPointUCO<T, Time>& pt2)
{
  return pt2 * pt1;
}

template<class T, class Time>
inline CoordPointUCO<T, Time> CoordPointUCO<T, Time>::operator /(const WithUCO<T, Time>& a) const
{
#if CHECK_DIVisION_BY_ZERO == 1
  if (a == T(0))
    return CoordPointUCO_DIVisION_BY_ZERO_RESULT;
  else
#endif
    return CoordPointUCO<T, Time>(x / a, y / a);
}

template<class T, class Time>
inline CoordPointUCO<T, Time> CoordPointUCO<T, Time>::operator /(const CoordPointUCO<T, Time>& pt) const
{
#if CHECK_DIVisION_BY_ZERO == 1
  if ( pt.x == T(0) ) {
    if ( pt.y == T(0) ) {
      return CoordPointUCO_DIVisION_BY_ZERO_RESULT;
    } else {
      return CoordPointUCO<T, Time>(T_DIVisION_BY_ZERO_RESULT, y / pt.y);
    }
  } else {
    if ( pt.y == T(0) ) {
      return CoordPointUCO<T, Time>(x / pt.x, T_DIVisION_BY_ZERO_RESULT);
    } else {
#endif
      return CoordPointUCO<T, Time>(x / pt.x, y / pt.y);
#if CHECK_DIVisION_BY_ZERO == 1      
    }
  }
#endif
}

template<class T, class Time>
CoordPointUCO<T, Time> operator /(const CoordPoint<T>& pt1, const CoordPointUCO<T, Time>& pt2)
{
  return CoordPointUCO<T, Time>(pt1) / pt2;
}

template<class T, class Time>
CoordPointUCO<T, Time> operator /(const CoordPointUnknown<T>& pt1, const CoordPointUCO<T, Time>& pt2)
{
  return CoordPointUCO<T, Time>(pt1) / pt2;
}

template<class T, class Time>
CoordPointUCO<T, Time> operator /(const CoordPointUC<T>& pt1, const CoordPointUCO<T, Time>& pt2)
{
  return CoordPointUCO<T, Time>(pt1) / pt2;
}

template<class T, class Time>
inline CoordPointUCO<T, Time>& CoordPointUCO<T, Time>::operator-=(const CoordPointUCO<T, Time>& pt)
{
  x -= pt.x;
  y -= pt.y;
  return *this;
}

template<class T, class Time>
inline CoordPointUCO<T, Time>& CoordPointUCO<T, Time>::operator-=(const CoordVector<T>& vec)
{
  *this = *this - vec;
  return *this;
}

template<class T, class Time>
inline CoordPointUCO<T, Time>& CoordPointUCO<T, Time>::operator-=(const CoordVectorUCO<T, Time>& vec)
{
  *this = *this - vec;
  return *this;
}

template<class T, class Time>
inline CoordPointUCO<T, Time>& CoordPointUCO<T, Time>::operator+=(const CoordVector<T>& vec)
{
  *this = *this + CoordPointUCO<T, Time>(vec);
  return *this;
}

template<class T, class Time>
inline CoordPointUCO<T, Time>& CoordPointUCO<T, Time>::operator+=(const CoordVectorUCO<T, Time>& vec)
{
  *this = *this + vec;
  return *this;
}

template<class T, class Time>
inline CoordPointUCO<T, Time>& CoordPointUCO<T, Time>::operator+=(const CoordPointUCO<T, Time>& pt)
{
  x += pt.x;
  y += pt.y;
  return *this;
}

template<class T, class Time>
inline CoordPointUCO<T, Time>& CoordPointUCO<T, Time>::operator*=(const WithUCO<T, Time>& a)
{
  x *= a;
  y *= a;
  return *this;
}

template<class T, class Time>
inline CoordPointUCO<T, Time>& CoordPointUCO<T, Time>::operator*=(const CoordPointUCO<T, Time>& pt)
{
  x *= pt.x;
  y *= pt.y;
  return *this;
}

template<class T, class Time>
CoordPointUCO<T, Time>& CoordPointUCO<T, Time>::operator/=(const WithUCO<T, Time>& a)
{
#if CHECK_DIVisION_BY_ZERO == 1
  if ( a == T(0)) {
    x = T_DIVisION_BY_ZERO_RESULT;
    y = T_DIVisION_BY_ZERO_RESULT;
  } else {
#endif
    x /= a;
    y /= a;
#if CHECK_DIVisION_BY_ZERO == 1
  }
#endif
  return *this;
}

template<class T, class Time>
inline CoordPointUCO<T, Time>& CoordPointUCO<T, Time>::operator/=(const CoordPointUCO<T, Time>& pt)
{
#if CHECK_DIVisION_BY_ZERO == 1
  if ( pt.x == T(0) )
    x = T_DIVisION_BY_ZERO_RESULT;
  else
#endif
    x /= pt.x;
  
#if CHECK_DIVisION_BY_ZERO == 1
  if ( pt.y == T(0) )
    y = T_DIVisION_BY_ZERO_RESULT;
  else
#endif
    y /= pt.y;

  return *this;
}

template<class T, class Time>
inline bool CoordPointUCO<T, Time>::operator !() const
{
  return (!y) && (!x);
}

template<class T, class Time>
inline bool CoordPointUCO<T, Time>::operator ==(const CoordPointUCO<T, Time>& pt) const
{
  return (y == pt.y) && (x == pt.x);
}

template<class T, class Time>
inline bool operator ==(const CoordPoint<T>& pt1,
			const CoordPointUCO<T, Time>& pt2)
{
  return pt2 == pt1;
}

template<class T, class Time>
inline bool operator ==(const CoordPointUnknown<T>& pt1,
			const CoordPointUCO<T, Time>& pt2)
{
  return pt2 == pt1;
}

template<class T, class Time>
inline bool CoordPointUCO<T, Time>::operator !=(const CoordPointUCO<T, Time>& pt) const
{
  return (y != pt.y) || (x != pt.x);
}

template<class T, class Time>
inline bool operator !=(const CoordPoint<T>& pt1,
			const CoordPointUCO<T, Time>& pt2)
{
  return pt2 != pt1;
}

template<class T, class Time>
inline bool operator !=(const CoordPointUnknown<T>& pt1,
			const CoordPointUCO<T, Time>& pt2)
{
  return pt2 != pt1;
}

/*
inline bool CoordPointUCO<T, Time>::InFront(const CoordPointUCO<T, Time>& pt) const
{
  return (x > pt.x);
}

inline bool CoordPointUCO<T, Time>::Behind(const CoordPointUCO<T, Time>& pt) const
{
  return (x < pt.x);
}
*/

/*
inline bool CoordPointUCO<T, Time>::LeftOf(const CoordPointUCO<T, Time>& pt) const
{
  return (x > pt.x);
}

inline bool CoordPointUCO<T, Time>::RightOf(const CoordPointUCO<T, Time>& pt) const
{
  return (x < pt.x);
}

inline bool CoordPointUCO<T, Time>::Above(const CoordPointUCO<T, Time>& pt) const
{
  return (y < pt.y);
}

inline bool CoordPointUCO<T, Time>::Below(const CoordPointUCO<T, Time>& pt) const
{
  return (y > pt.y);
}
*/

template<class T, class Time>
inline CoordPointUCO<T, Time>& CoordPointUCO<T, Time>::addX (const WithUCO<T, Time>& pt_x)
{
  x += pt_x;
  return *this;
}

template<class T, class Time>
inline CoordPointUCO<T, Time>& CoordPointUCO<T, Time>::addY (const WithUCO<T, Time>& pt_y)
{
  y += pt_y;
  return *this;
}

template<class T, class Time>
inline CoordPointUCO<T, Time>& CoordPointUCO<T, Time>::addXY (const WithUCO<T, Time>& pt_x,
						const WithUCO<T, Time>& pt_y)
{
  x += pt_x;
  y += pt_y;
  return *this;
}

template<class T, class Time>
inline WithUCO<T, Time> CoordPointUCO<T, Time>::getX() const
{
  return x;
}

template<class T, class Time>
inline WithUCO<T, Time> CoordPointUCO<T, Time>::getY() const
{
  return y;
}

template<class T, class Time>
inline CoordPointUCO<T, Time>& CoordPointUCO<T, Time>::setX(const WithUCO<T, Time>& pt_x)
{
  x = pt_x;
  return *this;
}

template<class T, class Time>
inline CoordPointUCO<T, Time>& CoordPointUCO<T, Time>::setY(const WithUCO<T, Time>& pt_y)
{
  y = pt_y;
  return *this;
}

template<class T, class Time>
inline CoordPointUCO<T, Time>& CoordPointUCO<T, Time>::set(const WithUCO<T, Time>& pt_x,
					     const WithUCO<T, Time>& pt_y)
{
  x = pt_x;
  y = pt_y;
  return *this;
}

template<class T, class Time>
AngleDegreesUC<T> CoordPointUCO<T, Time>::getDirectionTo(const CoordPointUCO<T, Time>& pt) const
{
  if ( pt.isUnknown() )
    return AngleDegreesUC<T>();
  else
    return ATan2((pt.y - y), (pt.x - x));
}

template<class T, class Time>
inline WithUCO<T, Time> CoordPointUCO<T, Time>::getDistanceTo(const CoordPointUCO<T, Time>& pt) const
{
  return sqrt(sqr(pt.x - x) + sqr(pt.y - y));
}

template<class T, class Time>
inline std::ostream& operator<<(std::ostream& os, const CoordPointUCO<T, Time>& pt)
{
#ifdef POINTUCO_HEAD
  return os << POINTUCO_HEAD << "(" << pt.getX() << ", " << pt.getY() << ")";
#else
  return os << "(" << pt.getX() << ", " << pt.getY() << ")";
#endif
}

template<class T, class Time>
inline const std::string toString(const CoordPointUCO<T, Time>& pt)
{
#ifdef POINTUCO_HEAD
  return POINTUCO_HEAD + "(" + toString(pt.getX()) + ", " + toString(pt.getY()) + ")";
#else
  return "(" + toString(pt.getX()) + ", " + toString(pt.getY()) + ")";
#endif
}

template<class T, class Time>
inline CoordVectorUCO<T, Time> CoordPointUCO<T, Time>::getVectorTo(const CoordPointUCO<T, Time>& pt) const
{
  return CoordVectorUCO<T, Time>(*this, pt);
}

template<class T, class Time>
std::istream& operator>>(std::istream& is, CoordPointUCO<T, Time>& pt)
{
  /* Input formats: f is a T
	(f,f) = (f,f)
  */

  WithUCO<T, Time> pt_x;
  WithUCO<T, Time> pt_y;
  char c = 0;

  is >> c;
  if (c == '(') {
    is >> pt_x >> c;
    if (c == ',') is >> pt_y >> c;
    if (c != ')') is.clear(std::ios::badbit); //error...;
  }
  else {
    is.clear(std::ios::badbit); //error...;
  }

  if (is) pt = CoordPointUCO<T, Time>(pt_x, pt_y);

  return is;
}

// Extend this to be the point on the line between pt1 and pt2 
// with a certain ratio 1:1 is the middle point?
template<class T, class Time>
inline CoordPointUCO<T, Time> middlePoint(const CoordPointUCO<T, Time>& pt1,
					  const CoordPointUCO<T, Time>& pt2)
{
  return pt1 + CoordPointUCO<T, Time>((pt2.getX() - pt1.getX())/2,
				      (pt2.getY() - pt1.getY())/2);
}


RS_END_NAMESPACE


#endif // !COORDPOINTUCO_H
