/* -*- Mode: C++ -*-
 * $Id: CoordVectorUC.h,v 1.4 2000/09/18 15:20:05 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: CoordVectorUC.h
 * Unitname: Library
 * $Revision: 1.4 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/18 15:20:05 $
 *
 *
 * HISTORY
 *
 * $Log: CoordVectorUC.h,v $
 * Revision 1.4  2000/09/18 15:20:05  frehe
 * Due to a problem with namespaces the name for atan2, atan, acos and asin is changed to ATan2, ATan, ASin and ACos.
 *
 * Revision 1.3  2000/09/04 15:21:50  frehe
 * Changed the name of the trigonometric functions to the same as in the standard math library (sin, cos, tan, asin, acos, atan, and atan2).
 *
 * Revision 1.2  2000/09/01 03:23:53  frehe
 * Made minor changes.
 *
 * Revision 1.1  2000/09/01 01:24:39  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 *
 */

/**
 * @file CoordVectorUC.h
 * Include file for the template CoordVectorUC.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.4 $
 * $Author: frehe $
 * $Date: 2000/09/18 15:20:05 $
 */


#ifndef COORDVECTORUC_H
#define COORDVECTORUC_H


#include "CoordVector.h"
#include "WithUC.h"
#include "AngleDegreesUC.h"
#include "CoordPointUC.h"


#define VECTORUC_HEAD std::string("#VectorUC")


RS_BEGIN_NAMESPACE


template<class T> class CoordPointUC;


/**
 * @class CoordVectorUC CoordVectorUC.h
 * Template class for representing a two dimensional mathematical vector
 * that can be unknown and have a confidence factor.
 * The distance and direction are of type WithUC<T>.
 * T is the base type for the angle: AngleDegreesUC<T> and the
 * distance WithUC<T>.
 *
 * <!-- @todo -->
 *
 * @par Unit:
 *      Library
 * $Revision: 1.4 $
 * $Author: frehe $
 * $Date: 2000/09/18 15:20:05 $
 */
template<class T> class CoordVectorUC
{
public:
  CoordVectorUC();
  CoordVectorUC(const T&, const T&);
  CoordVectorUC(const AngleDegrees<T>&, const T&);
  //CoordVectorUC(const AngleDegreesUnknown<T>&, const T&);
  //CoordVectorUC(const AngleDegreesUC<T>&, const T&);
  CoordVectorUC(const AngleDegreesUnknown<T>&, const WithUnknown<T>&);
  CoordVectorUC(const AngleDegreesUC<T>&, const WithUC<T>&);
  //CoordVectorUC(const CoordVector<T>&);
  CoordVectorUC(const CoordVectorUnknown<T>&);
  CoordVectorUC(const CoordVectorUC<T>&);
  CoordVectorUC(const CoordPoint<T>&, const CoordPoint<T>&);
  CoordVectorUC(const CoordPointUnknown<T>&, const CoordPointUnknown<T>&);
  CoordVectorUC(const CoordPointUC<T>&, const CoordPointUC<T>&);
  explicit CoordVectorUC(const CoordPoint<T>&);
  explicit CoordVectorUC(const CoordPointUnknown<T>&);
  explicit CoordVectorUC(const CoordPointUC<T>&);

  bool isKnown() const;
  bool isUnknown() const;
  CoordVectorUC<T>& makeUnknown();

  //CConfidence getConfidence() const; // What should this be?
  
  CoordVector<T> getVector() const throw(RSException);
  operator CoordVectorUnknown<T>();
  
  CoordVectorUC<T> operator -() const;
  CoordVectorUC<T> operator -(const CoordVectorUnknown<T>&) const;
  CoordVectorUC<T> operator -(const CoordVectorUC<T>&) const;
  CoordVectorUC<T> operator +(const CoordVectorUnknown<T>&) const; 
  CoordVectorUC<T> operator +(const CoordVectorUC<T>&) const; 
  CoordPointUC<T>  operator +(const CoordPointUC<T>&) const;
  CoordVectorUC<T> operator *(const WithUC<T>&) const;
  CoordVectorUC<T> operator /(const WithUC<T>&) const;

  CoordVectorUC<T>& operator =(const CoordVectorUnknown<T>&);
  CoordVectorUC<T>& operator =(const CoordVectorUC<T>&);
  CoordVectorUC<T>& operator =(const CoordPoint<T>&);  
  CoordVectorUC<T>& operator =(const CoordPointUnknown<T>&);  
  CoordVectorUC<T>& operator =(const CoordPointUC<T>&);  
  
  CoordVectorUC<T>& operator -=(const CoordVectorUnknown<T>&);
  CoordVectorUC<T>& operator -=(const CoordVectorUC<T>&);
  CoordVectorUC<T>& operator +=(const CoordVectorUnknown<T>&);
  CoordVectorUC<T>& operator +=(const CoordVectorUC<T>&);
  CoordVectorUC<T>& operator +=(const CoordPointUC<T>&);
  CoordVectorUC<T>& operator *=(const WithUC<T>&);
  CoordVectorUC<T>& operator /=(const WithUC<T>&);

  // is the vector the zero vector?
  bool     operator ! ()                 const;
  bool     operator ==(const CoordVectorUnknown<T>&) const;
  bool     operator ==(const CoordVectorUC<T>&)   const;
  bool     operator !=(const CoordVectorUnknown<T>&)   const;
  bool     operator !=(const CoordVectorUC<T>&)   const;
  // These compares the length of the vectors
  bool     operator < (const CoordVectorUnknown<T>&)   const;
  bool     operator < (const CoordVectorUC<T>&)   const;
  bool     operator <=(const CoordVectorUnknown<T>&)   const;
  bool     operator <=(const CoordVectorUC<T>&)   const;
  bool     operator > (const CoordVectorUnknown<T>&)   const;
  bool     operator > (const CoordVectorUC<T>&)   const;
  bool     operator >=(const CoordVectorUnknown<T>&)   const;
  bool     operator >=(const CoordVectorUC<T>&)   const;

  // Shoule I include these? Instead of the above operators?
  // bool SameLength
  // bool Longer
  // bool Shorter

  // Adds to the angle resp. to the distance of the vector.
  CoordVectorUC<T>& rotate(const AngleDegreesUC<T>&);
  // Bad name! I should have thought this was multiplication not addition.
  CoordVectorUC<T>& scale (const WithUC<T>&);

  WithUC<T> getLength() const;
  CoordVectorUC<T>& setLength(const WithUC<T>&);
  AngleDegreesUC<T> getAngle() const;
  CoordVectorUC<T>& setAngle(const AngleDegreesUC<T>&);

protected:
  AngleDegreesUC<T> dir; // The direction of the vector
  WithUC<T> dist; // The distance/length of the vector
};

template <class T>
CoordVectorUC<T>::CoordVectorUC()
  : dir(),
    dist()
{
}

template <class T>
CoordVectorUC<T>::CoordVectorUC(const T& vdir, const T& vdist)
  : dir(vdir),
    dist(vdist)
{
  if (dist < WithUC<T>(0)) {
    dir -= AngleDegreesUC<T>(180);
    dist = -dist;
  }
}

template <class T>
CoordVectorUC<T>::CoordVectorUC(const AngleDegrees<T>& vdir,
				const T& vdist)
  : dir(vdir),
    dist(vdist)
{
  if (dist < WithUC<T>(0)) {
    dir -= AngleDegreesUC<T>(180);
    dist = -dist;
  }
}

/*
template <class T>
CoordVectorUC<T>::CoordVectorUC(const AngleDegreesUC<T>& vdir,
					  const T& vdist)
  : dir(vdir),
    dist(vdist)
{
  if (dist < WithUC<T>(0)) {
    dir -= AngleDegreesUC<T>(180);
    dist = -dist;
  }
}
*/

template <class T>
CoordVectorUC<T>::CoordVectorUC(const AngleDegreesUnknown<T>& vdir,
				const WithUnknown<T>& vdist)
  : dir(vdir),
    dist(vdist)
{
  if (dist < WithUC<T>(0)) {
    dir -= AngleDegreesUC<T>(180);
    dist = -dist;
  }
}

template <class T>
CoordVectorUC<T>::CoordVectorUC(const AngleDegreesUC<T>& vdir,
				const WithUC<T>& vdist)
  : dir(vdir),
    dist(vdist)
{
  if (dist < WithUC<T>(0)) {
    dir -= AngleDegreesUC<T>(180);
    dist = -dist;
  }
}

/*
template <class T>
CoordVectorUC<T>::CoordVectorUC(const CoordVector<T>& v)
  : dir(v.getAngle()),
    dist(v.getLength())
{
}
*/

template <class T>
CoordVectorUC<T>::CoordVectorUC(const CoordVectorUnknown<T>& v)
  : dir(v.getAngle()),
    dist(v.getLength())
{
}

template <class T>
CoordVectorUC<T>::CoordVectorUC(const CoordVectorUC<T>& v)
  : dir(v.dir),
    dist(v.dist)
{
}

template <class T>
CoordVectorUC<T>::CoordVectorUC(const CoordPoint<T>& pt1,
				const CoordPoint<T>& pt2)
{
  if (pt2 == pt1) {
    dir = AngleDegreesUC<T>(0);
    dist = WithUC<T>(0);
  } else {
    dir = ATan2(pt2.getY()-pt1.getY(), pt2.getX()-pt1.getX());
    dist = sqrt(sqr(pt1.getX()-pt2.getX()) + sqr(pt1.getY()-pt2.getY()));
  }
}

template <class T>
CoordVectorUC<T>::CoordVectorUC(const CoordPointUnknown<T>& pt1,
				const CoordPointUnknown<T>& pt2)
{
  if (pt2 == pt1) {
    dir = AngleDegreesUC<T>(0);
    dist = WithUC<T>(0);
  } else {
    dir = ATan2(pt2.getY()-pt1.getY(), pt2.getX()-pt1.getX());
    dist = sqrt(sqr(pt1.getX()-pt2.getX()) + sqr(pt1.getY()-pt2.getY()));
  }
}

template <class T>
CoordVectorUC<T>::CoordVectorUC(const CoordPointUC<T>& pt1,
				const CoordPointUC<T>& pt2)
{
  if (pt2 == pt1) {
    dir = AngleDegreesUC<T>(0);
    dist = WithUC<T>(0);
  } else {
    dir = ATan2(pt2.getY()-pt1.getY(), pt2.getX()-pt1.getX());
    dist = sqrt(sqr(pt1.getX()-pt2.getX()) + sqr(pt1.getY()-pt2.getY()));
  }
}

template <class T>
CoordVectorUC<T>::CoordVectorUC(const CoordPoint<T>& pt)
{
  if ( pt.getX() == WithUC<T>(0) &&
       pt.getY() == WithUC<T>(0) ) {
    dir = AngleDegreesUC<T>(0);
    dist = WithUC<T>(0);
  } else {
    dir = ATan2(pt.getY(), pt.getX());
    dist = sqrt(sqr(pt.getX()) + sqr(pt.getY()));
  }
}

template <class T>
CoordVectorUC<T>::CoordVectorUC(const CoordPointUnknown<T>& pt)
{
  if ( pt.getX() == WithUC<T>(0) &&
       pt.getY() == WithUC<T>(0) ) {
    dir = AngleDegreesUC<T>(0);
    dist = WithUC<T>(0);
  } else {
    dir = ATan2(pt.getY(), pt.getX());
    dist = sqrt(sqr(pt.getX()) + sqr(pt.getY()));
  }
}

template <class T>
CoordVectorUC<T>::CoordVectorUC(const CoordPointUC<T>& pt)
{
  if ( pt.getX() == WithUC<T>(0) &&
       pt.getY() == WithUC<T>(0) ) {
    dir = AngleDegreesUC<T>(0);
    dist = WithUC<T>(0);
  } else {
    dir = ATan2(pt.getY(), pt.getX());
    dist = sqrt(sqr(pt.getX()) + sqr(pt.getY()));
  }
}

template<class T>
inline bool CoordVectorUC<T>::isKnown() const
{
  return dir.isKnown() && dist.isKnown();
}

template<class T>
inline bool CoordVectorUC<T>::isUnknown() const
{
  return dir.isUnknown() && dist.isUnknown();
}

template<class T>
CoordVectorUC<T>& CoordVectorUC<T>::makeUnknown()
{
  dir.makeUnknown();
  dist.makeUnknown();
  return *this;
}

template <class T> 
inline CoordVector<T> CoordVectorUC<T>::getVector() const throw(RSException)
{
  if ( isUnknown() ) {
    RS_THROW(UnknownValueException, "Tried to take the value of an unknown vector");
    return CoordVector<T>();
  } else
    return CoordVector<T>(dir.getAngle().getAngle(), dist.getValue().getValue());
}

template<class T>
inline CoordVectorUC<T>::operator CoordVectorUnknown<T>()
{
  return CoordVectorUnknown<T>(dir, dist);
}

// The minus operation - Start
template <class T>
inline CoordVectorUC<T> CoordVectorUC<T>::operator -() const
{
  return CoordVectorUC<T>(dir-AngleDegrees<T>(180), dist);
}

template <class T>
inline CoordVectorUC<T> CoordVectorUC<T>::operator -(const CoordVectorUnknown<T>& v) const
{
  return *this+CoordVectorUC<T>(v.getAngle()-AngleDegrees<T>(180), v.getLength());
}

template <class T>
inline CoordVectorUC<T> CoordVectorUC<T>::operator -(const CoordVectorUC<T>& v) const
{
  return *this+CoordVectorUC<T>(v.dir-AngleDegrees<T>(180), v.dist);
}

template <class T>
inline CoordVectorUC<T> operator -(const CoordVector<T>& v1,
				   const CoordVectorUC<T>& v2)
{
  return (-v2) + v1;
}

template <class T>
inline CoordVectorUC<T> operator -(const CoordVectorUnknown<T>& v1,
				   const CoordVectorUC<T>& v2)
{
  return (-v2) + v1;
}

template <class T>
inline CoordPointUC<T> operator -(const CoordPoint<T>& v1,
				  const CoordVectorUC<T>& v2)
{
  return (-v2) + v1;
}

template <class T>
inline CoordPointUC<T> operator -(const CoordPointUnknown<T>& v1,
				  const CoordVectorUC<T>& v2)
{
  return (-v2) + v1;
}
template <class T>
inline CoordPointUC<T> operator -(const CoordPointUC<T>& v1,
				  const CoordVectorUC<T>& v2)
{
  return (-v2) + v1;
}
// The minus operation - End

// The plus operation - Start
template <class T>
inline CoordVectorUC<T> CoordVectorUC<T>::operator +(const CoordVectorUnknown<T>& v) const
{
  return CoordVectorUC<T>(CoordPointUC<T>(v) + CoordPointUC<T>(*this));
}

template <class T>
inline CoordVectorUC<T> CoordVectorUC<T>::operator +(const CoordVectorUC<T>& v) const
{
  return CoordVectorUC<T>(CoordPointUC<T>(v) + CoordPointUC<T>(*this));
}

template <class T>
inline CoordPointUC<T> CoordVectorUC<T>::operator+(const CoordPointUC<T>& pt1) const
{
  return CoordPointUC<T>(dist*cos(dir), dist*sin(dir)) + pt1;
}
 
template <class T>
inline CoordVectorUC<T> operator +(const CoordVector<T>& v1,
				   const CoordVectorUC<T>& v2)
{
  return v2 + v1;
}

template <class T>
inline CoordVectorUC<T> operator +(const CoordVectorUnknown<T>& v1,
				   const CoordVectorUC<T>& v2)
{
  return v2 + v1;
}

template<class T>
inline CoordPointUC<T> operator +(const CoordPoint<T>& pt1,
				  const CoordVectorUC<T>& pt2)
{
  return pt2 + pt1;
}

template<class T>
inline CoordPointUC<T> operator +(const CoordPointUnknown<T>& pt1,
				  const CoordVectorUC<T>& pt2)
{
  return pt2 + pt1;
}

template<class T>
inline CoordPointUC<T> operator +(const CoordPointUC<T>& pt1,
				  const CoordVectorUC<T>& pt2)
{
  return pt2 + pt1;
}
// The plus operation - End

// The multiply operation - Start
template <class T>
inline CoordVectorUC<T> CoordVectorUC<T>::operator *(const WithUC<T>& a) const
{
  return CoordVectorUC<T>(dir, dist * a);
}

template <class T>
inline CoordVectorUC<T> operator *(const T& a,
				   const CoordVectorUC<T>& vec)
{
  return vec * a;
}

template <class T>
inline CoordVectorUC<T> operator *(const WithUnknown<T>& a,
				   const CoordVectorUC<T>& vec)
{
  return vec * a;
}

template <class T>
inline CoordVectorUC<T> operator *(const WithUC<T>& a,
				   const CoordVectorUC<T>& vec)
{
  return vec * a;
}
// The multiply operation - End

// The division operation - Start
template <class T>
inline CoordVectorUC<T> CoordVectorUC<T>::operator /(const WithUC<T>& a) const
{
#if CHECK_DIVisION_BY_ZERO == 1
  if (a == T(0))
    return COORDVECTOR_DIVisION_BY_ZERO_RESULT;
  else
#endif
    return CoordVectorUC<T>(dir, dist / a);
}
// The division operation - End

template <class T>
CoordVectorUC<T>& CoordVectorUC<T>::operator =(const CoordVectorUnknown<T>& v)
{
  dist = v.getLength();
  dir = v.getAngle();
  return *this;
}

template <class T>
CoordVectorUC<T>& CoordVectorUC<T>::operator =(const CoordVectorUC<T>& v)
{
  if (this != &v) {
    dist = v.dist;
    dir = v.dir;
  }

  return *this;
}

template <class T>
inline CoordVectorUC<T>& CoordVectorUC<T>::operator =(const CoordPoint<T>& pt)
{
  *this = CoordVectorUC<T>(pt);
  return *this;
}

template <class T>
inline CoordVectorUC<T>& CoordVectorUC<T>::operator =(const CoordPointUnknown<T>& pt)
{
  *this = CoordVectorUC<T>(pt);
  return *this;
}

template <class T>
inline CoordVectorUC<T>& CoordVectorUC<T>::operator =(const CoordPointUC<T>& pt)
{
  *this = CoordVectorUC<T>(pt);
  return *this;
}

template <class T>
inline CoordVectorUC<T>& CoordVectorUC<T>::operator-=(const CoordVectorUnknown<T>& v)
{
  return *this = *this - v;
}

template <class T>
inline CoordVectorUC<T>& CoordVectorUC<T>::operator-=(const CoordVectorUC<T>& v)
{
  return *this = *this - v;
}

template <class T>
inline CoordVectorUC<T>& CoordVectorUC<T>::operator+=(const CoordVectorUnknown<T>& v)
{
  return *this = *this + v;
}

template <class T>
inline CoordVectorUC<T>& CoordVectorUC<T>::operator+=(const CoordVectorUC<T>& v)
{
  return *this = *this + v;
}

template <class T>
inline CoordVectorUC<T>& CoordVectorUC<T>::operator+=(const CoordPointUC<T>& v)
{
  return *this = *this + v;
}

template <class T>
inline CoordVectorUC<T>& CoordVectorUC<T>::operator*=(const WithUC<T>& a)
{
  dist *= a;
  return *this;
}

template <class T>
inline CoordVectorUC<T>& CoordVectorUC<T>::operator/=(const WithUC<T>& a)
{
#if CHECK_DIVisION_BY_ZERO == 1
  if (a == T(0))
    dist = T_DIVisION_BY_ZERO_RESULT;
  else
#endif
    dist /= a;
  
  return *this;
}

template <class T>
inline bool CoordVectorUC<T>::operator ==(const CoordVectorUnknown<T>& v) const
{
  return (dist == v.getLength()) && (dir == v.getAngle());
}

template <class T>
inline bool CoordVectorUC<T>::operator ==(const CoordVectorUC<T>& v) const
{
  return (dist == v.dist) && (dir == v.dir);
}

template<class T>
inline bool operator==(const CoordVector<T>& v1,
		       const CoordVectorUC<T>& v2)
{
  return v2 == v1;
}

template<class T>
inline bool operator==(const CoordVectorUnknown<T>& v1,
		       const CoordVectorUC<T>& v2)
{
  return v2 == v1;
}

template <class T>
inline bool CoordVectorUC<T>::operator !=(const CoordVectorUnknown<T>& v) const
{
  return (dist != v.getLength()) || (dir != v.getAngle());
}

template <class T>
inline bool CoordVectorUC<T>::operator !=(const CoordVectorUC<T>& v) const
{
  return (dist != v.dist) || (dir != v.dir);
}

template<class T>
inline bool operator!=(const CoordVector<T>& v1,
		       const CoordVectorUC<T>& v2)
{
  return v2 != v1;
}

template<class T>
inline bool operator!=(const CoordVectorUnknown<T>& v1,
		       const CoordVectorUC<T>& v2)
{
  return v2 != v1;
}

template <class T>
inline bool CoordVectorUC<T>::operator < (const CoordVectorUnknown<T>& v) const
{
  return (dist <  v.getLength());
}

template <class T>
inline bool CoordVectorUC<T>::operator < (const CoordVectorUC<T>& v) const
{
  return (dist <  v.dist);
}

template<class T>
inline bool operator<(const CoordVector<T>& v1,
		      const CoordVectorUC<T>& v2)
{
  return v2 > v1;
}

template<class T>
inline bool operator<(const CoordVectorUnknown<T>& v1,
		      const CoordVectorUC<T>& v2)
{
  return v2 > v1;
}

template <class T>
inline bool CoordVectorUC<T>::operator <=(const CoordVectorUnknown<T>& v) const
{
  return (dist <= v.getLength());
}

template <class T>
inline bool CoordVectorUC<T>::operator <=(const CoordVectorUC<T>& v) const
{
  return (dist <= v.dist);
}

template<class T>
inline bool operator<=(const CoordVector<T>& v1,
		       const CoordVectorUC<T>& v2)
{
  return v2 >= v1;
}

template<class T>
inline bool operator<=(const CoordVectorUnknown<T>& v1,
		       const CoordVectorUC<T>& v2)
{
  return v2 >= v1;
}

template <class T>
inline bool CoordVectorUC<T>::operator > (const CoordVectorUnknown<T>& v) const
{
  return (dist >  v.getLength());
}

template <class T>
inline bool CoordVectorUC<T>::operator > (const CoordVectorUC<T>& v) const
{
  return (dist >  v.dist);
}

template<class T>
inline bool operator>(const CoordVector<T>& v1,
		       const CoordVectorUC<T>& v2)
{
  return v2 < v1;
}

template<class T>
inline bool operator>(const CoordVectorUnknown<T>& v1,
		       const CoordVectorUC<T>& v2)
{
  return v2 < v1;
}

template <class T>
inline bool CoordVectorUC<T>::operator >=(const CoordVectorUnknown<T>& v) const
{
  return (dist >= v.getLength());
}

template <class T>
inline bool CoordVectorUC<T>::operator >=(const CoordVectorUC<T>& v) const
{
  return (dist >= v.dist);
}

template<class T>
inline bool operator>=(const CoordVector<T>& v1,
		       const CoordVectorUC<T>& v2)
{
  return v2 <= v1;
}

template<class T>
inline bool operator>=(const CoordVectorUnknown<T>& v1,
		       const CoordVectorUC<T>& v2)
{
  return v2 <= v1;
}

template <class T>
inline bool CoordVectorUC<T>::operator !() const
{
  return (!dist) && (!dir);
}

template<class T>
inline WithUC<T> CoordVectorUC<T>::getLength() const
{
  return dist;
}

template<class T>
inline CoordVectorUC<T>& CoordVectorUC<T>::setLength(const WithUC<T>& d)
{
  dist = d;
  return *this;
}

template<class T>
inline AngleDegreesUC<T> CoordVectorUC<T>::getAngle() const
{
  return dir;
}

template<class T>
inline CoordVectorUC<T>& CoordVectorUC<T>::setAngle(const AngleDegreesUC<T>& a)
{
  dir = a;
  return *this;
}

template <class T>
inline CoordVectorUC<T>& CoordVectorUC<T>::rotate(const AngleDegreesUC<T>& vdir)
{
  dir += vdir;
  return *this;
}

template <class T>
inline CoordVectorUC<T>& CoordVectorUC<T>::scale(const WithUC<T>& vdist)
{
  dist += vdist;
  return *this;
}

template <class T>
inline std::ostream& operator<<(std::ostream& os, const CoordVectorUC<T>& v)
{
#ifdef VECTORUC_HEAD
  return os << VECTORUC_HEAD << "(" << v.getAngle() << ", "
	    << v.getLength() << ")";
#else
  return os << "(" << v.getAngle() << ", " << v.getLength() << ")";
#endif
}


RS_END_NAMESPACE


#endif // !COORDVECTORUC_H
