/* -*- Mode: C++ -*-
 * $Id: CoordVectorUCO.h,v 1.5 2000/09/18 15:20:06 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: CoordVectorUCO.h
 * Unitname: Library
 * $Revision: 1.5 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/18 15:20:06 $
 *
 *
 * HISTORY
 *
 * $Log: CoordVectorUCO.h,v $
 * Revision 1.5  2000/09/18 15:20:06  frehe
 * Due to a problem with namespaces the name for atan2, atan, acos and asin is changed to ATan2, ATan, ASin and ACos.
 *
 * Revision 1.4  2000/09/04 15:21:50  frehe
 * Changed the name of the trigonometric functions to the same as in the standard math library (sin, cos, tan, asin, acos, atan, and atan2).
 *
 * Revision 1.3  2000/09/01 11:43:01  frehe
 * Made minor changes in documentation.
 *
 * Revision 1.2  2000/09/01 03:23:53  frehe
 * Made minor changes.
 *
 * Revision 1.1  2000/09/01 01:24:39  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 *
 */

/**
 * @file CoordVectorUCO.h
 * Include file for the template CoordVectorUCO.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.5 $
 * $Author: frehe $
 * $Date: 2000/09/18 15:20:06 $
 */


#ifndef COORDVECTORUCO_H
#define COORDVECTORUCO_H

#include "WithUCO.h"
#include "AngleDegreesUCO.h"
#include "CoordVectorUC.h"
#include "CoordPointUCO.h"


#define VECTORUCO_HEAD std::string("#VectorUCO")


RS_BEGIN_NAMESPACE


template<class T, class Time> class CoordPointUCO;


/**
 * @class CoordVectorUCO CoordVectorUCO.h
 * Template class for representing a two dimensional mathematical vector
 * that can be unknown, have a confidence factor and an observation time.
 * The distance and direction are of type WithUCO<T, Time>.
 * T is the base type for the angle: AngleDegreesUCO<T, Time>
 * and the distance WithUCO<T, Time>.
 *
 * <!-- @todo -->
 *
 * @par Unit:
 *      Library
 * $Revision: 1.5 $
 * $Author: frehe $
 * $Date: 2000/09/18 15:20:06 $
 */
template<class T, class Time> class CoordVectorUCO
{
public:
  CoordVectorUCO();
  CoordVectorUCO(const T&, const T&);
  CoordVectorUCO(const AngleDegrees<T>&, const T&);
  //CoordVectorUCO(const AngleDegreesUnknown<T>&, const T&);
  //CoordVectorUCO(const AngleDegreesUCO<T, Time>&, const T&);
  CoordVectorUCO(const AngleDegreesUnknown<T>&, const WithUnknown<T>&);
  CoordVectorUCO(const AngleDegreesUC<T>&, const WithUC<T>&);
  CoordVectorUCO(const AngleDegreesUCO<T, Time>&, const WithUCO<T, Time>&);
  //CoordVectorUCO(const CoordVector<T>&);
  CoordVectorUCO(const CoordVectorUnknown<T>&);
  CoordVectorUCO(const CoordVectorUC<T>&);
  CoordVectorUCO(const CoordVectorUCO<T, Time>&);
  CoordVectorUCO(const CoordPoint<T>&, const CoordPoint<T>&);
  CoordVectorUCO(const CoordPointUnknown<T>&, const CoordPointUnknown<T>&);
  CoordVectorUCO(const CoordPointUC<T>&, const CoordPointUC<T>&);
  CoordVectorUCO(const CoordPointUCO<T, Time>&, const CoordPointUCO<T, Time>&);
  explicit CoordVectorUCO(const CoordPoint<T>&);
  explicit CoordVectorUCO(const CoordPointUnknown<T>&);
  explicit CoordVectorUCO(const CoordPointUC<T>&);
  explicit CoordVectorUCO(const CoordPointUCO<T, Time>&);

  bool isKnown() const;
  bool isUnknown() const;
  CoordVectorUCO<T, Time>& makeUnknown();

  //CConfidence getConfidence() const; // What should this be?
  
  CoordVector<T> getVector() const throw(RSException);
  //CoordVectorUnknown<T> getVectorUnknown() const;
  operator CoordVectorUnknown<T>();
  //operator CoordVectorUC<T>();
  
  CoordVectorUCO<T, Time> operator -() const;
  CoordVectorUCO<T, Time> operator -(const CoordVectorUnknown<T>&) const;
  CoordVectorUCO<T, Time> operator -(const CoordVectorUC<T>&) const;
  CoordVectorUCO<T, Time> operator -(const CoordVectorUCO<T, Time>&) const;
  CoordVectorUCO<T, Time> operator +(const CoordVectorUnknown<T>&) const; 
  CoordVectorUCO<T, Time> operator +(const CoordVectorUC<T>&) const;
  CoordVectorUCO<T, Time> operator +(const CoordVectorUCO<T, Time>&) const; 
  //CoordPointUCO<T, Time>  operator +(const CoordPointUC<T>&) const;
  CoordPointUCO<T, Time>  operator +(const CoordPointUCO<T, Time>&) const;
  CoordVectorUCO<T, Time> operator *(const WithUCO<T, Time>&) const;
  CoordVectorUCO<T, Time> operator /(const WithUCO<T, Time>&) const;

  CoordVectorUCO<T, Time>& operator =(const CoordVectorUnknown<T>&);
  CoordVectorUCO<T, Time>& operator =(const CoordVectorUC<T>&);
  CoordVectorUCO<T, Time>& operator =(const CoordVectorUCO<T, Time>&);
  CoordVectorUCO<T, Time>& operator =(const CoordPoint<T>&);  
  CoordVectorUCO<T, Time>& operator =(const CoordPointUnknown<T>&);  
  CoordVectorUCO<T, Time>& operator =(const CoordPointUC<T>&);  
  CoordVectorUCO<T, Time>& operator =(const CoordPointUCO<T, Time>&);  
  
  CoordVectorUCO<T, Time>& operator -=(const CoordVectorUnknown<T>&);
  CoordVectorUCO<T, Time>& operator -=(const CoordVectorUC<T>&);
  CoordVectorUCO<T, Time>& operator -=(const CoordVectorUCO<T, Time>&);
  CoordVectorUCO<T, Time>& operator +=(const CoordVectorUnknown<T>&);
  CoordVectorUCO<T, Time>& operator +=(const CoordVectorUC<T>&);
  CoordVectorUCO<T, Time>& operator +=(const CoordVectorUCO<T, Time>&);
  CoordVectorUCO<T, Time>& operator +=(const CoordPointUCO<T, Time>&);
  CoordVectorUCO<T, Time>& operator *=(const WithUCO<T, Time>&);
  CoordVectorUCO<T, Time>& operator /=(const WithUCO<T, Time>&);

  // is the vector the zero vector?
  bool     operator ! ()                 const;
  bool     operator ==(const CoordVectorUnknown<T>&)   const;
  bool     operator ==(const CoordVectorUC<T>&)        const;
  bool     operator ==(const CoordVectorUCO<T, Time>&) const;
  bool     operator !=(const CoordVectorUnknown<T>&)   const;
  bool     operator !=(const CoordVectorUC<T>&)        const;
  bool     operator !=(const CoordVectorUCO<T, Time>&) const;
  // These compares the length of the vectors
  bool     operator < (const CoordVectorUnknown<T>&)   const;
  bool     operator < (const CoordVectorUC<T>&)   const;
  bool     operator < (const CoordVectorUCO<T, Time>&)   const;
  bool     operator <=(const CoordVectorUnknown<T>&)   const;
  bool     operator <=(const CoordVectorUC<T>&)   const;
  bool     operator <=(const CoordVectorUCO<T, Time>&)   const;
  bool     operator > (const CoordVectorUnknown<T>&)   const;
  bool     operator > (const CoordVectorUC<T>&)   const;
  bool     operator > (const CoordVectorUCO<T, Time>&)   const;
  bool     operator >=(const CoordVectorUnknown<T>&)   const;
  bool     operator >=(const CoordVectorUC<T>&)   const;
  bool     operator >=(const CoordVectorUCO<T, Time>&)   const;

  // Shoule I include these? Instead of the above operators?
  // bool SameLength
  // bool Longer
  // bool Shorter

  // Adds to the angle resp. to the distance of the vector.
  CoordVectorUCO<T, Time>& rotate(const AngleDegreesUCO<T, Time>&);
  // Bad name! I should have thought this was multiplication not addition.
  CoordVectorUCO<T, Time>& scale (const WithUCO<T, Time>&);

  WithUCO<T, Time> getLength() const;
  CoordVectorUCO<T, Time>& setLength(const WithUCO<T, Time>&);
  AngleDegreesUCO<T, Time> getAngle() const;
  CoordVectorUCO<T, Time>& setAngle(const AngleDegreesUCO<T, Time>&);

protected:
  AngleDegreesUCO<T, Time> dir; // The direction of the vector
  WithUCO<T, Time> dist; // The distance/length of the vector
};

template <class T, class Time>
CoordVectorUCO<T, Time>::CoordVectorUCO()
  : dir(),
    dist()
{
}

template <class T, class Time>
CoordVectorUCO<T, Time>::CoordVectorUCO(const T& vdir, const T& vdist)
  : dir(vdir),
    dist(vdist)
{
  if (dist < WithUCO<T, Time>(0)) {
    dir -= AngleDegreesUCO<T, Time>(180);
    dist = -dist;
  }
}

template <class T, class Time>
CoordVectorUCO<T, Time>::CoordVectorUCO(const AngleDegrees<T>& vdir,
					const T& vdist)
  : dir(vdir),
    dist(vdist)
{
  if (dist < WithUCO<T, Time>(0)) {
    dir -= AngleDegreesUCO<T, Time>(180);
    dist = -dist;
  }
}

/*
template <class T, class Time>
CoordVectorUCO<T, Time>::CoordVectorUCO(const AngleDegreesUCO<T, Time>& vdir,
					  const T& vdist)
  : dir(vdir),
    dist(vdist)
{
  if (dist < WithUCO<T, Time>(0)) {
    dir -= AngleDegreesUCO<T, Time>(180);
    dist = -dist;
  }
}
*/

template <class T, class Time>
CoordVectorUCO<T, Time>::CoordVectorUCO(const AngleDegreesUnknown<T>& vdir,
					const WithUnknown<T>& vdist)
  : dir(vdir),
    dist(vdist)
{
  if (dist < WithUCO<T, Time>(0)) {
    dir -= AngleDegreesUCO<T, Time>(180);
    dist = -dist;
  }
}

template <class T, class Time>
CoordVectorUCO<T, Time>::CoordVectorUCO(const AngleDegreesUC<T>& vdir,
					const WithUC<T>& vdist)
  : dir(vdir),
    dist(vdist)
{
  if (dist < WithUCO<T, Time>(0)) {
    dir -= AngleDegreesUCO<T, Time>(180);
    dist = -dist;
  }
}

template <class T, class Time>
CoordVectorUCO<T, Time>::CoordVectorUCO(const AngleDegreesUCO<T, Time>& vdir,
					const WithUCO<T, Time>& vdist)
  : dir(vdir),
    dist(vdist)
{
  if (dist < WithUCO<T, Time>(0)) {
    dir -= AngleDegreesUCO<T, Time>(180);
    dist = -dist;
  }
}

/*
template <class T, class Time>
CoordVectorUCO<T, Time>::CoordVectorUCO(const CoordVector<T>& v)
  : dir(v.getAngle()),
    dist(v.getLength())
{
}
*/

template <class T, class Time>
CoordVectorUCO<T, Time>::CoordVectorUCO(const CoordVectorUnknown<T>& v)
  : dir(v.getAngle()),
    dist(v.getLength())
{
}

template <class T, class Time>
CoordVectorUCO<T, Time>::CoordVectorUCO(const CoordVectorUC<T>& v)
  : dir(v.getAngle()),
    dist(v.getLength())
{
}

template <class T, class Time>
CoordVectorUCO<T, Time>::CoordVectorUCO(const CoordVectorUCO<T, Time>& v)
  : dir(v.dir),
    dist(v.dist)
{
}

template <class T, class Time>
CoordVectorUCO<T, Time>::CoordVectorUCO(const CoordPoint<T>& pt1,
					const CoordPoint<T>& pt2)
{
  if (pt2 == pt1) {
    dir = AngleDegreesUCO<T, Time>(0);
    dist = WithUCO<T, Time>(0);
  } else {
    dir = ATan2(pt2.getY()-pt1.getY(), pt2.getX()-pt1.getX());
    dist = sqrt(sqr(pt1.getX()-pt2.getX()) + sqr(pt1.getY()-pt2.getY()));
  }
}

template <class T, class Time>
CoordVectorUCO<T, Time>::CoordVectorUCO(const CoordPointUnknown<T>& pt1,
					const CoordPointUnknown<T>& pt2)
{
  if (pt2 == pt1) {
    dir = AngleDegreesUCO<T, Time>(0);
    dist = WithUCO<T, Time>(0);
  } else {
    dir = ATan2(pt2.getY()-pt1.getY(), pt2.getX()-pt1.getX());
    dist = sqrt(sqr(pt1.getX()-pt2.getX()) + sqr(pt1.getY()-pt2.getY()));
  }
}

template <class T, class Time>
CoordVectorUCO<T, Time>::CoordVectorUCO(const CoordPointUC<T>& pt1,
					const CoordPointUC<T>& pt2)
{
  if (pt2 == pt1) {
    dir = AngleDegreesUCO<T, Time>(0);
    dist = WithUCO<T, Time>(0);
  } else {
    dir = ATan2(pt2.getY()-pt1.getY(), pt2.getX()-pt1.getX());
    dist = sqrt(sqr(pt1.getX()-pt2.getX()) + sqr(pt1.getY()-pt2.getY()));
  }
}

template <class T, class Time>
CoordVectorUCO<T, Time>::CoordVectorUCO(const CoordPointUCO<T, Time>& pt1,
					const CoordPointUCO<T, Time>& pt2)
{
  if (pt2 == pt1) {
    dir = AngleDegreesUCO<T, Time>(0);
    dist = WithUCO<T, Time>(0);
  } else {
    dir = ATan2(pt2.getY()-pt1.getY(), pt2.getX()-pt1.getX());
    dist = sqrt(sqr(pt1.getX()-pt2.getX()) + sqr(pt1.getY()-pt2.getY()));
  }
}

template <class T, class Time>
CoordVectorUCO<T, Time>::CoordVectorUCO(const CoordPoint<T>& pt)
{
  if ( pt.getX() == WithUCO<T, Time>(0) &&
       pt.getY() == WithUCO<T, Time>(0) ) {
    dir = AngleDegreesUCO<T, Time>(0);
    dist = WithUCO<T, Time>(0);
  } else {
    dir = ATan2(pt.getY(), pt.getX());
    dist = sqrt(sqr(pt.getX()) + sqr(pt.getY()));
  }
}

template <class T, class Time>
CoordVectorUCO<T, Time>::CoordVectorUCO(const CoordPointUnknown<T>& pt)
{
  if ( pt.getX() == WithUCO<T, Time>(0) &&
       pt.getY() == WithUCO<T, Time>(0) ) {
    dir = AngleDegreesUCO<T, Time>(0);
    dist = WithUCO<T, Time>(0);
  } else {
    dir = ATan2(pt.getY(), pt.getX());
    dist = sqrt(sqr(pt.getX()) + sqr(pt.getY()));
  }
}

template <class T, class Time>
CoordVectorUCO<T, Time>::CoordVectorUCO(const CoordPointUC<T>& pt)
{
  if ( pt.getX() == WithUCO<T, Time>(0) &&
       pt.getY() == WithUCO<T, Time>(0) ) {
    dir = AngleDegreesUCO<T, Time>(0);
    dist = WithUCO<T, Time>(0);
  } else {
    dir = ATan2(pt.getY(), pt.getX());
    dist = sqrt(sqr(pt.getX()) + sqr(pt.getY()));
  }
}

template <class T, class Time>
CoordVectorUCO<T, Time>::CoordVectorUCO(const CoordPointUCO<T, Time>& pt)
{
  if ( pt.getX() == WithUCO<T, Time>(0) &&
       pt.getY() == WithUCO<T, Time>(0) ) {
    dir = AngleDegreesUCO<T, Time>(0);
    dist = WithUCO<T, Time>(0);
  } else {
    dir = ATan2(pt.getY(), pt.getX());
    dist = sqrt(sqr(pt.getX()) + sqr(pt.getY()));
  }
}

template<class T, class Time>
inline bool CoordVectorUCO<T, Time>::isKnown() const
{
  return dir.isKnown() && dist.isKnown();
}

template<class T, class Time>
inline bool CoordVectorUCO<T, Time>::isUnknown() const
{
  return dir.isUnknown() && dist.isUnknown();
}

template<class T, class Time>
CoordVectorUCO<T, Time>& CoordVectorUCO<T, Time>::makeUnknown()
{
  dir.makeUnknown();
  dist.makeUnknown();
  return *this;
}

template <class T, class Time> 
inline CoordVector<T> CoordVectorUCO<T, Time>::getVector() const throw(RSException)
{
  if ( isUnknown() ) {
    RS_THROW(UnknownValueException, "Tried to take the value of an unknown vector");
    return CoordVector<T>();
  } else
    return CoordVector<T>(dir.getAngle().getAngle().getAngle(),
			  dist.getValue().getValue().getValue());
}

template<class T, class Time>
inline CoordVectorUCO<T, Time>::operator CoordVectorUnknown<T>()
{
  return CoordVectorUnknown<T>(dir, dist);
}

/*
template<class T, class Time>
inline CoordVectorUCO<T, Time>::operator CoordVectorUC<T>()
{
  return CoordVectorUC<T>(dir, dist);
}
*/

// The minus operation - Start
template <class T, class Time>
inline CoordVectorUCO<T, Time> CoordVectorUCO<T, Time>::operator -() const
{
  return CoordVectorUCO<T, Time>(dir-AngleDegrees<T>(180), dist);
}

template <class T, class Time>
inline CoordVectorUCO<T, Time> CoordVectorUCO<T, Time>::operator -(const CoordVectorUnknown<T>& v) const
{
  return *this+CoordVectorUCO<T, Time>(v.getAngle()-AngleDegrees<T>(180), v.getLength());
}

template <class T, class Time>
inline CoordVectorUCO<T, Time> CoordVectorUCO<T, Time>::operator -(const CoordVectorUC<T>& v) const
{
  return *this+CoordVectorUCO<T, Time>(v.getAngle()-AngleDegrees<T>(180), v.getLength());
}

template <class T, class Time>
inline CoordVectorUCO<T, Time> CoordVectorUCO<T, Time>::operator -(const CoordVectorUCO<T, Time>& v) const
{
  return *this+CoordVectorUCO<T, Time>(v.dir-AngleDegrees<T>(180), v.dist);
}

template <class T, class Time>
inline CoordVectorUCO<T, Time> operator -(const CoordVector<T>& v1,
					  const CoordVectorUCO<T, Time>& v2)
{
  return (-v2) + v1;
}

template <class T, class Time>
inline CoordVectorUCO<T, Time> operator -(const CoordVectorUnknown<T>& v1,
					  const CoordVectorUCO<T, Time>& v2)
{
  return (-v2) + v1;
}

template <class T, class Time>
inline CoordVectorUCO<T, Time> operator -(const CoordVectorUC<T>& v1,
					  const CoordVectorUCO<T, Time>& v2)
{
  return (-v2) + v1;
}

template <class T, class Time>
inline CoordPointUCO<T, Time> operator -(const CoordPoint<T>& v1,
					 const CoordVectorUCO<T, Time>& v2)
{
  return (-v2) + v1;
}

template <class T, class Time>
inline CoordPointUCO<T, Time> operator -(const CoordPointUnknown<T>& v1,
					 const CoordVectorUCO<T, Time>& v2)
{
  return (-v2) + v1;
}

/*
template <class T, class Time>
inline CoordPointUCO<T, Time> operator -(const CoordPointUC<T>& v1,
					 const CoordVectorUCO<T, Time>& v2)
{
  return (-v2) + v1;
}
*/

template <class T, class Time>
inline CoordPointUCO<T, Time> operator -(const CoordPointUCO<T, Time>& v1,
				  const CoordVectorUCO<T, Time>& v2)
{
  return (-v2) + v1;
}
// The minus operation - End

// The plus operation - Start
template <class T, class Time>
inline CoordVectorUCO<T, Time> CoordVectorUCO<T, Time>::operator +(const CoordVectorUnknown<T>& v) const
{
  return CoordVectorUCO<T, Time>(CoordPointUCO<T, Time>(v) + CoordPointUCO<T, Time>(*this));
}

template <class T, class Time>
inline CoordVectorUCO<T, Time> CoordVectorUCO<T, Time>::operator +(const CoordVectorUC<T>& v) const
{
  return CoordVectorUCO<T, Time>(CoordPointUCO<T, Time>(v) + CoordPointUCO<T, Time>(*this));
}

template <class T, class Time>
inline CoordVectorUCO<T, Time> CoordVectorUCO<T, Time>::operator +(const CoordVectorUCO<T, Time>& v) const
{
  return CoordVectorUCO<T, Time>(CoordPointUCO<T, Time>(v) + CoordPointUCO<T, Time>(*this));
}

template <class T, class Time>
inline CoordPointUCO<T, Time> CoordVectorUCO<T, Time>::operator+(const CoordPointUCO<T, Time>& pt1) const
{
  return CoordPointUCO<T, Time>(dist*cos(dir), dist*sin(dir)) + pt1;
}
 
template <class T, class Time>
inline CoordVectorUCO<T, Time> operator +(const CoordVector<T>& v1,
				   const CoordVectorUCO<T, Time>& v2)
{
  return v2 + v1;
}

template <class T, class Time>
inline CoordVectorUCO<T, Time> operator +(const CoordVectorUnknown<T>& v1,
				   const CoordVectorUCO<T, Time>& v2)
{
  return v2 + v1;
}

template<class T, class Time>
inline CoordPointUCO<T, Time> operator +(const CoordPoint<T>& pt1,
				  const CoordVectorUCO<T, Time>& pt2)
{
  return pt2 + pt1;
}

template<class T, class Time>
inline CoordPointUCO<T, Time> operator +(const CoordPointUnknown<T>& pt1,
				  const CoordVectorUCO<T, Time>& pt2)
{
  return pt2 + pt1;
}

template<class T, class Time>
inline CoordPointUCO<T, Time> operator +(const CoordPointUCO<T, Time>& pt1,
				  const CoordVectorUCO<T, Time>& pt2)
{
  return pt2 + pt1;
}
// The plus operation - End

// The multiply operation - Start
template <class T, class Time>
inline CoordVectorUCO<T, Time> CoordVectorUCO<T, Time>::operator *(const WithUCO<T, Time>& a) const
{
  return CoordVectorUCO<T, Time>(dir, dist * a);
}

template <class T, class Time>
inline CoordVectorUCO<T, Time> operator *(const T& a,
				   const CoordVectorUCO<T, Time>& vec)
{
  return vec * a;
}

template <class T, class Time>
inline CoordVectorUCO<T, Time> operator *(const WithUnknown<T>& a,
				   const CoordVectorUCO<T, Time>& vec)
{
  return vec * a;
}

template <class T, class Time>
inline CoordVectorUCO<T, Time> operator *(const WithUCO<T, Time>& a,
				   const CoordVectorUCO<T, Time>& vec)
{
  return vec * a;
}
// The multiply operation - End

// The division operation - Start
template <class T, class Time>
inline CoordVectorUCO<T, Time> CoordVectorUCO<T, Time>::operator /(const WithUCO<T, Time>& a) const
{
#if CHECK_DIVisION_BY_ZERO == 1
  if (a == T(0))
    return COORDVECTOR_DIVisION_BY_ZERO_RESULT;
  else
#endif
    return CoordVectorUCO<T, Time>(dir, dist / a);
}
// The division operation - End

template <class T, class Time>
CoordVectorUCO<T, Time>& CoordVectorUCO<T, Time>::operator =(const CoordVectorUnknown<T>& v)
{
  dist = v.getLength();
  dir = v.getAngle();
  return *this;
}

template <class T, class Time>
CoordVectorUCO<T, Time>& CoordVectorUCO<T, Time>::operator =(const CoordVectorUC<T>& v)
{
  dist = v.getLength();
  dir = v.getAngle();
  return *this;
}

template <class T, class Time>
CoordVectorUCO<T, Time>& CoordVectorUCO<T, Time>::operator =(const CoordVectorUCO<T, Time>& v)
{
  if (this != &v) {
    dist = v.dist;
    dir = v.dir;
  }

  return *this;
}

template <class T, class Time>
inline CoordVectorUCO<T, Time>& CoordVectorUCO<T, Time>::operator =(const CoordPoint<T>& pt)
{
  *this = CoordVectorUCO<T, Time>(pt);
  return *this;
}

template <class T, class Time>
inline CoordVectorUCO<T, Time>& CoordVectorUCO<T, Time>::operator =(const CoordPointUnknown<T>& pt)
{
  *this = CoordVectorUCO<T, Time>(pt);
  return *this;
}

template <class T, class Time>
inline CoordVectorUCO<T, Time>& CoordVectorUCO<T, Time>::operator =(const CoordPointUC<T>& pt)
{
  *this = CoordVectorUCO<T, Time>(pt);
  return *this;
}

template <class T, class Time>
inline CoordVectorUCO<T, Time>& CoordVectorUCO<T, Time>::operator =(const CoordPointUCO<T, Time>& pt)
{
  *this = CoordVectorUCO<T, Time>(pt);
  return *this;
}

template <class T, class Time>
inline CoordVectorUCO<T, Time>& CoordVectorUCO<T, Time>::operator-=(const CoordVectorUnknown<T>& v)
{
  return *this = *this - v;
}

template <class T, class Time>
inline CoordVectorUCO<T, Time>& CoordVectorUCO<T, Time>::operator-=(const CoordVectorUC<T>& v)
{
  return *this = *this - v;
}

template <class T, class Time>
inline CoordVectorUCO<T, Time>& CoordVectorUCO<T, Time>::operator-=(const CoordVectorUCO<T, Time>& v)
{
  return *this = *this - v;
}

template <class T, class Time>
inline CoordVectorUCO<T, Time>& CoordVectorUCO<T, Time>::operator+=(const CoordVectorUnknown<T>& v)
{
  return *this = *this + v;
}

template <class T, class Time>
inline CoordVectorUCO<T, Time>& CoordVectorUCO<T, Time>::operator+=(const CoordVectorUC<T>& v)
{
  return *this = *this + v;
}

template <class T, class Time>
inline CoordVectorUCO<T, Time>& CoordVectorUCO<T, Time>::operator+=(const CoordVectorUCO<T, Time>& v)
{
  return *this = *this + v;
}

template <class T, class Time>
inline CoordVectorUCO<T, Time>& CoordVectorUCO<T, Time>::operator+=(const CoordPointUCO<T, Time>& v)
{
  return *this = *this + v;
}

template <class T, class Time>
inline CoordVectorUCO<T, Time>& CoordVectorUCO<T, Time>::operator*=(const WithUCO<T, Time>& a)
{
  dist *= a;
  return *this;
}

template <class T, class Time>
inline CoordVectorUCO<T, Time>& CoordVectorUCO<T, Time>::operator/=(const WithUCO<T, Time>& a)
{
#if CHECK_DIVisION_BY_ZERO == 1
  if (a == T(0))
    dist = T_DIVisION_BY_ZERO_RESULT;
  else
#endif
    dist /= a;
  
  return *this;
}

template <class T, class Time>
inline bool CoordVectorUCO<T, Time>::operator ==(const CoordVectorUnknown<T>& v) const
{
  return (dist == v.getLength()) && (dir == v.getAngle());
}

template <class T, class Time>
inline bool CoordVectorUCO<T, Time>::operator ==(const CoordVectorUC<T>& v) const
{
  return (dist == v.getLength()) && (dir == v.getAngle());
}

template <class T, class Time>
inline bool CoordVectorUCO<T, Time>::operator ==(const CoordVectorUCO<T, Time>& v) const
{
  return (dist == v.dist) && (dir == v.dir);
}

template<class T, class Time>
inline bool operator==(const CoordVector<T>& v1,
		       const CoordVectorUCO<T, Time>& v2)
{
  return v2 == v1;
}

template<class T, class Time>
inline bool operator==(const CoordVectorUnknown<T>& v1,
		       const CoordVectorUCO<T, Time>& v2)
{
  return v2 == v1;
}

template<class T, class Time>
inline bool operator==(const CoordVectorUC<T>& v1,
		       const CoordVectorUCO<T, Time>& v2)
{
  return v2 == v1;
}

template <class T, class Time>
inline bool CoordVectorUCO<T, Time>::operator !=(const CoordVectorUnknown<T>& v) const
{
  return (dist != v.getLength()) || (dir != v.getAngle());
}

template <class T, class Time>
inline bool CoordVectorUCO<T, Time>::operator !=(const CoordVectorUC<T>& v) const
{
  return (dist != v.getLength()) || (dir != v.getAngle());
}

template <class T, class Time>
inline bool CoordVectorUCO<T, Time>::operator !=(const CoordVectorUCO<T, Time>& v) const
{
  return (dist != v.dist) || (dir != v.dir);
}

template<class T, class Time>
inline bool operator!=(const CoordVector<T>& v1,
		       const CoordVectorUCO<T, Time>& v2)
{
  return v2 != v1;
}

template<class T, class Time>
inline bool operator!=(const CoordVectorUnknown<T>& v1,
		       const CoordVectorUCO<T, Time>& v2)
{
  return v2 != v1;
}

template<class T, class Time>
inline bool operator!=(const CoordVectorUC<T>& v1,
		       const CoordVectorUCO<T, Time>& v2)
{
  return v2 != v1;
}

template <class T, class Time>
inline bool CoordVectorUCO<T, Time>::operator < (const CoordVectorUnknown<T>& v) const
{
  return (dist <  v.getLength());
}

template <class T, class Time>
inline bool CoordVectorUCO<T, Time>::operator < (const CoordVectorUC<T>& v) const
{
  return (dist <  v.getLength());
}

template <class T, class Time>
inline bool CoordVectorUCO<T, Time>::operator < (const CoordVectorUCO<T, Time>& v) const
{
  return (dist <  v.dist);
}

template<class T, class Time>
inline bool operator<(const CoordVector<T>& v1,
		      const CoordVectorUCO<T, Time>& v2)
{
  return v2 > v1;
}

template<class T, class Time>
inline bool operator<(const CoordVectorUnknown<T>& v1,
		      const CoordVectorUCO<T, Time>& v2)
{
  return v2 > v1;
}

template<class T, class Time>
inline bool operator<(const CoordVectorUC<T>& v1,
		      const CoordVectorUCO<T, Time>& v2)
{
  return v2 > v1;
}

template <class T, class Time>
inline bool CoordVectorUCO<T, Time>::operator <=(const CoordVectorUnknown<T>& v) const
{
  return (dist <= v.getLength());
}

template <class T, class Time>
inline bool CoordVectorUCO<T, Time>::operator <=(const CoordVectorUC<T>& v) const
{
  return (dist <= v.getLength());
}

template <class T, class Time>
inline bool CoordVectorUCO<T, Time>::operator <=(const CoordVectorUCO<T, Time>& v) const
{
  return (dist <= v.dist);
}

template<class T, class Time>
inline bool operator<=(const CoordVector<T>& v1,
		       const CoordVectorUCO<T, Time>& v2)
{
  return v2 >= v1;
}

template<class T, class Time>
inline bool operator<=(const CoordVectorUnknown<T>& v1,
		       const CoordVectorUCO<T, Time>& v2)
{
  return v2 >= v1;
}

template<class T, class Time>
inline bool operator<=(const CoordVectorUC<T>& v1,
		       const CoordVectorUCO<T, Time>& v2)
{
  return v2 >= v1;
}

template <class T, class Time>
inline bool CoordVectorUCO<T, Time>::operator > (const CoordVectorUnknown<T>& v) const
{
  return (dist > v.getLength());
}

template <class T, class Time>
inline bool CoordVectorUCO<T, Time>::operator > (const CoordVectorUC<T>& v) const
{
  return (dist > v.getLength());
}

template <class T, class Time>
inline bool CoordVectorUCO<T, Time>::operator > (const CoordVectorUCO<T, Time>& v) const
{
  return (dist >  v.dist);
}

template<class T, class Time>
inline bool operator>(const CoordVector<T>& v1,
		      const CoordVectorUCO<T, Time>& v2)
{
  return v2 < v1;
}

template<class T, class Time>
inline bool operator>(const CoordVectorUnknown<T>& v1,
		       const CoordVectorUCO<T, Time>& v2)
{
  return v2 < v1;
}

template<class T, class Time>
inline bool operator>(const CoordVectorUC<T>& v1,
		      const CoordVectorUCO<T, Time>& v2)
{
  return v2 < v1;
}

template <class T, class Time>
inline bool CoordVectorUCO<T, Time>::operator >=(const CoordVectorUnknown<T>& v) const
{
  return (dist >= v.getLength());
}

template <class T, class Time>
inline bool CoordVectorUCO<T, Time>::operator >=(const CoordVectorUC<T>& v) const
{
  return (dist >= v.getLength());
}

template <class T, class Time>
inline bool CoordVectorUCO<T, Time>::operator >=(const CoordVectorUCO<T, Time>& v) const
{
  return (dist >= v.dist);
}

template<class T, class Time>
inline bool operator>=(const CoordVector<T>& v1,
		       const CoordVectorUCO<T, Time>& v2)
{
  return v2 <= v1;
}

template<class T, class Time>
inline bool operator>=(const CoordVectorUnknown<T>& v1,
		       const CoordVectorUCO<T, Time>& v2)
{
  return v2 <= v1;
}

template<class T, class Time>
inline bool operator>=(const CoordVectorUC<T>& v1,
		       const CoordVectorUCO<T, Time>& v2)
{
  return v2 <= v1;
}

template <class T, class Time>
inline bool CoordVectorUCO<T, Time>::operator !() const
{
  return (!dist) && (!dir);
}

template<class T, class Time>
inline WithUCO<T, Time> CoordVectorUCO<T, Time>::getLength() const
{
  return dist;
}

template<class T, class Time>
inline CoordVectorUCO<T, Time>& CoordVectorUCO<T, Time>::setLength(const WithUCO<T, Time>& d)
{
  dist = d;
  return *this;
}

template<class T, class Time>
inline AngleDegreesUCO<T, Time> CoordVectorUCO<T, Time>::getAngle() const
{
  return dir;
}

template<class T, class Time>
inline CoordVectorUCO<T, Time>& CoordVectorUCO<T, Time>::setAngle(const AngleDegreesUCO<T, Time>& a)
{
  dir = a;
  return *this;
}

template <class T, class Time>
inline CoordVectorUCO<T, Time>& CoordVectorUCO<T, Time>::rotate(const AngleDegreesUCO<T, Time>& vdir)
{
  dir += vdir;
  return *this;
}

template <class T, class Time>
inline CoordVectorUCO<T, Time>& CoordVectorUCO<T, Time>::scale(const WithUCO<T, Time>& vdist)
{
  dist += vdist;
  return *this;
}

template <class T, class Time>
inline std::ostream& operator<<(std::ostream& os, const CoordVectorUCO<T, Time>& v)
{
#ifdef VECTORUCO_HEAD
  return os << VECTORUCO_HEAD << "(" << v.getAngle() << ", "
	    << v.getLength() << ")";
#else
  return os << "(" << v.getAngle() << ", " << v.getLength() << ")";
#endif
}


RS_END_NAMESPACE


#endif // !COORDVECTORUCO_H
