/* -*- Mode: C++ -*-
 * $Id: DashCommand.h,v 1.7 2000/09/09 11:49:44 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: DashCommand.h
 * Unitname: Library
 * $Revision: 1.7 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/09 11:49:44 $
 *
 *
 * HISTORY
 *
 * $Log: DashCommand.h,v $
 * Revision 1.7  2000/09/09 11:49:44  frehe
 * Added a new level in the command hieararchy, namely ImmediateCommand and PeriodicCommand.
 *
 * Revision 1.6  2000/09/05 21:08:49  frehe
 * Added a const version of getCommand called getConstCommand.
 *
 * Revision 1.5  2000/09/03 11:10:27  frehe
 * Made the clone method constant.
 *
 * Revision 1.4  2000/09/02 14:15:17  frehe
 * Made minor changes.
 *
 * Revision 1.3  2000/09/01 11:43:01  frehe
 * Made minor changes in documentation.
 *
 * Revision 1.2  2000/09/01 01:24:39  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file DashCommand.h
 * Include file for the class DashCommand.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.7 $
 * $Author: frehe $
 * $Date: 2000/09/09 11:49:44 $
 */


#ifndef DASHCOMMAND_H
#define DASHCOMMAND_H


#include "PeriodicCommand.h"


RS_BEGIN_NAMESPACE


/**
 * @class DashCommand DashCommand.h
 * Class used to represent the server command (dash Power).
 *
 * <!-- @invariant The class invariant is... -->
 *
 * <!-- @todo -->
 *
 * @par Unit:
 *      Library
 * $Revision: 1.7 $
 * $Author: frehe $
 * $Date: 2000/09/09 11:49:44 $
 */
class DashCommand : public PeriodicCommand {
public:
  DashCommand(const Float32 pow);
  DashCommand(const DashCommand& obj);
  virtual ~DashCommand();

  DashCommand& operator =(const DashCommand& obj);
  
  virtual bool onlyOneEachCycle() const;
  virtual const std::string serverString() const;

  Float32 getPower() const;
  void setPower(const Float32 new_power);
  
  /** Returns a new copy of the command.
    REMEMBER TO DELETE IT, OTHERWISE YOU WILL HAVE A MEMORY LEAK!!! */
  virtual DashCommand* clone() const;
  virtual const DashCommand* const getConstDashCommand() const;
  virtual DashCommand* getDashCommand();

protected:
  void copyCommand(const DashCommand& obj);

private:
  Float32 power;
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "DashCommand.icc"
#endif


#endif // !DASHCOMMAND_H
