/* -*- Mode: C++ -*-
 * $Id: FieldObject.h,v 1.2 2000/09/01 01:24:40 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: FieldObject.h
 * Unitname: Library
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/01 01:24:40 $
 *
 *
 * HISTORY
 *
 * $Log: FieldObject.h,v $
 * Revision 1.2  2000/09/01 01:24:40  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file FieldObject.h
 * Include file for the class FieldObject.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2000/09/01 01:24:40 $
 */


#ifndef FIELDOBJECT_H
#define FIELDOBJECT_H


#include "GameObject.h"


RS_BEGIN_NAMESPACE


/**
 * @class FieldObject FieldObject.h
 * Class used to represent objects on the soccer field
 * which have a position.
 *
 * <!-- @invariant The class invariant is... -->
 *
 * @todo Make a toString and operator<<
 *
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2000/09/01 01:24:40 $
 */
class FieldObject : public GameObject {
public:
  FieldObject();
  FieldObject(const PointUCO& def_point,
	      const VectorUCO& def_vector);
  FieldObject(const FieldObject& obj);
  //virtual ~FieldObject();

  FieldObject& operator= (const FieldObject& obj);

  void reset();

  FloatUCO getPosX() const;
  FloatUCO getPosY() const;
  PointUCO getPosition() const;
  void setPosition(const PointUCO& pos);

  VectorUCO getAbsoluteVectorTo(const PointUCO& pt) const;
  AngleDegUCO getAbsoluteDirectionTo(const PointUCO& pt) const;
  FloatUCO getDistanceTo(const PointUCO& pt) const;

  VectorUCO getAbsoluteVectorFrom(const PointUCO& pt) const;
  AngleDegUCO getAbsoluteDirectionFrom(const PointUCO& pt) const;
  FloatUCO getDistanceFrom(const PointUCO& pt) const;

  bool insideField() const;
  
  //bool inSector(const CSectorUCO& sec) const;
  //bool inRectangle(const CRectangleUCO& rect) const

private:
  PointUCO position;
  PointUCO defaultPosition;
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "FieldObject.icc"
#endif


#endif // !FIELDOBJECT_H
