/* -*- Mode: C++ -*-
 * $Id: GameObject.h,v 1.1 2000/09/01 01:24:40 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: GameObject.h
 * Unitname: Library
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 2000-08-31
 * Last modified by $Author: frehe $ $Date: 2000/09/01 01:24:40 $
 *
 *
 * HISTORY
 *
 * $Log: GameObject.h,v $
 * Revision 1.1  2000/09/01 01:24:40  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file GameObject.h
 * Include file for the class GameObject.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/01 01:24:40 $
 */


#ifndef GAMEOBJECT_H
#define GAMEOBJECT_H


#include "library_types_with_unknown.h"


RS_BEGIN_NAMESPACE


/**
 * @class GameObject GameObject.h
 * Class used as a base class for all objects in the
 * soccer simulation.
 *
 * <!-- @invariant The class invariant is... -->
 *
 * @todo Make a toString and operator<<
 *
 * @par Unit:
 *      Library
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/01 01:24:40 $
 */
class GameObject
{
public:
  GameObject();
  GameObject(const VectorUCO& def_vector);
  GameObject(const GameObject& obj);
  //virtual ~GameObject();

  GameObject& operator= (const GameObject& obj);

  void reset();

  void setDirection(const AngleDegUCO& dir);
  AngleDegUCO getDirection() const;
  void addDirection(const AngleDegUCO& dir);

  void setDistance(const FloatUCO dist);
  FloatUCO getDistance() const;
  void setVector(const VectorUCO& v);
  VectorUCO getVector() const;

private:
  VectorUCO vector;
  VectorUCO defaultVector;
};


std::ostream& operator<<(std::ostream& os, const GameObject& obj);


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "GameObject.icc"
#endif


#endif // !GAMEOBJECT_H
