/* -*- Mode: C++ -*-
 * $Id: KickCommand.h,v 1.4 2000/09/09 11:49:45 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: KickCommand.h
 * Unitname: Library
 * $Revision: 1.4 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/09 11:49:45 $
 *
 *
 * HISTORY
 *
 * $Log: KickCommand.h,v $
 * Revision 1.4  2000/09/09 11:49:45  frehe
 * Added a new level in the command hieararchy, namely ImmediateCommand and PeriodicCommand.
 *
 * Revision 1.3  2000/09/05 21:08:50  frehe
 * Added a const version of getCommand called getConstCommand.
 *
 * Revision 1.2  2000/09/03 11:10:28  frehe
 * Made the clone method constant.
 *
 * Revision 1.1  2000/09/02 14:16:24  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file KickCommand.h
 * Include file for the class KickCommand.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.4 $
 * $Author: frehe $
 * $Date: 2000/09/09 11:49:45 $
 */


#ifndef KICKCOMMAND_H
#define KICKCOMMAND_H


#include "PeriodicCommand.h"


RS_BEGIN_NAMESPACE


/**
 * @class KickCommand KickCommand.h
 * Class used to represent the server command (kick Power Direction).
 *
 * <!-- @invariant The class invariant is... -->
 *
 * <!-- @todo -->
 *
 * @par Unit:
 *      Library
 * $Revision: 1.4 $
 * $Author: frehe $
 * $Date: 2000/09/09 11:49:45 $
 */
class KickCommand : public PeriodicCommand {
public:
  KickCommand(const Float32 pow, const AngleDeg& ang);
  KickCommand(const KickCommand& obj);
  virtual ~KickCommand();

  KickCommand& operator =(const KickCommand& obj);
  
  virtual bool onlyOneEachCycle() const;
  virtual const std::string serverString() const;

  Float32 getPower() const;
  void setPower(const Float32);

  AngleDeg getAngle() const;
  void setAngle(const AngleDeg&);

  /** Returns a new copy of the command.
    REMEMBER TO DELETE IT, OTHERWISE YOU WILL HAVE A MEMORY LEAK!!! */
  virtual KickCommand* clone() const;
  virtual const KickCommand* const getConstKickCommand() const;
  virtual KickCommand* getKickCommand();

protected:
  void copyCommand(const KickCommand& obj);
  
private:
  Float32 power;
  AngleDeg angle;
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "KickCommand.icc"
#endif


#endif // !KICKCOMMAND_H
