/* -*- Mode: C++ -*-
 * $Id: LineObject.h,v 1.2 2000/09/01 01:24:40 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: LineObject.h
 * Unitname: Library
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/01 01:24:40 $
 *
 *
 * HISTORY
 *
 * $Log: LineObject.h,v $
 * Revision 1.2  2000/09/01 01:24:40  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file LineObject.h
 * Include file for the class LineObject.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2000/09/01 01:24:40 $
 */


#ifndef LINEOBJECT_H
#define LINEOBJECT_H


#include <iostream>

#include "GameObject.h"
#include "soccer_types.h"


RS_BEGIN_NAMESPACE


/**
 * @class LineObject LineObject.h
 * Class used to represent the four side lines in the
 * soccer field.
 *
 * <!-- @invariant The class invariant is... -->
 *
 * <!-- @todo -->
 *
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2000/09/01 01:24:40 $
 */
class LineObject : public GameObject
{
public:
  LineObject(const SideLineName = SL_UNKNOWN,
	     const VectorUCO& = VectorUCO());
  LineObject(const LineObject& obj);

  LineObject& operator= (const LineObject& obj);

  void reset();
  
  void setLineId(const SideLineName line);
  SideLineName getLineId() const;

  AngleDegUnknown getAbsoluteAngle() const;
  
private:
  SideLineName lineId;
  SideLineName defaultLine;
};


std::ostream& operator<<(std::ostream&, const LineObject&);


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "LineObject.icc"
#endif


#endif  // !LINEOBJECT_H
