/* -*- Mode: C++ -*-
 * $Id: LogArea.h,v 1.2 2000/09/05 11:07:41 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: LogArea.h
 * Unitname: Library
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/05 11:07:41 $
 *
 *
 * HISTORY
 *
 * $Log: LogArea.h,v $
 * Revision 1.2  2000/09/05 11:07:41  frehe
 * Changed the log area constants from NAME to LA_NAME.
 *
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file LogArea.h
 * Include file for the class LogArea.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2000/09/05 11:07:41 $
 */


#ifndef LOGAREA_H
#define LOGAREA_H


#include <iostream>

#include "LogAreaName.h"


RS_BEGIN_NAMESPACE


/**
 * @class LogArea LogArea.h
 * LogArea is used to keep track of the information
 * associated with a log area. Current information is
 * the parent area, if anything should be logged and
 * where to. Future information might be a function for
 * processing the data before/after printing it.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2000/09/05 11:07:41 $
 */
class LogArea {
public:
  /** Construct a log area that stores information about one log area. */
  LogArea(const LogAreaName parent_area = LA_ROBOSOC,
	  const bool do_file_log = true,
	  std::ostream* file_log_stream = NULL,
	  const bool do_screen_log = true,
	  std::ostream* screen_log_stream = NULL);
  /** Copy constructor. */
  LogArea(const LogArea& obj);
  /** Destructor. */
  ~LogArea();

  /** Assignment operator. */
  LogArea& operator=(const LogArea& obj);

  /** Set the parent area to @a parent_area. */
  void setParentArea(const LogAreaName parent_area);
  /** Get the parent area. */
  LogAreaName getParentArea() const;

  /** Turn on and off the logging to the screen. */
  void setLogScreen(const bool do_log);
  /** @c true iff the log should be sent to the screen. */
  bool logScreen() const;

  /** Turn on and off the logging to a file. */
  void setLogFile(const bool do_log);
  /** @c true iff the log should be sent to the file. */
  bool logFile() const;

  /** Set which stream the file log should be sent to. */
  void setFileLogStream(std::ostream* os);
  /** Get the file stream. */
  std::ostream* getFileLogStream();

  /** Set which stream the screen log should be sent to. */
  void setScreenLogStream(std::ostream* os);
  /** Get the screen stream. */
  std::ostream* getScreenLogStream();

private:
  /** Store which area is the parent of this area. */
  LogAreaName parentArea;
  /** If @c true then log should be sent to the file stream. */
  bool doLogScreen;
  /** Store the screen log stream. */
  std::ostream* screenLogStream;
  /** If @c true then log should be sent to the screen stream. */
  bool doLogFile;
  /** Store the file log stream. */
  std::ostream* fileLogStream;
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "LogArea.icc"
#endif


#endif
