/* -*- Mode: C++ -*-
 * $Id: LogAreaName.h,v 1.3 2000/09/05 21:06:55 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: LogAreaName.h
 * Unitname: Library
 * $Revision: 1.3 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/05 21:06:55 $
 *
 *
 * HISTORY
 *
 * $Log: LogAreaName.h,v $
 * Revision 1.3  2000/09/05 21:06:55  frehe
 * Added more log areas.
 *
 * Revision 1.2  2000/09/05 11:06:44  frehe
 * Added a number of log areas.
 *
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file LogAreaName.h
 * Include file for the type LogAreaName.
 *
 * @todo Find a better way to represent the global areas.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.3 $
 * $Author: frehe $
 * $Date: 2000/09/05 21:06:55 $
 */


#ifndef LOGAREANAME_H
#define LOGAREANAME_H


#include "machine_dependent_types.h"


RS_BEGIN_NAMESPACE


/** A type used to identify the different log areas. */
typedef Int32 LogAreaName;

/** @name Symbolic constants for log area names.
  The reason for doing it like this and not with an enumeration
  is that the user should be able to add/delete area names
  during the running of the program. */
//@{
/** Constant used to refere to a log area frequently used. */
static const LogAreaName LA_TOP     = 0;
static const LogAreaName LA_ROBOSOC = 1;

static const LogAreaName LA_FATAL     = 2;
static const LogAreaName LA_ERROR     = 3;
static const LogAreaName LA_WARNING   = 4;
static const LogAreaName LA_DEBUG     = 5;
static const LogAreaName LA_EXCEPTION = 6;
static const LogAreaName LA_ASSERT    = 7;
static const LogAreaName LA_INFO      = 8;

static const LogAreaName LA_LIBRARY = 10;

static const LogAreaName LA_BASIC_SYSTEM       = 20;
static const LogAreaName LA_ACTUATORS          = 21;
static const LogAreaName LA_CONTROLLER         = 22;
static const LogAreaName LA_DECISION           = 23;
static const LogAreaName LA_SENSORS            = 24;
static const LogAreaName LA_SERVER_INTERFACE   = 25;
static const LogAreaName LA_ACTUATOR_INTERFACE = 26;
static const LogAreaName LA_CONTROL_INTERFACE  = 27;
static const LogAreaName LA_SENSOR_INTERFACE   = 28;
static const LogAreaName LA_AGENT              = 29;
static const LogAreaName LA_BASIC_AGENT        = 30;

static const LogAreaName LA_FRAMEWORK       = 35;
static const LogAreaName LA_VIEW_MANAGER    = 36;
static const LogAreaName LA_FRAMEWORK_AGENT = 37;

static const LogAreaName LA_VIEW           = 40;
static const LogAreaName LA_PARAMETER_VIEW = 41;
static const LogAreaName LA_COMMAND_VIEW   = 42;
static const LogAreaName LA_MARKER_VIEW    = 43;
static const LogAreaName LA_AGENT_VIEW     = 44;
static const LogAreaName LA_BALL_VIEW      = 45;
static const LogAreaName LA_PLAYER_VIEW    = 46;
static const LogAreaName LA_COACH_VIEW     = 47;
static const LogAreaName LA_CONTROL_VIEW   = 48;

static const LogAreaName LA_SKILL                         = 70;
static const LogAreaName LA_RECEIVE_PASS                  = 71;
static const LogAreaName LA_BLOCK_PASS                    = 72;
static const LogAreaName LA_CUT_OFF_PASS_ANGLE            = 73;
static const LogAreaName LA_MAKE_PASSABLE                 = 74;
static const LogAreaName LA_MARK_PLAYER                   = 75;
static const LogAreaName LA_MOVE_ALONG                    = 76;
static const LogAreaName LA_PULL_OFFSIDE_TRAP             = 77;
static const LogAreaName LA_RUSH                          = 78;
static const LogAreaName LA_ZONE_DEFENCE                  = 79;
static const LogAreaName LA_MAN_TO_MAN_DEFENCE            = 80;
static const LogAreaName LA_FACE_DIR                      = 81;
static const LogAreaName LA_MOVE_TO_XY                    = 82;
static const LogAreaName LA_MOVE_TO_XY_WITH_BALL          = 83;
static const LogAreaName LA_TURN_TO_DIR                   = 84;
static const LogAreaName LA_DASH_TO_SPEED                 = 85;
static const LogAreaName LA_KICK_BALL_IN_DIR              = 86;
static const LogAreaName LA_TURN_TO_DIR_WHILE_FACING_DIR  = 87;
static const LogAreaName LA_DASH_TO_SPEED_VECTOR          = 88;
static const LogAreaName LA_KICK_BALL_TO_XY               = 89;
static const LogAreaName LA_KICK_BALL_TO_SPEED_VECTOR     = 90;
static const LogAreaName LA_MOVE_TO_XY_AVOIDING_OBSTACLES = 91;
static const LogAreaName LA_CATCH_BALL                    = 92;
static const LogAreaName LA_CLEAR_BALL                    = 93;
static const LogAreaName LA_DRIBBLE                       = 94;
static const LogAreaName LA_INTERCEPT_BALL                = 95;
static const LogAreaName LA_SCORE                         = 96;
static const LogAreaName LA_PASS                          = 97;
static const LogAreaName LA_AVOID_OFFSIDE                 = 98;
static const LogAreaName LA_CUT_OFF_GOAL_ANGLE            = 99;
static const LogAreaName LA_FIND_BALL                     = 100;
 
static const LogAreaName LA_PREDICATE      = 150;
//@}


RS_END_NAMESPACE


#endif // !LOGAREANAME_H
