/* -*- Mode: C++ -*-
 * $Id: MarkerObject.h,v 1.1 2000/09/01 03:24:31 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: MarkerObject.h
 * Unitname: Library
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/01 03:24:31 $
 *
 *
 * HISTORY
 *
 * $Log: MarkerObject.h,v $
 * Revision 1.1  2000/09/01 03:24:31  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file MarkerObject.h
 * Include file for the class MarkerObject.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/01 03:24:31 $
 */


#ifndef MARKEROBJECT_H
#define MARKEROBJECT_H


#include "StationaryObject.h"


RS_BEGIN_NAMESPACE


/**
 * @class MarkerObject MarkerObject.h
 * Class used to represent a marker.
 *
 * <!-- @invariant The class invariant is... -->
 *
 * @todo Make a toString and operator<<
 *
 * @par Unit:
 *      Library
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/01 03:24:31 $
 */
class MarkerObject : public StationaryObject
{
public:
  MarkerObject(const MarkerName = UNKNOWN_MARKER,
	       const PointUCO& def_point = PointUCO(),
	       const VectorUCO& def_vector = VectorUCO());
  MarkerObject(const MarkerObject& obj);

  MarkerObject& operator= (const MarkerObject& obj);

  void reset();
  
  void setMarkerId(const MarkerName marker);
  MarkerName getMarkerId() const;

  void setBehind(const bool new_behind);
  bool isBehind() const;
  
  void rotate();

  //PointUCO getAbsolutePosition(const AngleDegUCO& obj_ang, const Int hist) const;
  
private:
  MarkerName markerId;
  MarkerName defaultMarker;

  bool behind;
};

std::ostream& operator<<(std::ostream& os, const MarkerObject& marker);


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "MarkerObject.icc"
#endif


#endif // !MARKEROBJECT_H
