/* -*- Mode: C++ -*-
 * $Id: MobileObject.h,v 1.1 2000/09/01 03:24:31 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: MobileObject.h
 * Unitname: Library
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/01 03:24:31 $
 *
 *
 * HISTORY
 *
 * $Log: MobileObject.h,v $
 * Revision 1.1  2000/09/01 03:24:31  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file MobileObject.h
 * Include file for the class MobileObject.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/01 03:24:31 $
 */


#ifndef MOBILEOBJECT_H
#define MOBILEOBJECT_H


#include "FieldObject.h"


RS_BEGIN_NAMESPACE


/**
 * @class MobileObject MobileObject.h
 * Class used as the base class of all mobile objects,
 * i.e. those objects that can move during a game.
 *
 * <!-- @invariant The class invariant is... -->
 *
 * @todo Make a toString and operator<<
 *
 * @par Unit:
 *      Library
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/01 03:24:31 $
 */
class MobileObject : public FieldObject
{
public:
  MobileObject(const PointUCO& def_point,
	       const VectorUCO& def_vector,
	       const FloatUCO def_dist_changed,
	       const AngleDegUCO& def_dir_changed,
	       const VectorUCO& def_speed);
  MobileObject(const MobileObject& obj);
  //  ~MobileObject();

  MobileObject& operator= (const MobileObject& obj);

  void reset();

  void setDistanceChange(const FloatUCO dist);
  FloatUCO getDistanceChange() const;
  void setDirectionChange(const AngleDegUCO& dir);
  AngleDegUCO getDirectionChange() const;

  void setAbsoluteSpeedVector(const VectorUCO& v);
  // Absolute speed and direction
  VectorUCO getAbsoluteSpeedVector() const;
  // This is actually relative direction and absolute speed
  VectorUCO getRelativeSpeedVector(const AngleDegUCO& obj_ang) const;
  void setSpeed(const FloatUCO new_speed);
  FloatUCO getSpeed() const;
  void setAbsoluteSpeedDirection(const AngleDegUCO& ang);
  AngleDegUCO getAbsoluteSpeedDirection() const;
  AngleDegUCO getRelativeSpeedDirection(const AngleDegUCO& obj_ang) const;

  //void setAcceleration(const Vector& v,);
  //Vector getAcceleration() const;

  //void setSpeedDecay(const Float new_speed_decay);
  //Float getSpeedDecay() const;
  //void setSpeedMax(const Float new_speed_max);
  //Float getSpeedMax() const;
  
  // void updateMovement();
  
  // Point getFuturePosition(Int steps) const;
  // bool onCollisionCourse(const Vector& v) const;

protected:
  FloatUCO distChanged;
  AngleDegUCO dirChanged;
  VectorUCO speed;
  //VectorUCO acceleration;

  //FloatUnknown speedDecay;
  //FloatUnknown speedMax;

  FloatUCO defaultDistChanged;
  AngleDegUCO defaultDirChanged;
  VectorUCO defaultSpeed;
  //VectorUnknown defaultAcceleration;
  //FloatUnknown defaultSpeedDecay;
  //FloatUnknown defaultSpeedMax;
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "MobileObject.icc"
#endif


#endif // !MOBILEOBJECT_H
