/* -*- Mode: C++ -*-
 * $Id: MoveCommand.h,v 1.4 2000/09/09 11:49:45 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: MoveCommand.h
 * Unitname: Library
 * $Revision: 1.4 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/09 11:49:45 $
 *
 *
 * HISTORY
 *
 * $Log: MoveCommand.h,v $
 * Revision 1.4  2000/09/09 11:49:45  frehe
 * Added a new level in the command hieararchy, namely ImmediateCommand and PeriodicCommand.
 *
 * Revision 1.3  2000/09/05 21:08:50  frehe
 * Added a const version of getCommand called getConstCommand.
 *
 * Revision 1.2  2000/09/03 11:10:28  frehe
 * Made the clone method constant.
 *
 * Revision 1.1  2000/09/02 14:16:24  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file MoveCommand.h
 * Include file for the class MoveCommand.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.4 $
 * $Author: frehe $
 * $Date: 2000/09/09 11:49:45 $
 */


#ifndef MOVECOMMAND_H
#define MOVECOMMAND_H


#include "ImmediateCommand.h"


RS_BEGIN_NAMESPACE


/**
 * @class MoveCommand MoveCommand.h
 * Class used to represent the server command (move X Y).
 *
 * <!-- @invariant The class invariant is... -->
 *
 * <!-- @todo -->
 *
 * @par Unit:
 *      Library
 * $Revision: 1.4 $
 * $Author: frehe $
 * $Date: 2000/09/09 11:49:45 $
 */
class MoveCommand : public ImmediateCommand {
public:
  MoveCommand(const Point& dest);
  MoveCommand(const MoveCommand& obj);
  virtual ~MoveCommand();

  MoveCommand& operator =(const MoveCommand& obj);
  
  virtual bool onlyOneEachCycle() const;
  virtual const std::string serverString() const;

  Point getDestination() const;
  void setDestination(const Point& new_destination);

  /** Returns a new copy of the command.
    REMEMBER TO DELETE IT, OTHERWISE YOU WILL HAVE A MEMORY LEAK!!! */
  virtual MoveCommand* clone() const;
  virtual const MoveCommand* const getConstMoveCommand() const;
  virtual MoveCommand* getMoveCommand();

protected:
  void copyCommand(const MoveCommand& obj);  
  
private:
  Point destination;
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "MoveCommand.icc"
#endif


#endif // !MOVECOMMAND_H
