/* -*- Mode: C++ -*-
 * $Id: PeriodicCommand.h,v 1.1 2000/09/09 11:49:46 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: PeriodicCommand.h
 * Unitname: Library
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 2000-09-09
 * Last modified by $Author: frehe $ $Date: 2000/09/09 11:49:46 $
 *
 *
 * HISTORY
 *
 * $Log: PeriodicCommand.h,v $
 * Revision 1.1  2000/09/09 11:49:46  frehe
 * Added a new level in the command hieararchy, namely ImmediateCommand and PeriodicCommand.
 *
 */

/**
 * @file PeriodicCommand.h
 * Include file for the class PeriodicCommand.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/09 11:49:46 $
 */


#ifndef PERIODICCOMMAND_H
#define PERIODICCOMMAND_H


#include "ServerCommand.h"


RS_BEGIN_NAMESPACE


/**
 * @class PeriodicCommand PeriodicCommand.h
 * Class used as the base class of all periodic commands
 * defined in RoboSoc.
 *
 * <!-- @invariant The class invariant is... -->
 *
 * <!-- @todo -->
 *
 * @par Unit:
 *      Library
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/09 11:49:46 $
 */
class PeriodicCommand : public ServerCommand {
public:
  PeriodicCommand(const CommandType);
  PeriodicCommand(const PeriodicCommand& obj);
  virtual ~PeriodicCommand();

  PeriodicCommand& operator =(const PeriodicCommand& obj);

  
  /** Returns @c true since it is a periodic command. */
  virtual bool isPeriodicCommand() const;


protected:
  //virtual void copyCommand(const PeriodicCommand& obj);
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "PeriodicCommand.icc"
#endif


#endif // !PERIODICCOMMAND_H
