/* -*- Mode: C++ -*-
 * $Id: PlayerObject.h,v 1.1 2000/09/01 03:24:32 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: PlayerObject.h
 * Unitname: Library
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/01 03:24:32 $
 *
 *
 * HISTORY
 *
 * $Log: PlayerObject.h,v $
 * Revision 1.1  2000/09/01 03:24:32  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file PlayerObject.h
 * Include file for the class PlayerObject.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/01 03:24:32 $
 */


#ifndef PLAYEROBJECT_H
#define PLAYEROBJECT_H


#include "MobileObject.h"


RS_BEGIN_NAMESPACE


/**
 * @class PlayerObject PlayerObject.h
 * Class used to represent a player.
 *
 * <!-- @invariant The class invariant is... -->
 *
 * @todo Make a toString and operator<<
 *
 * @par Unit:
 *      Library
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/01 03:24:32 $
 */
class PlayerObject : public MobileObject {
public:
  PlayerObject(const Team def_team = T_UNKNOWN,
		const Side def_side = S_UNKNOWN,
		const UIntUnknown def_u_number = UIntUnknown(),
		//const AngleDegUCO& def_body_dir = AngleDegUCO(),
		const AngleDegUCO& def_neck_dir = AngleDegUCO(),
		const AngleDegUCO& def_face_dir = AngleDegUCO(),
		const PointUCO& def_point = PointUCO(),
		const VectorUCO& def_vector = VectorUCO(),
		const FloatUCO def_dist_changed = FloatUCO(),
		const AngleDegUCO& def_dir_changed = AngleDegUCO(),
		const VectorUCO& def_speed = VectorUCO());
  PlayerObject(const PlayerObject& obj);
  //virtual ~PlayerObject();

  PlayerObject& operator= (const PlayerObject& obj);
  bool operator< (const PlayerObject& obj) const;

  void reset();

  Team getTeam() const;
  void setTeam(const Team tm);
  void setTeam(const Side our_side); 

  Side getSide() const;
  void setSide(const Side new_side);
  
  UInt16U getUniformNumber() const;
  void setUniformNumber(const UInt16U u_num);

  AngleDegUCO getBodyDirection() const;
  //void setBodyDirection(const AngleDegUCO& ang);

  AngleDegUCO getNeckDirection() const;
  void setNeckDirection(const AngleDegUCO& ang);

  AngleDegUCO getFaceDirection() const;
  void setFaceDirection(const AngleDegUCO& ang);

  /*
  // Redefined since we now have access to the player_speed_max parameter.
  void setAbsoluteSpeedVector(const CVectorUCO& v);
  void setSpeed(const FloatUCO& new_speed);
  */
  
  VectorUCO getRelativeVectorTo(const PointUCO& pt) const;
  AngleDegUCO getRelativeDirectionTo(const PointUCO& pt) const;

protected:
  Team team;
  Side side;
  UInt16U uNumber;

  // One of these should be calculated from the other two! Probably bodyDir
  AngleDegUCO bodyDir;
  AngleDegUCO neckDir;
  AngleDegUCO faceDir;

  Team defaultTeam;
  Side defaultSide;
  UInt16U defaultUNumber;
  //AngleDegUCO defaultBodyDir;
  AngleDegUCO defaultNeckDir;
  AngleDegUCO defaultFaceDir;
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "PlayerObject.icc"
#endif


#endif // !PLAYEROBJECT_H
