/* -*- Mode: C++ -*-
 * $Id: ServerCommand.h,v 1.8 2000/09/09 11:49:47 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: ServerCommand.h
 * Unitname: Library
 * $Revision: 1.8 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/09 11:49:47 $
 *
 *
 * HISTORY
 *
 * $Log: ServerCommand.h,v $
 * Revision 1.8  2000/09/09 11:49:47  frehe
 * Added a new level in the command hieararchy, namely ImmediateCommand and PeriodicCommand.
 *
 * Revision 1.7  2000/09/05 21:08:51  frehe
 * Added a const version of getCommand called getConstCommand.
 *
 * Revision 1.6  2000/09/05 09:36:40  frehe
 * Made minor changes to improve the documentation.
 *
 * Revision 1.5  2000/09/03 14:23:19  frehe
 * Made minor changes to improve the documentation.
 *
 * Revision 1.4  2000/09/03 11:10:29  frehe
 * Made the clone method constant.
 *
 * Revision 1.3  2000/09/01 11:43:01  frehe
 * Made minor changes in documentation.
 *
 * Revision 1.2  2000/09/01 01:24:41  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 */

/**
 * @file ServerCommand.h
 * Include file for the class ServerCommand.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.8 $
 * $Author: frehe $
 * $Date: 2000/09/09 11:49:47 $
 */


#ifndef SERVERCOMMAND_H
#define SERVERCOMMAND_H


#include "Command.h"


RS_BEGIN_NAMESPACE


class ByeCommand;
class InitCommand;
class SenseBodyCommand;
class CatchCommand;
class KickCommand;
class ChangeViewCommand;
class MoveCommand;
class TurnCommand;
class DashCommand;
class SayCommand;
class TurnNeckCommand;


/**
 * @class ServerCommand ServerCommand.h
 * Class used as the base class of all server commands
 * defined in RoboSoc.
 *
 * <!-- @invariant The class invariant is... -->
 *
 * <!-- @todo -->
 *
 * @par Unit:
 *      Library
 * $Revision: 1.8 $
 * $Author: frehe $
 * $Date: 2000/09/09 11:49:47 $
 */
class ServerCommand : public Command {
public:
  ServerCommand(const CommandType);
  ServerCommand(const ServerCommand& obj);
  virtual ~ServerCommand();

  ServerCommand& operator =(const ServerCommand& obj);

  
  /** Returns a ServerCommand pointer if the object is
    actually a ServerCommand otherwise a NULL pointer. */
  virtual ServerCommand* getServerCommand();

  /** Returns a ControlCommand pointer if the object is
    actually a ControlCommand otherwise a NULL pointer. */
  virtual ControlCommand* getControlCommand();

  
  /** Returns @c true iff the Command is a ServerCommand. */
  virtual bool isServerCommand() const;

  /** Returns @c true iff the Command is a ControlCommand. */
  virtual bool isControlCommand() const;

  
  /** Returns @c true iff the ServerCommand is a PeriodicCommand. */
  virtual bool isPeriodicCommand() const;

  /** Returns @c true iff the ServerCommand is a ImmediateCommand. */
  virtual bool isImmediateCommand() const;

  
  /** Returns @c true iff only one command of this type can be
    sent each simulator cycle. */
  virtual bool onlyOneEachCycle() const = 0;

  
  /** Returns the server string that will execute the ServerCommand
    at the soccer server. */
  virtual const std::string serverString() const = 0;

  /** Get a string description of the command. */
  const std::string getDescription() const;

  
  /** Returns a new copy of the command.
   REMEMBER TO DELETE IT, OTHERWISE YOU WILL HAVE A MEMORY LEAK!!! */
  virtual ServerCommand* clone() const = 0;

  
  virtual const ByeCommand* const getConstByeCommand() const;
  virtual ByeCommand* getByeCommand();
  virtual const InitCommand* const getConstInitCommand() const;
  virtual InitCommand* getInitCommand();
  virtual const SenseBodyCommand* const getConstSenseBodyCommand() const;
  virtual SenseBodyCommand* getSenseBodyCommand();
  virtual const CatchCommand* const getConstCatchCommand() const;
  virtual CatchCommand* getCatchCommand();
  virtual const KickCommand* const getConstKickCommand() const;
  virtual KickCommand* getKickCommand();
  virtual const ChangeViewCommand* const getConstChangeViewCommand() const;
  virtual ChangeViewCommand* getChangeViewCommand();
  virtual const MoveCommand* const getConstMoveCommand() const;
  virtual MoveCommand* getMoveCommand();
  virtual const TurnCommand* const getConstTurnCommand() const;
  virtual TurnCommand* getTurnCommand();
  virtual const DashCommand* const getConstDashCommand() const;
  virtual DashCommand* getDashCommand();
  virtual const SayCommand* const getConstSayCommand() const;
  virtual SayCommand* getSayCommand();
  virtual const TurnNeckCommand* const getConstTurnNeckCommand() const;
  virtual TurnNeckCommand* getTurnNeckCommand();


protected:
  //virtual void copyCommand(const ServerCommand& obj);
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "ServerCommand.icc"
#endif


#endif // !SERVERCOMMAND_H
