/* -*- Mode: C++ -*-
 * $Id: SetCyclesSinceIOBeforeShutdown.h,v 1.2 2000/09/03 11:10:29 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: SetCyclesSinceIOBeforeShutdown.h
 * Unitname: Library
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/03 11:10:29 $
 *
 *
 * HISTORY
 *
 * $Log: SetCyclesSinceIOBeforeShutdown.h,v $
 * Revision 1.2  2000/09/03 11:10:29  frehe
 * Made the clone method constant.
 *
 * Revision 1.1  2000/09/02 15:33:03  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file SetCyclesSinceIOBeforeShutdown.h
 * Include file for the class SetCyclesSinceIOBeforeShutdown.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2000/09/03 11:10:29 $
 */


#ifndef SETCYCLESSINCEIOBEFORESHUTDOWN_H
#define SETCYCLESSINCEIOBEFORESHUTDOWN_H


#include "ControlCommand.h"


RS_BEGIN_NAMESPACE


/**
 * @class SetCyclesSinceIOBeforeShutdown SetCyclesSinceIOBeforeShutdown.h
 * Class used to represent the control command SetCyclesSinceIOBeforeShutdown
 * which should be used to control after how many cycles since the last io
 * the controller should shut down the agent since the server is probably
 * turn off anyway. The default value is 10.
 *
 * <!-- @invariant The class invariant is... -->
 *
 * <!-- @todo -->
 *
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2000/09/03 11:10:29 $
 */
class SetCyclesSinceIOBeforeShutdown : public ControlCommand {
public:
  SetCyclesSinceIOBeforeShutdown(const UInt cycs);
  SetCyclesSinceIOBeforeShutdown(const SetCyclesSinceIOBeforeShutdown& obj);
  virtual ~SetCyclesSinceIOBeforeShutdown();

  SetCyclesSinceIOBeforeShutdown& operator =(const SetCyclesSinceIOBeforeShutdown& obj);

  UInt getCycles() const;
  void setCycles(const UInt cycs);
  
  virtual const std::string getDescription() const;

  /** Returns a new copy of the command.
    REMEMBER TO DELETE IT, OTHERWISE YOU WILL HAVE A MEMORY LEAK!!! */
  virtual SetCyclesSinceIOBeforeShutdown* clone() const;
  virtual SetCyclesSinceIOBeforeShutdown* getSetCyclesSinceIOBeforeShutdown();

protected:
  void copyCommand(const SetCyclesSinceIOBeforeShutdown& obj);

private:
  UInt cycles;
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "SetCyclesSinceIOBeforeShutdown.icc"
#endif


#endif // !SETCYCLESSINCEIOBEFORESHUTDOWN_H
