/* -*- Mode: C++ -*-
 * $Id: SetTicksBeforeCommandWarning.h,v 1.2 2000/09/03 11:10:31 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: SetTicksBeforeCommandWarning.h
 * Unitname: Library
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/03 11:10:31 $
 *
 *
 * HISTORY
 *
 * $Log: SetTicksBeforeCommandWarning.h,v $
 * Revision 1.2  2000/09/03 11:10:31  frehe
 * Made the clone method constant.
 *
 * Revision 1.1  2000/09/02 15:33:04  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file SetTicksBeforeCommandWarning.h
 * Include file for the class SetTicksBeforeCommandWarning.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2000/09/03 11:10:31 $
 */


#ifndef SETTICKSBEFORECOMMANDWARNING_H
#define SETTICKSBEFORECOMMANDWARNING_H


#include "ControlCommand.h"


RS_BEGIN_NAMESPACE


/**
 * @class SetTicksBeforeCommandWarning SetTicksBeforeCommandWarning.h
 * Class used to represent the control command SetTicksBeforeCommandWarning
 * which should be used to control after how many ticks into a cycle
 * the controller will generate a command warning event to the decision
 * maker. This warning should indicate that the last time to send a
 * command this cycle is approaching.
 *
 * <!-- @invariant The class invariant is... -->
 *
 * <!-- @todo -->
 *
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2000/09/03 11:10:31 $
 */
class SetTicksBeforeCommandWarning : public ControlCommand {
public:
  SetTicksBeforeCommandWarning(const UInt _ticks);
  SetTicksBeforeCommandWarning(const SetTicksBeforeCommandWarning& obj);
  virtual ~SetTicksBeforeCommandWarning();

  SetTicksBeforeCommandWarning& operator =(const SetTicksBeforeCommandWarning& obj);

  UInt getTicks() const;
  void setTicks(const UInt _ticks);
  
  virtual const std::string getDescription() const;

  /** Returns a new copy of the command.
    REMEMBER TO DELETE IT, OTHERWISE YOU WILL HAVE A MEMORY LEAK!!! */
  virtual SetTicksBeforeCommandWarning* clone() const;
  virtual SetTicksBeforeCommandWarning* getSetTicksBeforeCommandWarning();

protected:
  void copyCommand(const SetTicksBeforeCommandWarning& obj);

private:
  UInt ticks;  
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "SetTicksBeforeCommandWarning.icc"
#endif


#endif // !SETTICKSBEFORECOMMANDWARNING_H
