/* -*- Mode: C++ -*-
 * $Id: SetTicksBetweenCycles.h,v 1.5 2000/09/03 11:10:31 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: SetTicksBetweenCycles.h
 * Unitname: Library
 * $Revision: 1.5 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/03 11:10:31 $
 *
 *
 * HISTORY
 *
 * $Log: SetTicksBetweenCycles.h,v $
 * Revision 1.5  2000/09/03 11:10:31  frehe
 * Made the clone method constant.
 *
 * Revision 1.4  2000/09/02 15:29:02  frehe
 * Made more minor changes.
 *
 * Revision 1.3  2000/09/01 11:43:01  frehe
 * Made minor changes in documentation.
 *
 * Revision 1.2  2000/09/01 01:24:41  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file SetTicksBetweenCycles.h
 * Include file for the class SetTicksBetweenCycles.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.5 $
 * $Author: frehe $
 * $Date: 2000/09/03 11:10:31 $
 */


#ifndef SETTICKSBETWEENCYCLES_H
#define SETTICKSBETWEENCYCLES_H


#include "ControlCommand.h"


RS_BEGIN_NAMESPACE


/**
 * @class SetTicksBetweenCycles SetTicksBetweenCycles.h
 * Class used to represent the control command SetTicksBetweenCycles
 * which should be used to control how many ticks a cycle usually is.
 *
 * <!-- @invariant The class invariant is... -->
 *
 * <!-- @todo -->
 *
 * @par Unit:
 *      Library
 * $Revision: 1.5 $
 * $Author: frehe $
 * $Date: 2000/09/03 11:10:31 $
 */
class SetTicksBetweenCycles : public ControlCommand
{
public:
  SetTicksBetweenCycles(const UInt _ticks);
  SetTicksBetweenCycles(const SetTicksBetweenCycles& obj);
  virtual ~SetTicksBetweenCycles();

  SetTicksBetweenCycles& operator =(const SetTicksBetweenCycles& obj);

  UInt getTicks() const;
  void setTicks(const UInt _ticks);
  
  virtual const std::string getDescription() const;

  /** Returns a new copy of the command.
   REMEMBER TO DELETE IT, OTHERWISE YOU WILL HAVE A MEMORY LEAK!!! */
  virtual SetTicksBetweenCycles* clone() const;
  virtual SetTicksBetweenCycles* getSetTicksBetweenCycles();

protected:
  void copyCommand(const SetTicksBetweenCycles& obj);

private:
  UInt ticks;
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "SetTicksBetweenCycles.icc"
#endif


#endif // !SETTICKSBETWEENCYCLES_H
