/* -*- Mode: C++ -*-
 * $Id: SetTicksBetweenSenses.h,v 1.2 2000/09/03 11:10:32 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: SetTicksBetweenSenses.h
 * Unitname: Library
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/03 11:10:32 $
 *
 *
 * HISTORY
 *
 * $Log: SetTicksBetweenSenses.h,v $
 * Revision 1.2  2000/09/03 11:10:32  frehe
 * Made the clone method constant.
 *
 * Revision 1.1  2000/09/02 15:33:05  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file SetTicksBetweenSenses.h
 * Include file for the class SetTicksBetweenSenses.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2000/09/03 11:10:32 $
 */


#ifndef SETTICKSBETWEENSENSES_H
#define SETTICKSBETWEENSENSES_H


#include "ControlCommand.h"


RS_BEGIN_NAMESPACE


/**
 * @class SetTicksBetweenSenses SetTicksBetweenSenses.h
 * Class used to represent the control command SetTicksBetweenSenses
 * which should be used to control how many ticks a cycle usually is.
 *
 * <!-- @invariant The class invariant is... -->
 *
 * <!-- @todo -->
 *
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2000/09/03 11:10:32 $
 */
class SetTicksBetweenSenses : public ControlCommand {
public:
  SetTicksBetweenSenses(const UInt _ticks);
  SetTicksBetweenSenses(const SetTicksBetweenSenses& obj);
  virtual ~SetTicksBetweenSenses();

  SetTicksBetweenSenses& operator =(const SetTicksBetweenSenses& obj);

  UInt getTicks() const;
  void setTicks(const UInt _ticks);
  
  virtual const std::string getDescription() const;

  /** Returns a new copy of the command.
    REMEMBER TO DELETE IT, OTHERWISE YOU WILL HAVE A MEMORY LEAK!!! */
  virtual SetTicksBetweenSenses* clone() const;
  virtual SetTicksBetweenSenses* getSetTicksBetweenSenses();

protected:
  void copyCommand(const SetTicksBetweenSenses& obj);

private:
  UInt ticks;
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "SetTicksBetweenSenses.icc"
#endif


#endif // !SETTICKSBETWEENSENSES_H
