/* -*- Mode: C++ -*-
 * $Id: Socket.h,v 1.1 2000/09/01 11:43:41 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: Socket.h
 * Unitname: Library
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/01 11:43:41 $
 *
 *
 * HISTORY
 *
 * $Log: Socket.h,v $
 * Revision 1.1  2000/09/01 11:43:41  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file Socket.h
 * Include file for the class Socket.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/01 11:43:41 $
 */


#ifndef SOCKET_H
#define SOCKET_H


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif


#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <netinet/in.h>

#include <stdio.h>

namespace {
#include <sys/types.h>
#include <sys/socket.h>
#if defined(HAVE_INTTYPES_H)
#  include <inttypes.h>
#endif  
#include <arpa/inet.h>
#include <netdb.h>

#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
}

#include <string>
#include "library_types.h"
#include "RSException.h"


RS_BEGIN_NAMESPACE


enum SocketType {
  SOCKET_TYPE_UNKOWN,
  SOCKET_TYPE_TCP,
  SOCKET_TYPE_UDP
};

enum SocketError {
  SOCKET_NO_ERROR,
  SOCKET_NO_DATA,
  SOCKET_NO_CONNECTION,
  SOCKET_ILLEGAL_ADDRESS,
  SOCKET_OPEN_FAILED,
  SOCKET_BIND_FAILED,
  SOCKET_CLOSE_FAILED,
  SOCKET_WRITE_FAILED,
  SOCKET_READ_FAILED,
  SOCKET_FD_ERROR,
  SOCKET_INVALID_ARGUMENT,
  SOCKET_IO_ERROR,
  SOCKET_NO_LINK,
  SOCKET_OVERFLOW,
  SOCKET_UNKNOWN_TYPE,
  SOCKET_UNKNOWN_ERROR
};



/**
 * @class SocketException Socket.h
 * Class used to represent an exception thrown
 * by a socket.
 *
 * <!-- @invariant The class invariant is... -->
 *
 * <!-- @todo -->
 *
 * @par Unit:
 *      Library
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/01 11:43:41 $
 */
class SocketException : public RSException {
public:
  SocketException(const std::string& descr,
		  const std::string& file_n = "",
		  const UInt32 line_no = 0) throw()
		    : RSException(descr, file_n, line_no) {};
};


/**
 * @class Socket Socket.h
 * Class used to represent a communication socket.
 *
 * <!-- @invariant The class invariant is... -->
 *
 * <!-- @todo -->
 *
 * @par Unit:
 *      Library
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/01 11:43:41 $
 */
class Socket {
public:
  // Constructors / destructors
  Socket(const std::string&, const UInt16 port,
	 const UInt32 max_msg_size,
	 const SocketType) throw();
  Socket(const Socket&) throw();
  ~Socket() throw(RSException);

  // Methods
  SocketType getType() const throw();

  int getFD() const throw();
  //int setFD(const int) throw();

  int getFlags() throw(RSException);
  bool setFlags(const int) throw(RSException);

  int getOwner() throw(RSException);
  bool setOwner(const int) throw(RSException);
  
  UInt32 getMaxMessageSize() const throw();
  void setMaxMessageSize(const UInt32) throw();

  bool isConnected() const throw();
  
  bool open() throw(RSException);
  bool close() throw(RSException);

  bool writeData(const std::string&) throw(RSException);
  bool writeData(const std::string&, const Int32) throw(RSException);
  bool readData(char* const, Int32*) throw(RSException);

  SocketError getErrorMessage() const throw();

  
protected:
  void setErrorMessage(int error_no) throw();
  const std::string errorMessage(int error_no) const throw();

  Socket& operator= (const Socket&);
  

private:
  // Attributes:
  std::string addr;
  UInt16 port;
  UInt32 maxMessageSize;
  const SocketType socketType;
  bool connected;
  int sockFD;
  SocketError theErrorMessage;

  struct hostent *hostEnt;
  struct in_addr *addrPtr;
  struct sockaddr_in clientAddr;
  struct sockaddr_in serverAddr;
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "Socket.icc"
#endif


#endif // !SOCKET_H
