/* -*- Mode: C++ -*-
 * $Id: Time.h,v 1.3 2000/09/20 15:06:07 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: Time.h
 * Unitname: Library
 * $Revision: 1.3 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/20 15:06:07 $
 *
 *
 * HISTORY
 *
 * $Log: Time.h,v $
 * Revision 1.3  2000/09/20 15:06:07  frehe
 * Added documentation.
 *
 * Revision 1.2  2000/09/03 20:05:51  frehe
 * Made minor changes.
 *
 * Revision 1.1  2000/09/01 01:24:41  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file Time.h
 * Include file for the class Time.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.3 $
 * $Author: frehe $
 * $Date: 2000/09/20 15:06:07 $
 */


#ifndef TIME_H
#define TIME_H


#include <string>
#include <iostream>

#include "machine_dependent_types.h"


/** String to use as the header when outputing a Time value. */
#define TIME_HEAD std::string("#Time")


RS_BEGIN_NAMESPACE


/**
 * @class Time Time.h
 * Class used to represent the soccer server time.
 * This class is based on CMUnited's implementation created by P. Stone & Co.
 *
 * <!-- @invariant The class invariant is... -->
 *
 * <!-- @todo -->
 *
 * @par Unit:
 *      Library
 * $Revision: 1.3 $
 * $Author: frehe $
 * $Date: 2000/09/20 15:06:07 $
 */
class Time {
public:
  Time(const UInt16 vt = 0, const UInt16 vs = 0);
  Time(const Time& obj);

  Time&  operator = (const UInt16);
  Time&  operator = (const Time&);

  Time   operator - (const UInt16) const;
  UInt16 operator - (const Time&)   const;
  Time   operator + (const UInt16) const;

  Time&  operator ++();
  Time&  operator ++(int);
  Time&  operator --();
  Time&  operator --(int);
  Time&  operator -=(const UInt16);
  Time&  operator -=(const Time&);
  Time&  operator +=(const UInt16);

  bool   operator ! ()             const;
  bool   operator ==(const Time&)  const;
  bool   operator ==(const UInt16) const;
  bool   operator !=(const Time&)  const;
  bool   operator !=(const UInt16) const;
  bool   operator < (const Time&)  const;
  bool   operator < (const UInt16) const;
  bool   operator <=(const Time&)  const;
  bool   operator <=(const UInt16) const;
  bool   operator > (const Time&)  const;
  bool   operator > (const UInt16) const;
  bool   operator >=(const Time&)  const;
  bool   operator >=(const UInt16) const;
  
  UInt16 getS() const;
  Time&  setS(const UInt16);
  UInt16 getT() const;
  Time&  setT(const UInt16);

  static bool isClockStopped();
  static void stopClock();
  static void startClock();
  
  static Time currentTime;
  static bool clockStopped;
  static Time lastStopClockTime;
  static Time secondLastStopClockTime;
  
private:
  /** Time from the server, this is the game time. */
  UInt16 t;
  /**  Stopped clock cycles, this is how many server cycles
    the clock has been stopped. */
  UInt16 s;
};

std::ostream& operator<<(std::ostream&  os, const Time& ct);


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "Time.icc"
#endif


#endif // !TIME_H
