/* -*- Mode: C++ -*-
 * $Id: TimeUnknown.h,v 1.2 2000/09/20 15:10:57 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: TimeUnknown.h
 * Unitname: Library
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/20 15:10:57 $
 *
 *
 * HISTORY
 *
 * $Log: TimeUnknown.h,v $
 * Revision 1.2  2000/09/20 15:10:57  frehe
 * Added documentation.
 *
 * Revision 1.1  2000/09/01 01:24:42  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file TimeUnknown.h
 * Include file for the class TimeUnknown.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2000/09/20 15:10:57 $
 */


#ifndef TIMEUNKNOWN_H
#define TIMEUNKNOWN_H


#include "Time.h"
#include "machine_dependent_types_with_unknown.h"
#include "UnknownValueException.h"


/** String to use as the header when outputing a TimeUnknown value. */
#define TIMEUNKNOWN_HEAD TIME_HEAD


RS_BEGIN_NAMESPACE


/**
 * @class TimeUnknown TimeUnknown.h
 * Class used to represent the soccer server time that can be unknown.
 * This class is based on CMUnited's implementation created by P. Stone & Co.
 *
 * <!-- @invariant The class invariant is... -->
 *
 * @todo Check to see if not more arguments could be unknown.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2000/09/20 15:10:57 $
 */
class TimeUnknown : private Time {
public:
  /** The default constructor will make the object value unknown. */
  TimeUnknown();
  TimeUnknown(const UInt16 vt, const UInt16 vs = 0);
  TimeUnknown(const Time& obj);
  TimeUnknown(const TimeUnknown& obj);

  TimeUnknown& makeUnknown();
  bool isKnown() const;
  bool isUnknown() const;
  Time getValue() const throw(UnknownValueException);

  TimeUnknown& operator = (const UInt16);
  TimeUnknown& operator = (const Time&);
  TimeUnknown& operator = (const TimeUnknown&);

  TimeUnknown  operator - (const UInt16) const;
  UInt16U      operator - (const TimeUnknown&) const;
  TimeUnknown  operator + (const UInt16) const;

  TimeUnknown& operator ++();
  TimeUnknown& operator ++(int);
  TimeUnknown& operator --();
  TimeUnknown& operator --(int);
  TimeUnknown& operator -=(const UInt16);
  TimeUnknown& operator +=(const UInt16);

  bool operator ! ()                    const;
  bool operator ==(const TimeUnknown&)  const;
  bool operator ==(const UInt16)        const;
  bool operator !=(const TimeUnknown&)  const;
  bool operator !=(const UInt16)        const;
  bool operator < (const TimeUnknown&)  const;
  bool operator < (const UInt16)        const;
  bool operator <=(const TimeUnknown&)  const;
  bool operator <=(const UInt16)        const;
  bool operator > (const TimeUnknown&)  const;
  bool operator > (const UInt16)        const;
  bool operator >=(const TimeUnknown&)  const;
  bool operator >=(const UInt16)        const;
  
  UInt16U getS() const;
  TimeUnknown&   setS(const UInt16);
  UInt16U getT() const;
  TimeUnknown&   setT(const UInt16);

private:
  bool unknown;
};


std::ostream& operator<<(std::ostream&  os, const TimeUnknown& ct);

const std::string toString(const TimeUnknown& obj);


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "TimeUnknown.icc"
#endif


#endif // !TIMEUNKNOWN_H
