/* -*- Mode: C++ -*-
 * $Id: WithUCO.h,v 1.4 2000/09/03 14:23:20 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: WithUCO.h
 * Unitname: Library
 * $Revision: 1.4 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/03 14:23:20 $
 *
 *
 * HISTORY
 *
 * $Log: WithUCO.h,v $
 * Revision 1.4  2000/09/03 14:23:20  frehe
 * Made minor changes to improve the documentation.
 *
 * Revision 1.3  2000/09/01 11:43:02  frehe
 * Made minor changes in documentation.
 *
 * Revision 1.2  2000/09/01 03:23:54  frehe
 * Made minor changes.
 *
 * Revision 1.1  2000/09/01 01:24:43  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 *
 */

/**
 * @file WithUCO.h
 * Include file for the template WithUCO.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.4 $
 * $Author: frehe $
 * $Date: 2000/09/03 14:23:20 $
 */


#ifndef WITHUCO_H
#define WITHUCO_H


#include "WithObservationTime.h"
#include "WithUC.h"


RS_BEGIN_NAMESPACE


/**
 * @class WithUCO WithUCO.h
 * Template class for representing basic types that can be both unknown,
 * have a confidence value and an observation time.
 * The only demands on the template argument T is that the arithmetic
 * functions are defined on it.
 *
 * <!-- @todo -->
 *
 * @par Unit:
 *      Library
 * $Revision: 1.4 $
 * $Author: frehe $
 * $Date: 2000/09/03 14:23:20 $
 */
template<class T, class Time> class WithUCO : public WithUC<T> {
public:
  WithUCO();
  WithUCO(const T& v,
	  const Time& t = Time(),
	  const Confidence& conf = Confidence(),
	  const bool print_obs_time = true,
	  const bool print_conf = true);
  WithUCO(const WithUnknown<T>& wu,
	  const Time& t = Time(),
	  const Confidence& conf = Confidence(),
	  const bool print_obs_time = true,
	  const bool print_conf = true);
  WithUCO(const WithConfidence<T>& wc,
	  const Time& t = Time(),
	  const bool print_obs_time = true);
  WithUCO(const WithObservationTime<T, Time>& wo,
	  const Confidence& conf = Confidence(),
	  const bool print_conf = true);
  WithUCO(const WithUC<T>& wuc,
	  const Time& t = Time(),
	  const bool print_obs_time = true);
  WithUCO(const WithUCO<T, Time>&  wuco);

  WithUC<T> getValue() const;
  Time getObservationTime() const;
  void setObservationTime(const Time& obs_time);
  T getCyclesSince(const Time& cur_time) const;

  bool printObservationTime() const;
  void setPrintObservationTime(const bool print_obs_time);
  
  WithUCO<T, Time>& operator= (const WithUCO<T, Time>& op);

  WithUCO<T, Time>& operator+= (const WithUCO<T, Time>& op);
  WithUCO<T, Time>& operator-= (const WithUCO<T, Time>& op);
  WithUCO<T, Time>& operator*= (const WithUCO<T, Time>& op);
  WithUCO<T, Time>& operator/= (const WithUCO<T, Time>& op);

  WithUCO<T, Time> operator- () const;
  WithUCO<T, Time> operator+ (const WithUCO<T, Time>& op) const;
  WithUCO<T, Time> operator- (const WithUCO<T, Time>& op) const;
  WithUCO<T, Time> operator* (const WithUCO<T, Time>& op) const;
  WithUCO<T, Time> operator/ (const WithUCO<T, Time>& op) const;

private:
  Time observationTime;
  bool doPrintObservationTime;
};

template<class T, class Time>
WithUCO<T, Time>::WithUCO()
  : WithUC<T>(),
    observationTime(),
    doPrintObservationTime(true)
{
}

template<class T, class Time>
WithUCO<T, Time>::WithUCO(const T& v,
			  const Time& t,
			  const Confidence& conf,
			  const bool print_obs_time,
			  const bool print_conf)
  : WithUC<T>(v, conf, print_conf),
    observationTime(t),
    doPrintObservationTime(print_obs_time)
{
}

template<class T, class Time>
WithUCO<T, Time>::WithUCO(const WithUnknown<T>& wu,
			  const Time& t,
			  const Confidence& conf,
			  const bool print_obs_time,
			  const bool print_conf)
  : WithUC<T>(wu, conf, print_conf),
    observationTime(t),
    doPrintObservationTime(print_obs_time)
{
}

template<class T, class Time>
WithUCO<T, Time>::WithUCO(const WithConfidence<T>& wc,
			  const Time& t,
			  const bool print_obs_time)
  : WithUC<T>(wc),
    observationTime(t),
    doPrintObservationTime(print_obs_time)
{
}

template<class T, class Time>
WithUCO<T, Time>::WithUCO(const WithObservationTime<T, Time>& wo,
			  const Confidence& conf,
			  const bool print_conf)
  : WithUC<T>(wo.getValue(), conf, print_conf),
    observationTime(wo.getObservationTime()),
    doPrintObservationTime(wo.doPrintObservationTime())
{
}

template<class T, class Time>
WithUCO<T, Time>::WithUCO(const WithUC<T>& wuc,
			  const Time& t,
			  const bool print_obs_time)
  : WithUC<T>(wuc),
    observationTime(t),
    doPrintObservationTime(print_obs_time)
{
}

template<class T, class Time>
WithUCO<T, Time>::WithUCO(const WithUCO<T, Time>& wuco)
  : WithUC<T>(wuco),
    observationTime(wuco.observationTime),
    doPrintObservationTime(wuco.doPrintObservationTime)
{
}

template <class T, class Time>
inline WithUC<T> WithUCO<T, Time>::getValue() const
{
  return WithUC<T>(*this);
}

template <class T, class Time> 
inline Time WithUCO<T, Time>::getObservationTime() const
{
  return observationTime;
}

template <class T, class Time> 
inline void WithUCO<T, Time>::setObservationTime(const Time& obs_time)
{
  observationTime = obs_time;
}

template <class T, class Time> 
inline T WithUCO<T, Time>::getCyclesSince(const Time& cur_time) const
{
  return observationTime - cur_time;
}

template <class T, class Time>
inline bool WithUCO<T, Time>::printObservationTime() const
{
  return doPrintObservationTime;
}

template <class T, class Time>
inline void WithUCO<T, Time>::setPrintObservationTime(const bool print_obs_time)
{
  doPrintObservationTime = print_obs_time;
}

template <class T, class Time>
WithUCO<T, Time>& WithUCO<T, Time>::operator= (const WithUCO<T, Time>& op)
{
  if ( &op != this ) {
    WithUC<T>::operator=(op);
    observationTime = op.observationTime;
    doPrintObservationTime = op.doPrintObservationTime;
  }

  return *this;
}

template<class T, class Time>
WithUCO<T, Time>& WithUCO<T, Time>::operator+= (const WithUCO<T, Time>& op)
{
  WithUC<T>::operator+=(op);
#ifdef RS_PESSIMisTIC_OBSERVATIONS
  observationTime = min(observationTime, op.observationTime);
#else
  observationTime = max(observationTime, op.observationTime);
#endif
  return *this;
}

template<class T, class Time>
WithUCO<T, Time>& WithUCO<T, Time>::operator-= (const WithUCO<T, Time>& op)
{
  WithUC<T>::operator-=(op);
#ifdef RS_PESSIMisTIC_OBSERVATIONS
  observationTime = min(observationTime, op.observationTime);
#else
  observationTime = max(observationTime, op.observationTime);
#endif
  return *this;
}

template<class T, class Time>
WithUCO<T, Time>& WithUCO<T, Time>::operator*= (const WithUCO<T, Time>& op)
{
  WithUC<T>::operator*=(op);
#ifdef RS_PESSIMisTIC_OBSERVATIONS
  observationTime = min(observationTime, op.observationTime);
#else
  observationTime = max(observationTime, op.observationTime);
#endif
  return *this;
}

template<class T, class Time>
WithUCO<T, Time>& WithUCO<T, Time>::operator/= (const WithUCO<T, Time>& op)
{
  WithUC<T>::operator/=(op);
#ifdef RS_PESSIMisTIC_OBSERVATIONS
  observationTime = min(observationTime, op.observationTime);
#else
  observationTime = max(observationTime, op.observationTime);
#endif
  return *this;
}

template<class T, class Time>
inline WithUCO<T, Time> WithUCO<T, Time>::operator- () const
{
  return WithUCO<T, Time>(WithUC<T>::operator-(),
			  observationTime,
			  doPrintObservationTime);
}

template<class T, class Time>
inline WithUCO<T, Time> WithUCO<T, Time>::operator+ (const WithUCO<T, Time>& op) const
{
#ifdef RS_PESSIMisTIC_OBSERVATIONS
  return WithUCO<T, Time>(WithUC<T>::operator+(op),
			  min(observationTime, op.observationTime),
			  doPrintObservationTime);
#else
  return WithUCO<T, Time>(WithUC<T>::operator+(op),
			  max(observationTime, op.observationTime),
			  doPrintObservationTime);
#endif
}

template<class T, class Time>
inline WithUCO<T, Time> WithUCO<T, Time>::operator- (const WithUCO<T, Time>& op) const
{
#ifdef RS_PESSIMisTIC_OBSERVATIONS
  return WithUCO<T, Time>(WithUC<T>::operator-(op),
			  min(observationTime, op.observationTime),
			  doPrintObservationTime);
#else
  return WithUCO<T, Time>(WithUC<T>::operator-(op),
			  max(observationTime, op.observationTime),
			  
			  doPrintObservationTime);
#endif
}

template<class T, class Time>
inline WithUCO<T, Time> WithUCO<T, Time>::operator* (const WithUCO<T, Time>& op) const
{
#ifdef RS_PESSIMisTIC_OBSERVATIONS
  return WithUCO<T, Time>(WithUC<T>::operator*(op),
			  min(observationTime, op.observationTime),
			  doPrintObservationTime);
#else
  return WithUCO<T, Time>(WithUC<T>::operator*(op),
			  max(observationTime, op.observationTime),

			  doPrintObservationTime);
#endif
}

template<class T, class Time>
inline WithUCO<T, Time> WithUCO<T, Time>::operator/ (const WithUCO<T, Time>& op) const
{
#ifdef RS_PESSIMisTIC_OBSERVATIONS
  return WithUCO<T, Time>(WithUC<T>::operator/(op),
			  min(observationTime, op.observationTime),
			  doPrintObservationTime);
#else
  return WithUCO<T, Time>(WithUC<T>::operator/(op),
			  max(observationTime, op.observationTime),

			  doPrintObservationTime);
#endif
}

template<class T, class Time>
inline WithUCO<T, Time> operator+ (const T& op1, const WithUCO<T, Time>& op2)
{
  return op2 + op1;
}

template<class T, class Time>
inline WithUCO<T, Time> operator- (const T& op1, const WithUCO<T, Time>& op2)
{
#ifdef RS_PESSIMisTIC_OBSERVATIONS
  return WithUCO<T, Time>(op1 - op2.getValue(),
			  min(Time(), op2.getObservationTime()),
			  op2.printObservationTime());
#else
  return WithUCO<T, Time>(op1 - op2.getValue(),
			  max(Time(), op2.observationTime()),
			  op2.printObservationTime());
#endif
}

template<class T, class Time>
inline WithUCO<T, Time> operator* (const T& op1, const WithUCO<T, Time>& op2)
{
  return op2 * op1;
}

template<class T, class Time>
inline WithUCO<T, Time> operator/ (const T& op1, const WithUCO<T, Time>& op2)
{
#ifdef RS_PESSIMisTIC_OBSERVATIONS
  return WithUCO<T, Time>(op1 / op2.getValue(),
			  min(Time(), op2.getObservationTime()),
			  op2.printObservationTime());
#else
  return WithUCO<T, Time>(op1 / op2.getValue(),
			  max(Time(), op2.observationTime()),
			  op2.printObservationTime());
#endif
}

template<class T, class Time>
inline WithUCO<T, Time> operator+ (const WithConfidence<T>& op1, const WithUCO<T, Time>& op2)
{
  return op2 + op1;
}

template<class T, class Time>
inline WithUCO<T, Time> operator- (const WithConfidence<T>& op1, const WithUCO<T, Time>& op2)
{
#ifdef RS_PESSIMisTIC_OBSERVATIONS
  return WithUCO<T, Time>(op1 - op2.getValue(),
			  min(Time(), op2.getObservationTime()),
			  op2.printObservationTime());
#else
  return WithUCO<T, Time>(op1 - op2.getValue(),
			  max(Time(), op2.observationTime()),
			  op2.printObservationTime());
#endif
}

template<class T, class Time>
inline WithUCO<T, Time> operator* (const WithConfidence<T>& op1, const WithUCO<T, Time>& op2)
{
  return op2 * op1;
}

template<class T, class Time>
inline WithUCO<T, Time> operator/ (const WithConfidence<T>& op1, const WithUCO<T, Time>& op2)
{
#ifdef RS_PESSIMisTIC_OBSERVATIONS
  return WithUCO<T, Time>(op1 / op2.getValue(),
			  min(Time(), op2.getObservationTime()),
			  op2.printObservationTime());
#else
  return WithUCO<T, Time>(op1 / op2.getValue(),
			  max(Time(), op2.observationTime()),
			  op2.printObservationTime());
#endif
}

template<class T, class Time>
inline WithUCO<T, Time> operator+ (const WithObservationTime<T, Time>& op1, const WithUCO<T, Time>& op2)
{
  return op2 + op1;
}

template<class T, class Time>
inline WithUCO<T, Time> operator- (const WithObservationTime<T, Time>& op1, const WithUCO<T, Time>& op2)
{
#ifdef RS_PESSIMisTIC_OBSERVATIONS
  return WithUCO<T, Time>(op1 - op2.getValue(),
			  min(Time(), op2.getObservationTime()),
			  op2.printObservationTime());
#else
  return WithUCO<T, Time>(op1 - op2.getValue(),
			  max(Time(), op2.observationTime()),
			  op2.printObservationTime());
#endif
}

template<class T, class Time>
inline WithUCO<T, Time> operator* (const WithObservationTime<T, Time>& op1, const WithUCO<T, Time>& op2)
{
  return op2 * op1;
}

template<class T, class Time>
inline WithUCO<T, Time> operator/ (const WithObservationTime<T, Time>& op1, const WithUCO<T, Time>& op2)
{
#ifdef RS_PESSIMisTIC_OBSERVATIONS
  return WithUCO<T, Time>(op1 / op2.getValue(),
			  min(Time(), op2.getObservationTime()),
			  op2.printObservationTime());
#else
  return WithUCO<T, Time>(op1 / op2.getValue(),
			  max(Time(), op2.observationTime()),
			  op2.printObservationTime());
#endif
}

template<class T, class Time>
inline WithUCO<T, Time> operator+ (const WithUC<T>& op1, const WithUCO<T, Time>& op2)
{
  return op2 + op1;
}

template<class T, class Time>
inline WithUCO<T, Time> operator- (const WithUC<T>& op1, const WithUCO<T, Time>& op2)
{
#ifdef RS_PESSIMisTIC_OBSERVATIONS
  return WithUCO<T, Time>(op1 - op2.getValue(),
			  min(Time(), op2.getObservationTime()),
			  op2.printObservationTime());
#else
  return WithUCO<T, Time>(op1 - op2.getValue(),
			  max(Time(), op2.observationTime()),
			  op2.printObservationTime());
#endif
}

template<class T, class Time>
inline WithUCO<T, Time> operator* (const WithUC<T>& op1, const WithUCO<T, Time>& op2)
{
  return op2 * op1;
}

template<class T, class Time>
inline WithUCO<T, Time> operator/ (const WithUC<T>& op1, const WithUCO<T, Time>& op2)
{
#ifdef RS_PESSIMisTIC_OBSERVATIONS
  return WithUCO<T, Time>(op1 / op2.getValue(),
			  min(Time(), op2.getObservationTime()),
			  op2.printObservationTime());
#else
  return WithUCO<T, Time>(op1 / op2.getValue(),
			  max(Time(), op2.observationTime()),
			  op2.printObservationTime());
#endif
}

template<class T, class Time>
inline WithUCO<T, Time> operator+ (const WithUnknown<T>& op1, const WithUCO<T, Time>& op2)
{
  return op2 + op1;
}

template<class T, class Time>
inline WithUCO<T, Time> operator- (const WithUnknown<T>& op1, const WithUCO<T, Time>& op2)
{
#ifdef RS_PESSIMisTIC_OBSERVATIONS
  return WithUCO<T, Time>(op1 - op2.getValue(),
			  min(Time(), op2.getObservationTime()),
			  op2.printObservationTime());
#else
  return WithUCO<T, Time>(op1 - op2.getValue(),
			  max(Time(), op2.observationTime()),
			  op2.printObservationTime());
#endif
}

template<class T, class Time>
inline WithUCO<T, Time> operator* (const WithUnknown<T>& op1, const WithUCO<T, Time>& op2)
{
  return op2 * op1;
}

template<class T, class Time>
inline WithUCO<T, Time> operator/ (const WithUnknown<T>& op1, const WithUCO<T, Time>& op2)
{
#ifdef RS_PESSIMisTIC_OBSERVATIONS
  return WithUCO<T, Time>(op1 / op2.getValue(),
			  min(Time(), op2.getObservationTime()),
			  op2.printObservationTime());
#else
  return WithUCO<T, Time>(op1 / op2.getValue(),
			  max(Time(), op2.observationTime()),
			  op2.printObservationTime());
#endif
}

template<class T, class Time>
std::ostream& operator<<(std::ostream& os, const WithUCO<T, Time>& obj)
{
  if (obj.printObservationTime())
    return os << obj.getValue() << " observed at " << obj.getObservationTime();
  else
    return os << obj.getValue(); // << " observed at " << obj.getObservationTime();
}

template<class T, class Time>
WithUCO<T, Time> sqrt(const WithUCO<T, Time>& op)
{
  return WithUCO<T, Time>(sqrt(op.getValue()));
}


RS_END_NAMESPACE


#endif // !WITHUCO_H
