/* -*- Mode: C++ -*-
 * $Id: library_types.h,v 1.2 2000/09/01 01:24:44 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: library_types.h
 * Unitname: Library
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/01 01:24:44 $
 *
 *
 * HISTORY
 *
 * $Log: library_types.h,v $
 * Revision 1.2  2000/09/01 01:24:44  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 * Revision 1.1.1.1  2000/08/31 13:26:03  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file library_types.h
 * Include file with the types used by the RoboSoc library.
 *
 * @todo Split up the types in this file on more files.
 *       Maybe have one file for the "basic" types, one with
 *       Unknown types and one with soccer related types.
 * @todo Remove the CommandType by using polymorphism instead.
 * @todo Find better names for some of the enumerations.
 * @todo Decide how to capitalize the enumeration constants.
 * @par Done:
 *      Since they are constants they should be all capitals.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2000/09/01 01:24:44 $
 */


#ifndef LIBRARY_TYPES_H
#define LIBRARY_TYPES_H


// Not needed! #include "machine_dependent_compiler_directives.h"
#include "machine_dependent_types.h"
#include "AngleDegrees.h"
#include "AngleRadians.h"
#include "CoordPoint.h"
#include "CoordVector.h"
#include "ObjectCollection.h"


RS_BEGIN_NAMESPACE


/** A type for representing the standard unsigned integer.
  It is used when the precision of the number only defines
  how exact the result will be. To increase/decrease the
  precision the user can redefine this type. */
typedef UInt16 UInt;

/** A type for representing the standard signed integer.
  It is used when the precision of the number only defines
  how exact the result will be. To increase/decrease the
  precision the user can redefine this type. */
typedef Int32 Int;

/** A type for representing the standard floating point
  number. It is used when the precision of the number only
  defines how exact the result will be. To increase/decrease
  the precision the user can redefine this type. */
typedef Float64 Float;


/** A type for representing the standard angles in degrees. */
typedef AngleDegrees<Float> AngleDeg;

/** A type for representing the standard angles in radians. */
typedef AngleRadians<Float> AngleRad;


/** A type for representing standard points. */
typedef CoordPoint<Float> Point;

/** A type for representing standard vectors. */
typedef CoordVector<Float> Vector;


/** Type for representing a collection of Int. */
typedef ObjectCollection<Int> IntCollection;

/** Type for representing a collection of Float. */
typedef ObjectCollection<Float> FloatCollection;

/** Type for representing a collection of UInt. */
typedef ObjectCollection<UInt> UIntCollection;

/** Type for representing a collection of Vector. */
typedef ObjectCollection<Vector> VectorCollection;

/** Type for representing a collection of Point. */
typedef ObjectCollection<Point> PointCollection;

/** Type for representing a collection of AngleDeg. */
typedef ObjectCollection<AngleDeg> AngleDegCollection;


/** A type for representing the different commands that can be sent. */
enum CommandType {
  CMD_Unknown, CMD_Bye, CMD_Catch,
  CMD_ChangeView, CMD_Dash,
  CMD_Init, CMD_Kick, CMD_Move,
  CMD_Say, CMD_SenseBody,
  CMD_Turn, CMD_TurnNeck,
  CMD_SetCyclesSinceIOBeforeShutdown,
  CMD_SetMinTicksBeforeSendingCommand,
  CMD_SetMaxTicksBeforeSendingCommand,
  CMD_SetTicksBeforeCommandWarning,
  CMD_SetTicksBetweenCycles,
  CMD_SetTicksBetweenSenses,
  CMD_SetTicksBetweenVisuals
};


/*
// Is this really something RoboSoc should do?
// I doubt it really strong!
// What RoboSoc *should* do is make sure an appropriate exception
// is raised.
#if CHECK_DIVISION_BY_ZERO == 1
const int INT_DIVISION_BY_ZERO_RESULT = 0;
const float FLOAT_DIVISION_BY_ZERO_RESULT = 0.0;
const double DOUBLE_DIVISION_BY_ZERO_RESULT = 0.0;
const t_float T_FLOAT_DIVISION_BY_ZERO_RESULT = DOUBLE_DIVISION_BY_ZERO_RESULT;
const t_int T_INT_DIVISION_BY_ZERO_RESULT = INT_DIVISION_BY_ZERO_RESULT;
#endif CHECK_DIVISION_BY_ZERO
*/


RS_END_NAMESPACE


#endif // !LIBRARY_TYPES_H
