/* -*- Mode: C++ -*-
 * $Id: machine_dependent_compiler_directives.h,v 1.2 2000/09/05 09:36:41 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: machine_depedent_compiler_directives.h
 * Unitname: Library
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/05 09:36:41 $
 *
 *
 * HISTORY
 *
 * $Log: machine_dependent_compiler_directives.h,v $
 * Revision 1.2  2000/09/05 09:36:41  frehe
 * Made minor changes to improve the documentation.
 *
 * Revision 1.1.1.1  2000/08/31 13:26:03  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 */

/**
 * @file machine_dependent_compiler_directives.h
 * Include file with the machine dependent compiler directives.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2000/09/05 09:36:41 $
 */


#ifndef MACHINE_DEPENDENT_COMPILER_DIRECTIVES_H
#define MACHINE_DEPENDENT_COMPILER_DIRECTIVES_H


/** @def RS_USE_INLINE If defined then inline definitions should
  be included in the header files. If it is not defined then they should be
  included in the implementation file. */
#define RS_USE_INLINE

#ifndef RS_USE_INLINE
/** @def inline is undefined if RS_USE_INLINE is not defined. */
#  define inline
#endif


/** @def RS_USE_NAMESPACE If defined then a namespace should be used
  to encapsulate all RoboSoc identifiers. The name of the namespace is
  also a macro RS_NAMESPACE. To mark the start of a potential namespace
  use RS_BEGIN_NAMESPACE and to mark the end of ir use RS_END_NAMESPACE. */
#define RS_USE_NAMESPACE

/** @def RS_NAMESPACE An abbreviation for the real name of the RoboSoc namespace. */
/** @def RS_BEGIN_NAMESPACE Used to mark the start of a potential namespace. */
/** @def RS_END_NAMESPACE Used to mark the end of a potential namespace. */
#ifdef RS_USE_NAMESPACE
/** @namespace RS Used for all classes, types and functions used by RoboSoc */
#  define RS_NAMESPACE RS
#  define RS_BEGIN_NAMESPACE namespace RS_NAMESPACE {
#  define RS_END_NAMESPACE }
#else
#  define RS_NAMESPACE
#  define RS_BEGIN_NAMESPACE
#  define RS_END_NAMESPACE
#endif


#endif // !MACHINE_DEPENDENT_COMPILER_DIRECTIVES_H
