/* -*- Mode: C++ -*-
 * $Id: rs_debug.h,v 1.3 2000/09/05 11:07:41 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: rs_debug.h
 * Unitname: Library
 * $Revision: 1.3 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/05 11:07:41 $
 *
 *
 * HISTORY
 *
 * $Log: rs_debug.h,v $
 * Revision 1.3  2000/09/05 11:07:41  frehe
 * Changed the log area constants from NAME to LA_NAME.
 *
 * Revision 1.2  2000/09/04 15:21:11  frehe
 * Made minor changes.
 *
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file rs_debug.h
 * Include file for the RoboSoc RS_DEBUG debugging macro.
 *
 * @todo Add a macro RS_DEBUG_AREA( area, msg ) that will onle debug certain
 *       areas.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.3 $
 * $Author: frehe $
 * $Date: 2000/09/05 11:07:41 $
 */


#ifndef RS_DEBUG_H
#define RS_DEBUG_H


#include "library_compiler_directives.h"


/** @def RS_DEBUG( str )
  This macro is used to report debug information.
  The debug information contains the file and line where the
  information was reported and also a string with the debug
  information. */
#ifdef RS_USE_DEBUG
  
#  ifdef RS_LOG_DEBUG

#    include "rs_log.h"

#    define RS_DEBUG( str ) RS_LOG ( RS_NAMESPACE::LA_DEBUG,  "DEBUG[On line " << __LINE__ << " in " << __FILE__ << "]: " << str )

#  else // !RS_LOG_DEBUG

#    include <string>
     RS_BEGIN_NAMESPACE
     extern void rs_debug(const std::string& str,
			  const std::string& file,
			  int line);
     RS_END_NAMESPACE
#    define RS_DEBUG( str ) RS_NAMESPACE::rs_debug((str), __FILE__, __LINE__)

#  endif // RS_LOG_DEBUG

#else // !RS_USE_DEBUG

#  define RS_DEBUG( str )

#endif // RS_USE_DEBUG


#endif // !RS_DEBUG_H
