/* -*- Mode: C++ -*-
 * $Id: rs_error.h,v 1.2 2000/09/05 11:07:42 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: rs_error.h
 * Unitname: Library
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/05 11:07:42 $
 *
 *
 * HISTORY
 *
 * $Log: rs_error.h,v $
 * Revision 1.2  2000/09/05 11:07:42  frehe
 * Changed the log area constants from NAME to LA_NAME.
 *
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file rs_error.h
 * Include file for the RoboSoc macros RS_ERROR, RS_WARNING, and RS_FATAL.
 * The macros are used for reporting errors.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2000/09/05 11:07:42 $
 */


#ifndef RS_ERROR_H
#define RS_ERROR_H


#include "library_compiler_directives.h"


/** @def RS_WARNING( str )
  This macro is used to report warnings.
  The warning contains the file and line where the warning was reported
  and also a string with warning information. */
/** @def RS_ERROR( str )
  This macro is used to report errors.
  The error contains the file and line where the error was reported
  and also a string with error information. */
/** @def RS_FATAL( str )
  This macro is used to report fatal errors.
  The fatal error contains the file and line where the fatal error was
  reported and also a string with fatal error information. */
#ifdef RS_USE_ERROR

#  ifdef RS_LOG_ERROR

#    include "rs_log.h"
#    define RS_WARNING( str ) RS_LOG( RS_NAMESPACE::LA_WARNING, "WARNING[On line " << __LINE__ << " in " << __FILE__ << "]: " << str )
#    define RS_ERROR( str ) RS_LOG( RS_NAMESPACE::LA_ERROR, "ERROR[On line " << __LINE__ << " in " << __FILE__ << "]: " << str )
#    define RS_FATAL( str ) RS_LOG( RS_NAMESPACE::LA_FATAL, "FATAL[On line " << __LINE__ << " in " << __FILE__ << "]: " << str )

#  else // !RS_LOG_ERROR

#    include <string>
     RS_BEGIN_NAMESPACE
     extern void rs_warning(const std::string& str,
			    const std::string& file,
			    int line);
     RS_END_NAMESPACE
#    define RS_WARNING( str ) RS_NAMESPACE::rs_warning(str, __FILE__, __LINE__)
     RS_BEGIN_NAMESPACE
     extern void rs_error(const std::string& str,
			  const std::string& file,
			  int line);
     RS_END_NAMESPACE
#    define RS_ERROR( str ) RS_NAMESPACE::rs_error(str, __FILE__, __LINE__)
     RS_BEGIN_NAMESPACE
     extern void rs_fatal(const std::string& str,
			  const std::string& file,
			  int line);
     RS_END_NAMESPACE
#    define RS_FATAL( str ) RS_NAMESPACE::rs_fatal(str, __FILE__, __LINE__)

#  endif // RS_LOG_ERROR

#else // !RS_USE_ERROR

#  define RS_WARNING( str )
#  define RS_ERROR( str )
#  define RS_FATAL( str )

#endif // RS_USE_ERROR


#endif // !RS_ERROR_H
