/* -*- Mode: C++ -*-
 * $Id: rs_log.h,v 1.2 2000/10/23 12:04:49 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: rs_log.h
 * Unitname: Library
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/10/23 12:04:49 $
 *
 *
 * HISTORY
 *
 * $Log: rs_log.h,v $
 * Revision 1.2  2000/10/23 12:04:49  frehe
 * Changed the std::endl to '\n' to make the output more efficient.
 *
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file rs_log.h
 * Include file for the RoboSoc RS_LOG logging macro.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2000/10/23 12:04:49 $
 */


#ifndef RS_LOG_H
#define RS_LOG_H


#include "library_compiler_directives.h"


/** @def RS_LOG( area, str ) Use this to log @a str associated with @a area to a
  LogStream object using the ostream operator<<.
  If RS_USE_LOG is defined then RS_LOG( area, str ) will send @a str to a
  LogStream object with the temporary area set to @a area. @a str can contain
  operator<< since it is textually substituted into the expression.
  If RS_USE_LOG is not defined then nothing is produced, use this if you need to
  produce fast code. */
#ifdef RS_USE_LOG

#  include "LogStream.h"

#  define RS_LOG( area, str ) *RS_NAMESPACE::LogStream::instance() << RS_NAMESPACE::setTemporaryArea(area) << str << "\n" << RS_NAMESPACE::resetTemporaryArea 
   
#else

#  define RS_LOG( area, str )

#endif // RS_USE_LOG

   
#endif // !RS_LOG_H
