/* -*- Mode: C++ -*-
 * $Id: rs_throw.h,v 1.1.1.1 2000/08/31 13:26:04 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: rs_throw.h
 * Unitname: Library
 * $Revision: 1.1.1.1 $
 * Created by: Fredrik Heintz 2000-08-29
 * Last modified by $Author: frehe $ $Date: 2000/08/31 13:26:04 $
 *
 *
 * HISTORY
 *
 * $Log: rs_throw.h,v $
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file rs_throw.h
 * Include file for the RoboSoc RS_THROW exception throwing macro.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.1.1.1 $
 * $Author: frehe $
 * $Date: 2000/08/31 13:26:04 $
 */


#ifndef RS_THROW_H
#define RS_THROW_H


#include "library_compiler_directives.h"


/** @def RS_THROW( Exception, msg )
  This macro is used to throw or raise exceptions.
  @a Exception should be the name of an exception and @a msg should
  be the string argument to the exception. */
#ifdef RS_USE_EXCEPTION

#  define RS_THROW( Exception, msg ) throw Exception(msg, __FILE__, __LINE__)

#else // !RS_USE_EXCEPTION

#  ifdef RS_LOG_EXCEPTION

#    include "rs_log.h"
#    define RS_THROW( Exception, msg ) RS_LOG( RS_NAMESPACE::EXCEPTION, "Raised " << #Exception << " on line " << __LINE__ << " in " << __FILE__ << ": " << msg )

#  else // !RS_LOG_EXCEPTION

#    include <string>
     extern void rs_throw(const std::string& exception,
			  const std::string& msg,
			  const std::string& file,
			  int line);
#    define RS_THROW( Exception, msg ) RS_NAMESPACE::rs_throw(#Exception, msg, __FILE__, __LINE__)

#  endif // RS_LOG_EXCEPTION

#endif // RS_USE_EXCEPTION


#endif // !RS_THROW_H
