/* -*- Mode: C++ -*-
 * $Id: soccer_types.h,v 1.3 2000/09/03 19:59:59 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: soccer_types.h
 * Unitname: Library
 * $Revision: 1.3 $
 * Created by: Fredrik Heintz 2000-08-29
 * Last modified by $Author: frehe $ $Date: 2000/09/03 19:59:59 $
 *
 *
 * HISTORY
 *
 * $Log: soccer_types.h,v $
 * Revision 1.3  2000/09/03 19:59:59  frehe
 * Added PM_ to all PlayMode constants.
 *
 * Revision 1.2  2000/09/01 03:23:55  frehe
 * Made minor changes.
 *
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file soccer_types.h
 * Include file with the soccer related types used by the RoboSoc library.
 *
 * @todo Find better names for some of the enumerations.
 * @todo Decide how to capitalize the enumeration constants.
 * @par Done:
 *      Since they are constants they should be all capitals.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.3 $
 * $Author: frehe $
 * $Date: 2000/09/03 19:59:59 $
 */


#ifndef SOCCER_TYPES_H
#define SOCCER_TYPES_H


// Not needed! #include "machine_dependent_compiler_directives.h"
#include "machine_dependent_types.h"


//RS_BEGIN_NAMESPACE


/** A type for representing the view width of a player. */
enum ViewWidth {
  VW_UNKNOWN,
  VW_NORMAL,
  VW_WIDE,
  VW_NARROW
};


/** A type for representing the view quality of a player. */
enum ViewQuality {
  VQ_UNKNOWN,
  VQ_HIGH,
  VQ_LOW
};


/** A type for representing the names of the side lines. */
enum SideLineName {
  SL_UNKNOWN,
  SL_LEFT,
  SL_RIGHT,
  SL_TOP,
  SL_BOTTOM
};


/** A type for representing the marker names. */
enum MarkerName {
  GOAL_L,
  GOAL_R,

  FLAG_C,
  FLAG_CT,
  FLAG_CB,
  FLAG_LT,
  FLAG_LB,
  FLAG_RT,
  FLAG_RB,

  FLAG_PLT,
  FLAG_PLC,
  FLAG_PLB,
  FLAG_PRT,
  FLAG_PRC,
  FLAG_PRB,

  FLAG_GLT,
  FLAG_GLB,
  FLAG_GRT,
  FLAG_GRB,

  FLAG_TL50,
  FLAG_TL40,
  FLAG_TL30,
  FLAG_TL20,
  FLAG_TL10,
  FLAG_T0,
  FLAG_TR10,
  FLAG_TR20,
  FLAG_TR30,
  FLAG_TR40,
  FLAG_TR50,

  FLAG_BL50,
  FLAG_BL40,
  FLAG_BL30,
  FLAG_BL20,
  FLAG_BL10,
  FLAG_B0,
  FLAG_BR10,
  FLAG_BR20,
  FLAG_BR30,
  FLAG_BR40,
  FLAG_BR50,

  FLAG_LT30,
  FLAG_LT20,
  FLAG_LT10,
  FLAG_L0,
  FLAG_LB10,
  FLAG_LB20,
  FLAG_LB30,

  FLAG_RT30,
  FLAG_RT20,
  FLAG_RT10,
  FLAG_R0,
  FLAG_RB10,
  FLAG_RB20,
  FLAG_RB30,

  UNKNOWN_MARKER,
  UNKNOWN_GOAL,
  UNKNOWN_FLAG
};


/** A type for representing the sides of the soccer field. */
enum Side {
  S_UNKNOWN,
  S_LEFT,
  S_RIGHT
};


/** A type for representing the teams in a soccer game. */
enum Team {
  T_UNKNOWN,
  T_OUR_TEAM,
  T_THEIR_TEAM
};


/** A type for representing the play modes in the soccer simulation. */
enum PlayMode {
  PM_UNKNOWN,
  PM_BEFORE_KICK_OFF,
  PM_HALF_TIME,
  PM_TIME_UP,
  PM_TIME_OVER,
  PM_EXTENDED_TIME,
  PM_PLAY_ON,
  PM_DROP_BALL,
  PM_OFFSIDE_KICK,
  PM_OUR_OFFSIDE_KICK,
  PM_THEIR_OFFSIDE_KICK,
  PM_KICK_OFF,
  PM_OUR_KICK_OFF,
  PM_THEIR_KICK_OFF,
  PM_KICK_IN,
  PM_OUR_KICK_IN,
  PM_THEIR_KICK_IN,
  PM_FREE_KICK,
  PM_OUR_FREE_KICK,
  PM_THEIR_FREE_KICK,
  PM_CORNER_KICK,
  PM_OUR_CORNER_KICK,
  PM_THEIR_CORNER_KICK,
  PM_GOAL_KICK,
  PM_OUR_GOAL_KICK,
  PM_THEIR_GOAL_KICK,
  PM_GOAL,
  PM_OUR_GOAL,
  PM_THEIR_GOAL,
  PM_GOALIE_GOT_BALL,
  PM_OUR_GOALIE_GOT_BALL,
  PM_THEIR_GOALIE_GOT_BALL
};


//RS_END_NAMESPACE


#endif // !SOCCER_TYPES_H
