/* -*- Mode: C++ -*-
 * $Id: AgentObject.cc,v 1.2 2000/10/18 23:05:41 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: AgentObject.cc
 * Unitname: Library
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/10/18 23:05:41 $
 *
 *
 * HISTORY
 *
 * $Log: AgentObject.cc,v $
 * Revision 1.2  2000/10/18 23:05:41  frehe
 * Added lastCoachMessage and lastPlayerMessage.
 *
 * Revision 1.1  2000/09/01 03:24:33  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file AgentObject.cc
 * Implementation of the class AgentObject.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2000/10/18 23:05:41 $
 */


#include "AgentObject.h"


RS_BEGIN_NAMESPACE


AgentObject::AgentObject(const Team def_team,
			 const Side def_side,
			 const UIntUnknown def_u_number,
			 const AngleDegUCO& def_neck_dir,
			 const AngleDegUCO& def_face_dir,
			 const PointUCO& def_point,
			 const VectorUCO& def_vector,
			 const FloatUCO def_dist_changed,
			 const AngleDegUCO& def_dir_changed,
			 const VectorUCO& def_speed,
			 const bool def_goalie,
			 const FloatUCO def_stamina,
			 const FloatUCO def_effort,
			 const FloatUCO def_recover,
			 const UIntUnknown def_hear_capacity,
			 const ViewWidth def_view_width,
			 const ViewQuality def_view_quality)
  : PlayerObject(def_team, def_side, def_u_number, def_neck_dir,
		 def_face_dir, def_point, def_vector,
		 def_dist_changed, def_dir_changed, def_speed),
    goalie(def_goalie),
    stamina(def_stamina),
    effort(def_effort),
    recover(def_recover),
    dashes(0),
    kicks(0),
    says(0),
    turns(0),
    turnNecks(0),
    viewWidth(def_view_width),
    viewQuality(def_view_quality),
    hearCapacityFromTeammate(def_hear_capacity),
    hearCapacityFromOpponent(def_hear_capacity),
    cyclesSinceCatch(0),
    cyclesSinceSay(0),
    defaultGoalie(def_goalie),
    defaultStamina(def_stamina),
    defaultEffort(def_effort),
    defaultRecover(def_recover),
    defaultHearCapacity(def_hear_capacity),
    defaultViewWidth(def_view_width),
    defaultViewQuality(def_view_quality),
    lastCoachMessage(""),
    lastPlayerMessage("")
{
}


AgentObject::AgentObject(const AgentObject& obj)
  : PlayerObject(obj),
    goalie      (obj.goalie),
    stamina     (obj.stamina),
    effort      (obj.effort),
    recover     (obj.recover),
    dashes      (obj.dashes),
    kicks       (obj.kicks),
    says        (obj.says),
    turns       (obj.turns),
    turnNecks   (obj.turnNecks),
    viewWidth   (obj.viewWidth),
    viewQuality (obj.viewQuality),
    hearCapacityFromTeammate(obj.hearCapacityFromTeammate),
    hearCapacityFromOpponent(obj.hearCapacityFromOpponent),
    cyclesSinceCatch   (obj.cyclesSinceCatch),
    cyclesSinceSay     (obj.cyclesSinceSay),
    defaultGoalie      (obj.defaultGoalie),
    defaultStamina     (obj.defaultStamina),
    defaultEffort      (obj.defaultEffort),
    defaultRecover     (obj.defaultRecover),
    defaultHearCapacity(obj.defaultHearCapacity),
    defaultViewWidth   (obj.defaultViewWidth),
    defaultViewQuality (obj.defaultViewQuality),
    lastCoachMessage   (obj.lastCoachMessage),
    lastPlayerMessage  (obj.lastPlayerMessage)
{
}


AgentObject::~AgentObject()
{
}


AgentObject& AgentObject::operator= (const AgentObject& obj)
{

  if (this != &obj) {
    PlayerObject::operator=(obj);
    goalie                 = obj.goalie;
    stamina                = obj.stamina;
    effort                 = obj.effort;
    recover                = obj.recover;
    dashes                 = obj.dashes;
    kicks                  = obj.kicks;
    turns                  = obj.turns;
    says                   = obj.says;
    turnNecks              = obj.turnNecks;
    viewWidth              = obj.viewWidth;
    viewQuality            = obj.viewQuality;
    hearCapacityFromTeammate = obj.hearCapacityFromTeammate;
    hearCapacityFromOpponent = obj.hearCapacityFromOpponent;
    cyclesSinceCatch      = obj.cyclesSinceCatch;
    cyclesSinceSay        = obj.cyclesSinceSay;
    defaultGoalie         = obj.defaultGoalie;
    defaultStamina        = obj.defaultStamina;
    defaultEffort         = obj.defaultEffort;
    defaultRecover        = obj.defaultRecover;
    defaultHearCapacity   = obj.defaultHearCapacity;
    defaultViewWidth      = obj.defaultViewWidth;
    defaultViewQuality    = obj.defaultViewQuality;
    lastCoachMessage      = obj.lastCoachMessage;
    lastPlayerMessage     = obj.lastPlayerMessage;
  }

  return *this;
  
}

void AgentObject::reset()
{

  PlayerObject::reset();

  goalie = defaultGoalie;
  stamina = defaultStamina;
  effort = defaultEffort;
  recover = defaultRecover;
  dashes = 0;
  kicks = 0;
  says = 0;
  turns = 0;
  turnNecks = 0;
  viewWidth = defaultViewWidth;
  viewQuality = defaultViewQuality;
  hearCapacityFromTeammate = defaultHearCapacity;
  hearCapacityFromOpponent = defaultHearCapacity;
  cyclesSinceCatch = 0;
  cyclesSinceSay = 0;
  lastCoachMessage = "";
  lastPlayerMessage = "";

}

/*
void AgentObject::NewCycle()
{

  //  std::cout << "Turns before: " << GetTurns(PREV) << " "
  //    << GetTurns(NOW) << " " << GetTurns(NEXT) << std::endl;
  CPlayerObject::NewCycle();
  Copy(PREV, NOW);
  senses = 0;
  sees = 0;
  // std::cout << "Turns after: " << GetTurns(PREV) << " "
  //    << GetTurns(NOW) << " " << GetTurns(NEXT) << std::endl;

}

void AgentObject::UpdateStamina(const t_int from, const t_int to)
{
#ifndef _REQUIRE_PARAMETERS_IN_OBJECTS_
  if ( !have_parameters ) {
    return;
  }
#endif
    
  // If the stamina is greater than the threshold for
  // increasing the effort, increase it.
  if (stamina[GetIndex(from)] <= (params->SPStamina_max * params->SPEffort_dec_thr)) {
    effort[GetIndex(to)] = Max(effort[GetIndex(from)] - params->SPEffort_dec,
			       params->SPEffort_min);
    // If it's below the threshold for decreasing it, decrease it.
  } else if (stamina[GetIndex(from)] >= (params->SPStamina_max * params->SPEffort_inc_thr)) {
    effort[GetIndex(to)] = Min(effort[GetIndex(from)] + params->SPEffort_inc,
			       params->SPEffort_max);
  } else {
    // Otherwise nothing.
    effort[GetIndex(to)] = effort[GetIndex(from)];
  }
  
  // If the new stamina is less than the threshold for
  // decreasing the recover, do it.
  // Since it never increases no check for that is needed.
  if (stamina[GetIndex(from)] <= (params->SPStamina_max * params->SP_recover_dec_thr)) {
    recover[GetIndex(to)] = Max(recover[GetIndex(from)] - params->SP_recover_dec,
				params->SP_recover_min);
  }
  
  stamina[GetIndex(to)] = Min(params->SPStamina_max,
			      stamina[GetIndex(from)] + recover[GetIndex(to)] * params->SPStamina_inc_max);
  // Since stamina never should be negative it can't be less than zero
  // before or after the previous statement.
}

void AgentObject::UpdateAuditoryInfo()
{
#ifndef _REQUIRE_PARAMETERS_IN_OBJECTS_
  if ( !have_parameters ) {
    return;
  }
#endif
    
  cyclesSinceSay += 1;
  hearCapacity_from_teammate = Min(hearCapacity_from_teammate + params->SP_hear_inc,
				    params->SP_hear_max);
  hearCapacity_from_opponent = Min(hearCapacity_from_opponent + params->SP_hear_inc,
				    params->SP_hear_max);
}

void AgentObject::UpdateCatchInfo()
{
  cyclesSinceCatch += 1;
}
*/

/*
void AgentObject::Print(const t_int hist) const
{

  if (hist == 9999) {
    for (t_int i = -max_hist_length; i < max_length - max_hist_length; ++i) {
      CPlayerObject::Print(i);
      std::cout << "[" << i << "]"
		<< " Stamina: " << GetStamina(i)
		<< " Effort: " << GetEffort(i)
		<< " Recover: " << GetRecover(i)
		<< " Dashes: " << GetDashes(i)
		<< " Kicks: " << GetKicks(i)
		<< " Turns: " << GetTurns(i)
		<< " TurnNecks: " << GetTurnNecks(i)
		<< " Says: " << GetSays(i)
		<< std::endl;
    }
  } else {
    CPlayerObject::Print(hist);
    std::cout << "[" << hist << "]"
	      << " Stamina: " << GetStamina(hist)
	      << " Effort: " << GetEffort(hist)
	      << " Recover: " << GetRecover(hist)
	      << " Dashes: " << GetDashes(hist)
	      << " Kicks: " << GetKicks(hist)
	      << " Turns: " << GetTurns(hist)
	      << " TurnNecks: " << GetTurnNecks(hist)
	      << " Says: " << GetSays(hist)
	      << std::endl;
  }
  
}
*/


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "AgentObject.icc"
#endif
