/* -*- Mode: C++ -*-
 * $Id: BallObject.cc,v 1.1 2000/09/01 03:24:33 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: BallObject.cc
 * Unitname: Library
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/01 03:24:33 $
 *
 *
 * HISTORY
 *
 * $Log: BallObject.cc,v $
 * Revision 1.1  2000/09/01 03:24:33  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file BallObject.cc
 * Implementation of the class BallObject.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/01 03:24:33 $
 */


#include "BallObject.h"


RS_BEGIN_NAMESPACE


BallObject::BallObject(const PointUCO& def_point,
		       const VectorUCO& def_vector,
		       const FloatUCO def_dist_changed,
		       const AngleDegUCO& def_dir_changed,
		       const VectorUCO& def_speed,
		       const PointUnknown def_caught_ball_position,
		       const VectorUnknown def_relative_ball_position,
		       const TimeUnknown def_last_seen_moving_time)
  : MobileObject(def_point, def_vector, def_dist_changed, def_dir_changed, def_speed),
    caughtBallPosition(def_caught_ball_position),
    relativeBallPosition(def_relative_ball_position),
    lastSeenMovingTime(def_last_seen_moving_time),
    defaultCaughtBallPosition(def_caught_ball_position),
    defaultRelativeBallPosition(def_relative_ball_position),
    defaultLastSeenMovingTime(def_last_seen_moving_time)
{
}


BallObject::BallObject(const BallObject& obj)
  : MobileObject(obj),
    caughtBallPosition(obj.caughtBallPosition),
    relativeBallPosition(obj.relativeBallPosition),
    lastSeenMovingTime(obj.lastSeenMovingTime),
    defaultCaughtBallPosition(obj.defaultCaughtBallPosition),
    defaultRelativeBallPosition(obj.defaultRelativeBallPosition),
    defaultLastSeenMovingTime(obj.defaultLastSeenMovingTime)    
{
}


void BallObject::reset()
{
  MobileObject::reset();
  caughtBallPosition = defaultCaughtBallPosition;
  relativeBallPosition = defaultRelativeBallPosition;
  lastSeenMovingTime = defaultLastSeenMovingTime;
}  


BallObject& BallObject::operator= (const BallObject& obj)
{
  if (this != &obj) {
    MobileObject::operator=(obj);

    caughtBallPosition = caughtBallPosition;
    relativeBallPosition = relativeBallPosition;
    lastSeenMovingTime = lastSeenMovingTime;
  }
  return *this;
}


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "BallObject.icc"
#endif
