/* -*- Mode: C++ -*-
 * $Id: DashCommand.cc,v 1.4 2000/09/09 11:49:53 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: DashCommand.cc
 * Unitname: Library
 * $Revision: 1.4 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/09 11:49:53 $
 *
 *
 * HISTORY
 *
 * $Log: DashCommand.cc,v $
 * Revision 1.4  2000/09/09 11:49:53  frehe
 * Added a new level in the command hieararchy, namely ImmediateCommand and PeriodicCommand.
 *
 * Revision 1.3  2000/09/01 11:43:03  frehe
 * Made minor changes in documentation.
 *
 * Revision 1.2  2000/08/31 13:59:36  frehe884
 * Added documentation. [frehe884]
 *
 * Revision 1.1.1.1  2000/08/31 13:26:03  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file DashCommand.cc
 * Implementation of the class DashCommand.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.4 $
 * $Author: frehe $
 * $Date: 2000/09/09 11:49:53 $
 */


#include "DashCommand.h"
#include "string_utils.h"


RS_BEGIN_NAMESPACE


DashCommand::DashCommand(const Float32 pow)
  : PeriodicCommand(CMD_Dash),
    power(pow)
{
}

DashCommand::DashCommand(const DashCommand& obj)
  : PeriodicCommand(obj),
    power(obj.power)
{
}

DashCommand::~DashCommand()
{
}

void DashCommand::copyCommand(const DashCommand& obj)
{
  PeriodicCommand::copyCommand(obj);
  power = obj.power;
}

DashCommand& DashCommand::operator =(const DashCommand& obj)
{
  if ( this != &obj ) {
    copyCommand(obj);
  }

  return *this;
}

const std::string DashCommand::serverString() const
{
  return "(dash " + toString(power) + ")";
}


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "DashCommand.icc"
#endif
