/* -*- Mode: C++ -*-
 * $Id: FieldObject.cc,v 1.1 2000/09/01 01:24:45 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: FieldObject.cc
 * Unitname: Library
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/01 01:24:45 $
 *
 *
 * HISTORY
 *
 * $Log: FieldObject.cc,v $
 * Revision 1.1  2000/09/01 01:24:45  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file FieldObject.cc
 * Implementation of the class FieldObject.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/01 01:24:45 $
 */


#include "FieldObject.h"


RS_BEGIN_NAMESPACE


FieldObject::FieldObject()
  : GameObject()
{
}

FieldObject::FieldObject(const PointUCO& def_point,
			 const VectorUCO& def_vector)
  : GameObject(def_vector),
    position(def_point),
    defaultPosition(def_point)
{
}

FieldObject::FieldObject(const FieldObject& obj)
  : GameObject(obj),
    position(obj.position),
    defaultPosition(obj.defaultPosition)
{
}

/*
FieldObject::~FieldObject()
{
}
*/


FieldObject& FieldObject::operator= (const FieldObject& obj)
{
  if (this != &obj) {
    GameObject::operator=(obj);
    position        = obj.position;
    defaultPosition = obj.defaultPosition;
  }

  return *this;
}


VectorUCO FieldObject::getAbsoluteVectorTo(const PointUCO& pt) const
{
  
  return position.getVectorTo(pt);

}


AngleDegUCO FieldObject::getAbsoluteDirectionTo(const PointUCO& pt) const
{

  return position.getDirectionTo(pt);

}


FloatUCO FieldObject::getDistanceTo(const PointUCO& pt) const
{

  return position.getDistanceTo(pt);

}


VectorUCO FieldObject::getAbsoluteVectorFrom(const PointUCO& pt) const
{
  
  return pt.getVectorTo(position);

}


AngleDegUCO FieldObject::getAbsoluteDirectionFrom(const PointUCO& pt) const
{

  return pt.getDirectionTo(position);

}


FloatUCO FieldObject::getDistanceFrom(const PointUCO& pt) const
{

  return pt.getDistanceTo(position);

}


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "FieldObject.icc"
#endif
