/* -*- Mode: C++ -*-
 * $Id: KickCommand.cc,v 1.2 2000/09/09 11:49:53 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: KickCommand.cc
 * Unitname: Library
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/09 11:49:53 $
 *
 *
 * HISTORY
 *
 * $Log: KickCommand.cc,v $
 * Revision 1.2  2000/09/09 11:49:53  frehe
 * Added a new level in the command hieararchy, namely ImmediateCommand and PeriodicCommand.
 *
 * Revision 1.1  2000/09/02 14:16:28  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file KickCommand.cc
 * Implementation of the class KickCommand.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2000/09/09 11:49:53 $
 */


#include "KickCommand.h"
#include "string_utils.h"


RS_BEGIN_NAMESPACE


KickCommand::KickCommand(const Float32 pow, const AngleDeg& ang)
  : PeriodicCommand(CMD_Kick),
    power(pow),
    angle(ang)
{
}


KickCommand::KickCommand(const KickCommand& obj)
  : PeriodicCommand(obj),
    power(obj.power),
    angle(obj.angle)
{
}


KickCommand::~KickCommand()
{
}


void KickCommand::copyCommand(const KickCommand& obj)
{
  PeriodicCommand::copyCommand(obj);
  power = obj.power;
  angle = obj.angle;
}


KickCommand& KickCommand::operator=(const KickCommand& obj)
{
  if ( this != &obj ) {
    copyCommand(obj);
  }

  return *this;
}


const std::string KickCommand::serverString() const
{
  return "(kick " + toString(power) + " " + toString(angle.getDeg()) + ")";
}


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "KickCommand.icc"
#endif
