/* -*- Mode: C++ -*-
 * $Id: LineObject.cc,v 1.1 2000/09/01 01:24:45 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: LineObject.cc
 * Unitname: Library
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/01 01:24:45 $
 *
 *
 * HISTORY
 *
 * $Log: LineObject.cc,v $
 * Revision 1.1  2000/09/01 01:24:45  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file LineObject.cc
 * Implementation of the class LineObject.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/01 01:24:45 $
 */


#include "LineObject.h"
#include "rs_error.h"
#include "rs_debug.h"


RS_BEGIN_NAMESPACE


LineObject::LineObject(const SideLineName line,
		       const VectorUCO& vector)
  : GameObject(vector),
    lineId(line),
    defaultLine(line)
{  
}


LineObject::LineObject(const LineObject& obj)
  : GameObject(obj),
    lineId(obj.lineId),
    defaultLine(obj.defaultLine)
{
}


LineObject& LineObject::operator= (const LineObject& obj)
{
  if (this != &obj) {
    GameObject::operator=(obj);
    lineId = obj.lineId;
    defaultLine = obj.defaultLine;
  }

  return *this;
}


void LineObject::reset()
{
  GameObject::reset();
  lineId = defaultLine;
}


std::ostream& operator<<(std::ostream& os, const LineObject& line)
{
  os << "#Line(" << line.getLineId() << /* GameObject(line) << */ ")";
  return os;
}


AngleDegUnknown LineObject::getAbsoluteAngle() const
{
  AngleDegUnknown angle = getDirection();

  switch ( lineId ) {
    case SL_TOP:
      if ( angle > 0 ) {
        return -angle;
      }
      else {
        return AngleDegUnknown(180) - angle;
      } 
      break;
    case SL_BOTTOM:
      if ( angle > 0 ) {
        return AngleDegUnknown(180) - angle;
      }
      else {
        return -angle;
      }
      break;
    case SL_LEFT:
      if ( angle > 0 ) {
        return AngleDegUnknown(270) - angle;
      }
      else if ( angle < 0 ) {
        return AngleDegUnknown(90) - angle;
      } 
      else {
	RS_DEBUG("Couldn't compute face dir. Special case 1 is not treated.\n");
	return AngleDegUnknown();
      }
      break;
    case SL_RIGHT:
      if ( angle > 0 ) {
        return AngleDegUnknown(90) - angle;
      }
      else if ( angle < 0 ) {
        return AngleDegUnknown(270) - angle;
      } 
      else {
	RS_DEBUG("Couldn't compute face dir. Special case 2 is not treated.\n");
	return AngleDegUnknown();
      }
      break;
    default:
      RS_WARNING("Unknown line id in computeFaceDir.\n");
      return AngleDegUnknown();
  }

  return AngleDegUnknown();
  
}


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "LineObject.icc"
#endif
