/* -*- Mode: C++ -*-
 * $Id: MarkerObject.cc,v 1.1 2000/09/01 03:24:33 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: MarkerObject.cc
 * Unitname: Library
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/01 03:24:33 $
 *
 *
 * HISTORY
 *
 * $Log: MarkerObject.cc,v $
 * Revision 1.1  2000/09/01 03:24:33  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file MarkerObject.cc
 * Implementation of the class MarkerObject.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/01 03:24:33 $
 */


#include "MarkerObject.h"


RS_BEGIN_NAMESPACE


MarkerObject::MarkerObject(const MarkerName marker,
			   const PointUCO& def_point,
			   const VectorUCO& def_vector)
  : StationaryObject(def_point, def_vector),
    markerId(marker),
    defaultMarker(marker),
    behind(false)
{
}

MarkerObject::MarkerObject(const MarkerObject& obj)
  : StationaryObject(obj),
    markerId(obj.markerId),
    defaultMarker(obj.defaultMarker),
    behind(obj.behind)
{
}

MarkerObject& MarkerObject::operator= (const MarkerObject& obj)
{
  if (this != &obj) {
    StationaryObject::operator=(obj);
    markerId      = obj.markerId;
    defaultMarker = obj.defaultMarker;
    behind        = obj.behind;
  }

  return *this;
}

void MarkerObject::reset()
{
  StationaryObject::reset();
  markerId = defaultMarker;
  behind   = false;
}

/*
PointUCO MarkerObject::getAbsolutePosition(const AngleDegUCO& obj_ang,
					   const Int hist) const
{
  AngleDeg abs_ang = getDirection(hist) + obj_ang;
  Vector rel_vec(abs_ang, getDistance(hist));
  
  return getPosition(hist) - rel_vec;
}
*/

std::ostream& operator<<(std::ostream& os, const MarkerObject& marker)
{
  os << "#Marker(" << marker.getMarkerId() << ", "
     << marker.getPosition() << ", "
     << marker.getVector() << ", "
     << marker.isBehind() << ")";
  return os;
}


RS_END_NAMESPACE
