/* -*- Mode: C++ -*-
 * $Id: Plan.cc,v 1.3 2000/10/25 19:34:00 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: Plan.cc
 * Unitname: Library
 * $Revision: 1.3 $
 * Created by: Fredrik Heintz 2000-09-03
 * Last modified by $Author: frehe $ $Date: 2000/10/25 19:34:00 $
 *
 *
 * HISTORY
 *
 * $Log: Plan.cc,v $
 * Revision 1.3  2000/10/25 19:34:00  frehe
 * Fixed Plan to check for empty steps and plans.
 *
 * Revision 1.2  2000/09/05 11:08:04  frehe
 * Improved the default constructor.
 *
 * Revision 1.1  2000/09/03 13:35:57  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file Plan.cc
 * Implementation file for the class Plan.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.3 $
 * $Author: frehe $
 * $Date: 2000/10/25 19:34:00 $
 */


#include "Plan.h"
#include "rs_assert.h"


RS_BEGIN_NAMESPACE


// --- CONSTRUCTORS ---

Plan::Plan(const Step& step)
  : stepSequence()
{
  if ( !step.empty() )
    addStep(step);
  RS_CHECK_INVARIANT;
}

Plan::Plan(const Plan& obj)
  : stepSequence(obj.stepSequence)
{
  RS_CHECK_INVARIANT;
}


// --- DESTRUCTOR ---

Plan::~Plan()
{
  RS_CHECK_INVARIANT;
  // Clean up the old data
  makeEmpty();
}


// --- OPERATORS ---

Plan& Plan::operator=(const Plan& obj)
{
  RS_CHECK_INVARIANT;
  if ( &obj != this ) {
    // Clean up the old data
    makeEmpty();
    // Copy the new data from obj
    stepSequence = obj.stepSequence;
  }
  
  RS_CHECK_INVARIANT;
  return *this;
}


// --- METHODS ---

Step Plan::getNextStep() const
{
  RS_CHECK_INVARIANT;
  Step step;
  if ( !empty() ) {
    step = stepSequence.front();
  }
  RS_CHECK_INVARIANT;
  return step;
}


Step Plan::getLastStep() const
{
  RS_CHECK_INVARIANT;
  Step step;
  if ( !empty() ) {
    step = stepSequence.back();
  }
  RS_CHECK_INVARIANT;
  return step;
}


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "plan.icc"
#endif
