/* -*- Mode: C++ -*-
 * $Id: PlayerObject.cc,v 1.1 2000/09/01 03:24:34 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: PlayerObject.cc
 * Unitname: Library
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/01 03:24:34 $
 *
 *
 * HISTORY
 *
 * $Log: PlayerObject.cc,v $
 * Revision 1.1  2000/09/01 03:24:34  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file PlayerObject.cc
 * Implementation of the class PlayerObject.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/01 03:24:34 $
 */


#include "PlayerObject.h"


RS_BEGIN_NAMESPACE


PlayerObject::PlayerObject(const Team def_team,
			   const Side def_side,
			   const UIntUnknown def_u_number,
			   //const AngleDegUCO& def_body_dir,
			   const AngleDegUCO& def_neck_dir,
			   const AngleDegUCO& def_face_dir,
			   const PointUCO& def_point,
			   const VectorUCO& def_vector,
			   const FloatUCO def_dist_changed,
			   const AngleDegUCO& def_dir_changed,
			   const VectorUCO& def_speed)
  : MobileObject(def_point, def_vector, def_dist_changed,
		 def_dir_changed, def_speed),
    team(def_team),
    side(def_side),
    uNumber(def_u_number),
    bodyDir(def_face_dir - def_neck_dir),
    neckDir(def_neck_dir),
    faceDir(def_face_dir),
    defaultTeam(def_team),
    defaultSide(def_side),
    defaultUNumber(def_u_number),
    //defaultBodyDir(def_body_dir),
    defaultNeckDir(def_neck_dir),
    defaultFaceDir(def_face_dir)
{
}  


PlayerObject::PlayerObject(const PlayerObject& obj)
  : MobileObject(obj),
    team(obj.team),
    side(obj.side),
    uNumber(obj.uNumber),
    bodyDir(obj.bodyDir),
    neckDir(obj.neckDir),
    faceDir(obj.faceDir),
    defaultTeam(obj.defaultTeam),
    defaultSide(obj.defaultSide),
    defaultUNumber(obj.defaultUNumber),
    //defaultBodyDir(obj.defaultBodyDir),
    defaultNeckDir(obj.defaultNeckDir),
    defaultFaceDir(obj.defaultFaceDir)
{
}  


/*
PlayerObject::~PlayerObject()
{
}
*/


PlayerObject& PlayerObject::operator= (const PlayerObject& obj)
{
  if (this != &obj) {
    MobileObject::operator=(obj);
    team           = obj.team;
    side           = obj.side;
    uNumber        = obj.uNumber;
    bodyDir        = obj.bodyDir;
    neckDir        = obj.neckDir;
    faceDir        = obj.faceDir;      
    defaultTeam    = obj.defaultTeam;
    defaultSide    = obj.defaultSide;
    defaultUNumber = obj.defaultUNumber;
    //defaultBodyDir = obj.defaultBodyDir;
    defaultNeckDir = obj.defaultNeckDir;
    defaultFaceDir = obj.defaultFaceDir;      
  }

  return *this;
}


bool PlayerObject::operator< (const PlayerObject& obj) const
{
  return ( getDistance() < obj.getDistance() );
}


void PlayerObject::reset()
{
  MobileObject::reset();
  team    = defaultTeam;
  side    = defaultSide;
  uNumber = defaultUNumber;
  bodyDir = defaultFaceDir - defaultNeckDir;
  neckDir = defaultNeckDir;
  faceDir = defaultFaceDir;      
}


/*
// Redefined since we now have access to the player_speed_max parameter.
void PlayerObject::setAbsoluteSpeedVector(const Vector& v, const t_int hist)
{
  setSpeed(v.getLength(), hist),
  setAbsoluteSpeedDirection(v.getAngle(), hist);
}

// Redefined since we now have access to the player_speed_max parameter.
void PlayerObject::setSpeed(const t_float_uco speed)
{
#ifndef _REQUIRE_PARAMETERS_IN_OBJECTS_
  if ( have_parameters ) {
#endif
    CMobileObject::setSpeed(MinMax(t_float(0), speed,
				   params->SP_player_speed_max), hist);
#ifndef _REQUIRE_PARAMETERS_IN_OBJECTS_
  } else {
    CMobileObject::setSpeed(speed, hist);
  }
#endif    
}
*/


void PlayerObject::setTeam(const Side our_side)
{
  if ( our_side == side )
    team = T_OUR_TEAM;
  else if ( our_side == S_UNKNOWN )
    team = T_UNKNOWN;
  else
    team = T_THEIR_TEAM;
}

VectorUCO PlayerObject::getRelativeVectorTo(const PointUCO& pt) const
{
  VectorUCO vec = FieldObject::getAbsoluteVectorTo(pt);
  return vec.rotate(-bodyDir);
}

AngleDegUCO PlayerObject::getRelativeDirectionTo(const PointUCO& pt) const
{
  AngleDegUCO ang = FieldObject::getAbsoluteDirectionTo(pt);
  return ang - bodyDir;
}

/* Make it into an operator<<
void PlayerObject::print(const t_int hist) const
{

  if (hist == UNKNOWN_T_INT) {
    for (t_int i = -max_hist_length; i < max_length - max_hist_length; ++i) {
      CMobileObject::Print(i);  
      std::cout << "[" << i << "]"
		<< " BodyDir: " << getBodyDir(i)
		<< " NeckDir: " << getNeckDir(i)
		<< " FaceDir: " << getFaceDir(i)
		<< std::endl;
    }
  } else {
    CMobileObject::Print(hist);
    std::cout << "[" << hist << "]"
	      << " BodyDir: " << getBodyDir(hist)
	      << " NeckDir: " << getNeckDir(hist)
	      << " FaceDir: " << getFaceDir(hist)
	      << std::endl;
  }
  
}
*/


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "PlayerObject.icc"
#endif
