/* -*- Mode: C++ -*-
 * $Id: RSException.cc,v 1.1.1.1 2000/08/31 13:26:03 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: RSException.cc
 * Unitname: Library
 * $Revision: 1.1.1.1 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/08/31 13:26:03 $
 *
 *
 * HISTORY
 *
 * $Log: RSException.cc,v $
 * Revision 1.1.1.1  2000/08/31 13:26:03  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file RSException.cc
 * Implementation of class RSException.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.1.1.1 $
 * $Author: frehe $
 * $Date: 2000/08/31 13:26:03 $
 */


#include "RSException.h"
#include "string_utils.h"


RS_BEGIN_NAMESPACE


RSException::RSException(const std::string& descr,
			 const std::string& file_name,
			 const UInt32 line_no,
			 const std::string& nme) throw()
  : exception(),
    description(descr),
    name(nme),
    fileName(file_name),
    lineNumber(line_no)
{
}


RSException::RSException(const RSException& obj) throw()
  : exception(obj),
    description(obj.description),
    name(obj.name),
    fileName(obj.fileName),
    lineNumber(obj.lineNumber)
{
}


RSException::~RSException() throw()
{
}


RSException& RSException::operator=(const RSException& obj) throw()
{
  if ( this != &obj ) {
    description = obj.description;
    name = obj.name;
    fileName = obj.fileName;
    lineNumber = obj.lineNumber;
  }
  return *this;
}


const std::string RSException::What() const throw()
{
  if ( fileName == "" )
    return "Raised an " + name + " '" + description + "'";
  else
    if ( lineNumber == 0 )
      return "Raised an " + name + " '" + description + "' in file " + fileName;
    else
      return "Raised an " + name + " '" + description + "' on line "
	+ toString(lineNumber) + " in file " + fileName;
}


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "RSException.icc"
#endif
