/* -*- Mode: C++ -*-
 * $Id: SayCommand.cc,v 1.2 2000/09/09 11:49:54 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: SayCommand.cc
 * Unitname: Library
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/09 11:49:54 $
 *
 *
 * HISTORY
 *
 * $Log: SayCommand.cc,v $
 * Revision 1.2  2000/09/09 11:49:54  frehe
 * Added a new level in the command hieararchy, namely ImmediateCommand and PeriodicCommand.
 *
 * Revision 1.1  2000/09/02 14:16:28  frehe
 * Imported the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file SayCommand.cc
 * Implementation of the class SayCommand.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2000/09/09 11:49:54 $
 */


#include "SayCommand.h"


RS_BEGIN_NAMESPACE


SayCommand::SayCommand(const std::string& msg)
  : ImmediateCommand(CMD_Say),
    message(msg)
{
}


SayCommand::SayCommand(const SayCommand& obj)
  : ImmediateCommand(obj),
    message(obj.message)
{
}


SayCommand::~SayCommand()
{
}


void SayCommand::copyCommand(const SayCommand& obj)
{
  ImmediateCommand::copyCommand(obj);
  message = obj.message;
}


SayCommand& SayCommand::operator =(const SayCommand& obj)
{
  if ( this != &obj ) {
    copyCommand(obj);
  }

  return *this;
}


const std::string SayCommand::serverString() const
{
  return "(say " + message + ")";
}


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "SayCommand.icc"
#endif
