/* -*- Mode: C++ -*-
 * $Id: TimeUnknown.cc,v 1.1 2000/09/01 01:24:47 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: TimeUnknown.cc
 * Unitname: Library
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 2000-xx-xx
 * Last modified by $Author: frehe $ $Date: 2000/09/01 01:24:47 $
 *
 *
 * HISTORY
 *
 * $Log: TimeUnknown.cc,v $
 * Revision 1.1  2000/09/01 01:24:47  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file TimeUnknown.cc
 * Implementation of the class TimeUnknown.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2000/09/01 01:24:47 $
 */


#include "TimeUnknown.h"
#include "string_utils.h"


RS_BEGIN_NAMESPACE


TimeUnknown::TimeUnknown()
  : Time(),
    unknown(true)
{
}


TimeUnknown::TimeUnknown(const UInt16 vt, const UInt16 vs)
  : Time(vt, vs),
    unknown(false)
{
}


TimeUnknown::TimeUnknown(const Time& obj)
  : Time(obj),
    unknown(false)
{
}


TimeUnknown::TimeUnknown(const TimeUnknown& obj)
  : Time(obj),
    unknown(obj.unknown)
{
}


TimeUnknown& TimeUnknown::operator=(const UInt16 a)
{
  Time::operator=(a);
  unknown = false;
  return *this;
}


TimeUnknown& TimeUnknown::operator=(const Time& a)
{
  Time::operator=(a);
  unknown = false;
  return *this;
}


TimeUnknown& TimeUnknown::operator=(const TimeUnknown& a)
{
  if ( this != &a ) {
    Time::operator=(a);
    unknown = a.unknown;
  }
  return *this;
}


std::istream& operator>>(std::istream& is, TimeUnknown& ct)
{
  /* Input formats: i is an UInt16
     i     = (i,0)
     (i)   = (i,0)
     (i,i) = (i,i)
     */

  UInt16 t = 0;
  UInt16 s = 0;
  char c = 0;

  is >> c;
  if (c == '(') {
    is >> t >> c;
    if (c == ',') is >> s >> c;
    if (c != ')') is.clear(std::ios::badbit); //error...;
  }
  else {
    is.putback(c);
    is >> t;
  }

  if (is)
    ct = TimeUnknown(t, s);
  else
    ct = TimeUnknown();

  return is;
}


std::ostream& operator<<(std::ostream&  os, const TimeUnknown& ct)
{
#ifdef TIMEUNKNOWN_HEAD
  if ( ct.isUnknown() )
    return os << "UNKNOWN(Time)";
  else
    return os << TIMEUNKNOWN_HEAD << "(" << ct.getT() << ", " << ct.getS() << ")";
#else
  if ( ct.isUnknown() )
    return os << "UNKNOWN(Time)";
  else
    return os << "(" << ct.getT() << ", " << ct.getS() << ")";
#endif
}


const std::string toString(const TimeUnknown& obj)
{
#ifdef TIMEUNKNOWN_HEAD
  if ( obj.isUnknown() )
    return "UNKNOWN(Time)";
  else
    return TIMEUNKNOWN_HEAD + "(" + toString(obj.getT()) + ", " + toString(obj.getS()) + ")";
#else
  if ( obj.isUnknown() )
    return "UNKNOWN(OBJime)";
  else
    return "(" + toString(obj.getT()) + ", " + toString(obj.getS()) + ")";
#endif
}


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "TimeUnknown.icc"
#endif

