/* -*- Mode: C++ -*-
 * $Id: math_utils.cc,v 1.5 2000/10/08 17:33:23 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: math_utils.cc
 * Unitname: Library
 * $Revision: 1.5 $
 * Created by: Fredrik Heintz 2000-08-29
 * Last modified by $Author: frehe $ $Date: 2000/10/08 17:33:23 $
 *
 *
 * HISTORY
 *
 * $Log: math_utils.cc,v $
 * Revision 1.5  2000/10/08 17:33:23  frehe
 * Remade the sqrt template to a single function and also added a test to see that the argument is positive.
 *
 * Revision 1.4  2000/09/18 15:20:13  frehe
 * Due to a problem with namespaces the name for atan2, atan, acos and asin is changed to ATan2, ATan, ASin and ACos.
 *
 * Revision 1.3  2000/09/05 21:09:55  frehe
 * Made sin, cos, tan, atan2, atan, acos, asin and sqrt call the standard function.
 *
 * Revision 1.2  2000/09/04 15:22:15  frehe
 * Changed the name of the trigonometric functions to the same as in the standard math library (sin, cos, tan, asin, acos, atan, and atan2).
 *
 * Revision 1.1.1.1  2000/08/31 13:26:03  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file math_utils.cc
 * Implementation of math utilities.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.5 $
 * $Author: frehe $
 * $Date: 2000/10/08 17:33:23 $
 */


#include "math_utils.h"


RS_BEGIN_NAMESPACE


/* Do a while loop and shift left if even otherwise decrease with one.
   Maybe not as efficient since we need to test for odd or even. */
Int32 Power(const Int32 x, const UInt32 y)
{
  Int32 a = 1;
  for (UInt32 i=0; i<y; ++i) {
    a*=x;
  }
  return a;
}


Float64 power(const Float64 x, const UInt32 y)
{
  Float64 a = 1;
  for (UInt32 i=0; i<y; ++i) {
    a*=x;
  }
  return a;
}


/*
Float64 asin(const Float64 op)
{
#ifdef RS_TRIG_IN_DEG
  return rad2Deg(std::asin(op));
#else
  return std::asin(op);
#endif
}


Float64 acos(const Float64 op)
{
#ifdef RS_TRIG_IN_DEG
  return rad2Deg(std::acos(op));
#else
  return std::acos(op);
#endif
}


Float64 atan(const Float64 op)
{
#ifdef RS_TRIG_IN_DEG
  return rad2Deg(std::atan(op));
#else
  return std::atan(op);
#endif
}


Float64 atan2(const Float64 op1, const Float64 op2)
{
#ifdef RS_TRIG_IN_DEG
  return rad2Deg(std::atan2(op1, op2));
#else
  return std::atan2(op1, op2);
#endif
}
*/


Float64 sin(const Float64 op)
{
#ifdef RS_TRIG_IN_DEG
  return std::sin(deg2Rad(op));
#else
  return std::sin(op);
#endif
}


Float64 cos(const Float64 op)
{
#ifdef RS_TRIG_IN_DEG
  return std::cos(deg2Rad(op));
#else
  return std::cos(op);
#endif
}


Float64 tan(const Float64 op)
{
#ifdef RS_TRIG_IN_DEG
  return std::tan(deg2Rad(op));
#else
  return std::tan(op);
#endif
}


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "math_utils.icc"
#endif
