/* -*- Mode: C++ -*-
 * $Id: CommandTest.cc,v 1.3 2001/06/19 14:01:51 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: CommandTest.cc
 * Unitname: Library/test
 * $Revision: 1.3 $
 * Created by: Fredrik Heintz 2000-08-30
 * Last modified by $Author: frehe $ $Date: 2001/06/19 14:01:51 $
 *
 *
 * HISTORY
 *
 * $Log: CommandTest.cc,v $
 * Revision 1.3  2001/06/19 14:01:51  frehe
 * Made minor changes to compile RoboSoc with gcc-3.0
 *
 * Revision 1.2  2000/09/14 20:38:38  frehe
 * Added support for building shared libraries with libtool.
 *
 * Revision 1.1  2000/08/31 20:02:33  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 * Revision 1.1.1.1  2000/08/31 13:26:04  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file CommandTest.cc
 * Test file for Command and its subclasses.
 *
 * @par Unit:
 *      Library/test
 * $Revision: 1.3 $
 * $Author: frehe $
 * $Date: 2001/06/19 14:01:51 $
 */


#include <iostream>

#include "Command.h"
#include "ServerCommand.h"
#include "ControlCommand.h"
#include "DashCommand.h"
#include "SetTicksBetweenCycles.h"
#include "ByeCommand.h"


int main()
{

  using namespace RS_NAMESPACE;

  ByeCommand b1;
  ByeCommand b2(b1);

  b2 = b1;
  
  // Illegal since abstract
  // Command cmd;

  // Also illegal
  // ServerCommand scmd;

  Command* cmdp;
  ServerCommand* scmdp;

  DashCommand dash(100);
  SetTicksBetweenCycles  ticks(11);

  // Example of polymorphism
  cmdp = &ticks;
  std::cout << *cmdp << " == " << ticks << std::endl;
  cmdp = &dash;
  std::cout << *cmdp << " == " << dash << std::endl;

  // Not possible even though  we "know" that
  // cmdp points to a DashCommand.
  // scmdp = cmdp;
  // Can be done with an explicit cast
  cmdp = &dash;
  scmdp = dynamic_cast<ServerCommand*>(cmdp);
  std::cout << *scmdp << " == " << dash << std::endl;
  
  cmdp = &ticks;
  scmdp = dynamic_cast<ServerCommand*>(cmdp);
  if ( scmdp == NULL )
    std::cout << "cmdp doesn't point to a ServerCommand object!" << std::endl;
  else
    std::cout << *scmdp << " == " << ticks << std::endl;
  

  return 0;
}
