/**
 * @file ViewDependencies.h
 *
 * This file will be used to keep track
 * of the current dependencies between the views
 * in the RoboSoc system.
 *
 */


/** Base class for all views. */
class View {};


/** Base class for all views using history. */
class ViewWithHistory : public View {};


/** View for storing all the parameters from the soccer
  server and the client.*/
class ParameterView : public View {};


/** View for calculating the effects of the commands. */
class CommandView : public ViewWithHistory {
  AgentView* agentView;
  BallView* ballView;
  GameView* gameView;
  ParameterView* parameterView;
};


/** View used to represent the current game, the time and score. */
class GameView : public ViewWithHistory {
  CommandView* commandView;
};


/** View used to store all the markers on the field. */
class MarkerView : public ViewWithHistory {
  AgentView* agentView;
  CommandView* commandView;
  GameView* gameView;
  ParameterView* parameterView;
};  


/** View used to store information about the agent. */
class AgentView : public ViewWithHistory {
  CommandView* commandView;
  GameView* gameView;
  MarkerView* markerView;
  ParameterView* parameterView;
};  


/** View used to store information about the ball. */
class BallView : public ViewWithHistory {
  AgentView* agentView;
  CommandView* commandView;
  ParameterView* parameterView;
};  


/** View used to store information about the other players on the field. */
class PlayerView : public ViewWithHistory {
  AgentView* agentView;
  CommandView* commandView;
  ParameterView* parameterView;
}  

/** Base class for views that calculates the future based entire on other
    views */
class ViewWithFuture : public View {
}

/** View used to calculate the balls position in the future or where it is
    unknown */
class FutureBallView : public ViewWithFuture {
  BallView* ballView;
  ParameterView* parameterView;
}
